#ifndef __SCE_LOG_H_
#define __SCE_LOG_H_

#include <stdint.h>
#ifdef CMAKE_CONFIG
#include "config.h"
#else
#define SCE_ENABLE_LOG
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define API __attribute__((visibility("default")))

#define SCE_LOG_MSG_LEN_START 256

#ifdef SCE_ENABLE_LOG

#define SCE_LOG_ALWAYS(...) sce_log_always(__VA_ARGS__)
#define SCE_LOG_ERR(...)                                      \
	do {                                                  \
		if (sce_log_get_stderr() >= (SCE_LOG_LL_ERR)) \
			sce_log(SCE_LOG_LL_ERR, __VA_ARGS__); \
	} while (0)

#define SCE_LOG_WRN(...)                                      \
	do {                                                  \
		if (sce_log_get_stderr() >= (SCE_LOG_LL_WRN)) \
			sce_log(SCE_LOG_LL_WRN, __VA_ARGS__); \
	} while (0)

#define SCE_LOG_INF(...)                                      \
	do {                                                  \
		if (sce_log_get_stderr() >= (SCE_LOG_LL_INF)) \
			sce_log(SCE_LOG_LL_INF, __VA_ARGS__); \
	} while (0)

#define SCE_LOG_DBG(...)                                      \
	do {                                                  \
		if (sce_log_get_stderr() >= (SCE_LOG_LL_DBG)) \
			sce_log(SCE_LOG_LL_DBG, __VA_ARGS__); \
	} while (0)

#else

#define SCE_LOG_ALWAYS(...)                   \
	do {                                  \
		fprintf(stderr, __VA_ARGS__); \
	} while (0)

#define SCE_LOG_ERR(...)                      \
	do {                                  \
		fprintf(stderr, __VA_ARGS__); \
	} while (0)

#define SCE_LOG_WRN(...)
#define SCE_LOG_INF(...)
#define SCE_LOG_DBG(...)

#endif

typedef enum {
	SCE_LOG_LL_NONE = 0,
	SCE_LOG_LL_ERR,
	SCE_LOG_LL_WRN,
	SCE_LOG_LL_INF,
	SCE_LOG_LL_DBG,
} sce_log_level_t;

typedef void (*sce_log_cb)(sce_log_level_t level, const char *message);

API void sce_log_msg(sce_log_level_t ll, const char *msg);

API void sce_log(sce_log_level_t ll, const char *format, ...);

void sce_log_set_stderr(sce_log_level_t log_level);

sce_log_level_t sce_log_get_stderr(void);

void sce_log_always(const char *format, ...);

#ifndef SCE_LOG_DISABLE_SYSLOG
#define SCE_LOG_SYSLOG
#else
#ifdef SCE_LOG_SYSLOG
#undef SCE_LOG_SYSLOG
#endif

#endif

#ifdef SCE_LOG_SYSLOG

void sce_set_syslog(const char *app_name, sce_log_level_t log_level);

sce_log_level_t sce_get_syslog(void);

#endif

void sce_set_log_callback(sce_log_cb log_callback);

#ifdef __cplusplus
}
#endif

#endif
