#ifndef _PHYTIUM_CE_LOG_H_
#define _PHYTIUM_CE_LOG_H_

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#define SCTO_LOG_MSG_LEN_START 256

#define SCTO_ENABLE_LOG /* Enabled log default */

#ifdef SCTO_ENABLE_LOG

#define SCTO_LOG_ALWAYS(...) scto_log_always(__VA_ARGS__)
#define SCTO_LOG_ERR(...)                                       \
	do {                                                    \
		if (scto_log_get_stderr() >= (SCTO_LOG_LL_ERR)) \
			scto_log(SCTO_LOG_LL_ERR, __VA_ARGS__); \
	} while (0)

#define SCTO_LOG_WRN(...)                                       \
	do {                                                    \
		if (scto_log_get_stderr() >= (SCTO_LOG_LL_WRN)) \
			scto_log(SCTO_LOG_LL_WRN, __VA_ARGS__); \
	} while (0)

#define SCTO_LOG_INF(...)                                       \
	do {                                                    \
		if (scto_log_get_stderr() >= (SCTO_LOG_LL_INF)) \
			scto_log(SCTO_LOG_LL_INF, __VA_ARGS__); \
	} while (0)

#define SCTO_LOG_DBG(...)                                       \
	do {                                                    \
		if (scto_log_get_stderr() >= (SCTO_LOG_LL_DBG)) \
			scto_log(SCTO_LOG_LL_DBG, __VA_ARGS__); \
	} while (0)

#else

#define SCTO_LOG_ALWAYS(...)                  \
	do {                                  \
		fprintf(stdout, __VA_ARGS__); \
	} while (0)

#define SCTO_LOG_ERR(...) fprintf(stderr, __VA_ARGS__)
#define SCTO_LOG_WRN(...)
#define SCTO_LOG_INF(...)
#define SCTO_LOG_DBG(...)

#endif

typedef enum {
	SCTO_LOG_LL_NONE = 0, /**< Do not print any messages. */
	SCTO_LOG_LL_ERR, /**< Print only error messages. */
	SCTO_LOG_LL_WRN, /**< Print error and warning messages. */
	SCTO_LOG_LL_INF, /**< Besides errors and warnings, print some other
                         informational messages. default level. */
	SCTO_LOG_LL_DBG, /**< Print all messages including some development debug
                         messages. */
} scto_log_level_t;

typedef void (*scto_log_cb)(scto_log_level_t level, const char *message);

void scto_log_msg(scto_log_level_t ll, const char *msg);

void scto_log(scto_log_level_t ll, const char *format, ...);

void scto_log_set_stderr(scto_log_level_t log_level);

scto_log_level_t scto_log_get_stderr(void);

void scto_log_always(const char *format, ...);

//#define SCTO_LOG_SYSLOG

#ifndef SCTO_LOG_DISABLE_SYSLOG
#define SCTO_LOG_SYSLOG
#else
#ifdef SCTO_LOG_SYSLOG
#undef SCTO_LOG_SYSLOG
#endif

#endif

#ifdef SCTO_LOG_SYSLOG

void scto_log_set_syslog(const char *app_name, scto_log_level_t log_level);

scto_log_level_t scto_log_get_syslog(void);

#endif

void scto_log_set_cb(scto_log_cb log_callback);

#ifdef __cplusplus
}
#endif

#endif
