#define _GNU_SOURCE
#include <sched.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <signal.h>
#include <dsce/sce.h>
#include <dsce/sce_log.h>
#include "test.h"
#include "hash.h"

static const hash_testvec_t hash_tv_template[] = {
	{ .psize = 0,
	  .dsize = 32,
	  .alg = SCE_SM3,
	  .mode = SCE_NORMAL,
	  .plaintext = "",
	  .digest = (char *)(char[]){ 0x1A, 0xB2, 0x1D, 0x83, 0x55, 0xCF, 0xA1,
				      0x7F, 0x8e, 0x61, 0x19, 0x48, 0x31, 0xE8,
				      0x1A, 0x8F, 0x22, 0xBE, 0xC8, 0xC7, 0x28,
				      0xFE, 0xFB, 0x74, 0x7E, 0xD0, 0x35, 0xEB,
				      0x50, 0x82, 0xAA, 0x2B } },
	{ .psize = 1,
	  .dsize = 32,
	  .alg = SCE_SM3,
	  .mode = SCE_NORMAL,
	  .plaintext = "a",
	  .digest = (char *)(char[]){ 0x62, 0x34, 0x76, 0xAC, 0x18, 0xF6, 0x5A,
				      0x29, 0x09, 0xE4, 0x3C, 0x7F, 0xEC, 0x61,
				      0xB4, 0x9C, 0x7E, 0x76, 0x4A, 0x91, 0xA1,
				      0x8C, 0xCB, 0x82, 0xF1, 0x91, 0x7A, 0x29,
				      0xC8, 0x6C, 0x5E, 0x88 } },
	{ .psize = 3,
	  .dsize = 32,
	  .alg = SCE_SM3,
	  .mode = SCE_NORMAL,
	  /* A.1. Example 1 */
	  .plaintext = "abc",
	  .digest = (char *)(char[]){ 0x66, 0xC7, 0xF0, 0xF4, 0x62, 0xEE, 0xED,
				      0xD9, 0xD1, 0xF2, 0xD4, 0x6B, 0xDC, 0x10,
				      0xE4, 0xE2, 0x41, 0x67, 0xC4, 0x87, 0x5C,
				      0xF2, 0xF7, 0xA2, 0x29, 0x7D, 0xA0, 0x2B,
				      0x8F, 0x4B, 0xA8, 0xE0 } },
	{ .psize = 26,
	  .dsize = 32,
	  .alg = SCE_SM3,
	  .mode = SCE_NORMAL,
	  .plaintext = "abcdefghijklmnopqrstuvwxyz",
	  .digest = (char *)(char[]){ 0xB8, 0x0F, 0xE9, 0x7A, 0x4D, 0xA2, 0x4A,
				      0xFC, 0x27, 0x75, 0x64, 0xF6, 0x6A, 0x35,
				      0x9E, 0xF4, 0x40, 0x46, 0x2A, 0xD2, 0x8D,
				      0xCC, 0x6D, 0x63, 0xAD, 0xB2, 0x4D, 0x5C,
				      0x20, 0xA6, 0x15, 0x95 } },
	{ .psize = 64,
	  .dsize = 32,
	  .alg = SCE_SM3,
	  .mode = SCE_NORMAL,
	  /* A.1. Example 2 */
	  .plaintext = "abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdab"
		       "cdabcdabcdabcdabcd",
	  .digest = (char *)(char[]){ 0xDE, 0xBE, 0x9F, 0xF9, 0x22, 0x75, 0xB8,
				      0xA1, 0x38, 0x60, 0x48, 0x89, 0xC1, 0x8E,
				      0x5A, 0x4D, 0x6F, 0xDB, 0x70, 0xE5, 0x38,
				      0x7E, 0x57, 0x65, 0x29, 0x3D, 0xCB, 0xA3,
				      0x9C, 0x0C, 0x57, 0x32 } },
	{ .psize = 256,
	  .dsize = 32,
	  .alg = SCE_SM3,
	  .mode = SCE_NORMAL,
	  .plaintext = "abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"
		       "abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"
		       "abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"
		       "abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"
		       "abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"
		       "abcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcdabcd"
		       "abcdabcdabcdabcdabcdabcdabcdabcd",
	  .digest = (char *)(char[]){ 0xB9, 0x65, 0x76, 0x4C, 0x8B, 0xEB, 0xB0,
				      0x91, 0xC7, 0x60, 0x2B, 0x74, 0xAF, 0xD3,
				      0x4E, 0xEF, 0xB5, 0x31, 0xDC, 0xCB, 0x4E,
				      0x00, 0x76, 0xD9, 0xB7, 0xCD, 0x81, 0x31,
				      0x99, 0xB4, 0x59, 0x71 } },

	{
		.psize = 0,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_NORMAL,
		.plaintext = "",
		.digest = "\xd4\x1d\x8c\xd9\x8f\x00\xb2\x04"
			  "\xe9\x80\x09\x98\xec\xf8\x42\x7e",
	},
	{
		.psize = 1,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_NORMAL,
		.plaintext = "a",
		.digest = "\x0c\xc1\x75\xb9\xc0\xf1\xb6\xa8"
			  "\x31\xc3\x99\xe2\x69\x77\x26\x61",
	},
	{
		.psize = 3,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_NORMAL,
		.plaintext = "abc",
		.digest = "\x90\x01\x50\x98\x3c\xd2\x4f\xb0"
			  "\xd6\x96\x3f\x7d\x28\xe1\x7f\x72",
	},
	{
		.psize = 14,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_NORMAL,
		.plaintext = "message digest",
		.digest = "\xf9\x6b\x69\x7d\x7c\xb7\x93\x8d"
			  "\x52\x5a\x2f\x31\xaa\xf1\x61\xd0",
	},
	{
		.psize = 26,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_NORMAL,
		.plaintext = "abcdefghijklmnopqrstuvwxyz",
		.digest = "\xc3\xfc\xd3\xd7\x61\x92\xe4\x00"
			  "\x7d\xfb\x49\x6c\xca\x67\xe1\x3b",
	},
	{
		.psize = 62,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_NORMAL,
		.plaintext =
			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
		.digest = "\xd1\x74\xab\x98\xd2\x77\xd9\xf5"
			  "\xa5\x61\x1c\x2c\x9f\x41\x9d\x9f",
	},
	{
		.psize = 80,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_NORMAL,
		.plaintext =
			"12345678901234567890123456789012345678901234567890123456789012"
			"345678901234567890",
		.digest = "\x57\xed\xf4\xa2\x2b\xe3\xc9\x55"
			  "\xac\x49\xda\x2e\x21\x07\xb6\x7a",
	},

	{
		.psize = 0,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_NORMAL,
		.plaintext = "",
		.digest = "\xe3\xb0\xc4\x42\x98\xfc\x1c\x14"
			  "\x9a\xfb\xf4\xc8\x99\x6f\xb9\x24"
			  "\x27\xae\x41\xe4\x64\x9b\x93\x4c"
			  "\xa4\x95\x99\x1b\x78\x52\xb8\x55",
	},
	{
		.psize = 3,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_NORMAL,
		.plaintext = "abc",
		.digest = "\xba\x78\x16\xbf\x8f\x01\xcf\xea"
			  "\x41\x41\x40\xde\x5d\xae\x22\x23"
			  "\xb0\x03\x61\xa3\x96\x17\x7a\x9c"
			  "\xb4\x10\xff\x61\xf2\x00\x15\xad",
	},
	{
		.psize = 56,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		.digest = "\x24\x8d\x6a\x61\xd2\x06\x38\xb8"
			  "\xe5\xc0\x26\x93\x0c\x3e\x60\x39"
			  "\xa3\x3c\xe4\x59\x64\xff\x21\x67"
			  "\xf6\xec\xed\xd4\x19\xdb\x06\xc1",
	},
	{
		.psize = 64,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_NORMAL,
		.plaintext =
			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-",
		.digest = "\xb5\xfe\xad\x56\x7d\xff\xcb\xa4"
			  "\x2c\x32\x29\x32\x19\xbb\xfb\xfa"
			  "\xd6\xff\x94\xa3\x72\x91\x85\x66"
			  "\x3b\xa7\x87\x77\x58\xa3\x40\x3a",
	},
	{
		.psize = 1023,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_NORMAL,
		.plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
			     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
			     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
			     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
			     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
			     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
			     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
			     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
			     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
			     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
			     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
			     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
			     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
			     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
			     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
			     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
			     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
			     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
			     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
			     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
			     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
			     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
			     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
			     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
			     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
			     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
			     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
			     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
			     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
			     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
			     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
			     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
			     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
			     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
			     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
			     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
			     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
			     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
			     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
			     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
			     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
			     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
			     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
			     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
			     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
			     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
			     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
			     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
			     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
			     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
			     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
			     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
			     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
			     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
			     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
			     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
			     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
			     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
			     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
			     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
			     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
			     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
			     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
			     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
			     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
			     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
			     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
			     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
			     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
			     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
			     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
			     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
			     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
			     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
			     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
			     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
			     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
			     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
			     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
			     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
			     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
			     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
			     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
			     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
			     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
			     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
			     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
			     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
			     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
			     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
			     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
			     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
			     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
			     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
			     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
			     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
			     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
			     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
			     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
			     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
			     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
			     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
			     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
			     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
			     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
			     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
			     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
			     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
			     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
			     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
			     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
			     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
			     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
			     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
			     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
			     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
			     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
			     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
			     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
			     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
			     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
			     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
			     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
			     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
			     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
			     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
			     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
			     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
		.digest = "\xc5\xce\x0c\xca\x01\x4f\x53\x3a"
			  "\x32\x32\x17\xcc\xd4\x6a\x71\xa9"
			  "\xf3\xed\x50\x10\x64\x8e\x06\xbe"
			  "\x9b\x4a\xa6\xbb\x05\x89\x59\x51",
	},

	{
		.psize = 0,
		.dsize = 48,
		.alg = SCE_SHA384,
		.mode = SCE_NORMAL,
		.plaintext = "",
		.psize = 0,
		.digest = "\x38\xb0\x60\xa7\x51\xac\x96\x38"
			  "\x4c\xd9\x32\x7e\xb1\xb1\xe3\x6a"
			  "\x21\xfd\xb7\x11\x14\xbe\x07\x43"
			  "\x4c\x0c\xc7\xbf\x63\xf6\xe1\xda"
			  "\x27\x4e\xde\xbf\xe7\x6f\x65\xfb"
			  "\xd5\x1a\xd2\xf1\x48\x98\xb9\x5b",
	},
	{
		.psize = 3,
		.dsize = 48,
		.alg = SCE_SHA384,
		.mode = SCE_NORMAL,
		.plaintext = "abc",
		.digest = "\xcb\x00\x75\x3f\x45\xa3\x5e\x8b"
			  "\xb5\xa0\x3d\x69\x9a\xc6\x50\x07"
			  "\x27\x2c\x32\xab\x0e\xde\xd1\x63"
			  "\x1a\x8b\x60\x5a\x43\xff\x5b\xed"
			  "\x80\x86\x07\x2b\xa1\xe7\xcc\x23"
			  "\x58\xba\xec\xa1\x34\xc8\x25\xa7",
	},
	{
		.psize = 56,
		.dsize = 48,
		.alg = SCE_SHA384,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		.digest = "\x33\x91\xfd\xdd\xfc\x8d\xc7\x39"
			  "\x37\x07\xa6\x5b\x1b\x47\x09\x39"
			  "\x7c\xf8\xb1\xd1\x62\xaf\x05\xab"
			  "\xfe\x8f\x45\x0d\xe5\xf3\x6b\xc6"
			  "\xb0\x45\x5a\x85\x20\xbc\x4e\x6f"
			  "\x5f\xe9\x5b\x1f\xe3\xc8\x45\x2b",
	},
	{
		.psize = 112,
		.dsize = 48,
		.alg = SCE_SHA384,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn"
			"hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu",
		.digest = "\x09\x33\x0c\x33\xf7\x11\x47\xe8"
			  "\x3d\x19\x2f\xc7\x82\xcd\x1b\x47"
			  "\x53\x11\x1b\x17\x3b\x3b\x05\xd2"
			  "\x2f\xa0\x80\x86\xe3\xb0\xf7\x12"
			  "\xfc\xc7\xc7\x1a\x55\x7e\x2d\xb9"
			  "\x66\xc3\xe9\xfa\x91\x74\x60\x39",
	},
	{
		.psize = 104,
		.dsize = 48,
		.alg = SCE_SHA384,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcd"
			"efghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyz",
		.digest = "\x3d\x20\x89\x73\xab\x35\x08\xdb"
			  "\xbd\x7e\x2c\x28\x62\xba\x29\x0a"
			  "\xd3\x01\x0e\x49\x78\xc1\x98\xdc"
			  "\x4d\x8f\xd0\x14\xe5\x82\x82\x3a"
			  "\x89\xe1\x6f\x9b\x2a\x7b\xbc\x1a"
			  "\xc9\x38\xe2\xd1\x99\xe8\xbe\xa4",
	},
	{
		.psize = 1023,
		.dsize = 48,
		.alg = SCE_SHA384,
		.mode = SCE_NORMAL,
		.plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
			     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
			     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
			     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
			     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
			     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
			     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
			     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
			     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
			     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
			     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
			     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
			     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
			     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
			     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
			     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
			     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
			     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
			     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
			     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
			     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
			     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
			     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
			     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
			     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
			     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
			     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
			     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
			     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
			     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
			     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
			     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
			     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
			     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
			     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
			     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
			     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
			     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
			     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
			     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
			     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
			     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
			     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
			     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
			     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
			     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
			     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
			     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
			     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
			     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
			     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
			     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
			     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
			     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
			     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
			     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
			     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
			     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
			     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
			     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
			     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
			     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
			     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
			     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
			     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
			     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
			     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
			     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
			     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
			     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
			     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
			     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
			     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
			     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
			     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
			     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
			     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
			     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
			     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
			     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
			     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
			     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
			     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
			     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
			     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
			     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
			     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
			     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
			     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
			     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
			     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
			     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
			     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
			     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
			     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
			     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
			     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
			     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
			     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
			     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
			     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
			     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
			     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
			     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
			     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
			     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
			     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
			     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
			     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
			     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
			     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
			     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
			     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
			     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
			     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
			     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
			     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
			     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
			     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
			     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
			     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
			     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
			     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
			     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
			     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
			     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
			     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
			     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
		.digest = "\x4d\x97\x23\xc8\xea\x7a\x7c\x15"
			  "\xb8\xff\x97\x9c\xf5\x13\x4f\x31"
			  "\xde\x67\xf7\x24\x73\xcd\x70\x1c"
			  "\x03\x4a\xba\x8a\x87\x49\xfe\xdc"
			  "\x75\x29\x62\x83\xae\x3f\x17\xab"
			  "\xfd\x10\x4d\x8e\x17\x1c\x1f\xca",
	},

	{
		.psize = 0,
		.dsize = 64,
		.alg = SCE_SHA512,
		.mode = SCE_NORMAL,
		.plaintext = "",
		.digest = "\xcf\x83\xe1\x35\x7e\xef\xb8\xbd"
			  "\xf1\x54\x28\x50\xd6\x6d\x80\x07"
			  "\xd6\x20\xe4\x05\x0b\x57\x15\xdc"
			  "\x83\xf4\xa9\x21\xd3\x6c\xe9\xce"
			  "\x47\xd0\xd1\x3c\x5d\x85\xf2\xb0"
			  "\xff\x83\x18\xd2\x87\x7e\xec\x2f"
			  "\x63\xb9\x31\xbd\x47\x41\x7a\x81"
			  "\xa5\x38\x32\x7a\xf9\x27\xda\x3e",
	},
	{
		.psize = 3,
		.dsize = 64,
		.alg = SCE_SHA512,
		.mode = SCE_NORMAL,
		.plaintext = "abc",
		.digest = "\xdd\xaf\x35\xa1\x93\x61\x7a\xba"
			  "\xcc\x41\x73\x49\xae\x20\x41\x31"
			  "\x12\xe6\xfa\x4e\x89\xa9\x7e\xa2"
			  "\x0a\x9e\xee\xe6\x4b\x55\xd3\x9a"
			  "\x21\x92\x99\x2a\x27\x4f\xc1\xa8"
			  "\x36\xba\x3c\x23\xa3\xfe\xeb\xbd"
			  "\x45\x4d\x44\x23\x64\x3c\xe8\x0e"
			  "\x2a\x9a\xc9\x4f\xa5\x4c\xa4\x9f",
	},
	{
		.psize = 56,
		.dsize = 64,
		.alg = SCE_SHA512,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		.digest = "\x20\x4a\x8f\xc6\xdd\xa8\x2f\x0a"
			  "\x0c\xed\x7b\xeb\x8e\x08\xa4\x16"
			  "\x57\xc1\x6e\xf4\x68\xb2\x28\xa8"
			  "\x27\x9b\xe3\x31\xa7\x03\xc3\x35"
			  "\x96\xfd\x15\xc1\x3b\x1b\x07\xf9"
			  "\xaa\x1d\x3b\xea\x57\x78\x9c\xa0"
			  "\x31\xad\x85\xc7\xa7\x1d\xd7\x03"
			  "\x54\xec\x63\x12\x38\xca\x34\x45",
	},
	{
		.psize = 112,
		.dsize = 64,
		.alg = SCE_SHA512,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn"
			"hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu",
		.digest = "\x8e\x95\x9b\x75\xda\xe3\x13\xda"
			  "\x8c\xf4\xf7\x28\x14\xfc\x14\x3f"
			  "\x8f\x77\x79\xc6\xeb\x9f\x7f\xa1"
			  "\x72\x99\xae\xad\xb6\x88\x90\x18"
			  "\x50\x1d\x28\x9e\x49\x00\xf7\xe4"
			  "\x33\x1b\x99\xde\xc4\xb5\x43\x3a"
			  "\xc7\xd3\x29\xee\xb6\xdd\x26\x54"
			  "\x5e\x96\xe5\x5b\x87\x4b\xe9\x09",
	},
	{
		.psize = 104,
		.dsize = 64,
		.alg = SCE_SHA512,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcd"
			"efghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyz",
		.digest = "\x93\x0d\x0c\xef\xcb\x30\xff\x11"
			  "\x33\xb6\x89\x81\x21\xf1\xcf\x3d"
			  "\x27\x57\x8a\xfc\xaf\xe8\x67\x7c"
			  "\x52\x57\xcf\x06\x99\x11\xf7\x5d"
			  "\x8f\x58\x31\xb5\x6e\xbf\xda\x67"
			  "\xb2\x78\xe6\x6d\xff\x8b\x84\xfe"
			  "\x2b\x28\x70\xf7\x42\xa5\x80\xd8"
			  "\xed\xb4\x19\x87\x23\x28\x50\xc9",
	},
	{
		.psize = 1023,
		.dsize = 64,
		.alg = SCE_SHA512,
		.mode = SCE_NORMAL,
		.plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
			     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
			     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
			     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
			     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
			     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
			     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
			     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
			     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
			     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
			     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
			     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
			     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
			     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
			     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
			     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
			     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
			     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
			     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
			     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
			     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
			     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
			     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
			     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
			     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
			     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
			     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
			     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
			     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
			     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
			     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
			     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
			     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
			     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
			     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
			     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
			     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
			     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
			     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
			     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
			     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
			     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
			     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
			     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
			     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
			     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
			     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
			     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
			     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
			     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
			     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
			     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
			     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
			     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
			     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
			     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
			     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
			     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
			     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
			     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
			     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
			     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
			     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
			     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
			     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
			     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
			     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
			     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
			     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
			     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
			     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
			     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
			     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
			     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
			     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
			     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
			     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
			     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
			     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
			     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
			     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
			     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
			     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
			     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
			     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
			     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
			     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
			     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
			     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
			     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
			     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
			     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
			     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
			     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
			     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
			     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
			     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
			     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
			     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
			     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
			     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
			     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
			     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
			     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
			     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
			     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
			     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
			     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
			     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
			     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
			     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
			     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
			     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
			     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
			     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
			     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
			     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
			     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
			     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
			     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
			     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
			     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
			     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
			     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
			     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
			     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
			     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
			     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
		.digest = "\x76\xc9\xd4\x91\x7a\x5f\x0f\xaa"
			  "\x13\x39\xf3\x01\x7a\xfa\xe5\x41"
			  "\x5f\x0b\xf8\xeb\x32\xfc\xbf\xb0"
			  "\xfa\x8c\xcd\x17\x83\xe2\xfa\xeb"
			  "\x1c\x19\xde\xe2\x75\xdc\x34\x64"
			  "\x5f\x35\x9c\x61\x2f\x10\xf9\xec"
			  "\x59\xca\x9d\xcc\x25\x0c\x43\xba"
			  "\x85\xa8\xf8\xfe\xb5\x24\xb2\xee",
	},

	{
		.psize = 0,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_NORMAL,
		.plaintext = "",
		.digest = "\xda\x39\xa3\xee\x5e\x6b\x4b\x0d\x32\x55"
			  "\xbf\xef\x95\x60\x18\x90\xaf\xd8\x07\x09",
	},
	{
		.psize = 3,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_NORMAL,
		.plaintext = "abc",
		.psize = 3,
		.digest = "\xa9\x99\x3e\x36\x47\x06\x81\x6a\xba\x3e"
			  "\x25\x71\x78\x50\xc2\x6c\x9c\xd0\xd8\x9d",
	},
	{
		.psize = 56,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		.psize = 56,
		.digest = "\x84\x98\x3e\x44\x1c\x3b\xd2\x6e\xba\xae"
			  "\x4a\xa1\xf9\x51\x29\xe5\xe5\x46\x70\xf1",
	},
	{
		.psize = 163,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_NORMAL,
		.plaintext = "\xec\x29\x56\x12\x44\xed\xe7\x06"
			     "\xb6\xeb\x30\xa1\xc3\x71\xd7\x44"
			     "\x50\xa1\x05\xc3\xf9\x73\x5f\x7f"
			     "\xa9\xfe\x38\xcf\x67\xf3\x04\xa5"
			     "\x73\x6a\x10\x6e\x92\xe1\x71\x39"
			     "\xa6\x81\x3b\x1c\x81\xa4\xf3\xd3"
			     "\xfb\x95\x46\xab\x42\x96\xfa\x9f"
			     "\x72\x28\x26\xc0\x66\x86\x9e\xda"
			     "\xcd\x73\xb2\x54\x80\x35\x18\x58"
			     "\x13\xe2\x26\x34\xa9\xda\x44\x00"
			     "\x0d\x95\xa2\x81\xff\x9f\x26\x4e"
			     "\xcc\xe0\xa9\x31\x22\x21\x62\xd0"
			     "\x21\xcc\xa2\x8d\xb5\xf3\xc2\xaa"
			     "\x24\x94\x5a\xb1\xe3\x1c\xb4\x13"
			     "\xae\x29\x81\x0f\xd7\x94\xca\xd5"
			     "\xdf\xaf\x29\xec\x43\xcb\x38\xd1"
			     "\x98\xfe\x4a\xe1\xda\x23\x59\x78"
			     "\x02\x21\x40\x5b\xd6\x71\x2a\x53"
			     "\x05\xda\x4b\x1b\x73\x7f\xce\x7c"
			     "\xd2\x1c\x0e\xb7\x72\x8d\x08\x23"
			     "\x5a\x90\x11",
		.digest = "\x97\x01\x11\xc4\xe7\x7b\xcc\x88\xcc\x20"
			  "\x45\x9c\x02\xb6\x9b\x4a\xa8\xf5\x82\x17",
	},
	{
		.psize = 64,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_NORMAL,
		.plaintext =
			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-",
		.digest = "\xc8\x71\xf6\x9a\x63\xcc\xa9\x84\x84\x82"
			  "\x64\xe7\x79\x95\x5d\xd7\x19\x41\x7c\x91",
	},
	{
		.psize = 1023,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_NORMAL,
		.plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
			     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
			     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
			     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
			     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
			     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
			     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
			     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
			     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
			     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
			     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
			     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
			     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
			     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
			     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
			     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
			     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
			     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
			     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
			     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
			     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
			     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
			     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
			     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
			     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
			     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
			     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
			     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
			     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
			     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
			     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
			     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
			     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
			     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
			     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
			     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
			     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
			     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
			     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
			     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
			     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
			     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
			     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
			     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
			     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
			     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
			     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
			     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
			     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
			     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
			     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
			     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
			     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
			     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
			     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
			     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
			     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
			     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
			     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
			     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
			     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
			     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
			     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
			     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
			     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
			     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
			     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
			     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
			     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
			     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
			     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
			     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
			     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
			     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
			     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
			     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
			     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
			     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
			     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
			     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
			     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
			     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
			     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
			     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
			     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
			     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
			     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
			     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
			     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
			     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
			     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
			     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
			     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
			     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
			     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
			     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
			     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
			     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
			     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
			     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
			     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
			     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
			     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
			     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
			     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
			     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
			     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
			     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
			     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
			     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
			     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
			     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
			     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
			     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
			     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
			     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
			     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
			     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
			     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
			     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
			     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
			     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
			     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
			     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
			     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
			     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
			     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
			     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
		.digest = "\xb8\xe3\x54\xed\xc5\xfc\xef\xa4"
			  "\x55\x73\x4a\x81\x99\xe4\x47\x2a"
			  "\x30\xd6\xc9\x85",
	},

	{
		.psize = 0,
		.dsize = 28,
		.alg = SCE_SHA224,
		.mode = SCE_NORMAL,
		.plaintext = "",
		.digest = "\xd1\x4a\x02\x8c\x2a\x3a\x2b\xc9"
			  "\x47\x61\x02\xbb\x28\x82\x34\xc4"
			  "\x15\xa2\xb0\x1f\x82\x8e\xa6\x2a"
			  "\xc5\xb3\xe4\x2f",
	},
	{
		.psize = 3,
		.dsize = 28,
		.alg = SCE_SHA224,
		.mode = SCE_NORMAL,
		.plaintext = "abc",
		.digest = "\x23\x09\x7D\x22\x34\x05\xD8\x22"
			  "\x86\x42\xA4\x77\xBD\xA2\x55\xB3"
			  "\x2A\xAD\xBC\xE4\xBD\xA0\xB3\xF7"
			  "\xE3\x6C\x9D\xA7",
	},
	{
		.psize = 56,
		.dsize = 28,
		.alg = SCE_SHA224,
		.mode = SCE_NORMAL,
		.plaintext =
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		.digest = "\x75\x38\x8B\x16\x51\x27\x76\xCC"
			  "\x5D\xBA\x5D\xA1\xFD\x89\x01\x50"
			  "\xB0\xC6\x45\x5C\xB4\xF5\x8B\x19"
			  "\x52\x52\x25\x25",
	},
	{
		.psize = 64,
		.dsize = 28,
		.alg = SCE_SHA224,
		.mode = SCE_NORMAL,
		.plaintext =
			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-",
		.digest = "\xc4\xdb\x2b\x3a\x58\xc3\x99\x01"
			  "\x42\xfd\x10\x92\xaa\x4e\x04\x08"
			  "\x58\xbb\xbb\xe8\xf8\x14\xa7\x0c"
			  "\xef\x3b\xcb\x0e",
	},
	{
		.psize = 1023,
		.dsize = 28,
		.alg = SCE_SHA224,
		.mode = SCE_NORMAL,
		.plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
			     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
			     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
			     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
			     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
			     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
			     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
			     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
			     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
			     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
			     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
			     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
			     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
			     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
			     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
			     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
			     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
			     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
			     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
			     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
			     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
			     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
			     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
			     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
			     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
			     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
			     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
			     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
			     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
			     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
			     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
			     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
			     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
			     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
			     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
			     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
			     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
			     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
			     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
			     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
			     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
			     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
			     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
			     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
			     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
			     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
			     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
			     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
			     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
			     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
			     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
			     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
			     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
			     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
			     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
			     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
			     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
			     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
			     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
			     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
			     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
			     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
			     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
			     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
			     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
			     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
			     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
			     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
			     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
			     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
			     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
			     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
			     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
			     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
			     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
			     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
			     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
			     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
			     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
			     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
			     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
			     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
			     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
			     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
			     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
			     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
			     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
			     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
			     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
			     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
			     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
			     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
			     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
			     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
			     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
			     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
			     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
			     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
			     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
			     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
			     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
			     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
			     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
			     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
			     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
			     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
			     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
			     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
			     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
			     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
			     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
			     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
			     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
			     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
			     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
			     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
			     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
			     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
			     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
			     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
			     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
			     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
			     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
			     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
			     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
			     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
			     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
			     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
		.digest = "\x98\x43\x07\x63\x75\xe0\xa7\x1c"
			  "\x78\xb1\x8b\xfd\x04\xf5\x2d\x91"
			  "\x20\x48\xa4\x28\xff\x55\xb1\xd3"
			  "\xe6\xf9\x4f\xcc",
	},
};

static const hmac_testvec_t hmac_tv_template[] = {
	{
		.ksize = 32,
		.psize = 112,
		.dsize = 32,
		.alg = SCE_SM3,
		.mode = SCE_HMAC,
		.key = "\x01\x02\x03\x04\x05\x06\x07\x08"
		       "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
		       "\x11\x12\x13\x14\x15\x16\x17\x18"
		       "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20",
		.plaintext =
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		.digest = "\xca\x05\xe1\x44\xed\x05\xd1\x85"
			  "\x78\x40\xd1\xf3\x18\xa4\xa8\x66"
			  "\x9e\x55\x9f\xc8\x39\x1f\x41\x44"
			  "\x85\xbf\xdf\x7b\xb4\x08\x96\x3a",
	},
	{
		.ksize = 37,
		.psize = 50,
		.dsize = 32,
		.alg = SCE_SM3,
		.mode = SCE_HMAC,
		.key = "\x01\x02\x03\x04\x05\x06\x07\x08"
		       "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
		       "\x11\x12\x13\x14\x15\x16\x17\x18"
		       "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20"
		       "\x21\x22\x23\x24\x25",
		.plaintext =
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd",
		.digest = "\x22\x0b\xf5\x79\xde\xd5\x55\x39"
			  "\x3f\x01\x59\xf6\x6c\x99\x87\x78"
			  "\x22\xa3\xec\xf6\x10\xd1\x55\x21"
			  "\x54\xb4\x1d\x44\xb9\x4d\xb3\xae",
	},
	{
		.ksize = 32,
		.psize = 8,
		.dsize = 32,
		.alg = SCE_SM3,
		.mode = SCE_HMAC,
		.key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b\x0b\x0b",
		.plaintext = "Hi There",
		.digest = "\xc0\xba\x18\xc6\x8b\x90\xc8\x8b"
			  "\xc0\x7d\xe7\x94\xbf\xc7\xd2\xc8"
			  "\xd1\x9e\xc3\x1e\xd8\x77\x3b\xc2"
			  "\xb3\x90\xc9\x60\x4e\x0b\xe1\x1e",
	},
	{
		.ksize = 4,
		.psize = 28,
		.dsize = 32,
		.alg = SCE_SM3,
		.mode = SCE_HMAC,
		.key = "Jefe",
		.plaintext = "what do ya want for nothing?",
		.digest = "\x2e\x87\xf1\xd1\x68\x62\xe6\xd9"
			  "\x64\xb5\x0a\x52\x00\xbf\x2b\x10"
			  "\xb7\x64\xfa\xa9\x68\x0a\x29\x6a"
			  "\x24\x05\xf2\x4b\xec\x39\xf8\x82",
	},

	{
		.ksize = 16,
		.psize = 8,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_HMAC,
		.key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b",
		.plaintext = "Hi There",
		.digest = "\x92\x94\x72\x7a\x36\x38\xbb\x1c"
			  "\x13\xf4\x8e\xf8\x15\x8b\xfc\x9d",
	},
	{
		.ksize = 4,
		.psize = 28,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_HMAC,
		.key = "Jefe",
		.plaintext = "what do ya want for nothing?",
		.digest = "\x75\x0c\x78\x3e\x6a\xb0\xb5\x03"
			  "\xea\xa8\x6e\x31\x0a\x5d\xb7\x38",
	},
	{
		.ksize = 16,
		.psize = 50,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_HMAC,
		.key = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa",
		.plaintext =
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd",
		.digest = "\x56\xbe\x34\x52\x1d\x14\x4c\x88"
			  "\xdb\xb8\xc7\x33\xf0\xe8\xb3\xf6",
	},
	{
		.ksize = 25,
		.psize = 50,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_HMAC,
		.key = "\x01\x02\x03\x04\x05\x06\x07\x08"
		       "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
		       "\x11\x12\x13\x14\x15\x16\x17\x18\x19",
		.plaintext =
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd",
		.digest = "\x69\x7e\xaf\x0a\xca\x3a\x3a\xea"
			  "\x3a\x75\x16\x47\x46\xff\xaa\x79",
	},
	{
		.ksize = 16,
		.psize = 20,
		.dsize = 16,
		.alg = SCE_MD5,
		.mode = SCE_HMAC,
		.key = "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c",
		.plaintext = "Test With Truncation",
		.digest = "\x56\x46\x1e\xf2\x34\x2e\xdc\x00"
			  "\xf9\xba\xb9\x95\x69\x0e\xfd\x4c",
	},

	{
		.ksize = 32,
		.psize = 3,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_HMAC,
		.key = "\x01\x02\x03\x04\x05\x06\x07\x08"
		       "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
		       "\x11\x12\x13\x14\x15\x16\x17\x18"
		       "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20",
		.plaintext = "abc",
		.digest = "\xa2\x1b\x1f\x5d\x4c\xf4\xf7\x3a"
			  "\x4d\xd9\x39\x75\x0f\x7a\x06\x6a"
			  "\x7f\x98\xcc\x13\x1c\xb1\x6a\x66"
			  "\x92\x75\x90\x21\xcf\xab\x81\x81",
	},
	{
		.ksize = 32,
		.psize = 56,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_HMAC,
		.key = "\x01\x02\x03\x04\x05\x06\x07\x08"
		       "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
		       "\x11\x12\x13\x14\x15\x16\x17\x18"
		       "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20",
		.plaintext =
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		.digest = "\x10\x4f\xdc\x12\x57\x32\x8f\x08"
			  "\x18\x4b\xa7\x31\x31\xc5\x3c\xae"
			  "\xe6\x98\xe3\x61\x19\x42\x11\x49"
			  "\xea\x8c\x71\x24\x56\x69\x7d\x30",
	},
	{
		.ksize = 32,
		.psize = 112,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_HMAC,
		.key = "\x01\x02\x03\x04\x05\x06\x07\x08"
		       "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
		       "\x11\x12\x13\x14\x15\x16\x17\x18"
		       "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20",
		.plaintext =
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
		.digest = "\x47\x03\x05\xfc\x7e\x40\xfe\x34"
			  "\xd3\xee\xb3\xe7\x73\xd9\x5a\xab"
			  "\x73\xac\xf0\xfd\x06\x04\x47\xa5"
			  "\xeb\x45\x95\xbf\x33\xa9\xd1\xa3",
	},
	{
		.ksize = 32,
		.psize = 8,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_HMAC,
		.key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b\x0b\x0b",
		.plaintext = "Hi There",
		.digest = "\x19\x8a\x60\x7e\xb4\x4b\xfb\xc6"
			  "\x99\x03\xa0\xf1\xcf\x2b\xbd\xc5"
			  "\xba\x0a\xa3\xf3\xd9\xae\x3c\x1c"
			  "\x7a\x3b\x16\x96\xa0\xb6\x8c\xf7",
	},
	{
		.ksize = 4,
		.psize = 28,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_HMAC,
		.key = "Jefe",
		.plaintext = "what do ya want for nothing?",
		.digest = "\x5b\xdc\xc1\x46\xbf\x60\x75\x4e"
			  "\x6a\x04\x24\x26\x08\x95\x75\xc7"
			  "\x5a\x00\x3f\x08\x9d\x27\x39\x83"
			  "\x9d\xec\x58\xb9\x64\xec\x38\x43",
	},
	{
		.ksize = 32,
		.psize = 50,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_HMAC,
		.key = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		       "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		       "\xaa\xaa\xaa\xaa\xaa\xaa\xaa",
		.plaintext =
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd",
		.digest = "\xcd\xcb\x12\x20\xd1\xec\xcc\xea"
			  "\x91\xe5\x3a\xba\x30\x92\xf9\x62"
			  "\xe5\x49\xfe\x6c\xe9\xed\x7f\xdc"
			  "\x43\x19\x1f\xbd\xe4\x5c\x30\xb0",
	},
	{
		.ksize = 37,
		.psize = 50,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_HMAC,
		.key = "\x01\x02\x03\x04\x05\x06\x07\x08"
		       "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
		       "\x11\x12\x13\x14\x15\x16\x17\x18"
		       "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20"
		       "\x21\x22\x23\x24\x25",
		.plaintext =
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd",
		.digest = "\xd4\x63\x3c\x17\xf6\xfb\x8d\x74"
			  "\x4c\x66\xde\xe0\xf8\xf0\x74\x55"
			  "\x6e\xc4\xaf\x55\xef\x07\x99\x85"
			  "\x41\x46\x8e\xb4\x9b\xd2\xe9\x17",
	},
	{
		.ksize = 32,
		.psize = 20,
		.dsize = 32,
		.alg = SCE_SHA256,
		.mode = SCE_HMAC,
		.key = "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c"
		       "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c"
		       "\x0c\x0c\x0c\x0c\x0c\x0c",
		.plaintext = "Test With Truncation",
		.digest = "\x75\x46\xaf\x01\x84\x1f\xc0\x9b"
			  "\x1a\xb9\xc3\x74\x9a\x5f\x1c\x17"
			  "\xd4\xf5\x89\x66\x8a\x58\x7b\x27"
			  "\x00\xa9\xc9\x7c\x11\x93\xcf\x42",
	},

	{
		.ksize = 20,
		.psize = 8,
		.dsize = 48,
		.alg = SCE_SHA384,
		.mode = SCE_HMAC,
		.key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b",
		.plaintext = "Hi There",
		.digest = "\xaf\xd0\x39\x44\xd8\x48\x95\x62"
			  "\x6b\x08\x25\xf4\xab\x46\x90\x7f"
			  "\x15\xf9\xda\xdb\xe4\x10\x1e\xc6"
			  "\x82\xaa\x03\x4c\x7c\xeb\xc5\x9c"
			  "\xfa\xea\x9e\xa9\x07\x6e\xde\x7f"
			  "\x4a\xf1\x52\xe8\xb2\xfa\x9c\xb6",
	},
	{
		.ksize = 4,
		.psize = 28,
		.dsize = 48,
		.alg = SCE_SHA384,
		.mode = SCE_HMAC,
		.key = "Jefe",
		.plaintext = "what do ya want for nothing?",
		.digest = "\xaf\x45\xd2\xe3\x76\x48\x40\x31"
			  "\x61\x7f\x78\xd2\xb5\x8a\x6b\x1b"
			  "\x9c\x7e\xf4\x64\xf5\xa0\x1b\x47"
			  "\xe4\x2e\xc3\x73\x63\x22\x44\x5e"
			  "\x8e\x22\x40\xca\x5e\x69\xe2\xc7"
			  "\x8b\x32\x39\xec\xfa\xb2\x16\x49",
	},

	{
		.ksize = 20,
		.psize = 8,
		.dsize = 64,
		.alg = SCE_SHA512,
		.mode = SCE_HMAC,
		.key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b",
		.plaintext = "Hi There",
		.digest = "\x87\xaa\x7c\xde\xa5\xef\x61\x9d"
			  "\x4f\xf0\xb4\x24\x1a\x1d\x6c\xb0"
			  "\x23\x79\xf4\xe2\xce\x4e\xc2\x78"
			  "\x7a\xd0\xb3\x05\x45\xe1\x7c\xde"
			  "\xda\xa8\x33\xb7\xd6\xb8\xa7\x02"
			  "\x03\x8b\x27\x4e\xae\xa3\xf4\xe4"
			  "\xbe\x9d\x91\x4e\xeb\x61\xf1\x70"
			  "\x2e\x69\x6c\x20\x3a\x12\x68\x54",
	},
	{
		.ksize = 4,
		.psize = 28,
		.dsize = 64,
		.alg = SCE_SHA512,
		.mode = SCE_HMAC,
		.key = "Jefe",
		.plaintext = "what do ya want for nothing?",
		.digest = "\x16\x4b\x7a\x7b\xfc\xf8\x19\xe2"
			  "\xe3\x95\xfb\xe7\x3b\x56\xe0\xa3"
			  "\x87\xbd\x64\x22\x2e\x83\x1f\xd6"
			  "\x10\x27\x0c\xd7\xea\x25\x05\x54"
			  "\x97\x58\xbf\x75\xc0\x5a\x99\x4a"
			  "\x6d\x03\x4f\x65\xf8\xf0\xe6\xfd"
			  "\xca\xea\xb1\xa3\x4d\x4a\x6b\x4b"
			  "\x63\x6e\x07\x0a\x38\xbc\xe7\x37",
	},

	{
		.ksize = 20,
		.psize = 8,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_HMAC,
		.key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b\x0b",
		.plaintext = "Hi There",
		.digest = "\xb6\x17\x31\x86\x55\x05\x72\x64"
			  "\xe2\x8b\xc0\xb6\xfb\x37\x8c\x8e\xf1"
			  "\x46\xbe",
	},
	{
		.ksize = 4,
		.psize = 28,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_HMAC,
		.key = "Jefe",
		.plaintext = "what do ya want for nothing?",
		.digest = "\xef\xfc\xdf\x6a\xe5\xeb\x2f\xa2\xd2\x74"
			  "\x16\xd5\xf1\x84\xdf\x9c\x25\x9a\x7c\x79",
	},
	{
		.ksize = 20,
		.psize = 50,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_HMAC,
		.key = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		       "\xaa\xaa\xaa\xaa\xaa",
		.plaintext =
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			"\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd",
		.digest = "\x12\x5d\x73\x42\xb9\xac\x11\xcd\x91\xa3"
			  "\x9a\xf4\x8a\xa1\x7b\x4f\x63\xf1\x75\xd3",
	},
	{
		.ksize = 25,
		.psize = 50,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_HMAC,
		.key = "\x01\x02\x03\x04\x05\x06\x07\x08"
		       "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
		       "\x11\x12\x13\x14\x15\x16\x17\x18\x19",
		.plaintext =
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
			"\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd",
		.digest = "\x4c\x90\x07\xf4\x02\x62\x50\xc6\xbc\x84"
			  "\x14\xf9\xbf\x50\xc8\x6c\x2d\x72\x35\xda",
	},
	{
		.ksize = 20,
		.psize = 20,
		.dsize = 20,
		.alg = SCE_SHA1,
		.mode = SCE_HMAC,
		.key = "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c"
		       "\x0c\x0c\x0c\x0c\x0c",
		.plaintext = "Test With Truncation",
		.digest = "\x4c\x1a\x03\x42\x4b\x55\xe0\x7f\xe7\xf2"
			  "\x7b\xe1\xd5\x8b\xb9\x32\x4a\x9a\x5a\x04",
	},

	{
		.ksize = 20,
		.psize = 8,
		.dsize = 28,
		.alg = SCE_SHA224,
		.mode = SCE_HMAC,
		.key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
		       "\x0b\x0b\x0b\x0b",
		/*  ("Hi There") */
		.plaintext = "\x48\x69\x20\x54\x68\x65\x72\x65",
		.digest = "\x89\x6f\xb1\x12\x8a\xbb\xdf\x19"
			  "\x68\x32\x10\x7c\xd4\x9d\xf3\x3f"
			  "\x47\xb4\xb1\x16\x99\x12\xba\x4f"
			  "\x53\x68\x4b\x22",
	},
	{
		.ksize = 4,
		.psize = 28,
		.dsize = 28,
		.alg = SCE_SHA224,
		.mode = SCE_HMAC,
		.key = "Jefe",
		/* ("what do ya want for nothing?") */
		.plaintext = "\x77\x68\x61\x74\x20\x64\x6f\x20"
			     "\x79\x61\x20\x77\x61\x6e\x74\x20"
			     "\x66\x6f\x72\x20\x6e\x6f\x74\x68"
			     "\x69\x6e\x67\x3f",
		.digest = "\xa3\x0e\x01\x09\x8b\xc6\xdb\xbf"
			  "\x45\x69\x0f\x3a\x7e\x9e\x6d\x0f"
			  "\x8b\xbe\xa2\xa3\x9e\x61\x48\x00"
			  "\x8f\xd0\x5e\x44",
	},
};

static void *hash_io_map(void *usr, void *va)
{
	(void)usr;
	return (void *)sce_virt_to_io(va);
}

int hash_test(uint32_t hash_dma_mode, int src_link_num)
{
	const hash_testvec_t *testvec;
	int i, offset, index, ret = 0;
	sce_rsp_t rsp;
	uint8_t *result = NULL, *src = NULL, *tag = NULL;
	sce_queue_handle sce_queue_desc = NULL;
	sce_op_data_t *op_data_tmp = NULL;
	void *op_data_tmp_buf = NULL;
	void **op_data = NULL;
	int num = ARRAY_SIZE(hash_tv_template);
	int enqueue_num = 0;
	sce_link_list_item_t *link_list = NULL;
	uint8_t *src_link = NULL;
	uint64_t tmp = 0;
	int k = 0, temp_src_link_num = 0;
	int err_num = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = hash_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	op_data_tmp_buf = (sce_op_data_t *)malloc(sizeof(sce_op_data_t) * num);
	if (!op_data_tmp_buf) {
		SCE_LOG_ERR("%s:alloc fail!\n", __func__);
		ret = -1;
		goto out;
	}
	op_data_tmp = op_data_tmp_buf;

	op_data = (void **)malloc(sizeof(void *) * num);
	if (!op_data) {
		SCE_LOG_ERR("%s:alloc fail!\n", __func__);
		ret = -1;
		goto out;
	}

	if (sce_request_queue(SCE_HASH, &sce_queue_desc)) {
		ret = -1;
		goto out;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &result);
	if (ret) {
		goto out;
	}

	ret = sce_requset_dmabuf(0x100000, &src);
	if (ret) {
		goto out;
	}

	ret = sce_requset_dmabuf(0x100000, &src_link);
	if (ret) {
		goto out;
	}

	src_link_num = src_link_num <= 32 ? src_link_num : 32;
	offset = 0;
	for (i = 0; i < ARRAY_SIZE(hash_tv_template); i++) {
		testvec = &hash_tv_template[i];
		memset(result + offset, 0xa5, testvec->dsize + 64);
		memcpy(src + offset, testvec->plaintext, testvec->psize);
		op_data_tmp->hash.in_length = testvec->psize;

		if (hash_dma_mode && (testvec->psize > 1)) {
			temp_src_link_num = src_link_num < testvec->psize ?
						    src_link_num :
						    testvec->psize;
			link_list =
				(sce_link_list_item_t
					 *)(src_link +
					    i * (src_link_num + 2) *
						    sizeof(sce_link_list_item_t));
			if (((uint64_t)link_list) & 0x1f) {
				link_list =
					(sce_link_list_item_t
						 *)((uint64_t)link_list + 0x1f);
				link_list = (sce_link_list_item_t
						     *)((uint64_t)link_list &
							(~0x1f));
			}
			tmp = sce_virt_to_io(src + offset);
			for (k = 0; k < temp_src_link_num - 1; k++) {
				link_list[k].address = tmp + k;
				link_list[k].address |= 0x300000000000;
				link_list[k].size = 0;
				link_list[k].reserved = 0;
			}
			// end of link_list
			link_list[k].address = tmp + k;
			link_list[k].address |= 0x300000000000;
			link_list[k].size = testvec->psize - k - 1;
			link_list[k].reserved = 0;
			k++;

			link_list[k].address = SCE_LINK_END_ADDRESS;
			link_list[k].size = SCE_LINK_END_SIZE;
			link_list[k].reserved = SCE_LINK_END_SIZE;
			op_data_tmp->hash.dma_mode = 1;
			op_data_tmp->hash.in = (uint64_t)link_list;
		} else {
			op_data_tmp->hash.dma_mode = 0;
			op_data_tmp->hash.in = (uint64_t)src + offset;
		}

		op_data_tmp->op_type = SCE_HASH;
		op_data_tmp->hash.alg = testvec->alg;
		op_data_tmp->hash.mode = testvec->mode;

		op_data_tmp->hash.out = (uint64_t)result + offset;
		op_data_tmp->hash.key = 0;
		op_data_tmp->hash.tag = i;
		op_data_tmp->hash.tag <<= 48;
		op_data_tmp->hash.tag |= (uint64_t)result + offset;

		op_data[i] = op_data_tmp;
		op_data_tmp++;
		offset += testvec->psize + testvec->dsize + 64;
		offset += (64 - (offset & 0x3F)) & 0x3F;
	}
	enqueue_num = sce_enqueue(sce_queue_desc, op_data, num);
	for (i = 0; i < enqueue_num; i++) {
		rsp.state = 0;
		while (running) {
			if (sce_dequeue(sce_queue_desc, &rsp, 1) == 1)
				break;
			usleep(1000);
		}

		index = rsp.tag >> 48;
		tag = (uint8_t *)(rsp.tag & 0xFFFFFFFFFFFF);
		testvec = &hash_tv_template[index];
		if (rsp.state != CMD_SUCCESS) {
			SCE_LOG_INF(
				"1:hash dgst[%d] fail!alg[0x%x], psize[%d], res[0x%x]\n",
				index, testvec->alg, testvec->psize, rsp.state);
			continue;
		}

		if (memcmp(tag, testvec->digest, testvec->dsize)) {
			printf("2:hash dgst[%d] error!alg[0x%x], psize[%d]\n",
			       index, testvec->alg, testvec->psize);
			print_buf("digest", tag, testvec->dsize);
			print_buf("testvec digest", (uint8_t *)testvec->digest,
				  testvec->dsize);
			err_num++;
			continue;
		}

		if (memcmp(tag + testvec->dsize,
			   "\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5"
			   "\xa5\xa5",
			   16)) {
			SCE_LOG_INF(
				"3:hash dgst[%d] error!alg[0x%x], psize[%d]\n",
				index, testvec->alg, testvec->psize);
			print_buf("dgst add", tag + testvec->dsize, 64);
			err_num++;
			continue;
		}
	}
	if (!err_num)
		SCE_LOG_ALWAYS("hash test success\n");

out:
	if (op_data) {
		free(op_data);
		op_data = NULL;
	}

	if (op_data_tmp_buf) {
		free(op_data_tmp_buf);
		op_data_tmp_buf = NULL;
	}

	sce_release_dmabuf(src_link);

	sce_release_dmabuf(result);

	sce_release_dmabuf(src);

	if (sce_queue_desc) {
		sce_stop_queue(sce_queue_desc);
		sce_release_queue(sce_queue_desc);
	}

	sce_exit_lib();
	return ret;
}

int hmac_test(uint32_t hash_dma_mode, int src_link_num)
{
	const hmac_testvec_t *testvec;
	int i, offset, index, ret = 0;
	sce_rsp_t rsp;
	uint8_t *result = NULL, *src = NULL, *key = NULL, *tag = NULL;
	sce_queue_handle sce_queue_desc = NULL;
	sce_op_data_t *op_data_tmp = NULL;
	void *op_data_tmp_buf = NULL;
	void **op_data = NULL;
	int num = ARRAY_SIZE(hmac_tv_template);
	int enqueue_num = 0;
	sce_link_list_item_t *link_list = NULL;
	uint8_t *src_link = NULL;
	uint64_t tmp = 0;
	int k = 0, temp_src_link_num = 0;
	int err_num = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = hash_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	op_data_tmp_buf = (sce_op_data_t *)malloc(sizeof(sce_op_data_t) * num);
	if (!op_data_tmp_buf) {
		SCE_LOG_ERR("%s:alloc fail!\n", __func__);
		ret = -1;
		goto out;
	}
	op_data_tmp = op_data_tmp_buf;

	op_data = (void **)malloc(sizeof(void *) * num);
	if (!op_data) {
		SCE_LOG_ERR("%s:alloc fail!\n", __func__);
		ret = -1;
		goto out;
	}

	if (sce_request_queue(SCE_HASH, &sce_queue_desc)) {
		ret = -1;
		goto out;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &result);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &key);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &src_link);
	if (ret) {
		goto out;
	}

	src_link_num = src_link_num <= 32 ? src_link_num : 32;
	offset = 0;
	for (i = 0; i < ARRAY_SIZE(hmac_tv_template); i++) {
		testvec = &hmac_tv_template[i];
		memset(result + offset, 0xa5, testvec->dsize + 64);
		memset(key + offset, 0, 128);
		memcpy(key + offset, testvec->key, testvec->ksize);
		memcpy(src + offset, testvec->plaintext, testvec->psize);
		op_data_tmp->hash.in_length = testvec->psize;

		op_data_tmp->op_type = SCE_HASH;
		op_data_tmp->hash.alg = testvec->alg;
		op_data_tmp->hash.mode = testvec->mode;
		op_data_tmp->hash.out = (uint64_t)result + offset;
		op_data_tmp->hash.key = (uint64_t)key + offset;
		op_data_tmp->hash.tag = i;
		op_data_tmp->hash.tag <<= 48;
		op_data_tmp->hash.tag |= (uint64_t)result + offset;

		if (hash_dma_mode && (testvec->psize > 1)) {
			temp_src_link_num = src_link_num < testvec->psize ?
						    src_link_num :
						    testvec->psize;
			link_list =
				(sce_link_list_item_t
					 *)(src_link +
					    i * (src_link_num + 2) *
						    sizeof(sce_link_list_item_t));
			if (((uint64_t)link_list) & 0x1f) {
				link_list =
					(sce_link_list_item_t
						 *)((uint64_t)link_list + 0x1f);
				link_list = (sce_link_list_item_t
						     *)((uint64_t)link_list &
							(~0x1f));
			}
			tmp = sce_virt_to_io(src + offset);
			for (k = 0; k < temp_src_link_num - 1; k++) {
				link_list[k].address = tmp + k;
				link_list[k].address |= 0x300000000000;
				link_list[k].size = 0;
				link_list[k].reserved = 0;
			}
			// end of link_list
			link_list[k].address = tmp + k;
			link_list[k].address |= 0x300000000000;
			link_list[k].size = testvec->psize - k - 1;
			link_list[k].reserved = 0;
			k++;

			link_list[k].address = SCE_LINK_END_ADDRESS;
			link_list[k].size = SCE_LINK_END_SIZE;
			link_list[k].reserved = SCE_LINK_END_SIZE;

			op_data_tmp->hash.dma_mode = 1;
			op_data_tmp->hash.in = (uint64_t)link_list;
		} else {
			op_data_tmp->hash.dma_mode = 0;
			op_data_tmp->hash.in = (uint64_t)src + offset;
		}

		op_data[i] = op_data_tmp;
		op_data_tmp++;

		offset += testvec->psize + testvec->dsize + 128;
		offset += (64 - (offset & 0x3F)) & 0x3F;
	}
	enqueue_num = sce_enqueue(sce_queue_desc, op_data, num);

	for (i = 0; i < enqueue_num; i++) {
		rsp.state = 0;
		while (running) {
			if (sce_dequeue(sce_queue_desc, &rsp, 1) == 1)
				break;
			usleep(1000);
		}

		index = rsp.tag >> 48;
		if (rsp.state != CMD_SUCCESS) {
			SCE_LOG_ERR(
				"1:hmac dgst[%d] fail!alg[0x%x], psize[%d], res[0x%x]\n",
				index, testvec->alg, testvec->psize, rsp.state);
			continue;
		}

		tag = (uint8_t *)(rsp.tag & 0xFFFFFFFFFFFF);
		testvec = &hmac_tv_template[index];

		if (memcmp(tag, testvec->digest, testvec->dsize)) {
			SCE_LOG_INF(
				"2:hmac dgst[%d] error!alg[0x%x], psize[%d]\n",
				index, testvec->alg, testvec->psize);
			print_buf("digest", tag, testvec->dsize);
			print_buf("testvec digest", (uint8_t *)testvec->digest,
				  testvec->dsize);
			err_num++;
			continue;
		}

		if (memcmp(tag + testvec->dsize,
			   "\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5"
			   "\xa5\xa5",
			   16)) {
			SCE_LOG_ERR(
				"3:hmac dgst[%d] error!alg[0x%x], psize[%d]\n",
				index, testvec->alg, testvec->psize);
			print_buf("dgst add", tag + testvec->dsize, 64);
			err_num++;
			continue;
		}
	}
	if (!err_num)
		SCE_LOG_ALWAYS("hash test success\n");

out:
	if (op_data) {
		free(op_data);
		op_data = NULL;
	}

	if (op_data_tmp_buf) {
		free(op_data_tmp_buf);
		op_data_tmp_buf = NULL;
	}

	sce_release_dmabuf(src_link);

	sce_release_dmabuf(result);

	sce_release_dmabuf(src);

	sce_release_dmabuf(key);

	if (sce_queue_desc) {
		sce_stop_queue(sce_queue_desc);
		sce_release_queue(sce_queue_desc);
	}

	sce_exit_lib();
	return ret;
}

int hash_speed_test(int queue_num, long num, int alg, int mode, int size,
		    uint32_t hash_dma_mode, int src_link_num)
{
	struct timeval start, end;
	long cmd_total = 0, rsp_total = 0, i, ret = -1;
	sce_rsp_t rsp[8];
	unsigned long time = 0;
	sce_queue_handle sce_queue_desc[4] = { NULL };
	uint8_t *buf = NULL;
	uint64_t result = 0, key = 0;
	sce_op_data_t op_data_tmp;
	void *op_data[1] = { &op_data_tmp };
	int enqueue_num = 0;
	sce_link_list_item_t *link_list = NULL;
	uint8_t *src_link = NULL;
	uint8_t *in_src = NULL;
	uint64_t tmp = 0, ult_in = 0;
	int k = 0, temp_src_link_num = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = hash_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	for (i = 0; i < queue_num; i++) {
		if (sce_request_queue(SCE_HASH, &sce_queue_desc[i])) {
			goto out;
		}

		ret = sce_init_queue(sce_queue_desc[i]);
		if (ret)
			goto out;
	}

	ret = sce_requset_dmabuf(256, &buf);
	if (ret) {
		goto out;
	}
	result = (uint64_t)(buf);
	ret = sce_requset_dmabuf(256, &buf);
	if (ret) {
		goto out;
	}

	key = (uint64_t)(buf);

	ret = sce_requset_dmabuf(size, &in_src);
	if (ret) {
		goto out;
	}

	ret = sce_requset_dmabuf(0x100000, &src_link);
	if (ret) {
		goto out;
	}

	cmd_total = num;
	rsp_total = 0;

	src_link_num = src_link_num <= 32 ? src_link_num : 32;
	if (hash_dma_mode && src_link_num > 1) {
		if (size > 1) {
			temp_src_link_num = src_link_num < size ? src_link_num :
								  size;
			link_list =
				(sce_link_list_item_t
					 *)(src_link +
					    (src_link_num +
					     2) * sizeof(sce_link_list_item_t));
			if (((uint64_t)link_list) & 0x1f) {
				link_list =
					(sce_link_list_item_t
						 *)((uint64_t)link_list + 0x1f);
				link_list = (sce_link_list_item_t
						     *)((uint64_t)link_list &
							(~0x1f));
			}
			tmp = sce_virt_to_io(in_src);
			for (k = 0; k < temp_src_link_num - 1; k++) {
				link_list[k].address = tmp + k;
				link_list[k].address |= 0x300000000000;
				link_list[k].size = 0;
				link_list[k].reserved = 0;
			}
			link_list[k].address = tmp + k;
			link_list[k].address |= 0x300000000000;
			link_list[k].size = size - k - 1;
			link_list[k].reserved = 0;
			k++;

			link_list[k].address = SCE_LINK_END_ADDRESS;
			link_list[k].size = SCE_LINK_END_SIZE;
			link_list[k].reserved = SCE_LINK_END_SIZE;

			hash_dma_mode = 1;
			ult_in = (uint64_t)link_list;
		} else {
			hash_dma_mode = 0;
			ult_in = (uint64_t)in_src;
		}
	} else {
		hash_dma_mode = 0;
		ult_in = (uint64_t)in_src;
	}

	gettimeofday(&start, NULL);

	do {
		for (i = 0; i < queue_num; i++) {
			if (cmd_total) {
				op_data_tmp.op_type = SCE_HASH;
				op_data_tmp.hash.alg = alg_val[alg];
				op_data_tmp.hash.mode = mode_val[mode];
				op_data_tmp.hash.in_length = size;
				op_data_tmp.hash.out = result;
				op_data_tmp.hash.key = key;
				op_data_tmp.hash.dma_mode = hash_dma_mode;
				op_data_tmp.hash.in = ult_in;

				enqueue_num = sce_enqueue(sce_queue_desc[i],
							  op_data, 1);
				cmd_total -= enqueue_num;
			}
			rsp_total += sce_dequeue(sce_queue_desc[i], rsp, 8);
		}
		if (rsp_total >= num)
			break;
	} while (running);
	gettimeofday(&end, NULL);

	for (i = 0; i < queue_num; i++) {
		sce_stop_queue(sce_queue_desc[i]);
	}

	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS("alg[%s] mode[%s] qnum[0x%x] testnum:%ld, testsize:%d, "
		       "complete:%ld,  speed:%ld Mbps-%ld pps\n",
		       alg_name[alg], mode_name[mode], queue_num, num, size,
		       rsp_total, (long)rsp_total * size * 8 / time,
		       (long)rsp_total * 1000000ul / time);

out:
	sce_release_dmabuf(in_src);

	sce_release_dmabuf(src_link);

	sce_release_dmabuf((uint8_t *)result);

	sce_release_dmabuf((uint8_t *)key);

	for (i = 0; i < queue_num; i++) {
		if (sce_queue_desc[i])
			sce_release_queue(sce_queue_desc[i]);
	}

	sce_exit_lib();
	return ret;
}

int hash_delay_test(int alg, int mode, int size, uint64_t hash_dma_mode,
		    int src_link_num)
{
	struct timeval start, end;
	long rsp_total = 0, ret = -1;
	sce_rsp_t rsp[8];
	unsigned long time;
	sce_queue_handle sce_queue_desc = NULL;
	uint8_t *in = NULL, *result = NULL, *key = NULL;
	sce_op_data_t *op_data_tmp = NULL;
	void *op_data_tmp_buf = NULL;
	void **op_data = NULL;
	sce_link_list_item_t *link_list = NULL;
	uint8_t *src_link = NULL;
	uint64_t tmp = 0;
	int k = 0, temp_src_link_num = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = hash_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	op_data_tmp_buf = (sce_op_data_t *)malloc(sizeof(sce_op_data_t));
	if (!op_data_tmp_buf) {
		SCE_LOG_ERR("%s:alloc fail!\n", __func__);
		goto out;
	}
	op_data_tmp = op_data_tmp_buf;

	op_data = (void **)malloc(sizeof(void *));
	if (!op_data) {
		SCE_LOG_ERR("%s:alloc fail!\n", __func__);
		goto out;
	}

	if (sce_request_queue(SCE_HASH, &sce_queue_desc)) {
		goto out;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(size, &in);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(256, &result);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(256, &key);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &src_link);
	if (ret) {
		goto out;
	}

	rsp_total = 0;
	gettimeofday(&start, NULL);

	op_data_tmp->op_type = SCE_HASH;
	op_data_tmp->hash.alg = alg_val[alg];
	op_data_tmp->hash.mode = mode_val[mode];
	op_data_tmp->hash.in_length = size;
	op_data_tmp->hash.out = (uint64_t)(result);
	op_data_tmp->hash.key = (uint64_t)(key);

	if (hash_dma_mode && src_link_num > 1) {
		if (size > 1) {
			temp_src_link_num = src_link_num < size ? src_link_num :
								  size;
			link_list =
				(sce_link_list_item_t
					 *)(src_link +
					    (src_link_num +
					     2) * sizeof(sce_link_list_item_t));
			if (((uint64_t)link_list) & 0x1f) {
				link_list =
					(sce_link_list_item_t
						 *)((uint64_t)link_list + 0x1f);
				link_list = (sce_link_list_item_t
						     *)((uint64_t)link_list &
							(~0x1f));
			}
			tmp = sce_virt_to_io(in);
			for (k = 0; k < temp_src_link_num - 1; k++) {
				link_list[k].address = tmp + k;
				link_list[k].address |= 0x300000000000;
				link_list[k].size = 0;
				link_list[k].reserved = 0;
			}
			// end of link_list
			link_list[k].address = tmp + k;
			link_list[k].address |= 0x300000000000;
			link_list[k].size = size - k - 1;
			link_list[k].reserved = 0;
			k++;

			link_list[k].address = SCE_LINK_END_ADDRESS;
			link_list[k].size = SCE_LINK_END_SIZE;
			link_list[k].reserved = SCE_LINK_END_SIZE;

			op_data_tmp->hash.dma_mode = 1;
			op_data_tmp->hash.in = (uint64_t)link_list;
		} else {
			op_data_tmp->hash.dma_mode = 0;
			op_data_tmp->hash.in = (uint64_t)(in);
		}
	} else {
		op_data_tmp->hash.dma_mode = 0;
		op_data_tmp->hash.in = (uint64_t)(in);
	}

	*op_data = op_data_tmp;
	ret = sce_enqueue(sce_queue_desc, op_data, 1);

	do {
		rsp_total += sce_dequeue(sce_queue_desc, rsp, 1);
	} while ((rsp_total == 0) && running);

	gettimeofday(&end, NULL);

	sce_stop_queue(sce_queue_desc);

	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS("alg[%s] mode[%s] testsize:%d  time:%ld us\n",
		       alg_name[alg], mode_name[mode], size, time);

out:
	if (op_data_tmp_buf) {
		free(op_data_tmp_buf);
		op_data_tmp_buf = NULL;
	}

	if (op_data) {
		free(op_data);
		op_data = NULL;
	}

	sce_release_dmabuf(src_link);

	sce_release_dmabuf(in);

	sce_release_dmabuf(result);

	sce_release_dmabuf(key);

	sce_release_queue(sce_queue_desc);

	sce_exit_lib();
	return ret;
}

int hash_sca_speed_test(int queue_num, long num, int hash_alg, int hash_mode,
			int sca_alg, int sca_mode, int op_mode, int size)
{
	struct timeval start, end;
	long hash_cmd_total, sca_cmd_total, hash_rsp_total = 0,
					    sca_rsp_total = 0, i, ret = -1;
	sce_rsp_t rsp[8];
	unsigned long time;
	sce_queue_handle sce_queue_desc[2][4] = { { NULL } };
	uint8_t *buf;
	void *hash_in = NULL, *hash_result = NULL, *hash_key = NULL,
	     *sca_in = NULL, *sca_result = NULL, *sca_key = NULL,
	     *sca_iv = NULL;
	sce_op_data_t op_data;
	void *op_datas[1] = { &op_data };
	int hash_cmd_init = 0, sca_cmd_init = 0;

	if (num > 1023) {
		SCE_LOG_ERR("num exceeded the maximum value 1023!\n");
		return -1;
	}

	if (mode_val[sca_mode] == GCM || mode_val[sca_mode] == CCM ||
	    mode_val[sca_mode] == CMAC || mode_val[sca_mode] == CBC_MAC) {
		SCE_LOG_ERR("hash_sca_speed: unsupport sca mode!\n");
		return -1;
	}

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = hash_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	size = (size + 0xF) & (~0xF);
	for (i = 0; i < queue_num; i++) {
		if (sce_request_queue(SCE_HASH, &sce_queue_desc[SCE_HASH][i])) {
			goto out;
		}

		ret = sce_init_queue(sce_queue_desc[SCE_HASH][i]);
		if (ret)
			goto out;
	}
	for (i = 0; i < queue_num; i++) {
		if (sce_request_queue(SCE_SCA, &sce_queue_desc[SCE_SCA][i])) {
			goto out;
		}

		ret = sce_init_queue(sce_queue_desc[SCE_SCA][i]);
		if (ret)
			goto out;
	}

	ret = sce_requset_dmabuf(size, &buf);
	if (ret)
		goto out;
	hash_in = buf;
	ret = sce_requset_dmabuf(256, &buf);
	if (ret)
		goto out;
	hash_result = buf;
	ret = sce_requset_dmabuf(256, &buf);
	if (ret)
		goto out;
	hash_key = buf;

	ret = sce_requset_dmabuf(size, &buf);
	if (ret)
		goto out;
	sca_in = buf;
	ret = sce_requset_dmabuf(size, &buf);
	if (ret)
		goto out;
	sca_result = buf;
	ret = sce_requset_dmabuf(256, &buf);
	if (ret)
		goto out;
	sca_key = buf;
	ret = sce_requset_dmabuf(256, &buf);
	if (ret)
		goto out;
	sca_iv = buf;

	hash_cmd_total = num;
	sca_cmd_total = num;
	gettimeofday(&start, NULL);

	do {
		for (i = 0; i < queue_num; i++) {
			if (hash_cmd_total) {
				if (hash_cmd_init < 1024) {
					hash_cmd_init++;
					op_data.op_type = SCE_HASH;
					op_data.hash.alg = alg_val[hash_alg];
					op_data.hash.mode = mode_val[hash_mode];
					op_data.hash.in_length = size;
					op_data.hash.out =
						(uint64_t)hash_result;
					op_data.hash.key = (uint64_t)hash_key;
					op_data.hash.in = (uint64_t)hash_in;
					op_data.hash.dma_mode = 0;
					hash_cmd_total -= sce_enqueue(
						sce_queue_desc[SCE_HASH][i],
						op_datas, 1);
				}
			}

			if (sca_cmd_total) {
				if (sca_cmd_init < 1024) {
					sca_cmd_init++;
					op_data.op_type = SCE_SCA;
					op_data.sca.alg = alg_val[sca_alg];
					op_data.sca.mode = mode_val[sca_mode];
					op_data.sca.op_mode = op_mode;
					op_data.sca.dma_mode = 0;
					op_data.sca.in_length = size;
					op_data.sca.out_length = size;
					op_data.sca.in = (uint64_t)sca_in;
					op_data.sca.out = (uint64_t)sca_result;
					op_data.sca.key = (uint64_t)sca_key;
					op_data.sca.iv = (uint64_t)sca_iv;
					sca_cmd_total -= sce_enqueue(
						sce_queue_desc[SCE_SCA][i],
						op_datas, 1);
				}
			}
			hash_rsp_total += sce_dequeue(
				sce_queue_desc[SCE_HASH][i], rsp, 8);
			sca_rsp_total +=
				sce_dequeue(sce_queue_desc[SCE_SCA][i], rsp, 8);
		}
		i--;
		if (hash_rsp_total >= num) {
			do {
				sca_rsp_total += sce_dequeue(
					sce_queue_desc[SCE_SCA][i], rsp, 8);
			} while (sca_rsp_total < num);
			break;
		}
		if (sca_rsp_total >= num) {
			do {
				hash_rsp_total += sce_dequeue(
					sce_queue_desc[SCE_HASH][i], rsp, 8);
			} while (hash_rsp_total < num);
			break;
		}

	} while (running);
	gettimeofday(&end, NULL);

	for (i = 0; i < queue_num; i++) {
		sce_stop_queue(sce_queue_desc[SCE_HASH][i]);
		sce_stop_queue(sce_queue_desc[SCE_SCA][i]);
	}

	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS(
		"hash_alg[%s] hash_mode[%s] qnum[0x%x] testnum:%ld, testsize:%d, "
		"complete:%ld,  speed:%ld Mbps-%ld pps\n",
		alg_name[hash_alg], mode_name[hash_mode], queue_num, num, size,
		hash_rsp_total, (long)hash_rsp_total * size * 8 / time,
		(long)hash_rsp_total * 1000000ul / time);
	SCE_LOG_ALWAYS(
		"sca_alg[%s] sca_mode[%s] qnum[0x%x] testnum:%ld, testsize:%d, "
		"complete:%ld,  speed:%ld Mbps-%ld pps\n",
		alg_name[sca_alg], mode_name[sca_mode], queue_num, num, size,
		sca_rsp_total, (long)sca_rsp_total * size * 8 / time,
		(long)sca_rsp_total * 1000000ul / time);
out:
	for (i = 0; i < queue_num; i++) {
		if (sce_queue_desc[SCE_HASH][i])
			sce_release_queue(sce_queue_desc[SCE_HASH][i]);
	}
	for (i = 0; i < queue_num; i++) {
		if (sce_queue_desc[SCE_SCA][i])
			sce_release_queue(sce_queue_desc[SCE_SCA][i]);
	}

	sce_release_dmabuf(hash_in);

	sce_release_dmabuf(hash_result);

	sce_release_dmabuf(hash_key);

	sce_release_dmabuf(sca_in);

	sce_release_dmabuf(sca_result);

	sce_release_dmabuf(sca_key);

	sce_release_dmabuf(sca_iv);

	sce_exit_lib();

	return ret;
}
