#include <stdio.h>
#include <sys/time.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <dsce/sce.h>
#include <dsce/sce_log.h>
#include <openssl/evp.h>

#include "test.h"

extern int pacc_ceu_genkey_sm2(uint8_t *priv_key_addr, uint8_t *pub_key_addr);

extern int pacc_ceu_sign_sm2(uint8_t *e_addr, uint8_t *priv_key_addr,
			     uint8_t *result_addr);
extern int pacc_ceu_verify_sm2(uint8_t *e_signature_addr,
			       uint8_t *pub_key_addr);

extern int pacc_ceu_enc_sm2(uint32_t msg_lengths, uint8_t *msg_addr,
			    uint8_t *pub_key_addr, uint8_t *result_addr);

extern int pacc_ceu_dec_sm2(uint32_t msg_lengths, uint8_t *msg_addr,
			    uint8_t *priv_key_addr, uint8_t *result_addr);

extern int pacc_ceu_exchange_sm2(uint16_t role, uint16_t keysize,
				 uint8_t *key_z_addr, uint8_t *result_addr);

#define BUF_SIZE (1024 * 1024)

static int sm2_sign_digest(uint8_t *user_id, uint32_t id_len, uint8_t *pubkey,
			   uint8_t *msg, uint32_t msg_len, uint8_t *E)
{
	uint8_t id_bit_len[2];
	uint8_t Z[32];
	uint32_t sm2p256v1_a[8] = { 0xFFFFFFFC, 0xFFFFFFFF, 0x00000000,
				    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
				    0xFFFFFFFF, 0xFFFFFFFE };
	uint32_t sm2p256v1_b[8] = { 0x4D940E93, 0xDDBCBD41, 0x15AB8F92,
				    0xF39789F5, 0xCF6509A7, 0x4D5A9E4B,
				    0x9D9F5E34, 0x28E9FA9E };
	uint32_t sm2p256v1_Gx[8] = { 0x334C74C7, 0x715A4589, 0xF2660BE1,
				     0x8FE30BBF, 0x6A39C994, 0x5F990446,
				     0x1F198119, 0x32C4AE2C };
	uint32_t sm2p256v1_Gy[8] = { 0x2139F0A0, 0x02DF32E5, 0xC62A4740,
				     0xD0A9877C, 0x6B692153, 0x59BDCEE3,
				     0xF4F6779C, 0xBC3736A2 };
	uint8_t id[16] = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
			   0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38 };
	int i;
	uint8_t tmp, *a, *b, *x, *y;
	EVP_MD_CTX *ctx = NULL;

	if (user_id == NULL || pubkey == NULL || msg == NULL || E == NULL)
		return -1;
	if (id_len == 0 || msg_len == 0)
		return -1;

	a = (uint8_t *)sm2p256v1_a;
	b = (uint8_t *)sm2p256v1_b;
	x = (uint8_t *)sm2p256v1_Gx;
	y = (uint8_t *)sm2p256v1_Gy;
	for (i = 0; i < 16; i++) {
		tmp = a[i];
		a[i] = a[31 - i];
		a[31 - i] = tmp;
		tmp = b[i];
		b[i] = b[31 - i];
		b[31 - i] = tmp;
		tmp = x[i];
		x[i] = x[31 - i];
		x[31 - i] = tmp;
		tmp = y[i];
		y[i] = y[31 - i];
		y[31 - i] = tmp;
	}

	if ((user_id == NULL) || (id_len == 0)) {
		user_id = id;
		id_len = 16;
	}

	if (id_len >= (1 << 13))
		return -1;

	id_bit_len[0] = ((id_len << 3) >> 8) & 0xFF;
	id_bit_len[1] = (id_len << 3) & 0xFF;

	ctx = EVP_MD_CTX_new();
	if (ctx == NULL) {
		SCE_LOG_INF("EVP_MD_CTX_new fail!\n");
		return -1;
	}

	EVP_DigestInit_ex(ctx, EVP_sm3(), NULL);
	EVP_DigestUpdate(ctx, id_bit_len, 2);
	EVP_DigestUpdate(ctx, user_id, id_len);
	EVP_DigestUpdate(ctx, sm2p256v1_a, 32);
	EVP_DigestUpdate(ctx, sm2p256v1_b, 32);
	EVP_DigestUpdate(ctx, sm2p256v1_Gx, 32);
	EVP_DigestUpdate(ctx, sm2p256v1_Gy, 32);
	EVP_DigestUpdate(ctx, pubkey, 64);
	EVP_DigestFinal_ex(ctx, Z, NULL);

	EVP_DigestInit_ex(ctx, EVP_sm3(), NULL);
	EVP_DigestUpdate(ctx, Z, 32);
	EVP_DigestUpdate(ctx, msg, msg_len);
	EVP_DigestFinal_ex(ctx, E, NULL);

	EVP_MD_CTX_free(ctx);

	return 0;
}

static uint32_t get_rand(uint8_t *a, uint32_t byteLen)
{
	int result = 0;
	int fd = open("/dev/urandom", O_RDONLY);
	if (fd < 0) {
		fd = open("/dev/random", O_RDONLY);
		if (fd < 0) {
			fprintf(stderr, "get rand fail\n");
			return 0;
		}
	}

	result = read(fd, a, byteLen);
	if (result < 0) {
		fprintf(stderr, "get rand fail 2\n");
	}

	close(fd);
	return result;
}

int kernel_sm2_enc_test(int testsize, int testcount)
{
	struct timeval start, end;
	unsigned long time;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 };

	uint8_t prikey[32] = { 0x83, 0xdc, 0xa0, 0xad, 0xa3, 0xd7, 0x7e, 0xa4,
			       0xed, 0x20, 0x89, 0x56, 0x1d, 0xc3, 0x96, 0x37,
			       0x5e, 0xa9, 0xce, 0x50, 0x63, 0x29, 0xe4, 0x9c,
			       0x0d, 0xe4, 0x99, 0x49, 0x0a, 0xcd, 0x03, 0x78 };
	uint8_t pubkey[65] = { 0x04, 0xd7, 0x4e, 0xd1, 0x3e, 0x2d, 0xc9, 0x91,
			       0x14, 0x73, 0x16, 0x38, 0x5b, 0x67, 0x4b, 0xd5,
			       0x60, 0x01, 0xa6, 0x0f, 0x03, 0x9b, 0xfc, 0x01,
			       0x88, 0x6a, 0xeb, 0x52, 0xa3, 0x43, 0x0c, 0xdc,
			       0xbc, 0xbc, 0x01, 0xa4, 0x1e, 0x32, 0xce, 0x92,
			       0xb9, 0x4e, 0x50, 0x44, 0x2e, 0xb1, 0x5c, 0xd4,
			       0x24, 0xc1, 0x23, 0x7e, 0xc4, 0x2e, 0x44, 0x2c,
			       0x90, 0x95, 0x6a, 0x26, 0x46, 0x43, 0x93, 0x12,
			       0x26 };

	if (testsize <= 0)
		return -1;

	get_rand(input, testsize);
	gettimeofday(&start, NULL);

	// print_buf("enc input", input, testsize);
	uint32_t outlen;
	outlen = testsize + 256;
	int tmp;
	for (tmp = 0; tmp < testcount; tmp++) {
		outlen = testsize + 256;
		sce_sm2_enc(input, testsize, pubkey, sizeof(pubkey), output,
			    &outlen);
	}
	// print_buf("enc output", output, outlen);

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	SCE_LOG_INF(
		"SM2 encryp success, testcount:%d, testsize:%d, time:%lu us, speed:%lu pps\n",
		testcount, testsize, time, testcount * 1000000ul / time);

	/* decode first */
	if (sce_sm2decrypt_decode(output, outlen, cmpbuf, &outlen)) {
		goto out;
	}

	/* 解密 */
	if (pacc_ceu_dec_sm2(outlen, cmpbuf, prikey, output)) {
		SCE_LOG_ERR("pacc_ceu_dec_sm2 error\r\n");
		goto out;
	}

	if (memcmp(input, output, testsize)) {
		print_buf("input", input, testsize);
		print_buf("output", output, testsize);
		SCE_LOG_ERR("sm2 encrypt error!\n");
		goto out;
	} else
		SCE_LOG_INF("sm2 enc (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

int kernel_sm2_dec_test(int testsize, int testcount)
{
	struct timeval start, end;
	unsigned long time;
	long len;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 };
	uint8_t prikey[32] = { 0x83, 0xdc, 0xa0, 0xad, 0xa3, 0xd7, 0x7e, 0xa4,
			       0xed, 0x20, 0x89, 0x56, 0x1d, 0xc3, 0x96, 0x37,
			       0x5e, 0xa9, 0xce, 0x50, 0x63, 0x29, 0xe4, 0x9c,
			       0x0d, 0xe4, 0x99, 0x49, 0x0a, 0xcd, 0x03, 0x78 };
	uint8_t pubkey[65] = { 0x04, 0xd7, 0x4e, 0xd1, 0x3e, 0x2d, 0xc9, 0x91,
			       0x14, 0x73, 0x16, 0x38, 0x5b, 0x67, 0x4b, 0xd5,
			       0x60, 0x01, 0xa6, 0x0f, 0x03, 0x9b, 0xfc, 0x01,
			       0x88, 0x6a, 0xeb, 0x52, 0xa3, 0x43, 0x0c, 0xdc,
			       0xbc, 0xbc, 0x01, 0xa4, 0x1e, 0x32, 0xce, 0x92,
			       0xb9, 0x4e, 0x50, 0x44, 0x2e, 0xb1, 0x5c, 0xd4,
			       0x24, 0xc1, 0x23, 0x7e, 0xc4, 0x2e, 0x44, 0x2c,
			       0x90, 0x95, 0x6a, 0x26, 0x46, 0x43, 0x93, 0x12,
			       0x26 };
	if (testsize <= 0)
		return -1;

	get_rand(input, testsize);
	uint32_t outlen;

	if (pacc_ceu_enc_sm2(testsize, input, pubkey + 1, cmpbuf)) {
		SCE_LOG_INF("enc error!\r\n");
		return -1;
	}

	/* encode ... */
	sce_sm2encrypt_encode(cmpbuf, testsize + 97, output, &outlen);
	len = outlen;

	// print_buf("get pacc_ceu_enc_sm2 output", output, testsize + 97);

	gettimeofday(&start, NULL);

	int tmp;
	outlen = len;
	for (tmp = 0; tmp < testcount; tmp++) {
		sce_sm2_dec(output, len, prikey, sizeof(prikey), cmpbuf,
			    &outlen);
	}

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	SCE_LOG_INF(
		"SM2 decryp success, testcount:%d, testsize:%d, time:%lu us, speed:%lu pps\n",
		testcount, testsize, time, testcount * 1000000ul / time);
	if (outlen != testsize) {
		SCE_LOG_INF("sm2 decrypt size error!\n");
		goto out;
	}

	// print_buf("cmpbuf", cmpbuf, testsize);
	if (memcmp(input, cmpbuf, testsize)) {
		print_buf("input", input, testsize);
		print_buf("cmpbuf", cmpbuf, testsize);

		SCE_LOG_INF("sm2 decrypt error!\n");
		goto out;
	} else
		SCE_LOG_INF("sm2 dec (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

int kernel_sm2_sign_test(int testsize, int testcount)
{
	struct timeval start, end;
	unsigned long time;
	int ret;

	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 };

	uint8_t prikey[32] = { 0x83, 0xdc, 0xa0, 0xad, 0xa3, 0xd7, 0x7e, 0xa4,
			       0xed, 0x20, 0x89, 0x56, 0x1d, 0xc3, 0x96, 0x37,
			       0x5e, 0xa9, 0xce, 0x50, 0x63, 0x29, 0xe4, 0x9c,
			       0x0d, 0xe4, 0x99, 0x49, 0x0a, 0xcd, 0x03, 0x78 };
	uint8_t pubkey[65] = { 0x04, 0xd7, 0x4e, 0xd1, 0x3e, 0x2d, 0xc9, 0x91,
			       0x14, 0x73, 0x16, 0x38, 0x5b, 0x67, 0x4b, 0xd5,
			       0x60, 0x01, 0xa6, 0x0f, 0x03, 0x9b, 0xfc, 0x01,
			       0x88, 0x6a, 0xeb, 0x52, 0xa3, 0x43, 0x0c, 0xdc,
			       0xbc, 0xbc, 0x01, 0xa4, 0x1e, 0x32, 0xce, 0x92,
			       0xb9, 0x4e, 0x50, 0x44, 0x2e, 0xb1, 0x5c, 0xd4,
			       0x24, 0xc1, 0x23, 0x7e, 0xc4, 0x2e, 0x44, 0x2c,
			       0x90, 0x95, 0x6a, 0x26, 0x46, 0x43, 0x93, 0x12,
			       0x26 };
	uint8_t userid[16] = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
			       0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38 };
	uint8_t E[1024];
	if (testsize <= 0)
		return -1;

	get_rand(input, testsize);

	gettimeofday(&start, NULL);

	if (sm2_sign_digest(userid, 16, pubkey + 1, input, testsize, E)) {
		SCE_LOG_INF("sm2_sign_digest failed!\n");
		goto out;
	}
	int tmp;
	uint32_t outlen;
	outlen = 256;
	for (tmp = 0; tmp < testcount; tmp++) {
		sce_sm2_sign(E, 32, prikey, sizeof(prikey), output, &outlen);
	}

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	SCE_LOG_INF(
		"SM2 sign success, testcount:%d, testsize:%d, time:%lu us, speed:%lu pps\n",
		testcount, testsize, time, testcount * 1000000ul / time);

	if (sce_sm2signature_decode(output, outlen, E + 32)) {
		goto out;
	}

	ret = pacc_ceu_verify_sm2(E, pubkey + 1);
	if (ret) {
		SCE_LOG_ERR("verify error %d\r\n", ret);
		goto out;
	}

	SCE_LOG_INF("sm2 sign (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

int kernel_sm2_verify_test(int testsize, int testcount)
{
	struct timeval start, end;
	unsigned long time;
	int ret;
	uint32_t len;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 };
	uint8_t cmd[1024];
	uint8_t prikey[32] = { 0x83, 0xdc, 0xa0, 0xad, 0xa3, 0xd7, 0x7e, 0xa4,
			       0xed, 0x20, 0x89, 0x56, 0x1d, 0xc3, 0x96, 0x37,
			       0x5e, 0xa9, 0xce, 0x50, 0x63, 0x29, 0xe4, 0x9c,
			       0x0d, 0xe4, 0x99, 0x49, 0x0a, 0xcd, 0x03, 0x78 };
	uint8_t pubkey[65] = { 0x04, 0xd7, 0x4e, 0xd1, 0x3e, 0x2d, 0xc9, 0x91,
			       0x14, 0x73, 0x16, 0x38, 0x5b, 0x67, 0x4b, 0xd5,
			       0x60, 0x01, 0xa6, 0x0f, 0x03, 0x9b, 0xfc, 0x01,
			       0x88, 0x6a, 0xeb, 0x52, 0xa3, 0x43, 0x0c, 0xdc,
			       0xbc, 0xbc, 0x01, 0xa4, 0x1e, 0x32, 0xce, 0x92,
			       0xb9, 0x4e, 0x50, 0x44, 0x2e, 0xb1, 0x5c, 0xd4,
			       0x24, 0xc1, 0x23, 0x7e, 0xc4, 0x2e, 0x44, 0x2c,
			       0x90, 0x95, 0x6a, 0x26, 0x46, 0x43, 0x93, 0x12,
			       0x26 };
	uint8_t userid[16] = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
			       0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38 };
	uint8_t E[32];
	if (testsize <= 0)
		return -1;

	get_rand(input, testsize);

	if (sm2_sign_digest(userid, 16, pubkey + 1, input, testsize, E)) {
		SCE_LOG_INF("sm2_sign_digest failed!\n");
		goto out;
	}

	if (pacc_ceu_sign_sm2(E, prikey, cmd)) {
		SCE_LOG_ERR("sign error\r\n");
		goto out;
	}

	/* encode it */
	sce_sm2signature_encode(cmd, output, &len);

	gettimeofday(&start, NULL);

	int tmp;
	for (tmp = 0; tmp < testcount; tmp++) {
		memcpy(output + len, E, 32);
		ret = sce_sm2_verify(E, 32, pubkey, sizeof(pubkey), output,
				     len + 32);
		if (ret) {
			SCE_LOG_ERR("verify error %d\r\n", ret);
			goto out;
		}
	}

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	SCE_LOG_INF(
		"SM2 verify success, testcount:%d, testsize:%d, time:%lu us, speed:%lu pps\n",
		testcount, testsize, time, testcount * 1000000ul / time);

	SCE_LOG_INF("sm2 verify (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

int sm2_test(int testsize, int testcount)
{
	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_IOVA,
			      .iomap = { .usr = NULL, .iova_map = NULL } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	if (kernel_sm2_enc_test(testsize, testcount)) {
		goto out;
	}

	if (kernel_sm2_dec_test(testsize, testcount)) {
		goto out;
	}

	if (kernel_sm2_sign_test(testsize, testcount)) {
		goto out;
	}

	if (kernel_sm2_verify_test(testsize, testcount)) {
		goto out;
	}

	sce_exit_lib();

	return 0;
out:
	sce_exit_lib();
	return -1;
}
