#!/bin/bash

function test_ske()
{
	ske_algs=(DES
			TDES
			AES_128
			AES_192
			AES_256
			SM4
			)
	lengths=(64 128 256 512 1024 2048 4096)
	ske_modes=(ECB CBC OFB CTR)
	local alg=
	local len=
	local mode=
	for alg in ${ske_algs[@]};do
		for mode in ${ske_modes[@]};do
			for len in ${lengths[@]}; do 
				./sce_test -s 4 -a $alg -m $mode -n 1000000 -z $len
			done
		done
	done
	
}

function test_xts()
{
	xts_algs=(AES_128
			AES_256
			SM4
			)
	lengths=(64 128 256 512 1024 2048 4096)
	local alg=
	local len=
	for alg in ${xts_algs[@]};do		
		for len in ${lengths[@]}; do 
			./sce_test -s 4 -a $alg -m XTS -n 1000000 -z $len
		done
	done
}

function test_trng()
{
	trng_modes=(TRNG_RNP
			TRNG_SM4)
	
	local mode=
	for mode in ${trng_modes[@]};do
		./sce_test -s 9 -m $mode -n 100000
	done
}


function test_hash_hmac()
{
	hash_algs=(SHA1
			SHA224
			SHA256
			SHA384
			SHA512
			MD5
			SM3
			)
	lengths=(64 128 256 512 1024 2048 4096)

	local alg=
	local len=
	for alg in ${hash_algs[@]};do
		for len in ${lengths[@]}; do 
			./sce_test -s 3 -a $alg -m NORMAL -n 1000000 -z $len
		done
		for len in ${lengths[@]}; do 
			./sce_test -s 3 -a $alg -m HMAC -n 1000000 -z $len
		done
	done
}

function test_cbcmac()
{
	local cbcmac_algs=( DES
			TDES
			AES_128
			AES_192
			AES_256
			SM4
			)
	
	lengths=(64 128 256 512 1024 2048 4096)
	local alg=
	local len=
	for alg in ${cbcmac_algs[@]};do
		for len in ${lengths[@]}; do 
			./sce_test -s 4 -a $alg -m CMAC -n 1000000 -z $len;
		done
		for len in ${lengths[@]}; do 
			./sce_test -s 4 -a $alg -m CBC_MAC -n 1000000 -z $len;
		done
	done
}


function test_aead()
{
	local aead_algs=( AES_128
					AES_192
					AES_256
					SM4
					)
	lengths=(64 128 256 512 1024 2048 4096)
	local alg=
	local len=
	for alg in ${aead_algs[@]};do
		for len in ${lengths[@]}; do 
			./sce_test -s 4 -a $alg -m GCM -n 1000000 -z $len
		done
		for len in ${lengths[@]}; do 
			./sce_test -s 4 -a $alg -m CCM -n 1000000 -z $len
		done
	done
}

function main()
{
	local start_time=`date +%s`
	local run_time=5
	local count=0

	if [ $# -gt 0 ]; then
		echo "get test duration $1 s"
		run_time=$1
	fi

	while true
	do
		now=`date +%s`
		if [ $[ $now - $start_time ] -gt $run_time ]; then
			echo "stop test..., total test count $count"
			break
		fi
		echo ""
		echo  "XXXXXXXXXXXXXXXXXXX " `date "+%D %T"`  "loop: $count"  "XXXXXXXXXXXXXXXXXXX "
		((count++))
		test_hash_hmac
#		test_cbcmac
#		test_aead
		test_ske
#		test_xts
		test_trng
		./sce_test -s 13 -n 10000 & ./sce_test -s 13 -n 10000 & ./sce_test -s 13 -n 10000 & ./sce_test -s 13 -n 10000
		echo ""
	done
}

main $@


