#define _GNU_SOURCE
#include <sched.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <signal.h>

#include <dsce/sce.h>
#include <dsce/sce_log.h>
#include <dsce/sce_crypto.h>

#include "test.h"
#include "ske.h"
#include "hash.h"
#include "trng.h"
#include "sm2_test.h"

char *mode_name[] = { "NORMAL", "HMAC",	    "ECB",     "CTR",  "CBC",
		      "OFB",	/*"XTS",	    "CBC_MAC", "CMAC", "GCM",
		      "CCM",*/	"TRNG_RNP", "TRNG_SM4" };
int mode_val[] = { SCE_NORMAL, SCE_HMAC, ECB, CTR, CBC,	     OFB,  /*   XTS,
		   CBC_MAC,    CMAC,	 GCM, CCM, */TRNG_RNP, TRNG_SM4 };
char *alg_name[] = { "SHA1", "SHA224", "SHA256",  "SHA384",  "SHA512",
		     "MD5",  "SM3",    "AES_128", "AES_192", "AES_256",
		     "DES",  "TDES",   "SM4" };
int alg_val[] = { SCE_SHA1, SCE_SHA224, SCE_SHA256,  SCE_SHA384,  SCE_SHA512,
		  SCE_MD5,  SCE_SM3,	SCE_AES_128, SCE_AES_192, SCE_AES_256,
		  SCE_DES,  SCE_TDES,	SCE_SM4 };

int running = 1;

void print_buf(void *name, uint8_t *buf, uint32_t len)
{
	uint32_t i;

	SCE_LOG_ALWAYS("%s[0x%x]:\n", (char *)name, len);
	for (i = 0; i < len; i += 16) {
		SCE_LOG_ALWAYS(
			"\"\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%"
			"02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\"\n",
			buf[i + 0], buf[i + 1], buf[i + 2], buf[i + 3],
			buf[i + 4], buf[i + 5], buf[i + 6], buf[i + 7],
			buf[i + 8], buf[i + 9], buf[i + 10], buf[i + 11],
			buf[i + 12], buf[i + 13], buf[i + 14], buf[i + 15]);
	}
}

/*
wreg se 0x32a33580 0x7f000070
wreg se 0x32a33584 0x7f0000b0

wreg se 0x32a33540 0x7f000070
wreg se 0x32a33544 0x7f0000b0
*/
static void signal_handler(int signal)
{
	running = 0;

	SCE_LOG_INF("signal:%d SIGINT:%d\n", signal, SIGINT);

	SCE_LOG_INF("%s:%d\n", __func__, __LINE__);
}

void cpu_bind(pthread_t tid, int cpu)
{
	cpu_set_t set;
	CPU_ZERO(&set);
	CPU_SET(cpu, &set);

	if (tid) {
		if (pthread_setaffinity_np(tid, sizeof(set), &set))
			SCE_LOG_INF("cpu_bind thread[%ld] failed\n", tid);
	} else {
		if (sched_setaffinity(getpid(), sizeof(set), &set))
			SCE_LOG_INF("cpu_bind process[%d] failed\n", getpid());
	}
}

void print_help(void)
{
	SCE_LOG_ALWAYS("-a  alg     test_alg:\n"
		       "\t\tSHA1\n"
		       "\t\tSHA224\n"
		       "\t\tSHA256\n"
		       "\t\tSHA384\n"
		       "\t\tSHA512\n"
		       "\t\tMD5\n"
		       "\t\tSM3\n"

		       "\t\tAES_128\n"
		       "\t\tAES_192\n"
		       "\t\tAES_256\n"
		       "\t\tDES\n"
		       "\t\tTDES\n"
		       "\t\tSM4\n");
	SCE_LOG_ALWAYS("-m  mode     test_mode:\n"
		       "\t\tNORMAL\n"
		       "\t\tHMAC\n"
		       "\t\tECB\n"
		       "\t\tCTR\n"
		       "\t\tCBC\n"
		       "\t\tOFB\n"
/*
		       "\t\tXTS\n"
		       "\t\tCMAC\n"
		       "\t\tCBC_MAC\n"
		       "\t\tGCM\n"
		       "\t\tCCM\n"
*/
		       "\t\tTRNG_RNP\n"
		       "\t\tTRNG_SM4\n");
	SCE_LOG_ALWAYS(
		"-s (1-17)  1:HASH 2:SCA 3:hash_speed 4:sca_speed 5:hash_delay_test "
		"6:sca_delay_test 7:hash_sca_speed 8:trng 9:trng_speed 13. sm2\n");
	SCE_LOG_ALWAYS("-n  num     test_num\n");
	SCE_LOG_ALWAYS("-q  queue_num     test_queue_num\n");
	SCE_LOG_ALWAYS("-z  size     speed test size\n");
	SCE_LOG_ALWAYS("-o  op_mode  speed test mode(ENC, DEC)\n");
	SCE_LOG_ALWAYS(
		"-l  dma-mode  dma-mode for link-list(0, 1, 2, 3 for sca, bit0 "
		"for dst, bit1 for src.),\n\t (0, 1 for hash 0:normal 1:src "
		"link-list), default 0.\n");
	SCE_LOG_ALWAYS("-k  link_num src link num, default 1.\n");
	SCE_LOG_ALWAYS("-K  link_num dst link num, default 1.\n");
}

int main(int argc, char **argv)
{
	long i, testcount = 1, flag = 1;
	int test_size = 64;
	int enc = SCE_ENC;
	uint32_t testmask = 0;
	int hash_alg = 0, sca_alg = 7, queue_num = 1;
	int hash_mode = 0, sca_mode = 2, trng_mode = 9;
	int dma_mode = 0;
	int src_link_num = 1;
	int dst_link_num = 1;
	int ch;

	sce_log_set_stderr(SCE_LOG_LL_INF);
	while ((ch = getopt(argc, argv, "k:K:z:m:q:a:s:n:o:w:hpl:")) != -1) {
		switch (ch) {
		case 'k':
			if (memcmp(optarg, "0x", 2) == 0)
				i = strtol(optarg, NULL, 16);
			else
				i = strtol(optarg, NULL, 10);
			src_link_num = i;
			SCE_LOG_INF("src link num: %x\n", src_link_num);
			break;
		case 'K':
			if (memcmp(optarg, "0x", 2) == 0)
				i = strtol(optarg, NULL, 16);
			else
				i = strtol(optarg, NULL, 10);
			dst_link_num = i;
			SCE_LOG_INF("dst link num: %x\n", dst_link_num);
			break;

		case 'l':
			if (memcmp(optarg, "0x", 2) == 0)
				dma_mode = strtol(optarg, NULL, 16);
			else
				dma_mode = strtol(optarg, NULL, 10);

			if (dma_mode > 3)
				dma_mode &= 0x3;

			SCE_LOG_INF("get dma_mode %x\r\n", dma_mode);
			break;
		case 's':
			i = atol(optarg);
			switch (i) {
			case 1 ... 13:
				testmask |= 1 << (i - 1);
				break;
			default:
				break;
			}
			flag = 0;
			break;
		case 'n':
			if (memcmp(optarg, "0x", 2) == 0)
				i = strtol(optarg, NULL, 16);
			else
				i = strtol(optarg, NULL, 10);
			testcount = i;
			break;
		case 'q':
			if (memcmp(optarg, "0x", 2) == 0)
				i = strtol(optarg, NULL, 16);
			else
				i = strtol(optarg, NULL, 10);
			queue_num = i % 4;
			if (queue_num == 0) {
				queue_num = 4;
			}
			SCE_LOG_INF("queue_num %d\r\n", queue_num);
			break;
		case 'a':
			for (i = 0; i < ARRAY_SIZE(alg_name); i++) {
				if (memcmp(optarg, alg_name[i],
					   strlen(alg_name[i])) == 0) {
					if (i <= 6)
						hash_alg = i;
					else
						sca_alg = i;
					SCE_LOG_INF("alg:%s\n", alg_name[i]);
					if (testmask == 8 && (i < 7)) {
						SCE_LOG_ERR("error sca_alg!\n");
						return -1;
					}
					if ((testmask == 4 || testmask == 16) &&
					    (i > 6)) {
						SCE_LOG_ERR(
							"error hash_alg!\n");
						return -1;
					}
					break;
				}
			}
			if (i == ARRAY_SIZE(alg_name)) {
				SCE_LOG_ERR("unsupport alg!\n");
				return -1;
			}
			break;
		case 'z':
			if (memcmp(optarg, "0x", 2) == 0)
				i = strtol(optarg, NULL, 16);
			else
				i = strtol(optarg, NULL, 10);
			test_size = i;
			break;
		case 'm':
			for (i = 0; i < ARRAY_SIZE(mode_name); i++) {
				if (strlen(optarg) == strlen(mode_name[i]) &&
				    memcmp(optarg, mode_name[i],
					   strlen(mode_name[i])) == 0) {
					if (i <= 1) {
						hash_mode = i;
						if (testmask != 4 &&
						    testmask != 16 && testmask != 64) {
							SCE_LOG_ERR(
								"error mode: %s!\n",
								optarg);
							return -1;
						}
					} else if (i > 1 && i <= 5) {
						sca_mode = i;
						if (testmask != 8 && testmask != 64) {
							SCE_LOG_ERR(
								"error mode: %s!\n",
								optarg);
							return -1;
						}
/*
						if ((sca_alg == 10 ||
						     sca_alg == 11) &&
						    (i == 6 || i == 9 ||
						     i == 10)) {
							SCE_LOG_ERR(
								"des/tdes unsupport sca_mode: %s!\n",
								optarg);
							return -1;
						}
						if (i == 6 && (sca_alg != 7 &&
							       sca_alg != 9 &&
							       sca_alg != 12)) {
							SCE_LOG_ERR(
								"alg unsupport xts!\n");
							return -1;
						}
*/
					} else {
						trng_mode = i;
						if (testmask != 256) {
							SCE_LOG_ERR(
								"error mode: %s!\n",
								optarg);
							return -1;
						}
					}
					SCE_LOG_ALWAYS("mode:%s\n",
						       mode_name[i]);
					break;
				}
			}
			if (i >= ARRAY_SIZE(mode_name)) {
				SCE_LOG_ERR("Invalid mode %s\n", optarg);
				return -1;
			}
			break;
		case 'o':
			if (memcmp(optarg, "ENC", strlen("ENC")) == 0) {
				enc = SCE_ENC;
				SCE_LOG_INF("op_mode:%s\n", optarg);
			}
			if (memcmp(optarg, "DEC", strlen("DEC")) == 0) {
				enc = SCE_DEC;
				SCE_LOG_INF("op_mode:%s\n", optarg);
			}
			break;
		default:
			print_help();
			return 0;
		}
	}

	signal(SIGINT, signal_handler);

	cpu_bind(0, 1);

	if (flag) {
		SCE_LOG_ERR("Missing -s, do nothing.");
	} else {
		if (testmask & (1 << 0)) {
			hash_test(dma_mode, src_link_num);
			hmac_test(dma_mode, src_link_num);
		}

		if (testmask & (1 << 1)) {
			sca_test(dma_mode, src_link_num, dst_link_num);
		}

		if (testmask & (1 << 2))
			hash_speed_test(queue_num, testcount, hash_alg,
					hash_mode, test_size, dma_mode,
					src_link_num);

		if (testmask & (1 << 3))
			sca_speed_test(queue_num, testcount, sca_alg, sca_mode,
				       enc, test_size, dma_mode, src_link_num,
				       dst_link_num);

		if (testmask & (1 << 4))
			hash_delay_test(hash_alg, hash_mode, test_size,
					dma_mode, src_link_num);

		if (testmask & (1 << 5))
			sca_delay_test(sca_alg, sca_mode, enc, test_size,
				       dma_mode, src_link_num, dst_link_num);

		if (testmask & (1 << 6))
			hash_sca_speed_test(queue_num, testcount, hash_alg,
					    hash_mode, sca_alg, sca_mode, enc,
					    test_size);

		if (testmask & (1 << 7)) {
			trng_test(test_size, trng_mode);
		}

		if (testmask & (1 << 8)) {
			trng_speed_test(queue_num, testcount, test_size,
					trng_mode);
		}

		if (testmask & (1 << 9)) {
//			ccm_test(dma_mode, src_link_num, dst_link_num);
		}
		if (testmask & (1 << 10)) {
//			gcm_test(dma_mode, src_link_num, dst_link_num);
		}
		if (testmask & (1 << 11)) {
//			sca_mac_test(dma_mode, src_link_num, dst_link_num);
		}

		if (testmask & (1 << 12)) {
			sm2_test(test_size, testcount);
		}
	}

	return 0;
}
