#define _GNU_SOURCE
#include <sched.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <signal.h>
#include <dsce/sce.h>
#include <dsce/sce_log.h>

#include "trng.h"
#include "test.h"

static void *trng_io_map(void *usr, void *va)
{
	(void)usr;
	return (void *)sce_virt_to_io(va);
}

int trng_test(int test_size, int mode)
{
	int i, offset, index, ret = 0;
	sce_rsp_t *out_rsp = NULL;
	uint8_t *result = NULL;
	sce_queue_handle sce_queue_desc = NULL;
	void **op_datas = NULL;
	sce_op_data_t *op_buf = NULL;
	void *origal_buf = NULL;
	int count = test_size / 64 + 1;

	int enqueued_count;
	int dequeued_count;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = trng_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	/*
            test sce_get_rand_bytes
    */
	{
		uint8_t data[256];
		if (test_size > 256) {
			test_size = 256;
		}

		if (sce_get_rand_bytes(data, test_size, mode_val[mode])) {
			goto out;
		}

		print_buf("trng test sync ", data, test_size);
	}

	if (sce_request_queue(SCE_TRNG, &sce_queue_desc)) {
		goto out;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &result);
	if (ret)
		goto out;

	/* test sce_get_rand_bytes_dma */
	{
		if (sce_get_rand_bytes_dma(result, test_size, mode_val[mode])) {
			goto out;
		}

		print_buf("trng test sync dma", result, test_size);
	}

	memset(result, 0, 0x100000);

	op_datas = (void **)malloc(sizeof(void *) * count);
	if (!op_datas) {
		SCE_LOG_ERR("alloc op_datas error\n");
		goto out;
	}

	origal_buf = (sce_op_data_t *)malloc(sizeof(sce_op_data_t) * count);
	if (!origal_buf) {
		SCE_LOG_ERR("alloc op_buf error\n");
		goto out;
	}
	op_buf = origal_buf;

	offset = 0;
	for (i = 0; i < count; i++) {
		op_datas[i] = op_buf;
		op_buf->op_type = SCE_TRNG;
		op_buf->trng.length = (i + 1) << 6;
		op_buf->trng.tag = i;
		op_buf->trng.buf = (uint64_t)result + offset;
		op_buf->trng.mode = mode_val[mode];

		op_buf->trng.tag <<= 48;
		op_buf->trng.tag |= (uint64_t)result + offset;
		op_buf->trng.tag |= 1ul << 63;
		op_buf++;

		offset += count + 80;
		offset += (64 - (offset & 0x3F)) & 0x3F;
	}

	enqueued_count = sce_enqueue(sce_queue_desc, op_datas, count);
	if (!enqueued_count)
		goto out;

	SCE_LOG_DBG("enqueued %d of count %d\n", enqueued_count, count);
	count = enqueued_count;
	out_rsp = malloc(sizeof(sce_rsp_t) * count);
	if (out_rsp == NULL) {
		goto out;
	}

	while (enqueued_count && running) {
		dequeued_count =
			sce_dequeue(sce_queue_desc, out_rsp, enqueued_count);
		if (0 == dequeued_count) {
			usleep(1000);
			continue;
		}

		enqueued_count -= dequeued_count;
		for (i = 0; i < dequeued_count; i++) {
			sce_rsp_t *rsp = &out_rsp[i];
			index = (rsp->tag >> 48) & 0x7FFF;
			if (rsp->state != CMD_SUCCESS) {
				SCE_LOG_ALWAYS("trng[%d] fail!, res[0x%x]\n",
					       index, rsp->state);
				continue;
			}

			result = (uint8_t *)(rsp->tag & 0xFFFFFFFFFFFF);
			print_buf("get random data", result, (i + 1) << 6);

			SCE_LOG_ALWAYS("test success[%d]\n", index);
		}
	}

out:
	if (op_datas)
		free(op_datas);

	if (origal_buf)
		free(origal_buf);

	if (out_rsp)
		free(out_rsp);

	if (sce_queue_desc) {
		sce_stop_queue(sce_queue_desc);
		sce_release_queue(sce_queue_desc);
	}

	sce_release_dmabuf(result);

	sce_exit_lib();

	return ret;
}

// static unsigned long lengths[6] = {16, 64, 256, 1024, 8192, 16384};

int trng_speed_test(int queue_num, long num, int size, int mode)
{
	struct timeval start, end;
	long cmd_total = 0, rsp_total = 0, i, ret = -1;
	sce_rsp_t rsp[8];
	unsigned long time = 0;
	sce_queue_handle sce_queue_desc[4] = { NULL };
	uint8_t *buf = NULL;
	uint64_t result = 0;
	sce_op_data_t op_data_tmp;
	void *op_data[1] = { &op_data_tmp };
	int enqueue_num = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = trng_io_map } };

	if (sce_init_lib(&cfg)) {
		SCE_LOG_ERR("Sce init failed\r\n");
		return -1;
	}

	for (i = 0; i < queue_num; i++) {
		if (sce_request_queue(SCE_TRNG, &sce_queue_desc[i])) {
			SCE_LOG_ERR("Request queue failed\r\n");
			goto out;
		}

		ret = sce_init_queue(sce_queue_desc[i]);
		if (ret) {
			SCE_LOG_ERR("Init queue failed\r\n");
			goto out;
		}
	}

	ret = sce_requset_dmabuf(16384, &buf);
	if (ret) {
		SCE_LOG_ERR("sce_requset_dmabuf failed\r\n");
		goto out;
	}

	result = (uint64_t)buf;

	cmd_total = num;
	rsp_total = 0;
	gettimeofday(&start, NULL);
	do {
		op_data_tmp.op_type = SCE_TRNG;
		op_data_tmp.trng.length = size;
		op_data_tmp.trng.tag = 0;
		op_data_tmp.trng.buf = result;
		op_data_tmp.trng.mode = mode_val[mode];

		for (i = 0; i < queue_num; i++) {
			if (cmd_total) {
				enqueue_num = sce_enqueue(sce_queue_desc[i],
							  op_data, 1);
				cmd_total -= enqueue_num;
			}
			rsp_total += sce_dequeue(sce_queue_desc[i], rsp, 8);
		}
		if (rsp_total >= num)
			break;
	} while (running);
	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS(
		"alg[%s] qnum[0x%x] testnum:%ld, testsize:%d, complete:%ld, "
		" speed:%ld Mbps-%ld pps\n",
		"trng-phytium", queue_num, num, size, rsp_total,
		(long)rsp_total * size * 8 / time,
		(long)rsp_total * 1000000ul / time);

	goto out; /* sce_get_rand_bytes 太慢了 */

	// test sce_get_rand_bytes
	cmd_total = num;
	rsp_total = 0;
	gettimeofday(&start, NULL);
	do {
		if (sce_get_rand_bytes(buf, size, mode_val[mode])) {
			SCE_LOG_ERR("get rand bytes falied.\r\n");
			goto out;
		}
		rsp_total++;
		if (rsp_total >= num)
			break;
	} while (running);
	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS("alg[%s] sce_get_rand_bytes testnum:%ld, testsize:%d, "
		       "complete:%ld,  speed:%ld Mbps-%ld pps\n",
		       "trng-phytium", num, size, rsp_total,
		       (long)rsp_total * size * 8 / time,
		       (long)rsp_total * 1000000ul / time);

	// test sce_get_rand_bytes
	cmd_total = num;
	rsp_total = 0;
	gettimeofday(&start, NULL);
	do {
		if (sce_get_rand_bytes_dma(buf, size, mode_val[mode])) {
			SCE_LOG_ERR("get rand bytes falied.\r\n");
			goto out;
		}
		rsp_total++;
		if (rsp_total >= num)
			break;
	} while (running);
	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS(
		"alg[%s] sce_get_rand_bytes_dma testnum:%ld, testsize:%d, "
		"complete:%ld,  speed:%ld Mbps-%ld pps\n",
		"trng-phytium", num, size, rsp_total,
		(long)rsp_total * size * 8 / time,
		(long)rsp_total * 1000000ul / time);

out:
	for (i = 0; i < queue_num; i++) {
		sce_stop_queue(sce_queue_desc[i]);
	}

	sce_release_dmabuf(buf);
	for (i = 0; i < queue_num; i++) {
		if (sce_queue_desc[i])
			sce_release_queue(sce_queue_desc[i]);
	}

	sce_exit_lib();
	return ret;
}
