

# 目的

SCTO内核态接口测试程序， 支持SM2(sign/verify/enc/dec), SM3(hash/hmac), SM4(ctr, cbc, ecb)。

## 使用

```shell
# 环境准备， 需要先安装驱动/编译用户态库

# 1. 编译驱动  make MODE=KERNEL
cd project_root/driver
make MODE=KERNEL
insmod scto.ko trng=1 # trng=1 注册内核trng算法，默认不注册

# 检查注册算法是否成功 cat /proc/crypto | grep phytium

root@keep-d2000:~# cat /proc/crypto | grep phytium
driver       : sm2-phytium
driver       : trng-phytium
driver       : sm4-ctr-phytium
driver       : sm4-cbc-phytium
driver       : sm4-ecb-phytium
driver       : sm3-hmac-phytium
driver       : sm3-phytium

```

# 2. 编译测试程序，并运行

```shell
cd project_root/test/driver
make clean
make 
```
# 3. 功能测试

```shell
## sm3 - hash 

root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# insmod mtest.ko  alg="sm3"
insmod: ERROR: could not insert module mtest.ko: Operation not permitted
root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver#
root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver#
root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# dmesg -c   
[ 1248.682362]
               testing of async sm3 (sm3-phytium)
[ 1248.682664]
               testing of async sm3 (sm3-phytium)
[ 1248.682809] Test sm3 success


## sm3 - hmac 

root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# insmod mtest.ko alg="hmac(sm3)"
insmod: ERROR: could not insert module mtest.ko: Operation not permitted
root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# dmesg -c  # 查看执行结果， 没有报错就是成功
[ 1265.700084]
               testing of async sm3 (sm3-hmac-phytium)
[ 1265.700399]
               testing of async sm3 (sm3-hmac-phytium)
[ 1265.700546] Test hmac(sm3) success

## sm4 - ecb 

root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# insmod mtest.ko alg="ecb(sm4)"
insmod: ERROR: could not insert module mtest.ko: Operation not permitted
root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# dmesg -c  # 查看执行结果， 没有报错就是成功
[ 1531.577425]
               testing of async ecb(sm4) (sm4-ecb-phytium) enc!
[ 1531.577731]
               testing of async ecb(sm4) (sm4-ecb-phytium) enc!
[ 1531.578006]
               testing of async ecb(sm4) (sm4-ecb-phytium) dec!
[ 1531.578280]
               testing of async ecb(sm4) (sm4-ecb-phytium) dec!
[ 1531.578420] Test ecb(sm4) success



## sm4 - cbc 
root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# insmod mtest.ko alg="cbc(sm4)"
insmod: ERROR: could not insert module mtest.ko: Operation not permitted
root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# dmesg -c  # 查看执行结果， 没有报错就是成功
[ 1547.162908]
               testing of async cbc(sm4) (sm4-cbc-phytium) enc!
[ 1547.163217]
               testing of async cbc(sm4) (sm4-cbc-phytium) enc!
[ 1547.163493]
               testing of async cbc(sm4) (sm4-cbc-phytium) dec!
[ 1547.163767]
               testing of async cbc(sm4) (sm4-cbc-phytium) dec!
[ 1547.163909] Test cbc(sm4) success


## sm4 - ctr 

root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# insmod mtest.ko alg="ctr(sm4)"
insmod: ERROR: could not insert module mtest.ko: Operation not permitted
root@keep-d2000:~/hmw/code/scto-driver/scto-driver/test/driver# dmesg -c  # 查看执行结果， 没有报错就是成功
[ 1562.067363]
               testing of async ctr(sm4) (sm4-ctr-phytium) enc!
[ 1562.067671]
               testing of async ctr(sm4) (sm4-ctr-phytium) enc!
[ 1562.067946]
               testing of async ctr(sm4) (sm4-ctr-phytium) dec!
[ 1562.068220]
               testing of async ctr(sm4) (sm4-ctr-phytium) dec!
[ 1562.068361] Test ctr(sm4) success


## sm2

insmod mtest.ko alg="sm2"
dmesg -c
[  968.837230] Test decrypt success[0]
[  968.838217] Test encrypt success[1]
[  968.838980] Test sign success[2]
[  968.839459] Test verify success[3]
[  968.839938] Test verify success[4]


## trng

insmod mtest.ko alg="trng"
dmesg -c
[ 1462.193369] get slen 256 elen 256
[ 1462.193481] random data[0x100]:
[ 1462.195861] 00000000: 03 03 f3 f4 e4 54 da 03 ec 8e 94 95 3c 4f 09 e1
[ 1462.195867] 00000010: 4b 81 ea 40 22 61 19 8a 56 b3 bf 73 9f f0 a2 b2
[ 1462.195872] 00000020: a9 2a 9c 97 20 ee ed 39 4b ab b3 5f 98 7b c4 df
[ 1462.195877] 00000030: 81 d0 05 cc a4 d2 d1 4f 32 0f 2c 8d dc 94 c7 11
[ 1462.195881] 00000040: 51 c1 c6 27 2c 1e 1d 52 50 d8 b1 56 fb 60 a5 40
[ 1462.195885] 00000050: 37 cf 15 b8 58 e5 9e 74 fa 78 c9 45 b1 5b 4b 6a
[ 1462.195889] 00000060: 49 ff d8 68 02 a0 34 f5 57 88 91 36 11 99 ea bc
[ 1462.195894] 00000070: 97 dc 6d c3 67 aa e0 72 01 5e 8f 54 1c 4b 32 77
[ 1462.195898] 00000080: 98 8e 3b d0 92 a2 91 8e 34 35 67 b5 4b c3 22 31
[ 1462.195902] 00000090: e8 cd b3 e7 7b d3 e4 bf fa ec 47 07 5a e9 5d 9e
[ 1462.195906] 000000a0: 97 ff 37 2e 68 59 91 30 0f 4d 20 9a 87 8d d6 a6
[ 1462.195911] 000000b0: a6 2e 60 d1 2d 99 7f 1a bd 40 88 1d 80 f0 43 a3
[ 1462.195915] 000000c0: 1c 8b e3 7d 44 37 38 a4 b5 b7 5c e3 8b 7c f9 d2
[ 1462.195919] 000000d0: 56 5e 85 35 b5 f6 b8 27 70 f4 6f c5 85 93 18 0d
[ 1462.195923] 000000e0: 88 f8 3c c5 11 55 cc 44 9b 0f e6 72 0a fc 1c 13
[ 1462.195928] 000000f0: 82 9c 8f bb 2d 44 b3 7d e4 15 46 f0 37 87 e8 7e # 查看结果，没有报错，并输出随机数


# 3. 性能测试

```shell
# speed=1  测试性能
# secs=3 每个长度测试3s时长，默认5s

## sm3 - hash 
dmesg -C

insmod mtest.ko alg="sm3" speed=1
dmesg -c

[242183.486644]
                testing speed of async sm3-phytium (sm3-phytium)
[242183.486657] test  0 (   64 byte blocks,   64 bytes per update,   1 updates):
[242188.479007] 536756 opers/sec,  34352396 bytes/sec 274 Mbps
[242188.479029] test  1 (  128 byte blocks,  128 bytes per update,   1 updates):
[242193.479048] 384347 opers/sec,  49196416 bytes/sec 393 Mbps
[242193.479061] test  2 (  256 byte blocks,  256 bytes per update,   1 updates):
[242198.479090] 377709 opers/sec,  96693555 bytes/sec 773 Mbps
[242198.479104] test  3 (  512 byte blocks,  512 bytes per update,   1 updates):
[242203.479130] 333052 opers/sec, 170522828 bytes/sec 1364 Mbps
[242203.479147] test  4 ( 1024 byte blocks, 1024 bytes per update,   1 updates):
[242208.479168] 285378 opers/sec, 292227891 bytes/sec 2337 Mbps
[242208.479189] test  5 ( 2048 byte blocks, 2048 bytes per update,   1 updates):
[242213.479211] 211208 opers/sec, 432553984 bytes/sec 3460 Mbps
[242213.479228] test  6 ( 4096 byte blocks, 4096 bytes per update,   1 updates):
[242218.479186] 138754 opers/sec, 568338022 bytes/sec 4546 Mbps
[242218.479203] test  7 ( 8192 byte blocks, 8192 bytes per update,   1 updates):
[242223.479019]  73985 opers/sec, 606088396 bytes/sec 4848 Mbps



## sm3 - hmac
insmod mtest.ko alg="hmac(sm3)" speed=1
dmesg -c
[242128.645683]
                testing speed of async sm3-hmac-phytium (sm3-hmac-phytium)
[242128.645697] test  0 (   64 byte blocks,   64 bytes per update,   1 updates):
[242133.638558] 215799 opers/sec,  13811161 bytes/sec 110 Mbps
[242133.638579] test  1 (  128 byte blocks,  128 bytes per update,   1 updates):
[242138.638600] 188581 opers/sec,  24138419 bytes/sec 193 Mbps
[242138.638618] test  2 (  256 byte blocks,  256 bytes per update,   1 updates):
[242143.638642] 188485 opers/sec,  48252364 bytes/sec 386 Mbps
[242143.638662] test  3 (  512 byte blocks,  512 bytes per update,   1 updates):
[242148.638684] 170845 opers/sec,  87472640 bytes/sec 699 Mbps
[242148.638700] test  4 ( 1024 byte blocks, 1024 bytes per update,   1 updates):
[242153.638725] 157887 opers/sec, 161676902 bytes/sec 1293 Mbps
[242153.638740] test  5 ( 2048 byte blocks, 2048 bytes per update,   1 updates):
[242158.638763] 128150 opers/sec, 262452019 bytes/sec 2099 Mbps
[242158.638776] test  6 ( 4096 byte blocks, 4096 bytes per update,   1 updates):
[242163.638808]  99299 opers/sec, 406731980 bytes/sec 3253 Mbps
[242163.638822] test  7 ( 8192 byte blocks, 8192 bytes per update,   1 updates):
[242168.638856]  61983 opers/sec, 507769651 bytes/sec 4062 Mbps



## sm4 - ecb
insmod mtest.ko alg="ecb(sm4)" speed=1
dmesg -c
[242010.693601]
                testing speed of sync sm4-ecb-phytium (sm4-ecb-phytium) encryption
[242010.693612] test 0 (128 bit key, 16 byte blocks): 8106258 operations in 5 seconds (129700128 bytes) 207 Mbps
[242015.687570] test 1 (128 bit key, 64 byte blocks): 2545610 operations in 5 seconds (162919040 bytes) 260 Mbps
[242020.687612] test 2 (128 bit key, 128 byte blocks): 2496482 operations in 5 seconds (319549696 bytes) 511 Mbps
[242025.687654] test 3 (128 bit key, 256 byte blocks): 1851165 operations in 5 seconds (473898240 bytes) 758 Mbps
[242030.687699] test 4 (128 bit key, 512 byte blocks): 1813444 operations in 5 seconds (928483328 bytes) 1485 Mbps
[242035.687740] test 5 (128 bit key, 1024 byte blocks): 1446988 operations in 5 seconds (1481715712 bytes) 2370 Mbps
[242040.687786] test 6 (128 bit key, 1424 byte blocks): 1281393 operations in 5 seconds (1824703632 bytes) 2919 Mbps
[242045.687834] test 7 (128 bit key, 2048 byte blocks): 1147910 operations in 5 seconds (2350919680 bytes) 3761 Mbps
[242050.687874] test 8 (128 bit key, 4096 byte blocks): 640757 operations in 5 seconds (2624540672 bytes) 4199 Mbps
[242055.687917] test 9 (128 bit key, 8192 byte blocks): 401335 operations in 5 seconds (3287736320 bytes) 5260 Mbps
[242060.688318]
                testing speed of sync sm4-ecb-phytium (sm4-ecb-phytium) decryption
[242060.688327] test 0 (128 bit key, 16 byte blocks): 8072835 operations in 5 seconds (129165360 bytes) 206 Mbps
[242065.688004] test 1 (128 bit key, 64 byte blocks): 2538167 operations in 5 seconds (162442688 bytes) 259 Mbps
[242070.688044] test 2 (128 bit key, 128 byte blocks): 2562338 operations in 5 seconds (327979264 bytes) 524 Mbps
[242075.688084] test 3 (128 bit key, 256 byte blocks): 1847990 operations in 5 seconds (473085440 bytes) 756 Mbps
[242080.688129] test 4 (128 bit key, 512 byte blocks): 1880015 operations in 5 seconds (962567680 bytes) 1540 Mbps
[242085.688175] test 5 (128 bit key, 1024 byte blocks): 1453289 operations in 5 seconds (1488167936 bytes) 2381 Mbps
[242090.688215] test 6 (128 bit key, 1424 byte blocks): 1281436 operations in 5 seconds (1824764864 bytes) 2919 Mbps
[242095.688257] test 7 (128 bit key, 2048 byte blocks): 1122600 operations in 5 seconds (2299084800 bytes) 3678 Mbps
[242100.688303] test 8 (128 bit key, 4096 byte blocks): 650843 operations in 5 seconds (2665852928 bytes) 4265 Mbps
[242105.688351] test 9 (128 bit key, 8192 byte blocks): 395288 operations in 5 seconds (3238199296 bytes) 5181 Mbps




## sm4 - cbc
insmod mtest.ko alg="cbc(sm4)" speed=1
dmesg -c
[241865.492775]
                testing speed of sync sm4-cbc-phytium (sm4-cbc-phytium) encryption
[241865.492786] test 0 (128 bit key, 16 byte blocks): 7889450 operations in 5 seconds (126231200 bytes) 201 Mbps
[241870.486243] test 1 (128 bit key, 64 byte blocks): 2484164 operations in 5 seconds (158986496 bytes) 254 Mbps
[241875.486287] test 2 (128 bit key, 128 byte blocks): 2432247 operations in 5 seconds (311327616 bytes) 498 Mbps
[241880.486334] test 3 (128 bit key, 256 byte blocks): 1921613 operations in 5 seconds (491932928 bytes) 787 Mbps
[241885.486382] test 4 (128 bit key, 512 byte blocks): 1159755 operations in 5 seconds (593794560 bytes) 950 Mbps
[241890.486432] test 5 (128 bit key, 1024 byte blocks): 929751 operations in 5 seconds (952065024 bytes) 1523 Mbps
[241895.486478] test 6 (128 bit key, 1424 byte blocks): 756617 operations in 5 seconds (1077422608 bytes) 1723 Mbps
[241900.486523] test 7 (128 bit key, 2048 byte blocks): 616636 operations in 5 seconds (1262870528 bytes) 2020 Mbps
[241905.486572] test 8 (128 bit key, 4096 byte blocks): 375174 operations in 5 seconds (1536712704 bytes) 2458 Mbps
[241910.486622] test 9 (128 bit key, 8192 byte blocks): 213576 operations in 5 seconds (1749614592 bytes) 2799 Mbps
[241915.487014]
                testing speed of sync sm4-cbc-phytium (sm4-cbc-phytium) decryption
[241915.487023] test 0 (128 bit key, 16 byte blocks): 7948661 operations in 5 seconds (127178576 bytes) 203 Mbps
[241920.486711] test 1 (128 bit key, 64 byte blocks): 2469039 operations in 5 seconds (158018496 bytes) 252 Mbps
[241925.486760] test 2 (128 bit key, 128 byte blocks): 2277488 operations in 5 seconds (291518464 bytes) 466 Mbps
[241930.486803] test 3 (128 bit key, 256 byte blocks): 1814003 operations in 5 seconds (464384768 bytes) 743 Mbps
[241935.486852] test 4 (128 bit key, 512 byte blocks): 1381498 operations in 5 seconds (707326976 bytes) 1131 Mbps
[241940.486891] test 5 (128 bit key, 1024 byte blocks): 915290 operations in 5 seconds (937256960 bytes) 1499 Mbps
[241945.486938] test 6 (128 bit key, 1424 byte blocks): 757404 operations in 5 seconds (1078543296 bytes) 1725 Mbps
[241950.486985] test 7 (128 bit key, 2048 byte blocks): 611225 operations in 5 seconds (1251788800 bytes) 2002 Mbps
[241955.487033] test 8 (128 bit key, 4096 byte blocks): 381001 operations in 5 seconds (1560580096 bytes) 2496 Mbps
[241960.487083] test 9 (128 bit key, 8192 byte blocks): 213526 operations in 5 seconds (1749204992 bytes) 2798 Mbps

## sm4 - ctr 
insmod mtest.ko alg="ctr(sm4)" speed=1
dmesg -c
[241576.431300]
                testing speed of sync sm4-ctr-phytium (sm4-ctr-phytium) encryption
[241576.431312] test 0 (128 bit key, 16 byte blocks): 7918411 operations in 5 seconds (126694576 bytes) 202 Mbps
[241581.423150] test 1 (128 bit key, 64 byte blocks): 2478596 operations in 5 seconds (158630144 bytes) 253 Mbps
[241586.423207] test 2 (128 bit key, 128 byte blocks): 2516452 operations in 5 seconds (322105856 bytes) 515 Mbps
[241591.423267] test 3 (128 bit key, 256 byte blocks): 1865107 operations in 5 seconds (477467392 bytes) 763 Mbps
[241596.423329] test 4 (128 bit key, 512 byte blocks): 1745804 operations in 5 seconds (893851648 bytes) 1430 Mbps
[241601.423384] test 5 (128 bit key, 1024 byte blocks): 1383541 operations in 5 seconds (1416745984 bytes) 2266 Mbps
[241606.423449] test 6 (128 bit key, 1420 byte blocks): 960748 operations in 5 seconds (1364262160 bytes) 2182 Mbps
[241611.423503] test 7 (128 bit key, 2048 byte blocks): 1047018 operations in 5 seconds (2144292864 bytes) 3430 Mbps
[241616.423562] test 8 (128 bit key, 4096 byte blocks): 640402 operations in 5 seconds (2623086592 bytes) 4196 Mbps
[241621.423624] test 9 (128 bit key, 8192 byte blocks): 388928 operations in 5 seconds (3186098176 bytes) 5097 Mbps
[241626.424030]
                testing speed of sync sm4-ctr-phytium (sm4-ctr-phytium) decryption
[241626.424038] test 0 (128 bit key, 16 byte blocks): 7953255 operations in 5 seconds (127252080 bytes) 203 Mbps
[241631.423737] test 1 (128 bit key, 64 byte blocks): 2473848 operations in 5 seconds (158326272 bytes) 253 Mbps
[241636.423794] test 2 (128 bit key, 128 byte blocks): 2438617 operations in 5 seconds (312142976 bytes) 499 Mbps
[241641.423849] test 3 (128 bit key, 256 byte blocks): 1721171 operations in 5 seconds (440619776 bytes) 704 Mbps
[241646.423904] test 4 (128 bit key, 512 byte blocks): 1648489 operations in 5 seconds (844026368 bytes) 1350 Mbps
[241651.423967] test 5 (128 bit key, 1024 byte blocks): 1380197 operations in 5 seconds (1413321728 bytes) 2261 Mbps
[241656.424023] test 6 (128 bit key, 1420 byte blocks): 991780 operations in 5 seconds (1408327600 bytes) 2253 Mbps
[241661.424079] test 7 (128 bit key, 2048 byte blocks): 1054480 operations in 5 seconds (2159575040 bytes) 3455 Mbps
[241666.424132] test 8 (128 bit key, 4096 byte blocks): 640724 operations in 5 seconds (2624405504 bytes) 4199 Mbps
[241671.424192] test 9 (128 bit key, 8192 byte blocks): 388602 operations in 5 seconds (3183427584 bytes) 5093 Mbps

## sm2
insmod mtest.ko alg="sm2" speed=1
dmesg -c
[241536.002919] Doing SM2 encrypt for 5s :   8121 opers in 5 s   1624 opers/sec
[241541.002922] Doing SM2 decrypt for 5s:  13613 opers in 5 s   2722 opers/sec
[241546.002710] Doing SM2 sign for 5s :  17121 opers in 5 s   3424 opers/sec
[241551.002890] Doing SM2 verify for 5s:  10497 opers in 5 s   2099 opers/sec




## trng

insmod mtest.ko alg="trng" speed=1
dmesg -c
[  758.451811] Doing trng-phytium for 5s on 16 size: 733792 opers in 5 s 146758 opers/sec,        18 Mbps
[  763.451030] Doing trng-phytium for 5s on 64 size: 183611 opers in 5 s  36722 opers/sec,        18 Mbps
[  768.450270] Doing trng-phytium for 5s on 128 size:  91800 opers in 5 s  18360 opers/sec,        18 Mbps
[  773.449540] Doing trng-phytium for 5s on 256 size:  45901 opers in 5 s   9180 opers/sec,        18 Mbps
[  778.448735] Doing trng-phytium for 5s on 512 size:  22951 opers in 5 s   4590 opers/sec,        18 Mbps
[  783.448024] Doing trng-phytium for 5s on 1024 size:  11475 opers in 5 s   2295 opers/sec,        18 Mbps
[  788.447750] Doing trng-phytium for 5s on 2048 size:   5738 opers in 5 s   1147 opers/sec,        18 Mbps
[  793.447400] Doing trng-phytium for 5s on 4096 size:   2869 opers in 5 s    573 opers/sec,        18 Mbps
[  798.448822] Doing trng-phytium for 5s on 8192 size:   1435 opers in 5 s    287 opers/sec,        18 Mbps




```
 