/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __SM3_SOFTWARE_H__
#define __SM3_SOFTWARE_H__

struct sm3_context {
	u32 total[2];   /* number of bytes processed */
	u32 state[8];   /* intermediate digest state */
	u8 buffer[64];  /* data block being processed */
	u8 ipad[64];
	u8 opad[64];
};

struct sm3_func {
	void (*sm3_init)(struct sm3_context *ctx);
	void (*sm3_update)(struct sm3_context *ctx, const u8 *input, size_t ilen);
	void (*sm3_final)(struct sm3_context *ctx, u8 output[32]);
	void (*sm3_finup)(struct sm3_context *ctx, const u8 *data, unsigned int len, u8 *out);
	void (*sm3_digest)(struct sm3_context *ctx, const u8 *data, unsigned int len, u8 *out);
	int (*hmac_sm3_setkey)(struct sm3_context *ctx, const u8 *key, unsigned int keylen);
	int (*hmac_sm3_init)(struct sm3_context *ctx);
	int (*hmac_sm3_final)(struct sm3_context *ctx, u8 *out);
	int (*hmac_sm3_finup)(struct sm3_context *ctx, const u8 *data, unsigned int len, u8 *out);
};

void sm3_func_alloc(struct sm3_func *func);
#endif
