#define _GNU_SOURCE
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <signal.h>
#include <sys/socket.h>
#include <linux/if_alg.h>

#include "lib_smx_common.h"
#include "lib_phytium_scto.h"
#include "lib_sm3_phytium.h"
#include "lib_sm4_phytium.h"
#include "lib_sm2_phytium.h"
#include "lib_phytium_log.h"
#include "test.h"
#include "sm2_test.h"

extern int pacc_ceu_genkey_sm2(uint8_t *priv_key_addr, uint8_t *pub_key_addr);

extern int pacc_ceu_sign_sm2(uint8_t *e_addr, uint8_t *priv_key_addr,
			     uint8_t *result_addr);
extern int pacc_ceu_verify_sm2(uint8_t *e_signature_addr,
			       uint8_t *pub_key_addr);

extern int pacc_ceu_enc_sm2(uint32_t msg_lengths, uint8_t *msg_addr,
			    uint8_t *pub_key_addr, uint8_t *result_addr);

extern int pacc_ceu_dec_sm2(uint32_t msg_lengths, uint8_t *msg_addr,
			    uint8_t *priv_key_addr, uint8_t *result_addr);

extern int pacc_ceu_exchange_sm2(uint16_t role, uint16_t keysize,
				 uint8_t *key_z_addr, uint8_t *result_addr);

static void test_print_buf(uint8_t *name, uint8_t *buf, uint32_t len)
{
	uint32_t i;

	SCTO_LOG_INF("%s[0x%x]:\n", name, len);
	for (i = 0; i < len; i += 16) {
		SCTO_LOG_INF(
			"\"\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%"
			"02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\\x%02x\"\n",
			buf[i + 0], buf[i + 1], buf[i + 2], buf[i + 3],
			buf[i + 4], buf[i + 5], buf[i + 6], buf[i + 7],
			buf[i + 8], buf[i + 9], buf[i + 10], buf[i + 11],
			buf[i + 12], buf[i + 13], buf[i + 14], buf[i + 15]);
	}
}

void cpu_bind(pthread_t tid, int cpu)
{
	if (tid == 0 || cpu < 0)
		return;
	cpu_set_t set;
	CPU_ZERO(&set);
	CPU_SET(cpu, &set);

	if (pthread_setaffinity_np(tid, sizeof(set), &set))
		SCTO_LOG_INF("cpu_bind failed\n");
}

static int get_random_data(void *buf, uint32_t size)
{
	if (buf == NULL || size == 0)
		return -1;
	uint32_t offset;
	int len;
restart:
	offset = 0;
	do {
		len = phytium_scto_get_rand_data((uint8_t *)buf + offset,
						 size - offset);
		offset += len;
	} while ((offset < size) && (len >= 0));

	if (offset != (long)size) {
		SCTO_LOG_ERR("read random dev fail!offset:%u size;%u\n", offset,
			     size);
		return -1;
	}

	for (offset = 0; offset < size; offset++) {
		if (*((uint8_t *)buf + offset))
			break;
	}

	if (offset == size)
		goto restart;

	if (printf_data) {
		SCTO_LOG_INF("%s\n", __func__);
		for (offset = 0; offset < size; offset++) {
			SCTO_LOG_INF("0x%02x ", *((uint8_t *)buf + offset));
			if ((offset & 0xf) == 0xf)
				SCTO_LOG_INF("\n");
		}
		SCTO_LOG_INF("\n");
	}

	return 0;
}

static int sm2_sign_digest(uint8_t *user_id, uint32_t id_len, uint8_t *pubkey,
			   uint8_t *msg, uint32_t msg_len, uint8_t *E)
{
	uint8_t id_bit_len[2];
	uint8_t Z[32];
	uint32_t sm2p256v1_a[8] = { 0xFFFFFFFC, 0xFFFFFFFF, 0x00000000,
				    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
				    0xFFFFFFFF, 0xFFFFFFFE };
	uint32_t sm2p256v1_b[8] = { 0x4D940E93, 0xDDBCBD41, 0x15AB8F92,
				    0xF39789F5, 0xCF6509A7, 0x4D5A9E4B,
				    0x9D9F5E34, 0x28E9FA9E };
	uint32_t sm2p256v1_Gx[8] = { 0x334C74C7, 0x715A4589, 0xF2660BE1,
				     0x8FE30BBF, 0x6A39C994, 0x5F990446,
				     0x1F198119, 0x32C4AE2C };
	uint32_t sm2p256v1_Gy[8] = { 0x2139F0A0, 0x02DF32E5, 0xC62A4740,
				     0xD0A9877C, 0x6B692153, 0x59BDCEE3,
				     0xF4F6779C, 0xBC3736A2 };
	uint8_t id[16] = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
			   0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38 };
	int i;
	uint8_t tmp, *a, *b, *x, *y;
	EVP_MD_CTX *ctx = NULL;

	if (user_id == NULL || pubkey == NULL || msg == NULL || E == NULL)
		return -1;
	if (id_len == 0 || msg_len == 0)
		return -1;

	a = (uint8_t *)sm2p256v1_a;
	b = (uint8_t *)sm2p256v1_b;
	x = (uint8_t *)sm2p256v1_Gx;
	y = (uint8_t *)sm2p256v1_Gy;
	for (i = 0; i < 16; i++) {
		tmp = a[i];
		a[i] = a[31 - i];
		a[31 - i] = tmp;
		tmp = b[i];
		b[i] = b[31 - i];
		b[31 - i] = tmp;
		tmp = x[i];
		x[i] = x[31 - i];
		x[31 - i] = tmp;
		tmp = y[i];
		y[i] = y[31 - i];
		y[31 - i] = tmp;
	}

	if ((user_id == NULL) || (id_len == 0)) {
		user_id = id;
		id_len = 16;
	}

	if (id_len >= (1 << 13))
		return -1;

	id_bit_len[0] = ((id_len << 3) >> 8) & 0xFF;
	id_bit_len[1] = (id_len << 3) & 0xFF;

	ctx = EVP_MD_CTX_new();
	if (ctx == NULL) {
		SCTO_LOG_INF("EVP_MD_CTX_new fail!\n");
		return -1;
	}

	EVP_DigestInit_ex(ctx, EVP_sm3(), NULL);
	EVP_DigestUpdate(ctx, id_bit_len, 2);
	EVP_DigestUpdate(ctx, user_id, id_len);
	EVP_DigestUpdate(ctx, sm2p256v1_a, 32);
	EVP_DigestUpdate(ctx, sm2p256v1_b, 32);
	EVP_DigestUpdate(ctx, sm2p256v1_Gx, 32);
	EVP_DigestUpdate(ctx, sm2p256v1_Gy, 32);
	EVP_DigestUpdate(ctx, pubkey, 64);
	EVP_DigestFinal_ex(ctx, Z, NULL);

	EVP_DigestInit_ex(ctx, EVP_sm3(), NULL);
	EVP_DigestUpdate(ctx, Z, 32);
	EVP_DigestUpdate(ctx, msg, msg_len);
	EVP_DigestFinal_ex(ctx, E, NULL);

	EVP_MD_CTX_free(ctx);

	return 0;
}

int sm3_dgst_test(int testsize, int cpuid)
{
	int sm3_desc_id = -1;
	struct timeval start, end;
	unsigned long time;
	int i;
	char input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
	     cmpbuf[BUF_SIZE] = { 0 };
	EVP_MD_CTX *ctx = NULL;
	int size, offset;

	if (testsize <= 0 || cpuid < 0)
		return -1;

	memcpy(input, randomdata[cpuid], testsize);
	if (multestsize) {
		gettimeofday(&start, NULL);
		for (i = 0; i < test_num; i++) {
			sm3_desc_id = -1;
			while (phytium_scto_sm3_init(&sm3_desc_id)) {
			}
			phytium_scto_sm3_update(sm3_desc_id, input, testsize);
			phytium_scto_sm3_final(sm3_desc_id, output);
			if (sm3_desc_id > 0)
				phytium_scto_mem_free(sm3_desc_id);
			multestnum[cpuid][0]++;
		}
		gettimeofday(&end, NULL);

		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);

		if (test_time)
			SCTO_LOG_INF(
				"scto dgst success, testnum:%d, testsize:%d, speed:%lu MBps\n",
				test_num, testsize, test_num * testsize / time);
	} else {
		sm3_desc_id = -1;
		while (phytium_scto_sm3_init(&sm3_desc_id)) {
		}
		size = testsize;
		offset = 0;
		i = 0;
		while (size) {
			get_random_data(&i, 2);
			if (i > size)
				i = size;
			phytium_scto_sm3_update(sm3_desc_id, input + offset, i);
			size -= i;
			offset += i;
		}
		phytium_scto_sm3_final(sm3_desc_id, output);
		if (sm3_desc_id > 0)
			phytium_scto_mem_free(sm3_desc_id);
	}

	ctx = EVP_MD_CTX_new();
	if (ctx == NULL) {
		SCTO_LOG_INF("EVP_MD_CTX_new fail!\n");
		return -1;
	}
	EVP_DigestInit_ex(ctx, EVP_sm3(), NULL);
	EVP_DigestUpdate(ctx, input, testsize);
	EVP_DigestFinal_ex(ctx, cmpbuf, NULL);
	EVP_MD_CTX_free(ctx);

	if (memcmp(output, cmpbuf, 32)) {
		SCTO_LOG_INF("sm3 dgst error!\n");
		goto out;
	} else
		SCTO_LOG_INF("sm3 dgst (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

int sm4_cbc_test(int testsize, int cpuid)
{
	int sm4_desc_id;
	struct timeval start, end;
	unsigned long time;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 };
	uint8_t std_key[16] = { 0xE0, 0x70, 0x99, 0xF1, 0xBF, 0xAF, 0xFD, 0x7F,
				0x24, 0x0C, 0xD7, 0x90, 0xCA, 0x4F, 0xE1, 0x34 };
	uint8_t std_iv[16] = { 0xC7, 0x2B, 0x65, 0x91, 0xA0, 0xD7, 0xDE, 0x8F,
			       0x6B, 0x40, 0x72, 0x33, 0xAD, 0x35, 0x81, 0xD6 };
	int i;
	EVP_CIPHER_CTX *ectx;
	int size, offset;

	if (testsize <= 0 || cpuid < 0)
		return -1;

	memcpy(input, randomdata[cpuid], testsize);
	if (multestsize) {
		gettimeofday(&start, NULL);
		for (i = 0; i < test_num; i++) {
			sm4_desc_id = -1;
			while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_CBC,
						     SM4_CRYPTO_ENCRYPT,
						     std_key, std_iv)) {
			}
			phytium_scto_sm4_update(sm4_desc_id, input, testsize,
						output);
			if (sm4_desc_id > 0)
				phytium_scto_mem_free(sm4_desc_id);
			multestnum[cpuid][0]++;
		}
		gettimeofday(&end, NULL);

		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);

		if (test_time)
			SCTO_LOG_INF(
				"scto cbc enc success, testnum:%d, testsize:%d, speed:%.8lu MBps\n",
				test_num, testsize, test_num * testsize / time);
	} else {
		sm4_desc_id = -1;
		while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_CBC,
					     SM4_CRYPTO_ENCRYPT, std_key,
					     std_iv)) {
		}
		size = testsize;
		offset = 0;
		i = 0;
		while (size) {
			get_random_data(&i, 2);
			i &= 0xFFF0;
			if (i > size)
				i = size;
			phytium_scto_sm4_update(sm4_desc_id, input + offset, i,
						output + offset);
			size -= i;
			offset += i;
		}
		if (sm4_desc_id > 0)
			phytium_scto_mem_free(sm4_desc_id);
	}
	ectx = EVP_CIPHER_CTX_new();
	if (ectx == NULL) {
		SCTO_LOG_INF("EVP_CIPHER_CTX_new fail!\n");
		return -1;
	}
	EVP_CipherInit_ex(ectx, EVP_sm4_cbc(), NULL, std_key, std_iv, 1);
	EVP_CipherUpdate(ectx, cmpbuf, &i, input, testsize);
	EVP_CipherFinal(ectx, cmpbuf + testsize, &i);
	EVP_CIPHER_CTX_free(ectx);

	if (memcmp(output, cmpbuf, testsize)) {
		SCTO_LOG_INF("sm4 cbc encrypt error!\n");
		return -1;
	} else
		SCTO_LOG_INF("sm4-cbc enc (testsize %d) test passed!\n",
			     testsize);

	if (multestsize) {
		gettimeofday(&start, NULL);
		for (i = 0; i < test_num; i++) {
			sm4_desc_id = -1;
			while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_CBC,
						     SM4_CRYPTO_DECRYPT,
						     std_key, std_iv)) {
			}
			phytium_scto_sm4_update(sm4_desc_id, output, testsize,
						cmpbuf);
			if (sm4_desc_id > 0)
				phytium_scto_mem_free(sm4_desc_id);
			multestnum[cpuid][0]++;
		}
		gettimeofday(&end, NULL);

		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);

		if (test_time)
			SCTO_LOG_INF(
				"scto cbc dec success, testnum:%d, testsize:%d, speed:%lu MBps\n",
				test_num, testsize, test_num * testsize / time);
	} else {
		sm4_desc_id = -1;
		while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_CBC,
					     SM4_CRYPTO_DECRYPT, std_key,
					     std_iv)) {
		}
		size = testsize;
		offset = 0;
		i = 0;
		while (size) {
			get_random_data(&i, 2);
			i &= 0xFFF0;
			if (i > size)
				i = size;
			phytium_scto_sm4_update(sm4_desc_id, output + offset, i,
						cmpbuf + offset);
			size -= i;
			offset += i;
		}
		if (sm4_desc_id > 0)
			phytium_scto_mem_free(sm4_desc_id);
	}

	if (memcmp(input, cmpbuf, testsize)) {
		SCTO_LOG_INF("sm4 cbc decrypt error!\n");
		return -1;
	} else
		SCTO_LOG_INF("sm4-cbc dec (testsize %d) test passed!\n",
			     testsize);

	return 0;
}

int sm4_ecb_test(int testsize, int cpuid)
{
	struct timeval start, end;
	unsigned long time;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 };
	uint8_t std_key[16] = { 0xE0, 0x70, 0x99, 0xF1, 0xBF, 0xAF, 0xFD, 0x7F,
				0x24, 0x0C, 0xD7, 0x90, 0xCA, 0x4F, 0xE1, 0x34 };
	int i;
	int sm4_desc_id;
	EVP_CIPHER_CTX *ectx;
	int size, offset;

	if (testsize <= 0 || cpuid < 0)
		return -1;

	memcpy(input, randomdata[cpuid], testsize);
	if (multestsize) {
		gettimeofday(&start, NULL);
		for (i = 0; i < test_num; i++) {
			sm4_desc_id = -1;
			while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_ECB,
						     SM4_CRYPTO_ENCRYPT,
						     std_key, NULL)) {
			}
			phytium_scto_sm4_update(sm4_desc_id, input, testsize,
						output);
			if (sm4_desc_id > 0)
				phytium_scto_mem_free(sm4_desc_id);
			multestnum[cpuid][0]++;
		}
		gettimeofday(&end, NULL);

		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);

		if (test_time)
			SCTO_LOG_INF(
				"TA ecb enc success, testnum:%d, testsize:%d, speed:%lu MBps\n",
				test_num, testsize, test_num * testsize / time);
	} else {
		sm4_desc_id = -1;
		while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_ECB,
					     SM4_CRYPTO_ENCRYPT, std_key,
					     NULL)) {
		}
		size = testsize;
		offset = 0;
		i = 0;
		while (size) {
			get_random_data(&i, 2);
			i &= 0xFFF0;
			if (i > size)
				i = size;
			phytium_scto_sm4_update(sm4_desc_id, input + offset, i,
						output + offset);
			size -= i;
			offset += i;
		}
		if (sm4_desc_id > 0)
			phytium_scto_mem_free(sm4_desc_id);
	}

	ectx = EVP_CIPHER_CTX_new();
	if (ectx == NULL) {
		SCTO_LOG_INF("EVP_CIPHER_CTX_new fail!\n");
		return -1;
	}
	EVP_CipherInit_ex(ectx, EVP_sm4_ecb(), NULL, std_key, NULL, 1);
	EVP_CipherUpdate(ectx, cmpbuf, &i, input, testsize);
	EVP_CipherFinal(ectx, cmpbuf + testsize, &i);
	EVP_CIPHER_CTX_free(ectx);
	if (memcmp(output, cmpbuf, testsize)) {
		SCTO_LOG_INF("sm4 ecb encrypt error!\n");
		return -1;
	} else
		SCTO_LOG_INF("sm4-ecb enc (testsize %d) test passed!\n",
			     testsize);

	if (multestsize) {
		gettimeofday(&start, NULL);
		for (i = 0; i < test_num; i++) {
			sm4_desc_id = -1;
			while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_ECB,
						     SM4_CRYPTO_DECRYPT,
						     std_key, NULL)) {
			}
			phytium_scto_sm4_update(sm4_desc_id, output, testsize,
						cmpbuf);
			if (sm4_desc_id > 0)
				phytium_scto_mem_free(sm4_desc_id);
			multestnum[cpuid][0]++;
		}
		gettimeofday(&end, NULL);

		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);

		if (test_time)
			SCTO_LOG_INF(
				"scto ecb dec success, testnum:%d, testsize:%d, speed:%lu MBps\n",
				test_num, testsize, test_num * testsize / time);
	} else {
		sm4_desc_id = -1;
		while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_ECB,
					     SM4_CRYPTO_DECRYPT, std_key,
					     NULL)) {
		}
		size = testsize;
		offset = 0;
		i = 0;
		while (size) {
			get_random_data(&i, 2);
			i &= 0xFFF0;
			if (i > size)
				i = size;
			phytium_scto_sm4_update(sm4_desc_id, output + offset, i,
						cmpbuf + offset);
			size -= i;
			offset += i;
		}
		if (sm4_desc_id > 0)
			phytium_scto_mem_free(sm4_desc_id);
	}

	if (memcmp(input, cmpbuf, testsize)) {
		SCTO_LOG_INF("sm4 ecb decrypt error!\n");
		return -1;
	} else
		SCTO_LOG_INF("sm4-ecb dec (testsize %d) test passed!\n",
			     testsize);

	return 0;
}

int sm4_ctr_test(int testsize, int cpuid)
{
	struct timeval start, end;
	unsigned long time;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 };
	uint8_t std_key[16] = { 0xE0, 0x70, 0x99, 0xF1, 0xBF, 0xAF, 0xFD, 0x7F,
				0x24, 0x0C, 0xD7, 0x90, 0xCA, 0x4F, 0xE1, 0x34 };
	uint8_t std_iv[16] = { 0xC7, 0x2B, 0x65, 0x91, 0xA0, 0xD7, 0xDE, 0x8F,
			       0x6B, 0x40, 0x72, 0x33, 0xAD, 0x35, 0x81, 0xD6 };
	int i;
	int sm4_desc_id;
	EVP_CIPHER_CTX *ectx;
	int size, offset;

	if (testsize <= 0 || cpuid < 0)
		return -1;

	memcpy(input, randomdata[cpuid], testsize);
	if (multestsize) {
		gettimeofday(&start, NULL);
		for (i = 0; i < test_num; i++) {
			sm4_desc_id = -1;
			while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_CTR,
						     SM4_CRYPTO_ENCRYPT,
						     std_key, std_iv)) {
			}
			phytium_scto_sm4_update(sm4_desc_id, input, testsize,
						output);
			if (sm4_desc_id > 0)
				phytium_scto_mem_free(sm4_desc_id);
			multestnum[cpuid][0]++;
		}
		gettimeofday(&end, NULL);

		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);

		if (test_time)
			SCTO_LOG_INF(
				"scto ctr enc success, testnum:%d, testsize:%d, speed:%lu MBps\n",
				test_num, testsize, test_num * testsize / time);
	} else {
		sm4_desc_id = -1;
		while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_CTR,
					     SM4_CRYPTO_ENCRYPT, std_key,
					     std_iv)) {
		}
		size = testsize;
		offset = 0;
		i = 0;
		while (size) {
			get_random_data(&i, 2);
			if (i > size)
				i = size;
			phytium_scto_sm4_update(sm4_desc_id, input + offset, i,
						output + offset);
			size -= i;
			offset += i;
		}
		if (sm4_desc_id > 0)
			phytium_scto_mem_free(sm4_desc_id);
	}

	ectx = EVP_CIPHER_CTX_new();
	if (ectx == NULL) {
		SCTO_LOG_INF("EVP_CIPHER_CTX_new fail!\n");
		return -1;
	}
	EVP_CipherInit_ex(ectx, EVP_sm4_ctr(), NULL, std_key, std_iv, 1);
	EVP_CipherUpdate(ectx, cmpbuf, &i, input, testsize);
	EVP_CipherFinal(ectx, cmpbuf + testsize, &i);
	EVP_CIPHER_CTX_free(ectx);

	if (memcmp(output, cmpbuf, testsize)) {
		SCTO_LOG_INF("sm4 ctr encrypt error!\n");
		return -1;
	} else
		SCTO_LOG_INF("sm4-ctr enc (testsize %d) test passed!\n",
			     testsize);

	if (multestsize) {
		gettimeofday(&start, NULL);
		for (i = 0; i < test_num; i++) {
			sm4_desc_id = -1;
			while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_CTR,
						     SM4_CRYPTO_ENCRYPT,
						     std_key, std_iv)) {
			}
			phytium_scto_sm4_update(sm4_desc_id, output, testsize,
						cmpbuf);
			if (sm4_desc_id > 0)
				phytium_scto_mem_free(sm4_desc_id);
			multestnum[cpuid][0]++;
		}
		gettimeofday(&end, NULL);

		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);

		if (test_time)
			SCTO_LOG_INF(
				"scto ctr dec success, testnum:%d, testsize:%d, speed:%lu MBps\n",
				test_num, testsize, test_num * testsize / time);
	} else {
		sm4_desc_id = -1;
		while (phytium_scto_sm4_init(&sm4_desc_id, SM4_MODE_CTR,
					     SM4_CRYPTO_ENCRYPT, std_key,
					     std_iv)) {
		}
		size = testsize;
		offset = 0;
		i = 0;
		while (size) {
			get_random_data(&i, 2);
			if (i > size)
				i = size;
			phytium_scto_sm4_update(sm4_desc_id, output + offset, i,
						cmpbuf + offset);
			size -= i;
			offset += i;
		}
		if (sm4_desc_id > 0)
			phytium_scto_mem_free(sm4_desc_id);
	}

	if (memcmp(input, cmpbuf, testsize)) {
		SCTO_LOG_INF("sm4 ctr decrypt error!\n");
		return -1;
	} else
		SCTO_LOG_INF("sm4-ctr dec (testsize %d) test passed!\n",
			     testsize);

	return 0;
}

int kernel_sm4_cbc_test(int testsize, int cpuid, int op_fd)
{
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 };
	uint8_t std_key[16] = { 0xE0, 0x70, 0x99, 0xF1, 0xBF, 0xAF, 0xFD, 0x7F,
				0x24, 0x0C, 0xD7, 0x90, 0xCA, 0x4F, 0xE1, 0x34 };
	uint8_t std_iv[16] = { 0xC7, 0x2B, 0x65, 0x91, 0xA0, 0xD7, 0xDE, 0x8F,
			       0x6B, 0x40, 0x72, 0x33, 0xAD, 0x35, 0x81, 0xD6 };
	int i;
	long tmp;
	EVP_CIPHER_CTX *ectx;
	int size, offset;
	memcpy(input, randomdata[cpuid], testsize);

	struct af_alg_iv *alg_iv;
	struct cmsghdr *header;
	uint32_t *type;
	struct iovec iov;
	int iv_msg_size = CMSG_SPACE(sizeof(*alg_iv) + 16);
	char buffer[CMSG_SPACE(sizeof(*type)) + iv_msg_size];
	struct msghdr msg;

	if (testsize <= 0 || cpuid < 0)
		return -1;

	size = testsize;
	offset = 0;
	i = 0;
	while (size) {
		get_random_data(&i, 2);
		i &= 0xFFF0;
		if (i > size)
			i = size;

		memset(buffer, 0, sizeof(buffer));

		iov.iov_base = input + offset;
		iov.iov_len = i;

		msg.msg_control = buffer;
		msg.msg_controllen = sizeof(buffer);
		msg.msg_iov = &iov;
		msg.msg_iovlen = 1;

		/* Set encrypt/decrypt operation */
		header = CMSG_FIRSTHDR(&msg);
		if (!header)
			return -EINVAL;

		header->cmsg_level = SOL_ALG;
		header->cmsg_type = ALG_SET_OP;
		header->cmsg_len = CMSG_LEN(sizeof(*type));
		type = (void *)CMSG_DATA(header);
		*type = ALG_OP_ENCRYPT;

		/* Set IV */
		header = CMSG_NXTHDR(&msg, header);
		if (!header)
			return -EINVAL;

		header->cmsg_level = SOL_ALG;
		header->cmsg_type = ALG_SET_IV;
		header->cmsg_len = iv_msg_size;
		alg_iv = (void *)CMSG_DATA(header);
		alg_iv->ivlen = 16;
		if (offset == 0) {
			memcpy(alg_iv->iv, std_iv, 16);
		} else {
			memcpy(alg_iv->iv, output + offset - 16, 16);
		}

		tmp = sendmsg(op_fd, &msg, 0);
		if (tmp != i)
			return -1;
		else {
			tmp = read(op_fd, output + offset, i);
			if (tmp != i)
				return -1;
		}

		size -= i;
		offset += i;
	}

	ectx = EVP_CIPHER_CTX_new();
	if (ectx == NULL) {
		SCTO_LOG_INF("EVP_CIPHER_CTX_new fail!\n");
		return -1;
	}
	EVP_CipherInit_ex(ectx, EVP_sm4_cbc(), NULL, std_key, std_iv, 1);
	EVP_CipherUpdate(ectx, cmpbuf, &i, input, testsize);
	EVP_CipherFinal(ectx, cmpbuf + testsize, &i);
	EVP_CIPHER_CTX_free(ectx);

	if (memcmp(output, cmpbuf, testsize)) {
		SCTO_LOG_INF("sm4 kernel cbc encrypt error!\n");
		return -1;
	} else
		SCTO_LOG_INF("sm4 kernel cbc enc (testsize %d) test passed!\n",
			     testsize);

	size = testsize;
	offset = 0;
	i = 0;
	while (size) {
		get_random_data(&i, 2);
		i &= 0xFFF0;
		if (i > size)
			i = size;

		memset(buffer, 0, sizeof(buffer));

		iov.iov_base = output + offset;
		iov.iov_len = i;

		msg.msg_control = buffer;
		msg.msg_controllen = sizeof(buffer);
		msg.msg_iov = &iov;
		msg.msg_iovlen = 1;

		/* Set encrypt/decrypt operation */
		header = CMSG_FIRSTHDR(&msg);
		if (!header)
			return -EINVAL;

		header->cmsg_level = SOL_ALG;
		header->cmsg_type = ALG_SET_OP;
		header->cmsg_len = CMSG_LEN(sizeof(*type));
		type = (void *)CMSG_DATA(header);
		*type = ALG_OP_DECRYPT;

		/* Set IV */
		header = CMSG_NXTHDR(&msg, header);
		if (!header)
			return -EINVAL;

		header->cmsg_level = SOL_ALG;
		header->cmsg_type = ALG_SET_IV;
		header->cmsg_len = iv_msg_size;
		alg_iv = (void *)CMSG_DATA(header);
		alg_iv->ivlen = 16;
		if (offset == 0) {
			memcpy(alg_iv->iv, std_iv, 16);
		} else {
			memcpy(alg_iv->iv, output + offset - 16, 16);
		}

		tmp = sendmsg(op_fd, &msg, 0);
		if (tmp != i)
			return -1;
		else {
			tmp = read(op_fd, cmpbuf + offset, i);
			if (tmp != i)
				return -1;
		}

		size -= i;
		offset += i;
	}

	if (memcmp(input, cmpbuf, testsize)) {
		SCTO_LOG_INF("sm4 kernel cbc decrypt error!\n");
		return -1;
	} else
		SCTO_LOG_INF("sm4 kernel cbc dec (testsize %d) test passed!\n",
			     testsize);

	return 0;
}

int test_sm2_pubkey_case(int testsize, int cpuid)
{
	struct timeval start, end;
	unsigned long time;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 }, data[BUF_SIZE] = { 0 };
	long len;

	/* From gmssl key */
	uint8_t prikey[32] = { 0xff, 0xb5, 0x51, 0x02, 0x54, 0xfd, 0x92, 0xf5,
			       0xa4, 0x40, 0x64, 0x89, 0x07, 0x3f, 0xd5, 0x25,
			       0xa6, 0xfd, 0x0d, 0x4e, 0xb3, 0xfa, 0x23, 0x4f,
			       0x47, 0xa1, 0x5f, 0x0b, 0xf8, 0x41, 0x79, 0x99 };

	uint8_t pubkey[65] = { 0x04, 0x85, 0x89, 0xd0, 0x4d, 0xce, 0x23, 0x3a,
			       0xde, 0x30, 0x01, 0xe6, 0xb6, 0x16, 0x48, 0xb8,
			       0xa4, 0x67, 0xe5, 0xe7, 0xcf, 0x2c, 0x93, 0xfd,
			       0x4c, 0x4c, 0x3a, 0x26, 0xc7, 0x39, 0x9f, 0x11,
			       0x2b, 0xd9, 0xc0, 0x32, 0x3e, 0xa3, 0x2a, 0x93,
			       0x33, 0xf4, 0x67, 0x4a, 0x33, 0xf0, 0xde, 0x01,
			       0x2b, 0x1c, 0x6d, 0xf5, 0x90, 0xea, 0x1d, 0xd1,
			       0x1b, 0xb3, 0x26, 0x5c, 0x6f, 0x68, 0xa6, 0x29,
			       0xf7 };
	uint8_t pubkey2[65] = { 0 };

	gettimeofday(&start, NULL);

	int tmp;
	for (tmp = 0; tmp < test_num; tmp++) {
		phytium_scto_sm2_pubkey(prikey, pubkey2);

		if (multestsize)
			multestnum[cpuid][0]++;
	}

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	if (test_func) {
		test_print_buf("public key", pubkey2, 65);

		if (memcmp(pubkey, pubkey2, 65) != 0) {
			SCTO_LOG_ERR("pubkey error\r\n");
			test_print_buf("Expected public key", pubkey, 65);
		}
	}

	if (test_time)
		SCTO_LOG_INF(
			"TA sm2 pubkey success, testnum:%d,  time:%lu us, speed:%lu pps\n",
			test_num, time, test_num * 1000000ul / time);

	/* use it to encrypt what ? */
	memcpy(data, randomdata[cpuid], testsize);
	memcpy(input, data, testsize);

	uint32_t outlen;

	if (pacc_ceu_enc_sm2(testsize, input, pubkey2 + 1, cmpbuf)) {
		SCTO_LOG_INF("enc error!\r\n");
		return -1;
	}

	/* encode ... */
	scto_sm2encrypt_encode(cmpbuf, testsize + 97, output, &outlen);
	len = outlen;

	/* decrypt */
	phytium_scto_sm2_dec(output, len, prikey, sizeof(prikey), cmpbuf,
			     &outlen);
	if (outlen != testsize) {
		SCTO_LOG_INF("sm2 decrypt size error!\n");
		goto out;
	}

	if (memcmp(input, cmpbuf, testsize)) {
		test_print_buf("cmpbuf", cmpbuf, testsize);
		test_print_buf("input", input, testsize);
		test_print_buf("data", data, testsize);
		SCTO_LOG_INF("sm2 pubkey decrypt error!\n");
		goto out;
	} else
		SCTO_LOG_INF("sm2 pub key test passed!\n");

	return 0;

out:
	return -1;
}

int test_sm2_genkey(int testsize, int cpuid)
{
	struct timeval start, end;
	unsigned long time;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 }, data[BUF_SIZE] = { 0 };
	long len;
	uint8_t prikey[32] = { 0 };
	uint8_t pubkey[65] = { 0 };
	uint8_t pubkey2[65] = { 0 };

	gettimeofday(&start, NULL);

	int tmp;
	for (tmp = 0; tmp < test_num; tmp++) {
		phytium_scto_sm2_genkey(prikey, pubkey);

		if (multestsize)
			multestnum[cpuid][0]++;
	}

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	if (test_func) {
		test_print_buf("private key", prikey, 32);
		test_print_buf("public key", pubkey, 65);

		phytium_scto_sm2_pubkey(prikey, pubkey2);
		if (memcmp(pubkey, pubkey2, 65) != 0) {
			SCTO_LOG_ERR("pubkey error\r\n");
			test_print_buf("Unexpected public key", pubkey2, 65);
		} else {
			SCTO_LOG_INF("pubkey success\r\n");
		}
	}

	if (test_time)
		SCTO_LOG_INF(
			"TA sm2 pair success, testnum:%d,  time:%lu us, speed:%lu pps\n",
			test_num, time, test_num * 1000000ul / time);

	/* use it to encrypt what ? */
	memcpy(data, randomdata[cpuid], testsize);
	memcpy(input, data, testsize);

	uint32_t outlen;

	if (pacc_ceu_enc_sm2(testsize, input, pubkey + 1, cmpbuf)) {
		SCTO_LOG_INF("enc error!\r\n");
		return -1;
	}

	/* encode ... */
	scto_sm2encrypt_encode(cmpbuf, testsize + 97, output, &outlen);
	len = outlen;

	/* decrypt */
	phytium_scto_sm2_dec(output, len, prikey, sizeof(prikey), cmpbuf,
			     &outlen);
	if (outlen != testsize) {
		SCTO_LOG_INF("sm2 decrypt size error!\n");
		goto out;
	}

	if (memcmp(input, cmpbuf, testsize)) {
		test_print_buf("cmpbuf", cmpbuf, testsize);
		test_print_buf("input", input, testsize);
		test_print_buf("data", data, testsize);
		SCTO_LOG_INF("sm2 key decrypt error!\n");
		goto out;
	} else
		SCTO_LOG_INF("sm2 gen key test passed!\n");

	return 0;

out:
	return -1;
}

static uint8_t sm2_getZ(uint8_t *usrid, uint32_t usrid_len, uint8_t *pubkey,
			uint8_t Z[32])
{
	uint8_t id_bit_len[2];
	uint32_t sm2p256v1_a[8] = { 0xFFFFFFFC, 0xFFFFFFFF, 0x00000000,
				    0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
				    0xFFFFFFFF, 0xFFFFFFFE };
	uint32_t sm2p256v1_b[8] = { 0x4D940E93, 0xDDBCBD41, 0x15AB8F92,
				    0xF39789F5, 0xCF6509A7, 0x4D5A9E4B,
				    0x9D9F5E34, 0x28E9FA9E };
	uint32_t sm2p256v1_Gx[8] = { 0x334C74C7, 0x715A4589, 0xF2660BE1,
				     0x8FE30BBF, 0x6A39C994, 0x5F990446,
				     0x1F198119, 0x32C4AE2C };
	uint32_t sm2p256v1_Gy[8] = { 0x2139F0A0, 0x02DF32E5, 0xC62A4740,
				     0xD0A9877C, 0x6B692153, 0x59BDCEE3,
				     0xF4F6779C, 0xBC3736A2 };
	uint8_t id[16] = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
			   0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38 };
	int i;
	uint8_t tmp, *a, *b, *x, *y;

	if ((usrid == NULL) || (usrid_len == 0)) {
		usrid = id;
		usrid_len = 16;
	}

	a = (uint8_t *)sm2p256v1_a;
	b = (uint8_t *)sm2p256v1_b;
	x = (uint8_t *)sm2p256v1_Gx;
	y = (uint8_t *)sm2p256v1_Gy;
	for (i = 0; i < 16; i++) {
		tmp = a[i];
		a[i] = a[31 - i];
		a[31 - i] = tmp;
		tmp = b[i];
		b[i] = b[31 - i];
		b[31 - i] = tmp;
		tmp = x[i];
		x[i] = x[31 - i];
		x[31 - i] = tmp;
		tmp = y[i];
		y[i] = y[31 - i];
		y[31 - i] = tmp;
	}

	if (usrid_len >= (1 << 13))
		return -1;

	id_bit_len[0] = ((usrid_len << 3) >> 8) & 0xFF;
	id_bit_len[1] = (usrid_len << 3) & 0xFF;

	{
		int sm3_desc_id = -1;
		while (phytium_scto_sm3_init(&sm3_desc_id)) {
		}

		phytium_scto_sm3_update(sm3_desc_id, id_bit_len, 2);
		phytium_scto_sm3_update(sm3_desc_id, usrid, usrid_len);
		phytium_scto_sm3_update(sm3_desc_id, (uint8_t *)sm2p256v1_a,
					32);
		phytium_scto_sm3_update(sm3_desc_id, (uint8_t *)sm2p256v1_b,
					32);
		phytium_scto_sm3_update(sm3_desc_id, (uint8_t *)sm2p256v1_Gx,
					32);
		phytium_scto_sm3_update(sm3_desc_id, (uint8_t *)sm2p256v1_Gy,
					32);
		phytium_scto_sm3_update(sm3_desc_id, pubkey, 64);

		phytium_scto_sm3_final(sm3_desc_id, Z);
		if (sm3_desc_id > 0)
			phytium_scto_mem_free(sm3_desc_id);
	}

	return 0;
}

int test_sm2_exchangekey_func(int testsize, int cpuid)
{
	int i, ret;
	sm2_exchange_key_testvec_t *testvec;
	uint8_t shared_keys[2][256];

	for (i = 0; i < 1; i++) {
		uint8_t ZA[32] = { 0 };
		uint8_t ZB[32] = { 0 };
		uint8_t S1[32] = { 0 };
		uint8_t SA[32] = { 0 };

		testvec = &sm2_exchange_key_tv_template[i];

		if (sm2_getZ(testvec->IDA, testvec->IDA_len, testvec->Apubkey,
			     ZA)) {
			SCTO_LOG_ERR("Get ZA failed. %d\r\n", i);
		}
		if (sm2_getZ(testvec->IDB, testvec->IDB_len, testvec->Bpubkey,
			     ZB)) {
			SCTO_LOG_ERR("Get ZB failed. %d\r\n", i);
		}

		// test_print_buf("ZA  ", ZA, 32);
		// test_print_buf("ZB  ", ZB, 32);

		ret = phytium_scto_sm2_exchangekey(
			0, testvec->Aprikey, testvec->Bpubkey, testvec->aprikey,
			testvec->apubkey, testvec->bpubkey, ZA, ZB,
			testvec->c_len, shared_keys[0], S1, SA);
		if (ret) {
			SCTO_LOG_ERR("sm2 exchange failed[%d].\r\n", i);
			return ret;
		}

		ret = phytium_scto_sm2_exchangekey(
			1, testvec->Bprikey, testvec->Apubkey, testvec->bprikey,
			testvec->bpubkey, testvec->apubkey, ZB, ZA,
			testvec->c_len, shared_keys[1], NULL, NULL);

		if (ret) {
			SCTO_LOG_ERR("sm2 exchange failed[%d].\r\n", i);
			return ret;
		}

		if (memcmp(shared_keys[0], shared_keys[1], testvec->c_len) ||
		    memcmp(shared_keys[0], testvec->c, testvec->c_len)) {
			SCTO_LOG_ERR(
				"sm2 exchange not equal sharedkey[%d].\r\n", i);
			test_print_buf("A shared keys", shared_keys[0],
				       testvec->c_len);
			test_print_buf("B shared keys", shared_keys[1],
				       testvec->c_len);
			return -1;
		} else {
			SCTO_LOG_INF("sm2 exchange key  test passed!\n");
		}
	}

	/* test speed ... */
	{
		struct timeval start, end;
		unsigned long time;
		uint8_t ZA[32] = { 0 };
		uint8_t ZB[32] = { 0 };
		uint8_t S1[32] = { 0 };
		uint8_t SA[32] = { 0 };
		testvec = &sm2_exchange_key_tv_template[0];
		if (testsize > 256)
			testsize = 256;

		if (sm2_getZ("AAAAAAAAAA123456", 16, testvec->Apubkey, ZA)) {
			SCTO_LOG_ERR("Get ZA failed. %d\r\n", i);
		}
		if (sm2_getZ("BBBBBBBBBB123456", 16, testvec->Bpubkey, ZB)) {
			SCTO_LOG_ERR("Get ZB failed. %d\r\n", i);
		}

		gettimeofday(&start, NULL);
		for (i = 0; i < test_num; i++) {
			ret = phytium_scto_sm2_exchangekey(
				0, testvec->Aprikey, testvec->Bpubkey,
				testvec->aprikey, testvec->apubkey,
				testvec->bpubkey, ZA, ZB, testsize,
				shared_keys[0], S1, SA);
			if (ret) {
				SCTO_LOG_ERR("sm2 exchange failed[%d].\r\n", i);
				return ret;
			}

			if (multestsize)
				multestnum[cpuid][0]++;
		}
		gettimeofday(&end, NULL);
		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);

		if (test_time)
			SCTO_LOG_INF(
				"TA sm2 exchange success, testnum:%d,  time:%lu us, speed:%lu pps\n",
				test_num, time, test_num * 1000000ul / time);

		// compare with lib
		{
			uint8_t key_z_addr[1024];
			memcpy(key_z_addr, testvec->Bprikey, 32);
			memcpy(key_z_addr + 32, testvec->Bpubkey, 64);
			memcpy(key_z_addr + 96, testvec->bprikey, 32);
			memcpy(key_z_addr + 128, testvec->bpubkey, 64);
			memcpy(key_z_addr + 192, testvec->Apubkey, 64);
			memcpy(key_z_addr + 256, testvec->apubkey, 64);
			memcpy(key_z_addr + 320, ZB, 32);
			memcpy(key_z_addr + 352, ZA, 32);

			ret = pacc_ceu_exchange_sm2(1, testsize, key_z_addr,
						    shared_keys[1]);
			if (memcmp(shared_keys[0], shared_keys[1],
				   testvec->c_len)) {
				SCTO_LOG_ERR(
					"sm2 exchange not equal sharedkey[%d].\r\n",
					i);
				test_print_buf("A shared keys", shared_keys[0],
					       testvec->c_len);
				test_print_buf("B shared keys", shared_keys[1],
					       testvec->c_len);
				return -1;
			}
		}
	}

	return ret;
}

int kernel_sm2_enc_test(int testsize, int cpuid)
{
	struct timeval start, end;
	unsigned long time;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 };

	uint8_t prikey[32] = { 0x83, 0xdc, 0xa0, 0xad, 0xa3, 0xd7, 0x7e, 0xa4,
			       0xed, 0x20, 0x89, 0x56, 0x1d, 0xc3, 0x96, 0x37,
			       0x5e, 0xa9, 0xce, 0x50, 0x63, 0x29, 0xe4, 0x9c,
			       0x0d, 0xe4, 0x99, 0x49, 0x0a, 0xcd, 0x03, 0x78 };
	uint8_t pubkey[65] = { 0x04, 0xd7, 0x4e, 0xd1, 0x3e, 0x2d, 0xc9, 0x91,
			       0x14, 0x73, 0x16, 0x38, 0x5b, 0x67, 0x4b, 0xd5,
			       0x60, 0x01, 0xa6, 0x0f, 0x03, 0x9b, 0xfc, 0x01,
			       0x88, 0x6a, 0xeb, 0x52, 0xa3, 0x43, 0x0c, 0xdc,
			       0xbc, 0xbc, 0x01, 0xa4, 0x1e, 0x32, 0xce, 0x92,
			       0xb9, 0x4e, 0x50, 0x44, 0x2e, 0xb1, 0x5c, 0xd4,
			       0x24, 0xc1, 0x23, 0x7e, 0xc4, 0x2e, 0x44, 0x2c,
			       0x90, 0x95, 0x6a, 0x26, 0x46, 0x43, 0x93, 0x12,
			       0x26 };

	if (testsize <= 0 || cpuid < 0)
		return -1;

	memcpy(input, randomdata[cpuid], testsize);
	gettimeofday(&start, NULL);

	// test_print_buf("enc input", input, testsize);
	uint32_t outlen;
	outlen = testsize + 256;
	int tmp;
	for (tmp = 0; tmp < test_num; tmp++) {
		outlen = testsize + 256;
		phytium_scto_sm2_enc(input, testsize, pubkey, sizeof(pubkey),
				     output, &outlen);

		if (multestsize)
			multestnum[cpuid][0]++;
	}
	// test_print_buf("enc output", output, outlen);

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	if (test_time)
		SCTO_LOG_INF(
			"TA sm2 encryp success, testnum:%d, testsize:%d, time:%lu us, speed:%lu pps\n",
			test_num, testsize, time, test_num * 1000000ul / time);

	/* decode first */
	if (scto_sm2decrypt_decode(output, outlen, cmpbuf, &outlen)) {
		goto out;
	}

	/* 解密 */
	if (pacc_ceu_dec_sm2(outlen, cmpbuf, prikey, output)) {
		SCTO_LOG_ERR("pacc_ceu_dec_sm2 error\r\n");
		goto out;
	}

	if (memcmp(input, output, testsize)) {
		SCTO_LOG_ERR("sm2 encrypt error!\n");
		goto out;
	} else
		SCTO_LOG_INF("sm2 enc (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

int kernel_sm2_dec_test(int testsize, int cpuid)
{
	struct timeval start, end;
	unsigned long time;
	long len;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 },
		cmpbuf[BUF_SIZE] = { 0 };

	uint8_t prikey[32] = { 0x83, 0xdc, 0xa0, 0xad, 0xa3, 0xd7, 0x7e, 0xa4,
			       0xed, 0x20, 0x89, 0x56, 0x1d, 0xc3, 0x96, 0x37,
			       0x5e, 0xa9, 0xce, 0x50, 0x63, 0x29, 0xe4, 0x9c,
			       0x0d, 0xe4, 0x99, 0x49, 0x0a, 0xcd, 0x03, 0x78 };
	uint8_t pubkey[65] = { 0x04, 0xd7, 0x4e, 0xd1, 0x3e, 0x2d, 0xc9, 0x91,
			       0x14, 0x73, 0x16, 0x38, 0x5b, 0x67, 0x4b, 0xd5,
			       0x60, 0x01, 0xa6, 0x0f, 0x03, 0x9b, 0xfc, 0x01,
			       0x88, 0x6a, 0xeb, 0x52, 0xa3, 0x43, 0x0c, 0xdc,
			       0xbc, 0xbc, 0x01, 0xa4, 0x1e, 0x32, 0xce, 0x92,
			       0xb9, 0x4e, 0x50, 0x44, 0x2e, 0xb1, 0x5c, 0xd4,
			       0x24, 0xc1, 0x23, 0x7e, 0xc4, 0x2e, 0x44, 0x2c,
			       0x90, 0x95, 0x6a, 0x26, 0x46, 0x43, 0x93, 0x12,
			       0x26 };
	if (testsize <= 0 || cpuid < 0)
		return -1;

	memcpy(input, randomdata[cpuid], testsize);

	uint32_t outlen;

	if (pacc_ceu_enc_sm2(testsize, input, pubkey + 1, cmpbuf)) {
		SCTO_LOG_INF("enc error!\r\n");
		return -1;
	}

	/* encode ... */
	scto_sm2encrypt_encode(cmpbuf, testsize + 97, output, &outlen);
	len = outlen;

	// test_print_buf("get pacc_ceu_enc_sm2 output", output, testsize + 97);

	gettimeofday(&start, NULL);

	int tmp;
	outlen = len;
	for (tmp = 0; tmp < test_num; tmp++) {
		phytium_scto_sm2_dec(output, len, prikey, sizeof(prikey),
				     cmpbuf, &outlen);

		if (multestsize)
			multestnum[cpuid][0]++;
	}

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	if (test_time)
		SCTO_LOG_INF(
			"TA sm2 decryp success, testnum:%d, testsize:%d, time:%lu us, speed:%lu pps\n",
			test_num, testsize, time, test_num * 1000000ul / time);
	if (outlen != testsize) {
		SCTO_LOG_INF("sm2 decrypt size error!\n");
		goto out;
	}

	// test_print_buf("cmpbuf", cmpbuf, testsize);
	if (memcmp(input, cmpbuf, testsize)) {
		test_print_buf("cmpbuf", cmpbuf, testsize);
		SCTO_LOG_INF("sm2 decrypt error!\n");
		goto out;
	} else
		SCTO_LOG_INF("sm2 dec (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

int kernel_sm2_sign_test(int testsize, int cpuid)
{
	struct timeval start, end;
	unsigned long time;
	int ret;

	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 };

	uint8_t prikey[32] = { 0x83, 0xdc, 0xa0, 0xad, 0xa3, 0xd7, 0x7e, 0xa4,
			       0xed, 0x20, 0x89, 0x56, 0x1d, 0xc3, 0x96, 0x37,
			       0x5e, 0xa9, 0xce, 0x50, 0x63, 0x29, 0xe4, 0x9c,
			       0x0d, 0xe4, 0x99, 0x49, 0x0a, 0xcd, 0x03, 0x78 };
	uint8_t pubkey[65] = { 0x04, 0xd7, 0x4e, 0xd1, 0x3e, 0x2d, 0xc9, 0x91,
			       0x14, 0x73, 0x16, 0x38, 0x5b, 0x67, 0x4b, 0xd5,
			       0x60, 0x01, 0xa6, 0x0f, 0x03, 0x9b, 0xfc, 0x01,
			       0x88, 0x6a, 0xeb, 0x52, 0xa3, 0x43, 0x0c, 0xdc,
			       0xbc, 0xbc, 0x01, 0xa4, 0x1e, 0x32, 0xce, 0x92,
			       0xb9, 0x4e, 0x50, 0x44, 0x2e, 0xb1, 0x5c, 0xd4,
			       0x24, 0xc1, 0x23, 0x7e, 0xc4, 0x2e, 0x44, 0x2c,
			       0x90, 0x95, 0x6a, 0x26, 0x46, 0x43, 0x93, 0x12,
			       0x26 };
	uint8_t userid[16] = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
			       0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38 };
	uint8_t E[1024];

	if (testsize <= 0 || cpuid < 0)
		return -1;
	memcpy(input, randomdata[cpuid], testsize);

	gettimeofday(&start, NULL);

	if (sm2_sign_digest(userid, 16, pubkey + 1, input, testsize, E)) {
		SCTO_LOG_INF("sm2_sign_digest failed!\n");
		goto out;
	}
	int tmp;
	uint32_t outlen;
	outlen = 256;
	for (tmp = 0; tmp < test_num; tmp++) {
		phytium_scto_sm2_sign(E, 32, prikey, sizeof(prikey), output,
				      &outlen);

		if (multestsize)
			multestnum[cpuid][0]++;
	}

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	if (test_time)
		SCTO_LOG_INF(
			"TA sm2 sign success, testnum:%d, testsize:%d, time:%lu us, speed:%lu pps\n",
			test_num, testsize, time, test_num * 1000000ul / time);

	if (scto_sm2signature_decode(output, outlen, E + 32)) {
		goto out;
	}

	ret = pacc_ceu_verify_sm2(E, pubkey + 1);
	if (ret) {
		SCTO_LOG_ERR("verify error %d\r\n", ret);
		goto out;
	}

	SCTO_LOG_INF("sm2 sign (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

int kernel_sm2_verify_test(int testsize, int cpuid)
{
	struct timeval start, end;
	unsigned long time;
	int ret;
	uint32_t len;
	uint8_t input[BUF_SIZE] = { 0 }, output[BUF_SIZE] = { 0 };
	char cmd[1024];
	uint8_t prikey[32] = { 0x83, 0xdc, 0xa0, 0xad, 0xa3, 0xd7, 0x7e, 0xa4,
			       0xed, 0x20, 0x89, 0x56, 0x1d, 0xc3, 0x96, 0x37,
			       0x5e, 0xa9, 0xce, 0x50, 0x63, 0x29, 0xe4, 0x9c,
			       0x0d, 0xe4, 0x99, 0x49, 0x0a, 0xcd, 0x03, 0x78 };
	uint8_t pubkey[65] = { 0x04, 0xd7, 0x4e, 0xd1, 0x3e, 0x2d, 0xc9, 0x91,
			       0x14, 0x73, 0x16, 0x38, 0x5b, 0x67, 0x4b, 0xd5,
			       0x60, 0x01, 0xa6, 0x0f, 0x03, 0x9b, 0xfc, 0x01,
			       0x88, 0x6a, 0xeb, 0x52, 0xa3, 0x43, 0x0c, 0xdc,
			       0xbc, 0xbc, 0x01, 0xa4, 0x1e, 0x32, 0xce, 0x92,
			       0xb9, 0x4e, 0x50, 0x44, 0x2e, 0xb1, 0x5c, 0xd4,
			       0x24, 0xc1, 0x23, 0x7e, 0xc4, 0x2e, 0x44, 0x2c,
			       0x90, 0x95, 0x6a, 0x26, 0x46, 0x43, 0x93, 0x12,
			       0x26 };
	uint8_t userid[16] = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
			       0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38 };
	uint8_t E[32];

	if (testsize <= 0 || cpuid < 0)
		return -1;

	memcpy(input, randomdata[cpuid], testsize);

	if (sm2_sign_digest(userid, 16, pubkey + 1, input, testsize, E)) {
		SCTO_LOG_INF("sm2_sign_digest failed!\n");
		goto out;
	}

	if (pacc_ceu_sign_sm2(E, prikey, cmd)) {
		SCTO_LOG_ERR("sign error\r\n");
		goto out;
	}

	/* encode it */
	scto_sm2signature_encode(cmd, output, &len);

	gettimeofday(&start, NULL);

	int tmp;
	for (tmp = 0; tmp < test_num; tmp++) {
		memcpy(output + len, E, 32);
		ret = phytium_scto_sm2_verify(E, 32, pubkey, sizeof(pubkey),
					      output, len);
		if (ret) {
			SCTO_LOG_ERR("verify error %d\r\n", ret);
			goto out;
		}

		if (multestsize)
			multestnum[cpuid][0]++;
	}

	gettimeofday(&end, NULL);
	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);

	if (test_time)
		SCTO_LOG_INF(
			"TA sm2 verify success, testnum:%d, testsize:%d, time:%lu us, speed:%lu pps\n",
			test_num, testsize, time, test_num * 1000000ul / time);

	SCTO_LOG_INF("sm2 verify (testsize %d) test passed!\n", testsize);

	return 0;

out:

	return -1;
}

long count[8] = { 0 };

void *test(void *arg)
{
	int i = *(int *)arg;
	cpu_bind(tid[i], (i + 1) & 7);
	int size = start_size;
	struct sockaddr_alg sa = {
		.salg_family = AF_ALG,
		.salg_type = "skcipher",
		.salg_name = "cbc(sm4)",
	};
	int tfm_fd = -1;
	long op_fd;
	uint8_t std_key[16] = { 0xE0, 0x70, 0x99, 0xF1, 0xBF, 0xAF, 0xFD, 0x7F,
				0x24, 0x0C, 0xD7, 0x90, 0xCA, 0x4F, 0xE1, 0x34 };

	get_random_data(randomdata[i], test_size);

	if (test_kernel_sm4_cbc) {
		tfm_fd = socket(AF_ALG, SOCK_SEQPACKET, 0);
		if (tfm_fd < 0)
			return NULL;

		if (bind(tfm_fd, (struct sockaddr *)&sa, sizeof(sa)) < 0)
			return NULL;

		if (setsockopt(tfm_fd, SOL_ALG, ALG_SET_KEY, std_key, 16) < 0)
			return NULL;

		op_fd = accept(tfm_fd, NULL, 0);
		if (op_fd < 0)
			return NULL;
	}

	if (test_func) {
		if (test_sm2_key)
			test_sm2_genkey(test_size, i);

		if (test_sm2_pubkey)
			test_sm2_pubkey_case(test_size, i);

		if (test_sm2_exchangekey)
			test_sm2_exchangekey_func(test_size, i);

		if (test_sm3_dgst)
			sm3_dgst_test(test_size, i);
		if (test_sm2_en)
			kernel_sm2_enc_test(test_size, i);
		if (test_sm2_de)
			kernel_sm2_dec_test(test_size, i);
		if (test_sm2_sign)
			kernel_sm2_sign_test(test_size, i);
		if (test_sm2_verify)
			kernel_sm2_verify_test(test_size, i);
		if (test_sm4_cbc)
			sm4_cbc_test((test_size + 0xF) & 0xFFFFFFF0, i);
		if (test_sm4_ecb)
			sm4_ecb_test((test_size + 0xF) & 0xFFFFFFF0, i);
		if (test_sm4_ctr)
			sm4_ctr_test(test_size, i);
		if (test_kernel_sm4_cbc)
			kernel_sm4_cbc_test((test_size + 0xF) & 0xFFFFFFF0, i,
					    op_fd);
	} else {
		while (running) {
			if (multestsize)
				size = multestsize;

			if (test_sm3_dgst && sm3_dgst_test(size, i))
				break;
			if (test_sm2_en && kernel_sm2_enc_test(size, i))
				break;
			if (test_sm2_de && kernel_sm2_dec_test(size, i))
				break;
			if (test_sm2_sign && kernel_sm2_sign_test(size, i))
				break;
			if (test_sm2_verify && kernel_sm2_verify_test(size, i))
				break;
			if (test_sm4_cbc &&
			    sm4_cbc_test((size + 0xF) & 0xFFFFFFF0, i))
				break;
			if (test_sm4_ecb &&
			    sm4_ecb_test((size + 0xF) & 0xFFFFFFF0, i))
				break;
			if (test_sm4_ctr && sm4_ctr_test(size, i))
				break;
			if (test_kernel_sm4_cbc) {
				if (kernel_sm4_cbc_test((size + 0xF) &
								0xFFFFFFF0,
							i, op_fd))
					break;
			}
			size++;
			if (size > test_size) {
				get_random_data(randomdata[i], test_size);
				size = start_size;
			}
			count[i]++;
		}
	}

	if (test_kernel_sm4_cbc) {
		close(tfm_fd);
		close(op_fd);
	}

	test_num = 0;
	running = 0;

	return NULL;
}

void *print(void *arg)
{
	struct timeval start, end;
	unsigned long time, startcount, endcount;
	int i;

	if (arg == NULL)
		return NULL;

	while (running) {
		startcount = 0;
		endcount = 0;
		SCTO_LOG_INF("count:%ld %ld %ld %ld %ld %ld %ld %ld\n",
			     count[0], count[1], count[2], count[3], count[4],
			     count[5], count[6], count[7]);
		gettimeofday(&start, NULL);
		for (i = 0; i < 8; i++)
			startcount += multestnum[i][0];
		sleep(1);
		gettimeofday(&end, NULL);
		for (i = 0; i < 8; i++)
			endcount += multestnum[i][0];
		time = (end.tv_sec - start.tv_sec) * 1000000ul +
		       (end.tv_usec - start.tv_usec);
		if (multestsize)
			SCTO_LOG_INF(
				"multestsize:%d, startcount:0x%lx endcount:0x%lx  count:0x%lx / s\n",
				multestsize, startcount, endcount,
				(endcount - startcount) * 1000000ul / time);
		if (multestsize)
			SCTO_LOG_INF(
				"multestsize:%d, startcount:0x%lx speed:%ld MBps\n",
				multestsize, startcount,
				(endcount - startcount) * multestsize / time);
	}

	return 0;
}

void print_help(void)
{
	SCTO_LOG_ALWAYS(
		"-s (1-9)  1:sm2_en 2:sm2_de 3:sm2_sign 4:sm2_verify 5:sm3_dgst 6:sm4_cbc 7:sm4_ecb 8:sm4_ctr 9:kernel_sm4_cbc\n"
		" 10:sm2_key 11:sm2_exchangekey 12:sm2_pubkey\n");
	SCTO_LOG_ALWAYS("-t          test_time\n");
	SCTO_LOG_ALWAYS("-n  num     test_num\n");
	SCTO_LOG_ALWAYS("-l          test_size\n");
	SCTO_LOG_ALWAYS("-a  num     thread_num(1-8)\n");
	SCTO_LOG_ALWAYS("example: ./scto -s 5 -l 10000 -a 1\n");
}

static void signal_handler(int signal)
{
	SCTO_LOG_INF("signal:%d SIGINT:%d\n", signal, SIGINT);

	test_num = 0;
	running = 0;
}

int main(int argc, char *argv[])
{
	long i, ret;
	pthread_t ltid;
	int flag = 1, sflag = 0;
	int cpuid[thread_num];
	int ch;

	scto_log_set_stderr(SCTO_LOG_LL_INF);
	while ((ch = getopt(argc, argv, "m:s:tn:l:a:hpf")) != -1) {
		switch (ch) {
		case 'm':
			i = atol(optarg);
			multestsize = i;
			break;
		case 's':
			i = atol(optarg);
			switch (i) {
			case 1:
				test_sm2_en = 1;
				break;
			case 2:
				test_sm2_de = 1;
				break;
			case 3:
				test_sm2_sign = 1;
				break;
			case 4:
				test_sm2_verify = 1;
				break;
			case 5:
				test_sm3_dgst = 1;
				break;
			case 6:
				test_sm4_cbc = 1;
				break;
			case 7:
				test_sm4_ecb = 1;
				break;
			case 8:
				test_sm4_ctr = 1;
				break;
			case 9:
				test_kernel_sm4_cbc = 1;
				break;
			case 10:
				test_sm2_key = 1;
				break;
			case 11:
				test_sm2_exchangekey = 1;
				break;
			case 12:
				test_sm2_pubkey = 1;
				break;
			default:
				break;
			}
			flag = 0;
			break;
		case 't':
			test_time = 1;
			break;
		case 'n':
			i = atol(optarg);
			test_num = i;
			break;
		case 'l':
			i = atol(optarg);
			if (sflag == 0) {
				test_size = i;
				sflag = 1;
			} else if (sflag == 1) {
				if (test_size < i) {
					start_size = test_size;
					test_size = i;
				} else {
					start_size = i;
				}
				sflag = 2;
			}
			break;
		case 'a':
			i = atol(optarg);
			thread_num = i % 9;
			break;
		case 'h':
			print_help();
			return 0;
		case 'p':
			printf_data = 1;
			break;
		case 'f':
			test_func = 1;
			break;
		default:
			break;
		}
	}

	if (flag) {
		test_sm3_dgst = 1;
		test_sm4_cbc = 1;
		test_sm4_ecb = 1;
		test_sm4_ctr = 1;
	}

	signal(SIGINT, signal_handler);

	ret = phytium_scto_lib_scto_init();
	if (ret < 0)
		return -1;

	ERR_load_BIO_strings();
	ENGINE_load_builtin_engines();
	OpenSSL_add_all_algorithms();
	OPENSSL_init_crypto(OPENSSL_INIT_ENGINE_AFALG, NULL);

	for (i = 0; i < thread_num; i++) {
		cpuid[i] = i;
		ret = pthread_create(&tid[i], NULL, test, (void *)&cpuid[i]);
		if (ret) {
			SCTO_LOG_INF("pthread create failed!\n");
			return ret;
		}
	}

	ret = pthread_create(&ltid, NULL, print, (void *)i);
	if (ret) {
		SCTO_LOG_INF("pthread create failed!\n");
		return ret;
	}

	for (i = 0; i < thread_num; i++) {
		pthread_join(tid[i], NULL);
	}

	pthread_join(ltid, NULL);
	phytium_scto_lib_scto_exit();

	return 0;
}
