/* SPDX-License-Identifier: GPL-2.0-or-later */
#ifndef __CIPHER_H__
#define __CIPHER_H__

enum SCA_ALGO {
	AES = 0x0,
	DES = 0x1,
	TDES = 0x2,
	GHASH = 0X3,
	SM4 = 0x4,
};

enum SCA_MODE {
	ECB = 0,
	CTR = 1,
	CBC = 2,
	OFB = 3,
	XTS = 4,
	CBC_MAC = 5,
	CMAC = 6,
	GCM = 7,
	CCM = 8,
};

enum OP_MODE {
	ENC = 0,
	DEC = 1,
};

#define AES_BLOCK_BYTE_LEN (16)
#define AES_MIN_KEY_BYTE_LEN (16)
#define AES_MID_KEY_BYTE_LEN (24)
#define AES_MAX_KEY_BYTE_LEN (32)
#define AES_IV_BYTE_LEN (16)

#define AES_XTS_MIN_KEY_LEN (32)
#define AES_XTS_MAX_KEY_LEN (64)

#define DES_BLOCK_BYTE_LEN (8)
#define DES_KEY_BYTE_LEN (8)
#define DES_IV_BYTE_LEN (8)

#define TDES_BLOCK_BYTE_LEN (8)
#define TDES_KEY_BYTE_LEN (24)
#define TDES_IV_BYTE_LEN (8)

#define SM4_BLOCK_BYTE_LEN (16)
#define SM4_KEY_BYTE_LEN (16)
#define SM4_IV_BYTE_LEN (16)

#define SM4_XTS_KEY_LEN (32)

#define WALK_BYTE_LEN (512)

struct cipher_session_context {
	u8 iv[CACHE_LINE_SIZE];
	u8 pad[CACHE_LINE_SIZE];
	u8 xts_tweak[CACHE_LINE_SIZE];
	struct link_list_item src_list_buf[MAX_LINKS + 2];
	struct link_list_item dst_list_buf[MAX_LINKS + 2];
} __packed;

struct cipher_alg_context {
	u8 alg;
	u8 mode;
	u8 block_size;
	u8 block_shift;
	u8 iv_size;
	u8 key_len;
	u16 key_len_flag;
	u64 key_ioaddr;
	u64 ecb_key_ioaddr;
	u8 key[64];
	u8 ecb_key[64];
} __packed;

int sce_cipher_algs_register(void);
void sce_cipher_algs_unregister(void);

#endif
