/* SPDX-License-Identifier: GPL-2.0-or-later */
#ifndef __HASH_H__
#define __HASH_H__

enum HASH_ALGO {
	SHA1 = 0,
	SHA224 = 1,
	SHA256 = 2,
	SHA384 = 3,
	SHA512 = 4,
	MD5 = 5,
	SM3 = 8,
	MAC = 9,
};
enum HASH_MODE {
	NORMAL = 0,
	HMAC = 1,
};

#define SHA1_DIGEST_BYTE_LEN (20)
#define SHA1_BLOCK_BYTE_LEN (64)
#define SHA224_DIGEST_BYTE_LEN (28)
#define SHA224_BLOCK_BYTE_LEN (64)
#define SHA256_DIGEST_BYTE_LEN (32)
#define SHA256_BLOCK_BYTE_LEN (64)
#define SHA384_DIGEST_BYTE_LEN (48)
#define SHA384_BLOCK_BYTE_LEN (128)
#define SHA512_DIGEST_BYTE_LEN (64)
#define SHA512_BLOCK_BYTE_LEN (128)
#define MD5_DIGEST_BYTE_LEN (16)
#define MD5_BLOCK_BYTE_LEN (64)
#define SM3_DIGEST_BYTE_LEN (32)
#define SM3_BLOCK_BYTE_LEN (64)
#define HASH_BUFFER_SIZE (2048)

struct hash_session_context {
	u8 buffer[HASH_BUFFER_SIZE]; /* data block being processed */
	u8 state[128]; /* intermediate digest state */
	struct link_list_item list_buf[MAX_LINKS + 2];
	u64 state_ioaddr;
	u64 buffer_ioaddr;
	u64 list_buf_ioaddr;
	union {
		u64 data[3];
		struct {
			u64 total; /* number of bytes processed */
			u64 buffer_data_len;
			u64 hash_flag;
		};
	};
} __packed;

struct hash_alg_context {
	u8 alg;
	u8 mode;
	u8 block_size;
	u8 digest_size;
	u32 key_len;
	u64 key_ioaddr;
	u8 key[128];
} __packed;
int sce_hash_algs_register(void);
void sce_hash_algs_unregister(void);

#endif
