/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __SCE_DRIVER_H__
#define __SCE_DRIVER_H__

#define SCE_MAGIC_NUMBER (0x20220426736f14b9ul)

#define SCE_MAGIC_NUM 'p'
#define SCE_QUEUE_ALLOC _IO(SCE_MAGIC_NUM, 0xA)
#define SCE_QUEUE_FREE _IO(SCE_MAGIC_NUM, 0xB)
#define SCE_MEM_ALLOC _IO(SCE_MAGIC_NUM, 0xC)
#define SCE_MEM_FREE _IO(SCE_MAGIC_NUM, 0xD)

union sce_ioctl_info {
	struct {
		u64 tag;
		u64 paddr;
		u64 ioaddr;
		u64 outsize;
	};
	struct {
		u64 insize;
		u64 queue_type;
	};
};

struct sqlist {
	struct sqlist *next;
	struct sqlist *prev;
};

struct sce_msg {
	struct sce_mutex lock;
	struct sqlist list;
	u32 *base;
	u32 hash_cmd_space;
	u32 hash_rsp_space;
	u32 sca_cmd_space;
	u32 sca_rsp_space;
	u32 reserved[16];
	u32 queue_mask[4];
};

struct buf_desc {
	struct sqlist node;
	u64 magic;
	void *vaddr;
	u64 ioaddr;
	u64 size;
	int isio;
	int type;
};

struct sce_queue_reg {
	u32 cmd_wr_ptr;
	u32 rsp_rd_ptr;
	u32 reserved0[14];
	u32 cmd_rd_ptr;
	u32 rsp_wr_ptr;
	u32 reserved1[14];
	u32 stop_flag;
	u32 start_flag;
	u32 reserved2[14];
};

struct sce_queue_desc {
	void *reg;
};

static inline void sqlist_init(struct sqlist *list)
{
	list->next = list;
	list->prev = list;
}

static inline int sqlist_is_empty(struct sqlist *list)
{
	return list->next == list;
}

static inline void sqlist_remove(struct sqlist *node)
{
	node->prev->next = node->next;
	node->next->prev = node->prev;
}

static inline void sqlist_insert_tail(struct sqlist *list, struct sqlist *node)
{
	node->next = list;
	node->prev = list->prev;

	list->prev->next = node;
	list->prev = node;
}

static inline struct sqlist *sqlist_get_head(struct sqlist *list)
{
	struct sqlist *node;

	if (sqlist_is_empty(list))
		return NULL;

	node = list->next;
	sqlist_remove(node);

	return node;
}

extern struct sce_msg sce_driver;
extern struct sce_queue_desc queue_desc[3];

#endif
