/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __SCE_DRIVER_REG_H__
#define __SCE_DRIVER_REG_H__

#define HOST_OFFSET (0x5000)
#define SCE_STAT_OFFSET (0x00)
#define TRNG_OFFSET (0x6000)

#define RN_POOL_OFFSET (0x300)
#define HASH_OFFSET (0x280)
#define SCA_OFFSET (0x200)
#define ACA_OFFSET (0x00)

#define HASH_CQ_FUNC_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x00)
#define HASH_CQ_PARA_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x04)
#define HASH_CSQ_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x08)
#define HASH_CTRL_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x0C)
#define HASH_STAT_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x10)

#define HASH_INTR_STAT0_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x14)
#define HASH_INTR_STAT1_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x18)
#define HASH_INTR_STAT2_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x1C)
#define HASH_INTR_MASK0_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x40)
#define HASH_INTR_MASK1_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x44)
#define HASH_INTR_MASK2_OFFSET (HOST_OFFSET + HASH_OFFSET + 0x48)

#define SCA_CQ_FUNC_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x00)
#define SCA_CQ_PARA_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x04)
#define SCA_CSQ_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x08)
#define SCA_CTRL_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x0C)
#define SCA_STAT_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x10)

#define SCA_INTR_STAT0_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x14)
#define SCA_INTR_STAT1_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x18)
#define SCA_INTR_STAT2_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x1C)
#define SCA_INTR_MASK0_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x40)
#define SCA_INTR_MASK1_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x44)
#define SCA_INTR_MASK2_OFFSET (HOST_OFFSET + SCA_OFFSET + 0x48)

#define SCE_STAT_CLOCK_OFFSET (SCE_STAT_OFFSET + 0x00)
#define SCE_STAT_VERSION_OFFSET (SCE_STAT_OFFSET + 0x04)
#define SCE_STAT_CFG0_OFFSET (SCE_STAT_OFFSET + 0x08)
#define SCE_STAT_CFG1_OFFSET (SCE_STAT_OFFSET + 0x0C)
#define SCE_STAT_CFG2_OFFSET (SCE_STAT_OFFSET + 0x10)
#define SCE_STAT_MISC_OFFSET (SCE_STAT_OFFSET + 0x14)
#define SCE_STAT_SRAM_OFFSET (SCE_STAT_OFFSET + 0x18)

#define SCE_STAT_INTR_HOSTi_OFFSET(i) (SCE_STAT_OFFSET + 0x20 + (i << 3))
#define SCE_STAT_CFG_HOSTi_OFFSET(i) (SCE_STAT_OFFSET + 0x24 + (i << 3))

#define SCE_STAT_DMA_FIFO_OFFSET (SCE_STAT_OFFSET + 0xD0)
#define SCE_STAT_DMA_AXI_OFFSET (SCE_STAT_OFFSET + 0xD4)
#define SCE_STAT_OTP_SPACE0_OFFSET (SCE_STAT_OFFSET + 0xD8)
#define SCE_STAT_OTP_SPACE1_OFFSET (SCE_STAT_OFFSET + 0xDC)

#define RN_POOL_CTL_OFFSET (HOST_OFFSET + RN_POOL_OFFSET + 0x00)
#define RN_POOL_STAT_OFFSET (HOST_OFFSET + RN_POOL_OFFSET + 0x04)
#define RN_POOL_INTR_OFFSET (HOST_OFFSET + RN_POOL_OFFSET + 0x08)
#define RN_POOL_INTR_MSK_OFFSET (HOST_OFFSET + RN_POOL_OFFSET + 0x0c)
#define RN_POOL_DATA_WORD_OFFSET(i) \
	(HOST_OFFSET + RN_POOL_OFFSET + 0x10 + ((i) << 2))

#define TRNG_CR_OFFSET (TRNG_OFFSET + 0x00)
#define TRNG_MSEL_OFFSET (TRNG_OFFSET + 0x04)
#define TRNG_SR_OFFSET (TRNG_OFFSET + 0x08)
#define TRNG_DR_OFFSET (TRNG_OFFSET + 0x0c)

enum opcode_idx {
	INIT = 0x1,
	PROC = 0x2,
	FINISH = 0x3,
};

union clock_stat_reg {
	u32 data;
	struct {
		u32 hash_clk_en : 1;
		u32 sca_clk_en : 1;
		u32 aca_clk_en : 1;
		u32 trng_clk_en : 1;
		u32 otp_clk_en : 1;
		u32 reserved0 : 1;
		u32 dma_clk_en0 : 1;
		u32 dma_clk_en1 : 1;
		u32 dma_clk_en2 : 1;
		u32 reserved1 : 23;
	} __packed;
} __packed;

union cfg0_stat_reg {
	u32 data;
	struct {
		u32 hash_host_num : 5;
		u32 sca_host_num : 5;
		u32 aca_host_num : 5;
		u32 rn_pool_host_num : 5;
		u32 reserved0 : 11;
		u32 enc_when_wrapout : 1;
	} __packed;
} __packed;

union cfg1_stat_reg {
	u32 data;
	struct {
		u32 hash_cq_depth : 5;
		u32 sca_cq_depth : 5;
		u32 aca_cq_depth : 5;
		u32 reserved0 : 6;
		u32 wrapout_ctx_max_size : 6;
		u32 aca_sram_size : 5;
	} __packed;
} __packed;

union cfg2_stat_reg {
	u32 data;
	struct {
		u32 trng_inter_src_exist : 1;
		u32 otp_exist : 1;
		u32 otp_shadow_save_ao : 1;
		u32 otp_init_value : 1;
		u32 reserved0 : 28;
	} __packed;
} __packed;

union cfg_host_stat_reg {
	u32 data;
	struct {
		u32 hash_host_ctx1_num : 6;
		u32 hash_host_ctx2_num : 6;
		u32 sca_host_ctx1_num : 6;
		u32 sca_host_ctx2_num : 6;

		u32 hash_host_attr_sec : 1;
		u32 sca_host_attr_sec : 1;
		u32 aca_host_attr_sec : 1;
		u32 rn_pool_host_attr_sec : 1;
		u32 reserved : 4;
	} __packed;
} __packed;

union rnpool_ctrl_reg {
	u32 data;
	struct {
		u32 rn_pool_fill_req : 1;
		u32 trng_clock_enable : 1;
		u32 reserved : 30;
	} __packed;
} __packed;

union rnpool_stat_reg {
	u32 data;
	struct {
		u32 autocorr_test_err : 1;
		u32 trng_core_stat : 2;
		u32 reserved : 29;
	} __packed;
} __packed;

union rnpool_intr_reg {
	u32 data;
	struct {
		u32 intr_fill_done : 1;
		u32 reserved : 31;
	} __packed;
} __packed;

union rnpool_intr_msk_reg {
	u32 data;
	struct {
		u32 fill_done_msk : 1;
		u32 reserved : 31;
	} __packed;
} __packed;

union trng_cr_reg {
	u32 data;
	struct {
		u32 rngen : 1;
		u32 reserved1 : 3;
		u32 rosen : 4;
		u32 reserved2 : 8;
		u32 dien : 1;
		u32 erien : 1;
		u32 reserved3 : 6;
		u32 irqen : 1;
		u32 reserved4 : 7;
	} __packed;
} __packed;

union trng_msel_reg {
	u32 data;
	struct {
		u32 msel : 1;
		u32 reserved : 31;
	} __packed;
} __packed;

union trng_sr_reg {
	u32 data;
	struct {
		u32 htf : 1;
		u32 drdy : 1;
		u32 ererr : 1;
		u32 reserved : 29;
	} __packed;
} __packed;

union trng_dr_reg {
	u32 data;
	struct {
		u32 rdata : 32;
	} __packed;
} __packed;

union hash_init_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 key_len : 1;
		u32 key_src : 2;
		u32 mode : 3;
		u32 algo : 4;
		u32 reserved : 8;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union hash_proc_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 src_addr_type : 1;
		u32 reserved0 : 1;
		u32 padding_mode : 1;
		u32 is_last_proc : 1;
		u32 is_zero_proc : 1;
		u32 reserved1 : 13;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union hash_finish_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 mac_tlen : 7;
		u32 reserved : 11;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union hash_clone_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 reserved : 13;
		u32 dst_slot_id : 5;
		u32 src_slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union hash_warp_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 reserved : 18;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union hash_clear_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 reserved : 18;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union hash_csq_reg {
	u32 data;
	struct {
		u32 stat_idx : 5;
		u32 opcode_idx : 3;
		u32 done_slot_id : 5;
		u32 reserved : 19;
	} __packed;
} __packed;

union hash_ctrl_reg {
	u32 data;
	struct {
		u32 reserved0 : 1;
		u32 cq_wm : 5;
		u32 clk_en : 1;
		u32 reserved1 : 25;
	} __packed;
} __packed;

union hash_stat_reg {
	u32 data;
	struct {
		u32 host_stat : 2;
		u32 actv_slot_id : 5;
		u32 csq_ocpd_slots : 4;
		u32 cq_avail_slots : 5;
		u32 reserved : 16;
	} __packed;
} __packed;

union hash_intr_stat0_reg {
	u32 data;
	struct {
		u32 cq_wm_intr : 1;
		u32 opcode_err_intr : 1;
		u32 csq_rd_err_intr : 1;
		u32 cq_wr_err_intr : 1;
		u32 axi_to_err_intr : 1;
		u32 para_err_intr : 1;
		u32 reserved : 26;
	} __packed;
} __packed;

union hash_intr_stat1_reg {
	u32 cmd_finish_intr;
} __packed;

union hash_intr_stat2_reg {
	u32 op_err_intr;
} __packed;

union hash_intr_mask0_reg {
	u32 data;
	struct {
		u32 cq_wm_intr_mask : 1;
		u32 opcode_err_intr_mask : 1;
		u32 csq_rd_err_intr_mask : 1;
		u32 cq_wr_err_intr_mask : 1;
		u32 axi_to_err_intr_mask : 1;
		u32 para_err_intr_mask : 1;
		u32 reserved : 26;
	} __packed;
} __packed;

union hash_intr_mask1_reg {
	u32 cmd_finish_intr_mask;
} __packed;

union hash_intr_mask2_reg {
	u32 op_err_intr_mask;
} __packed;

union sca_init_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 mode : 4;
		u32 iv_load : 1;
		u32 key_len : 2;
		u32 key_src : 2;
		u32 algo : 3;
		u32 reserved : 6;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union sca_proc_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 dst_src_addr_type : 2;
		u32 is_last_proc : 1;
		u32 op_mode : 1;
		u32 reserved : 14;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union sca_finish_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 mac_tlen : 5;
		u32 reserved : 13;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union sca_clear_cmd {
	u32 data;
	struct {
		u32 trigger_intr : 1;
		u32 reserved : 18;
		u32 slot_id : 5;
		u32 opcode : 8;
	} __packed;
} __packed;

union sca_csq_reg {
	u32 data;
	struct {
		u32 stat_idx : 5;
		u32 opcode_idx : 3;
		u32 done_slot_id : 5;
		u32 reserved : 19;
	} __packed;
} __packed;

union sca_stat_reg {
	u32 data;
	struct {
		u32 host_stat : 2;
		u32 actv_slot_id : 5;
		u32 csq_ocpd_slots : 4;
		u32 cq_avail_slots : 5;
		u32 reserved : 16;
	} __packed;
} __packed;

#endif
