// SPDX-License-Identifier: GPL-2.0-or-later
#include <linux/dma-mapping.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <linux/delay.h>
#include <linux/kthread.h>
#include <linux/highmem.h>
#include <linux/libnvdimm.h>
#include <linux/delay.h>
#include "sce_driver_reg.h"
#include "sce_driver_common.h"
#include "sce_driver_resource.h"
#include "sce_driver_thread.h"
#include "sce_driver.h"

#define TRNG_SUCCESS 0x10000
#define INIT_OPCODE 0x80
#define PROC_OPCODE 0x40
#define FINISH_OPCODE 0x20
#define CLEAR_OPCODE 0xFF

static int do_sca_process(int index, int num, struct sca_cmd *sca_prv_cmd);
static u64 iv_paddr;
static u8 *iv_vaddr;
static void *reg_base[3][4] = { { NULL } };
static struct cmd *cmd_base[3][4] = { { NULL } };
static struct rsp *rsp_base[3][4] = { { NULL } };
static u8 is_ccm;
static u32 is_last;
static union hash_init_cmd hash_init = {
	.trigger_intr = 0,
	.key_len = 0,
	.key_src = 2,
	.mode = 0,
	.algo = 8,
	.reserved = 0,
	.slot_id = 0,
	.opcode = INIT_OPCODE,
};

static union hash_proc_cmd hash_proc = {
	.trigger_intr = 0,
	.src_addr_type = 0,
	.reserved0 = 0,
	.padding_mode = 1,
	.is_last_proc = 1,
	.is_zero_proc = 0,
	.reserved1 = 0,
	.slot_id = 0,
	.opcode = PROC_OPCODE,
};

static union hash_finish_cmd hash_finish = {
	.trigger_intr = 0,
	.mac_tlen = 31,
	.reserved = 0,
	.slot_id = 0,
	.opcode = FINISH_OPCODE,
};

static u8 hash_mac_len[9] = { 19, 27, 31, 47, 63, 15, 0, 0, 31 };
static u32 hash_cmd_step;
static u32 hash_slot_id = 8;
static u64 hash_cmd_tag[16] = { 0 };
static u32 hash_cmd_index[16] = { 0 };
static struct hash_cmd *hash_cmd_base[16] = { NULL };
static struct rsp *hash_cmd_rsp_buf[16] = { NULL };
static u32 hash_cmd_state[16] = { 0 };
static u32 hash_cmd_complete_num[16] = { 0 };
static void *hash_cmd_callback[16] = { NULL };
static void *hash_cmd_callback_data[16] = { NULL };

static union sca_init_cmd sca_init = {
	.trigger_intr = 0,
	.mode = 2,
	.iv_load = 1,
	.key_len = 0,
	.key_src = 2,
	.algo = 0,
	.reserved = 0,
	.slot_id = 0,
	.opcode = INIT_OPCODE,
};

static union sca_proc_cmd sca_proc = {
	.trigger_intr = 0,
	.dst_src_addr_type = 0,
	.is_last_proc = 1,
	.op_mode = 0,
	.reserved = 0,
	.slot_id = 0,
	.opcode = PROC_OPCODE,
};

static union sca_finish_cmd sca_finish = {
	.trigger_intr = 0,
	.mac_tlen = 0,
	.reserved = 0,
	.slot_id = 0,
	.opcode = FINISH_OPCODE,
};

static u32 sca_cmd_step;
static u32 sca_slot_id;
static u64 sca_cmd_tag[16] = { 0 };
static u32 sca_cmd_index[16] = { 0 };
static struct sca_cmd *sca_cmd_base[16] = { NULL };
static struct rsp *sca_cmd_rsp_buf[16] = { NULL };
static u32 sca_cmd_state[16] = { 0 };
static u32 sca_cmd_complete_num[16] = { 0 };
static void *sca_cmd_callback[16] = { NULL };

static struct hash_cmd *hash_cmd;
static struct trng_cmd *trng_cmd;
struct sca_cmd *sca_cmd;

static u8 sha1_iv_data[20] = {
	0x67, 0x45, 0x23, 0x01, 0xef, 0xcd, 0xab, 0x89, 0x98, 0xba,
	0xdc, 0xfe, 0x10, 0x32, 0x54, 0x76, 0xc3, 0xd2, 0xe1, 0xf0,
};

static u8 sha224_iv_data[32] = {
	0xc1, 0x05, 0x9e, 0xd8, 0x36, 0x7c, 0xd5, 0x07, 0x30, 0x70, 0xdd,
	0x17, 0xf7, 0x0e, 0x59, 0x39, 0xff, 0xc0, 0x0b, 0x31, 0x68, 0x58,
	0x15, 0x11, 0x64, 0xf9, 0x8f, 0xa7, 0xbe, 0xfa, 0x4f, 0xa4,
};

static u8 sha256_iv_data[32] = {
	0x6a, 0x09, 0xe6, 0x67, 0xbb, 0x67, 0xae, 0x85, 0x3c, 0x6e, 0xf3,
	0x72, 0xa5, 0x4f, 0xf5, 0x3a, 0x51, 0x0e, 0x52, 0x7f, 0x9b, 0x05,
	0x68, 0x8c, 0x1f, 0x83, 0xd9, 0xab, 0x5b, 0xe0, 0xcd, 0x19,
};

static u8 sha384_iv_data[64] = {
	0xcb, 0xbb, 0x9d, 0x5d, 0xc1, 0x05, 0x9e, 0xd8, 0x62, 0x9a, 0x29,
	0x2a, 0x36, 0x7c, 0xd5, 0x07, 0x91, 0x59, 0x01, 0x5a, 0x30, 0x70,
	0xdd, 0x17, 0x15, 0x2f, 0xec, 0xd8, 0xf7, 0x0e, 0x59, 0x39, 0x67,
	0x33, 0x26, 0x67, 0xff, 0xc0, 0x0b, 0x31, 0x8e, 0xb4, 0x4a, 0x87,
	0x68, 0x58, 0x15, 0x11, 0xdb, 0x0c, 0x2e, 0x0d, 0x64, 0xf9, 0x8f,
	0xa7, 0x47, 0xb5, 0x48, 0x1d, 0xbe, 0xfa, 0x4f, 0xa4,
};

static u8 sha512_iv_data[64] = {
	0x6a, 0x09, 0xe6, 0x67, 0xf3, 0xbc, 0xc9, 0x08, 0xbb, 0x67, 0xae,
	0x85, 0x84, 0xca, 0xa7, 0x3b, 0x3c, 0x6e, 0xf3, 0x72, 0xfe, 0x94,
	0xf8, 0x2b, 0xa5, 0x4f, 0xf5, 0x3a, 0x5f, 0x1d, 0x36, 0xf1, 0x51,
	0x0e, 0x52, 0x7f, 0xad, 0xe6, 0x82, 0xd1, 0x9b, 0x05, 0x68, 0x8c,
	0x2b, 0x3e, 0x6c, 0x1f, 0x1f, 0x83, 0xd9, 0xab, 0xfb, 0x41, 0xbd,
	0x6b, 0x5b, 0xe0, 0xcd, 0x19, 0x13, 0x7e, 0x21, 0x79,
};

static u8 md5_iv_data[16] = {
	0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
	0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10,
};

static u8 sm3_iv_data[32] = { 0x73, 0x80, 0x16, 0x6f, 0x49, 0x14, 0xb2, 0xb9,
			      0x17, 0x24, 0x42, 0xd7, 0xda, 0x8a, 0x06, 0x00,
			      0xa9, 0x6f, 0x30, 0xbc, 0x16, 0x31, 0x38, 0xaa,
			      0xe3, 0x8d, 0xee, 0x4d, 0xb0, 0xfb, 0x0e, 0x4e };

static int hash_task_index;
static int sca_task_index;
static int trng_task_index;
void *task;

//cmac func
void bit128_xor(uint8_t *a, uint8_t *b, uint8_t *out)
{
	int i;

	dsb(sy);
	dmb(sy);

	for (i = 0; i < 16; i++) {
		dsb(sy);
		smp_mb(); //memory barrier
		out[i] = a[i] ^ b[i];
		smp_mb(); //memory barrier
	}
	smp_wmb(); //memory barrier
	smp_rmb(); //memory barrier
	dmb(sy);
	dsb(sy);
}
void bit128_left_shift(uint8_t *in, uint8_t *out)
{
	int i;
	uint8_t temp[16];

	memcpy(temp, in, 16);
	for (i = 0; i < 16; i++) {
		out[i] = temp[i] << 1;
		if (i < 15)
			out[i] |= (temp[i + 1] & 0x80) >> 7;
	}
}
void bit128_right_shift(uint8_t *in, uint8_t *out)
{
	int i;
	uint8_t temp[16];

	memcpy(temp, in, 16);
	for (i = 0; i < 16; i++) {
		out[i] = temp[i] >> 1;
		if (i)
			out[i] |= (temp[i - 1] & 0x1) << 7;
	}
}
int bit128_check_bit(uint8_t *in, uint8_t bit)
{
	uint8_t byteindex, bitmask;

	byteindex = bit / 8;

	bitmask = 0x80 >> (bit & 0x7);

	return in[byteindex] & bitmask;
}
void gf128_mul(uint8_t *u, uint8_t *v, uint8_t *out)
{
	uint8_t r[16], w[16], z[16];
	int i;

	//a)令R=b8(11100001) || b120(0)
	memset(r, 0, 16);
	r[0] = 0xE1;

	//b)令W=b128(0)
	memset(w, 0, 16);

	//c)令Z = U
	memcpy(z, u, 16);

	for (i = 0; i < 128; i++) {
		if (bit128_check_bit(v, i))
			bit128_xor(w, z, w);

		if (z[15] & 0x1) {
			bit128_right_shift(z, z);
			bit128_xor(z, r, z);
		} else
			bit128_right_shift(z, z);
	}

	memcpy(out, w, 16);
}
void gf128_hash(uint8_t *h, uint8_t *w, uint64_t w_size, uint8_t *z,
		uint64_t z_size, uint8_t *out)
{
	uint64_t k, u, l, v, i;
	uint8_t btemp[16], x[16];

	//0 < u <= 128, 0 < v <= 128
	//len(w) = 128 * (k - 1) + u, len(z) = 128 * (l - 1) + v, bits length
	w_size *= 8; //bit size
	z_size *= 8; //bit size

	if (w_size & 0x7F) {
		k = w_size / 128 + 1;
		u = w_size & 0x7F;
	} else {
		k = w_size / 128;
		u = 128;
	}

	if (z_size & 0x7F) {
		l = z_size / 128 + 1;
		v = z_size & 0x7F;
	} else {
		l = z_size / 128;
		v = 128;
	}

	//a)X(0) = b128(0)
	memset(x, 0, 16);

	//b)if 1 <= i <= K-1, X(i) =  (X(i-1) ^ W(i)) * H;
	for (i = 1; i < k; i++) {
		bit128_xor(x, w, x);
		gf128_mul(x, h, x);
		w += 16;
	}
	//c)if k > 0, X(k) = (X(k-1) ^ (W(k) || (b128-u)(0))) * H
	if (k) {
		memset(btemp, 0, 16);
		memcpy(btemp, w, u / 8);
		bit128_xor(x, btemp, x);
		gf128_mul(x, h, x);
	}
	//d)if K+1 <= i <= K+L-1, X(i) =  (X(i-1) ^ Z(i-k)) * H;
	for (i = 1; i < l; i++) {
		bit128_xor(x, z, x);
		gf128_mul(x, h, x);
		z += 16;
	}
	//e)if l > 0, X(k+l) = (X(k+l-1) ^ (Z(l) || (b128-v)(0))) * H
	if (l) {
		memset(btemp, 0, 16);
		memcpy(btemp, z, v / 8);
		bit128_xor(x, btemp, x);
		gf128_mul(x, h, x);
	}
	//f)X(k+l+1) = (X(k+l) ^ (b64(len(w)) || b64(len(z)))) * H
	btemp[0] = (w_size >> 56) & 0xFF;
	btemp[1] = (w_size >> 48) & 0xFF;
	btemp[2] = (w_size >> 40) & 0xFF;
	btemp[3] = (w_size >> 32) & 0xFF;
	btemp[4] = (w_size >> 24) & 0xFF;
	btemp[5] = (w_size >> 16) & 0xFF;
	btemp[6] = (w_size >> 8) & 0xFF;
	btemp[7] = (w_size >> 0) & 0xFF;
	btemp[8] = (z_size >> 56) & 0xFF;
	btemp[9] = (z_size >> 48) & 0xFF;
	btemp[10] = (z_size >> 40) & 0xFF;
	btemp[11] = (z_size >> 32) & 0xFF;
	btemp[12] = (z_size >> 24) & 0xFF;
	btemp[13] = (z_size >> 16) & 0xFF;
	btemp[14] = (z_size >> 8) & 0xFF;
	btemp[15] = (z_size >> 0) & 0xFF;

	bit128_xor(x, btemp, x);
	gf128_mul(x, h, x);
	memcpy(out, x, 16);
}

static void _sce_gcm_read_size(uint8_t *dst, size_t ofs, uint64_t size)
{
	size_t i = 0;

	for (i = 0; i < sizeof(size); i++)
		dst[ofs - i] = (size >> (i * 8)) & 0xFF;

}

static void get_set_tag(struct sce_link_list_item *src_link_list, int skip, u8 *t,
			int tag_len, bool set)
{
	int i = 0;
	int tmp_len = 0, cp_len = 0;

	while (src_link_list[i].address != 0) {
		if (tmp_len + src_link_list[i].size + 1 <= skip) {
			tmp_len += src_link_list[i].size + 1;
			i++;
			continue;
		}

		if (tmp_len < skip) {
			if (tmp_len + src_link_list[i].size + 1 >=
			    skip + tag_len) {
				if (set) {
					memcpy(phys_to_virt(
						src_link_list[i].address +
						skip - tmp_len),
						t, tag_len);
				} else {
					memcpy(t,
						phys_to_virt(
						src_link_list[i].address +
						skip - tmp_len),
						tag_len);
				}
				break;
			}
			if (set) {
				memcpy(phys_to_virt(
					src_link_list[i].address +
					skip - tmp_len),
				t, src_link_list[i].size + 1);
			} else {
				memcpy(t,
				phys_to_virt(
					src_link_list[i].address +
					skip - tmp_len),
				src_link_list[i].size + 1);
			}
			cp_len += src_link_list[i].size + 1;
			i++;
			continue;
		}
		if (tmp_len + src_link_list[i].size + 1 >= skip + tag_len) {
			if (set) {
				memcpy(phys_to_virt(src_link_list[i].address),
					t + cp_len, tag_len - cp_len);
			} else {
				memcpy(t + cp_len,
					phys_to_virt(src_link_list[i].address),
					tag_len - cp_len);
			}
			break;
		}
		if (set) {
			memcpy(phys_to_virt(src_link_list[i].address),
				t + cp_len, src_link_list[i].size + 1);
		} else {
			memcpy(t + cp_len,
				phys_to_virt(src_link_list[i].address),
				src_link_list[i].size + 1);
		}
		cp_len += src_link_list[i].size + 1;
		i++;

	}
}

static int make_link_list(struct sce_link_list_item *dst_link_list, int k,
			  struct sce_link_list_item *src_link_list, int start,
			  int end)
{
	int i = 0;
	int tmp_len = 0;

	if (start >= end)
		return -1;

	while (src_link_list[i].address != 0) {
		if (tmp_len + src_link_list[i].size + 1 <= start) {
			tmp_len += src_link_list[i].size + 1;
			i++;
			continue;
		}
		if (tmp_len < start) {
			dst_link_list[k].address =
				src_link_list[i].address + start - tmp_len;
			if (tmp_len + src_link_list[i].size + 1 >= end) {
				dst_link_list[k].size = end - start - 1;
				dst_link_list[k].address |= 0x300000000000;
				dst_link_list[k].reserved = 0;
				k++;
				break;
			}
			dst_link_list[k].size = src_link_list[i].size;

			dst_link_list[k].address |= 0x300000000000;
			dst_link_list[k].reserved = 0;
			tmp_len += src_link_list[i].size + 1;
			i++;
			k++;
			continue;

		}
		if (tmp_len + src_link_list[i].size + 1 >= end) {
			dst_link_list[k].address = src_link_list[i].address;
			dst_link_list[k].size = end - tmp_len - 1;
			dst_link_list[k].address |= 0x300000000000;
			dst_link_list[k].reserved = 0;
			k++;
			break;
		}
		dst_link_list[k].address = src_link_list[i].address;
		dst_link_list[k].address |= 0x300000000000;
		dst_link_list[k].size = src_link_list[i].size;
		dst_link_list[k].reserved = 0;
		tmp_len += src_link_list[i].size + 1;
		i++;
		k++;

	}
	flush_dcache_page(virt_to_page(dst_link_list));
	dsb(sy);

	return k;
}

static int sca_gcm_process(int index, int num, struct sca_cmd *cmd)
{
	u32 cmd_rdptr;
	int ret = 0, i, k = 0;
	struct sca_cmd *sca_gcm_cmd = NULL;
	u8 *h = NULL, *y0 = NULL, *temp = NULL, *y = NULL, *t = NULL,
	   *ghashzero = NULL;
	u64 h_paddr, y_paddr, y0_paddr, t_paddr, temp_paddr, in_paddr,
		src_addr = 0, dst_addr = 0;
	u8 *aad_vaddr = NULL, *out_vaddr = NULL, *iv_vaddr = NULL, *pad = NULL;
	struct sce_link_list_item *src_items = NULL, *dst_items = NULL,
			     *src_link_list = NULL, *dst_link_list = NULL,
			     *mac_src_link_list = NULL;
	int padding_sz = 0, data_flag = 0, tmp_len = 0;

	h = sce_malloc(sizeof(u8) * 16);
	if (!h) {
		sce_err("%s:%d malloc h failed!\n", __func__, __LINE__);
		ret = -1;
		goto err1;
	}
	memset(h, 0, 16);

	y = sce_malloc(sizeof(u8) * 16);
	if (!y) {
		sce_err("%s:%d malloc y failed!\n", __func__, __LINE__);
		ret = -1;
		goto err2;
	}
	memset(y, 0, 16);

	y0 = sce_malloc(sizeof(u8) * 16);
	if (!y0) {
		sce_err("%s:%d malloc y0 failed!\n", __func__, __LINE__);
		ret = -1;
		goto err3;
	}
	memset(y0, 0, 16);

	temp = sce_malloc(sizeof(u8) * 16);
	if (!temp) {
		sce_err("%s:%d malloc temp failed!\n", __func__, __LINE__);
		ret = -1;
		goto err4;
	}
	memset(temp, 0, 16);

	t = sce_malloc(sizeof(u8) * 16 * 2);
	if (!t) {
		sce_err("%s:%d malloc t failed!\n", __func__, __LINE__);
		ret = -1;
		goto err5;
	}
	memset(t, 0, 16 * 2);

	sca_gcm_cmd = sce_malloc(sizeof(struct sca_cmd) + CACHE_LINE_SIZE);
	if (!sca_gcm_cmd) {
		sce_err("%s:%d malloc sca_xts_cmd failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err6;
	}
	memset(sca_gcm_cmd, 0, sizeof(struct sca_cmd));

	src_link_list =
		sce_malloc(sizeof(struct sce_link_list_item) * 40 + CACHE_LINE_SIZE);
	if (!src_link_list) {
		sce_err("%s:%d malloc src_link_list failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err7;
	}
	memset(src_link_list, 0,
	       sizeof(struct sce_link_list_item) * 40 + CACHE_LINE_SIZE);

	dst_link_list =
		sce_malloc(sizeof(struct sce_link_list_item) * 40 + CACHE_LINE_SIZE);
	if (!dst_link_list) {
		sce_err("%s:%d malloc dst_link_list failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err8;
	}
	memset(dst_link_list, 0,
	       sizeof(struct sce_link_list_item) * 40 + CACHE_LINE_SIZE);

	ghashzero = sce_malloc(48);
	if (!ghashzero) {
		sce_err("%s:%d malloc ghashzero failed!\n", __func__, __LINE__);
		ret = -1;
		goto err9;
	}
	memset(ghashzero, 0, 48);

	mac_src_link_list =
		sce_malloc(sizeof(struct sce_link_list_item) * 40 + CACHE_LINE_SIZE);
	if (!mac_src_link_list) {
		sce_err("%s:%d malloc mac_src_link_list failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err10;
	}
	memset(mac_src_link_list, 0, sizeof(struct sce_link_list_item) * 40);

	pad = ghashzero + 16;

	is_ccm = 1;

	memset(sca_gcm_cmd, 0, sizeof(struct sca_cmd));
	memcpy(sca_gcm_cmd, cmd, sizeof(struct sca_cmd));

	aad_vaddr = phys_to_virt(cmd->in);
	in_paddr = cmd->in + cmd->aad_len;
	out_vaddr = phys_to_virt(cmd->out);
	iv_vaddr = phys_to_virt(cmd->iv);
	y_paddr = virt_to_phys(y);
	y0_paddr = virt_to_phys(y0);
	t_paddr = virt_to_phys(t);
	h_paddr = virt_to_phys(h);
	temp_paddr = virt_to_phys(temp);
	//1.H = enc(b128(0))
	sca_gcm_cmd->algo = sca_cmd->algo;
	sca_gcm_cmd->mode = ECB;
	sca_gcm_cmd->op_mode = 0; //ENC: 0
	sca_gcm_cmd->in = h_paddr;
	sca_gcm_cmd->in_size = 16;
	sca_gcm_cmd->key = cmd->key;
	sca_gcm_cmd->iv = 0;
	sca_gcm_cmd->out = h_paddr;
	sca_gcm_cmd->dma_mode = 0;
	do_sca_process(index, num, sca_gcm_cmd);

	//2.如果len(S) = 96, Y0 = S || b31(0) || 1,否则 Y0 = G(H, {}, S).

	memcpy(y0, iv_vaddr, 12);
	y0[15] = 1;

	if (!cmd->op_mode) {
		//3.CTR: Y(i) = inc(Y(i-1)); C(i) = D(i) ^ enc(Y(i))
		memcpy(y, y0, 16);
		for (i = 16; i > 0; i--)
			if (++y[i - 1])
				break;

		if (cmd->in_size && (cmd->dma_mode & 0x2)) { //src
			src_items = (struct sce_link_list_item *)(aad_vaddr);
			i = 0;
			k = 0;
			data_flag = 0;
			tmp_len = 0;
			while (src_items[i].address != 0) {
			if (tmp_len + src_items[i].size + 1 <=
				cmd->aad_len) {
				tmp_len +=
					src_items[i].size + 1;
				i++;
				continue;
			} else {
				if (data_flag == 0) {
					src_link_list[k]
						.address =
						src_items[i]
							.address +
						cmd->aad_len -
						tmp_len;
					src_link_list[k]
						.address |=
						0x300000000000;
					tmp_len +=
						src_items[i]
							.size +
						1;
					if (tmp_len >=
						cmd->aad_len +
							cmd->in_size) {
						src_link_list[k]
							.size =
							cmd->in_size -
							1;
					} else {
						src_link_list[k]
						.size =
						src_items[i]
						.size + 1 -
						(tmp_len -
						cmd->aad_len);
					}
					src_link_list[k]
						.reserved = 0;
					data_flag = 1;
					k++;
					i++;
				} else {
					if (tmp_len <=
						cmd->aad_len +
							cmd->in_size) {
					src_link_list[k]
						.address =
						src_items[i]
						.address;
					src_link_list[k]
						.address |=
						0x300000000000;
					src_link_list[k]
						.size =
						src_items[i]
						.size;
					src_link_list[k]
						.reserved = 0;
					k++;
					i++;
					} else {
					src_link_list[k]
						.address =
						src_items[i]
						.address;
					src_link_list[k]
						.address |=
						0x300000000000;
					src_link_list[k]
						.size =
						cmd->aad_len +
						cmd->in_size -
						tmp_len;
					src_link_list[k]
						.reserved =
						0;
					k++;
					i++;
					break;
					}
					tmp_len +=
						src_items[i]
							.size +
						1;
				}
			}
			}
			src_link_list[k].address = 0;
			src_link_list[k].size = SCE_LINK_END_SIZE;
			src_link_list[k].reserved = SCE_LINK_END_SIZE;
			src_addr = virt_to_phys(src_link_list);
			flush_dcache_page(virt_to_page(src_link_list));
			dsb(sy);
		} else {
			src_addr = in_paddr;
		}

		if (cmd->in_size && (cmd->dma_mode & 0x1)) { //dst
			dst_items =
				(struct sce_link_list_item *)(phys_to_virt(
					cmd->out));
			i = 0;
			k = 0;
			data_flag = 0;
			tmp_len = 0;
			while (dst_items[i].address != 0) {
				if (dst_items[i].size + 1 + tmp_len >=
					cmd->in_size) {
					dst_link_list[k].address =
						dst_items[i].address;
					dst_link_list[k].address |=
						0x300000000000;
					dst_link_list[k].size =
						cmd->in_size - 1 -
						tmp_len;
					dst_link_list[k].reserved = 0;
					k++;
					break;
				}
				dst_link_list[k].address =
					dst_items[i].address;
				dst_link_list[k].address |=
					0x300000000000;
				dst_link_list[k].size =
					dst_items[i].size;
				dst_link_list[k].reserved = 0;
				tmp_len +=
					dst_items[i].size + 1;
				k++;
				i++;

			}
			dst_link_list[k].address = 0;
			dst_link_list[k].size = SCE_LINK_END_SIZE;
			dst_link_list[k].reserved = SCE_LINK_END_SIZE;
			dst_addr = virt_to_phys(dst_link_list);
			flush_dcache_page(virt_to_page(dst_link_list));
			dsb(sy);
		} else {
			dst_addr = cmd->out;
		}

		if (cmd->in_size) {
			sca_gcm_cmd->algo = sca_cmd->algo;
			sca_gcm_cmd->mode = CTR;
			sca_gcm_cmd->op_mode = 0; //ENC: 0
			sca_gcm_cmd->in = src_addr; //输入为plain
			sca_gcm_cmd->in_size = cmd->in_size; //datalen
			sca_gcm_cmd->key = cmd->key;
			sca_gcm_cmd->iv = y_paddr;
			sca_gcm_cmd->out = dst_addr;
			sca_gcm_cmd->dma_mode = cmd->dma_mode;
			do_sca_process(index, num, sca_gcm_cmd);
		}

		k = 0;
		padding_sz = (16 - (cmd->in_size & 0xf)) & 0xf;
		_sce_gcm_read_size(pad, padding_sz + 7, cmd->aad_len * 8);
		_sce_gcm_read_size(pad, padding_sz + 15, cmd->in_size * 8);

		data_flag = 0;
		memset(mac_src_link_list, 0, sizeof(*mac_src_link_list));
		src_items = (struct sce_link_list_item *)mac_src_link_list;
		if (cmd->aad_len) {
			if (cmd->dma_mode & 0x2) {
				k = make_link_list(
					src_items, k,
					(struct sce_link_list_item *)(aad_vaddr), 0,
					cmd->aad_len);
			} else {
				src_items[k].address = cmd->in;
				src_items[k].address |= 0x300000000000;
				src_items[k].size = cmd->aad_len - 1;
				src_items[k].reserved = 0;
				k++;
			}
			if (cmd->aad_len & 0xf) {
				src_items[k].address = virt_to_phys(ghashzero);
				src_items[k].address |= 0x300000000000;
				src_items[k].size =
					16 - (cmd->aad_len & 0xf) - 1;
				src_items[k].reserved = 0;
				k++;
			}
		}
		if (cmd->in_size) {
			if (cmd->dma_mode & 0x1) {
				k = make_link_list(
					src_items, k,
					(struct sce_link_list_item *)(phys_to_virt(
						cmd->out)),
					0, cmd->in_size);
			} else {
				src_items[k].address = cmd->out;
				src_items[k].address |= 0x300000000000;
				src_items[k].size = cmd->in_size - 1;
				src_items[k].reserved = 0;
				k++;
			}
		}
		src_items[k].address = virt_to_phys(pad);
		src_items[k].address |= 0x300000000000;
		src_items[k].size = 16 + padding_sz - 1;
		src_items[k].reserved = 0;
		k++;

		src_items[k].address = 0;
		src_items[k].size = SCE_LINK_END_SIZE;
		src_items[k].reserved = SCE_LINK_END_SIZE;
		k++;

		flush_dcache_page(virt_to_page(mac_src_link_list));
		dsb(sy);

		sca_gcm_cmd->algo = GHASH;
		sca_gcm_cmd->mode = CMAC;
		sca_gcm_cmd->op_mode = 0;
		sca_gcm_cmd->in = virt_to_phys(mac_src_link_list);
		sca_gcm_cmd->in_size = 16;
		sca_gcm_cmd->key = h_paddr;
		sca_gcm_cmd->iv = 0;
		sca_gcm_cmd->out = t_paddr;
		sca_gcm_cmd->dma_mode = 2;
		do_sca_process(index, num, sca_gcm_cmd);

		sca_gcm_cmd->algo = sca_cmd->algo;
		sca_gcm_cmd->mode = ECB;
		sca_gcm_cmd->op_mode = 0; //ENC: 0
		sca_gcm_cmd->in = y0_paddr;
		sca_gcm_cmd->in_size = 16;
		sca_gcm_cmd->key = cmd->key;
		sca_gcm_cmd->iv = 0;
		sca_gcm_cmd->out = temp_paddr;
		sca_gcm_cmd->dma_mode = 0;
		do_sca_process(index, num, sca_gcm_cmd);

		flush_dcache_page(phys_to_page(temp_paddr));
		dsb(sy);
		bit128_xor(t, temp, t);
		dsb(sy);
		if (cmd->dma_mode & 0x1) {
			get_set_tag((struct sce_link_list_item *)(phys_to_virt(
					    cmd->out)),
				    cmd->in_size, t, cmd->tag_len, true);
		} else {
			memcpy(out_vaddr + cmd->in_size, t, 16);
		}

	} else {
		//Y(i) = inc(Y(i-1)); D(i) = C(i) ^ enc(Y(i))
		memcpy(y, y0, 16);
		for (i = 16; i > 0; i--)
			if (++y[i - 1])
				break;

		memset(src_link_list, 0, sizeof(*src_link_list));
		flush_dcache_page(virt_to_page(src_link_list));
		dsb(sy);
		k = 0;
		if (cmd->dma_mode & 0x2) { //src
			k = make_link_list(
				src_link_list, k,
				(struct sce_link_list_item *)(phys_to_virt(cmd->in)),
				cmd->aad_len, cmd->in_size + cmd->aad_len);
			src_link_list[k].address = 0;
			src_link_list[k].size = SCE_LINK_END_SIZE;
			src_link_list[k].reserved = SCE_LINK_END_SIZE;

			src_addr = virt_to_phys(src_link_list);
			flush_dcache_page(virt_to_page(src_link_list));
			dsb(sy);
		} else {
			src_addr = in_paddr;
		}

		k = 0;

		dst_items = (struct sce_link_list_item *)(phys_to_virt(cmd->out));
		if (cmd->in_size) {
			if (cmd->dma_mode & 0x1) {
				k = make_link_list(dst_link_list, 0, dst_items,
						   0, cmd->in_size);
				dst_link_list[k].address = 0;
				dst_link_list[k].size = SCE_LINK_END_SIZE;
				dst_link_list[k].reserved = SCE_LINK_END_SIZE;
				dst_addr = virt_to_phys(dst_link_list);
				flush_dcache_page(virt_to_page(dst_link_list));
				dsb(sy);
			} else {
				dst_addr = cmd->out;
			}

			sca_gcm_cmd->algo = sca_cmd->algo;
			sca_gcm_cmd->mode = CTR;
			sca_gcm_cmd->op_mode = 0; //ENC: 0
			sca_gcm_cmd->in = src_addr; //输入为plain
			sca_gcm_cmd->in_size = cmd->in_size; //datalen
			sca_gcm_cmd->key = cmd->key;
			sca_gcm_cmd->iv = y_paddr;
			sca_gcm_cmd->out = dst_addr;
			sca_gcm_cmd->dma_mode = cmd->dma_mode;
			flush_dcache_page(virt_to_page(sca_gcm_cmd));
			dsb(sy);
			do_sca_process(index, num, sca_gcm_cmd);
		}

		k = 0;

		padding_sz = (16 - (cmd->in_size & 0xf)) & 0xf;

		_sce_gcm_read_size(pad, padding_sz + 7, cmd->aad_len * 8);
		_sce_gcm_read_size(pad, padding_sz + 15, cmd->in_size * 8);

		flush_dcache_page(virt_to_page(pad));
		dsb(sy);
		memset(mac_src_link_list, 0, sizeof(*mac_src_link_list));
		flush_dcache_page(virt_to_page(mac_src_link_list));
		dsb(sy);
		src_items = (struct sce_link_list_item *)mac_src_link_list;
		k = 0;
		if (cmd->dma_mode & 0x2) {
			if (cmd->aad_len) {
				k = make_link_list(
					src_items, k,
					(struct sce_link_list_item *)(aad_vaddr), 0,
					cmd->aad_len);
				dsb(sy);
				if (cmd->aad_len & 0xf) {
					src_items[k].address =
						virt_to_phys(ghashzero);
					src_items[k].address |= 0x300000000000;
					src_items[k].size =
						16 - (cmd->aad_len & 0xf) - 1;
					src_items[k].reserved = 0;
					k++;
				}
			}
			if (cmd->in_size) {
				k = make_link_list(
					src_items, k,
					(struct sce_link_list_item *)(aad_vaddr),
					cmd->aad_len,
					cmd->in_size + cmd->aad_len);
				flush_dcache_page(virt_to_page(src_items));
				dsb(sy);
			}
		} else {
			if (cmd->aad_len) {
				src_items[k].address = cmd->in;
				src_items[k].address |= 0x300000000000;
				src_items[k].size = cmd->aad_len - 1;
				src_items[k].reserved = 0;
				k++;
				if (cmd->aad_len & 0xf) {
					src_items[k].address =
						virt_to_phys(ghashzero);
					src_items[k].address |= 0x300000000000;
					src_items[k].size =
						16 - (cmd->aad_len & 0xf) - 1;
					src_items[k].reserved = 0;
					k++;
				}
				flush_dcache_page(virt_to_page(src_items));
				dsb(sy);
			}
			if (cmd->in_size) {
				src_items[k].address = cmd->in + cmd->aad_len;
				src_items[k].address |= 0x300000000000;
				src_items[k].size = cmd->in_size - 1;
				src_items[k].reserved = 0;
				k++;
			}
		}
		src_items[k].address = virt_to_phys(pad);
		src_items[k].address |= 0x300000000000;
		src_items[k].size = 16 + padding_sz - 1;
		src_items[k].reserved = 0;
		k++;
		dsb(sy);

		src_items[k].address = 0;
		src_items[k].size = SCE_LINK_END_SIZE;
		src_items[k].reserved = SCE_LINK_END_SIZE;

		flush_dcache_page(virt_to_page(mac_src_link_list));
		dsb(sy);

		sca_gcm_cmd->algo = GHASH;
		sca_gcm_cmd->mode = CMAC;
		sca_gcm_cmd->op_mode = 0;
		sca_gcm_cmd->in = virt_to_phys(mac_src_link_list);
		sca_gcm_cmd->in_size = 16;
		sca_gcm_cmd->key = h_paddr;
		sca_gcm_cmd->iv = 0;
		sca_gcm_cmd->out = t_paddr;
		sca_gcm_cmd->dma_mode = 2;
		flush_dcache_page(virt_to_page(sca_gcm_cmd));
		dsb(sy);

		do_sca_process(index, num, sca_gcm_cmd);

		flush_dcache_page(phys_to_page(t_paddr));
		dsb(sy);

		sca_gcm_cmd->algo = sca_cmd->algo;
		sca_gcm_cmd->mode = ECB;
		sca_gcm_cmd->op_mode = 0; //ENC: 0
		sca_gcm_cmd->in = virt_to_phys(y0);
		sca_gcm_cmd->in_size = 16;
		sca_gcm_cmd->key = cmd->key;
		sca_gcm_cmd->iv = 0;
		sca_gcm_cmd->out = temp_paddr;
		sca_gcm_cmd->dma_mode = 0;
		do_sca_process(index, num, sca_gcm_cmd);

		flush_dcache_page(phys_to_page(temp_paddr));
		dsb(sy);

		bit128_xor(t, temp, t);
		if (cmd->dma_mode & 0x2) {
			get_set_tag(
				(struct sce_link_list_item *)(phys_to_virt(cmd->in)),
				cmd->aad_len + cmd->in_size, t + 16,
				cmd->tag_len, false);
			flush_dcache_page(phys_to_page(t_paddr));
			dsb(sy);
			if (memcmp(t, t + 16, cmd->tag_len)) {
				pr_err("verrfy tag failed!\n");
				goto out;
			}
		} else {
			if (memcmp(t,
				   phys_to_virt(cmd->in) + cmd->aad_len +
					   cmd->in_size,
				   cmd->tag_len)) {
				pr_err("verrfy tag failed!\n");
				goto out;
			}
		}
	}

out:
	cmd_rdptr = (sca_cmd_index[sca_slot_id] + 1) & (QUEUE_DEPTH - 1);
	dsb(sy);
	REG_WR_32(sca_cmd_base[sca_slot_id], RSP_WRPTR_OFFSET, cmd_rdptr);
	REG_WR_32(sca_cmd_base[sca_slot_id], CMD_RDPTR_OFFSET, cmd_rdptr);

	sca_slot_id++;
	if (sca_slot_id >= 8)
		sca_slot_id = 0;

	num--;
	sce_free(mac_src_link_list);
err10:
	sce_free(ghashzero);
err9:
	sce_free(dst_link_list);
err8:
	sce_free(src_link_list);
err7:
	sce_free(sca_gcm_cmd);
err6:
	sce_free(t);
err5:
	sce_free(temp);
err4:
	sce_free(y0);
err3:
	sce_free(y);
err2:
	sce_free(h);
err1:
	is_ccm = 0;
	return ret;
}

//hash
static inline void hash_queue_recv(void)
{
	int i, rsp_space;
	union hash_stat_reg stat;
	union hash_csq_reg csq[4];
	int slot;
	u32 state;
	struct rsp *rsp;

	stat.data = REG_RD_32_SYNC(sce_driver.base, HASH_STAT_OFFSET);
	rsp_space = stat.csq_ocpd_slots;
	sce_driver.hash_rsp_space += rsp_space;
	sce_driver.hash_cmd_space = stat.cq_avail_slots;

	if (unlikely(!rsp_space))
		return;

	for (i = 0; i < rsp_space; i++)
		csq[i].data = REG_RD_32(sce_driver.base, HASH_CSQ_OFFSET);


	for (i = 0; i < rsp_space; i++) {
		slot = csq[i].done_slot_id;
		state = csq[i].stat_idx;
		hash_cmd_complete_num[slot]++;
		if (unlikely((hash_cmd_state[slot] == 0) && state))
			hash_cmd_state[slot] = state;
		if (unlikely(hash_cmd_complete_num[slot] >= 3)) {
			if (likely(!hash_cmd_callback[slot] ||
				   !hash_cmd_callback_data[slot])) {
				state = hash_cmd_state[slot] | (1 << 16);
				rsp = hash_cmd_rsp_buf[slot] +
				      hash_cmd_index[slot];
				rsp->state = state;
				rsp->tag = hash_cmd_tag[slot];
				REG_WR_32_SYNC(hash_cmd_base[slot],
					       RSP_WRPTR_OFFSET,
					       (hash_cmd_index[slot] + 1) &
						       (QUEUE_DEPTH - 1));
			}
		}
	}
}

static inline void hash_proccess(int index, int num)
{
	u32 cmd_rdptr;
	struct cmd *cmd_buf;
	struct rsp *rsp_buf;

	while (num > 0) {
HASH_FLUSH:
		if (hash_cmd_step)
			hash_queue_recv();
		switch (hash_cmd_step) {
		case 0:
			break;
		case 1:
			goto STEP1;
		case 2:
			goto STEP2;
		case 3:
			goto STEP3;
		case 4:
			goto STEP4;
		case 5:
			goto STEP5;
		case 6:
			goto STEP6;
		case 7:
			goto STEP7;
		case 8:
			goto STEP8;
		case 9:
			goto STEP9;
		case 10:
			goto STEP10;
		case 11:
			goto STEP11;
		case 12:
			goto STEP12;
		case 13:
			goto STEP13;
		case 14:
			goto STEP14;
		case 15:
			goto STEP15;
		case 16:
			goto STEP16;
		default:
			break;
		}

		cmd_buf = cmd_base[HASH][index];
		rsp_buf = rsp_base[HASH][index];
		cmd_rdptr = REG_RD_32(reg_base[HASH][index], CMD_RDPTR_OFFSET);
		hash_cmd = (struct hash_cmd *)(&cmd_buf[cmd_rdptr]);
		hash_cmd_tag[hash_slot_id] = hash_cmd->tag;
		hash_cmd_index[hash_slot_id] = cmd_rdptr;
		hash_cmd_base[hash_slot_id] = reg_base[HASH][index];
		hash_cmd_rsp_buf[hash_slot_id] = rsp_buf;
		hash_cmd_state[hash_slot_id] = 0;
		hash_cmd_complete_num[hash_slot_id] = 0;
		hash_cmd_callback[hash_slot_id] = NULL;

		hash_init.mode = hash_cmd->mode;
		hash_init.algo = hash_cmd->algo;
		hash_init.slot_id = hash_slot_id;
		hash_cmd->in |= 0x300000000000;
		hash_cmd->out |= 0x300000000000;
		hash_cmd->key |= 0x300000000000;
		hash_cmd->tag |= 0x300000000000;
		if (unlikely(hash_cmd->dma_mode))
			hash_proc.src_addr_type = 1;
		else
			hash_proc.src_addr_type = 0;

		if (unlikely(hash_cmd->in_size == 0))
			hash_proc.is_zero_proc = 1;
		else
			hash_proc.is_zero_proc = 0;

		hash_proc.slot_id = hash_slot_id;

		hash_finish.mac_tlen = hash_mac_len[hash_cmd->algo];
		hash_finish.slot_id = hash_slot_id;

STEP1:
		if (sce_driver.hash_rsp_space && sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_FUNC_OFFSET,
				  hash_init.data);
			sce_driver.hash_rsp_space--;
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 1;
			goto HASH_FLUSH;
		}

STEP2:
		if (sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
				  (iv_paddr + hash_cmd->algo * 128u) &
					  0xFFFFFFFF);
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 2;
			goto HASH_FLUSH;
		}

STEP3:
		if (sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
				  (iv_paddr + hash_cmd->algo * 128u) >> 32);
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 3;
			goto HASH_FLUSH;
		}

STEP4:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->key & 0xFFFFFFFF);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 4;
				goto HASH_FLUSH;
			}
		}

STEP5:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->key >> 32);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 5;
				goto HASH_FLUSH;
			}
		}

STEP6:
		if (sce_driver.hash_rsp_space && sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_FUNC_OFFSET,
				  hash_proc.data);
			sce_driver.hash_rsp_space--;
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 6;
			goto HASH_FLUSH;
		}

STEP7:
		if (likely(hash_cmd->in_size)) {
			if (likely(hash_cmd->dma_mode == 0)) {
				if (sce_driver.hash_cmd_space) {
					REG_WR_32(sce_driver.base,
						  HASH_CQ_PARA_OFFSET,
						  hash_cmd->in_size - 1);
					sce_driver.hash_cmd_space--;
				} else {
					hash_cmd_step = 7;
					goto HASH_FLUSH;
				}
			}
		}

STEP8:
		if (likely(hash_cmd->in_size)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->in & 0xFFFFFFFF);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 8;
				goto HASH_FLUSH;
			}
		}

STEP9:
		if (likely(hash_cmd->in_size)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->in >> 32);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 9;
				goto HASH_FLUSH;
			}
		}

STEP10:
		if (sce_driver.hash_rsp_space && sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_FUNC_OFFSET,
				  hash_finish.data);
			sce_driver.hash_rsp_space--;
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 10;
			goto HASH_FLUSH;
		}

STEP11:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  (iv_paddr + hash_cmd->algo * 128u) &
						  0xFFFFFFFF);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 11;
				goto HASH_FLUSH;
			}
		}

STEP12:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  (iv_paddr + hash_cmd->algo * 128u) >>
						  32);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 12;
				goto HASH_FLUSH;
			}
		}

STEP13:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->key & 0xFFFFFFFF);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 13;
				goto HASH_FLUSH;
			}
		}

STEP14:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->key >> 32);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 14;
				goto HASH_FLUSH;
			}
		}

STEP15:
		if (sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
				  hash_cmd->out & 0xFFFFFFFF);
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 15;
			goto HASH_FLUSH;
		}

STEP16:
		if (sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
				  hash_cmd->out >> 32);
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 16;
			goto HASH_FLUSH;
		}

		cmd_rdptr = (hash_cmd_index[hash_slot_id] + 1) &
			    (QUEUE_DEPTH - 1);
		REG_WR_32(hash_cmd_base[hash_slot_id], CMD_RDPTR_OFFSET,
			  cmd_rdptr);

		hash_cmd_step = 0;
		hash_slot_id++;
		if (hash_slot_id >= 16)
			hash_slot_id = 8;

		num--;
	}
}

static int sm4_trng_read(void *buf, size_t len)
{
	size_t i = 0, step;
	u32 val;

	for (i = 0; i < len; i += step) {
		step = (len - i) > 4 ? 4 : (len - i);
		val = REG_RD_32_SYNC(sce_driver.base, TRNG_DR_OFFSET);
		memcpy(buf + i, &val, step);
	}

	return i;
}

static int trng_read(void *buf, size_t len)
{
	size_t i = 0, step;
	u32 val;

	for (i = 0; i < len; i += step) {
		step = (len - i) > 4 ? 4 : (len - i);
		val = REG_RD_32_SYNC(sce_driver.base,
				     RN_POOL_DATA_WORD_OFFSET(i >> 2));
		memcpy(buf + i, &val, step);
	}

	return i;
}

static int sm4_trng_process_cmd(struct trng_cmd *trng_cmd)
{
	size_t offset = 0;
	size_t step_len = 0;
	union trng_sr_reg initr = { 0 };
	int count;
	void *v_data_buf;

	if (!trng_cmd->data) {
		sce_err("invalid data buf");
		return -1;
	}

	v_data_buf = phys_to_virt(trng_cmd->data);

	while (offset < trng_cmd->len) {
		step_len = 8;
		initr.data = 0x7;
		count = 0;
		do {
			initr.data = REG_RD_32_SYNC(sce_driver.base,
						    TRNG_SR_OFFSET); //5
			smp_rmb(); //memory barrier
			if (initr.drdy) {
				sm4_trng_read(v_data_buf + offset, step_len);
				offset += step_len;
				REG_WR_32_SYNC(sce_driver.base, TRNG_SR_OFFSET,
					       0x7);

				break;
			}
			udelay(DRIVER_WAIT_POOL_FILLED_US);
			count++;
			if (count > DRIVER_WAIT_POOL_CYCLE_INDEX) {
				sce_err("Wait timeout for trng");
				return -1;
			}
		} while (1);
	}

	return TRNG_SUCCESS;
}

static int trng_process_cmd(struct trng_cmd *trng_cmd)
{
	size_t offset = 0;
	size_t step_len = 0;
	union rnpool_ctrl_reg ctrl = { 0 };
	union rnpool_intr_reg intr = { 0 };
	int count;
	void *v_data_buf;

	if (!trng_cmd->data) {
		sce_err("invalid data buf");
		return -1;
	}

	v_data_buf = phys_to_virt(trng_cmd->data);

	while (offset < trng_cmd->len) {
		step_len = (trng_cmd->len - offset) > TRNG_POOL_SIZE ?
				   TRNG_POOL_SIZE :
				   trng_cmd->len - offset;

		// set_ctl
		ctrl.trng_clock_enable = 1;
		ctrl.rn_pool_fill_req = 1;
		REG_WR_32_SYNC(sce_driver.base, RN_POOL_CTL_OFFSET, ctrl.data);
		count = 0;
		do {
			// get int state
			intr.data = REG_RD_32_SYNC(sce_driver.base,
						   RN_POOL_INTR_OFFSET);
			if (intr.intr_fill_done) {
				trng_read(v_data_buf + offset, step_len);
				offset += step_len;

				// eoi
				REG_WR_32_SYNC(sce_driver.base,
					       RN_POOL_INTR_OFFSET, intr.data);
				break;
			}
			udelay(DRIVER_WAIT_POOL_FILLED_US);
			count++;
			if (count > DRIVER_WAIT_POOL_CYCLE_INDEX) {
				sce_err("Wait timeout for trng");
				return -1;
			}
		} while (1);
	}

	return TRNG_SUCCESS;
}

static inline void trng_proccess(int index, int num)
{
	u32 cmd_rdptr;
	struct cmd *cmd_buf;
	struct rsp *rsp_buf;

	while (num > 0) {
		cmd_buf = cmd_base[TRNG][index];
		rsp_buf = rsp_base[TRNG][index];
		cmd_rdptr = REG_RD_32(reg_base[TRNG][index], CMD_RDPTR_OFFSET);

		trng_cmd = (struct trng_cmd *)(&cmd_buf[cmd_rdptr]);

		// read random data here
		if (trng_cmd->mode == TRNG_RNP) {
			rsp_buf[cmd_rdptr].state = trng_process_cmd(trng_cmd);
		} else {
			rsp_buf[cmd_rdptr].state =
				sm4_trng_process_cmd(trng_cmd);
		};
		rsp_buf[cmd_rdptr].tag = trng_cmd->tag;

		cmd_rdptr = (cmd_rdptr + 1) & (QUEUE_DEPTH - 1);
		REG_WR_32(reg_base[TRNG][index], RSP_WRPTR_OFFSET, cmd_rdptr);

		REG_WR_32(reg_base[TRNG][index], CMD_RDPTR_OFFSET, cmd_rdptr);
		num--;
	}
}

static inline void sca_queue_recv(void)
{
	int rsp_space;
	union sca_stat_reg stat;
	union sca_csq_reg csq;
	int slot;
	u32 state;
	struct rsp *rsp;
	int tmp = 0;

	stat.data = REG_RD_32_SYNC(sce_driver.base, SCA_STAT_OFFSET);

	rsp_space = stat.csq_ocpd_slots;
	sce_driver.sca_rsp_space += rsp_space;
	sce_driver.sca_cmd_space = stat.cq_avail_slots;

	if (unlikely(!rsp_space))
		return;

	if (is_last) {
		do {
			csq.data =
				REG_RD_32_SYNC(sce_driver.base, SCA_CSQ_OFFSET);
			if (tmp != csq.opcode_idx)
				tmp = csq.opcode_idx;

		} while (csq.opcode_idx != FINISH);
		slot = csq.done_slot_id;
		sca_slot_id = slot;
		state = csq.stat_idx;
		sca_cmd_complete_num[slot] += 3;
		state = sca_cmd_state[slot] | (1 << 16);
		rsp = sca_cmd_rsp_buf[slot] + sca_cmd_index[slot];
		rsp->state = state;
		rsp->tag = sca_cmd_tag[slot];
		is_last = 0;
	}
}

static int do_sca_process(int index, int num, struct sca_cmd *sca_prv_cmd)
{
	int ret = 0;
	struct rsp *rsp_buf;
	u64 in_addr, out_addr, key_addr, iv_addr;

	in_addr = sca_prv_cmd->in | 0x300000000000;
	out_addr = sca_prv_cmd->out | 0x300000000000;
	key_addr = sca_prv_cmd->key | 0x300000000000;
	iv_addr = sca_prv_cmd->iv | 0x300000000000;

	smp_mb(); //memory barrier
SCA_FLUSH:
	if (sca_cmd_step)
		sca_queue_recv();
	switch (sca_cmd_step) {
	case 0:
		break;
	case 1:
		goto STEP1;
	case 2:
		goto STEP2;
	case 3:
		goto STEP3;
	case 4:
		goto STEP4;
	case 5:
		goto STEP5;
	case 6:
		goto STEP6;
	case 7:
		goto STEP7;
	case 8:
		goto STEP8;
	case 9:
		goto STEP9;
	case 10:
		goto STEP10;
	case 11:
		goto STEP11;
	case 12:
		goto STEP12;
	case 13:
		goto STEP13;
	case 14:
		goto STEP14;
	default:
		break;
	}
	rsp_buf = rsp_base[SCA][index];
	sca_cmd_tag[sca_slot_id] = sca_prv_cmd->tag;

	sca_cmd_base[sca_slot_id] = reg_base[SCA][index];
	sca_cmd_rsp_buf[sca_slot_id] = rsp_buf;
	sca_cmd_state[sca_slot_id] = 0;
	sca_cmd_complete_num[sca_slot_id] = 0;
	sca_cmd_callback[sca_slot_id] = NULL;

	sca_init.mode = sca_prv_cmd->mode;
	sca_init.key_len = sca_prv_cmd->algo >> 4;
	sca_init.algo = sca_prv_cmd->algo & 0xF;
	sca_init.slot_id = sca_slot_id;
	if (sca_init.mode == CBC_MAC || sca_init.mode == CMAC)
		sca_init.iv_load = 0;
	else
		sca_init.iv_load = 1;

	sca_proc.dst_src_addr_type = sca_prv_cmd->dma_mode;
	sca_proc.op_mode = sca_prv_cmd->op_mode;
	sca_proc.slot_id = sca_slot_id;

	dsb(sy);
	sca_finish.slot_id = sca_slot_id;
	if (sca_prv_cmd->mode == CBC_MAC || sca_prv_cmd->mode == CMAC) {
		if ((sca_prv_cmd->algo != 0x1 && sca_prv_cmd->algo != 0x2))
			sca_finish.mac_tlen = 0xf;
		else
			sca_finish.mac_tlen = 0x7;
	} else
		sca_finish.mac_tlen = 0;

STEP1:
	if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
		REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET, sca_init.data);
		sce_driver.sca_rsp_space--;
		sce_driver.sca_cmd_space--;
	} else {
		sca_cmd_step = 1;
		goto SCA_FLUSH;
	}
STEP2:
	if (likely(sca_prv_cmd->mode != ECB && sca_init.iv_load != 0)) {
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  iv_addr & 0xFFFFFFFF);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 2;
			goto SCA_FLUSH;
		}
	}

STEP3:
	if (likely(sca_prv_cmd->mode != ECB && sca_init.iv_load != 0)) {
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  iv_addr >> 32);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 3;
			goto SCA_FLUSH;
		}
	}
STEP4:
	if (sce_driver.sca_cmd_space) {
		REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
			  key_addr & 0xFFFFFFFF);
		sce_driver.sca_cmd_space--;
	} else {
		sca_cmd_step = 4;
		goto SCA_FLUSH;
	}

STEP5:
	if (sce_driver.sca_cmd_space) {
		REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET, key_addr >> 32);
		sce_driver.sca_cmd_space--;
	} else {
		sca_cmd_step = 5;
		goto SCA_FLUSH;
	}

STEP6:
	if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
		REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET, sca_proc.data);
		sce_driver.sca_rsp_space--;
		sce_driver.sca_cmd_space--;
	} else {
		sca_cmd_step = 6;
		goto SCA_FLUSH;
	}

STEP7:
	if (likely((sca_prv_cmd->dma_mode & 0x2) == 0)) {
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_prv_cmd->in_size - 1);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 7;
			goto SCA_FLUSH;
		}
	}

STEP8:
	if (sce_driver.sca_cmd_space) {
		REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
			  in_addr & 0xFFFFFFFF);
		sce_driver.sca_cmd_space--;
	} else {
		sca_cmd_step = 8;
		goto SCA_FLUSH;
	}

STEP9:
	if (sce_driver.sca_cmd_space) {
		REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET, in_addr >> 32);
		sce_driver.sca_cmd_space--;
	} else {
		sca_cmd_step = 9;
		goto SCA_FLUSH;
	}

STEP10:
	if (likely(sca_prv_cmd->mode != CBC_MAC && sca_prv_cmd->mode != CMAC)) {
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  out_addr & 0xFFFFFFFF);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 10;
			goto SCA_FLUSH;
		}
	}

STEP11:
	if (likely(sca_prv_cmd->mode != CBC_MAC && sca_prv_cmd->mode != CMAC)) {
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  out_addr >> 32);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 11;
			goto SCA_FLUSH;
		}
	}

STEP12:
	if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
		REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET, sca_finish.data);
		sce_driver.sca_rsp_space--;
		sce_driver.sca_cmd_space--;
	} else {
		sca_cmd_step = 12;
		goto SCA_FLUSH;
	}
STEP13:
	if (sca_prv_cmd->mode == CBC_MAC || sca_prv_cmd->mode == CMAC) {
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  out_addr & 0xFFFFFFFF);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 13;
			goto SCA_FLUSH;
		}
	}

STEP14:
	if (sca_prv_cmd->mode == CBC_MAC || sca_prv_cmd->mode == CMAC) {
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  out_addr >> 32);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 14;
			goto SCA_FLUSH;
		}
	}
	is_last = 1;
	sca_queue_recv();
	sca_cmd_step = 0;

	return ret;
}

static int sca_cbcmac_proccess(int index, int num, struct sca_cmd *cmd)
{
	u32 cmd_rdptr;
	u64 src_addr, dst_addr;
	int ret = 0;
	int tmp = 0;
	struct sca_cmd *sca_cbcmac_cmd;
	u8 *src;

	sca_cbcmac_cmd = sce_malloc(sizeof(struct sca_cmd));
	if (!sca_cbcmac_cmd) {
		sce_err("%s:%d malloc sca_cbcmac_cmd failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err1;
	}

	src = sce_malloc(cmd->in_size + 64);
	if (!src) {
		sce_err("%s:%d malloc src failed!\n", __func__, __LINE__);
		ret = -1;
		goto err2;
	}
	dmb(nsh);
	memset(src, 0x00, cmd->in_size + 64);

	memcpy(src, phys_to_virt(cmd->in), cmd->in_size);

	memset(sca_cbcmac_cmd, 0, sizeof(struct sca_cmd));

	memcpy(sca_cbcmac_cmd, cmd, sizeof(struct sca_cmd));
	src_addr = virt_to_phys(src);
	sca_cbcmac_cmd->in = src_addr;
	if (cmd->algo != 0x1 && cmd->algo != 0x2)
		sca_cbcmac_cmd->in_size = 16 * (cmd->in_size / 16 + 1);
	else
		sca_cbcmac_cmd->in_size = 8 * (cmd->in_size / 8 + 1);

	dmb(sy);
	tmp = is_ccm;
	is_ccm = 1;
	dst_addr = cmd->out;
	do_sca_process(index, num, sca_cbcmac_cmd);

	flush_dcache_page(phys_to_page(dst_addr));
	is_ccm = tmp;
	dsb(sy);
	if (!is_ccm) {
		cmd_rdptr = (sca_cmd_index[sca_slot_id] + 1) &
			    (QUEUE_DEPTH - 1);
		REG_WR_32(sca_cmd_base[sca_slot_id], RSP_WRPTR_OFFSET,
			  cmd_rdptr);
		REG_WR_32(sca_cmd_base[sca_slot_id], CMD_RDPTR_OFFSET,
			  cmd_rdptr);
		sca_slot_id++;
		if (sca_slot_id >= 8)
			sca_slot_id = 0;
		num--;
	}
	sce_free(src);
err2:
	sce_free(sca_cbcmac_cmd);
err1:
	return ret;
}

static int sca_ccm_process(int index, int num, struct sca_cmd *cmd)
{
	u32 cmd_rdptr, cbcmac_len = 0;
	int ret = 0, k = 0;
	struct sca_cmd *sca_ccm_cmd;
	u8 *temp = NULL, *zero = NULL, *ccm_mac_out = NULL, *ccm_mac_src = NULL,
	   *ctr_out = NULL;
	u64 temp_paddr, in_paddr, zero_paddr, src_addr = 0, dst_addr = 0;
	u8 *aad_vaddr = NULL, *in_vaddr = NULL, *out_vaddr = NULL,
	   *iv_vaddr = NULL;
	u8 *civ = NULL;
	struct sce_link_list_item *src_items, *src_link_list = NULL,
					 *dst_link_list = NULL,
					 *mac_src_link_list = NULL;

	is_ccm = 1;
	temp = sce_malloc(sizeof(u8) * 16);
	if (!temp) {
		sce_err("%s:%d malloc temp failed!\n", __func__, __LINE__);
		ret = -1;
		goto err1;
	}
	memset(temp, 0, 16);

	civ = sce_malloc(sizeof(u8) * 16);
	if (!civ) {
		sce_err("%s:%d malloc temp failed!\n", __func__, __LINE__);
		ret = -1;
		goto err1;
	}
	memset(civ, 0, 16);

	zero = sce_malloc(sizeof(u8) * 16 * 2);
	if (!zero) {
		sce_err("%s:%d malloc zero failed!\n", __func__, __LINE__);
		ret = -1;
		goto err2;
	}
	memset(zero, 0, 16 * 2);

	ccm_mac_src = sce_malloc(sizeof(u8) * 0x10000);
	if (!ccm_mac_src) {
		sce_err("%s:%d malloc ccm_mac_out failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err3;
	}
	memset(ccm_mac_src, 0, 0x10000);

	ctr_out = sce_malloc(sizeof(u8) * 32);
	if (!ctr_out) {
		sce_err("%s:%d malloc ccm_mac_out failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err3;
	}
	memset(ctr_out, 0, 32);

	ccm_mac_out = sce_malloc(sizeof(u8) * 16 * 2);
	if (!ccm_mac_out) {
		sce_err("%s:%d malloc ccm_mac_out failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err4;
	}
	memset(ccm_mac_out, 0, 16 * 2);

	sca_ccm_cmd = sce_malloc(sizeof(struct sca_cmd));
	if (!sca_ccm_cmd) {
		sce_err("%s:%d malloc sca_xts_cmd failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err5;
	}

	src_link_list =
		sce_malloc(sizeof(struct sce_link_list_item) * 40 + CACHE_LINE_SIZE);
	if (!src_link_list) {
		sce_err("%s:%d malloc src_link_list failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err6;
	}
	memset(src_link_list, 0, sizeof(struct sce_link_list_item) * 40);

	dst_link_list =
		sce_malloc(sizeof(struct sce_link_list_item) * 40 + CACHE_LINE_SIZE);
	if (!dst_link_list) {
		sce_err("%s:%d malloc dst_link_list failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err7;
	}
	memset(dst_link_list, 0, sizeof(struct sce_link_list_item) * 40);

	mac_src_link_list =
		sce_malloc(sizeof(struct sce_link_list_item) * 40 + CACHE_LINE_SIZE);
	if (!mac_src_link_list) {
		sce_err("%s:%d malloc mac_src_link_list failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err8;
	}
	memset(mac_src_link_list, 0, sizeof(struct sce_link_list_item) * 40);

	memset(sca_ccm_cmd, 0, sizeof(struct sca_cmd));
	memcpy(sca_ccm_cmd, cmd, sizeof(struct sca_cmd));

	aad_vaddr = phys_to_virt(cmd->in);
	in_vaddr = aad_vaddr + sca_cmd->aad_len;
	in_paddr = sca_cmd->in + sca_cmd->aad_len;
	out_vaddr = phys_to_virt(sca_cmd->out);
	iv_vaddr = phys_to_virt(sca_cmd->iv);
	temp_paddr = virt_to_phys(temp);
	zero_paddr = virt_to_phys(zero);
	cbcmac_len = cmd->aad_len + ((16 - (cmd->aad_len & 0xF)) & 0xF) +
		     sca_cmd->in_size;

	if (!cmd->op_mode) {
		memset(mac_src_link_list, 0, sizeof(*mac_src_link_list));
		flush_dcache_page(virt_to_page(mac_src_link_list));
		dsb(sy);
		src_items = (struct sce_link_list_item *)mac_src_link_list;
		k = 0;
		if (cmd->dma_mode & 0x2) {
			k = make_link_list(src_items, k,
					   (struct sce_link_list_item *)(aad_vaddr),
					   0, cmd->aad_len);
		} else {
			src_items[k].address = cmd->in;
			src_items[k].address |= 0x300000000000;
			src_items[k].size = cmd->aad_len - 1;
			src_items[k].reserved = 0;
			k++;
		}

		if (cmd->aad_len & 0xf) {
			src_items[k].address = zero_paddr;
			src_items[k].address |= 0x300000000000;
			src_items[k].size = 16 - (cmd->aad_len & 0xf) - 1;
			src_items[k].reserved = 0;
			k++;
		}

		if (cmd->in_size) {
			if (cmd->dma_mode & 0x2) {
				k = make_link_list(
					src_items, k,
					(struct sce_link_list_item *)(aad_vaddr),
					cmd->aad_len,
					cmd->in_size + cmd->aad_len);
			} else {
				src_items[k].address = in_paddr;
				src_items[k].address |= 0x300000000000;
				src_items[k].size = cmd->in_size - 1;
				src_items[k].reserved = 0;
				k++;
			}

			if (cmd->in_size & 0xf) {
				src_items[k].address = zero_paddr + 16;
				src_items[k].address |= 0x300000000000;
				src_items[k].size =
					16 - (cmd->in_size & 0xf) - 1;
				src_items[k].reserved = 0;
				k++;
			}
		}

		src_items[k].address = 0;
		src_items[k].size = SCE_LINK_END_SIZE;
		src_items[k].reserved = SCE_LINK_END_SIZE;

		flush_dcache_page(virt_to_page(mac_src_link_list));
		dsb(sy);
		sca_ccm_cmd->algo = sca_cmd->algo;
		sca_ccm_cmd->mode = CBC_MAC;
		sca_ccm_cmd->in = virt_to_phys(mac_src_link_list);
		sca_ccm_cmd->in_size = cmd->aad_len +
				       ((16 - (cmd->aad_len & 0xF)) & 0xF) +
				       cmd->in_size;
		sca_ccm_cmd->key = cmd->key;
		sca_ccm_cmd->out = virt_to_phys(ccm_mac_out);
		sca_ccm_cmd->dma_mode = 2;
		do_sca_process(index, num, sca_ccm_cmd);
		flush_dcache_page(virt_to_page(ccm_mac_out));
		dsb(sy);

		sca_ccm_cmd->algo = sca_cmd->algo;
		sca_ccm_cmd->mode = ECB;
		sca_ccm_cmd->op_mode = 0; //ENC: 0
		sca_ccm_cmd->in = cmd->iv; //输入为plain
		sca_ccm_cmd->in_size = 16; //datalen
		sca_ccm_cmd->key = cmd->key;
		sca_ccm_cmd->iv = 0;
		sca_ccm_cmd->out = temp_paddr;
		sca_ccm_cmd->dma_mode = 0;
		do_sca_process(index, num, sca_ccm_cmd);
		flush_dcache_page(virt_to_page(temp));
		dsb(sy);
		//3. bit128_xor(temp, output, output);
		bit128_xor(temp, ccm_mac_out, ccm_mac_out + 16);
		if (cmd->dma_mode & 0x1) {
			get_set_tag((struct sce_link_list_item *)(phys_to_virt(
					    cmd->out)),
				    cmd->in_size, ccm_mac_out + 16,
				    cmd->tag_len, true);
		} else {
			memcpy(out_vaddr + cmd->in_size, ccm_mac_out + 16,
			       cmd->tag_len);
		}

		flush_dcache_page(virt_to_page(out_vaddr));
		//4.sm4_crypt_ctr(ctx, plain_len, civ, plain, output);
		if (cmd->in_size) {
			memset(src_link_list, 0, sizeof(*src_link_list));
			k = 0;
			if (cmd->dma_mode & 0x2) { //src
				k = make_link_list(
					src_link_list, k,
					(struct sce_link_list_item *)(phys_to_virt(
						cmd->in)),
					cmd->aad_len,
					cmd->in_size + cmd->aad_len);

				src_link_list[k].address = 0;
				src_link_list[k].size = SCE_LINK_END_SIZE;
				src_link_list[k].reserved = SCE_LINK_END_SIZE;

				flush_dcache_page(virt_to_page(src_link_list));
				dsb(sy);
				src_addr = virt_to_phys(src_link_list);
			} else {
				src_addr = in_paddr;
			}

			k = 0;

			if (cmd->dma_mode & 0x1) {
				k = make_link_list(
					dst_link_list, k,
					(struct sce_link_list_item *)(phys_to_virt(
						cmd->out)),
					0, cmd->in_size);
				dst_link_list[k].address = 0;
				dst_link_list[k].size = SCE_LINK_END_SIZE;
				dst_link_list[k].reserved = SCE_LINK_END_SIZE;
				dst_addr = virt_to_phys(dst_link_list);
				flush_dcache_page(virt_to_page(dst_link_list));
				dsb(sy);
			} else {
				dst_addr = cmd->out;
			}

			memcpy(civ, iv_vaddr, 16);
			civ[15] += 1;
			sca_ccm_cmd->algo = sca_cmd->algo;
			sca_ccm_cmd->mode = CTR;
			sca_ccm_cmd->op_mode = 0; //ENC: 0
			sca_ccm_cmd->in = src_addr; //输入为plain
			sca_ccm_cmd->in_size = cmd->in_size; //datalen
			sca_ccm_cmd->key = cmd->key;
			sca_ccm_cmd->iv = virt_to_phys(civ);
			sca_ccm_cmd->out = dst_addr; //virt_to_phys(ctr_out);
			sca_ccm_cmd->dma_mode = cmd->dma_mode;
			do_sca_process(index, num, sca_ccm_cmd);
		}
	} else {
		//1.sm4_crypt_ctr(ctx, cipher_len, civ, cipher, output);
		if (cmd->in_size) {
			memset(src_link_list, 0, sizeof(*src_link_list));
			flush_dcache_page(virt_to_page(src_link_list));
			dsb(sy);
			k = 0;
			if (cmd->dma_mode & 0x2) { //src
				k = make_link_list(
					src_link_list, k,
					(struct sce_link_list_item *)(phys_to_virt(
						cmd->in)),
					cmd->aad_len,
					cmd->in_size + cmd->aad_len);

				src_link_list[k].address = 0;
				src_link_list[k].size = SCE_LINK_END_SIZE;
				src_link_list[k].reserved = SCE_LINK_END_SIZE;

				src_addr = virt_to_phys(src_link_list);
				flush_dcache_page(virt_to_page(src_link_list));
				dsb(sy);
			} else {
				src_addr = in_paddr;
			}

			k = 0;

			if (cmd->dma_mode & 0x1) {
				k = make_link_list(
					dst_link_list, k,
					(struct sce_link_list_item *)(phys_to_virt(
						cmd->out)),
					0, cmd->in_size);
				dst_link_list[k].address = 0;
				dst_link_list[k].size = SCE_LINK_END_SIZE;
				dst_link_list[k].reserved = SCE_LINK_END_SIZE;
				dst_addr = virt_to_phys(dst_link_list);
				flush_dcache_page(virt_to_page(src_link_list));
				dsb(sy);
			} else {
				dst_addr = cmd->out;
			}
			memcpy(civ, iv_vaddr, 16);
			civ[15] += 1;
			sca_ccm_cmd->algo = sca_cmd->algo;
			sca_ccm_cmd->mode = CTR;
			sca_ccm_cmd->op_mode = 0; //ENC: 0
			sca_ccm_cmd->in = src_addr; //输入为cipher
			sca_ccm_cmd->in_size = cmd->in_size; //datalen`
			sca_ccm_cmd->key = cmd->key;
			sca_ccm_cmd->iv = virt_to_phys(civ);
			sca_ccm_cmd->out = dst_addr;
			sca_ccm_cmd->dma_mode = cmd->dma_mode;
			do_sca_process(index, num, sca_ccm_cmd);
		}
		//2.cbcmac(ctx, key, ccm_mac_src, ccm_mac_out)
		memset(mac_src_link_list, 0, sizeof(*mac_src_link_list));
		flush_dcache_page(virt_to_page(mac_src_link_list));
		dsb(sy);
		src_items = (struct sce_link_list_item *)mac_src_link_list;
		k = 0;
		if (cmd->dma_mode & 0x2) {
			k = make_link_list(src_items, k,
					   (struct sce_link_list_item *)(aad_vaddr),
					   0, cmd->aad_len);
		} else {
			src_items[k].address = cmd->in;
			src_items[k].address |= 0x300000000000;
			src_items[k].size = cmd->aad_len - 1;
			src_items[k].reserved = 0;
			k++;
		}

		if (cmd->aad_len & 0xf) {
			src_items[k].address = zero_paddr;
			src_items[k].address |= 0x300000000000;
			src_items[k].size = 16 - (cmd->aad_len & 0xf) - 1;
			src_items[k].reserved = 0;
			k++;
		}

		if (cmd->in_size) {
			if (cmd->dma_mode & 0x1) {
				k = make_link_list(
					src_items, k,
					(struct sce_link_list_item *)(out_vaddr), 0,
					cmd->in_size);
			} else {
				src_items[k].address = cmd->out;
				src_items[k].address |= 0x300000000000;
				src_items[k].size = cmd->in_size - 1;
				src_items[k].reserved = 0;
				k++;
			}

			if (cmd->in_size & 0xf) {
				src_items[k].address = zero_paddr + 16;
				src_items[k].address |= 0x300000000000;
				src_items[k].size =
					16 - (cmd->in_size & 0xf) - 1;
				src_items[k].reserved = 0;
				k++;
			}
		}

		flush_dcache_page(virt_to_page(mac_src_link_list));
		dsb(sy);

		mac_src_link_list[k].address = 0;
		mac_src_link_list[k].size = SCE_LINK_END_SIZE;
		mac_src_link_list[k].reserved = SCE_LINK_END_SIZE;

		sca_ccm_cmd->algo = sca_cmd->algo;
		sca_ccm_cmd->mode = CBC_MAC;
		sca_ccm_cmd->in = virt_to_phys(mac_src_link_list);
		sca_ccm_cmd->in_size = cbcmac_len;
		sca_ccm_cmd->key = cmd->key;
		sca_ccm_cmd->out = virt_to_phys(ccm_mac_out);
		sca_ccm_cmd->dma_mode = 2;
		do_sca_process(index, num, sca_ccm_cmd);
		flush_dcache_page(virt_to_page(ccm_mac_out));
		dsb(sy);

		//3. sm4_crypt_ecb_enc(ctx, 16, NULL, iv, temp);
		sca_ccm_cmd->algo = sca_cmd->algo;
		sca_ccm_cmd->mode = ECB;
		sca_ccm_cmd->op_mode = 0; //ENC: 0
		sca_ccm_cmd->in = cmd->iv; //输入为plain
		sca_ccm_cmd->in_size = 16; //datalen
		sca_ccm_cmd->key = cmd->key;
		sca_ccm_cmd->iv = 0;
		sca_ccm_cmd->out = temp_paddr;
		sca_ccm_cmd->dma_mode = 0;
		do_sca_process(index, num, sca_ccm_cmd);

		flush_dcache_page(phys_to_page(temp_paddr));
		dsb(sy);

		//4. bit128_xor(temp, output, output);
		bit128_xor(temp, ccm_mac_out, ccm_mac_out);
		if (cmd->dma_mode & 0x2) {
			get_set_tag(
				(struct sce_link_list_item *)(phys_to_virt(cmd->in)),
				cmd->aad_len + cmd->in_size, ccm_mac_out + 16,
				cmd->tag_len, false);
		} else {
			memcpy(ccm_mac_out + 16,
			       aad_vaddr + cmd->in_size + cmd->aad_len,
			       cmd->tag_len);
		}
		dsb(sy);
		//5.memcmp(cipher + cipher_len, tag, tag_len）
		if (memcmp(ccm_mac_out + 16, ccm_mac_out, cmd->tag_len))
			pr_err("verify tag failed!\n");
	}
	cmd_rdptr = (sca_cmd_index[sca_slot_id] + 1) & (QUEUE_DEPTH - 1);
	REG_WR_32(sca_cmd_base[sca_slot_id], RSP_WRPTR_OFFSET, cmd_rdptr);
	REG_WR_32(sca_cmd_base[sca_slot_id], CMD_RDPTR_OFFSET,
		  ((sca_cmd_index[sca_slot_id] + 1) & (QUEUE_DEPTH - 1)));
	smp_wmb(); //memory barrier

	sca_slot_id++;
	if (sca_slot_id >= 8)
		sca_slot_id = 0;

	num--;

	is_ccm = 0;

	sce_free(mac_src_link_list);
err8:
	sce_free(dst_link_list);
err7:
	sce_free(src_link_list);
err6:
	sce_free(sca_ccm_cmd);
err5:
	sce_free(ccm_mac_out);
err4:

	sce_free(ctr_out);
	sce_free(ccm_mac_src);
err3:
	sce_free(zero);
err2:
	sce_free(civ);
	sce_free(temp);
err1:
	return ret;
}

static int sca_xts_process(int index, int num, struct sca_cmd *cmd)
{
	u32 cmd_rdptr;
	u64 etwk_addr;
	u64 ecb_key_ioaddr;
	int ret = 0;
	struct sca_cmd *sca_xts_cmd;
	u8 *xts_tweak;

	sca_xts_cmd = sce_malloc(sizeof(struct sca_cmd));
	if (!sca_xts_cmd) {
		sce_err("%s:%d malloc sca_xts_cmd failed!\n", __func__,
			__LINE__);
		ret = -1;
		goto err1;
	}

	xts_tweak = sce_malloc(CACHE_LINE_SIZE);
	if (!xts_tweak) {
		sce_err("%s:%d malloc xts_wteak failed!\n", __func__, __LINE__);
		ret = -1;
		goto err2;
	}

	memset(sca_xts_cmd, 0, sizeof(struct sca_cmd));
	memcpy(sca_xts_cmd, cmd, sizeof(struct sca_cmd));

	if ((sca_xts_cmd->algo & 0xf) == AES) {
		ecb_key_ioaddr =
			sca_xts_cmd->key + ((sca_cmd->algo >> 4) ? 32 : 16);
	} else {
		ecb_key_ioaddr = sca_xts_cmd->key + 16;
	}
	etwk_addr = virt_to_phys(xts_tweak);
	//iv
	sca_xts_cmd->mode = ECB;
	sca_xts_cmd->op_mode = 0; //ENC: 0
	sca_xts_cmd->in = sca_xts_cmd->iv;
	sca_xts_cmd->in_size = 16;
	sca_xts_cmd->key = ecb_key_ioaddr;
	sca_xts_cmd->out = etwk_addr;
	sca_xts_cmd->dma_mode = 0;
	do_sca_process(index, num, sca_xts_cmd);
	//xts
	sca_xts_cmd->mode = XTS;
	sca_xts_cmd->op_mode = cmd->op_mode;
	sca_xts_cmd->in = cmd->in;
	sca_xts_cmd->in_size = cmd->in_size;
	sca_xts_cmd->key = cmd->key;
	sca_xts_cmd->out = cmd->out;
	sca_xts_cmd->iv = etwk_addr;
	sca_xts_cmd->dma_mode = cmd->dma_mode;
	do_sca_process(index, num, sca_xts_cmd);
	//iv
	cmd_rdptr = (sca_cmd_index[sca_slot_id] + 1) & (QUEUE_DEPTH - 1);
	REG_WR_32(sca_cmd_base[sca_slot_id], RSP_WRPTR_OFFSET, cmd_rdptr);
	REG_WR_32(sca_cmd_base[sca_slot_id], CMD_RDPTR_OFFSET, cmd_rdptr);

	sca_slot_id++;
	if (sca_slot_id >= 8)
		sca_slot_id = 0;

	num--;
	sce_free(xts_tweak);
err2:
	sce_free(sca_xts_cmd);
err1:
	return ret;
}

static inline void sca_proccess(int index, int num)
{
	u32 cmd_rdptr;
	u64 dst_addr = 0;
	struct cmd *cmd_buf;
	struct rsp *rsp_buf;
	int ret;

	while (num > 0) {
SCA_FLUSH:
		if (sca_cmd_step)
			sca_queue_recv();
		switch (sca_cmd_step) {
		case 0:
			break;
		case 1:
			goto STEP1;
		case 2:
			goto STEP2;
		case 3:
			goto STEP3;
		case 4:
			goto STEP4;
		case 5:
			goto STEP5;
		case 6:
			goto STEP6;
		case 7:
			goto STEP7;
		case 8:
			goto STEP8;
		case 9:
			goto STEP9;
		case 10:
			goto STEP10;
		case 11:
			goto STEP11;
		case 12:
			goto STEP12;
		case 13:
			goto STEP13;
		case 14:
			goto STEP14;
		default:
			break;
		}

		cmd_buf = cmd_base[SCA][index];
		rsp_buf = rsp_base[SCA][index];
		cmd_rdptr = REG_RD_32(reg_base[SCA][index], CMD_RDPTR_OFFSET);
		sca_cmd = (struct sca_cmd *)(&cmd_buf[cmd_rdptr]);
		sca_cmd_index[sca_slot_id] = cmd_rdptr;
		if (sca_cmd->mode == XTS) {
			ret = sca_xts_process(index, num, sca_cmd);
			return;
		}
		if (sca_cmd->mode == CBC_MAC && (sca_cmd->in_size % 16 != 0) &&
		    ((sca_cmd->algo != 0x1 && sca_cmd->algo != 0x2))) {
			ret = sca_cbcmac_proccess(index, num, sca_cmd);
			return;
		}
		if (sca_cmd->mode == CBC_MAC && (sca_cmd->in_size % 8 != 0) &&
		    ((sca_cmd->algo == 0X1 || sca_cmd->algo == 0X2))) {
			ret = sca_cbcmac_proccess(index, num, sca_cmd);
			return;
		}

		if (sca_cmd->mode == CCM) {
			ret = sca_ccm_process(index, num, sca_cmd);
			return;
		}
		if (sca_cmd->mode == GCM) {
			ret = sca_gcm_process(index, num, sca_cmd);
			return;
		}
		sca_cmd_tag[sca_slot_id] = sca_cmd->tag;
		sca_cmd_base[sca_slot_id] = reg_base[SCA][index];
		sca_cmd_rsp_buf[sca_slot_id] = rsp_buf;
		sca_cmd_state[sca_slot_id] = 0;
		sca_cmd_complete_num[sca_slot_id] = 0;
		sca_cmd_callback[sca_slot_id] = NULL;

		sca_init.mode = sca_cmd->mode;
		sca_init.key_len = sca_cmd->algo >> 4;
		sca_init.algo = sca_cmd->algo & 0xF;
		sca_init.slot_id = sca_slot_id;
		if (sca_init.mode == CBC_MAC || sca_init.mode == CMAC)
			sca_init.iv_load = 0;
		else
			sca_init.iv_load = 1;

		sca_proc.dst_src_addr_type = sca_cmd->dma_mode;
		sca_proc.op_mode = sca_cmd->op_mode;
		sca_proc.slot_id = sca_slot_id;

		sca_finish.slot_id = sca_slot_id;
		if (sca_cmd->mode == CBC_MAC || sca_cmd->mode == CMAC) {
			if ((sca_cmd->algo != 0x1 && sca_cmd->algo != 0x2))
				sca_finish.mac_tlen = 0xf;
			else
				sca_finish.mac_tlen = 0x7;
		} else
			sca_finish.mac_tlen = 0;


		dst_addr = sca_cmd->out;
		sca_cmd->in |= 0x300000000000;
		sca_cmd->out |= 0x300000000000;
		sca_cmd->key |= 0x300000000000;
		sca_cmd->iv |= 0x300000000000;
STEP1:
		if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET,
				  sca_init.data);
			sce_driver.sca_rsp_space--;
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 1;
			goto SCA_FLUSH;
		}

STEP2:
		if (likely(sca_cmd->mode != ECB) && sca_init.iv_load != 0) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->iv & 0xFFFFFFFF);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 2;
				goto SCA_FLUSH;
			}
		}

STEP3:
		if (likely(sca_cmd->mode != ECB && sca_init.iv_load != 0)) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->iv >> 32);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 3;
				goto SCA_FLUSH;
			}
		}

STEP4:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->key & 0xFFFFFFFF);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 4;
			goto SCA_FLUSH;
		}

STEP5:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->key >> 32);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 5;
			goto SCA_FLUSH;
		}

STEP6:
		if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET,
				  sca_proc.data);
			sce_driver.sca_rsp_space--;
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 6;
			goto SCA_FLUSH;
		}

STEP7:
		if (likely((sca_cmd->dma_mode & 0x2) == 0)) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->in_size - 1);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 7;
				goto SCA_FLUSH;
			}
		}

STEP8:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->in & 0xFFFFFFFF);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 8;
			goto SCA_FLUSH;
		}

STEP9:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->in >> 32);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 9;
			goto SCA_FLUSH;
		}

STEP10:
		if (likely(sca_cmd->mode != CBC_MAC && sca_cmd->mode != CMAC)) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->out & 0xFFFFFFFF);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 10;
				goto SCA_FLUSH;
			}
		}

STEP11:
		if (likely(sca_cmd->mode != CBC_MAC && sca_cmd->mode != CMAC)) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->out >> 32);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 11;
				goto SCA_FLUSH;
			}
		}

STEP12:
		if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET,
				  sca_finish.data);
			sce_driver.sca_rsp_space--;
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 12;
			goto SCA_FLUSH;
		}
STEP13:
		if (sca_cmd->mode == CBC_MAC || sca_cmd->mode == CMAC) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->out & 0xFFFFFFFF);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 13;
				goto SCA_FLUSH;
			}
		}

STEP14:
		if (sca_cmd->mode == CBC_MAC || sca_cmd->mode == CMAC) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->out >> 32);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 14;
				goto SCA_FLUSH;
			}
		}

		is_last = 1;
		sca_queue_recv();
		cmd_rdptr = (sca_cmd_index[sca_slot_id] + 1) &
			    (QUEUE_DEPTH - 1);
		REG_WR_32_SYNC(sca_cmd_base[sca_slot_id], RSP_WRPTR_OFFSET,
			       cmd_rdptr);
		REG_WR_32(sca_cmd_base[sca_slot_id], CMD_RDPTR_OFFSET,
			  cmd_rdptr);

		sca_cmd_step = 0;
		sca_slot_id++;
		if (sca_slot_id >= 8)
			sca_slot_id = 0;

		num--;
		return;
	}
}

static inline void hash_sca_proccess(int hash_index, int sca_index,
				     int hash_num, int sca_num)
{
	u32 cmd_rdptr;
	struct cmd *cmd_buf;
	struct rsp *rsp_buf;
	int ret;
	u64 dst_addr = 0;

	while (sca_num > 0) {
HASH_FLUSH:
		if (hash_cmd_step)
			hash_queue_recv();
		switch (hash_cmd_step) {
		case 0:
			break;
		case 1:
			goto STEP1;
		case 2:
			goto STEP2;
		case 3:
			goto STEP3;
		case 4:
			goto STEP4;
		case 5:
			goto STEP5;
		case 6:
			goto STEP6;
		case 7:
			goto STEP7;
		case 8:
			goto STEP8;
		case 9:
			goto STEP9;
		case 10:
			goto STEP10;
		case 11:
			goto STEP11;
		case 12:
			goto STEP12;
		case 13:
			goto STEP13;
		case 14:
			goto STEP14;
		case 15:
			goto STEP15;
		case 16:
			goto STEP16;
		default:
			break;
		}

		cmd_buf = cmd_base[HASH][hash_index];
		rsp_buf = rsp_base[HASH][hash_index];
		cmd_rdptr =
			REG_RD_32(reg_base[HASH][hash_index], CMD_RDPTR_OFFSET);
		hash_cmd = (struct hash_cmd *)(&cmd_buf[cmd_rdptr]);
		hash_cmd_tag[hash_slot_id] = hash_cmd->tag;
		hash_cmd_index[hash_slot_id] = cmd_rdptr;
		hash_cmd_base[hash_slot_id] = reg_base[HASH][hash_index];
		hash_cmd_rsp_buf[hash_slot_id] = rsp_buf;
		hash_cmd_state[hash_slot_id] = 0;
		hash_cmd_complete_num[hash_slot_id] = 0;
		hash_cmd_callback[hash_slot_id] = NULL;

		hash_init.mode = hash_cmd->mode;
		hash_init.algo = hash_cmd->algo;
		hash_init.slot_id = hash_slot_id;
		hash_cmd->in |= 0x300000000000;
		hash_cmd->out |= 0x300000000000;
		hash_cmd->key |= 0x300000000000;
		hash_cmd->tag |= 0x300000000000;

		if (unlikely(hash_cmd->dma_mode))
			hash_proc.src_addr_type = 1;
		else
			hash_proc.src_addr_type = 0;

		if (unlikely(hash_cmd->in_size == 0))
			hash_proc.is_zero_proc = 1;
		else
			hash_proc.is_zero_proc = 0;

		hash_proc.slot_id = hash_slot_id;

		hash_finish.mac_tlen = hash_mac_len[hash_cmd->algo];
		hash_finish.slot_id = hash_slot_id;

STEP1:
		if (sce_driver.hash_rsp_space && sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_FUNC_OFFSET,
				  hash_init.data);
			sce_driver.hash_rsp_space--;
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 1;
			goto SCA_FLUSH;
		}

STEP2:
		if (sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
				  (iv_paddr + hash_cmd->algo * 128u) &
					  0xFFFFFFFF);
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 2;
			goto SCA_FLUSH;
		}

STEP3:
		if (sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
				  (iv_paddr + hash_cmd->algo * 128u) >> 32);
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 3;
			goto SCA_FLUSH;
		}

STEP4:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->key & 0xFFFFFFFF);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 4;
				goto SCA_FLUSH;
			}
		}

STEP5:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->key >> 32);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 5;
				goto SCA_FLUSH;
			}
		}

STEP6:
		if (sce_driver.hash_rsp_space && sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_FUNC_OFFSET,
				  hash_proc.data);
			sce_driver.hash_rsp_space--;
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 6;
			goto SCA_FLUSH;
		}

STEP7:
		if (likely(hash_cmd->in_size)) {
			if (likely(hash_cmd->dma_mode == 0)) {
				if (sce_driver.hash_cmd_space) {
					REG_WR_32(sce_driver.base,
						  HASH_CQ_PARA_OFFSET,
						  hash_cmd->in_size - 1);
					sce_driver.hash_cmd_space--;
				} else {
					hash_cmd_step = 7;
					goto SCA_FLUSH;
				}
			}
		}

STEP8:
		if (likely(hash_cmd->in_size)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->in & 0xFFFFFFFF);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 8;
				goto SCA_FLUSH;
			}
		}

STEP9:
		if (likely(hash_cmd->in_size)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->in >> 32);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 9;
				goto SCA_FLUSH;
			}
		}

STEP10:
		if (sce_driver.hash_rsp_space && sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_FUNC_OFFSET,
				  hash_finish.data);
			sce_driver.hash_rsp_space--;
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 10;
			goto SCA_FLUSH;
		}

STEP11:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  (iv_paddr + hash_cmd->algo * 128u) &
						  0xFFFFFFFF);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 11;
				goto SCA_FLUSH;
			}
		}

STEP12:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  (iv_paddr + hash_cmd->algo * 128u) >>
						  32);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 12;
				goto SCA_FLUSH;
			}
		}

STEP13:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->key & 0xFFFFFFFF);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 13;
				goto SCA_FLUSH;
			}
		}

STEP14:
		if (unlikely(hash_cmd->mode)) {
			if (sce_driver.hash_cmd_space) {
				REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
					  hash_cmd->key >> 32);
				sce_driver.hash_cmd_space--;
			} else {
				hash_cmd_step = 14;
				goto SCA_FLUSH;
			}
		}

STEP15:
		if (sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
				  hash_cmd->out & 0xFFFFFFFF);
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 15;
			goto SCA_FLUSH;
		}

STEP16:
		if (sce_driver.hash_cmd_space) {
			REG_WR_32(sce_driver.base, HASH_CQ_PARA_OFFSET,
				  hash_cmd->out >> 32);
			sce_driver.hash_cmd_space--;
		} else {
			hash_cmd_step = 16;
			goto SCA_FLUSH;
		}

		cmd_rdptr = (hash_cmd_index[hash_slot_id] + 1) &
			    (QUEUE_DEPTH - 1);
		REG_WR_32(hash_cmd_base[hash_slot_id], CMD_RDPTR_OFFSET,
			  cmd_rdptr);

		hash_cmd_step = 0;
		hash_slot_id++;
		if (hash_slot_id >= 16)
			hash_slot_id = 8;

		hash_num--;
		if (hash_num <= 0)
			return;

SCA_FLUSH:
		if (sca_cmd_step)
			sca_queue_recv();
		switch (sca_cmd_step) {
		case 0:
			break;
		case 1:
			goto STEP21;
		case 2:
			goto STEP22;
		case 3:
			goto STEP23;
		case 4:
			goto STEP24;
		case 5:
			goto STEP25;
		case 6:
			goto STEP26;
		case 7:
			goto STEP27;
		case 8:
			goto STEP28;
		case 9:
			goto STEP29;
		case 10:
			goto STEP30;
		case 11:
			goto STEP31;
		case 12:
			goto STEP32;
		default:
			break;
		}

		cmd_buf = cmd_base[SCA][sca_index];
		rsp_buf = rsp_base[SCA][sca_index];
		cmd_rdptr =
			REG_RD_32(reg_base[SCA][sca_index], CMD_RDPTR_OFFSET);
		sca_cmd = (struct sca_cmd *)(&cmd_buf[cmd_rdptr]);
		sca_cmd_tag[sca_slot_id] = sca_cmd->tag;
		sca_cmd_index[sca_slot_id] = cmd_rdptr;
		if (sca_cmd->mode == XTS) {
			ret = sca_xts_process(sca_index, sca_num, sca_cmd);
			return;
		}
		if (sca_cmd->mode == GCM || sca_cmd->mode == CCM ||
		    sca_cmd->mode == CMAC || sca_cmd->mode == CBC_MAC) {
			pr_err("hash_sca_speed: unsupport sca mode!\n");
			return;
		}
		sca_cmd_base[sca_slot_id] = reg_base[SCA][sca_index];
		sca_cmd_rsp_buf[sca_slot_id] = rsp_buf;
		sca_cmd_state[sca_slot_id] = 0;
		sca_cmd_complete_num[sca_slot_id] = 0;
		sca_cmd_callback[sca_slot_id] = NULL;

		sca_init.mode = sca_cmd->mode;
		sca_init.key_len = sca_cmd->algo >> 4;
		sca_init.algo = sca_cmd->algo & 0xF;
		sca_init.slot_id = sca_slot_id;

		sca_proc.dst_src_addr_type = sca_cmd->dma_mode;
		sca_proc.op_mode = sca_cmd->op_mode;
		sca_proc.slot_id = sca_slot_id;

		sca_finish.slot_id = sca_slot_id;

		dst_addr = sca_cmd->out;
		sca_cmd->in |= 0x300000000000;
		sca_cmd->out |= 0x300000000000;
		sca_cmd->key |= 0x300000000000;
		sca_cmd->iv |= 0x300000000000;
STEP21:
		if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET,
				  sca_init.data);
			sce_driver.sca_rsp_space--;
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 21;
			goto HASH_FLUSH;
		}

STEP22:
		if (likely(sca_cmd->mode != ECB)) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->iv & 0xFFFFFFFF);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 2;
				goto HASH_FLUSH;
			}
		}

STEP23:
		if (likely(sca_cmd->mode != ECB)) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->iv >> 32);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 3;
				goto HASH_FLUSH;
			}
		}

STEP24:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->key & 0xFFFFFFFF);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 4;
			goto HASH_FLUSH;
		}

STEP25:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->key >> 32);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 5;
			goto HASH_FLUSH;
		}

STEP26:
		if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET,
				  sca_proc.data);
			sce_driver.sca_rsp_space--;
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 6;
			goto HASH_FLUSH;
		}

STEP27:
		if (likely((sca_cmd->dma_mode & 0x2) == 0)) {
			if (sce_driver.sca_cmd_space) {
				REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
					  sca_cmd->in_size - 1);
				sce_driver.sca_cmd_space--;
			} else {
				sca_cmd_step = 7;
				goto HASH_FLUSH;
			}
		}

STEP28:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->in & 0xFFFFFFFF);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 8;
			goto HASH_FLUSH;
		}

STEP29:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->in >> 32);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 9;
			goto HASH_FLUSH;
		}

STEP30:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->out & 0xFFFFFFFF);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 10;
			goto HASH_FLUSH;
		}

STEP31:
		if (sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_PARA_OFFSET,
				  sca_cmd->out >> 32);
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 11;
			goto HASH_FLUSH;
		}

STEP32:
		if (sce_driver.sca_rsp_space && sce_driver.sca_cmd_space) {
			REG_WR_32(sce_driver.base, SCA_CQ_FUNC_OFFSET,
				  sca_finish.data);
			sce_driver.sca_rsp_space--;
			sce_driver.sca_cmd_space--;
		} else {
			sca_cmd_step = 12;
			goto HASH_FLUSH;
		}

		is_last = 1;
		sca_queue_recv();
		cmd_rdptr = (sca_cmd_index[sca_slot_id] + 1) &
			    (QUEUE_DEPTH - 1);
		REG_WR_32_SYNC(sca_cmd_base[sca_slot_id], RSP_WRPTR_OFFSET,
			       cmd_rdptr);
		REG_WR_32(sca_cmd_base[sca_slot_id], CMD_RDPTR_OFFSET,
			  cmd_rdptr);

		sca_cmd_step = 0;
		sca_slot_id++;
		if (sca_slot_id >= 8)
			sca_slot_id = 0;

		sca_num--;
	}
}
static void sce_init(u32 *base)
{
	int type, index;
	union rnpool_intr_msk_reg int_msk = { .fill_done_msk = 1 };

	REG_WR_32(base, HASH_CTRL_OFFSET, 0x49);
	REG_WR_32(base, HASH_INTR_STAT0_OFFSET, 0x3F);
	REG_WR_32(base, HASH_INTR_STAT1_OFFSET, 0xFFFFFFFF);
	REG_WR_32(base, HASH_INTR_STAT2_OFFSET, 0xFFFFFFFF);

	REG_WR_32(base, SCA_CTRL_OFFSET, 0x49);
	REG_WR_32(base, SCA_INTR_STAT0_OFFSET, 0x3F);
	REG_WR_32(base, SCA_INTR_STAT1_OFFSET, 0xFFFFFFFF);
	REG_WR_32(base, SCA_INTR_STAT2_OFFSET, 0xFFFFFFFF);

	REG_WR_32(base, RN_POOL_INTR_MSK_OFFSET, int_msk.data);

	REG_WR_32(base, 0x6000, 0x10100F0);
	REG_WR_32(base, 0x6008, 7);
	REG_WR_32(base, 0x6004, 0);
	REG_WR_32_SYNC(base, 0x6000, 0x10100F1);

	REG_RD_32(base, HASH_CSQ_OFFSET);
	REG_RD_32(base, HASH_CSQ_OFFSET);
	REG_RD_32(base, HASH_CSQ_OFFSET);
	REG_RD_32(base, HASH_CSQ_OFFSET);

	REG_RD_32(base, SCA_CSQ_OFFSET);
	REG_RD_32(base, SCA_CSQ_OFFSET);
	REG_RD_32(base, SCA_CSQ_OFFSET);
	REG_RD_32(base, SCA_CSQ_OFFSET);

	iv_vaddr = sce_malloc(0x800);
	if (iv_vaddr) {
		iv_paddr = virt_to_phys(iv_vaddr);
		iv_paddr |= 0x300000000000;
		memcpy(iv_vaddr + SHA1 * 128, sha1_iv_data, 20);
		memcpy(iv_vaddr + SHA224 * 128, sha224_iv_data, 32);
		memcpy(iv_vaddr + SHA256 * 128, sha256_iv_data, 32);
		memcpy(iv_vaddr + SHA384 * 128, sha384_iv_data, 64);
		memcpy(iv_vaddr + SHA512 * 128, sha512_iv_data, 64);
		memcpy(iv_vaddr + MD5 * 128, md5_iv_data, 16);
		memcpy(iv_vaddr + SM3 * 128, sm3_iv_data, 32);
	}

	for (type = HASH; type <= TRNG; type++)
		for (index = 0; index < 4; index++) {
			reg_base[type][index] =
				(void *)((u64)queue_desc[type].reg +
					 index * 0x40000);
			cmd_base[type][index] =
				(void *)((u64)queue_desc[type].reg +
					 index * 0x40000 + 0x10000);
			rsp_base[type][index] =
				(void *)((u64)queue_desc[type].reg +
					 index * 0x40000 + 0x20000);
		}
}

static int sce_alloc_task(int type, int index)
{
	void *base = reg_base[type][index];
	union ring_cfg ring_cfg;
	u32 cmd_wrptr;
	u32 cmd_rdptr;
	u32 rsp_rdptr;
	u32 rsp_wrptr;
	u32 cmd_num, rsp_space_num;

	ring_cfg.data = REG_RD_32(base, RING_CFG_OFFSET);
	if (unlikely(ring_cfg.cmd_suspend)) {
		if (ring_cfg.ring_flush) {
			REG_WR_32(base, CMD_WRPTR_OFFSET, 0);
			REG_WR_32(base, CMD_RDPTR_OFFSET, 0);
			REG_WR_32(base, RSP_RDPTR_OFFSET, 0);
			REG_WR_32(base, RSP_WRPTR_OFFSET, 0);
			REG_WR_32(base, CMD_BADDR0_OFFSET, 0);
			REG_WR_32(base, CMD_BADDR1_OFFSET, 0);
			REG_WR_32(base, RSP_BADDR0_OFFSET, 0);
			REG_WR_32(base, RSP_BADDR1_OFFSET, 0);

			ring_cfg.ring_flush = 0;
			REG_WR_32_SYNC(base, RING_CFG_OFFSET, ring_cfg.data);
		}

		cmd_wrptr = REG_RD_32(base, CMD_WRPTR_OFFSET);
		if (unlikely(cmd_wrptr > (QUEUE_DEPTH - 1)))
			return 0;

		cmd_rdptr = REG_RD_32(base, CMD_RDPTR_OFFSET);
		if (unlikely(cmd_rdptr > (QUEUE_DEPTH - 1)))
			return 0;

		rsp_wrptr = REG_RD_32(base, RSP_WRPTR_OFFSET);
		if (unlikely(rsp_wrptr > (QUEUE_DEPTH - 1)))
			return 0;

		rsp_rdptr = REG_RD_32(base, RSP_RDPTR_OFFSET);
		if (unlikely(rsp_rdptr > (QUEUE_DEPTH - 1)))
			return 0;

		if (cmd_rdptr == rsp_wrptr) {
			REG_WR_32_SYNC(base, RING_STATUS_OFFSET, 0);
			return 0;
		}
		if (type == HASH)
			hash_queue_recv();
		if (type == SCA)
			sca_queue_recv();
		return 0;
	}

	cmd_wrptr = REG_RD_32(base, CMD_WRPTR_OFFSET);
	if (unlikely(cmd_wrptr > (QUEUE_DEPTH - 1)))
		return 0;

	cmd_rdptr = REG_RD_32(base, CMD_RDPTR_OFFSET);
	if (unlikely(cmd_rdptr > (QUEUE_DEPTH - 1)))
		return 0;

	rsp_wrptr = REG_RD_32(base, RSP_WRPTR_OFFSET);
	if (unlikely(rsp_wrptr > (QUEUE_DEPTH - 1)))
		return 0;

	rsp_rdptr = REG_RD_32(base, RSP_RDPTR_OFFSET);
	if (unlikely(rsp_rdptr > (QUEUE_DEPTH - 1)))
		return 0;

	if (unlikely(cmd_wrptr == cmd_rdptr)) {
		if (cmd_wrptr == rsp_wrptr) {
			REG_WR_32_SYNC(base, RING_STATUS_OFFSET, 0);
			return 0;
		}
		if (type == HASH)
			hash_queue_recv();
		if (type == SCA)
			sca_queue_recv();

		return 0;
	}

	if (cmd_wrptr >= cmd_rdptr)
		cmd_num = cmd_wrptr - cmd_rdptr;
	else
		cmd_num = QUEUE_DEPTH - (cmd_rdptr - cmd_wrptr);

	if (rsp_wrptr >= rsp_rdptr)
		rsp_space_num = QUEUE_DEPTH - (rsp_wrptr - rsp_rdptr) - 4;
	else
		rsp_space_num = rsp_rdptr - rsp_wrptr - 4;
	if (cmd_num > rsp_space_num)
		cmd_num = rsp_space_num;
	if (!cmd_num)
		return 0;

	REG_WR_32(base, RING_STATUS_OFFSET, 1);

	if (type == HASH)
		hash_task_index = index;
	else if (type == SCA)
		sca_task_index = index;
	else
		trng_task_index = index;


	return cmd_num;
}

static int sce_work_thread(void *arg)
{
	int queue = 0;
	u64 time = sce_get_jiffies();
	int hash_work_num, sca_work_num, trng_work_num, i;

	sce_init(sce_driver.base);

	while (1) {
		hash_work_num = 0;
		sca_work_num = 0;
		trng_work_num = 0;

		for (i = 0; i < 4; i++) {
			if (queue >= 4)
				queue = 0;
			if ((hash_work_num == 0) &&
			    ((sce_driver.queue_mask[HASH] >> queue) & 0x1)) {
				hash_work_num = sce_alloc_task(HASH, queue);
			}

			if ((sca_work_num == 0) &&
			    ((sce_driver.queue_mask[SCA] >> queue) & 0x1)) {
				sca_work_num = sce_alloc_task(SCA, queue);
			}

			if ((trng_work_num == 0) &&
			    ((sce_driver.queue_mask[TRNG] >> queue) & 0x1)) {
				trng_work_num = sce_alloc_task(TRNG, queue);
			}

			queue++;
			if (hash_work_num && sca_work_num && trng_work_num)
				break;
		}

		if ((hash_work_num | sca_work_num | trng_work_num) == 0) {
			if (kthread_should_stop())
				break;
			schedule_timeout_interruptible(0);
		} else {
			if (hash_work_num && sca_work_num) {
				hash_sca_proccess(hash_task_index,
						  sca_task_index, hash_work_num,
						  sca_work_num);
			} else {
				if (hash_work_num)
					hash_proccess(hash_task_index,
						      hash_work_num);

				if (sca_work_num)
					sca_proccess(sca_task_index,
						     sca_work_num);
			}

			if (trng_work_num)
				trng_proccess(trng_task_index, trng_work_num);

		}

		if (time + sce_msecs_to_jiffies(100) < sce_get_jiffies()) {
			time = sce_get_jiffies();
			schedule();
		}
	}
	sce_info("%s:%d\n", __func__, __LINE__);

	return 0;
}

int sce_add_thread(unsigned int cpu)
{
	char name[32] = { 0 };

	sprintf(name, "sce_kthread_%d", 0);
	task = sce_create_kthread_on_node(sce_work_thread, NULL, 0, name);
	if (task == NULL) {
		sce_err("%s:create kthread fail!\n", __func__);
		return -1;
	}

	if (cpu < NR_CPUS)
		kthread_bind(task, cpu);

	sce_wake_up_kthread(task);
	sce_info("%s:%d ok!\n", __func__, __LINE__);

	return 0;
}

void sce_del_thread(void)
{
	sce_stop_kthread(task);
}

void sce_wake_up_thread(void)
{
	sce_wake_up_kthread(task);
}
