#!/bin/bash

test_file=`find -name mtest.ko`

sca_alg=(aes sm4 des tdes)
sca_mode=(ecb ctr cbc ofb)

sm4_alg=(sm4)
mac_mode=(cmac cbcmac)
aead_mode=(gcm ccm)

aes_alg=aes

aes_klen=(16 32)
aes_24_klen=(16 24 32)

hash_alg=(sha1 sha224 sha256 sha384 sm3 md5)
hash_mode=("hash" hmac)

trng_alg=(trng)
trng_mode=(rnp sm4)


test_alg=${sca_alg[@]}
test_mode=${sca_mode[@]}
test_klen=${aes_klen[@]}
function check_test() {
	if [ -z $test_file ] ;then
		echo "no test"
		make
	fi
}

function test_func() {
	check_test
	for alg in ${test_alg[@]}
	do
		for mode in ${test_mode[@]}
		do
			insmod mtest.ko alg=$alg mode=$mode;
		done
	done
}

function test_aes_func() {
        check_test
        for klen in ${test_klen[@]}
        do
                for mode in ${test_mode[@]}
                do
                        insmod mtest.ko alg=aes mode=$mode klen=$klen;
                done
        done
	
}



function test_xts() {
	check_test
	insmod mtest.ko alg=aes mode=xts klen=32 ;
	insmod mtest.ko alg=aes mode=xts klen=64 ;
	insmod mtest.ko alg=sm4 mode=xts klen=32 ;
}

function test_skcipher() {
	test_alg=${sca_alg[@]}
	test_mode=${sca_mode[@]}
	test_klen=${aes_24_klen[@]}

	test_func
	test_aes_func
#	test_xts
}

function test_hash() {
	test_alg=${hash_alg[@]}
	test_mode=${hash_mode[@]}
	test_func
}

function test_mac() {
	test_alg=${sm4_alg[@]}
	test_mode=${mac_mode[@]}
	test_klen=${aes_klen[@]}
	test_func
	test_aes_func
}

function test_aead() {
        test_alg=${sm4_alg[@]}
        test_mode=${aead_mode[@]}
        test_klen=${aes_klen[@]}
        test_func
        test_aes_func
}

function test_trng() {
	test_alg=${trng_alg[@]}
	test_mode=${trng_mode[@]}
	test_func
}

function test_sm2() {
	check_test
	insmod mtest.ko alg=sm2
}

function test_all() {
	test_skcipher
#	test_mac
	#test_aead
	test_hash
	test_trng
	test_sm2
}

case $1 in
	check)
		check_test
		;;
	trng)
		test_trng
		;;
	cipher)
		test_skcipher
		;;
#	mac)
#		test_mac
#		;;
	sm2)
		test_sm2
		;;
	hash)
		test_hash
		;;
	all)
		test_all
		;;
	*)
		test_all
		;;
esac

echo "finish auto test"
