#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "md5_test.h"
#include "sm3_test.h"
#include "sha_test.h"

typedef void (*_test_fn)(void);
typedef struct {
    char *alg_name;
    _test_fn hash_fn;
    _test_fn hmac_fn;
} hash_test_t;

static const hash_test_t test_maps[] = {
    {.alg_name = "md5", .hash_fn = md5_test, .hmac_fn = hmac_md5_test},
    {.alg_name = "sm3", .hash_fn = sm3_test, .hmac_fn = hmac_sm3_test},
    {.alg_name = "sha1", .hash_fn = sha1_test, .hmac_fn = hmac_sha1_test},
    {.alg_name = "sha224", .hash_fn = sha224_test, .hmac_fn = hmac_sha224_test},
    {.alg_name = "sha256", .hash_fn = sha256_test, .hmac_fn = hmac_sha256_test},
    {.alg_name = "sha384", .hash_fn = sha384_test, .hmac_fn = hmac_sha384_test},
    {.alg_name = "sha512", .hash_fn = sha512_test, .hmac_fn = hmac_sha512_test},
    {.alg_name = "aes", .hash_fn = aes_cbcmac_test, .hmac_fn = aes_cbcmac_test},
    {.alg_name = "aes", .hash_fn = aes_cmac_test, .hmac_fn = aes_cmac_test},
    {.alg_name = "sm4", .hash_fn = sm4_cbcmac_test, .hmac_fn = sm4_cbcmac_test},
    {.alg_name = "sm4", .hash_fn = sm4_cmac_test, .hmac_fn = sm4_cmac_test},
    {.alg_name = "des", .hash_fn = des_cbcmac_test, .hmac_fn = des_cbcmac_test},
    {.alg_name = "des", .hash_fn = des_cmac_test, .hmac_fn = des_cmac_test},
    {.alg_name = "tdes", .hash_fn = des3_ede_cbcmac_test, .hmac_fn = des3_ede_cbcmac_test},
    {.alg_name = "tdes", .hash_fn = des3_ede_cmac_test, .hmac_fn = des3_ede_cmac_test},
};

void hmac_test(char *alg, char *mode, u32 klen, u32 slen, u32 elen)
{
    u32 i;
    for (i = 0; i < sizeof(test_maps) / sizeof(test_maps[0]); i++) {
        if (strlen(test_maps[i].alg_name) == strlen(alg) &&
            0 == memcmp(alg, test_maps[i].alg_name, strlen(alg))) {
            test_maps[i].hmac_fn();
        }
    }
}

void hash_test(char *alg, char *mode, u32 klen, u32 slen, u32 elen)
{
    u32 i;
    for (i = 0; i < sizeof(test_maps) / sizeof(test_maps[0]); i++) {
        if (strlen(test_maps[i].alg_name) == strlen(alg) &&
            0 == memcmp(alg, test_maps[i].alg_name, strlen(alg))) {
	    if (0 == memcmp(mode, "cmac", 4)) {
	        i++;
	    }
            test_maps[i].hash_fn();
	    break;
        }
    }
}
