#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "hash_test.h"
#include "../common/common.h"

static hash_testvec_t md5_tv_template[] = {
    {
        .psize = 0,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_NORMAL,
        .plaintext = "",
        .digest = "\xd4\x1d\x8c\xd9\x8f\x00\xb2\x04"
                  "\xe9\x80\x09\x98\xec\xf8\x42\x7e",
    },
    {
        .psize = 1,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_NORMAL,
        .plaintext = "a",
        .digest = "\x0c\xc1\x75\xb9\xc0\xf1\xb6\xa8"
                  "\x31\xc3\x99\xe2\x69\x77\x26\x61",
    },
    {
        .psize = 3,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_NORMAL,
        .plaintext = "abc",
        .digest = "\x90\x01\x50\x98\x3c\xd2\x4f\xb0"
                  "\xd6\x96\x3f\x7d\x28\xe1\x7f\x72",
    },
    {
        .psize = 14,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_NORMAL,
        .plaintext = "message digest",
        .digest = "\xf9\x6b\x69\x7d\x7c\xb7\x93\x8d"
                  "\x52\x5a\x2f\x31\xaa\xf1\x61\xd0",
    },
    {
        .psize = 26,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_NORMAL,
        .plaintext = "abcdefghijklmnopqrstuvwxyz",
        .digest = "\xc3\xfc\xd3\xd7\x61\x92\xe4\x00"
                  "\x7d\xfb\x49\x6c\xca\x67\xe1\x3b",
    },
    {
        .psize = 62,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_NORMAL,
        .plaintext =
            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
        .digest = "\xd1\x74\xab\x98\xd2\x77\xd9\xf5"
                  "\xa5\x61\x1c\x2c\x9f\x41\x9d\x9f",
    },
    {
        .psize = 80,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_NORMAL,
        .plaintext =
            "12345678901234567890123456789012345678901234567890123456789012"
            "345678901234567890",
        .digest = "\x57\xed\xf4\xa2\x2b\xe3\xc9\x55"
                  "\xac\x49\xda\x2e\x21\x07\xb6\x7a",
    },
};

void md5_test(void)
{
    u32 i;
    u8 *src, *dst;
    hash_testvec_t *test_vec;
    u8 success = 1;

    src = kmalloc(1024 + 512, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    dst = src + 1024;

    for (i = 0; i < sizeof(md5_tv_template) / sizeof(md5_tv_template[0]); i++) {
        test_vec = &md5_tv_template[i];
        memset(src, 0, 1024 + 512);
        memcpy(src, test_vec->plaintext, test_vec->psize);
        ahash_dgst_test("md5", NULL, 0, src, test_vec->psize, dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("md5_test dgst err!\n");
            success = 0;
            continue;
        }

        memset(dst, 0, test_vec->dsize);
        ahash_finup_test("md5", NULL, 0, src, test_vec->psize, dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("md5_test finup err!\n");
            success = 0;
            continue;
        }

        if (!test_vec->psize) {
            continue;
        }

        memset(dst, 0, test_vec->dsize);
        ahash_step_test("md5", NULL, 0, src, test_vec->psize, dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("md5_test step err!\n");
            success = 0;
        }
    }

    if (success) {
        pr_info("md5_test success ");
    }

    kfree(src);
}

static hmac_testvec_t hmac_md5_tv_template[] = {
    {
        .ksize = 16,
        .psize = 8,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_HMAC,
        .key =
            "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b",
        .plaintext = "Hi There",
        .digest = "\x92\x94\x72\x7a\x36\x38\xbb\x1c"
                  "\x13\xf4\x8e\xf8\x15\x8b\xfc\x9d",
    },
    {
        .ksize = 4,
        .psize = 28,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_HMAC,
        .key = "Jefe",
        .plaintext = "what do ya want for nothing?",
        .digest = "\x75\x0c\x78\x3e\x6a\xb0\xb5\x03"
                  "\xea\xa8\x6e\x31\x0a\x5d\xb7\x38",
    },
    {
        .ksize = 16,
        .psize = 50,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_HMAC,
        .key =
            "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa",
        .plaintext = "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd",
        .digest = "\x56\xbe\x34\x52\x1d\x14\x4c\x88"
                  "\xdb\xb8\xc7\x33\xf0\xe8\xb3\xf6",
    },
    {
        .ksize = 25,
        .psize = 50,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_HMAC,
        .key = "\x01\x02\x03\x04\x05\x06\x07\x08"
               "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
               "\x11\x12\x13\x14\x15\x16\x17\x18\x19",
        .plaintext = "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd",
        .digest = "\x69\x7e\xaf\x0a\xca\x3a\x3a\xea"
                  "\x3a\x75\x16\x47\x46\xff\xaa\x79",
    },
    {
        .ksize = 16,
        .psize = 20,
        .dsize = 16,
        .alg = SCE_MD5,
        .mode = SCE_HMAC,
        .key =
            "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c",
        .plaintext = "Test With Truncation",
        .digest = "\x56\x46\x1e\xf2\x34\x2e\xdc\x00"
                  "\xf9\xba\xb9\x95\x69\x0e\xfd\x4c",
    },
};

void hmac_md5_test(void)
{
    u32 i;
    u8 *src, *dst, *key;
    hmac_testvec_t *test_vec;
    u8 success = 1;

    src = kmalloc(1024 + 512 + 512, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    dst = src + 1024;
    key = dst + 512;

    for (i = 0;
         i < sizeof(hmac_md5_tv_template) / sizeof(hmac_md5_tv_template[0]);
         i++) {
        test_vec = &hmac_md5_tv_template[i];
        memset(src, 0, 1024 + 512 + 512);
        memcpy(src, test_vec->plaintext, test_vec->psize);
        memcpy(key, test_vec->key, test_vec->ksize);

        ahash_dgst_test("hmac(md5)", key, test_vec->ksize, src, test_vec->psize,
                        dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("hmac_md5_test dgst err!\n");
            print_buf("key", key, test_vec->ksize);
            print_buf("src", src, test_vec->psize);
            print_buf("dst", dst, test_vec->dsize);
            print_buf("digest", (u8 *)test_vec->digest, test_vec->dsize);

            success = 0;
            break;
        }

        memset(dst, 0, test_vec->dsize);
        ahash_finup_test("hmac(md5)", key, test_vec->ksize, src,
                         test_vec->psize, dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("hmac_md5_test finup err!\n");
            print_buf("key", key, test_vec->ksize);
            print_buf("src", src, test_vec->psize);
            print_buf("dst", dst, test_vec->dsize);
            print_buf("digest", (u8 *)test_vec->digest, test_vec->dsize);
            success = 0;
            continue;
        }

        if (!test_vec->psize) {
            continue;
        }

        memset(dst, 0, test_vec->dsize);
        ahash_step_test("hmac(md5)", key, test_vec->ksize, src, test_vec->psize,
                        dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("hmac_md5_test step err!\n");
            print_buf("key", key, test_vec->ksize);
            print_buf("src", src, test_vec->psize);
            print_buf("dst", dst, test_vec->dsize);
            print_buf("digest", (u8 *)test_vec->digest, test_vec->dsize);
            success = 0;
        }
    }

    if (success) {
        pr_info("hmac_md5_test success ");
    }

    kfree(src);
}
