
#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "hash_test.h"
#include "../common/common.h"

static hmac_testvec_t cbcmac_tv_template[] = {
	{
        .key            = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                            "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                            "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96",
        .digest         = "\xa5\x1c\x52\x77\x25\x63\x2c\xcf",
        .psize          = 8,
        .ksize          = 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    },{
        .key            = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                            "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                            "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                            "\xae\x2d\x8a\x57",
        .digest         = "\x7a\xe0\xe1\x7e\x61\x86\x99\x74",
        .psize          = 20,
        .ksize          = 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\xE4\x23\x33\x8A\x35\x64\x61\xE2\x49\x03\xDD\xC6\xB8\xCA\x55\x7A" 
                    "\xd0\xe7\x4b\xfb\x5d\xe5\x0c\xe7",
        .plaintext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93" 
                    "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
        .digest		= "\x6c\xe5\xbf\xa1\x49\x9e\xd4\x49",
        .psize		= 32,
        .ksize		= 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                    "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                    "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .digest		= "\x00\x00\x00\x00\x00\x00\x00\x00",
        .psize		= 0,
        .ksize		= 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    }, 
    {
        .key		= "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                    "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                    "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .plaintext	= "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                    "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                    "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                    "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51",
        .digest		= "\xdf\xbb\x75\xaf\xe7\x48\xba\xa0",
        .psize		= 32,
        .ksize		= 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    },
    {
        .key            = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96",
        .digest         = "\x4e\xd6\x35\x72\x6d\x82\x43\x06",
        .psize          = 8,
        .ksize          = 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key            = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                            "\xae\x2d\x8a\x57",
        .digest         = "\xb6\xc5\x18\x67\x5b\x83\x16\x16",
        .psize          = 20,
        .ksize          = 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key            = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
        .plaintext      = "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
                            "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
        .digest         = "\xfd\x64\x03\xf0\x51\x74\x81\xf7",
        .psize          = 32,
        .ksize          = 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
        .plaintext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93" 
                    "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
                    "\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4" 
                    "\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b",
        .digest		= "\x0b\xcc\x79\xcf\x40\xd5\x85\x54",
        .psize		= 64,
        .ksize		= 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
        .plaintext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93" 
                    "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
                    "\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4" 
                    "\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
                    "\x59\xda\x19\x4f\xec\x07\x8e\xe2\xf0\x61\xf9\x27\x61\x6f\xf8\xdf" 
                    "\x62\x4d\xaf\x06\xfe\x41\xa6\xa6\xf9\xa2\x06\x40\xb3\x04\xbd\xe6" 
                    "\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e\xa6\x58\xdc\x91\x17\x58\x42\x95" 
                    "\xa3\x7c\x81\x78\xa6\x3d\x3f\x75\x74\x17\x1a\xd3\x6c\x2f\x48\x39",
        .digest		= "\x9b\xd2\xee\x0a\xa7\x51\xa7\xa4",
        .psize		= 128,
        .ksize		= 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
        .plaintext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93" 
                    "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
                    "\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4" 
                    "\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
                    "\x59\xda\x19\x4f\xec\x07\x8e\xe2\xf0\x61\xf9\x27\x61\x6f\xf8\xdf" 
                    "\x62\x4d\xaf\x06\xfe\x41\xa6\xa6\xf9\xa2\x06\x40\xb3\x04\xbd\xe6" 
                    "\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e\xa6\x58\xdc\x91\x17\x58\x42\x95" 
                    "\xa3\x7c\x81\x78\xa6\x3d\x3f\x75\x74\x17\x1a\xd3\x6c\x2f\x48\x39"
                    "\x20\x20\xc1\x9a\x29\x84\x7d\x2d\x52\xa1\xf9\x5c\xf3\x4f\x91\xfc"
                    "\x75\xcf\xd6\x2d\xe7\x9a\x59\x6e\x00\x0e\x8d\x22\x17\xbd\xa0\xdd" 
                    "\x79\x1f\x71\xe6\xcd\x2f\xb1\xb6\xbc\xc3\xdb\x02\x91\x41\x9b\x09"
                    "\xa9\xd2\x7e\xbd\x2c\x18\xae\xc0\x93\x0c\x02\x9a\xdb\x4e\xaa\xeb" 
                    "\x84\x4b\x43\x5e\xf0\x98\xf2\x5f\x86\x70\x96\x90\x15\x30\xcf\x3a" 
                    "\xc9\x33\x21\xec\x59\x86\xfc\x65\x7d\xbe\xb9\xf8\x97\xf9\x30\xa9" 
                    "\x6d\xfc\x0c\x6e\x36\x67\xd5\xa6\x67\xd9\xbd\x9b\x34\x5d\xa7\xdd" 
                    "\xda\x46\x33\x25\x60\x4a\x18\xf1\x55\x07\xb2\xb7\x26\x7b\xa6\x1e", 
        .digest		= "\x9c\xbe\x25\x7b\xa1\xc0\xb6\x41",
        .psize		= 256,
        .ksize		= 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    }, 	
	{
        .key            = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
                            "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a",
        .digest         = "\x3a\xd7\x7b\xb4\x0d\x7a\x36\x60"
                            "\xa8\x9e\xca\xf3\x24\x66\xef\x97",
        .psize          = 16,
        .dsize          = 16,
        .ksize		= 16,
        .mode		= SCE_HMAC,
        .alg		= SCE_MAC_AES,
    }, {
        .key            = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
                            "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                            "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                            "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                            "\x30",
        .digest         = "\x9d\x0d\xd0\x63\xfb\xcb\x24\x43"
                            "\xf8\xf2\x76\x03\xac\x39\xb0\x9d",
        .psize          = 33,
        .ksize          = 16,
        .dsize		= 16,
        .alg		= SCE_MAC_AES,
        .mode		= SCE_HMAC,
    }, {
        .key            = "\x60\x3d\xeb\x10\x15\xca\x71\xbe"
                            "\x2b\x73\xae\xf0\x85\x7d\x77\x81"
                            "\x1f\x35\x2c\x07\x3b\x61\x08\xd7"
                            "\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                            "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                            "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                            "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
                            "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
                            "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
                            "\xad\x2b\x41\x7b\xe6\x6c\x37\x10"
                            "\x1c",
        .digest         = "\x6a\x4e\xdb\x21\x47\x51\xdf\x4f"
                            "\xa8\x4d\x4c\x10\x3b\x72\x7d\xd6",
        .psize          = 65,
        .ksize          = 32,
        .dsize          = 16,
        .alg            = SCE_MAC_AES,
        .mode           = SCE_HMAC,
    }, {
        .key            = "\xff\xee\xdd\xcc\xbb\xaa\x99\x88"
                            "\x77\x66\x55\x44\x33\x22\x11\x00",
        .plaintext      = "\x01\x23\x45\x67\x89\xab\xcd\xef"
                            "\xfe\xdc\xba\x98\x76\x54\x32\x10",
        .digest         = "\x97\xb4\x75\x8f\x84\x92\x3d\x3f"
                            "\x86\x81\x0e\x0e\xea\x14\x6d\x73",
        .psize          = 16,
        .ksize          = 16,
        .dsize          = 16,
        .alg            = SCE_MAC_SM4,
        .mode           = SCE_HMAC,
    }, {
        .key            = "\x01\x23\x45\x67\x89\xab\xcd\xef"
                            "\xfe\xdc\xBA\x98\x76\x54\x32\x10",
        .plaintext      = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
                            "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb"
                            "\xcc\xcc\xcc\xcc\xcc\xcc\xcc\xcc"
                            "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                            "\xee",
        .digest         = "\xc7\xdb\x17\x71\xa1\x5c\x0d\x22"
                            "\xa3\x39\x3a\x31\x88\x91\x49\xa1",
        .psize          = 33,
        .ksize          = 16,
        .dsize          = 16,
        .alg            = SCE_MAC_SM4,
        .mode           = SCE_HMAC,
    },

};

static hmac_testvec_t cmac_tv_template[] = {
	{
        .key            = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
                            "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a",
        .digest         = "\x07\x0a\x16\xb4\x6b\x4d\x41\x44"
                            "\xf7\x9b\xdd\x9d\xd0\x4a\x28\x7c",
        .psize          = 16,
        .ksize          = 16,
        .dsize          = 16,
        .alg            = SCE_MAC_AES,
        .mode           = SCE_HMAC,
    }, {
        .key            = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
                            "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                            "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                            "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                            "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11",
        .digest         = "\xdf\xa6\x67\x47\xde\x9a\xe6\x30"
                            "\x30\xca\x32\x61\x14\x97\xc8\x27",
        .psize          = 40,
        .ksize          = 16,
        .dsize          = 16,
        .alg            = SCE_MAC_AES,
        .mode           = SCE_HMAC,
    }, {
        .key            = "\x60\x3d\xeb\x10\x15\xca\x71\xbe"
                            "\x2b\x73\xae\xf0\x85\x7d\x77\x81"
                            "\x1f\x35\x2c\x07\x3b\x61\x08\xd7"
                            "\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                            "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                            "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                            "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
                            "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
                            "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
                            "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
        .digest         = "\xe1\x99\x21\x90\x54\x9f\x6e\xd5"
                            "\x69\x6a\x2c\x05\x6c\x31\x54\x10",
        .psize          = 64,
        .ksize          = 32,
        .dsize          = 16,
        .alg            = SCE_MAC_AES,
        .mode           = SCE_HMAC,
    }, {
        .key            = "\xff\xee\xdd\xcc\xbb\xaa\x99\x88"
                            "\x77\x66\x55\x44\x33\x22\x11\x00",
        .plaintext      = "\x01\x23\x45\x67\x89\xab\xcd\xef"
                            "\xfe\xdc\xba\x98\x76\x54\x32\x10",
        .digest         = "\x00\xd4\x63\xb4\x9a\xf3\x52\xe2"
                            "\x74\xa9\x00\x55\x13\x54\x2a\xd1",
        .psize          = 16,
        .ksize          = 16,
        .dsize          = 16,
        .alg            = SCE_MAC_SM4,
        .mode           = SCE_HMAC,
    }, {
        .key            = "\x01\x23\x45\x67\x89\xab\xcd\xef"
                            "\xfe\xdc\xBA\x98\x76\x54\x32\x10",
        .plaintext      = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
                            "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb"
                            "\xcc\xcc\xcc\xcc\xcc\xcc\xcc\xcc"
                            "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                            "\xee",
        .digest         = "\x8a\x8a\xe9\xc0\xc8\x97\x0e\x85"
                            "\x21\x57\x02\x10\x1a\xbf\x9c\xc6",
        .psize          = 33,
        .ksize          = 16,
        .dsize          = 16,
        .alg            = SCE_MAC_SM4,
        .mode           = SCE_HMAC,
    },
    {
        .key            = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                            "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                            "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96",
        .digest         = "\x8e\x8f\x29\x31\x36\x28\x37\x97",
        .psize          = 8,
        .ksize          = 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    }, {
        .key            = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                            "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                            "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                            "\xae\x2d\x8a\x57",
        .digest         = "\x74\x3d\xdb\xe0\xce\x2d\xc2\xed",
        .psize          = 20,
        .ksize          = 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\xE4\x23\x33\x8A\x35\x64\x61\xE2\x49\x03\xDD\xC6\xB8\xCA\x55\x7A" 
                    "\xd0\xe7\x4b\xfb\x5d\xe5\x0c\xe7",
        .plaintext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93" 
                    "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
        .digest		= "\x29\x8D\x1B\x72\xD8\xBD\x42\xCA",
        .psize		= 32,
        .ksize		= 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                    "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                    "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .digest		= "\xb7\xa6\x88\xe1\x22\xff\xaf\x95",
        .psize		= 0,
        .ksize		= 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    }, 
    {
        .key		= "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                    "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                    "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .plaintext	= "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                    "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                    "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                    "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51",
        .digest		= "\x33\xe6\xb1\x09\x24\x00\xea\xe5",
        .psize		= 32,
        .ksize		= 24,
        .dsize          = 8,
        .alg            = SCE_MAC_DES3_EDE,
        .mode           = SCE_HMAC,
    },
    {
        .key            = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
        .plaintext      = "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
                            "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
        .digest         = "\x1B\x57\xDD\xBD\xE6\xA4\x44\x70",
        .psize          = 32,
        .ksize          = 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
        .plaintext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93" 
                    "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
                    "\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4" 
                    "\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b",
        .digest		= "\x65\x20\xB3\xAE\xAB\x60\x7B\xEB",
        .psize		= 64,
        .ksize		= 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
        .plaintext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93" 
                    "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
                    "\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4" 
                    "\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
                    "\x59\xda\x19\x4f\xec\x07\x8e\xe2\xf0\x61\xf9\x27\x61\x6f\xf8\xdf" 
                    "\x62\x4d\xaf\x06\xfe\x41\xa6\xa6\xf9\xa2\x06\x40\xb3\x04\xbd\xe6" 
                    "\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e\xa6\x58\xdc\x91\x17\x58\x42\x95" 
                    "\xa3\x7c\x81\x78\xa6\x3d\x3f\x75\x74\x17\x1a\xd3\x6c\x2f\x48\x39",
        .digest		= "\xB8\xEC\x22\x52\x22\x9A\xDA\x62",
        .psize		= 128,
        .ksize		= 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key		= "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
        .plaintext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93" 
                    "\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
                    "\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4" 
                    "\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
                    "\x59\xda\x19\x4f\xec\x07\x8e\xe2\xf0\x61\xf9\x27\x61\x6f\xf8\xdf" 
                    "\x62\x4d\xaf\x06\xfe\x41\xa6\xa6\xf9\xa2\x06\x40\xb3\x04\xbd\xe6" 
                    "\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e\xa6\x58\xdc\x91\x17\x58\x42\x95" 
                    "\xa3\x7c\x81\x78\xa6\x3d\x3f\x75\x74\x17\x1a\xd3\x6c\x2f\x48\x39"
                    "\x20\x20\xc1\x9a\x29\x84\x7d\x2d\x52\xa1\xf9\x5c\xf3\x4f\x91\xfc"
                    "\x75\xcf\xd6\x2d\xe7\x9a\x59\x6e\x00\x0e\x8d\x22\x17\xbd\xa0\xdd" 
                    "\x79\x1f\x71\xe6\xcd\x2f\xb1\xb6\xbc\xc3\xdb\x02\x91\x41\x9b\x09"
                    "\xa9\xd2\x7e\xbd\x2c\x18\xae\xc0\x93\x0c\x02\x9a\xdb\x4e\xaa\xeb" 
                    "\x84\x4b\x43\x5e\xf0\x98\xf2\x5f\x86\x70\x96\x90\x15\x30\xcf\x3a" 
                    "\xc9\x33\x21\xec\x59\x86\xfc\x65\x7d\xbe\xb9\xf8\x97\xf9\x30\xa9" 
                    "\x6d\xfc\x0c\x6e\x36\x67\xd5\xa6\x67\xd9\xbd\x9b\x34\x5d\xa7\xdd" 
                    "\xda\x46\x33\x25\x60\x4a\x18\xf1\x55\x07\xb2\xb7\x26\x7b\xa6\x1e", 
        .digest		= "\xDE\xF1\x05\x9E\xBD\x6D\x7B\x37",
        .psize		= 256,
        .ksize		= 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key            = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96",
        .digest         = "\x20\x37\x34\xc0\x22\xb2\x26\xc8",
        .psize          = 8,
        .ksize          = 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },
    {
        .key            = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62",
        .plaintext      = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                            "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                            "\xae\x2d\x8a\x57",
        .digest         = "\xe3\xa8\xdd\x10\x1a\x7b\xcb\xb5",
        .psize          = 20,
        .ksize          = 8,
        .dsize          = 8,
        .alg            = SCE_MAC_DES,
        .mode           = SCE_HMAC,
    },


};


static hash_testvec_t hash_tv_template[] = {
    {
        .psize = 0,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_NORMAL,
        .plaintext = "",
        .digest = "\xe3\xb0\xc4\x42\x98\xfc\x1c\x14"
                  "\x9a\xfb\xf4\xc8\x99\x6f\xb9\x24"
                  "\x27\xae\x41\xe4\x64\x9b\x93\x4c"
                  "\xa4\x95\x99\x1b\x78\x52\xb8\x55",
    },
    {
        .psize = 3,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_NORMAL,
        .plaintext = "abc",
        .digest = "\xba\x78\x16\xbf\x8f\x01\xcf\xea"
                  "\x41\x41\x40\xde\x5d\xae\x22\x23"
                  "\xb0\x03\x61\xa3\x96\x17\x7a\x9c"
                  "\xb4\x10\xff\x61\xf2\x00\x15\xad",
    },
    {
        .psize = 56,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_NORMAL,
        .plaintext = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
        .digest = "\x24\x8d\x6a\x61\xd2\x06\x38\xb8"
                  "\xe5\xc0\x26\x93\x0c\x3e\x60\x39"
                  "\xa3\x3c\xe4\x59\x64\xff\x21\x67"
                  "\xf6\xec\xed\xd4\x19\xdb\x06\xc1",
    },
    {
        .psize = 64,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_NORMAL,
        .plaintext =
            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-",
        .digest = "\xb5\xfe\xad\x56\x7d\xff\xcb\xa4"
                  "\x2c\x32\x29\x32\x19\xbb\xfb\xfa"
                  "\xd6\xff\x94\xa3\x72\x91\x85\x66"
                  "\x3b\xa7\x87\x77\x58\xa3\x40\x3a",
    },
    {
        .psize = 1023,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_NORMAL,
        .plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
                     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
                     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
                     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
                     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
                     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
                     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
                     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
                     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
                     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
                     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
                     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
                     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
                     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
                     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
                     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
                     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
                     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
                     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
                     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
                     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
                     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
                     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
                     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
                     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
                     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
                     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
                     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
                     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
                     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
                     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
                     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
                     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
                     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
                     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
                     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
                     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
                     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
                     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
                     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
                     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
                     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
                     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
                     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
                     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
                     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
                     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
                     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
                     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
                     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
                     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
                     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
                     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
                     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
                     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
                     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
                     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
                     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
                     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
                     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
                     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
                     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
                     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
                     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
                     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
                     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
                     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
                     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
                     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
                     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
                     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
                     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
                     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
                     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
                     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
                     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
                     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
                     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
                     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
                     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
                     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
                     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
                     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
                     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
                     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
                     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
                     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
                     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
                     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
                     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
                     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
                     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
                     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
                     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
                     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
                     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
                     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
                     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
                     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
                     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
                     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
                     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
                     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
                     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
                     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
                     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
                     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
                     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
                     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
                     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
                     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
                     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
                     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
                     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
                     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
                     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
                     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
                     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
                     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
                     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
                     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
                     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
                     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
                     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
                     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
                     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
                     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
                     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
        .digest = "\xc5\xce\x0c\xca\x01\x4f\x53\x3a"
                  "\x32\x32\x17\xcc\xd4\x6a\x71\xa9"
                  "\xf3\xed\x50\x10\x64\x8e\x06\xbe"
                  "\x9b\x4a\xa6\xbb\x05\x89\x59\x51",
    },

    {
        .psize = 0,
        .dsize = 48,
        .alg = SCE_SHA384,
        .mode = SCE_NORMAL,
        .plaintext = "",
        .psize = 0,
        .digest = "\x38\xb0\x60\xa7\x51\xac\x96\x38"
                  "\x4c\xd9\x32\x7e\xb1\xb1\xe3\x6a"
                  "\x21\xfd\xb7\x11\x14\xbe\x07\x43"
                  "\x4c\x0c\xc7\xbf\x63\xf6\xe1\xda"
                  "\x27\x4e\xde\xbf\xe7\x6f\x65\xfb"
                  "\xd5\x1a\xd2\xf1\x48\x98\xb9\x5b",
    },
    {
        .psize = 3,
        .dsize = 48,
        .alg = SCE_SHA384,
        .mode = SCE_NORMAL,
        .plaintext = "abc",
        .digest = "\xcb\x00\x75\x3f\x45\xa3\x5e\x8b"
                  "\xb5\xa0\x3d\x69\x9a\xc6\x50\x07"
                  "\x27\x2c\x32\xab\x0e\xde\xd1\x63"
                  "\x1a\x8b\x60\x5a\x43\xff\x5b\xed"
                  "\x80\x86\x07\x2b\xa1\xe7\xcc\x23"
                  "\x58\xba\xec\xa1\x34\xc8\x25\xa7",
    },
    {
        .psize = 56,
        .dsize = 48,
        .alg = SCE_SHA384,
        .mode = SCE_NORMAL,
        .plaintext = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
        .digest = "\x33\x91\xfd\xdd\xfc\x8d\xc7\x39"
                  "\x37\x07\xa6\x5b\x1b\x47\x09\x39"
                  "\x7c\xf8\xb1\xd1\x62\xaf\x05\xab"
                  "\xfe\x8f\x45\x0d\xe5\xf3\x6b\xc6"
                  "\xb0\x45\x5a\x85\x20\xbc\x4e\x6f"
                  "\x5f\xe9\x5b\x1f\xe3\xc8\x45\x2b",
    },
    {
        .psize = 112,
        .dsize = 48,
        .alg = SCE_SHA384,
        .mode = SCE_NORMAL,
        .plaintext = "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn"
                     "hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu",
        .digest = "\x09\x33\x0c\x33\xf7\x11\x47\xe8"
                  "\x3d\x19\x2f\xc7\x82\xcd\x1b\x47"
                  "\x53\x11\x1b\x17\x3b\x3b\x05\xd2"
                  "\x2f\xa0\x80\x86\xe3\xb0\xf7\x12"
                  "\xfc\xc7\xc7\x1a\x55\x7e\x2d\xb9"
                  "\x66\xc3\xe9\xfa\x91\x74\x60\x39",
    },
    {
        .psize = 104,
        .dsize = 48,
        .alg = SCE_SHA384,
        .mode = SCE_NORMAL,
        .plaintext = "abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcd"
                     "efghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyz",
        .digest = "\x3d\x20\x89\x73\xab\x35\x08\xdb"
                  "\xbd\x7e\x2c\x28\x62\xba\x29\x0a"
                  "\xd3\x01\x0e\x49\x78\xc1\x98\xdc"
                  "\x4d\x8f\xd0\x14\xe5\x82\x82\x3a"
                  "\x89\xe1\x6f\x9b\x2a\x7b\xbc\x1a"
                  "\xc9\x38\xe2\xd1\x99\xe8\xbe\xa4",
    },
    {
        .psize = 1023,
        .dsize = 48,
        .alg = SCE_SHA384,
        .mode = SCE_NORMAL,
        .plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
                     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
                     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
                     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
                     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
                     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
                     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
                     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
                     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
                     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
                     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
                     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
                     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
                     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
                     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
                     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
                     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
                     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
                     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
                     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
                     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
                     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
                     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
                     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
                     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
                     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
                     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
                     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
                     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
                     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
                     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
                     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
                     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
                     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
                     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
                     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
                     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
                     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
                     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
                     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
                     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
                     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
                     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
                     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
                     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
                     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
                     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
                     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
                     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
                     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
                     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
                     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
                     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
                     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
                     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
                     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
                     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
                     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
                     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
                     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
                     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
                     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
                     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
                     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
                     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
                     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
                     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
                     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
                     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
                     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
                     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
                     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
                     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
                     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
                     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
                     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
                     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
                     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
                     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
                     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
                     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
                     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
                     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
                     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
                     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
                     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
                     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
                     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
                     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
                     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
                     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
                     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
                     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
                     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
                     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
                     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
                     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
                     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
                     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
                     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
                     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
                     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
                     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
                     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
                     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
                     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
                     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
                     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
                     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
                     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
                     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
                     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
                     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
                     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
                     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
                     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
                     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
                     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
                     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
                     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
                     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
                     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
                     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
                     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
                     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
                     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
                     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
                     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
        .digest = "\x4d\x97\x23\xc8\xea\x7a\x7c\x15"
                  "\xb8\xff\x97\x9c\xf5\x13\x4f\x31"
                  "\xde\x67\xf7\x24\x73\xcd\x70\x1c"
                  "\x03\x4a\xba\x8a\x87\x49\xfe\xdc"
                  "\x75\x29\x62\x83\xae\x3f\x17\xab"
                  "\xfd\x10\x4d\x8e\x17\x1c\x1f\xca",
    },

    {
        .psize = 0,
        .dsize = 64,
        .alg = SCE_SHA512,
        .mode = SCE_NORMAL,
        .plaintext = "",
        .digest = "\xcf\x83\xe1\x35\x7e\xef\xb8\xbd"
                  "\xf1\x54\x28\x50\xd6\x6d\x80\x07"
                  "\xd6\x20\xe4\x05\x0b\x57\x15\xdc"
                  "\x83\xf4\xa9\x21\xd3\x6c\xe9\xce"
                  "\x47\xd0\xd1\x3c\x5d\x85\xf2\xb0"
                  "\xff\x83\x18\xd2\x87\x7e\xec\x2f"
                  "\x63\xb9\x31\xbd\x47\x41\x7a\x81"
                  "\xa5\x38\x32\x7a\xf9\x27\xda\x3e",
    },
    {
        .psize = 3,
        .dsize = 64,
        .alg = SCE_SHA512,
        .mode = SCE_NORMAL,
        .plaintext = "abc",
        .digest = "\xdd\xaf\x35\xa1\x93\x61\x7a\xba"
                  "\xcc\x41\x73\x49\xae\x20\x41\x31"
                  "\x12\xe6\xfa\x4e\x89\xa9\x7e\xa2"
                  "\x0a\x9e\xee\xe6\x4b\x55\xd3\x9a"
                  "\x21\x92\x99\x2a\x27\x4f\xc1\xa8"
                  "\x36\xba\x3c\x23\xa3\xfe\xeb\xbd"
                  "\x45\x4d\x44\x23\x64\x3c\xe8\x0e"
                  "\x2a\x9a\xc9\x4f\xa5\x4c\xa4\x9f",
    },
    {
        .psize = 56,
        .dsize = 64,
        .alg = SCE_SHA512,
        .mode = SCE_NORMAL,
        .plaintext = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
        .digest = "\x20\x4a\x8f\xc6\xdd\xa8\x2f\x0a"
                  "\x0c\xed\x7b\xeb\x8e\x08\xa4\x16"
                  "\x57\xc1\x6e\xf4\x68\xb2\x28\xa8"
                  "\x27\x9b\xe3\x31\xa7\x03\xc3\x35"
                  "\x96\xfd\x15\xc1\x3b\x1b\x07\xf9"
                  "\xaa\x1d\x3b\xea\x57\x78\x9c\xa0"
                  "\x31\xad\x85\xc7\xa7\x1d\xd7\x03"
                  "\x54\xec\x63\x12\x38\xca\x34\x45",
    },
    {
        .psize = 112,
        .dsize = 64,
        .alg = SCE_SHA512,
        .mode = SCE_NORMAL,
        .plaintext = "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn"
                     "hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu",
        .digest = "\x8e\x95\x9b\x75\xda\xe3\x13\xda"
                  "\x8c\xf4\xf7\x28\x14\xfc\x14\x3f"
                  "\x8f\x77\x79\xc6\xeb\x9f\x7f\xa1"
                  "\x72\x99\xae\xad\xb6\x88\x90\x18"
                  "\x50\x1d\x28\x9e\x49\x00\xf7\xe4"
                  "\x33\x1b\x99\xde\xc4\xb5\x43\x3a"
                  "\xc7\xd3\x29\xee\xb6\xdd\x26\x54"
                  "\x5e\x96\xe5\x5b\x87\x4b\xe9\x09",
    },
    {
        .psize = 104,
        .dsize = 64,
        .alg = SCE_SHA512,
        .mode = SCE_NORMAL,
        .plaintext = "abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcd"
                     "efghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyz",
        .digest = "\x93\x0d\x0c\xef\xcb\x30\xff\x11"
                  "\x33\xb6\x89\x81\x21\xf1\xcf\x3d"
                  "\x27\x57\x8a\xfc\xaf\xe8\x67\x7c"
                  "\x52\x57\xcf\x06\x99\x11\xf7\x5d"
                  "\x8f\x58\x31\xb5\x6e\xbf\xda\x67"
                  "\xb2\x78\xe6\x6d\xff\x8b\x84\xfe"
                  "\x2b\x28\x70\xf7\x42\xa5\x80\xd8"
                  "\xed\xb4\x19\x87\x23\x28\x50\xc9",
    },
    {
        .psize = 1023,
        .dsize = 64,
        .alg = SCE_SHA512,
        .mode = SCE_NORMAL,
        .plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
                     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
                     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
                     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
                     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
                     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
                     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
                     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
                     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
                     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
                     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
                     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
                     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
                     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
                     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
                     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
                     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
                     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
                     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
                     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
                     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
                     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
                     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
                     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
                     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
                     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
                     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
                     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
                     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
                     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
                     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
                     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
                     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
                     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
                     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
                     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
                     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
                     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
                     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
                     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
                     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
                     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
                     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
                     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
                     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
                     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
                     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
                     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
                     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
                     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
                     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
                     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
                     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
                     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
                     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
                     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
                     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
                     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
                     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
                     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
                     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
                     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
                     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
                     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
                     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
                     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
                     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
                     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
                     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
                     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
                     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
                     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
                     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
                     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
                     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
                     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
                     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
                     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
                     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
                     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
                     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
                     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
                     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
                     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
                     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
                     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
                     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
                     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
                     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
                     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
                     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
                     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
                     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
                     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
                     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
                     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
                     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
                     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
                     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
                     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
                     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
                     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
                     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
                     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
                     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
                     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
                     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
                     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
                     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
                     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
                     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
                     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
                     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
                     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
                     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
                     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
                     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
                     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
                     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
                     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
                     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
                     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
                     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
                     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
                     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
                     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
                     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
                     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
        .digest = "\x76\xc9\xd4\x91\x7a\x5f\x0f\xaa"
                  "\x13\x39\xf3\x01\x7a\xfa\xe5\x41"
                  "\x5f\x0b\xf8\xeb\x32\xfc\xbf\xb0"
                  "\xfa\x8c\xcd\x17\x83\xe2\xfa\xeb"
                  "\x1c\x19\xde\xe2\x75\xdc\x34\x64"
                  "\x5f\x35\x9c\x61\x2f\x10\xf9\xec"
                  "\x59\xca\x9d\xcc\x25\x0c\x43\xba"
                  "\x85\xa8\xf8\xfe\xb5\x24\xb2\xee",
    },

    {
        .psize = 0,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_NORMAL,
        .plaintext = "",
        .digest = "\xda\x39\xa3\xee\x5e\x6b\x4b\x0d\x32\x55"
                  "\xbf\xef\x95\x60\x18\x90\xaf\xd8\x07\x09",
    },
    {
        .psize = 3,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_NORMAL,
        .plaintext = "abc",
        .psize = 3,
        .digest = "\xa9\x99\x3e\x36\x47\x06\x81\x6a\xba\x3e"
                  "\x25\x71\x78\x50\xc2\x6c\x9c\xd0\xd8\x9d",
    },
    {
        .psize = 56,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_NORMAL,
        .plaintext = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
        .psize = 56,
        .digest = "\x84\x98\x3e\x44\x1c\x3b\xd2\x6e\xba\xae"
                  "\x4a\xa1\xf9\x51\x29\xe5\xe5\x46\x70\xf1",
    },
    {
        .psize = 163,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_NORMAL,
        .plaintext = "\xec\x29\x56\x12\x44\xed\xe7\x06"
                     "\xb6\xeb\x30\xa1\xc3\x71\xd7\x44"
                     "\x50\xa1\x05\xc3\xf9\x73\x5f\x7f"
                     "\xa9\xfe\x38\xcf\x67\xf3\x04\xa5"
                     "\x73\x6a\x10\x6e\x92\xe1\x71\x39"
                     "\xa6\x81\x3b\x1c\x81\xa4\xf3\xd3"
                     "\xfb\x95\x46\xab\x42\x96\xfa\x9f"
                     "\x72\x28\x26\xc0\x66\x86\x9e\xda"
                     "\xcd\x73\xb2\x54\x80\x35\x18\x58"
                     "\x13\xe2\x26\x34\xa9\xda\x44\x00"
                     "\x0d\x95\xa2\x81\xff\x9f\x26\x4e"
                     "\xcc\xe0\xa9\x31\x22\x21\x62\xd0"
                     "\x21\xcc\xa2\x8d\xb5\xf3\xc2\xaa"
                     "\x24\x94\x5a\xb1\xe3\x1c\xb4\x13"
                     "\xae\x29\x81\x0f\xd7\x94\xca\xd5"
                     "\xdf\xaf\x29\xec\x43\xcb\x38\xd1"
                     "\x98\xfe\x4a\xe1\xda\x23\x59\x78"
                     "\x02\x21\x40\x5b\xd6\x71\x2a\x53"
                     "\x05\xda\x4b\x1b\x73\x7f\xce\x7c"
                     "\xd2\x1c\x0e\xb7\x72\x8d\x08\x23"
                     "\x5a\x90\x11",
        .digest = "\x97\x01\x11\xc4\xe7\x7b\xcc\x88\xcc\x20"
                  "\x45\x9c\x02\xb6\x9b\x4a\xa8\xf5\x82\x17",
    },
    {
        .psize = 64,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_NORMAL,
        .plaintext =
            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-",
        .digest = "\xc8\x71\xf6\x9a\x63\xcc\xa9\x84\x84\x82"
                  "\x64\xe7\x79\x95\x5d\xd7\x19\x41\x7c\x91",
    },
    {
        .psize = 1023,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_NORMAL,
        .plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
                     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
                     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
                     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
                     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
                     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
                     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
                     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
                     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
                     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
                     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
                     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
                     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
                     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
                     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
                     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
                     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
                     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
                     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
                     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
                     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
                     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
                     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
                     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
                     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
                     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
                     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
                     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
                     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
                     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
                     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
                     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
                     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
                     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
                     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
                     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
                     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
                     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
                     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
                     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
                     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
                     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
                     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
                     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
                     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
                     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
                     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
                     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
                     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
                     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
                     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
                     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
                     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
                     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
                     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
                     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
                     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
                     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
                     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
                     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
                     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
                     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
                     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
                     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
                     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
                     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
                     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
                     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
                     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
                     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
                     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
                     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
                     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
                     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
                     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
                     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
                     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
                     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
                     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
                     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
                     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
                     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
                     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
                     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
                     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
                     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
                     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
                     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
                     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
                     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
                     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
                     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
                     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
                     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
                     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
                     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
                     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
                     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
                     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
                     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
                     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
                     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
                     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
                     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
                     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
                     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
                     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
                     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
                     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
                     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
                     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
                     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
                     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
                     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
                     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
                     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
                     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
                     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
                     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
                     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
                     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
                     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
                     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
                     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
                     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
                     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
                     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
                     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
        .digest = "\xb8\xe3\x54\xed\xc5\xfc\xef\xa4"
                  "\x55\x73\x4a\x81\x99\xe4\x47\x2a"
                  "\x30\xd6\xc9\x85",
    },

    {
        .psize = 0,
        .dsize = 28,
        .alg = SCE_SHA224,
        .mode = SCE_NORMAL,
        .plaintext = "",
        .digest = "\xd1\x4a\x02\x8c\x2a\x3a\x2b\xc9"
                  "\x47\x61\x02\xbb\x28\x82\x34\xc4"
                  "\x15\xa2\xb0\x1f\x82\x8e\xa6\x2a"
                  "\xc5\xb3\xe4\x2f",
    },
    {
        .psize = 3,
        .dsize = 28,
        .alg = SCE_SHA224,
        .mode = SCE_NORMAL,
        .plaintext = "abc",
        .digest = "\x23\x09\x7D\x22\x34\x05\xD8\x22"
                  "\x86\x42\xA4\x77\xBD\xA2\x55\xB3"
                  "\x2A\xAD\xBC\xE4\xBD\xA0\xB3\xF7"
                  "\xE3\x6C\x9D\xA7",
    },
    {
        .psize = 56,
        .dsize = 28,
        .alg = SCE_SHA224,
        .mode = SCE_NORMAL,
        .plaintext = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
        .digest = "\x75\x38\x8B\x16\x51\x27\x76\xCC"
                  "\x5D\xBA\x5D\xA1\xFD\x89\x01\x50"
                  "\xB0\xC6\x45\x5C\xB4\xF5\x8B\x19"
                  "\x52\x52\x25\x25",
    },
    {
        .psize = 64,
        .dsize = 28,
        .alg = SCE_SHA224,
        .mode = SCE_NORMAL,
        .plaintext =
            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-",
        .digest = "\xc4\xdb\x2b\x3a\x58\xc3\x99\x01"
                  "\x42\xfd\x10\x92\xaa\x4e\x04\x08"
                  "\x58\xbb\xbb\xe8\xf8\x14\xa7\x0c"
                  "\xef\x3b\xcb\x0e",
    },
    {
        .psize = 1023,
        .dsize = 28,
        .alg = SCE_SHA224,
        .mode = SCE_NORMAL,
        .plaintext = "\x08\x9f\x13\xaa\x41\xd8\x4c\xe3"
                     "\x7a\x11\x85\x1c\xb3\x27\xbe\x55"
                     "\xec\x60\xf7\x8e\x02\x99\x30\xc7"
                     "\x3b\xd2\x69\x00\x74\x0b\xa2\x16"
                     "\xad\x44\xdb\x4f\xe6\x7d\x14\x88"
                     "\x1f\xb6\x2a\xc1\x58\xef\x63\xfa"
                     "\x91\x05\x9c\x33\xca\x3e\xd5\x6c"
                     "\x03\x77\x0e\xa5\x19\xb0\x47\xde"
                     "\x52\xe9\x80\x17\x8b\x22\xb9\x2d"
                     "\xc4\x5b\xf2\x66\xfd\x94\x08\x9f"
                     "\x36\xcd\x41\xd8\x6f\x06\x7a\x11"
                     "\xa8\x1c\xb3\x4a\xe1\x55\xec\x83"
                     "\x1a\x8e\x25\xbc\x30\xc7\x5e\xf5"
                     "\x69\x00\x97\x0b\xa2\x39\xd0\x44"
                     "\xdb\x72\x09\x7d\x14\xab\x1f\xb6"
                     "\x4d\xe4\x58\xef\x86\x1d\x91\x28"
                     "\xbf\x33\xca\x61\xf8\x6c\x03\x9a"
                     "\x0e\xa5\x3c\xd3\x47\xde\x75\x0c"
                     "\x80\x17\xae\x22\xb9\x50\xe7\x5b"
                     "\xf2\x89\x20\x94\x2b\xc2\x36\xcd"
                     "\x64\xfb\x6f\x06\x9d\x11\xa8\x3f"
                     "\xd6\x4a\xe1\x78\x0f\x83\x1a\xb1"
                     "\x25\xbc\x53\xea\x5e\xf5\x8c\x00"
                     "\x97\x2e\xc5\x39\xd0\x67\xfe\x72"
                     "\x09\xa0\x14\xab\x42\xd9\x4d\xe4"
                     "\x7b\x12\x86\x1d\xb4\x28\xbf\x56"
                     "\xed\x61\xf8\x8f\x03\x9a\x31\xc8"
                     "\x3c\xd3\x6a\x01\x75\x0c\xa3\x17"
                     "\xae\x45\xdc\x50\xe7\x7e\x15\x89"
                     "\x20\xb7\x2b\xc2\x59\xf0\x64\xfb"
                     "\x92\x06\x9d\x34\xcb\x3f\xd6\x6d"
                     "\x04\x78\x0f\xa6\x1a\xb1\x48\xdf"
                     "\x53\xea\x81\x18\x8c\x23\xba\x2e"
                     "\xc5\x5c\xf3\x67\xfe\x95\x09\xa0"
                     "\x37\xce\x42\xd9\x70\x07\x7b\x12"
                     "\xa9\x1d\xb4\x4b\xe2\x56\xed\x84"
                     "\x1b\x8f\x26\xbd\x31\xc8\x5f\xf6"
                     "\x6a\x01\x98\x0c\xa3\x3a\xd1\x45"
                     "\xdc\x73\x0a\x7e\x15\xac\x20\xb7"
                     "\x4e\xe5\x59\xf0\x87\x1e\x92\x29"
                     "\xc0\x34\xcb\x62\xf9\x6d\x04\x9b"
                     "\x0f\xa6\x3d\xd4\x48\xdf\x76\x0d"
                     "\x81\x18\xaf\x23\xba\x51\xe8\x5c"
                     "\xf3\x8a\x21\x95\x2c\xc3\x37\xce"
                     "\x65\xfc\x70\x07\x9e\x12\xa9\x40"
                     "\xd7\x4b\xe2\x79\x10\x84\x1b\xb2"
                     "\x26\xbd\x54\xeb\x5f\xf6\x8d\x01"
                     "\x98\x2f\xc6\x3a\xd1\x68\xff\x73"
                     "\x0a\xa1\x15\xac\x43\xda\x4e\xe5"
                     "\x7c\x13\x87\x1e\xb5\x29\xc0\x57"
                     "\xee\x62\xf9\x90\x04\x9b\x32\xc9"
                     "\x3d\xd4\x6b\x02\x76\x0d\xa4\x18"
                     "\xaf\x46\xdd\x51\xe8\x7f\x16\x8a"
                     "\x21\xb8\x2c\xc3\x5a\xf1\x65\xfc"
                     "\x93\x07\x9e\x35\xcc\x40\xd7\x6e"
                     "\x05\x79\x10\xa7\x1b\xb2\x49\xe0"
                     "\x54\xeb\x82\x19\x8d\x24\xbb\x2f"
                     "\xc6\x5d\xf4\x68\xff\x96\x0a\xa1"
                     "\x38\xcf\x43\xda\x71\x08\x7c\x13"
                     "\xaa\x1e\xb5\x4c\xe3\x57\xee\x85"
                     "\x1c\x90\x27\xbe\x32\xc9\x60\xf7"
                     "\x6b\x02\x99\x0d\xa4\x3b\xd2\x46"
                     "\xdd\x74\x0b\x7f\x16\xad\x21\xb8"
                     "\x4f\xe6\x5a\xf1\x88\x1f\x93\x2a"
                     "\xc1\x35\xcc\x63\xfa\x6e\x05\x9c"
                     "\x10\xa7\x3e\xd5\x49\xe0\x77\x0e"
                     "\x82\x19\xb0\x24\xbb\x52\xe9\x5d"
                     "\xf4\x8b\x22\x96\x2d\xc4\x38\xcf"
                     "\x66\xfd\x71\x08\x9f\x13\xaa\x41"
                     "\xd8\x4c\xe3\x7a\x11\x85\x1c\xb3"
                     "\x27\xbe\x55\xec\x60\xf7\x8e\x02"
                     "\x99\x30\xc7\x3b\xd2\x69\x00\x74"
                     "\x0b\xa2\x16\xad\x44\xdb\x4f\xe6"
                     "\x7d\x14\x88\x1f\xb6\x2a\xc1\x58"
                     "\xef\x63\xfa\x91\x05\x9c\x33\xca"
                     "\x3e\xd5\x6c\x03\x77\x0e\xa5\x19"
                     "\xb0\x47\xde\x52\xe9\x80\x17\x8b"
                     "\x22\xb9\x2d\xc4\x5b\xf2\x66\xfd"
                     "\x94\x08\x9f\x36\xcd\x41\xd8\x6f"
                     "\x06\x7a\x11\xa8\x1c\xb3\x4a\xe1"
                     "\x55\xec\x83\x1a\x8e\x25\xbc\x30"
                     "\xc7\x5e\xf5\x69\x00\x97\x0b\xa2"
                     "\x39\xd0\x44\xdb\x72\x09\x7d\x14"
                     "\xab\x1f\xb6\x4d\xe4\x58\xef\x86"
                     "\x1d\x91\x28\xbf\x33\xca\x61\xf8"
                     "\x6c\x03\x9a\x0e\xa5\x3c\xd3\x47"
                     "\xde\x75\x0c\x80\x17\xae\x22\xb9"
                     "\x50\xe7\x5b\xf2\x89\x20\x94\x2b"
                     "\xc2\x36\xcd\x64\xfb\x6f\x06\x9d"
                     "\x11\xa8\x3f\xd6\x4a\xe1\x78\x0f"
                     "\x83\x1a\xb1\x25\xbc\x53\xea\x5e"
                     "\xf5\x8c\x00\x97\x2e\xc5\x39\xd0"
                     "\x67\xfe\x72\x09\xa0\x14\xab\x42"
                     "\xd9\x4d\xe4\x7b\x12\x86\x1d\xb4"
                     "\x28\xbf\x56\xed\x61\xf8\x8f\x03"
                     "\x9a\x31\xc8\x3c\xd3\x6a\x01\x75"
                     "\x0c\xa3\x17\xae\x45\xdc\x50\xe7"
                     "\x7e\x15\x89\x20\xb7\x2b\xc2\x59"
                     "\xf0\x64\xfb\x92\x06\x9d\x34\xcb"
                     "\x3f\xd6\x6d\x04\x78\x0f\xa6\x1a"
                     "\xb1\x48\xdf\x53\xea\x81\x18\x8c"
                     "\x23\xba\x2e\xc5\x5c\xf3\x67\xfe"
                     "\x95\x09\xa0\x37\xce\x42\xd9\x70"
                     "\x07\x7b\x12\xa9\x1d\xb4\x4b\xe2"
                     "\x56\xed\x84\x1b\x8f\x26\xbd\x31"
                     "\xc8\x5f\xf6\x6a\x01\x98\x0c\xa3"
                     "\x3a\xd1\x45\xdc\x73\x0a\x7e\x15"
                     "\xac\x20\xb7\x4e\xe5\x59\xf0\x87"
                     "\x1e\x92\x29\xc0\x34\xcb\x62\xf9"
                     "\x6d\x04\x9b\x0f\xa6\x3d\xd4\x48"
                     "\xdf\x76\x0d\x81\x18\xaf\x23\xba"
                     "\x51\xe8\x5c\xf3\x8a\x21\x95\x2c"
                     "\xc3\x37\xce\x65\xfc\x70\x07\x9e"
                     "\x12\xa9\x40\xd7\x4b\xe2\x79\x10"
                     "\x84\x1b\xb2\x26\xbd\x54\xeb\x5f"
                     "\xf6\x8d\x01\x98\x2f\xc6\x3a\xd1"
                     "\x68\xff\x73\x0a\xa1\x15\xac\x43"
                     "\xda\x4e\xe5\x7c\x13\x87\x1e\xb5"
                     "\x29\xc0\x57\xee\x62\xf9\x90\x04"
                     "\x9b\x32\xc9\x3d\xd4\x6b\x02\x76"
                     "\x0d\xa4\x18\xaf\x46\xdd\x51\xe8"
                     "\x7f\x16\x8a\x21\xb8\x2c\xc3\x5a"
                     "\xf1\x65\xfc\x93\x07\x9e\x35\xcc"
                     "\x40\xd7\x6e\x05\x79\x10\xa7\x1b"
                     "\xb2\x49\xe0\x54\xeb\x82\x19\x8d"
                     "\x24\xbb\x2f\xc6\x5d\xf4\x68\xff"
                     "\x96\x0a\xa1\x38\xcf\x43\xda\x71"
                     "\x08\x7c\x13\xaa\x1e\xb5\x4c",
        .digest = "\x98\x43\x07\x63\x75\xe0\xa7\x1c"
                  "\x78\xb1\x8b\xfd\x04\xf5\x2d\x91"
                  "\x20\x48\xa4\x28\xff\x55\xb1\xd3"
                  "\xe6\xf9\x4f\xcc",
    },
};

static hmac_testvec_t hmac_tv_template[] = {
    {
        .ksize = 32,
        .psize = 3,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_HMAC,
        .key = "\x01\x02\x03\x04\x05\x06\x07\x08"
               "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
               "\x11\x12\x13\x14\x15\x16\x17\x18"
               "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20",
        .plaintext = "abc",
        .digest = "\xa2\x1b\x1f\x5d\x4c\xf4\xf7\x3a"
                  "\x4d\xd9\x39\x75\x0f\x7a\x06\x6a"
                  "\x7f\x98\xcc\x13\x1c\xb1\x6a\x66"
                  "\x92\x75\x90\x21\xcf\xab\x81\x81",
    },
    {
        .ksize = 32,
        .psize = 56,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_HMAC,
        .key = "\x01\x02\x03\x04\x05\x06\x07\x08"
               "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
               "\x11\x12\x13\x14\x15\x16\x17\x18"
               "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20",
        .plaintext = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
        .digest = "\x10\x4f\xdc\x12\x57\x32\x8f\x08"
                  "\x18\x4b\xa7\x31\x31\xc5\x3c\xae"
                  "\xe6\x98\xe3\x61\x19\x42\x11\x49"
                  "\xea\x8c\x71\x24\x56\x69\x7d\x30",
    },
    {
        .ksize = 32,
        .psize = 112,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_HMAC,
        .key = "\x01\x02\x03\x04\x05\x06\x07\x08"
               "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
               "\x11\x12\x13\x14\x15\x16\x17\x18"
               "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20",
        .plaintext = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
                     "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
        .digest = "\x47\x03\x05\xfc\x7e\x40\xfe\x34"
                  "\xd3\xee\xb3\xe7\x73\xd9\x5a\xab"
                  "\x73\xac\xf0\xfd\x06\x04\x47\xa5"
                  "\xeb\x45\x95\xbf\x33\xa9\xd1\xa3",
    },
    {
        .ksize = 32,
        .psize = 8,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_HMAC,
        .key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b\x0b\x0b",
        .plaintext = "Hi There",
        .digest = "\x19\x8a\x60\x7e\xb4\x4b\xfb\xc6"
                  "\x99\x03\xa0\xf1\xcf\x2b\xbd\xc5"
                  "\xba\x0a\xa3\xf3\xd9\xae\x3c\x1c"
                  "\x7a\x3b\x16\x96\xa0\xb6\x8c\xf7",
    },
    {
        .ksize = 4,
        .psize = 28,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_HMAC,
        .key = "Jefe",
        .plaintext = "what do ya want for nothing?",
        .digest = "\x5b\xdc\xc1\x46\xbf\x60\x75\x4e"
                  "\x6a\x04\x24\x26\x08\x95\x75\xc7"
                  "\x5a\x00\x3f\x08\x9d\x27\x39\x83"
                  "\x9d\xec\x58\xb9\x64\xec\x38\x43",
    },
    {
        .ksize = 32,
        .psize = 50,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_HMAC,
        .key = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
               "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
               "\xaa\xaa\xaa\xaa\xaa\xaa\xaa",
        .plaintext = "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd",
        .digest = "\xcd\xcb\x12\x20\xd1\xec\xcc\xea"
                  "\x91\xe5\x3a\xba\x30\x92\xf9\x62"
                  "\xe5\x49\xfe\x6c\xe9\xed\x7f\xdc"
                  "\x43\x19\x1f\xbd\xe4\x5c\x30\xb0",
    },
    {
        .ksize = 37,
        .psize = 50,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_HMAC,
        .key = "\x01\x02\x03\x04\x05\x06\x07\x08"
               "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
               "\x11\x12\x13\x14\x15\x16\x17\x18"
               "\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20"
               "\x21\x22\x23\x24\x25",
        .plaintext = "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd",
        .digest = "\xd4\x63\x3c\x17\xf6\xfb\x8d\x74"
                  "\x4c\x66\xde\xe0\xf8\xf0\x74\x55"
                  "\x6e\xc4\xaf\x55\xef\x07\x99\x85"
                  "\x41\x46\x8e\xb4\x9b\xd2\xe9\x17",
    },
    {
        .ksize = 32,
        .psize = 20,
        .dsize = 32,
        .alg = SCE_SHA256,
        .mode = SCE_HMAC,
        .key = "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c"
               "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c"
               "\x0c\x0c\x0c\x0c\x0c\x0c",
        .plaintext = "Test With Truncation",
        .digest = "\x75\x46\xaf\x01\x84\x1f\xc0\x9b"
                  "\x1a\xb9\xc3\x74\x9a\x5f\x1c\x17"
                  "\xd4\xf5\x89\x66\x8a\x58\x7b\x27"
                  "\x00\xa9\xc9\x7c\x11\x93\xcf\x42",
    },

    {
        .ksize = 20,
        .psize = 8,
        .dsize = 48,
        .alg = SCE_SHA384,
        .mode = SCE_HMAC,
        .key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b",
        .plaintext = "Hi There",
        .digest = "\xaf\xd0\x39\x44\xd8\x48\x95\x62"
                  "\x6b\x08\x25\xf4\xab\x46\x90\x7f"
                  "\x15\xf9\xda\xdb\xe4\x10\x1e\xc6"
                  "\x82\xaa\x03\x4c\x7c\xeb\xc5\x9c"
                  "\xfa\xea\x9e\xa9\x07\x6e\xde\x7f"
                  "\x4a\xf1\x52\xe8\xb2\xfa\x9c\xb6",
    },
    {
        .ksize = 4,
        .psize = 28,
        .dsize = 48,
        .alg = SCE_SHA384,
        .mode = SCE_HMAC,
        .key = "Jefe",
        .plaintext = "what do ya want for nothing?",
        .digest = "\xaf\x45\xd2\xe3\x76\x48\x40\x31"
                  "\x61\x7f\x78\xd2\xb5\x8a\x6b\x1b"
                  "\x9c\x7e\xf4\x64\xf5\xa0\x1b\x47"
                  "\xe4\x2e\xc3\x73\x63\x22\x44\x5e"
                  "\x8e\x22\x40\xca\x5e\x69\xe2\xc7"
                  "\x8b\x32\x39\xec\xfa\xb2\x16\x49",
    },

    {
        .ksize = 20,
        .psize = 8,
        .dsize = 64,
        .alg = SCE_SHA512,
        .mode = SCE_HMAC,
        .key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b",
        .plaintext = "Hi There",
        .digest = "\x87\xaa\x7c\xde\xa5\xef\x61\x9d"
                  "\x4f\xf0\xb4\x24\x1a\x1d\x6c\xb0"
                  "\x23\x79\xf4\xe2\xce\x4e\xc2\x78"
                  "\x7a\xd0\xb3\x05\x45\xe1\x7c\xde"
                  "\xda\xa8\x33\xb7\xd6\xb8\xa7\x02"
                  "\x03\x8b\x27\x4e\xae\xa3\xf4\xe4"
                  "\xbe\x9d\x91\x4e\xeb\x61\xf1\x70"
                  "\x2e\x69\x6c\x20\x3a\x12\x68\x54",
    },
    {
        .ksize = 4,
        .psize = 28,
        .dsize = 64,
        .alg = SCE_SHA512,
        .mode = SCE_HMAC,
        .key = "Jefe",
        .plaintext = "what do ya want for nothing?",
        .digest = "\x16\x4b\x7a\x7b\xfc\xf8\x19\xe2"
                  "\xe3\x95\xfb\xe7\x3b\x56\xe0\xa3"
                  "\x87\xbd\x64\x22\x2e\x83\x1f\xd6"
                  "\x10\x27\x0c\xd7\xea\x25\x05\x54"
                  "\x97\x58\xbf\x75\xc0\x5a\x99\x4a"
                  "\x6d\x03\x4f\x65\xf8\xf0\xe6\xfd"
                  "\xca\xea\xb1\xa3\x4d\x4a\x6b\x4b"
                  "\x63\x6e\x07\x0a\x38\xbc\xe7\x37",
    },

    {
        .ksize = 20,
        .psize = 8,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_HMAC,
        .key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b\x0b",
        .plaintext = "Hi There",
        .digest = "\xb6\x17\x31\x86\x55\x05\x72\x64"
                  "\xe2\x8b\xc0\xb6\xfb\x37\x8c\x8e\xf1"
                  "\x46\xbe",
    },
    {
        .ksize = 4,
        .psize = 28,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_HMAC,
        .key = "Jefe",
        .plaintext = "what do ya want for nothing?",
        .digest = "\xef\xfc\xdf\x6a\xe5\xeb\x2f\xa2\xd2\x74"
                  "\x16\xd5\xf1\x84\xdf\x9c\x25\x9a\x7c\x79",
    },
    {
        .ksize = 20,
        .psize = 50,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_HMAC,
        .key = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
               "\xaa\xaa\xaa\xaa\xaa",
        .plaintext = "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                     "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd",
        .digest = "\x12\x5d\x73\x42\xb9\xac\x11\xcd\x91\xa3"
                  "\x9a\xf4\x8a\xa1\x7b\x4f\x63\xf1\x75\xd3",
    },
    {
        .ksize = 25,
        .psize = 50,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_HMAC,
        .key = "\x01\x02\x03\x04\x05\x06\x07\x08"
               "\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10"
               "\x11\x12\x13\x14\x15\x16\x17\x18\x19",
        .plaintext = "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd"
                     "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd",
        .digest = "\x4c\x90\x07\xf4\x02\x62\x50\xc6\xbc\x84"
                  "\x14\xf9\xbf\x50\xc8\x6c\x2d\x72\x35\xda",
    },
    {
        .ksize = 20,
        .psize = 20,
        .dsize = 20,
        .alg = SCE_SHA1,
        .mode = SCE_HMAC,
        .key = "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c"
               "\x0c\x0c\x0c\x0c\x0c",
        .plaintext = "Test With Truncation",
        .digest = "\x4c\x1a\x03\x42\x4b\x55\xe0\x7f\xe7\xf2"
                  "\x7b\xe1\xd5\x8b\xb9\x32\x4a\x9a\x5a\x04",
    },

    {
        .ksize = 20,
        .psize = 8,
        .dsize = 28,
        .alg = SCE_SHA224,
        .mode = SCE_HMAC,
        .key = "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
               "\x0b\x0b\x0b\x0b",
        /*  ("Hi There") */
        .plaintext = "\x48\x69\x20\x54\x68\x65\x72\x65",
        .digest = "\x89\x6f\xb1\x12\x8a\xbb\xdf\x19"
                  "\x68\x32\x10\x7c\xd4\x9d\xf3\x3f"
                  "\x47\xb4\xb1\x16\x99\x12\xba\x4f"
                  "\x53\x68\x4b\x22",
    },
    {
        .ksize = 4,
        .psize = 28,
        .dsize = 28,
        .alg = SCE_SHA224,
        .mode = SCE_HMAC,
        .key = "Jefe",
        /* ("what do ya want for nothing?") */
        .plaintext = "\x77\x68\x61\x74\x20\x64\x6f\x20"
                     "\x79\x61\x20\x77\x61\x6e\x74\x20"
                     "\x66\x6f\x72\x20\x6e\x6f\x74\x68"
                     "\x69\x6e\x67\x3f",
        .digest = "\xa3\x0e\x01\x09\x8b\xc6\xdb\xbf"
                  "\x45\x69\x0f\x3a\x7e\x9e\x6d\x0f"
                  "\x8b\xbe\xa2\xa3\x9e\x61\x48\x00"
                  "\x8f\xd0\x5e\x44",
    },
};

static void sha_test_common(char *alg_name, int sce_alg,
                            hash_testvec_t *test_vecs, int count)
{
    u32 i;
    u8 *src, *dst;
    hash_testvec_t *test_vec;
    u8 success = 1;

    src = kmalloc(1024 + 512, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    dst = src + 1024;

    for (i = 0; i < count; i++) {
        test_vec = &test_vecs[i];
        if (sce_alg != test_vec->alg) {
            continue;
        }

        memset(src, 0, 1024 + 512);
        memcpy(src, test_vec->plaintext, test_vec->psize);
        ahash_dgst_test(alg_name, NULL, 0, src, test_vec->psize, dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("%s test dgst err!\n", alg_name);
            success = 0;
            continue;
        }

        memset(dst, 0, test_vec->dsize);
        ahash_finup_test(alg_name, NULL, 0, src, test_vec->psize, dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("%s test dgst err!\n", alg_name);
            success = 0;
            continue;
        }

        if (!test_vec->psize) {
            continue;
        }

        memset(dst, 0, test_vec->dsize);
        ahash_step_test(alg_name, NULL, 0, src, test_vec->psize, dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("%s test dgst err!\n", alg_name);
            success = 0;
        }
    }

    if (success) {
        pr_info("%s test success ", alg_name);
    }

    kfree(src);
}

static void hmac_sha_test_common(char *alg_name, int sce_alg,
                                 hmac_testvec_t *test_vecs, int count)
{
    u32 i;
    u8 *src, *dst, *key;
    hmac_testvec_t *test_vec;
    u8 success = 1;

    src = kmalloc(1024 + 512 + 512, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    dst = src + 1024;
    key = dst + 512;

    for (i = 0; i < count; i++) {
        test_vec = &test_vecs[i];
        if (test_vec->alg != sce_alg) {
            continue;
        }
        memset(src, 0, 1024 + 512 + 512);
        memcpy(src, test_vec->plaintext, test_vec->psize);
        memcpy(key, test_vec->key, test_vec->ksize);

        ahash_dgst_test(alg_name, key, test_vec->ksize, src, test_vec->psize,
                        dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("hmac %s test step err!\n", alg_name);
            print_buf("key", key, test_vec->ksize);
            print_buf("src", src, test_vec->psize);
            print_buf("dst", dst, test_vec->dsize);
            print_buf("digest", (u8 *)test_vec->digest, test_vec->dsize);

            success = 0;
            break;
        }

        memset(dst, 0, test_vec->dsize);
        ahash_finup_test(alg_name, key, test_vec->ksize, src, test_vec->psize,
                         dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("hmac %s test step err!\n", alg_name);
            print_buf("key", key, test_vec->ksize);
            print_buf("src", src, test_vec->psize);
            print_buf("dst", dst, test_vec->dsize);
            print_buf("digest", (u8 *)test_vec->digest, test_vec->dsize);
            success = 0;
            continue;
        }

        if (!test_vec->psize) {
            continue;
        }

        memset(dst, 0, test_vec->dsize);
        ahash_step_test(alg_name, key, test_vec->ksize, src, test_vec->psize,
                        dst);
        if (memcmp(dst, test_vec->digest, test_vec->dsize)) {
            pr_err("hmac %s test step err!\n", alg_name);
            print_buf("key", key, test_vec->ksize);
            print_buf("src", src, test_vec->psize);
            print_buf("dst", dst, test_vec->dsize);
            print_buf("digest", (u8 *)test_vec->digest, test_vec->dsize);
            success = 0;
        }
    }

    if (success) {
        pr_info("hmac %s test success ", alg_name);
    }

    kfree(src);
}

void sha1_test(void)
{
    sha_test_common("sha1", SCE_SHA1, hash_tv_template,
                    sizeof(hash_tv_template) / sizeof(hash_tv_template[0]));
}

void hmac_sha1_test(void)
{
    hmac_sha_test_common("hmac(sha1)", SCE_SHA1, hmac_tv_template,
                         sizeof(hmac_tv_template) /
                             sizeof(hmac_tv_template[0]));
}

void sha224_test(void)
{
    sha_test_common("sha224", SCE_SHA224, hash_tv_template,
                    sizeof(hash_tv_template) / sizeof(hash_tv_template[0]));
}

void hmac_sha224_test(void)
{
    hmac_sha_test_common("hmac(sha224)", SCE_SHA224, hmac_tv_template,
                         sizeof(hmac_tv_template) /
                             sizeof(hmac_tv_template[0]));
}

void sha256_test(void)
{
    sha_test_common("sha256", SCE_SHA256, hash_tv_template,
                    sizeof(hash_tv_template) / sizeof(hash_tv_template[0]));
}

void hmac_sha256_test(void)
{
    hmac_sha_test_common("hmac(sha256)", SCE_SHA256, hmac_tv_template,
                         sizeof(hmac_tv_template) /
                             sizeof(hmac_tv_template[0]));
}

void sha384_test(void)
{
    sha_test_common("sha384", SCE_SHA384, hash_tv_template,
                    sizeof(hash_tv_template) / sizeof(hash_tv_template[0]));
}

void hmac_sha384_test(void)
{
    hmac_sha_test_common("hmac(sha384)", SCE_SHA384, hmac_tv_template,
                         sizeof(hmac_tv_template) /
                             sizeof(hmac_tv_template[0]));
}

void sha512_test(void)
{
    sha_test_common("sha512", SCE_SHA512, hash_tv_template,
                    sizeof(hash_tv_template) / sizeof(hash_tv_template[0]));
}

void hmac_sha512_test(void)
{
    hmac_sha_test_common("hmac(sha512)", SCE_SHA512, hmac_tv_template,
                         sizeof(hmac_tv_template) /
                             sizeof(hmac_tv_template[0]));
}

void aes_cbcmac_test(void)
{
    hmac_sha_test_common("cbcmac(aes)", SCE_MAC_AES, cbcmac_tv_template,
                        sizeof(cbcmac_tv_template)/
                            sizeof(cbcmac_tv_template[0]));
}

void aes_cmac_test(void)
{
    hmac_sha_test_common("cmac(aes)", SCE_MAC_AES, cmac_tv_template,
                        sizeof(cmac_tv_template)/
                            sizeof(cmac_tv_template[0]));
}

void sm4_cbcmac_test(void)
{
    hmac_sha_test_common("cbcmac(sm4)", SCE_MAC_SM4, cbcmac_tv_template,
                        sizeof(cbcmac_tv_template)/
                            sizeof(cbcmac_tv_template[0]));
}

void sm4_cmac_test(void)
{
    hmac_sha_test_common("cmac(sm4)", SCE_MAC_SM4, cmac_tv_template,
                        sizeof(cmac_tv_template)/
                            sizeof(cmac_tv_template[0]));
}
void des_cbcmac_test(void)
{
    hmac_sha_test_common("cbcmac(des)", SCE_MAC_DES, cbcmac_tv_template,
                        sizeof(cbcmac_tv_template)/
                            sizeof(cbcmac_tv_template[0]));
}

void des_cmac_test(void)
{
    hmac_sha_test_common("cmac(des)", SCE_MAC_DES, cmac_tv_template,
                        sizeof(cmac_tv_template)/
                            sizeof(cmac_tv_template[0]));
}
void des3_ede_cbcmac_test(void)
{
    hmac_sha_test_common("cbcmac(des3_ede)", SCE_MAC_DES3_EDE, cbcmac_tv_template,
                        sizeof(cbcmac_tv_template)/
                            sizeof(cbcmac_tv_template[0]));
}

void des3_ede_cmac_test(void)
{
    hmac_sha_test_common("cmac(des3_ede)", SCE_MAC_DES3_EDE, cmac_tv_template,
                        sizeof(cmac_tv_template)/
                            sizeof(cmac_tv_template[0]));
}

