#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "sca_test.h"
#include "../common/common.h"

ske_testvec_t aead_aes_tv_template[] = {
	{
                .key    = "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
			"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
                .klen   = 32,
                .ctext = "\x53\x0f\x8a\xfb\xc7\x45\x36\xb9"
                          "\xa9\x63\xb4\xf1\xc4\xcb\x73\x8b",
                .clen   = 16,
		.alg	= SCE_AES_256,
		.mode	= GCM,
        },
	{
                .key    = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
                          "\x6d\x6a\x8f\x94\x67\x30\x83\x08"
                          "\xfe\xff\xe9\x92\x86\x65\x73\x1c",
                .klen   = 24,
		.ilen	= 12,
                .iv     = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
                          "\xde\xca\xf8\x88",
                .ptext  = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
                          "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
                          "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
                          "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
                          "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
                          "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
                          "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
                          "\xba\x63\x7b\x39\x1a\xaf\xd2\x55",
                .len   = 64,
                .ctext = "\x39\x80\xca\x0b\x3c\x00\xe8\x41"
                          "\xeb\x06\xfa\xc4\x87\x2a\x27\x57"
                          "\x85\x9e\x1c\xea\xa6\xef\xd9\x84"
                          "\x62\x85\x93\xb4\x0c\xa1\xe1\x9c"
                          "\x7d\x77\x3d\x00\xc1\x44\xc5\x25"
                          "\xac\x61\x9d\x18\xc8\x4a\x3f\x47"
                          "\x18\xe2\x44\x8b\x2f\xe3\x24\xd9"
                          "\xcc\xda\x27\x10\xac\xad\xe2\x56"
                          "\x99\x24\xa7\xc8\x58\x73\x36\xbf"
                          "\xb1\x18\x02\x4d\xb8\x67\x4a\x14",
                .clen   = 80,
		.alg	= SCE_AES_192,
		.mode	= GCM,
        },
        {
                .key    = "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
			"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
                .klen   = 24,
                .ctext = "\xcd\x33\xb2\x8a\xc7\x73\xf7\x4b"
                          "\xa0\x0e\xd1\xf3\x12\x57\x24\x35",
                .clen   = 16,
		.alg	= SCE_AES_192,
		.mode	= GCM,
        },
        {
                .key    = "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
                .klen   = 16,
                .ptext  = "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
                .len   = 16,
                .ctext = "\x03\x88\xda\xce\x60\xb6\xa3\x92"
                          "\xf3\x28\xc2\xb9\x71\xb2\xfe\x78"
                          "\xab\x6e\x47\xd4\x2c\xec\x13\xbd"
                          "\xf5\x3a\x67\xb2\x12\x57\xbd\xdf",
                .clen   = 32,
		.alg	= SCE_AES_128,
		.mode	= GCM,
        },
	{ /* From McGrew & Viega - http://citeseer.ist.psu.edu/656989.html */
                .key    = "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
                .klen   = 16,
                .ctext = "\x58\xe2\xfc\xce\xfa\x7e\x30\x61"
                          "\x36\x7f\x1d\x57\xa4\xe7\x45\x5a",
                .clen   = 16,
		.alg	= SCE_AES_128,
		.mode	= GCM,
		//.ilen   = 12,
		.iv	= "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00",
		.aadlen	= 0,
		.len	= 0,
        },
	{
                .alg = SCE_AES_128,
                .mode = GCM,
                .ilen   = 12,
                .key    = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
                          "\x6d\x6a\x8f\x94\x67\x30\x83\x08",
                .klen   = 16,
                .iv     = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
                          "\xde\xca\xf8\x88",
                .ptext  = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
                          "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
                          "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
                          "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
                          "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
                          "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
                          "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
                          "\xba\x63\x7b\x39\x1a\xaf\xd2\x55",
                .len   = 64,
                .ctext  = "\x42\x83\x1e\xc2\x21\x77\x74\x24"
                          "\x4b\x72\x21\xb7\x84\xd0\xd4\x9c"
                          "\xe3\xaa\x21\x2f\x2c\x02\xa4\xe0"
                          "\x35\xc1\x7e\x23\x29\xac\xa1\x2e"
                          "\x21\xd5\x14\xb2\x54\x66\x93\x1c"
                          "\x7d\x8f\x6a\x5a\xac\x84\xaa\x05"
                          "\x1b\xa3\x0b\x39\x6a\x0a\xac\x97"
                          "\x3d\x58\xe0\x91\x47\x3f\x59\x85"
                          "\x4d\x5c\x2a\xf3\x27\xcd\x64\xa6"
                          "\x2c\xf3\x5a\xbd\x2b\xa6\xfa\xb4",
                .clen   = 80,
        },
	{
                .key    = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
                          "\x6d\x6a\x8f\x94\x67\x30\x83\x08",
                .klen   = 16,
                .iv     = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
                          "\xde\xca\xf8\x88",
		.ilen	= 12,
                .ptext  = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
                          "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
                          "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
                          "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
                          "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
                          "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
                          "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
                          "\xba\x63\x7b\x39",
                .len   = 60,
                .assoc  = "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
                          "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
                          "\xab\xad\xda\xd2",
                .aadlen   = 20,
                .ctext = "\x42\x83\x1e\xc2\x21\x77\x74\x24"
                          "\x4b\x72\x21\xb7\x84\xd0\xd4\x9c"
                          "\xe3\xaa\x21\x2f\x2c\x02\xa4\xe0"
                          "\x35\xc1\x7e\x23\x29\xac\xa1\x2e"
                          "\x21\xd5\x14\xb2\x54\x66\x93\x1c"
                          "\x7d\x8f\x6a\x5a\xac\x84\xaa\x05"
                          "\x1b\xa3\x0b\x39\x6a\x0a\xac\x97"
                          "\x3d\x58\xe0\x91"
                          "\x5b\xc9\x4f\xbc\x32\x21\xa5\xdb"
                          "\x94\xfa\xe9\x5a\xe7\x12\x1a\x47",
                .clen   = 76,
		.alg	= SCE_AES_128,
		.mode	= GCM,
        },
{
                .key    = "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
                          "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf",
                .klen   = 16,
                .iv     = "\x01\x00\x00\x00\x07\x06\x05\x04"
                          "\xa0\xa1\xa2\xa3\xa4\xa5\x00\x00",
                .assoc  = "\x00\x01\x02\x03\x04\x05\x06\x07"
                          "\x08\x09\x0a\x0b",
                .aadlen   = 12,
                .ptext  = "\x0c\x0d\x0e\x0f\x10\x11\x12\x13"
                          "\x14\x15\x16\x17\x18\x19\x1a\x1b"
                          "\x1c\x1d\x1e\x1f",
                .ilen  = 16,
		.len   = 20,
                .ctext = "\xdc\xf1\xfb\x7b\x5d\x9e\x23\xfb"
                          "\x9d\x4e\x13\x12\x53\x65\x8a\xd8"
                          "\x6e\xbd\xca\x3e\x51\xe8\x3f\x07"
                          "\x7d\x9c\x2d\x93",
                .clen   = 28,
		.alg    = SCE_AES_128,
		.mode   = CCM,
        },
	{
                .key    = "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
                          "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf",
                .klen   = 16,
                .iv     = "\x01\x00\x00\x00\x0b\x0a\x09\x08"
                          "\xa0\xa1\xa2\xa3\xa4\xa5\x00\x00",
                .assoc  = "\x00\x01\x02\x03\x04\x05\x06\x07",
                .aadlen   = 8,
                .ptext  = "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
                          "\x10\x11\x12\x13\x14\x15\x16\x17"
                          "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f"
                          "\x20",
                .ilen   = 16,
		.len    = 25,
                .ctext =  "\x82\x53\x1a\x60\xcc\x24\x94\x5a"
                          "\x4b\x82\x79\x18\x1a\xb5\xc8\x4d"
                          "\xf2\x1c\xe7\xf9\xb7\x3f\x42\xe1"
                          "\x97\xea\x9c\x07\xe5\x6b\x5e\xb1"
                          "\x7e\x5f\x4e",
                .clen   = 35,
		.alg    = SCE_AES_128,
                .mode   = CCM,
        },
        {
                .alg = SCE_AES_128,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
                          "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
                .klen   = 16,
                .iv     = "\x01\x00\x33\x56\x8e\xf7\xb2\x63"
                          "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
                .assoc  = "\x63\x01\x8f\x76\xdc\x8a\x1b\xcb",
                .aadlen   = 8,
                .ptext  = "\x90\x20\xea\x6f\x91\xbd\xd8\x5a"
                          "\xfa\x00\x39\xba\x4b\xaf\xf9\xbf"
                          "\xb7\x9c\x70\x28\x94\x9c\xd0\xec",
                .len   = 24,
                .ctext  = "\x4c\xcb\x1e\x7c\xa9\x81\xbe\xfa"
                          "\xa0\x72\x6c\x55\xd3\x78\x06\x12"
                          "\x98\xc8\x5c\x92\x81\x4a\xbc\x33"
                          "\xc5\x2e\xe8\x1d\x7d\x77\xc0\x8a",
                .clen   = 32,
        },
        {
                .alg = SCE_AES_128,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
                          "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
                .klen   = 16,
                .iv     = "\x01\x00\xd5\x60\x91\x2d\x3f\x70"
                          "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
                .assoc  = "\xcd\x90\x44\xd2\xb7\x1f\xdb\x81"
                          "\x20\xea\x60\xc0",
                .aadlen   = 12,
                .ptext  = "\x64\x35\xac\xba\xfb\x11\xa8\x2e"
                          "\x2f\x07\x1d\x7c\xa4\xa5\xeb\xd9"
                          "\x3a\x80\x3b\xa8\x7f",
                .len   = 21,
                .ctext  = "\x00\x97\x69\xec\xab\xdf\x48\x62"
                          "\x55\x94\xc5\x92\x51\xe6\x03\x57"
                          "\x22\x67\x5e\x04\xc8\x47\x09\x9e"
                          "\x5a\xe0\x70\x45\x51",
                .clen   = 29,
        }, {
                .alg = SCE_AES_128,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
                          "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
                .klen   = 16,
                .iv     = "\x01\x00\x42\xff\xf8\xf1\x95\x1c"
                          "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
                .assoc  = "\xd8\x5b\xc7\xe6\x9f\x94\x4f\xb8",
                .aadlen   = 8,
                .ptext  = "\x8a\x19\xb9\x50\xbc\xf7\x1a\x01"
                          "\x8e\x5e\x67\x01\xc9\x17\x87\x65"
                          "\x98\x09\xd6\x7d\xbe\xdd\x18",
                .len   = 23,
                .ctext  = "\xbc\x21\x8d\xaa\x94\x74\x27\xb6"
                          "\xdb\x38\x6a\x99\xac\x1a\xef\x23"
                          "\xad\xe0\xb5\x29\x39\xcb\x6a\x63"
                          "\x7c\xf9\xbe\xc2\x40\x88\x97\xc6"
                          "\xba",
                .clen   = 33,
        },
        {
                /* This is taken from FIPS CAVS. */
                .alg = SCE_AES_128,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\x83\xac\x54\x66\xc2\xeb\xe5\x05"
                          "\x2e\x01\xd1\xfc\x5d\x82\x66\x2e",
                .klen   = 16,
                .iv     = "\x03\x96\xac\x59\x30\x07\xa1\xe2\xa2\xc7\x55\x24\0\0\0\0",
                .aadlen   = 0,
                .ptext  = "\x19\xc8\x81\xf6\xe9\x86\xff\x93"
                          "\x0b\x78\x67\xe5\xbb\xb7\xfc\x6e"
                          "\x83\x77\xb3\xa6\x0c\x8c\x9f\x9c"
                          "\x35\x2e\xad\xe0\x62\xf9\x91\xa1",
                .len   = 32,
                .ctext  = "\xab\x6f\xe1\x69\x1d\x19\x99\xa8"
                          "\x92\xa0\xc4\x6f\x7e\xe2\x8b\xb1"
                          "\x70\xbb\x8c\xa6\x4c\x6e\x97\x8a"
                          "\x57\x2b\xbe\x5d\x98\xa6\xb1\x32"
                          "\xda\x24\xea\xd9\xa1\x39\x98\xfd"
                          "\xa4\xbe\xd9\xf2\x1a\x6d\x22\xa8",
                .clen   = 48,
        },
        {
                .alg = SCE_AES_128,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\x1e\x2c\x7e\x01\x41\x9a\xef\xc0"
                          "\x0d\x58\x96\x6e\x5c\xa2\x4b\xd3",
                .klen   = 16,
                .iv     = "\x03\x4f\xa3\x19\xd3\x01\x5a\xd8"
                          "\x30\x60\x15\x56\x00\x00\x00\x00",
                .assoc  = "\xda\xe6\x28\x9c\x45\x2d\xfd\x63"
                          "\x5e\xda\x4c\xb6\xe6\xfc\xf9\xb7"
                          "\x0c\x56\xcb\xe4\xe0\x05\x7a\xe1"
                          "\x0a\x63\x09\x78\xbc\x2c\x55\xde",
                .aadlen   = 32,
                .ptext  = "\x87\xa3\x36\xfd\x96\xb3\x93\x78"
                          "\xa9\x28\x63\xba\x12\xa3\x14\x85"
                          "\x57\x1e\x06\xc9\x7b\x21\xef\x76"
                          "\x7f\x38\x7e\x8e\x29\xa4\x3e\x7e",
                .len   = 32,
                .ctext  = "\x8a\x1e\x11\xf0\x02\x6b\xe2\x19"
                          "\xfc\x70\xc4\x6d\x8e\xb7\x99\xab"
                          "\xc5\x4b\xa2\xac\xd3\xf3\x48\xff"
                          "\x3b\xb5\xce\x53\xef\xde\xbb\x02"
                          "\xa9\x86\x15\x6c\x13\xfe\xda\x0a"
                          "\x22\xb8\x29\x3d\xd8\x39\x9a\x23",
                .clen   = 48,
        },
        {
                .alg = SCE_AES_192,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\x56\xdf\x5c\x8f\x26\x3f\x0e\x42"
                          "\xef\x7a\xd3\xce\xfc\x84\x60\x62"
                          "\xca\xb4\x40\xaf\x5f\xc9\xc9\x01",
                .klen   = 24,
                .iv     = "\x03\xd6\x3c\x8c\x86\x84\xb6\xcd"
                          "\xef\x09\x2e\x94\x00\x00\x00\x00",
                .assoc  = "\x02\x65\x78\x3c\xe9\x21\x30\x91"
                          "\xb1\xb9\xda\x76\x9a\x78\x6d\x95"
                          "\xf2\x88\x32\xa3\xf2\x50\xcb\x4c"
                          "\xe3\x00\x73\x69\x84\x69\x87\x79",
                .aadlen   = 32,
                .ptext  = "\x9f\xd2\x02\x4b\x52\x49\x31\x3c"
                          "\x43\x69\x3a\x2d\x8e\x70\xad\x7e"
                          "\xe0\xe5\x46\x09\x80\x89\x13\xb2"
                          "\x8c\x8b\xd9\x3f\x86\xfb\xb5\x6b",
                .len   = 32,
                .ctext  = "\x39\xdf\x7c\x3c\x5a\x29\xb9\x62"
                          "\x5d\x51\xc2\x16\xd8\xbd\x06\x9f"
                          "\x9b\x6a\x09\x70\xc1\x51\x83\xc2"
                          "\x66\x88\x1d\x4f\x9a\xda\xe0\x1e"
                          "\xc7\x79\x11\x58\xe5\x6b\x20\x40"
                          "\x7a\xea\x46\x42\x8b\xe4\x6f\xe1",
                .clen   = 48,
        }, {
                .alg = SCE_AES_256,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\xe0\x8d\x99\x71\x60\xd7\x97\x1a"
                          "\xbd\x01\x99\xd5\x8a\xdf\x71\x3a"
                          "\xd3\xdf\x24\x4b\x5e\x3d\x4b\x4e"
                          "\x30\x7a\xb9\xd8\x53\x0a\x5e\x2b",
                .klen   = 32,
                .iv     = "\x03\x1e\x29\x91\xad\x8e\xc1\x53"
                          "\x0a\xcf\x2d\xbe\x00\x00\x00\x00",
                .assoc  = "\x19\xb6\x1f\x57\xc4\xf3\xf0\x8b"
                          "\x78\x2b\x94\x02\x29\x0f\x42\x27"
                          "\x6b\x75\xcb\x98\x34\x08\x7e\x79"
                          "\xe4\x3e\x49\x0d\x84\x8b\x22\x87",
                .aadlen   = 32,
                .ptext  = "\xe1\xd9\xd8\x13\xeb\x3a\x75\x3f"
                          "\x9d\xbd\x5f\x66\xbe\xdc\xbb\x66"
                          "\xbf\x17\x99\x62\x4a\x39\x27\x1f"
                          "\x1d\xdc\x24\xae\x19\x2f\x98\x4c",
                .len   = 32,
                .ctext  = "\x19\xb8\x61\x33\x45\x2b\x43\x96"
                          "\x6f\x51\xd0\x20\x30\x7d\x9b\xc6"
                          "\x26\x3d\xf8\xc9\x65\x16\xa8\x9f"
                          "\xf0\x62\x17\x34\xf2\x1e\x8d\x75"
                          "\x4e\x13\xcc\xc0\xc3\x2a\x54\x2d",
                .clen   = 40,
        },
	{
                .alg = SCE_AES_256,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\x7c\xc8\x18\x3b\x8d\x99\xe0\x7c"
                          "\x45\x41\xb8\xbd\x5c\xa7\xc2\x32"
                          "\x8a\xb8\x02\x59\xa4\xfe\xa9\x2c"
                          "\x09\x75\x9a\x9b\x3c\x9b\x27\x39",
                .klen   = 32,
                .iv     = "\x03\xf9\xd9\x4e\x63\xb5\x3d\x9d"
                          "\x43\xf6\x1e\x50\0\0\0\0",
                .assoc  = "\x57\xf5\x6b\x8b\x57\x5c\x3d\x3b"
                          "\x13\x02\x01\x0c\x83\x4c\x96\x35"
                          "\x8e\xd6\x39\xcf\x7d\x14\x9b\x94"
                          "\xb0\x39\x36\xe6\x8f\x57\xe0\x13",
                .aadlen   = 32,
                .ptext  = "\x3b\x6c\x29\x36\xb6\xef\x07\xa6"
                          "\x83\x72\x07\x4f\xcf\xfa\x66\x89"
                          "\x5f\xca\xb1\xba\xd5\x8f\x2c\x27"
                          "\x30\xdb\x75\x09\x93\xd4\x65\xe4",
                .len   = 32,
                .ctext  = "\xb0\x88\x5a\x33\xaa\xe5\xc7\x1d"
                          "\x85\x23\xc7\xc6\x2f\xf4\x1e\x3d"
                          "\xcc\x63\x44\x25\x07\x78\x4f\x9e"
                          "\x96\xb8\x88\xeb\xbc\x48\x1f\x06"
                          "\x39\xaf\x39\xac\xd8\x4a\x80\x39"
                          "\x7b\x72\x8a\xf7",
                .clen   = 44,
        }, {
                .alg = SCE_AES_256,
                .mode = CCM,
                .ilen   = 16,
                .key    = "\xab\xd0\xe9\x33\x07\x26\xe5\x83"
                          "\x8c\x76\x95\xd4\xb6\xdc\xf3\x46"
                          "\xf9\x8f\xad\xe3\x02\x13\x83\x77"
                          "\x3f\xb0\xf1\xa1\xa1\x22\x0f\x2b",
                .klen   = 32,
                .iv     = "\x03\x24\xa7\x8b\x07\xcb\xcc\x0e"
                          "\xe6\x33\xbf\xf5\x00\x00\x00\x00",
                .assoc  = "\xd4\xdb\x30\x1d\x03\xfe\xfd\x5f"
                          "\x87\xd4\x8c\xb6\xb6\xf1\x7a\x5d"
                          "\xab\x90\x65\x8d\x8e\xca\x4d\x4f"
                          "\x16\x0c\x40\x90\x4b\xc7\x36\x73",
                .aadlen   = 32,
                .ptext  = "\xf5\xc6\x7d\x48\xc1\xb7\xe6\x92"
                          "\x97\x5a\xca\xc4\xa9\x6d\xf9\x3d"
                          "\x6c\xde\xbc\xf1\x90\xea\x6a\xb2"
                          "\x35\x86\x36\xaf\x5c\xfe\x4b\x3a",
                .len   = 32,
                .ctext  = "\x83\x6f\x40\x87\x72\xcf\xc1\x13"
                          "\xef\xbb\x80\x21\x04\x6c\x58\x09"
                          "\x07\x1b\xfc\xdf\xc0\x3f\x5b\xc7"
                          "\xe0\x79\xa8\x6e\x71\x7c\x3f\xcf"
                          "\x5c\xda\xb2\x33\xe5\x13\xe2\x0d"
                          "\x74\xd1\xef\xb5\x0f\x3a\xb5\xf8",
                .clen   = 48,
        },


};

ske_testvec_t aead_sm4_tv_template[] = {
        {
                .klen = 16,
                .len = 19,
                .clen = 29,
                .aadlen = 12,
                .ilen = 16,
                .alg = SCE_SM4,
                .mode = CCM,
                .key    = "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
                          "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf",
                .iv     = "\x01\x00\x00\x00\x0c\x0b\x0a\x09"
                          "\xa0\xa1\xa2\xa3\xa4\xa5\x00\x00",
                .assoc  = "\x00\x01\x02\x03\x04\x05\x06\x07"
                          "\x08\x09\x0a\x0b",
                .ptext  = "\x0c\x0d\x0e\x0f\x10\x11\x12\x13"
                          "\x14\x15\x16\x17\x18\x19\x1a\x1b"
                          "\x1c\x1d\x1e",
                .ctext  = "\x82\x55\x65\x01\xE0\xC6\x00\xD7\x2B\xEE\x4A\x8A\x88\xD3\x41\x2B"
                                "\xF8\x38\x94"
                                "\x38\xE9\x9A\xF2\x70\x67\x1C\x3F\xEE\x8F",
        },
{
                .klen = 16,
                .len = 21,
                .clen = 29,
                .aadlen = 12,
                .ilen = 16,
                .alg = SCE_SM4,
                .mode = CCM,
                .key    = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
                          "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
                .iv     = "\x01\x00\xd5\x60\x91\x2d\x3f\x70"
                          "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
                .assoc  = "\xcd\x90\x44\xd2\xb7\x1f\xdb\x81"
                          "\x20\xea\x60\xc0",
                .ptext  = "\x64\x35\xac\xba\xfb\x11\xa8\x2e"
                          "\x2f\x07\x1d\x7c\xa4\xa5\xeb\xd9"
                          "\x3a\x80\x3b\xa8\x7f",
                .ctext  = "\x5E\xA8\x04\xBE\x9D\x53\x26\x3E\x27\x49\x8E\x31\xA0\x44\xD2\x0F"
                                "\x3F\x79\x4E\x8E\xE6"
                                "\x99\x53\x24\x30\x05\xBE\x5C\x64",
        },
        {
                .klen = 16,
                .len = 24,
                .clen = 32,
                .aadlen = 8,
                .ilen = 16,
                .alg = SCE_SM4,
                .mode = CCM,
                .key    = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
                          "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
                .iv     = "\x01\x00\x33\x56\x8e\xf7\xb2\x63"
                          "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
                .assoc  = "\x63\x01\x8f\x76\xdc\x8a\x1b\xcb",
                .ptext  = "\x90\x20\xea\x6f\x91\xbd\xd8\x5a"
                          "\xfa\x00\x39\xba\x4b\xaf\xf9\xbf"
                          "\xb7\x9c\x70\x28\x94\x9c\xd0\xec",
                .ctext  = "\xDA\x5D\x76\x41\x96\x1A\x57\x73\x3D\x11\xB0\x51\xBA\xE6\x5A\x45"
                                "\x6B\xB0\x83\x03\x10\xCC\x98\x0C"
                                "\x52\x88\xF5\xA6\x8F\x0E\x54\xF0",
        },
		{ /* From https://datatracker.ietf.org/doc/html/rfc8998#appendix-A.1 */
                .alg = SCE_SM4,
                .mode = GCM,

		.klen = 16,
		.len = 64,
		.clen = 80,
		.aadlen = 20,
		.ilen = 12,
		.key	= "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
			  "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.iv	= "\x00\x00\x12\x34\x56\x78\x00\x00"
			  "\x00\x00\xAB\xCD",
		.ptext	= "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
			  "\xBB\xBB\xBB\xBB\xBB\xBB\xBB\xBB"
			  "\xCC\xCC\xCC\xCC\xCC\xCC\xCC\xCC"
			  "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
			  "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
			  "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"
			  "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
			  "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA",
		.assoc	= "\xFE\xED\xFA\xCE\xDE\xAD\xBE\xEF"
			  "\xFE\xED\xFA\xCE\xDE\xAD\xBE\xEF"
			  "\xAB\xAD\xDA\xD2",
		.ctext	= "\x17\xF3\x99\xF0\x8C\x67\xD5\xEE"
			  "\x19\xD0\xDC\x99\x69\xC4\xBB\x7D"
			  "\x5F\xD4\x6F\xD3\x75\x64\x89\x06"
			  "\x91\x57\xB2\x82\xBB\x20\x07\x35"
			  "\xD8\x27\x10\xCA\x5C\x22\xF0\xCC"
			  "\xFA\x7C\xBF\x93\xD4\x96\xAC\x15"
			  "\xA5\x68\x34\xCB\xCF\x98\xC3\x97"
			  "\xB4\x02\x4A\x26\x91\x23\x3B\x8D"
			  "\x83\xDE\x35\x41\xE4\xC2\xB5\x81"
			  "\x77\xE0\x65\xA9\xBF\x7B\x62\xEC",
	},
	{
		.klen = 16,
		.len = 96,
		.clen = 112,
		.aadlen = 36,
		.ilen = 12,
		.mode = GCM,
		.alg = SCE_SM4,
		.key	= "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
			  "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.iv	= "\x00\x00\x12\x34\x56\x78\x00\x00"
			  "\x00\x00\xAB\xCD",
		.ptext	= "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
			  "\xBB\xBB\xBB\xBB\xBB\xBB\xBB\xBB"
			  "\xCC\xCC\xCC\xCC\xCC\xCC\xCC\xCC"
			  "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
			  "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
			  "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"
			  "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
			  "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
			  "\xCC\xCC\xCC\xCC\xCC\xCC\xCC\xCC"
			  "\xBB\xBB\xBB\xBB\xBB\xBB\xBB\xBB"
			  "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
			  "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA",
		.assoc	= "\xFE\xED\xFA\xCE\xDE\xAD\xBE\xEF"
			  "\xFE\xED\xFA\xCE\xDE\xAD\xBE\xEF"
			  "\x5F\xD4\x6F\xD3\x75\x64\x89\x06"
			  "\x17\xF3\x99\xF0\x8C\x67\xD5\xEE"
			  "\xAB\xAD\xDA\xD2",
		.ctext	= "\x17\xf3\x99\xf0\x8c\x67\xd5\xee"
			  "\x19\xd0\xdc\x99\x69\xc4\xbb\x7d"
			  "\x5f\xd4\x6f\xd3\x75\x64\x89\x06"
			  "\x91\x57\xb2\x82\xbb\x20\x07\x35"
			  "\xd8\x27\x10\xca\x5c\x22\xf0\xcc"
			  "\xfa\x7c\xbf\x93\xd4\x96\xac\x15"
			  "\xa5\x68\x34\xcb\xcf\x98\xc3\x97"
			  "\xc3\x75\x3d\x51\xe6\x54\x4c\xfa"
			  "\xe2\x09\x61\x87\x79\x0d\x3c\xd2"
			  "\xa7\x51\x56\x3e\xe0\xc0\xea\x31"
			  "\xff\xab\xf9\xba\xc6\xe4\x90\xb6"
			  "\x5c\x66\x01\xee\x1c\x0f\x45\x5e"
			  "\xfc\x42\xc4\x78\x45\xe2\xc7\x7f"
			  "\xc8\x47\xe8\x15\xf3\xf9\xf9\xd3",
	},
	{
		.klen = 16,
		.len = 77,
		.clen = 93,
		.aadlen = 4,
		.ilen = 12,
		.mode = GCM,
		.alg = SCE_SM4,
		.key	= "\xb1\xb5\xad\x30\x8c\x77\x68\x70\x80\xcb\xd8\xf1\x6d\x63\xde\x3e",
		.iv	= "\xb9\xbe\x1d\xad\xac\xe8\x4b\x84\xdf\x19\x21\x94",
		.ptext	= "\x44\x01\x34\x2c\x3c\x8c\x8f\x69\x05\x6c\xef\x6b"
				"\xb9\x6e\x80\xcf\x55\x2a\xcb\x6e\x50\x87\x9b\xc4\x99\x71\x58\x3d"
				"\x7b\x58\x71\x36\x59\x2b\xcc\x9b\x1b\xf5\x84\x48\x3a\xd6\x0c\x14"
				"\x33\xf8\xe1\x42\xce\x36\x90\x6f\xc5\x16\xe1\xbc\x25\xd0\x4e\xa1"
				"\x45\x1e\x6f\x6c\xb3\xab\x18\xb0\xd9\xc5\x1c\x49\xb0\xf3\x02\xa1"
				"\x36",
		.assoc	= "\x61\xad\x09\x65",
		.ctext	= "\xcb\xa4\xfd\xd1\x33\x4c\x30\x55\xa3\x30\xe1\xa8"
				"\xcf\x93\xca\xad\x75\x90\x3b\xd7\xca\x00\x46\xb1\x2f\xf0\x18\x48"
				"\x3d\x65\x28\x15\x32\xaa\x5b\x6d\xde\xf1\x8f\x07\xf4\x24\x74\x66"
				"\x97\x12\x1d\xe4\x86\x2b\x70\x3f\xdd\x5a\x6e\xf4\x1c\xf7\xf0\x76"
				"\xbe\xe5\x9f\x47\x97\x6c\xae\x3b\x57\x5b\xdd\x2b\xf7\x7a\x22\x48"
				"\x65\xd9\x28\x65\xb3\x36\x42\x57\xa0\xff\xd8\x08\x79\xfd\x76\xef"
				"\xd1",
	},
	{
		.klen = 16,
		.len = 77,
		.clen = 92,
		.aadlen = 4,
		.ilen = 12,
		.mode = GCM,
		.alg = SCE_SM4,
		.key	= "\xb1\xb5\xad\x30\x8c\x77\x68\x70\x80\xcb\xd8\xf1\x6d\x63\xde\x3e",
		.iv	= "\xb9\xbe\x1d\xad\xac\xe8\x4b\x84\xdf\x19\x21\x94",
		.ptext	= "\x44\x01\x34\x2c\x3c\x8c\x8f\x69\x05\x6c\xef\x6b"
				"\xb9\x6e\x80\xcf\x55\x2a\xcb\x6e\x50\x87\x9b\xc4\x99\x71\x58\x3d"
				"\x7b\x58\x71\x36\x59\x2b\xcc\x9b\x1b\xf5\x84\x48\x3a\xd6\x0c\x14"
				"\x33\xf8\xe1\x42\xce\x36\x90\x6f\xc5\x16\xe1\xbc\x25\xd0\x4e\xa1"
				"\x45\x1e\x6f\x6c\xb3\xab\x18\xb0\xd9\xc5\x1c\x49\xb0\xf3\x02\xa1"
				"\x36",
		.assoc	= "\x61\xad\x09\x65",
		.ctext	= "\xcb\xa4\xfd\xd1\x33\x4c\x30\x55\xa3\x30\xe1\xa8"
				"\xcf\x93\xca\xad\x75\x90\x3b\xd7\xca\x00\x46\xb1\x2f\xf0\x18\x48"
				"\x3d\x65\x28\x15\x32\xaa\x5b\x6d\xde\xf1\x8f\x07\xf4\x24\x74\x66"
				"\x97\x12\x1d\xe4\x86\x2b\x70\x3f\xdd\x5a\x6e\xf4\x1c\xf7\xf0\x76"
				"\xbe\xe5\x9f\x47\x97\x6c\xae\x3b\x57\x5b\xdd\x2b\xf7\x7a\x22\x48"
				"\x65\xd9\x28\x65\xb3\x36\x42\x57\xa0\xff\xd8\x08\x79\xfd\x76\xef",
	},
	{
		.klen = 16,
		.len = 77,
		.clen = 91,
		.aadlen = 4,
		.ilen = 12,
		.mode = GCM,
		.alg = SCE_SM4,
		.key	= "\xb1\xb5\xad\x30\x8c\x77\x68\x70\x80\xcb\xd8\xf1\x6d\x63\xde\x3e",
		.iv	= "\xb9\xbe\x1d\xad\xac\xe8\x4b\x84\xdf\x19\x21\x94",
		.ptext	= "\x44\x01\x34\x2c\x3c\x8c\x8f\x69\x05\x6c\xef\x6b"
				"\xb9\x6e\x80\xcf\x55\x2a\xcb\x6e\x50\x87\x9b\xc4\x99\x71\x58\x3d"
				"\x7b\x58\x71\x36\x59\x2b\xcc\x9b\x1b\xf5\x84\x48\x3a\xd6\x0c\x14"
				"\x33\xf8\xe1\x42\xce\x36\x90\x6f\xc5\x16\xe1\xbc\x25\xd0\x4e\xa1"
				"\x45\x1e\x6f\x6c\xb3\xab\x18\xb0\xd9\xc5\x1c\x49\xb0\xf3\x02\xa1"
				"\x36",
		.assoc	= "\x61\xad\x09\x65",
		.ctext	= "\xcb\xa4\xfd\xd1\x33\x4c\x30\x55\xa3\x30\xe1\xa8"
				"\xcf\x93\xca\xad\x75\x90\x3b\xd7\xca\x00\x46\xb1\x2f\xf0\x18\x48"
				"\x3d\x65\x28\x15\x32\xaa\x5b\x6d\xde\xf1\x8f\x07\xf4\x24\x74\x66"
				"\x97\x12\x1d\xe4\x86\x2b\x70\x3f\xdd\x5a\x6e\xf4\x1c\xf7\xf0\x76"
				"\xbe\xe5\x9f\x47\x97\x6c\xae\x3b\x57\x5b\xdd\x2b\xf7\x7a\x22\x48"
				"\x65\xd9\x28\x65\xb3\x36\x42\x57\xa0\xff\xd8\x08\x79\xfd\x76",
	},


};

void aead_test_one_dec(char *alg_name, ske_testvec_t *test_vec)
{
    u8 *src, *dst, *key, *iv;

    src = kmalloc(1024 * 3, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    memset(src, 0, 1024 * 3);
    dst = src + 1024;
    key = dst + 1024;
    iv = key + 512;
    // test enc
    memcpy(src, test_vec->assoc, test_vec->aadlen);
    memcpy(src + test_vec->aadlen, test_vec->ptext, test_vec->len);
    memcpy(key, test_vec->key, test_vec->klen);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }

    // test one segment
    aead_test(alg_name, 0, key, test_vec->klen, iv, src, test_vec->len, test_vec->aadlen,
                  dst, test_vec->clen);
    if (memcmp(dst + test_vec->aadlen, test_vec->ctext, test_vec->clen)) {
        pr_err("encrypt %s err!\n", alg_name);
        //print_buf("src", src, test_vec->aadlen + test_vec->len);
        //print_buf("dst", dst - 16, test_vec->aadlen+test_vec->clen + 32);
        //print_buf("result", test_vec->ctext, test_vec->clen);
        goto out;
    }

#if 1
    // test enc
    memcpy(src, test_vec->assoc, test_vec->aadlen);

    memcpy(src + test_vec->aadlen, test_vec->ctext, test_vec->clen);

    memcpy(key, test_vec->key, test_vec->klen);
    memset(dst, 0, 1024);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }
    aead_test(alg_name, 1, key, test_vec->klen, iv, src, test_vec->clen, test_vec->aadlen,
                  dst, test_vec->len);
    dsb(sy);
    if (memcmp(dst + test_vec->aadlen, test_vec->ptext, test_vec->len)) {
        pr_err("decrypt %s err!\n", alg_name);
        //print_buf("src", src, test_vec->aadlen + test_vec->clen);
        //print_buf("dst", dst - 16, test_vec->aadlen+test_vec->clen + 32);
        goto out;
    }
#endif
    pr_info("Test %s klen %d successed!\n", alg_name, test_vec->klen);
out:
    kfree(src);
}



void aead_test_one(char *alg_name, ske_testvec_t *test_vec)
{
    u8 *src, *dst, *key, *iv;

    src = kmalloc(1024 * 3, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    memset(src, 0, 1024 * 3);
    dst = src + 1024;
    key = dst + 1024;
    iv = key + 512;

    // test enc
    if (test_vec->aadlen) {
        memcpy(src, test_vec->assoc, test_vec->aadlen);
    }
    if (test_vec->len)
        memcpy(src + test_vec->aadlen, test_vec->ptext, test_vec->len);
    
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }
    memcpy(key, test_vec->key, test_vec->klen);

    // test one segment
    aead_test(alg_name, 1, key, test_vec->klen, iv, src, test_vec->len, test_vec->aadlen,
                  dst, test_vec->clen);
    if (memcmp(dst + test_vec->aadlen, test_vec->ctext, test_vec->clen)) {
        pr_err("encrypt %s err!\n", alg_name);
	//print_buf("src", src, test_vec->aadlen + test_vec->len);
	//print_buf("dst", dst - 16, test_vec->aadlen+test_vec->clen + 32);
	//print_buf("result", test_vec->ctext, test_vec->clen);
        goto out;
    }

#if 1
    // test dec
    if (test_vec->aadlen) {
	memcpy(src, test_vec->assoc, test_vec->aadlen);
    }
    if (test_vec->clen) {
        memcpy(src + test_vec->aadlen, test_vec->ctext, test_vec->clen);
    }

    memcpy(key, test_vec->key, test_vec->klen);
    memset(dst, 0, 1024);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }
    aead_test(alg_name, 0, key, test_vec->klen, iv, src, test_vec->clen, test_vec->aadlen,
                  dst, test_vec->len);
    dsb(sy);
    if (memcmp(dst + test_vec->aadlen, test_vec->ptext, test_vec->len)) {
        pr_err("decrypt %s err!\n", alg_name);
	print_buf("src", src, test_vec->aadlen + test_vec->clen);
	print_buf("dst", dst - 16, test_vec->aadlen+test_vec->clen + 32);
        goto out;
    }
#endif
    pr_info("Test %s klen %d successed!\n", alg_name, test_vec->klen);
out:
    kfree(src);
}

void aead_check_test(char *alg, char *mode, u32 klen, u32 slen, u32 elen)
{
    char alg_name[64] = {0};
    u32 alg_mode, i;
    u32 test = 0;
    ske_testvec_t *test_vec;
    int test_alg = 0;

    if (0 == memcmp(alg,"sm4",3)) {
        sce_get_alg_name(alg_name, "sm4", mode);
	test_alg = 0;
    } else if (0 == memcmp(alg, "aes",3)) {
	sce_get_alg_name(alg_name, "aes", mode);
	test_alg = 1;
    }
    alg_mode = get_alg_mode(mode);
    if (alg_mode == 0xff) {
        pr_err("Invalid mode %s", mode);
        return;
    }

    //pr_info("get mode %s, klen %d slen %d elen %d alg_mode %d", mode, klen,
    //        slen, elen, alg_mode);
    // test specified vec
    if (test_alg == 1) {
    for (i = 0; i < sizeof(aead_aes_tv_template) / sizeof(aead_aes_tv_template[0]); i++) {
        test_vec = &aead_aes_tv_template[i];
        if (test_vec->mode == alg_mode) {
            if (!klen || test_vec->klen == klen) {
		aead_test_one(alg_name, test_vec);

                test = 1;
            }
        }
    }
    } else {
    for (i = 0; i < sizeof(aead_sm4_tv_template) / sizeof(aead_sm4_tv_template[0]); i++) {
        test_vec = &aead_sm4_tv_template[i];
        if (test_vec->mode == alg_mode) {
            if (!klen || test_vec->klen == klen) {
                aead_test_one(alg_name, test_vec);
                test = 1;
            }
        }
    }
    }

    if (0 == test) {
        pr_err("Missing test vector for alg %s klen %d", alg_name, klen);
    }
}
