#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "sca_test.h"
#include "../common/common.h"

ske_testvec_t aes_tv_template[] = {
    {
        /* From FIPS-197 */
        .klen = 16,
        .len = 16,
        .alg = SCE_AES_128,
        .mode = ECB,
        .key = "\x00\x01\x02\x03\x04\x05\x06\x07"
               "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
        .ptext = "\x00\x11\x22\x33\x44\x55\x66\x77"
                 "\x88\x99\xaa\xbb\xcc\xdd\xee\xff",
        .ctext = "\x69\xc4\xe0\xd8\x6a\x7b\x04\x30"
                 "\xd8\xcd\xb7\x80\x70\xb4\xc5\x5a",
    },
    {
        .klen = 24,
        .len = 16,
        .alg = SCE_AES_192,
        .mode = ECB,
        .key = "\x00\x01\x02\x03\x04\x05\x06\x07"
               "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
               "\x10\x11\x12\x13\x14\x15\x16\x17",
        .ptext = "\x00\x11\x22\x33\x44\x55\x66\x77"
                 "\x88\x99\xaa\xbb\xcc\xdd\xee\xff",
        .ctext = "\xdd\xa9\x7c\xa4\x86\x4c\xdf\xe0"
                 "\x6e\xaf\x70\xa0\xec\x0d\x71\x91",
    },
    {
        .klen = 32,
        .len = 16,
        .alg = SCE_AES_256,
        .mode = ECB,
        .key = "\x00\x01\x02\x03\x04\x05\x06\x07"
               "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
               "\x10\x11\x12\x13\x14\x15\x16\x17"
               "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f",
        .ptext = "\x00\x11\x22\x33\x44\x55\x66\x77"
                 "\x88\x99\xaa\xbb\xcc\xdd\xee\xff",
        .ctext = "\x8e\xa2\xb7\xca\x51\x67\x45\xbf"
                 "\xea\xfc\x49\x90\x4b\x49\x60\x89",
    },
    {
        /* Generated with Crypto++ */
        .klen = 32,
        .len = 496,
        .alg = SCE_AES_256,
        .mode = ECB,
        .key = "\xA6\xC9\x83\xA6\xC9\xEC\x0F\x32"
               "\x55\x0F\x32\x55\x78\x9B\xBE\x78"
               "\x9B\xBE\xE1\x04\x27\xE1\x04\x27"
               "\x4A\x6D\x90\x4A\x6D\x90\xB3\xD6",
        .ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
                 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
                 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
                 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
                 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
                 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
                 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
                 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
                 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
                 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
                 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
                 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
                 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
                 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
                 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
                 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
                 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
                 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
                 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
                 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
                 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
                 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
                 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
                 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
                 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
                 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
                 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
                 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
                 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
                 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
                 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB"
                 "\x54\xE0\x49\xB2\x1B\xA7\x10\x79"
                 "\x05\x6E\xD7\x40\xCC\x35\x9E\x07"
                 "\x93\xFC\x65\xF1\x5A\xC3\x2C\xB8"
                 "\x21\x8A\x16\x7F\xE8\x51\xDD\x46"
                 "\xAF\x18\xA4\x0D\x76\x02\x6B\xD4"
                 "\x3D\xC9\x32\x9B\x04\x90\xF9\x62"
                 "\xEE\x57\xC0\x29\xB5\x1E\x87\x13"
                 "\x7C\xE5\x4E\xDA\x43\xAC\x15\xA1"
                 "\x0A\x73\xFF\x68\xD1\x3A\xC6\x2F"
                 "\x98\x01\x8D\xF6\x5F\xEB\x54\xBD"
                 "\x26\xB2\x1B\x84\x10\x79\xE2\x4B"
                 "\xD7\x40\xA9\x12\x9E\x07\x70\xFC"
                 "\x65\xCE\x37\xC3\x2C\x95\x21\x8A"
                 "\xF3\x5C\xE8\x51\xBA\x23\xAF\x18"
                 "\x81\x0D\x76\xDF\x48\xD4\x3D\xA6"
                 "\x0F\x9B\x04\x6D\xF9\x62\xCB\x34"
                 "\xC0\x29\x92\x1E\x87\xF0\x59\xE5"
                 "\x4E\xB7\x20\xAC\x15\x7E\x0A\x73"
                 "\xDC\x45\xD1\x3A\xA3\x0C\x98\x01"
                 "\x6A\xF6\x5F\xC8\x31\xBD\x26\x8F"
                 "\x1B\x84\xED\x56\xE2\x4B\xB4\x1D"
                 "\xA9\x12\x7B\x07\x70\xD9\x42\xCE"
                 "\x37\xA0\x09\x95\xFE\x67\xF3\x5C"
                 "\xC5\x2E\xBA\x23\x8C\x18\x81\xEA"
                 "\x53\xDF\x48\xB1\x1A\xA6\x0F\x78"
                 "\x04\x6D\xD6\x3F\xCB\x34\x9D\x06"
                 "\x92\xFB\x64\xF0\x59\xC2\x2B\xB7"
                 "\x20\x89\x15\x7E\xE7\x50\xDC\x45"
                 "\xAE\x17\xA3\x0C\x75\x01\x6A\xD3"
                 "\x3C\xC8\x31\x9A\x03\x8F\xF8\x61"
                 "\xED\x56\xBF\x28\xB4\x1D\x86\x12",
        .ctext = "\x71\x73\xF7\xDB\x24\x93\x21\x6D"
                 "\x61\x1E\xBB\x63\x42\x79\xDB\x64"
                 "\x6F\x82\xC0\xCA\xA3\x9B\xFA\x0B"
                 "\xD9\x08\xC7\x4A\x90\xAE\x8F\x5F"
                 "\x5E\x06\xF0\x5F\x31\x51\x18\x37"
                 "\x45\xD7\xCA\x3A\xFD\x6C\x3F\xE1"
                 "\xDD\x8D\x22\x65\x2B\x00\x50\xCE"
                 "\xBA\x28\x67\xD7\xCE\x0E\x0D\xEA"
                 "\x78\x69\x7F\xAE\x8F\x8B\x69\x37"
                 "\x75\xE0\xDC\x96\xE0\xB7\xF4\x09"
                 "\xCB\x6D\xA2\xFB\xDA\xAF\x09\xF8"
                 "\x81\x82\x27\xFA\x45\x9C\x29\xA4"
                 "\x22\x8B\x78\x69\x5B\x46\xF9\x39"
                 "\x1B\xCC\xF9\x1D\x09\xEB\xBC\x5C"
                 "\x41\x72\x51\x97\x1D\x07\x49\xA0"
                 "\x1B\x8E\x65\x4B\xB2\x6A\x12\x03"
                 "\x6A\x60\x95\xAC\xBD\xAC\x1A\x64"
                 "\xDE\x5A\xA5\xF0\x83\x2F\xCB\xCA"
                 "\x22\x74\xA6\x6C\x9B\x73\xCE\x3F"
                 "\xE1\x8B\x22\x17\x59\x0C\x47\x89"
                 "\x33\xA1\xD6\x47\x03\x19\x4F\xA8"
                 "\x67\x69\xF0\x5B\xF0\x20\xAD\x06"
                 "\x27\x81\x92\xD8\xC5\xBA\x98\x12"
                 "\xBE\x24\xB5\x2F\x75\x02\xC2\xAD"
                 "\x12\x2F\x07\x32\xEE\x39\xAF\x64"
                 "\x05\x8F\xB3\xD4\xEB\x1B\x46\x6E"
                 "\xD9\x21\xF9\xC4\xB7\xC9\x45\x68"
                 "\xB4\xA1\x74\x9F\x82\x47\xEB\xCC"
                 "\xBD\x0A\x14\x95\x0F\x8B\xA8\x2F"
                 "\x4B\x1B\xA7\xBF\x82\xA6\x43\x0C"
                 "\xB9\x39\x4A\xA8\x10\x6F\x50\x7B"
                 "\x25\xFB\x26\x81\xE0\x2F\xF0\x96"
                 "\x8D\x8B\xAC\x92\x0F\xF6\xED\x64"
                 "\x63\x29\x4C\x8E\x18\x13\xC5\xBF"
                 "\xFC\xA0\xD9\xBF\x7C\x3A\x0E\x29"
                 "\x6F\xD1\x6C\x6F\xA5\xDA\xBF\xB1"
                 "\x30\xEA\x44\x2D\xC3\x8F\x16\xE1"
                 "\x66\xFA\xA3\x21\x3E\xFC\x13\xCA"
                 "\xF0\xF6\xF0\x59\xBD\x8F\x38\x50"
                 "\x31\xCB\x69\x3F\x96\x15\xD6\xF5"
                 "\xAE\xFF\xF6\xAA\x41\x85\x4C\x10"
                 "\x58\xE3\xF9\x44\xE6\x28\xDA\x9A"
                 "\xDC\x6A\x80\x34\x73\x97\x1B\xC5"
                 "\xCA\x26\x16\x77\x0E\x60\xAB\x89"
                 "\x0F\x04\x27\xBD\xCE\x3E\x71\xB4"
                 "\xA0\xD7\x22\x7E\xDB\xEB\x24\x70"
                 "\x42\x71\x51\x78\x70\xB3\xE0\x3D"
                 "\x84\x8E\x8D\x7B\xD0\x6D\xEA\x92"
                 "\x11\x08\x42\x4F\xE5\xAD\x26\x92"
                 "\xD2\x00\xAE\xA8\xE3\x4B\x37\x47"
                 "\x22\xC1\x95\xC1\x63\x7F\xCB\x03"
                 "\xF3\xE3\xD7\x9D\x60\xC7\xBC\xEA"
                 "\x35\xA2\xFD\x45\x52\x39\x13\x6F"
                 "\xC1\x53\xF3\x53\xDF\x33\x84\xD7"
                 "\xD2\xC8\x37\xB0\x75\xE3\x41\x46"
                 "\xB3\xC7\x83\x2E\x8A\xBB\xA4\xE5"
                 "\x7F\x3C\xFD\x8B\xEB\xEA\x63\xBD"
                 "\xB7\x46\xE7\xBF\x09\x9C\x0D\x0F"
                 "\x40\x86\x7F\x51\xE1\x11\x9C\xCB"
                 "\x88\xE6\x68\x47\xE3\x2B\xC5\xFF"
                 "\x09\x79\xA0\x43\x5C\x0D\x08\x58"
                 "\x17\xBB\xC0\x6B\x62\x3F\x56\xE9",
    },

    {
        /* From RFC 3602 */
        .klen = 16,
        .len = 16,
        .ilen = 16,
        .alg = SCE_AES_128,
        .mode = CBC,
        .key = "\x06\xa9\x21\x40\x36\xb8\xa1\x5b"
               "\x51\x2e\x03\xd5\x34\x12\x00\x06",
        .iv = "\x3d\xaf\xba\x42\x9d\x9e\xb4\x30"
              "\xb4\x22\xda\x80\x2c\x9f\xac\x41",
        .ptext = "Single block msg",
        .ctext = "\xe3\x53\x77\x9c\x10\x79\xae\xb8"
                 "\x27\x08\x94\x2d\xbe\x77\x18\x1a",
    },
    {
        .klen = 16,
        .len = 32,
        .ilen = 16,
        .alg = SCE_AES_128,
        .mode = CBC,
        .key = "\xc2\x86\x69\x6d\x88\x7c\x9a\xa0"
               "\x61\x1b\xbb\x3e\x20\x25\xa4\x5a",
        .iv = "\x56\x2e\x17\x99\x6d\x09\x3d\x28"
              "\xdd\xb3\xba\x69\x5a\x2e\x6f\x58",
        .ptext = "\x00\x01\x02\x03\x04\x05\x06\x07"
                 "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
                 "\x10\x11\x12\x13\x14\x15\x16\x17"
                 "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f",
        .ctext = "\xd2\x96\xcd\x94\xc2\xcc\xcf\x8a"
                 "\x3a\x86\x30\x28\xb5\xe1\xdc\x0a"
                 "\x75\x86\x60\x2d\x25\x3c\xff\xf9"
                 "\x1b\x82\x66\xbe\xa6\xd6\x1a\xb1",
    },
    {
        /* From NIST SP800-38A */
        .klen = 24,
        .len = 64,
        .ilen = 16,
        .alg = SCE_AES_192,
        .mode = CBC,
        .key = "\x8e\x73\xb0\xf7\xda\x0e\x64\x52"
               "\xc8\x10\xf3\x2b\x80\x90\x79\xe5"
               "\x62\xf8\xea\xd2\x52\x2c\x6b\x7b",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
        .ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
                 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
                 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
                 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
        .ctext = "\x4f\x02\x1d\xb2\x43\xbc\x63\x3d"
                 "\x71\x78\x18\x3a\x9f\xa0\x71\xe8"
                 "\xb4\xd9\xad\xa9\xad\x7d\xed\xf4"
                 "\xe5\xe7\x38\x76\x3f\x69\x14\x5a"
                 "\x57\x1b\x24\x20\x12\xfb\x7a\xe0"
                 "\x7f\xa9\xba\xac\x3d\xf1\x02\xe0"
                 "\x08\xb0\xe2\x79\x88\x59\x88\x81"
                 "\xd9\x20\xa9\xe6\x4f\x56\x15\xcd",
    },
    {
        .klen = 32,
        .len = 64,
        .ilen = 16,
        .alg = SCE_AES_256,
        .mode = CBC,
        .key = "\x60\x3d\xeb\x10\x15\xca\x71\xbe"
               "\x2b\x73\xae\xf0\x85\x7d\x77\x81"
               "\x1f\x35\x2c\x07\x3b\x61\x08\xd7"
               "\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
        .ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
                 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
                 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
                 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
        .ctext = "\xf5\x8c\x4c\x04\xd6\xe5\xf1\xba"
                 "\x77\x9e\xab\xfb\x5f\x7b\xfb\xd6"
                 "\x9c\xfc\x4e\x96\x7e\xdb\x80\x8d"
                 "\x67\x9f\x77\x7b\xc6\x70\x2c\x7d"
                 "\x39\xf2\x33\x69\xa9\xd9\xba\xcf"
                 "\xa5\x30\xe2\x63\x04\x23\x14\x61"
                 "\xb2\xeb\x05\xe2\xc3\x9b\xe9\xfc"
                 "\xda\x6c\x19\x07\x8c\x6a\x9d\x1b",
    },
    {
        /* Generated with Crypto++ */
        .klen = 32,
        .len = 496,
        .ilen = 16,
        .alg = SCE_AES_256,
        .mode = CBC,
        .key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55"
               "\x0F\x32\x55\x78\x9B\xBE\x78\x9B"
               "\xBE\xE1\x04\x27\xE1\x04\x27\x4A"
               "\x6D\x90\x4A\x6D\x90\xB3\xD6\xF9",
        .iv = "\xE7\x82\x1D\xB8\x53\x11\xAC\x47"
              "\xE2\x7D\x18\xD6\x71\x0C\xA7\x42",
        .ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
                 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
                 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
                 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
                 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
                 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
                 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
                 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
                 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
                 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
                 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
                 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
                 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
                 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
                 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
                 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
                 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
                 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
                 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
                 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
                 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
                 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
                 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
                 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
                 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
                 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
                 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
                 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
                 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
                 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
                 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB"
                 "\x54\xE0\x49\xB2\x1B\xA7\x10\x79"
                 "\x05\x6E\xD7\x40\xCC\x35\x9E\x07"
                 "\x93\xFC\x65\xF1\x5A\xC3\x2C\xB8"
                 "\x21\x8A\x16\x7F\xE8\x51\xDD\x46"
                 "\xAF\x18\xA4\x0D\x76\x02\x6B\xD4"
                 "\x3D\xC9\x32\x9B\x04\x90\xF9\x62"
                 "\xEE\x57\xC0\x29\xB5\x1E\x87\x13"
                 "\x7C\xE5\x4E\xDA\x43\xAC\x15\xA1"
                 "\x0A\x73\xFF\x68\xD1\x3A\xC6\x2F"
                 "\x98\x01\x8D\xF6\x5F\xEB\x54\xBD"
                 "\x26\xB2\x1B\x84\x10\x79\xE2\x4B"
                 "\xD7\x40\xA9\x12\x9E\x07\x70\xFC"
                 "\x65\xCE\x37\xC3\x2C\x95\x21\x8A"
                 "\xF3\x5C\xE8\x51\xBA\x23\xAF\x18"
                 "\x81\x0D\x76\xDF\x48\xD4\x3D\xA6"
                 "\x0F\x9B\x04\x6D\xF9\x62\xCB\x34"
                 "\xC0\x29\x92\x1E\x87\xF0\x59\xE5"
                 "\x4E\xB7\x20\xAC\x15\x7E\x0A\x73"
                 "\xDC\x45\xD1\x3A\xA3\x0C\x98\x01"
                 "\x6A\xF6\x5F\xC8\x31\xBD\x26\x8F"
                 "\x1B\x84\xED\x56\xE2\x4B\xB4\x1D"
                 "\xA9\x12\x7B\x07\x70\xD9\x42\xCE"
                 "\x37\xA0\x09\x95\xFE\x67\xF3\x5C"
                 "\xC5\x2E\xBA\x23\x8C\x18\x81\xEA"
                 "\x53\xDF\x48\xB1\x1A\xA6\x0F\x78"
                 "\x04\x6D\xD6\x3F\xCB\x34\x9D\x06"
                 "\x92\xFB\x64\xF0\x59\xC2\x2B\xB7"
                 "\x20\x89\x15\x7E\xE7\x50\xDC\x45"
                 "\xAE\x17\xA3\x0C\x75\x01\x6A\xD3"
                 "\x3C\xC8\x31\x9A\x03\x8F\xF8\x61"
                 "\xED\x56\xBF\x28\xB4\x1D\x86\x12",
        .ctext = "\xEA\x65\x8A\x19\xB0\x66\xC1\x3F"
                 "\xCE\xF1\x97\x75\xC1\xFD\xB5\xAF"
                 "\x52\x65\xF7\xFF\xBC\xD8\x2D\x9F"
                 "\x2F\xB9\x26\x9B\x6F\x10\xB7\xB8"
                 "\x26\xA1\x02\x46\xA2\xAD\xC6\xC0"
                 "\x11\x15\xFF\x6D\x1E\x82\x04\xA6"
                 "\xB1\x74\xD1\x08\x13\xFD\x90\x7C"
                 "\xF5\xED\xD3\xDB\x5A\x0A\x0C\x2F"
                 "\x0A\x70\xF1\x88\x07\xCF\x21\x26"
                 "\x40\x40\x8A\xF5\x53\xF7\x24\x4F"
                 "\x83\x38\x43\x5F\x08\x99\xEB\xE3"
                 "\xDC\x02\x64\x67\x50\x6E\x15\xC3"
                 "\x01\x1A\xA0\x81\x13\x65\xA6\x73"
                 "\x71\xA6\x3B\x91\x83\x77\xBE\xFA"
                 "\xDB\x71\x73\xA6\xC1\xAE\x43\xC3"
                 "\x36\xCE\xD6\xEB\xF9\x30\x1C\x4F"
                 "\x80\x38\x5E\x9C\x6E\xAB\x98\x2F"
                 "\x53\xAF\xCF\xC8\x9A\xB8\x86\x43"
                 "\x3E\x86\xE7\xA1\xF4\x2F\x30\x40"
                 "\x03\xA8\x6C\x50\x42\x9F\x77\x59"
                 "\x89\xA0\xC5\xEC\x9A\xB8\xDD\x99"
                 "\x16\x24\x02\x07\x48\xAE\xF2\x31"
                 "\x34\x0E\xC3\x85\xFE\x1C\x95\x99"
                 "\x87\x58\x98\x8B\xE7\xC6\xC5\x70"
                 "\x73\x81\x07\x7C\x56\x2F\xD8\x1B"
                 "\xB7\xB9\x2B\xAB\xE3\x01\x87\x0F"
                 "\xD8\xBB\xC0\x0D\xAC\x2C\x2F\x98"
                 "\x3C\x0B\xA2\x99\x4A\x8C\xF7\x04"
                 "\xE0\xE0\xCF\xD1\x81\x5B\xFE\xF5"
                 "\x24\x04\xFD\xB8\xDF\x13\xD8\xCD"
                 "\xF1\xE3\x3D\x98\x50\x02\x77\x9E"
                 "\xBC\x22\xAB\xFA\xC2\x43\x1F\x66"
                 "\x20\x02\x23\xDA\xDF\xA0\x89\xF6"
                 "\xD8\xF3\x45\x24\x53\x6F\x16\x77"
                 "\x02\x3E\x7B\x36\x5F\xA0\x3B\x78"
                 "\x63\xA2\xBD\xB5\xA4\xCA\x1E\xD3"
                 "\x57\xBC\x0B\x9F\x43\x51\x28\x4F"
                 "\x07\x50\x6C\x68\x12\x07\xCF\xFA"
                 "\x6B\x72\x0B\xEB\xF8\x88\x90\x2C"
                 "\x7E\xF5\x91\xD1\x03\xD8\xD5\xBD"
                 "\x22\x39\x7B\x16\x03\x01\x69\xAF"
                 "\x3D\x38\x66\x28\x0C\xBE\x5B\xC5"
                 "\x03\xB4\x2F\x51\x8A\x56\x17\x2B"
                 "\x88\x42\x6D\x40\x68\x8F\xD0\x11"
                 "\x19\xF9\x1F\x43\x79\x95\x31\xFA"
                 "\x28\x7A\x3D\xF7\x66\xEB\xEF\xAC"
                 "\x06\xB2\x01\xAD\xDB\x68\xDB\xEC"
                 "\x8D\x53\x6E\x72\x68\xA3\xC7\x63"
                 "\x43\x2B\x78\xE0\x04\x29\x8F\x72"
                 "\xB2\x2C\xE6\x84\x03\x30\x6D\xCD"
                 "\x26\x92\x37\xE1\x2F\xBB\x8B\x9D"
                 "\xE4\x4C\xF6\x93\xBC\xD9\xAD\x44"
                 "\x52\x65\xC7\xB0\x0E\x3F\x0E\x61"
                 "\x56\x5D\x1C\x6D\xA7\x05\x2E\xBC"
                 "\x58\x08\x15\xAB\x12\xAB\x17\x4A"
                 "\x5E\x1C\xF2\xCD\xB8\xA2\xAE\xFB"
                 "\x9B\x2E\x0E\x85\x34\x80\x0E\x3F"
                 "\x4C\xB8\xDB\xCE\x1C\x90\xA1\x61"
                 "\x6C\x69\x09\x35\x9E\xD4\xF4\xAD"
                 "\xBC\x06\x41\xE3\x01\xB4\x4E\x0A"
                 "\xE0\x1F\x91\xF8\x82\x96\x2D\x65"
                 "\xA3\xAA\x13\xCC\x50\xFF\x7B\x02",
    },

    {
        /* From NIST Special Publication 800-38A, Appendix F.5 */
        .klen = 16,
        .len = 64,
        .ilen = 16,
        .alg = SCE_AES_128,
        .mode = CTR,
        .key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
               "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
        .iv = "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
              "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff",
        .ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
                 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
                 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
                 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
        .ctext = "\x87\x4d\x61\x91\xb6\x20\xe3\x26"
                 "\x1b\xef\x68\x64\x99\x0d\xb6\xce"
                 "\x98\x06\xf6\x6b\x79\x70\xfd\xff"
                 "\x86\x17\x18\x7b\xb9\xff\xfd\xff"
                 "\x5a\xe4\xdf\x3e\xdb\xd5\xd3\x5e"
                 "\x5b\x4f\x09\x02\x0d\xb0\x3e\xab"
                 "\x1e\x03\x1d\xda\x2f\xbe\x03\xd1"
                 "\x79\x21\x70\xa0\xf3\x00\x9c\xee",
    },
    {
        .klen = 24,
        .len = 64,
        .ilen = 16,
        .alg = SCE_AES_192,
        .mode = CTR,
        .key = "\x8e\x73\xb0\xf7\xda\x0e\x64\x52"
               "\xc8\x10\xf3\x2b\x80\x90\x79\xe5"
               "\x62\xf8\xea\xd2\x52\x2c\x6b\x7b",
        .iv = "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
              "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff",
        .ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
                 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
                 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
                 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
        .ctext = "\x1a\xbc\x93\x24\x17\x52\x1c\xa2"
                 "\x4f\x2b\x04\x59\xfe\x7e\x6e\x0b"
                 "\x09\x03\x39\xec\x0a\xa6\xfa\xef"
                 "\xd5\xcc\xc2\xc6\xf4\xce\x8e\x94"
                 "\x1e\x36\xb2\x6b\xd1\xeb\xc6\x70"
                 "\xd1\xbd\x1d\x66\x56\x20\xab\xf7"
                 "\x4f\x78\xa7\xf6\xd2\x98\x09\x58"
                 "\x5a\x97\xda\xec\x58\xc6\xb0\x50",
    },
    {
        .klen = 32,
        .len = 64,
        .ilen = 16,
        .alg = SCE_AES_256,
        .mode = CTR,
        .key = "\x60\x3d\xeb\x10\x15\xca\x71\xbe"
               "\x2b\x73\xae\xf0\x85\x7d\x77\x81"
               "\x1f\x35\x2c\x07\x3b\x61\x08\xd7"
               "\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
        .iv = "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
              "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff",
        .ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
                 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
                 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
                 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
        .ctext = "\x60\x1e\xc3\x13\x77\x57\x89\xa5"
                 "\xb7\xa7\xf5\x04\xbb\xf3\xd2\x28"
                 "\xf4\x43\xe3\xca\x4d\x62\xb5\x9a"
                 "\xca\x84\xe9\x90\xca\xca\xf5\xc5"
                 "\x2b\x09\x30\xda\xa2\x3d\xe9\x4c"
                 "\xe8\x70\x17\xba\x2d\x84\x98\x8d"
                 "\xdf\xc9\xc5\x8d\xb6\x7a\xad\xa6"
                 "\x13\xc2\xdd\x08\x45\x79\x41\xa6",
    },
    {
        /* Generated with Crypto++ */
        .klen = 32,
        .len = 496,
        .ilen = 16,
        .alg = SCE_AES_256,
        .mode = CTR,
        .key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55"
               "\x0F\x32\x55\x78\x9B\xBE\x78\x9B"
               "\xBE\xE1\x04\x27\xE1\x04\x27\x4A"
               "\x6D\x90\x4A\x6D\x90\xB3\xD6\xF9",
        .iv = "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"
              "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFD",
        .ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
                 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
                 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
                 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
                 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
                 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
                 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
                 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
                 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
                 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
                 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
                 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
                 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
                 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
                 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
                 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
                 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
                 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
                 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
                 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
                 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
                 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
                 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
                 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
                 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
                 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
                 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
                 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
                 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
                 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
                 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB"
                 "\x54\xE0\x49\xB2\x1B\xA7\x10\x79"
                 "\x05\x6E\xD7\x40\xCC\x35\x9E\x07"
                 "\x93\xFC\x65\xF1\x5A\xC3\x2C\xB8"
                 "\x21\x8A\x16\x7F\xE8\x51\xDD\x46"
                 "\xAF\x18\xA4\x0D\x76\x02\x6B\xD4"
                 "\x3D\xC9\x32\x9B\x04\x90\xF9\x62"
                 "\xEE\x57\xC0\x29\xB5\x1E\x87\x13"
                 "\x7C\xE5\x4E\xDA\x43\xAC\x15\xA1"
                 "\x0A\x73\xFF\x68\xD1\x3A\xC6\x2F"
                 "\x98\x01\x8D\xF6\x5F\xEB\x54\xBD"
                 "\x26\xB2\x1B\x84\x10\x79\xE2\x4B"
                 "\xD7\x40\xA9\x12\x9E\x07\x70\xFC"
                 "\x65\xCE\x37\xC3\x2C\x95\x21\x8A"
                 "\xF3\x5C\xE8\x51\xBA\x23\xAF\x18"
                 "\x81\x0D\x76\xDF\x48\xD4\x3D\xA6"
                 "\x0F\x9B\x04\x6D\xF9\x62\xCB\x34"
                 "\xC0\x29\x92\x1E\x87\xF0\x59\xE5"
                 "\x4E\xB7\x20\xAC\x15\x7E\x0A\x73"
                 "\xDC\x45\xD1\x3A\xA3\x0C\x98\x01"
                 "\x6A\xF6\x5F\xC8\x31\xBD\x26\x8F"
                 "\x1B\x84\xED\x56\xE2\x4B\xB4\x1D"
                 "\xA9\x12\x7B\x07\x70\xD9\x42\xCE"
                 "\x37\xA0\x09\x95\xFE\x67\xF3\x5C"
                 "\xC5\x2E\xBA\x23\x8C\x18\x81\xEA"
                 "\x53\xDF\x48\xB1\x1A\xA6\x0F\x78"
                 "\x04\x6D\xD6\x3F\xCB\x34\x9D\x06"
                 "\x92\xFB\x64\xF0\x59\xC2\x2B\xB7"
                 "\x20\x89\x15\x7E\xE7\x50\xDC\x45"
                 "\xAE\x17\xA3\x0C\x75\x01\x6A\xD3"
                 "\x3C\xC8\x31\x9A\x03\x8F\xF8\x61"
                 "\xED\x56\xBF\x28\xB4\x1D\x86\x12",
        .ctext = "\x04\xF3\xD3\x88\x17\xEF\xDC\xEF"
                 "\x8B\x04\xF8\x3A\x66\x8D\x1A\x53"
                 "\x57\x1F\x4B\x23\xE4\xA0\xAF\xF9"
                 "\x69\x95\x35\x98\x8D\x4D\x8C\xC1"
                 "\xF0\xB2\x7F\x80\xBB\x54\x28\xA2"
                 "\x7A\x1B\x9F\x77\xEC\x0E\x6E\xDE"
                 "\xF0\xEC\xB8\xE4\x20\x62\xEE\xDB"
                 "\x5D\xF5\xDD\xE3\x54\xFC\xDD\xEB"
                 "\x6A\xEE\x65\xA1\x21\xD6\xD7\x81"
                 "\x47\x61\x12\x4D\xC2\x8C\xFA\x78"
                 "\x1F\x28\x02\x01\xC3\xFC\x1F\xEC"
                 "\x0F\x10\x4F\xB3\x12\x45\xC6\x3B"
                 "\x7E\x08\xF9\x5A\xD0\x5D\x73\x2D"
                 "\x58\xA4\xE5\xCB\x1C\xB4\xCE\x74"
                 "\x32\x41\x1F\x31\x9C\x08\xA2\x5D"
                 "\x67\xEB\x72\x1D\xF8\xE7\x70\x54"
                 "\x34\x4B\x31\x69\x84\x66\x96\x44"
                 "\x56\xCC\x1E\xD9\xE6\x13\x6A\xB9"
                 "\x2D\x0A\x05\x45\x2D\x90\xCC\xDF"
                 "\x16\x5C\x5F\x79\x34\x52\x54\xFE"
                 "\xFE\xCD\xAD\x04\x2E\xAD\x86\x06"
                 "\x1F\x37\xE8\x28\xBC\xD3\x8F\x5B"
                 "\x92\x66\x87\x3B\x8A\x0A\x1A\xCC"
                 "\x6E\xAB\x9F\x0B\xFA\x5C\xE6\xFD"
                 "\x3C\x98\x08\x12\xEC\xAA\x9E\x11"
                 "\xCA\xB2\x1F\xCE\x5E\x5B\xB2\x72"
                 "\x9C\xCC\x5D\xC5\xE0\x32\xC0\x56"
                 "\xD5\x45\x16\xD2\xAF\x13\x66\xF7"
                 "\x8C\x67\xAC\x79\xB2\xAF\x56\x27"
                 "\x3F\xCC\xFE\xCB\x1E\xC0\x75\xF1"
                 "\xA7\xC9\xC3\x1D\x8E\xDD\xF9\xD4"
                 "\x42\xC8\x21\x08\x16\xF7\x01\xD7"
                 "\xAC\x8E\x3F\x1D\x56\xC1\x06\xE4"
                 "\x9C\x62\xD6\xA5\x6A\x50\x44\xB3"
                 "\x35\x1C\x82\xB9\x10\xF9\x42\xA1"
                 "\xFC\x74\x9B\x44\x4F\x25\x02\xE3"
                 "\x08\xF5\xD4\x32\x39\x08\x11\xE8"
                 "\xD2\x6B\x50\x53\xD4\x08\xD1\x6B"
                 "\x3A\x4A\x68\x7B\x7C\xCD\x46\x5E"
                 "\x0D\x07\x19\xDB\x67\xD7\x98\x91"
                 "\xD7\x17\x10\x9B\x7B\x8A\x9B\x33"
                 "\xAE\xF3\x00\xA6\xD4\x15\xD9\xEA"
                 "\x85\x99\x22\xE8\x91\x38\x70\x83"
                 "\x93\x01\x24\x6C\xFA\x9A\xB9\x07"
                 "\xEA\x8D\x3B\xD9\x2A\x43\x59\x16"
                 "\x2F\x69\xEE\x84\x36\x44\x76\x98"
                 "\xF3\x04\x2A\x7C\x74\x3D\x29\x2B"
                 "\x0D\xAD\x8F\x44\x82\x9E\x57\x8D"
                 "\xAC\xED\x18\x1F\x50\xA4\xF5\x98"
                 "\x1F\xBD\x92\x91\x1B\x2D\xA6\xD6"
                 "\xD2\xE3\x02\xAA\x92\x3B\xC6\xB3"
                 "\x1B\x39\x72\xD5\x26\xCA\x04\xE0"
                 "\xFC\x58\x78\xBB\xB1\x3F\xA1\x9C"
                 "\x42\x24\x3E\x2E\x22\xBB\x4B\xBA"
                 "\xF4\x52\x0A\xE6\xAE\x47\xB4\x7D"
                 "\x1D\xA8\xBE\x81\x1A\x75\xDA\xAC"
                 "\xA6\x25\x1E\xEF\x3A\xC0\x6C\x63"
                 "\xEF\xDC\xC9\x79\x10\x26\xE8\x61"
                 "\x29\xFC\xA4\x05\xDF\x7D\x5C\x63"
                 "\x10\x09\x9B\x46\x9B\xF2\x2C\x2B"
                 "\xFA\x3A\x05\x4C\xFA\xD1\xFF\xFE"
                 "\xF1\x4C\xE5\xB2\x91\x64\x0C\x51",
    },
    {
        /* Generated with Crypto++ */
        .klen = 32,
        .len = 499,
        .ilen = 16,
        .alg = SCE_AES_256,
        .mode = CTR,
        .key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55"
               "\x0F\x32\x55\x78\x9B\xBE\x78\x9B"
               "\xBE\xE1\x04\x27\xE1\x04\x27\x4A"
               "\x6D\x90\x4A\x6D\x90\xB3\xD6\xF9",
        .iv = "\xE7\x82\x1D\xB8\x53\x11\xAC\x47"
              "\xE2\x7D\x18\xD6\x71\x0C\xA7\x42",
        .ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
                 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
                 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
                 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
                 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
                 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
                 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
                 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
                 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
                 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
                 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
                 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
                 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
                 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
                 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
                 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
                 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
                 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
                 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
                 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
                 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
                 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
                 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
                 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
                 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
                 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
                 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
                 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
                 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
                 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
                 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB"
                 "\x54\xE0\x49\xB2\x1B\xA7\x10\x79"
                 "\x05\x6E\xD7\x40\xCC\x35\x9E\x07"
                 "\x93\xFC\x65\xF1\x5A\xC3\x2C\xB8"
                 "\x21\x8A\x16\x7F\xE8\x51\xDD\x46"
                 "\xAF\x18\xA4\x0D\x76\x02\x6B\xD4"
                 "\x3D\xC9\x32\x9B\x04\x90\xF9\x62"
                 "\xEE\x57\xC0\x29\xB5\x1E\x87\x13"
                 "\x7C\xE5\x4E\xDA\x43\xAC\x15\xA1"
                 "\x0A\x73\xFF\x68\xD1\x3A\xC6\x2F"
                 "\x98\x01\x8D\xF6\x5F\xEB\x54\xBD"
                 "\x26\xB2\x1B\x84\x10\x79\xE2\x4B"
                 "\xD7\x40\xA9\x12\x9E\x07\x70\xFC"
                 "\x65\xCE\x37\xC3\x2C\x95\x21\x8A"
                 "\xF3\x5C\xE8\x51\xBA\x23\xAF\x18"
                 "\x81\x0D\x76\xDF\x48\xD4\x3D\xA6"
                 "\x0F\x9B\x04\x6D\xF9\x62\xCB\x34"
                 "\xC0\x29\x92\x1E\x87\xF0\x59\xE5"
                 "\x4E\xB7\x20\xAC\x15\x7E\x0A\x73"
                 "\xDC\x45\xD1\x3A\xA3\x0C\x98\x01"
                 "\x6A\xF6\x5F\xC8\x31\xBD\x26\x8F"
                 "\x1B\x84\xED\x56\xE2\x4B\xB4\x1D"
                 "\xA9\x12\x7B\x07\x70\xD9\x42\xCE"
                 "\x37\xA0\x09\x95\xFE\x67\xF3\x5C"
                 "\xC5\x2E\xBA\x23\x8C\x18\x81\xEA"
                 "\x53\xDF\x48\xB1\x1A\xA6\x0F\x78"
                 "\x04\x6D\xD6\x3F\xCB\x34\x9D\x06"
                 "\x92\xFB\x64\xF0\x59\xC2\x2B\xB7"
                 "\x20\x89\x15\x7E\xE7\x50\xDC\x45"
                 "\xAE\x17\xA3\x0C\x75\x01\x6A\xD3"
                 "\x3C\xC8\x31\x9A\x03\x8F\xF8\x61"
                 "\xED\x56\xBF\x28\xB4\x1D\x86\x12"
                 "\x7B\xE4\x4D",
        .ctext = "\xDA\x4E\x3F\xBC\xE8\xB6\x3A\xA2"
                 "\xD5\x4D\x84\x4A\xA9\x0C\xE1\xA5"
                 "\xB8\x73\xBC\xF9\xBB\x59\x2F\x44"
                 "\x8B\xAB\x82\x6C\xB4\x32\x9A\xDE"
                 "\x5A\x0B\xDB\x7A\x6B\xF2\x38\x9F"
                 "\x06\xF7\xF7\xFF\xFF\xC0\x8A\x2E"
                 "\x76\xEA\x06\x32\x23\xF3\x59\x2E"
                 "\x75\xDE\x71\x86\x3C\x98\x23\x44"
                 "\x5B\xF2\xFA\x6A\x00\xBB\xC1\xAD"
                 "\x58\xBD\x3E\x6F\x2E\xB4\x19\x04"
                 "\x70\x8B\x92\x55\x23\xE9\x6A\x3A"
                 "\x78\x7A\x1B\x10\x85\x52\x9C\x12"
                 "\xE4\x55\x81\x21\xCE\x53\xD0\x3B"
                 "\x63\x77\x2C\x74\xD1\xF5\x60\xF3"
                 "\xA1\xDE\x44\x3C\x8F\x4D\x2F\xDD"
                 "\x8A\xFE\x3C\x42\x8E\xD3\xF2\x8E"
                 "\xA8\x28\x69\x65\x31\xE1\x45\x83"
                 "\xE4\x49\xC4\x9C\xA7\x28\xAA\x21"
                 "\xCD\x5D\x0F\x15\xB7\x93\x07\x26"
                 "\xB0\x65\x6D\x91\x90\x23\x7A\xC6"
                 "\xDB\x68\xB0\xA1\x8E\xA4\x76\x4E"
                 "\xC6\x91\x83\x20\x92\x4D\x63\x7A"
                 "\x45\x18\x18\x74\x19\xAD\x71\x01"
                 "\x6B\x23\xAD\x9D\x4E\xE4\x6E\x46"
                 "\xC9\x73\x7A\xF9\x02\x95\xF4\x07"
                 "\x0E\x7A\xA6\xC5\xAE\xFA\x15\x2C"
                 "\x51\x71\xF1\xDC\x22\xB6\xAC\xD8"
                 "\x19\x24\x44\xBC\x0C\xFB\x3C\x2D"
                 "\xB1\x50\x47\x15\x0E\xDB\xB6\xD7"
                 "\xE8\x61\xE5\x95\x52\x1E\x3E\x49"
                 "\x70\xE9\x66\x04\x4C\xE1\xAF\xBD"
                 "\xDD\x15\x3B\x20\x59\x24\xFF\xB0"
                 "\x39\xAA\xE7\xBF\x23\xA3\x6E\xD5"
                 "\x15\xF0\x61\x4F\xAE\x89\x10\x58"
                 "\x5A\x33\x95\x52\x2A\xB5\x77\x9C"
                 "\xA5\x43\x80\x40\x27\x2D\xAE\xD9"
                 "\x3F\xE0\x80\x94\x78\x79\xCB\x7E"
                 "\xAD\x12\x44\x4C\xEC\x27\xB0\xEE"
                 "\x0B\x05\x2A\x82\x99\x58\xBB\x7A"
                 "\x8D\x6D\x9D\x8E\xE2\x8E\xE7\x93"
                 "\x2F\xB3\x09\x8D\x06\xD5\xEE\x70"
                 "\x16\xAE\x35\xC5\x52\x0F\x46\x1F"
                 "\x71\xF9\x5E\xF2\x67\xDC\x98\x2F"
                 "\xA3\x23\xAA\xD5\xD0\x49\xF4\xA6"
                 "\xF6\xB8\x32\xCD\xD6\x85\x73\x60"
                 "\x59\x20\xE7\x55\x0E\x91\xE2\x0C"
                 "\x3F\x1C\xEB\x3D\xDF\x52\x64\xF2"
                 "\x7D\x8B\x5D\x63\x16\xB9\xB2\x5D"
                 "\x5E\xAB\xB2\x97\xAB\x78\x44\xE7"
                 "\xC6\x72\x20\xC5\x90\x9B\xDC\x5D"
                 "\xB0\xEF\x44\xEF\x87\x31\x8D\xF4"
                 "\xFB\x81\x5D\xF7\x96\x96\xD4\x50"
                 "\x89\xA7\xF6\xB9\x67\x76\x40\x9E"
                 "\x9D\x40\xD5\x2C\x30\xB8\x01\x8F"
                 "\xE4\x7B\x71\x48\xA9\xA0\xA0\x1D"
                 "\x87\x52\xA4\x91\xA9\xD7\xA9\x51"
                 "\xD9\x59\xF7\xCC\x63\x22\xC1\x8D"
                 "\x84\x7B\xD8\x22\x32\x5C\x6F\x1D"
                 "\x6E\x9F\xFA\xDD\x49\x40\xDC\x37"
                 "\x14\x8C\xE1\x80\x1B\xDD\x36\x2A"
                 "\xD0\xE9\x54\x99\x5D\xBA\x3B\x11"
                 "\xD8\xFE\xC9\x5B\x5C\x25\xE5\x76"
                 "\xFB\xF2\x3F",
    },
    {
		.alg = SCE_AES_256,
		.mode = OFB,
		.key    = "\x60\x3d\xeb\x10\x15\xca\x71\xbe\x2b\x73\xae\xf0\x85\x7d\x77\x81"
				"\x1f\x35\x2c\x07\x3b\x61\x08\xd7\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
		.klen   = 32,
		.ilen   = 16,
		.iv     = "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext  = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96\xe9\x3d\x7e\x11\x73\x93\x17\x2a",
		.ctext  = "\xdc\x7e\x84\xbf\xda\x79\x16\x4b\x7e\xcd\x84\x86\x98\x5d\x38\x60",
		.len    = 16,
	},
	{
		.alg = SCE_AES_192,
		.mode = OFB,
		.key    = "\x8e\x73\xb0\xf7\xda\x0e\x64\x52\xc8\x10\xf3\x2b"
					"\x80\x90\x79\xe5\x62\xf8\xea\xd2\x52\x2c\x6b\x7b",
		.klen   = 24,
		.ilen   = 16,
		.iv     = "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext  = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96\xe9\x3d\x7e\x11\x73\x93\x17\x2a",
		.ctext  = "\xcd\xc8\x0d\x6f\xdd\xf1\x8c\xab\x34\xc2\x59\x09\xc9\x9a\x41\x74",
		.len    = 16,
	},

    {
        /* From NIST Special Publication 800-38A, Appendix F.5 */
        .klen = 16,
        .len = 64,
        .ilen = 16,
        .alg = SCE_AES_128,
        .mode = OFB,
        .key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
               "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07\x08"
              "\x09\x0a\x0b\x0c\x0d\x0e\x0f",
        .ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
                 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
                 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
                 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
                 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
                 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
        .ctext = "\x3b\x3f\xd9\x2e\xb7\x2d\xad\x20"
                 "\x33\x34\x49\xf8\xe8\x3c\xfb\x4a"
                 "\x77\x89\x50\x8d\x16\x91\x8f\x03\xf5"
                 "\x3c\x52\xda\xc5\x4e\xd8\x25"
                 "\x97\x40\x05\x1e\x9c\x5f\xec\xf6\x43"
                 "\x44\xf7\xa8\x22\x60\xed\xcc"
                 "\x30\x4c\x65\x28\xf6\x59\xc7\x78"
                 "\x66\xa5\x10\xd9\xc1\xd6\xae\x5e",
    },
    {
        /* > 16 bytes, not a multiple of 16 bytes */
        .klen = 16,
        .len = 17,
        .ilen = 16,
        .alg = SCE_AES_128,
        .mode = OFB,
        .key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
               "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
        .ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
                 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
                 "\xae",
        .ctext = "\x3b\x3f\xd9\x2e\xb7\x2d\xad\x20"
                 "\x33\x34\x49\xf8\xe8\x3c\xfb\x4a"
                 "\x77",
    },
    {
        /* < 16 bytes */
        .klen = 16,
        .len = 7,
        .ilen = 16,
        .alg = SCE_AES_128,
        .mode = OFB,
        .key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
               "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
        .ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f",
        .ctext = "\x3b\x3f\xd9\x2e\xb7\x2d\xad",
    },
    { /* XTS-AES 1 */
	.klen = 32,
	.len = 32,
	.ilen = 16,
	.alg = SCE_AES_128,
	.mode = XTS,
	.key    ="\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .iv     = "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ptext  ="\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ctext  = "\x91\x7c\xf6\x9e\xbd\x68\xb2\xec"
                  "\x9b\x9f\xe9\xa3\xea\xdd\xa6\x92"
                  "\xcd\x43\xd2\xf5\x95\x98\xed\x85"
                  "\x8c\x02\xc2\x65\x2f\xbf\x92\x2e",
    },
    { /* XTS-AES 2 */
        .klen = 32,
        .len = 32,
        .ilen = 16,
        .alg = SCE_AES_128,
        .mode = XTS,
        .key    ="\x11\x11\x11\x11\x11\x11\x11\x11"
                  "\x11\x11\x11\x11\x11\x11\x11\x11"
                  "\x22\x22\x22\x22\x22\x22\x22\x22"
                  "\x22\x22\x22\x22\x22\x22\x22\x22",
        .iv     = "\x33\x33\x33\x33\x33\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ptext  ="\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44",
        .ctext  = "\xc4\x54\x18\x5e\x6a\x16\x93\x6e"
                  "\x39\x33\x40\x38\xac\xef\x83\x8b"
                  "\xfb\x18\x6f\xff\x74\x80\xad\xc4"
                  "\x28\x93\x82\xec\xd6\xd3\x94\xf0",
    },
    { /* XTS-AES 3 */
        .klen = 32,
        .len = 32,
        .ilen = 16,
        .alg = SCE_AES_128,
        .mode = XTS,
        .key    ="\xff\xfe\xfd\xfc\xfb\xfa\xf9\xf8"
                  "\xf7\xf6\xf5\xf4\xf3\xf2\xf1\xf0"
                  "\x22\x22\x22\x22\x22\x22\x22\x22"
                  "\x22\x22\x22\x22\x22\x22\x22\x22",
        .iv     = "\x33\x33\x33\x33\x33\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ptext  ="\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44",
        .ctext  = "\xaf\x85\x33\x6b\x59\x7a\xfc\x1a"
                  "\x90\x0b\x2e\xb2\x1e\xc9\x49\xd2"
                  "\x92\xdf\x4c\x04\x7e\x0b\x21\x53"
                  "\x21\x86\xa5\x97\x1a\x22\x7a\x89",
    },
    { /* XTS-AES 10, XTS-AES-256, data unit 512 bytes */
		.klen = 64,
		.len = 512,
		.ilen = 16,
		.alg = SCE_AES_256,
		.mode = XTS,
		.key	= "\x27\x18\x28\x18\x28\x45\x90\x45"
			  "\x23\x53\x60\x28\x74\x71\x35\x26"
			  "\x62\x49\x77\x57\x24\x70\x93\x69"
			  "\x99\x59\x57\x49\x66\x96\x76\x27"
			  "\x31\x41\x59\x26\x53\x58\x97\x93"
			  "\x23\x84\x62\x64\x33\x83\x27\x95"
			  "\x02\x88\x41\x97\x16\x93\x99\x37"
			  "\x51\x05\x82\x09\x74\x94\x45\x92",
		.iv	= "\xff\x00\x00\x00\x00\x00\x00\x00"
			  "\x00\x00\x00\x00\x00\x00\x00\x00",
		.ptext	= "\x00\x01\x02\x03\x04\x05\x06\x07"
			  "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
			  "\x10\x11\x12\x13\x14\x15\x16\x17"
			  "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f"
			  "\x20\x21\x22\x23\x24\x25\x26\x27"
			  "\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f"
			  "\x30\x31\x32\x33\x34\x35\x36\x37"
			  "\x38\x39\x3a\x3b\x3c\x3d\x3e\x3f"
			  "\x40\x41\x42\x43\x44\x45\x46\x47"
			  "\x48\x49\x4a\x4b\x4c\x4d\x4e\x4f"
			  "\x50\x51\x52\x53\x54\x55\x56\x57"
			  "\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f"
			  "\x60\x61\x62\x63\x64\x65\x66\x67"
			  "\x68\x69\x6a\x6b\x6c\x6d\x6e\x6f"
			  "\x70\x71\x72\x73\x74\x75\x76\x77"
			  "\x78\x79\x7a\x7b\x7c\x7d\x7e\x7f"
			  "\x80\x81\x82\x83\x84\x85\x86\x87"
			  "\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f"
			  "\x90\x91\x92\x93\x94\x95\x96\x97"
			  "\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f"
			  "\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7"
			  "\xa8\xa9\xaa\xab\xac\xad\xae\xaf"
			  "\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7"
			  "\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf"
			  "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
			  "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf"
			  "\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7"
			  "\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf"
			  "\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7"
			  "\xe8\xe9\xea\xeb\xec\xed\xee\xef"
			  "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
			  "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff"
			  "\x00\x01\x02\x03\x04\x05\x06\x07"
			  "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
			  "\x10\x11\x12\x13\x14\x15\x16\x17"
			  "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f"
			  "\x20\x21\x22\x23\x24\x25\x26\x27"
			  "\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f"
			  "\x30\x31\x32\x33\x34\x35\x36\x37"
			  "\x38\x39\x3a\x3b\x3c\x3d\x3e\x3f"
			  "\x40\x41\x42\x43\x44\x45\x46\x47"
			  "\x48\x49\x4a\x4b\x4c\x4d\x4e\x4f"
			  "\x50\x51\x52\x53\x54\x55\x56\x57"
			  "\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f"
			  "\x60\x61\x62\x63\x64\x65\x66\x67"
			  "\x68\x69\x6a\x6b\x6c\x6d\x6e\x6f"
			  "\x70\x71\x72\x73\x74\x75\x76\x77"
			  "\x78\x79\x7a\x7b\x7c\x7d\x7e\x7f"
			  "\x80\x81\x82\x83\x84\x85\x86\x87"
			  "\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f"
			  "\x90\x91\x92\x93\x94\x95\x96\x97"
			  "\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f"
			  "\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7"
			  "\xa8\xa9\xaa\xab\xac\xad\xae\xaf"
			  "\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7"
			  "\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf"
			  "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
			  "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf"
			  "\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7"
			  "\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf"
			  "\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7"
			  "\xe8\xe9\xea\xeb\xec\xed\xee\xef"
			  "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
			  "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff",
		.ctext	= "\x1c\x3b\x3a\x10\x2f\x77\x03\x86"
			  "\xe4\x83\x6c\x99\xe3\x70\xcf\x9b"
			  "\xea\x00\x80\x3f\x5e\x48\x23\x57"
			  "\xa4\xae\x12\xd4\x14\xa3\xe6\x3b"
			  "\x5d\x31\xe2\x76\xf8\xfe\x4a\x8d"
			  "\x66\xb3\x17\xf9\xac\x68\x3f\x44"
			  "\x68\x0a\x86\xac\x35\xad\xfc\x33"
			  "\x45\xbe\xfe\xcb\x4b\xb1\x88\xfd"
			  "\x57\x76\x92\x6c\x49\xa3\x09\x5e"
			  "\xb1\x08\xfd\x10\x98\xba\xec\x70"
			  "\xaa\xa6\x69\x99\xa7\x2a\x82\xf2"
			  "\x7d\x84\x8b\x21\xd4\xa7\x41\xb0"
			  "\xc5\xcd\x4d\x5f\xff\x9d\xac\x89"
			  "\xae\xba\x12\x29\x61\xd0\x3a\x75"
			  "\x71\x23\xe9\x87\x0f\x8a\xcf\x10"
			  "\x00\x02\x08\x87\x89\x14\x29\xca"
			  "\x2a\x3e\x7a\x7d\x7d\xf7\xb1\x03"
			  "\x55\x16\x5c\x8b\x9a\x6d\x0a\x7d"
			  "\xe8\xb0\x62\xc4\x50\x0d\xc4\xcd"
			  "\x12\x0c\x0f\x74\x18\xda\xe3\xd0"
			  "\xb5\x78\x1c\x34\x80\x3f\xa7\x54"
			  "\x21\xc7\x90\xdf\xe1\xde\x18\x34"
			  "\xf2\x80\xd7\x66\x7b\x32\x7f\x6c"
			  "\x8c\xd7\x55\x7e\x12\xac\x3a\x0f"
			  "\x93\xec\x05\xc5\x2e\x04\x93\xef"
			  "\x31\xa1\x2d\x3d\x92\x60\xf7\x9a"
			  "\x28\x9d\x6a\x37\x9b\xc7\x0c\x50"
			  "\x84\x14\x73\xd1\xa8\xcc\x81\xec"
			  "\x58\x3e\x96\x45\xe0\x7b\x8d\x96"
			  "\x70\x65\x5b\xa5\xbb\xcf\xec\xc6"
			  "\xdc\x39\x66\x38\x0a\xd8\xfe\xcb"
			  "\x17\xb6\xba\x02\x46\x9a\x02\x0a"
			  "\x84\xe1\x8e\x8f\x84\x25\x20\x70"
			  "\xc1\x3e\x9f\x1f\x28\x9b\xe5\x4f"
			  "\xbc\x48\x14\x57\x77\x8f\x61\x60"
			  "\x15\xe1\x32\x7a\x02\xb1\x40\xf1"
			  "\x50\x5e\xb3\x09\x32\x6d\x68\x37"
			  "\x8f\x83\x74\x59\x5c\x84\x9d\x84"
			  "\xf4\xc3\x33\xec\x44\x23\x88\x51"
			  "\x43\xcb\x47\xbd\x71\xc5\xed\xae"
			  "\x9b\xe6\x9a\x2f\xfe\xce\xb1\xbe"
			  "\xc9\xde\x24\x4f\xbe\x15\x99\x2b"
			  "\x11\xb7\x7c\x04\x0f\x12\xbd\x8f"
			  "\x6a\x97\x5a\x44\xa0\xf9\x0c\x29"
			  "\xa9\xab\xc3\xd4\xd8\x93\x92\x72"
			  "\x84\xc5\x87\x54\xcc\xe2\x94\x52"
			  "\x9f\x86\x14\xdc\xd2\xab\xa9\x91"
			  "\x92\x5f\xed\xc4\xae\x74\xff\xac"
			  "\x6e\x33\x3b\x93\xeb\x4a\xff\x04"
			  "\x79\xda\x9a\x41\x0e\x44\x50\xe0"
			  "\xdd\x7a\xe4\xc6\xe2\x91\x09\x00"
			  "\x57\x5d\xa4\x01\xfc\x07\x05\x9f"
			  "\x64\x5e\x8b\x7e\x9b\xfd\xef\x33"
			  "\x94\x30\x54\xff\x84\x01\x14\x93"
			  "\xc2\x7b\x34\x29\xea\xed\xb4\xed"
			  "\x53\x76\x44\x1a\x77\xed\x43\x85"
			  "\x1a\xd7\x7f\x16\xf5\x41\xdf\xd2"
			  "\x69\xd5\x0d\x6a\x5f\x14\xfb\x0a"
			  "\xab\x1c\xbb\x4c\x15\x50\xbe\x97"
			  "\xf7\xab\x40\x66\x19\x3c\x4c\xaa"
			  "\x77\x3d\xad\x38\x01\x4b\xd2\x09"
			  "\x2f\xa7\x55\xc8\x24\xbb\x5e\x54"
			  "\xc4\xf3\x6f\xfd\xa9\xfc\xea\x70"
			  "\xb9\xc6\xe6\x93\xe1\x48\xc1\x51",
	},


};

void aes_test_one(char *alg_name, ske_testvec_t *test_vec)
{
    u8 *src, *dst, *key, *iv;

    src = kmalloc(1024 * 3, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    memset(src, 0, 1024 * 3);
    dst = src + 1024;
    key = dst + 1024;
    iv = key + 512;

    // test enc
    memcpy(src, test_vec->ptext, test_vec->len);
    memcpy(key, test_vec->key, test_vec->klen);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }

    // test one segment
    skcipher_test(alg_name, 1, key, test_vec->klen, iv, src, test_vec->len,
                  dst);
    if (memcmp(dst, test_vec->ctext, test_vec->len)) {
        pr_err("encrypt %s err!\n", alg_name);
        goto out;
    }

    // test step
    if (0 != memcmp(alg_name, "xts(", 4)) {
    	if (test_vec->iv) {
    	    memcpy(iv, test_vec->iv, test_vec->ilen);
    	} else {
    	    iv = NULL;
    	}
    	memset(dst, 0, test_vec->klen);
    	skcipher_step_test(alg_name, 1, key, test_vec->klen, iv, src, test_vec->len,
    	                   dst);
    	if (memcmp(dst, test_vec->ctext, test_vec->len)) {
    	    pr_err("encrypt step %s err!\n", alg_name);
    	    goto out;
    	}
    }

    // test dec
    memcpy(src, test_vec->ctext, test_vec->len);
    memcpy(key, test_vec->key, test_vec->klen);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }
    skcipher_test(alg_name, 0, key, test_vec->klen, iv, src, test_vec->len,
                  dst);
    if (memcmp(dst, test_vec->ptext, test_vec->len)) {
        pr_err("decrypt %s err!\n", alg_name);
        goto out;
    }

    // test dec step
    if (0 != memcmp(alg_name, "xts(", 4)) {
    	if (test_vec->iv) {
    	    memcpy(iv, test_vec->iv, test_vec->ilen);
    	} else {
    	    iv = NULL;
    	}
    	skcipher_step_test(alg_name, 0, key, test_vec->klen, iv, src, test_vec->len,
    	                   dst);
    	if (memcmp(dst, test_vec->ptext, test_vec->len)) {
    	    pr_err("decrypt %s err!\n", alg_name);
    	    goto out;
    	}
    }

    pr_info("Test %s klen %d successed!\n", alg_name, test_vec->klen);
out:
    kfree(src);
}

void aes_test(char *mode, u32 klen, u32 slen, u32 elen)
{
    char alg_name[64] = {0};
    u32 alg_mode, i;
    u32 test = 0;
    ske_testvec_t *test_vec;

    sce_get_alg_name(alg_name, "aes", mode);
    alg_mode = get_alg_mode(mode);
    if (alg_mode == 0xff) {
        pr_err("Invalid mode %s", mode);
        return;
    }

    pr_info("get mode %s, klen %d slen %d elen %d alg_mode %d", mode, klen,
            slen, elen, alg_mode);

    // test specified vec
    for (i = 0; i < sizeof(aes_tv_template) / sizeof(aes_tv_template[0]); i++) {
        test_vec = &aes_tv_template[i];
        if (test_vec->mode == alg_mode) {
            if (!klen || test_vec->klen == klen) {
                aes_test_one(alg_name, test_vec);
                test = 1;
            }
        }
    }

    if (0 == test) {
        pr_err("Missing test vector for alg %s klen %d", alg_name, klen);
    }
}
