#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "sca_test.h"
#include "../common/common.h"

ske_testvec_t des_tv_template[] = {
    {
        .klen = 8,
        .len = 8,
        .ilen = 0,
        .alg = SCE_DES,
        .mode = ECB,
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7",
        .ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d",
    },
    {
        /* Same key, different plaintext block */
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .ptext = "\x22\x33\x44\x55\x66\x77\x88\x99",
        .ctext = "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b",
        .len = 8,
        .ilen = 0,
        .alg = SCE_DES,
        .mode = ECB,
    },
    {
        /* Sbox test from NBS */
        .key = "\x7c\xa1\x10\x45\x4a\x1a\x6e\x57",
        .klen = 8,
        .ptext = "\x01\xa1\xd6\xd0\x39\x77\x67\x42",
        .ctext = "\x69\x0f\x5b\x0d\x9a\x26\x93\x9b",
        .len = 8,
        .ilen = 0,
        .alg = SCE_DES,
        .mode = ECB,
    },
    {
        /* Three blocks */
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7"
                 "\x22\x33\x44\x55\x66\x77\x88\x99"
                 "\xca\xfe\xba\xbe\xfe\xed\xbe\xef",
        .ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d"
                 "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b"
                 "\xb4\x99\x26\xf7\x1f\xe1\xd4\x90",
        .len = 24,
        .ilen = 0,
        .alg = SCE_DES,
        .mode = ECB,
    },
    {
        /* Two blocks -- for testing encryption across pages */
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7"
                 "\x22\x33\x44\x55\x66\x77\x88\x99",
        .ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d"
                 "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b",
        .len = 16,
        .ilen = 0,
        .alg = SCE_DES,
        .mode = ECB,
    },
    {
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7"
                 "\xa3\x99\x7b\xca\xaf\x69\xa0\xf5",
        .ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d"
                 "\x69\x0f\x5b\x0d\x9a\x26\x93\x9b",
        .len = 16,
        .ilen = 0,
        .alg = SCE_DES,
        .mode = ECB,
    },
    {
        /* Four blocks -- for testing encryption with chunking */
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7"
                 "\x22\x33\x44\x55\x66\x77\x88\x99"
                 "\xca\xfe\xba\xbe\xfe\xed\xbe\xef"
                 "\x22\x33\x44\x55\x66\x77\x88\x99",
        .ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d"
                 "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b"
                 "\xb4\x99\x26\xf7\x1f\xe1\xd4\x90"
                 "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b",
        .len = 32,
        .ilen = 0,
        .alg = SCE_DES,
        .mode = ECB,
    },
    {
        /* Generated with Crypto++ */
        .ilen = 0,
        .alg = SCE_DES,
        .mode = ECB,
        .key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55",
        .klen = 8,
        .ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
                 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
                 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
                 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
                 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
                 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
                 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
                 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
                 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
                 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
                 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
                 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
                 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
                 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
                 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
                 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
                 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
                 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
                 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
                 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
                 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
                 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
                 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
                 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
                 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
                 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
                 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
                 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
                 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
                 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
                 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB",
        .ctext = "\x88\xCB\x1F\xAB\x2F\x2A\x49\x57"
                 "\x92\xB9\x77\xFF\x2F\x47\x58\xDD"
                 "\xD7\x8A\x91\x95\x26\x33\x78\xB2"
                 "\x33\xBA\xB2\x3E\x02\xF5\x1F\xEF"
                 "\x98\xC5\xA6\xD2\x7D\x79\xEC\xB3"
                 "\x45\xF3\x4C\x61\xAC\x6C\xC2\x55"
                 "\xE5\xD3\x06\x58\x8A\x42\x3E\xDD"
                 "\x3D\x20\x45\xE9\x6F\x0D\x25\xA8"
                 "\xA5\xC7\x69\xCE\xD5\x3B\x7B\xC9"
                 "\x9E\x65\xE7\xA3\xF2\xE4\x18\x94"
                 "\xD2\x81\xE9\x33\x2B\x2D\x49\xC4"
                 "\xFE\xDA\x7F\xE2\xF2\x8C\x9C\xDC"
                 "\x73\x58\x11\x1F\x81\xD7\x21\x1A"
                 "\x80\xD0\x0D\xE8\x45\xD6\xD8\xD5"
                 "\x2E\x51\x16\xCA\x09\x89\x54\x62"
                 "\xF7\x04\x3D\x75\xB9\xA3\x84\xF4"
                 "\x62\xF0\x02\x58\x83\xAF\x30\x87"
                 "\x85\x3F\x01\xCD\x8E\x58\x42\xC4"
                 "\x41\x73\xE0\x15\x0A\xE6\x2E\x80"
                 "\x94\xF8\x5B\x3A\x4E\xDF\x51\xB2"
                 "\x9D\xE4\xC4\x9D\xF7\x3F\xF8\x8E"
                 "\x37\x22\x4D\x00\x2A\xEF\xC1\x0F"
                 "\x14\xA0\x66\xAB\x79\x39\xD0\x8E"
                 "\xE9\x95\x61\x74\x12\xED\x07\xD7"
                 "\xDD\x95\xDC\x7B\x57\x25\x27\x9C"
                 "\x51\x96\x16\xF7\x94\x61\xB8\x87"
                 "\xF0\x21\x1B\x32\xFB\x07\x0F\x29"
                 "\x56\xBD\x9D\x22\xA2\x9F\xA2\xB9"
                 "\x46\x31\x4C\x5E\x2E\x95\x61\xEF"
                 "\xE1\x58\x39\x09\xB4\x8B\x40\xAC"
                 "\x5F\x62\xC7\x72\xD9\xFC\xCB\x9A",
        .len = 248,
    },
    {
        /* From OpenSSL */
        .ilen = 8,
        .alg = SCE_DES,
        .mode = CBC,
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .iv = "\xfe\xdc\xba\x98\x76\x54\x32\x10",
        .ptext = "\x37\x36\x35\x34\x33\x32\x31\x20"
                 "\x4e\x6f\x77\x20\x69\x73\x20\x74"
                 "\x68\x65\x20\x74\x69\x6d\x65\x20",
        .ctext = "\xcc\xd1\x73\xff\xab\x20\x39\xf4"
                 "\xac\xd8\xae\xfd\xdf\xd8\xa1\xeb"
                 "\x46\x8e\x91\x15\x78\x88\xba\x68",
        .len = 24,
    },
    {
        /* FIPS Pub 81 */
        .ilen = 8,
        .alg = SCE_DES,
        .mode = CBC,
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .iv = "\x12\x34\x56\x78\x90\xab\xcd\xef",
        .ptext = "\x4e\x6f\x77\x20\x69\x73\x20\x74",
        .ctext = "\xe5\xc7\xcd\xde\x87\x2b\xf2\x7c",
        .len = 8,
    },
    {
        .ilen = 8,
        .alg = SCE_DES,
        .mode = CBC,
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .iv = "\xe5\xc7\xcd\xde\x87\x2b\xf2\x7c",
        .ptext = "\x68\x65\x20\x74\x69\x6d\x65\x20",
        .ctext = "\x43\xe9\x34\x00\x8c\x38\x9c\x0f",
        .len = 8,
    },
    {
        .ilen = 8,
        .alg = SCE_DES,
        .mode = CBC,
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .klen = 8,
        .iv = "\x43\xe9\x34\x00\x8c\x38\x9c\x0f",
        .ptext = "\x66\x6f\x72\x20\x61\x6c\x6c\x20",
        .ctext = "\x68\x37\x88\x49\x9a\x7c\x05\xf6",
        .len = 8,
    },
    {
        /* Generated with Crypto++ */
        .ilen = 8,
        .alg = SCE_DES,
        .mode = CBC,
        .key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55",
        .klen = 8,
        .iv = "\xE7\x82\x1D\xB8\x53\x11\xAC\x47",
        .ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
                 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
                 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
                 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
                 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
                 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
                 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
                 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
                 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
                 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
                 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
                 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
                 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
                 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
                 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
                 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
                 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
                 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
                 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
                 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
                 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
                 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
                 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
                 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
                 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
                 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
                 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
                 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
                 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
                 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
                 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB",
        .ctext = "\x71\xCC\x56\x1C\x87\x2C\x43\x20"
                 "\x1C\x20\x13\x09\xF9\x2B\x40\x47"
                 "\x99\x10\xD1\x1B\x65\x33\x33\xBA"
                 "\x88\x0D\xA2\xD1\x86\xFF\x4D\xF4"
                 "\x5A\x0C\x12\x96\x32\x57\xAA\x26"
                 "\xA7\xF4\x32\x8D\xBC\x10\x31\x9E"
                 "\x81\x72\x74\xDE\x30\x19\x69\x49"
                 "\x54\x9C\xC3\xEB\x0B\x97\xDD\xD1"
                 "\xE8\x6D\x0D\x05\x83\xA5\x12\x08"
                 "\x47\xF8\x88\x03\x86\x51\x3C\xEF"
                 "\xE7\x11\x73\x4D\x44\x2B\xE2\x16"
                 "\xE8\xA5\x06\x50\x66\x70\x0E\x14"
                 "\xBA\x21\x3B\xD5\x23\x5B\xA7\x8F"
                 "\x56\xB6\xA7\x44\xDB\x86\xAB\x69"
                 "\x33\x3C\xBE\x64\xC4\x22\xD3\xFE"
                 "\x49\x90\x88\x6A\x09\x8F\x76\x59"
                 "\xCB\xB7\xA0\x2D\x79\x75\x92\x8A"
                 "\x82\x1D\xC2\xFE\x09\x1F\x78\x6B"
                 "\x2F\xD6\xA4\x87\x1E\xC4\x53\x63"
                 "\x80\x02\x61\x2F\xE3\x46\xB6\xB5"
                 "\xAA\x95\xF4\xEE\xA7\x64\x2B\x4F"
                 "\x20\xCF\xD2\x47\x4E\x39\x65\xB3"
                 "\x11\x87\xA2\x6C\x49\x7E\x36\xC7"
                 "\x62\x8B\x48\x0D\x6A\x64\x00\xBD"
                 "\x71\x91\x8C\xE9\x70\x19\x01\x4F"
                 "\x4E\x68\x23\xBA\xDA\x24\x2E\x45"
                 "\x02\x14\x33\x21\xAE\x58\x4B\xCF"
                 "\x3B\x4B\xE8\xF8\xF6\x4F\x34\x93"
                 "\xD7\x07\x8A\xD7\x18\x92\x36\x8C"
                 "\x82\xA9\xBD\x6A\x31\x91\x39\x11"
                 "\xC6\x4A\xF3\x55\xC7\x29\x2E\x63",
        .len = 248,
    },
    {
        /* Generated with Crypto++ */
        .ilen = 8,
        .alg = SCE_DES,
        .mode = CTR,
        .key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55",
        .klen = 8,
        .iv = "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFD",
        .ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
                 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
                 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
                 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
                 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
                 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
                 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
                 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
                 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
                 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
                 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
                 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
                 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
                 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
                 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
                 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
                 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
                 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
                 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
                 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
                 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
                 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
                 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
                 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
                 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
                 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
                 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
                 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
                 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
                 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
                 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB",
        .ctext = "\x2F\x96\x06\x0F\x50\xC9\x68\x03"
                 "\x0F\x31\xD4\x64\xA5\x29\x77\x35"
                 "\xBC\x7A\x9F\x19\xE7\x0D\x33\x3E"
                 "\x12\x0B\x8C\xAE\x48\xAE\xD9\x02"
                 "\x0A\xD4\xB0\xD6\x37\xB2\x65\x1C"
                 "\x4B\x65\xEB\x24\xB5\x8E\xAD\x47"
                 "\x0D\xDA\x79\x77\xA0\x29\xA0\x2B"
                 "\xC8\x0F\x85\xDC\x03\x13\xA9\x04"
                 "\x19\x40\xBE\xBE\x5C\x49\x4A\x69"
                 "\xED\xE8\xE1\x9E\x14\x43\x74\xDE"
                 "\xEC\x6E\x11\x3F\x36\xEF\x7B\xFB"
                 "\xBE\x4C\x91\x43\x22\x65\x72\x48"
                 "\xE2\x12\xED\x88\xAC\xA7\xC9\x91"
                 "\x14\xA2\x36\x1C\x29\xFF\xC8\x4F"
                 "\x72\x5C\x4B\xB0\x1E\x93\xC2\xFA"
                 "\x9D\x53\x86\xA0\xAE\xC6\xB7\x3C"
                 "\x59\x0C\xD0\x8F\xA6\xD8\xA4\x31"
                 "\xB7\x30\x1C\x21\x38\xFB\x68\x8C"
                 "\x2E\xF5\x6E\x73\xC3\x16\x5F\x12"
                 "\x0C\x33\xB9\x1E\x7B\x70\xDE\x86"
                 "\x32\xB3\xC1\x16\xAB\xD9\x49\x0B"
                 "\x96\x28\x72\x6B\xF3\x30\xA9\xEB"
                 "\x69\xE2\x1E\x58\x46\xA2\x8E\xC7"
                 "\xC0\xEF\x07\xB7\x77\x2C\x00\x05"
                 "\x46\xBD\xFE\x53\x81\x8B\xA4\x03"
                 "\x20\x0F\xDB\x78\x0B\x1F\x53\x04"
                 "\x4C\x60\x4C\xC3\x2A\x86\x86\x7E"
                 "\x13\xD2\x26\xED\x5D\x3E\x9C\xF2"
                 "\x5C\xC4\x15\xC9\x9A\x21\xC5\xCD"
                 "\x19\x7F\x99\x19\x53\xCE\x1D\x14"
                 "\x69\x74\xA1\x06\x46\x0F\x4E\x75",
        .len = 248,
    },
    {
        /* Generated with Crypto++ */
        .ilen = 8,
        .alg = SCE_DES,
        .mode = CTR,
        .key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55",
        .klen = 8,
        .iv = "\xE7\x82\x1D\xB8\x53\x11\xAC\x47",
        .ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
                 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
                 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
                 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
                 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
                 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
                 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
                 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
                 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
                 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
                 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
                 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
                 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
                 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
                 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
                 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
                 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
                 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
                 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
                 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
                 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
                 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
                 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
                 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
                 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
                 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
                 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
                 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
                 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
                 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
                 "\xC6\x2F\xBB\x24\x8D\x19\x82",
        .ctext = "\x62\xE5\xF4\xDC\x99\xE7\x89\xE3"
                 "\xF4\x10\xCC\x21\x99\xEB\xDC\x15"
                 "\x19\x13\x93\x27\x9D\xB6\x6F\x45"
                 "\x17\x55\x61\x72\xC8\xD3\x7F\xA5"
                 "\x32\xD0\xD3\x02\x15\xA4\x05\x23"
                 "\x9C\x23\x61\x60\x77\x7B\x6C\x95"
                 "\x26\x49\x42\x2E\xF3\xC1\x8C\x6D"
                 "\xC8\x47\xD5\x94\xE7\x53\xC8\x23"
                 "\x1B\xA5\x0B\xCB\x12\xD3\x7A\x12"
                 "\xA4\x42\x15\x34\xF7\x5F\xDC\x58"
                 "\x5B\x58\x4C\xAD\xD1\x33\x8E\xE6"
                 "\xE5\xA0\xDA\x4D\x94\x3D\x63\xA8"
                 "\x02\x82\xBB\x16\xB8\xDC\xB5\x58"
                 "\xC3\x2D\x79\xE4\x25\x79\x43\xF9"
                 "\x6D\xD3\xCA\xC0\xE8\x12\xD4\x7E"
                 "\x04\x25\x79\xFD\x27\xFB\xC4\xEA"
                 "\x32\x94\x48\x92\xF3\x68\x1A\x7F"
                 "\x36\x33\x43\x79\xF7\xCA\xC2\x38"
                 "\xC0\x68\xD4\x53\xA9\xCC\x43\x0C"
                 "\x40\x57\x3E\xED\x00\x9F\x22\x6E"
                 "\x80\x99\x0B\xCC\x40\x63\x46\x8A"
                 "\xE8\xC4\x9B\x6D\x7A\x08\x6E\xA9"
                 "\x6F\x84\xBC\xB3\xF4\x95\x0B\x2D"
                 "\x6A\xBA\x37\x50\xC3\xCF\x9F\x7C"
                 "\x59\x5E\xDE\x0B\x30\xFA\x34\x8A"
                 "\xF8\xD1\xA2\xF8\x4E\xBD\x5D\x5E"
                 "\x7D\x71\x99\xE0\xF6\xE5\x7C\xE0"
                 "\x6D\xEE\x82\x89\x92\xD4\xF5\xD7"
                 "\xDF\x85\x2D\xE1\xB2\xD6\xAB\x94"
                 "\xA5\xA6\xE7\xB0\x51\x36\x52\x37"
                 "\x91\x45\x05\x3E\x58\xBF\x32",
        .len = 247,
    },
    {
    	.alg	= SCE_DES,
    	.mode	= OFB,
    	.ilen	= 8,
    	.key	= "\xe4\x23\x33\x8a\x35\x64\x61\xe2",
    	.klen	= 8,
    	.iv	= "\x00\x01\x02\x03\x04\x05\x06\x07",
    	.ptext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b"
    			"\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
    			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60"
    			"\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
    	.ctext	= "\x66\x77\xe6\xf1\x4d\x05\x9f\x34"
    			"\x71\x0b\x86\xc5\x7e\xc1\x1b\xb6"
    			"\xf2\x72\x43\xcc\xe5\x0b\x43\xc9"
    			"\xeb\x60\x5c\x3d\x60\x03\x5f\x27",
    	.len	= 32,
    },
    {
        .alg    = SCE_DES,
        .mode   = OFB,
    	.ilen	= 8,
    	.key	= "\xe4\x23\x33\x8a\x35\x64\x61\xe2",
    	.klen	= 8,
    	.iv	= "\x00\x01\x02\x03\x04\x05\x06\x07",
    	.ptext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b"
    			"\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
    			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60"
    			"\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
    			"\x8a\xe0\x4d\xb0\x70\x38\xcc\x48"
    			"\x61\x7d\xee\xd6\x35\x49\xae\xb4"
    			"\xaf\x6b\xdd\xe6\x21\xc0\x60\xce"
    			"\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b",
    	.ctext	= "\x66\x77\xe6\xf1\x4d\x05\x9f\x34"
    			"\x71\x0b\x86\xc5\x7e\xc1\x1b\xb6"
    			"\xf2\x72\x43\xcc\xe5\x0b\x43\xc9"
    			"\xeb\x60\x5c\x3d\x60\x03\x5f\x27"
    			"\xf0\x04\x05\x94\x9b\x5e\x66\x25"
    			"\x7c\x81\xaf\xbd\xfb\x66\x71\x57"
    			"\x71\x33\xd9\xb3\xd3\x9d\x02\xe2"
    			"\xfc\x62\x9a\x01\x01\x42\xf4\x00",
    	.len	= 64,
    },
    {
        .alg    = SCE_DES,
        .mode   = OFB,
    	.ilen	= 8,
    	.key	= "\xe4\x23\x33\x8a\x35\x64\x61\xe2",
    	.klen	= 8,
    	.iv	= "\x00\x01\x02\x03\x04\x05\x06\x07",
    	.ptext	= "\xff\xca\xfb\xf1\x38\x20\x2f\x7b"
    			"\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
    			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60"
    			"\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
    			"\x8a\xe0\x4d\xb0\x70\x38\xcc\x48"
    			"\x61\x7d\xee\xd6\x35\x49\xae\xb4"
    			"\xaf\x6b\xdd\xe6\x21\xc0\x60\xce"
    			"\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
    			"\x59\xda\x19\x4f\xec\x07\x8e\xe2"
    			"\xf0\x61\xf9\x27\x61\x6f\xf8\xdf"
    			"\x62\x4d\xaf\x06\xfe\x41\xa6\xa6"
    			"\xf9\xa2\x06\x40\xb3\x04\xbd\xe6"
    			"\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e"
    			"\xa6\x58\xdc\x91\x17\x58\x42\x95"
    			"\xa3\x7c\x81\x78\xa6\x3d\x3f\x75"
    			"\x74\x17\x1a\xd3\x6c\x2f\x48\x39",
    	.ctext	= "\x66\x77\xe6\xf1\x4d\x05\x9f\x34"
    			"\x71\x0b\x86\xc5\x7e\xc1\x1b\xb6"
    			"\xf2\x72\x43\xcc\xe5\x0b\x43\xc9"
    			"\xeb\x60\x5c\x3d\x60\x03\x5f\x27"
    			"\xf0\x04\x05\x94\x9b\x5e\x66\x25"
    			"\x7c\x81\xaf\xbd\xfb\x66\x71\x57"
    			"\x71\x33\xd9\xb3\xd3\x9d\x02\xe2"
    			"\xfc\x62\x9a\x01\x01\x42\xf4\x00"
    			"\xc6\x76\x90\xa1\xdd\x16\x10\xf9"
    			"\x15\x92\x6b\x64\xd0\x7d\x76\x20"
    			"\x6a\x57\xe3\x6f\x66\xfa\x72\xd2"
    			"\x1d\x98\x3c\x86\xc4\xc9\xf4\xbc"
    			"\x60\xa0\x59\x6a\x8e\xc9\x0d\x1e"
    			"\x38\xc8\x06\x69\xd7\x54\xcc\xa0"
    			"\x87\x4b\x3b\x20\x46\x02\x61\xdb"
    			"\xb6\x10\x8e\x0a\x7e\x1e\xa3\xce",
    	.len	= 128,
    },

    {
        /* These are from openssl */
        .ilen = 0,
        .alg = SCE_TDES,
        .mode = ECB,
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef"
               "\x55\x55\x55\x55\x55\x55\x55\x55"
               "\xfe\xdc\xba\x98\x76\x54\x32\x10",
        .klen = 24,
        .ptext = "\x73\x6f\x6d\x65\x64\x61\x74\x61",
        .ctext = "\x18\xd7\x48\xe5\x63\x62\x05\x72",
        .len = 8,
    },
    {
        .ilen = 0,
        .alg = SCE_TDES,
        .mode = ECB,
        .key = "\x03\x52\x02\x07\x67\x20\x82\x17"
               "\x86\x02\x87\x66\x59\x08\x21\x98"
               "\x64\x05\x6a\xbd\xfe\xa9\x34\x57",
        .klen = 24,
        .ptext = "\x73\x71\x75\x69\x67\x67\x6c\x65",
        .ctext = "\xc0\x7d\x2a\x0f\xa5\x66\xfa\x30",
        .len = 8,
    },
    {
        .ilen = 0,
        .alg = SCE_TDES,
        .mode = ECB,
        .key = "\x10\x46\x10\x34\x89\x98\x80\x20"
               "\x91\x07\xd0\x15\x89\x19\x01\x01"
               "\x19\x07\x92\x10\x98\x1a\x01\x01",
        .klen = 24,
        .ptext = "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ctext = "\xe1\xef\x62\xc3\x32\xfe\x82\x5b",
        .len = 8,
    },
    {
        /* Generated with Crypto++ */
        .ilen = 0,
        .alg = SCE_TDES,
        .mode = ECB,
        .key = "\xF3\x9C\xD6\xF3\x9C\xB9\x5A\x67"
               "\x00\x5A\x67\x00\x2D\xCE\xEB\x2D"
               "\xCE\xEB\xB4\x51\x72\xB4\x51\x72",
        .klen = 24,
        .ptext = "\x05\xEC\x77\xFB\x42\xD5\x59\x20"
                 "\x8B\x12\x86\x69\xF0\x5B\xCF\x56"
                 "\x39\xAD\x34\x9F\x66\xEA\x7D\xC4"
                 "\x48\xD3\xBA\x0D\xB1\x18\xE3\x4A"
                 "\xFE\x41\x28\x5C\x27\x8E\x11\x85"
                 "\x6C\xF7\x5E\xC2\x55\x3C\xA0\x0B"
                 "\x92\x65\xE9\x70\xDB\x4F\xD6\xB9"
                 "\x00\xB4\x1F\xE6\x49\xFD\x44\x2F"
                 "\x53\x3A\x8D\x14\x98\x63\xCA\x5D"
                 "\xC1\xA8\x33\xA7\x0E\x91\x78\xEC"
                 "\x77\xDE\x42\xD5\xBC\x07\x8B\x12"
                 "\xE5\x4C\xF0\x5B\x22\x56\x39\x80"
                 "\x6B\x9F\x66\xC9\x50\xC4\xAF\x36"
                 "\xBA\x0D\x94\x7F\xE3\x4A\xDD\x41"
                 "\x28\xB3\x1A\x8E\x11\xF8\x43\xF7"
                 "\x5E\x21\x55\x3C\x87\x6E\x92\x65"
                 "\xCC\x57\xDB\xA2\x35\xB9\x00\xEB"
                 "\x72\xE6\x49\xD0\x44\x2F\xB6\x19"
                 "\x8D\x14\xFF\x46\xCA\x5D\x24\xA8"
                 "\x33\x9A\x6D\x91\x78\xC3\x77\xDE"
                 "\xA1\x08\xBC\x07\xEE\x71\xE5\x4C"
                 "\xD7\x5B\x22\xB5\x1C\x80\x6B\xF2"
                 "\x45\xC9\x50\x3B\xAF\x36\x99\x60"
                 "\x94\x7F\xC6\x4A\xDD\xA4\x0F\xB3"
                 "\x1A\xED\x74\xF8\x43\x2A\x5E\x21"
                 "\x88\x13\x87\x6E\xF1\x58\xCC\x57"
                 "\x3E\xA2\x35\x9C\x67\xEB\x72\xC5"
                 "\x49\xD0\xBB\x02\xB6\x19\xE0\x4B"
                 "\xFF\x46\x29\x5D\x24\x8F\x16\x9A"
                 "\x6D\xF4\x5F\xC3\xAA\x3D\xA1\x08"
                 "\x93\x7A\xEE\x71\xD8\x4C\xD7\xBE"
                 "\x01\xB5\x1C\xE7\x4E\xF2\x45\x2C"
                 "\x50\x3B\x82\x15\x99\x60\xCB\x52"
                 "\xC6\xA9\x30\xA4\x0F\x96\x79\xED"
                 "\x74\xDF\x43\x2A\xBD\x04\x88\x13"
                 "\xFA\x4D\xF1\x58\x23\x57\x3E\x81"
                 "\x68\x9C\x67\xCE\x51\xC5\xAC\x37"
                 "\xBB\x02\x95\x7C\xE0\x4B\xD2\x46"
                 "\x29\xB0\x1B\x8F\x16\xF9\x40\xF4"
                 "\x5F\x26\xAA\x3D\x84\x6F\x93\x7A"
                 "\xCD\x54\xD8\xA3\x0A\xBE\x01\xE8"
                 "\x73\xE7\x4E\xD1\x45\x2C\xB7\x1E"
                 "\x82\x15\xFC\x47\xCB\x52\x25\xA9"
                 "\x30\x9B\x62\x96\x79\xC0\x74\xDF"
                 "\xA6\x09\xBD\x04\xEF\x76\xFA\x4D"
                 "\xD4\x58\x23\x8A\x1D\x81\x68\xF3"
                 "\x5A\xCE\x51\x38\xAC\x37\x9E\x61"
                 "\x95\x7C\xC7\x4B\xD2\xA5\x0C\xB0"
                 "\x1B\xE2\x75\xF9\x40\x2B\x5F\x26"
                 "\x89\x10\x84\x6F\xF6\x59\xCD\x54"
                 "\x3F\xA3\x0A\x9D\x64\xE8\x73\xDA"
                 "\x4E\xD1\xB8\x03\xB7\x1E\xE1\x48"
                 "\xFC\x47\x2E\x52\x25\x8C\x17\x9B"
                 "\x62\xF5\x5C\xC0\xAB\x32\xA6\x09"
                 "\x90\x7B\xEF\x76\xD9\x4D\xD4\xBF"
                 "\x06\x8A\x1D\xE4\x4F\xF3\x5A\x2D"
                 "\x51\x38\x83\x6A\x9E\x61\xC8\x53"
                 "\xC7\xAE\x31\xA5\x0C\x97\x7E\xE2"
                 "\x75\xDC\x40\x2B\xB2\x05\x89\x10"
                 "\xFB\x42\xF6\x59\x20\x54\x3F\x86"
                 "\x69\x9D\x64\xCF\x56\xDA\xAD\x34"
                 "\xB8\x03\xEA\x7D\xE1\x48\xD3\x47",
        .ctext = "\x4E\x9A\x40\x3D\x61\x7D\x17\xFA"
                 "\x16\x86\x88\x0B\xD8\xAE\xF8\xE4"
                 "\x81\x01\x04\x00\x76\xFA\xED\xD3"
                 "\x44\x7E\x21\x9D\xF0\xFB\x2B\x64"
                 "\xCA\x4E\x90\xE0\xC0\x63\x28\x92"
                 "\xF3\x1F\xA4\x53\x2C\x77\xCC\x77"
                 "\x69\x56\xD0\x19\xAD\x00\x2D\x97"
                 "\xBC\xDE\x49\x6A\x82\xBC\x16\xE2"
                 "\x2F\x3E\x72\xEE\xD1\xCE\xFC\x1B"
                 "\xEA\x32\x56\xE4\x0B\xAF\x27\x36"
                 "\xAF\x08\xB9\x61\xB7\x48\x23\x27"
                 "\xEE\x4D\xC8\x79\x56\x06\xEB\xC7"
                 "\x5B\xCA\x0A\xC6\x5E\x5C\xCB\xB6"
                 "\x9D\xDA\x04\x59\xE2\x09\x48\x7E"
                 "\x6B\x37\xC6\xFE\x92\xA9\x1E\x6E"
                 "\x0D\x19\xFA\x33\x0F\xEE\x36\x68"
                 "\x11\xBB\xF9\x5A\x73\xAB\x3A\xEA"
                 "\xAC\x28\xD8\xD5\x27\xE8\x6B\x16"
                 "\x45\x86\x50\x01\x70\x35\x99\x92"
                 "\xDF\x0C\x07\x88\x8B\x7F\x9E\x4B"
                 "\xD2\x04\x84\x90\xC4\x27\xDF\x0A"
                 "\x49\xA8\xA7\x1A\x6D\x78\x16\xCA"
                 "\xB3\x18\x5C\xC3\x93\x63\x5A\x68"
                 "\x77\x02\xBA\xED\x62\x71\xB1\xD9"
                 "\x5E\xE5\x6F\x1A\xCC\x1D\xBE\x2E"
                 "\x11\xF3\xA6\x97\xCA\x8E\xBF\xB4"
                 "\x56\xA1\x36\x6B\xB1\x0A\x3E\x70"
                 "\xEA\xD7\xCD\x72\x7B\x79\xC8\xAD"
                 "\x6B\xFE\xFB\xBA\x64\xAE\x19\xC1"
                 "\x82\xCF\x8A\xA1\x50\x17\x7F\xB2"
                 "\x6F\x7B\x0F\x52\xC5\x3E\x4A\x52"
                 "\x3F\xD9\x3F\x01\xA6\x41\x1A\xB3"
                 "\xB3\x7A\x0E\x8E\x75\xB2\xB1\x5F"
                 "\xDB\xEA\x84\x13\x26\x6C\x85\x4E"
                 "\xAE\x6B\xDC\xE7\xE7\xAD\xB0\x06"
                 "\x5C\xBA\x92\xD0\x30\xBB\x8D\xD2"
                 "\xAE\x4C\x70\x85\xA0\x07\xE3\x2C"
                 "\xD1\x27\x9C\xCF\xDB\x13\xB7\xE5"
                 "\xF9\x6A\x02\xD0\x39\x9D\xB6\xE7"
                 "\xD1\x17\x25\x08\xF9\xA9\xA6\x67"
                 "\x38\x80\xD1\x22\xAB\x1A\xD7\x26"
                 "\xAD\xCA\x19\x1B\xFA\x18\xA7\x57"
                 "\x31\xEC\xC9\xED\xDB\x79\xC0\x48"
                 "\xAC\x31\x9F\x03\x8B\x62\x5B\x7E"
                 "\x0E\xA6\xD0\x64\xEE\xEA\x00\xFC"
                 "\x58\xC8\xDE\x51\x4E\x17\x15\x11"
                 "\x66\x58\xB6\x90\xDC\xDF\xA1\x49"
                 "\xCA\x79\xE9\x31\x31\x42\xDC\x56"
                 "\x0B\xCD\xB6\x0D\xC7\x64\xF7\x19"
                 "\xD9\x42\x05\x7F\xBC\x2F\xFC\x90"
                 "\xAE\x29\x86\xAA\x43\x7A\x4F\x6B"
                 "\xCE\xEA\xBC\x31\x8D\x65\x9D\x46"
                 "\xEA\x77\xB4\xF9\x58\xEA\x5D\x84"
                 "\xE4\xDC\x14\xBB\xBD\x15\x0E\xDA"
                 "\xD8\xE4\xA4\x5D\x61\xF9\x58\x0F"
                 "\xE4\x82\x77\xCE\x87\xC0\x09\xF0"
                 "\xD6\x10\x9E\x34\xE1\x0C\x67\x55"
                 "\x7B\x6D\xD5\x51\x4B\x00\xEE\xBA"
                 "\xF2\x7B\xBE\x75\x07\x42\x9D\x99"
                 "\x12\xE1\x71\x4A\xF9\x2A\xF5\xF6"
                 "\x93\x03\xD7\x51\x09\xFA\xBE\x68"
                 "\xD8\x45\xFF\x33\xBA\xBB\x2B\x63",
        .len = 496,
    },

    {
        /* Generated from openssl */
        .ilen = 8,
        .alg = SCE_TDES,
        .mode = CBC,
        .key = "\xE9\xC0\xFF\x2E\x76\x0B\x64\x24"
               "\x44\x4D\x99\x5A\x12\xD6\x40\xC0"
               "\xEA\xC2\x84\xE8\x14\x95\xDB\xE8",
        .klen = 24,
        .iv = "\x7D\x33\x88\x93\x0F\x93\xB2\x42",
        .ptext = "\x6f\x54\x20\x6f\x61\x4d\x79\x6e"
                 "\x53\x20\x63\x65\x65\x72\x73\x74"
                 "\x54\x20\x6f\x6f\x4d\x20\x6e\x61"
                 "\x20\x79\x65\x53\x72\x63\x74\x65"
                 "\x20\x73\x6f\x54\x20\x6f\x61\x4d"
                 "\x79\x6e\x53\x20\x63\x65\x65\x72"
                 "\x73\x74\x54\x20\x6f\x6f\x4d\x20"
                 "\x6e\x61\x20\x79\x65\x53\x72\x63"
                 "\x74\x65\x20\x73\x6f\x54\x20\x6f"
                 "\x61\x4d\x79\x6e\x53\x20\x63\x65"
                 "\x65\x72\x73\x74\x54\x20\x6f\x6f"
                 "\x4d\x20\x6e\x61\x20\x79\x65\x53"
                 "\x72\x63\x74\x65\x20\x73\x6f\x54"
                 "\x20\x6f\x61\x4d\x79\x6e\x53\x20"
                 "\x63\x65\x65\x72\x73\x74\x54\x20"
                 "\x6f\x6f\x4d\x20\x6e\x61\x0a\x79",
        .ctext = "\x0e\x2d\xb6\x97\x3c\x56\x33\xf4"
                 "\x67\x17\x21\xc7\x6e\x8a\xd5\x49"
                 "\x74\xb3\x49\x05\xc5\x1c\xd0\xed"
                 "\x12\x56\x5c\x53\x96\xb6\x00\x7d"
                 "\x90\x48\xfc\xf5\x8d\x29\x39\xcc"
                 "\x8a\xd5\x35\x18\x36\x23\x4e\xd7"
                 "\x76\xd1\xda\x0c\x94\x67\xbb\x04"
                 "\x8b\xf2\x03\x6c\xa8\xcf\xb6\xea"
                 "\x22\x64\x47\xaa\x8f\x75\x13\xbf"
                 "\x9f\xc2\xc3\xf0\xc9\x56\xc5\x7a"
                 "\x71\x63\x2e\x89\x7b\x1e\x12\xca"
                 "\xe2\x5f\xaf\xd8\xa4\xf8\xc9\x7a"
                 "\xd6\xf9\x21\x31\x62\x44\x45\xa6"
                 "\xd6\xbc\x5a\xd3\x2d\x54\x43\xcc"
                 "\x9d\xde\xa5\x70\xe9\x42\x45\x8a"
                 "\x6b\xfa\xb1\x91\x13\xb0\xd9\x19",
        .len = 128,
    },
    {
        /* Generated with Crypto++ */
        .ilen = 16,
        .alg = SCE_TDES,
        .mode = CBC,
        .key = "\x9C\xD6\xF3\x9C\xB9\x5A\x67\x00"
               "\x5A\x67\x00\x2D\xCE\xEB\x2D\xCE"
               "\xEB\xB4\x51\x72\xB4\x51\x72\x1F",
        .klen = 24,
        .iv = "\xB2\xD7\x48\xED\x06\x44\xF9\x12"
              "\xB7\x28\x4D\x83\x24\x59\xF2\x17",
        .ptext = "\x05\xEC\x77\xFB\x42\xD5\x59\x20"
                 "\x8B\x12\x86\x69\xF0\x5B\xCF\x56"
                 "\x39\xAD\x34\x9F\x66\xEA\x7D\xC4"
                 "\x48\xD3\xBA\x0D\xB1\x18\xE3\x4A"
                 "\xFE\x41\x28\x5C\x27\x8E\x11\x85"
                 "\x6C\xF7\x5E\xC2\x55\x3C\xA0\x0B"
                 "\x92\x65\xE9\x70\xDB\x4F\xD6\xB9"
                 "\x00\xB4\x1F\xE6\x49\xFD\x44\x2F"
                 "\x53\x3A\x8D\x14\x98\x63\xCA\x5D"
                 "\xC1\xA8\x33\xA7\x0E\x91\x78\xEC"
                 "\x77\xDE\x42\xD5\xBC\x07\x8B\x12"
                 "\xE5\x4C\xF0\x5B\x22\x56\x39\x80"
                 "\x6B\x9F\x66\xC9\x50\xC4\xAF\x36"
                 "\xBA\x0D\x94\x7F\xE3\x4A\xDD\x41"
                 "\x28\xB3\x1A\x8E\x11\xF8\x43\xF7"
                 "\x5E\x21\x55\x3C\x87\x6E\x92\x65"
                 "\xCC\x57\xDB\xA2\x35\xB9\x00\xEB"
                 "\x72\xE6\x49\xD0\x44\x2F\xB6\x19"
                 "\x8D\x14\xFF\x46\xCA\x5D\x24\xA8"
                 "\x33\x9A\x6D\x91\x78\xC3\x77\xDE"
                 "\xA1\x08\xBC\x07\xEE\x71\xE5\x4C"
                 "\xD7\x5B\x22\xB5\x1C\x80\x6B\xF2"
                 "\x45\xC9\x50\x3B\xAF\x36\x99\x60"
                 "\x94\x7F\xC6\x4A\xDD\xA4\x0F\xB3"
                 "\x1A\xED\x74\xF8\x43\x2A\x5E\x21"
                 "\x88\x13\x87\x6E\xF1\x58\xCC\x57"
                 "\x3E\xA2\x35\x9C\x67\xEB\x72\xC5"
                 "\x49\xD0\xBB\x02\xB6\x19\xE0\x4B"
                 "\xFF\x46\x29\x5D\x24\x8F\x16\x9A"
                 "\x6D\xF4\x5F\xC3\xAA\x3D\xA1\x08"
                 "\x93\x7A\xEE\x71\xD8\x4C\xD7\xBE"
                 "\x01\xB5\x1C\xE7\x4E\xF2\x45\x2C"
                 "\x50\x3B\x82\x15\x99\x60\xCB\x52"
                 "\xC6\xA9\x30\xA4\x0F\x96\x79\xED"
                 "\x74\xDF\x43\x2A\xBD\x04\x88\x13"
                 "\xFA\x4D\xF1\x58\x23\x57\x3E\x81"
                 "\x68\x9C\x67\xCE\x51\xC5\xAC\x37"
                 "\xBB\x02\x95\x7C\xE0\x4B\xD2\x46"
                 "\x29\xB0\x1B\x8F\x16\xF9\x40\xF4"
                 "\x5F\x26\xAA\x3D\x84\x6F\x93\x7A"
                 "\xCD\x54\xD8\xA3\x0A\xBE\x01\xE8"
                 "\x73\xE7\x4E\xD1\x45\x2C\xB7\x1E"
                 "\x82\x15\xFC\x47\xCB\x52\x25\xA9"
                 "\x30\x9B\x62\x96\x79\xC0\x74\xDF"
                 "\xA6\x09\xBD\x04\xEF\x76\xFA\x4D"
                 "\xD4\x58\x23\x8A\x1D\x81\x68\xF3"
                 "\x5A\xCE\x51\x38\xAC\x37\x9E\x61"
                 "\x95\x7C\xC7\x4B\xD2\xA5\x0C\xB0"
                 "\x1B\xE2\x75\xF9\x40\x2B\x5F\x26"
                 "\x89\x10\x84\x6F\xF6\x59\xCD\x54"
                 "\x3F\xA3\x0A\x9D\x64\xE8\x73\xDA"
                 "\x4E\xD1\xB8\x03\xB7\x1E\xE1\x48"
                 "\xFC\x47\x2E\x52\x25\x8C\x17\x9B"
                 "\x62\xF5\x5C\xC0\xAB\x32\xA6\x09"
                 "\x90\x7B\xEF\x76\xD9\x4D\xD4\xBF"
                 "\x06\x8A\x1D\xE4\x4F\xF3\x5A\x2D"
                 "\x51\x38\x83\x6A\x9E\x61\xC8\x53"
                 "\xC7\xAE\x31\xA5\x0C\x97\x7E\xE2"
                 "\x75\xDC\x40\x2B\xB2\x05\x89\x10"
                 "\xFB\x42\xF6\x59\x20\x54\x3F\x86"
                 "\x69\x9D\x64\xCF\x56\xDA\xAD\x34"
                 "\xB8\x03\xEA\x7D\xE1\x48\xD3\x47",
        .ctext = "\xF8\xF6\xB5\x60\x5C\x5A\x75\x84"
                 "\x87\x81\x53\xBA\xC9\x6F\xEC\xD5"
                 "\x1E\x68\x8E\x85\x12\x86\x1D\x38"
                 "\x1C\x91\x40\xCC\x69\x6A\xD5\x35"
                 "\x0D\x7C\xB5\x07\x7C\x7B\x2A\xAF"
                 "\x32\xBC\xA1\xB3\x84\x31\x1B\x3C"
                 "\x0A\x2B\xFA\xD3\x9F\xB0\x8C\x37"
                 "\x8F\x9D\xA7\x6D\x6C\xFA\xD7\x90"
                 "\xE3\x69\x54\xED\x3A\xC4\xF1\x6B"
                 "\xB1\xCC\xFB\x7D\xD8\x8E\x17\x0B"
                 "\x9C\xF6\x4C\xD6\xFF\x03\x4E\xD9"
                 "\xE6\xA5\xAD\x25\xE6\x17\x69\x63"
                 "\x11\x35\x61\x94\x88\x7B\x1C\x48"
                 "\xF1\x24\x20\x29\x6B\x93\x1A\x8E"
                 "\x43\x03\x89\xD8\xB1\xDA\x47\x7B"
                 "\x79\x3A\x83\x76\xDA\xAE\xC6\xBB"
                 "\x22\xF8\xE8\x3D\x9A\x65\x54\xD8"
                 "\x4C\xE9\xE7\xE4\x63\x2F\x5C\x73"
                 "\x5A\xC3\xAE\x46\xA8\xCD\x57\xE6"
                 "\x67\x88\xA5\x20\x6F\x5F\x97\xC7"
                 "\xCC\x15\xA2\x0A\x93\xEA\x33\xE7"
                 "\x03\x5F\xEC\x64\x30\x6F\xEE\xD7"
                 "\x7E\xDF\xD6\xE9\x6F\x3F\xD6\x1E"
                 "\xBE\x67\x6C\x5B\x97\xA0\x09\xE6"
                 "\xEE\xFE\x55\xA3\x29\x65\xE0\x12"
                 "\xA1\x6A\x8A\x6F\xF2\xE6\xF1\x96"
                 "\x87\xFB\x9C\x05\xDD\x80\xEC\xFF"
                 "\xC5\xED\x50\xFE\xFC\x91\xCD\xCE"
                 "\x25\x2C\x5F\xD9\xAD\x95\x7D\x99"
                 "\xF0\x05\xC4\x71\x46\x5F\xF9\x0D"
                 "\xD2\x63\xDF\x9B\x96\x2E\x2B\xA6"
                 "\x2B\x1C\xD5\xFB\x96\x24\x60\x60"
                 "\x54\x40\xB8\x62\xA4\xF8\x46\x95"
                 "\x73\x28\xA3\xA6\x16\x2B\x17\xE7"
                 "\x7A\xF8\x62\x54\x3B\x64\x69\xE1"
                 "\x71\x34\x29\x5B\x4E\x05\x9B\xFA"
                 "\x5E\xF1\x96\xB7\xCE\x16\x9B\x59"
                 "\xF1\x1A\x4C\x51\x26\xFD\x79\xE2"
                 "\x3B\x8E\x71\x69\x6A\x91\xB6\x65"
                 "\x32\x09\xB8\xE4\x09\x1F\xEA\x39"
                 "\xCE\x20\x65\x9F\xD6\xD1\xC7\xF0"
                 "\x73\x50\x08\x56\x20\x9B\x94\x23"
                 "\x14\x39\xB7\x2B\xB1\x2D\x6D\x6F"
                 "\x41\x5B\xCC\xE2\x18\xAE\x62\x89"
                 "\x78\x8E\x67\x23\xD0\xFB\x2B\xE5"
                 "\x25\xC9\x48\x97\xB5\xD3\x17\xD5"
                 "\x6A\x9F\xA7\x48\x0C\x2B\x73\x3B"
                 "\x57\x08\xAE\x91\xF2\xB7\x57\x89"
                 "\xF4\xD0\xB0\x07\xB0\x42\x6C\xAF"
                 "\x98\x1A\xE7\xD1\xAC\x1E\xB5\x02"
                 "\xD4\x56\x42\x79\x79\x7F\x2A\x77"
                 "\x25\xE9\x7D\xC1\x88\x19\x2B\x49"
                 "\x6F\x46\x59\xAB\x56\x1F\x61\xE0"
                 "\x0C\x24\x9C\xC9\x5B\x63\xA9\x12"
                 "\xCF\x88\x96\xB6\xA8\x24\xC6\xA8"
                 "\x21\x85\x1A\x62\x7E\x34\xBB\xEB"
                 "\xBD\x02\x2A\xC7\xD8\x89\x80\xC5"
                 "\xB1\xBB\x60\xA5\x22\xFC\x6F\x38"
                 "\x02\x80\xA3\x28\x22\x75\xE1\xE9"
                 "\x90\xE9\xFA\x4B\x00\x10\xAC\x58"
                 "\x83\x70\xFF\x86\xE6\xAA\x0F\x1F"
                 "\x95\x63\x73\xA2\x44\xAC\xF8\xA5",
        .len = 496,
    },
    {
        /* Generated with Crypto++ */
        .ilen = 8,
        .alg = SCE_TDES,
        .mode = CTR,
        .key = "\x9C\xD6\xF3\x9C\xB9\x5A\x67\x00"
               "\x5A\x67\x00\x2D\xCE\xEB\x2D\xCE"
               "\xEB\xB4\x51\x72\xB4\x51\x72\x1F",
        .klen = 24,
        .iv = "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF",
        .ptext = "\x05\xEC\x77\xFB\x42\xD5\x59\x20"
                 "\x8B\x12\x86\x69\xF0\x5B\xCF\x56"
                 "\x39\xAD\x34\x9F\x66\xEA\x7D\xC4"
                 "\x48\xD3\xBA\x0D\xB1\x18\xE3\x4A"
                 "\xFE\x41\x28\x5C\x27\x8E\x11\x85"
                 "\x6C\xF7\x5E\xC2\x55\x3C\xA0\x0B"
                 "\x92\x65\xE9\x70\xDB\x4F\xD6\xB9"
                 "\x00\xB4\x1F\xE6\x49\xFD\x44\x2F"
                 "\x53\x3A\x8D\x14\x98\x63\xCA\x5D"
                 "\xC1\xA8\x33\xA7\x0E\x91\x78\xEC"
                 "\x77\xDE\x42\xD5\xBC\x07\x8B\x12"
                 "\xE5\x4C\xF0\x5B\x22\x56\x39\x80"
                 "\x6B\x9F\x66\xC9\x50\xC4\xAF\x36"
                 "\xBA\x0D\x94\x7F\xE3\x4A\xDD\x41"
                 "\x28\xB3\x1A\x8E\x11\xF8\x43\xF7"
                 "\x5E\x21\x55\x3C\x87\x6E\x92\x65"
                 "\xCC\x57\xDB\xA2\x35\xB9\x00\xEB"
                 "\x72\xE6\x49\xD0\x44\x2F\xB6\x19"
                 "\x8D\x14\xFF\x46\xCA\x5D\x24\xA8"
                 "\x33\x9A\x6D\x91\x78\xC3\x77\xDE"
                 "\xA1\x08\xBC\x07\xEE\x71\xE5\x4C"
                 "\xD7\x5B\x22\xB5\x1C\x80\x6B\xF2"
                 "\x45\xC9\x50\x3B\xAF\x36\x99\x60"
                 "\x94\x7F\xC6\x4A\xDD\xA4\x0F\xB3"
                 "\x1A\xED\x74\xF8\x43\x2A\x5E\x21"
                 "\x88\x13\x87\x6E\xF1\x58\xCC\x57"
                 "\x3E\xA2\x35\x9C\x67\xEB\x72\xC5"
                 "\x49\xD0\xBB\x02\xB6\x19\xE0\x4B"
                 "\xFF\x46\x29\x5D\x24\x8F\x16\x9A"
                 "\x6D\xF4\x5F\xC3\xAA\x3D\xA1\x08"
                 "\x93\x7A\xEE\x71\xD8\x4C\xD7\xBE"
                 "\x01\xB5\x1C\xE7\x4E\xF2\x45\x2C"
                 "\x50\x3B\x82\x15\x99\x60\xCB\x52"
                 "\xC6\xA9\x30\xA4\x0F\x96\x79\xED"
                 "\x74\xDF\x43\x2A\xBD\x04\x88\x13"
                 "\xFA\x4D\xF1\x58\x23\x57\x3E\x81"
                 "\x68\x9C\x67\xCE\x51\xC5\xAC\x37"
                 "\xBB\x02\x95\x7C\xE0\x4B\xD2\x46"
                 "\x29\xB0\x1B\x8F\x16\xF9\x40\xF4"
                 "\x5F\x26\xAA\x3D\x84\x6F\x93\x7A"
                 "\xCD\x54\xD8\xA3\x0A\xBE\x01\xE8"
                 "\x73\xE7\x4E\xD1\x45\x2C\xB7\x1E"
                 "\x82\x15\xFC\x47\xCB\x52\x25\xA9"
                 "\x30\x9B\x62\x96\x79\xC0\x74\xDF"
                 "\xA6\x09\xBD\x04\xEF\x76\xFA\x4D"
                 "\xD4\x58\x23\x8A\x1D\x81\x68\xF3"
                 "\x5A\xCE\x51\x38\xAC\x37\x9E\x61"
                 "\x95\x7C\xC7\x4B\xD2\xA5\x0C\xB0"
                 "\x1B\xE2\x75\xF9\x40\x2B\x5F\x26"
                 "\x89\x10\x84\x6F\xF6\x59\xCD\x54"
                 "\x3F\xA3\x0A\x9D\x64\xE8\x73\xDA"
                 "\x4E\xD1\xB8\x03\xB7\x1E\xE1\x48"
                 "\xFC\x47\x2E\x52\x25\x8C\x17\x9B"
                 "\x62\xF5\x5C\xC0\xAB\x32\xA6\x09"
                 "\x90\x7B\xEF\x76\xD9\x4D\xD4\xBF"
                 "\x06\x8A\x1D\xE4\x4F\xF3\x5A\x2D"
                 "\x51\x38\x83\x6A\x9E\x61\xC8\x53"
                 "\xC7\xAE\x31\xA5\x0C\x97\x7E\xE2"
                 "\x75\xDC\x40\x2B\xB2\x05\x89\x10"
                 "\xFB\x42\xF6\x59\x20\x54\x3F\x86"
                 "\x69\x9D\x64\xCF\x56\xDA\xAD\x34"
                 "\xB8\x03\xEA\x7D\xE1\x48\xD3\x47",
        .ctext = "\x07\xC2\x08\x20\x72\x1F\x49\xEF"
                 "\x19\xCD\x6F\x32\x53\x05\x22\x15"
                 "\xA2\x85\x2B\xDB\x85\xD2\xD8\xB9"
                 "\xDD\x0D\x1B\x45\xCB\x69\x11\xD4"
                 "\xEA\xBE\xB2\x45\x5D\x0C\xAE\xBE"
                 "\xA0\xC1\x27\xAC\x65\x9F\x53\x7E"
                 "\xAF\xC2\x1B\xB5\xB8\x6D\x36\x0C"
                 "\x25\xC0\xF8\x6D\x0B\x29\x01\xDA"
                 "\x13\x78\xDC\x89\x12\x12\x43\xFA"
                 "\xF6\x12\xEF\x8D\x87\x62\x78\x83"
                 "\xE2\xBE\x41\x20\x4C\x6D\x35\x1B"
                 "\xD1\x0C\x30\xCF\xE2\xDE\x2B\x03"
                 "\xBF\x45\x73\xD4\xE5\x59\x95\xD1"
                 "\xB3\x9B\x27\x62\x97\xBD\xDE\x7F"
                 "\xA4\xD2\x39\x80\xAA\x50\x23\xF0"
                 "\x74\x88\x3D\xA8\x6A\x18\x79\x3B"
                 "\xC4\x96\x6C\x8D\x22\x40\x92\x6E"
                 "\xD6\xAD\x2A\x1F\xDE\x63\xC0\xE7"
                 "\x07\xF7\x2D\xF7\xB5\xF3\xF0\xCC"
                 "\x01\x7C\x2A\x9B\xC2\x10\xCA\xAA"
                 "\xFD\x2B\x3F\xC5\xF3\xF6\xFC\x9B"
                 "\x45\xDB\x53\xE4\x5B\xF3\xC9\x7B"
                 "\x8E\x52\xFF\xC8\x02\xB8\xAC\x9D"
                 "\xA1\x00\x39\xDA\x3D\x2D\x0E\x01"
                 "\x09\x7D\x8D\x5E\xBE\x53\xB9\xB0"
                 "\x8E\xE7\xE2\x96\x6A\xB2\x78\xEA"
                 "\xDE\x23\x8B\xA5\xFA\x5C\xE3\xDA"
                 "\xBF\x8E\x31\x6A\x55\xD1\x6A\xB2"
                 "\xB5\x46\x6F\xA5\xF0\xEE\xBA\x1F"
                 "\x9F\x98\xB0\x66\x4F\xD0\x3F\xA9"
                 "\xDF\x5F\x58\xC4\xF4\xFF\x75\x5C"
                 "\x40\x3A\x09\x7E\x6E\x1C\x97\xD4"
                 "\xCC\xE7\xE7\x71\xCF\x0B\x15\x08"
                 "\x71\xFA\x07\x97\xCD\xE6\xCA\x1D"
                 "\x14\x28\x0C\xCF\x99\x13\x7A\xF1"
                 "\xEB\xFA\xFA\x92\x07\xDE\x1D\xA1"
                 "\xD3\x36\x69\xFE\x51\x4D\x9F\x2E"
                 "\x83\x37\x4F\x1F\x48\x30\xED\x04"
                 "\x4D\xA4\xEF\x3A\xCA\x76\xF4\x1C"
                 "\x41\x8F\x63\x37\x78\x2F\x86\xA6"
                 "\xEF\x41\x7E\xD2\xAF\x88\xAB\x67"
                 "\x52\x71\xC3\x8E\xF8\x26\x93\x72"
                 "\xAA\xD6\x0E\xE7\x0B\x46\xB1\x3A"
                 "\xB4\x08\xA9\xA8\xA0\xCF\x20\x0C"
                 "\x52\xBC\x8B\x05\x56\xB2\xBC\x31"
                 "\x9B\x74\xB9\x29\x29\x96\x9A\x50"
                 "\xDC\x45\xDC\x1A\xEB\x0C\x64\xD4"
                 "\xD3\x05\x7E\x59\x55\xC3\xF4\x90"
                 "\xC2\xAB\xF8\x9B\x8A\xDA\xCE\xA1"
                 "\xC3\xF4\xAD\x77\xDD\x44\xC8\xAC"
                 "\xA3\xF1\xC9\xD2\x19\x5C\xB0\xCA"
                 "\xA2\x34\xC1\xF7\x6C\xFD\xAC\x65"
                 "\x32\xDC\x48\xC4\xF2\x00\x6B\x77"
                 "\xF1\x7D\x76\xAC\xC0\x31\x63\x2A"
                 "\xA5\x3A\x62\xC8\x91\xB1\x03\x65"
                 "\xCB\x43\xD1\x06\xDF\xC3\x67\xBC"
                 "\xDC\xE0\xCD\x35\xCE\x49\x65\xA0"
                 "\x52\x7B\xA7\x0D\x07\xA9\x1B\xB0"
                 "\x40\x77\x72\xC2\xEA\x0E\x3A\x78"
                 "\x46\xB9\x91\xB6\xE7\x3D\x51\x42"
                 "\xFD\x51\xB0\xC6\x2C\x63\x13\x78"
                 "\x5C\xEE\xFC\xCF\xC4\x70\x00\x34",
        .len = 496,
    },
    {
        /* Generated with Crypto++ */
        .ilen = 8,
        .alg = SCE_TDES,
        .mode = CTR,
        .key = "\x9C\xD6\xF3\x9C\xB9\x5A\x67\x00"
               "\x5A\x67\x00\x2D\xCE\xEB\x2D\xCE"
               "\xEB\xB4\x51\x72\xB4\x51\x72\x1F",
        .klen = 24,
        .iv = "\xB2\xD7\x48\xED\x06\x44\xF9\x12",
        .ptext = "\x05\xEC\x77\xFB\x42\xD5\x59\x20"
                 "\x8B\x12\x86\x69\xF0\x5B\xCF\x56"
                 "\x39\xAD\x34\x9F\x66\xEA\x7D\xC4"
                 "\x48\xD3\xBA\x0D\xB1\x18\xE3\x4A"
                 "\xFE\x41\x28\x5C\x27\x8E\x11\x85"
                 "\x6C\xF7\x5E\xC2\x55\x3C\xA0\x0B"
                 "\x92\x65\xE9\x70\xDB\x4F\xD6\xB9"
                 "\x00\xB4\x1F\xE6\x49\xFD\x44\x2F"
                 "\x53\x3A\x8D\x14\x98\x63\xCA\x5D"
                 "\xC1\xA8\x33\xA7\x0E\x91\x78\xEC"
                 "\x77\xDE\x42\xD5\xBC\x07\x8B\x12"
                 "\xE5\x4C\xF0\x5B\x22\x56\x39\x80"
                 "\x6B\x9F\x66\xC9\x50\xC4\xAF\x36"
                 "\xBA\x0D\x94\x7F\xE3\x4A\xDD\x41"
                 "\x28\xB3\x1A\x8E\x11\xF8\x43\xF7"
                 "\x5E\x21\x55\x3C\x87\x6E\x92\x65"
                 "\xCC\x57\xDB\xA2\x35\xB9\x00\xEB"
                 "\x72\xE6\x49\xD0\x44\x2F\xB6\x19"
                 "\x8D\x14\xFF\x46\xCA\x5D\x24\xA8"
                 "\x33\x9A\x6D\x91\x78\xC3\x77\xDE"
                 "\xA1\x08\xBC\x07\xEE\x71\xE5\x4C"
                 "\xD7\x5B\x22\xB5\x1C\x80\x6B\xF2"
                 "\x45\xC9\x50\x3B\xAF\x36\x99\x60"
                 "\x94\x7F\xC6\x4A\xDD\xA4\x0F\xB3"
                 "\x1A\xED\x74\xF8\x43\x2A\x5E\x21"
                 "\x88\x13\x87\x6E\xF1\x58\xCC\x57"
                 "\x3E\xA2\x35\x9C\x67\xEB\x72\xC5"
                 "\x49\xD0\xBB\x02\xB6\x19\xE0\x4B"
                 "\xFF\x46\x29\x5D\x24\x8F\x16\x9A"
                 "\x6D\xF4\x5F\xC3\xAA\x3D\xA1\x08"
                 "\x93\x7A\xEE\x71\xD8\x4C\xD7\xBE"
                 "\x01\xB5\x1C\xE7\x4E\xF2\x45\x2C"
                 "\x50\x3B\x82\x15\x99\x60\xCB\x52"
                 "\xC6\xA9\x30\xA4\x0F\x96\x79\xED"
                 "\x74\xDF\x43\x2A\xBD\x04\x88\x13"
                 "\xFA\x4D\xF1\x58\x23\x57\x3E\x81"
                 "\x68\x9C\x67\xCE\x51\xC5\xAC\x37"
                 "\xBB\x02\x95\x7C\xE0\x4B\xD2\x46"
                 "\x29\xB0\x1B\x8F\x16\xF9\x40\xF4"
                 "\x5F\x26\xAA\x3D\x84\x6F\x93\x7A"
                 "\xCD\x54\xD8\xA3\x0A\xBE\x01\xE8"
                 "\x73\xE7\x4E\xD1\x45\x2C\xB7\x1E"
                 "\x82\x15\xFC\x47\xCB\x52\x25\xA9"
                 "\x30\x9B\x62\x96\x79\xC0\x74\xDF"
                 "\xA6\x09\xBD\x04\xEF\x76\xFA\x4D"
                 "\xD4\x58\x23\x8A\x1D\x81\x68\xF3"
                 "\x5A\xCE\x51\x38\xAC\x37\x9E\x61"
                 "\x95\x7C\xC7\x4B\xD2\xA5\x0C\xB0"
                 "\x1B\xE2\x75\xF9\x40\x2B\x5F\x26"
                 "\x89\x10\x84\x6F\xF6\x59\xCD\x54"
                 "\x3F\xA3\x0A\x9D\x64\xE8\x73\xDA"
                 "\x4E\xD1\xB8\x03\xB7\x1E\xE1\x48"
                 "\xFC\x47\x2E\x52\x25\x8C\x17\x9B"
                 "\x62\xF5\x5C\xC0\xAB\x32\xA6\x09"
                 "\x90\x7B\xEF\x76\xD9\x4D\xD4\xBF"
                 "\x06\x8A\x1D\xE4\x4F\xF3\x5A\x2D"
                 "\x51\x38\x83\x6A\x9E\x61\xC8\x53"
                 "\xC7\xAE\x31\xA5\x0C\x97\x7E\xE2"
                 "\x75\xDC\x40\x2B\xB2\x05\x89\x10"
                 "\xFB\x42\xF6\x59\x20\x54\x3F\x86"
                 "\x69\x9D\x64\xCF\x56\xDA\xAD\x34"
                 "\xB8\x03\xEA\x7D\xE1\x48\xD3\x47"
                 "\x2E\xB1\x18",
        .ctext = "\x23\xFF\x5C\x99\x75\xBB\x1F\xD4"
                 "\xBC\x27\x9D\x36\x60\xA9\xC9\xF7"
                 "\x94\x9D\x1B\xFF\x8E\x95\x57\x89"
                 "\x8C\x2E\x33\x70\x43\x61\xE6\xD2"
                 "\x82\x33\x63\xB6\xC4\x34\x5E\xF8"
                 "\x96\x07\xA7\xD2\x3B\x8E\xC9\xAA"
                 "\x7C\xA0\x55\x89\x2E\xE1\x85\x25"
                 "\x14\x04\xDA\x6B\xE0\xEE\x56\xCF"
                 "\x08\x2E\x69\xD4\x54\xDE\x22\x84"
                 "\x69\xA6\xA7\xD3\x3A\x9A\xE8\x05"
                 "\x63\xDB\xBF\x46\x3A\x26\x2E\x0F"
                 "\x58\x5C\x46\xEA\x07\x40\xDA\xE1"
                 "\x14\x1D\xCD\x4F\x06\xC0\xCA\x54"
                 "\x1E\xC9\x45\x85\x67\x7C\xC2\xB5"
                 "\x97\x5D\x61\x78\x2E\x46\xEC\x6A"
                 "\x53\xF4\xD0\xAE\xFA\xB4\x86\x29"
                 "\x9F\x17\x33\x24\xD8\xB9\xB2\x05"
                 "\x93\x88\xEA\xF7\xA0\x70\x69\x49"
                 "\x88\x6B\x73\x40\x41\x8D\xD9\xD9"
                 "\x7E\x78\xE9\xBE\x6C\x14\x22\x7A"
                 "\x66\xE1\xDA\xED\x10\xFF\x69\x1D"
                 "\xB9\xAA\xF2\x56\x72\x1B\x23\xE2"
                 "\x45\x54\x8B\xA3\x70\x23\xB4\x5E"
                 "\x8E\x96\xC9\x05\x00\xB3\xB6\xC2"
                 "\x2A\x02\x43\x7A\x62\xD5\xC8\xD2"
                 "\xC2\xD0\xE4\x78\xA1\x7B\x3E\xE8"
                 "\x9F\x7F\x7D\x40\x54\x30\x3B\xC0"
                 "\xA5\x54\xFD\xCA\x25\xEC\x44\x3E"
                 "\x1A\x54\x7F\x88\xD0\xE1\xFE\x71"
                 "\xCE\x05\x49\x89\xBA\xD6\x72\xE7"
                 "\xD6\x5D\x3F\xA2\xD9\xAB\xC5\x02"
                 "\xD6\x43\x22\xAF\xA2\xE4\x80\x85"
                 "\xD7\x87\xB9\xEA\x43\xDB\xC8\xEF"
                 "\x5C\x82\x2E\x98\x0D\x30\x41\x6B"
                 "\x08\x48\x8D\xF0\xF8\x60\xD7\x9D"
                 "\xE9\xDE\x40\xAD\x0D\xAD\x0D\x58"
                 "\x2A\x98\x35\xFE\xF7\xDD\x4B\x40"
                 "\xDE\xB0\x05\xD9\x7B\x09\x4D\xBC"
                 "\x42\xC0\xF1\x15\x0B\xFA\x26\x6B"
                 "\xC6\x12\x13\x4F\xCB\x35\xBA\x35"
                 "\xDD\x7A\x36\x9C\x12\x57\x55\x83"
                 "\x78\x58\x09\xD0\xB0\xCF\x7C\x5C"
                 "\x38\xCF\xBD\x79\x5B\x13\x4D\x97"
                 "\xC1\x85\x6F\x97\xC9\xE8\xC2\xA4"
                 "\x98\xE2\xBD\x77\x6B\x53\x39\x1A"
                 "\x28\x10\xE7\xE0\xE7\xDE\x9D\x69"
                 "\x78\x6F\x8E\xD2\xD9\x5D\xD2\x15"
                 "\x9E\xB5\x4D\x8C\xC0\x78\x22\x2F"
                 "\x17\x11\x2E\x99\xD7\xE3\xA4\x4F"
                 "\x65\xA5\x6B\x03\x2C\x35\x6F\xDA"
                 "\x8A\x19\x08\xE1\x08\x48\x59\x51"
                 "\x53\x4B\xD1\xDF\xDA\x14\x50\x5F"
                 "\xDF\xB5\x8C\xDF\xC6\xFD\x85\xFA"
                 "\xD4\xF9\x64\x45\x65\x0D\x7D\xF4"
                 "\xC8\xCD\x3F\x32\xAF\xDD\x30\xED"
                 "\x7B\xAA\xAC\xF0\xDA\x7F\xDF\x75"
                 "\x1C\xA4\xF1\xCB\x5E\x4F\x0B\xB4"
                 "\x97\x73\x28\xDE\xCF\xAF\x82\xBD"
                 "\xC4\xBA\xB4\x9C\x0D\x16\x77\x42"
                 "\x42\x39\x7C\x53\xA4\xD4\xDD\x40"
                 "\x5C\x60\x1F\x6E\xA7\xE2\xDC\xE7"
                 "\x32\x0F\x05\x2F\xF2\x4C\x95\x3B"
                 "\xF2\x79\xD9",
        .len = 499,
    },
    {
        .alg    = SCE_TDES,
        .mode   = OFB,
        .key    = "\xE4\x23\x33\x8A\x35\x64\x61\xE2"
                        "\x49\x03\xDD\xC6\xB8\xCA\x55\x7A"
                        "\xd0\xe7\x4b\xfb\x5d\xe5\x0c\xe7",
        .klen   = 24,
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07",
        .ilen = 8,
        .ptext  = "\xff\xca\xfb\xf1\x38\x20\x2f\x7b"
                        "\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
                        "\xd9\xef\xbd\xad\x4e\x40\xbd\x60"
                        "\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
        .ctext  = "\x87\x8B\x22\xA2\x08\xFA\xB7\xAD"
                        "\xB9\x65\xE0\x36\x60\x11\x3A\x40"
                        "\x76\xBA\x66\xDF\xAA\x53\xEC\x86"
                        "\x84\x13\x53\xC8\x9A\x26\x54\xCD",
        .len    = 32,
    },
    {
        .alg    = SCE_TDES,
        .mode   = OFB,
        .key    = "\xE4\x23\x33\x8A\x35\x64\x61\xE2"
                        "\x49\x03\xDD\xC6\xB8\xCA\x55\x7A"
                        "\xd0\xe7\x4b\xfb\x5d\xe5\x0c\xe7",
        .klen   = 24,
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07",
        .ilen = 8,
        .ptext  = "\xff\xca\xfb\xf1\x38\x20\x2f\x7b"
                        "\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
                        "\xd9\xef\xbd\xad\x4e\x40\xbd\x60"
                        "\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
                        "\x8a\xe0\x4d\xb0\x70\x38\xcc\x48"
                        "\x61\x7d\xee\xd6\x35\x49\xae\xb4"
                        "\xaf\x6b\xdd\xe6\x21\xc0\x60\xce"
                        "\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b",
        .ctext  = "\x87\x8B\x22\xA2\x08\xFA\xB7\xAD"
                        "\xB9\x65\xE0\x36\x60\x11\x3A\x40"
                        "\x76\xBA\x66\xDF\xAA\x53\xEC\x86"
                        "\x84\x13\x53\xC8\x9A\x26\x54\xCD"
                        "\x29\x96\xEC\x30\x84\x0B\xAC\x1F"
                        "\xA1\xC4\x3B\xB8\x86\x82\xF1\x78"
                        "\xDB\xBB\x95\xE7\x52\x26\x57\x43"
                        "\x2B\xD0\x4A\xC8\xB9\xCB\x8D\xDC",
        .len    = 64,
    },
};

void des_test_one(char *alg_name, ske_testvec_t *test_vec)
{
    u8 *src, *dst, *key, *iv;

    src = kmalloc(1024 * 3, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    memset(src, 0, 1024 * 3);
    dst = src + 1024;
    key = dst + 1024;
    iv = key + 512;

    // test enc
    memcpy(src, test_vec->ptext, test_vec->len);
    memcpy(key, test_vec->key, test_vec->klen);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }

    // test one segment
    skcipher_test(alg_name, 1, key, test_vec->klen, iv, src, test_vec->len,
                  dst);
    if (memcmp(dst, test_vec->ctext, test_vec->len)) {
        pr_err("encrypt %s err!\n", alg_name);
        goto out;
    }

    // test step
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }
    memset(dst, 0, test_vec->klen);
    skcipher_step_test(alg_name, 1, key, test_vec->klen, iv, src, test_vec->len,
                       dst);
    if (memcmp(dst, test_vec->ctext, test_vec->len)) {
        pr_err("encrypt step %s err!\n", alg_name);
        goto out;
    }

    // test dec
    memcpy(src, test_vec->ctext, test_vec->len);
    memcpy(key, test_vec->key, test_vec->klen);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }
    skcipher_test(alg_name, 0, key, test_vec->klen, iv, src, test_vec->len,
                  dst);
    if (memcmp(dst, test_vec->ptext, test_vec->len)) {
        pr_err("decrypt %s err!\n", alg_name);
        goto out;
    }

    // test dec step
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }
    skcipher_step_test(alg_name, 0, key, test_vec->klen, iv, src, test_vec->len,
                       dst);
    if (memcmp(dst, test_vec->ptext, test_vec->len)) {
        pr_err("decrypt %s err!\n", alg_name);
        goto out;
    }

    pr_info("Test %s klen %d successed!\n", alg_name, test_vec->klen);
out:
    kfree(src);
}

void des_test(char *mode, u32 klen, u32 slen, u32 elen)
{
    char alg_name[64] = {0};
    u32 alg_mode, i;
    u32 test = 0;
    ske_testvec_t *test_vec;

    sce_get_alg_name(alg_name, "des", mode);
    alg_mode = get_alg_mode(mode);
    if (alg_mode == 0xff) {
        pr_err("Invalid mode %s", mode);
        return;
    }

    pr_info("get mode %s, klen %d slen %d elen %d alg_mode %d", mode, klen,
            slen, elen, alg_mode);

    // test specified vec
    for (i = 0; i < sizeof(des_tv_template) / sizeof(des_tv_template[0]); i++) {
        test_vec = &des_tv_template[i];
        if (test_vec->mode == alg_mode && test_vec->alg == SCE_DES) {
            des_test_one(alg_name, test_vec);
            test = 1;
        }
    }

    if (0 == test) {
        pr_err("Missing test vector for alg %s klen %d", alg_name, klen);
    }
}

void tdes_test(char *mode, u32 klen, u32 slen, u32 elen)
{
    char alg_name[64] = {0};
    u32 alg_mode, i;
    u32 test = 0;
    ske_testvec_t *test_vec;

    sce_get_alg_name(alg_name, "des3_ede", mode);
    alg_mode = get_alg_mode(mode);
    if (alg_mode == 0xff) {
        pr_err("Invalid mode %s", mode);
        return;
    }

    pr_info("get mode %s, klen %d slen %d elen %d alg_mode %d", mode, klen,
            slen, elen, alg_mode);

    // test specified vec
    for (i = 0; i < sizeof(des_tv_template) / sizeof(des_tv_template[0]); i++) {
        test_vec = &des_tv_template[i];
        if (test_vec->mode == alg_mode && test_vec->alg == SCE_TDES) {
            des_test_one(alg_name, test_vec);
            test = 1;
        }
    }

    if (0 == test) {
        pr_err("Missing test vector for alg %s klen %d", alg_name, klen);
    }
}
