#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "aes_test.h"
#include "sm4_test.h"
#include "des_test.h"

enum SCE_SCA_ALGO_MODE {
    ECB = 0,
    CTR = 1,
    CBC = 2,
    OFB = 3,
    XTS = 4,
    CBC_MAC = 5,
    CMAC = 6,
    GCM = 7,
    CCM = 8,
    UNKNOWN = 0xff
};

typedef struct {
    char *mode_str;
    u32 mode;
} sca_mode_map_t;

sca_mode_map_t sca_mode_maps[] = {
    {.mode_str = "ecb", .mode = ECB},   {.mode_str = "ctr", .mode = CTR},
    {.mode_str = "cbc", .mode = CBC},   {.mode_str = "ofb", .mode = OFB},
    {.mode_str = "xts", .mode = XTS},   {.mode_str = "cbcmac", .mode = CBC_MAC},
    {.mode_str = "cmac", .mode = CMAC}, {.mode_str = "gcm", .mode = GCM},
    {.mode_str = "ccm", .mode = CCM},
};

u32 get_alg_mode(char *mode)
{
    u32 ret = UNKNOWN;
    u32 i;

    if (!mode) {
        return ret;
    }

    for (i = 0; i < sizeof(sca_mode_maps) / sizeof(sca_mode_maps[0]); i++) {
        if (strlen(sca_mode_maps[i].mode_str) == strlen(mode) &&
            0 == memcmp(mode, sca_mode_maps[i].mode_str, strlen(mode))) {
            ret = sca_mode_maps[i].mode;
            break;
        }
    }

    return ret;
}

void cipher_test(char *alg, char *mode, u32 klen, u32 slen, u32 elen)
{
    if (0 == memcmp(alg, "aes", 3)) {
        aes_test(mode, klen, slen, elen);
    } else if (0 == memcmp(alg, "sm4", 3)) {
        sm4_test(mode, klen, slen, elen);
    } else if (0 == memcmp(alg, "des", 3)) {
        des_test(mode, klen, slen, elen);
    } else if (0 == memcmp(alg, "tdes", 4)) {
        tdes_test(mode, klen, slen, elen);
    }
}
