#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "sca_test.h"
#include "../common/common.h"

ske_testvec_t sm4_tv_template[] = {
    {
        /* GB/T 32907-2016 Example 1. */
        .klen = 16,
        .len = 16,
        .ilen = 0,
        .alg = SCE_SM4,
        .mode = ECB,
        .key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
               "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
        .ptext = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
                 "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
        .ctext = "\x68\x1E\xDF\x34\xD2\x06\x96\x5E"
                 "\x86\xB3\xE9\x4F\x53\x6E\x42\x46",
    },
    {
        /* Last 10 iterations of GB/T 32907-2016 Example 2. */
        .klen = 16,
        .len = 160,
        .ilen = 0,
        .alg = SCE_SM4,
        .mode = ECB,
        .key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
               "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
        .ptext = "\x99\x4a\xc3\xe7\xc3\x57\x89\x6a"
                 "\x81\xfc\xa8\xe\x38\x3e\xef\x80"
                 "\xb1\x98\xf2\xde\x3f\x4b\xae\xd1"
                 "\xf0\xf1\x30\x4c\x1\x27\x5a\x8f"
                 "\x45\xe1\x39\xb7\xae\xff\x1f\x27"
                 "\xad\x57\x15\xab\x31\x5d\xc\xef"
                 "\x8c\xc8\x80\xbd\x11\x98\xf3\x7b"
                 "\xa2\xdd\x14\x20\xf9\xe8\xbb\x82"
                 "\xf7\x32\xca\x4b\xa8\xf7\xb3\x4d"
                 "\x27\xd1\xcd\xe6\xb6\x65\x5a\x23"
                 "\xc2\xf3\x54\x84\x53\xe3\xb9\x20"
                 "\xa5\x37\x0\xbe\xe7\x7b\x48\xfb"
                 "\x21\x3d\x9e\x48\x1d\x9e\xf5\xbf"
                 "\x77\xd5\xb4\x4a\x53\x71\x94\x7a"
                 "\x88\xa6\x6e\x6\x93\xca\x43\xa5"
                 "\xc4\xf6\xcd\x53\x4b\x7b\x8e\xfe"
                 "\xb4\x28\x7c\x42\x29\x32\x5d\x88"
                 "\xed\xce\x0\x19\xe\x16\x2\x6e"
                 "\x87\xff\x2c\xac\xe8\xe7\xe9\xbf"
                 "\x31\x51\xec\x47\xc3\x51\x83\xc1",
        .ctext = "\xb1\x98\xf2\xde\x3f\x4b\xae\xd1"
                 "\xf0\xf1\x30\x4c\x1\x27\x5a\x8f"
                 "\x45\xe1\x39\xb7\xae\xff\x1f\x27"
                 "\xad\x57\x15\xab\x31\x5d\xc\xef"
                 "\x8c\xc8\x80\xbd\x11\x98\xf3\x7b"
                 "\xa2\xdd\x14\x20\xf9\xe8\xbb\x82"
                 "\xf7\x32\xca\x4b\xa8\xf7\xb3\x4d"
                 "\x27\xd1\xcd\xe6\xb6\x65\x5a\x23"
                 "\xc2\xf3\x54\x84\x53\xe3\xb9\x20"
                 "\xa5\x37\x0\xbe\xe7\x7b\x48\xfb"
                 "\x21\x3d\x9e\x48\x1d\x9e\xf5\xbf"
                 "\x77\xd5\xb4\x4a\x53\x71\x94\x7a"
                 "\x88\xa6\x6e\x6\x93\xca\x43\xa5"
                 "\xc4\xf6\xcd\x53\x4b\x7b\x8e\xfe"
                 "\xb4\x28\x7c\x42\x29\x32\x5d\x88"
                 "\xed\xce\x0\x19\xe\x16\x2\x6e"
                 "\x87\xff\x2c\xac\xe8\xe7\xe9\xbf"
                 "\x31\x51\xec\x47\xc3\x51\x83\xc1"
                 "\x59\x52\x98\xc7\xc6\xfd\x27\x1f"
                 "\x4\x2\xf8\x4\xc3\x3d\x3f\x66",
    },
    {
        /* A.2.1.1 SM4-ECB Example 1 */
        .klen = 16,
        .len = 32,
        .ilen = 0,
        .alg = SCE_SM4,
        .mode = ECB,
        .key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
               "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
        .ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
                 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
                 "\xee\xee\xee\xee\xff\xff\xff\xff"
                 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
        .ctext = "\x5e\xc8\x14\x3d\xe5\x09\xcf\xf7"
                 "\xb5\x17\x9f\x8f\x47\x4b\x86\x19"
                 "\x2f\x1d\x30\x5a\x7f\xb1\x7d\xf9"
                 "\x85\xf8\x1c\x84\x82\x19\x23\x04",
    },
    {
        /* A.2.1.2 SM4-ECB Example 2 */
        .klen = 16,
        .len = 32,
        .ilen = 0,
        .alg = SCE_SM4,
        .mode = ECB,
        .key = "\xFE\xDC\xBA\x98\x76\x54\x32\x10"
               "\x01\x23\x45\x67\x89\xAB\xCD\xEF",
        .ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
                 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
                 "\xee\xee\xee\xee\xff\xff\xff\xff"
                 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
        .ctext = "\xC5\x87\x68\x97\xE4\xA5\x9B\xBB"
                 "\xA7\x2A\x10\xC8\x38\x72\x24\x5B"
                 "\x12\xDD\x90\xBC\x2D\x20\x06\x92"
                 "\xB5\x29\xA4\x15\x5A\xC9\xE6\x00",
    },

    {
        /* A.2.2.1 SM4-CBC Example 1 */
        .klen = 16,
        .len = 32,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = CBC,
        .key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
               "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
        .ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
                 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
                 "\xee\xee\xee\xee\xff\xff\xff\xff"
                 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
        .ctext = "\x78\xEB\xB1\x1C\xC4\x0B\x0A\x48"
                 "\x31\x2A\xAE\xB2\x04\x02\x44\xCB"
                 "\x4C\xB7\x01\x69\x51\x90\x92\x26"
                 "\x97\x9B\x0D\x15\xDC\x6A\x8F\x6D",
    },
    {
        /* A.2.2.2 SM4-CBC Example 2 */
        .klen = 16,
        .len = 32,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = CBC,
        .key = "\xFE\xDC\xBA\x98\x76\x54\x32\x10"
               "\x01\x23\x45\x67\x89\xAB\xCD\xEF",
        .ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
                 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
                 "\xee\xee\xee\xee\xff\xff\xff\xff"
                 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
        .ctext = "\x0d\x3a\x6d\xdc\x2d\x21\xc6\x98"
                 "\x85\x72\x15\x58\x7b\x7b\xb5\x9a"
                 "\x91\xf2\xc1\x47\x91\x1a\x41\x44"
                 "\x66\x5e\x1f\xa1\xd4\x0b\xae\x38",
    },

    {
        /* A.2.5.1 SM4-CTR Example 1 */
        .klen = 16,
        .len = 64,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = CTR,
        .key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
               "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
        .ptext = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
                 "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb"
                 "\xcc\xcc\xcc\xcc\xcc\xcc\xcc\xcc"
                 "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                 "\xee\xee\xee\xee\xee\xee\xee\xee"
                 "\xff\xff\xff\xff\xff\xff\xff\xff"
                 "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
                 "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
        .ctext = "\xac\x32\x36\xcb\x97\x0c\xc2\x07"
                 "\x91\x36\x4c\x39\x5a\x13\x42\xd1"
                 "\xa3\xcb\xc1\x87\x8c\x6f\x30\xcd"
                 "\x07\x4c\xce\x38\x5c\xdd\x70\xc7"
                 "\xf2\x34\xbc\x0e\x24\xc1\x19\x80"
                 "\xfd\x12\x86\x31\x0c\xe3\x7b\x92"
                 "\x6e\x02\xfc\xd0\xfa\xa0\xba\xf3"
                 "\x8b\x29\x33\x85\x1d\x82\x45\x14",
    },
    {
        /* A.2.5.2 SM4-CTR Example 2 */
        .klen = 16,
        .len = 64,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = CTR,
        .key = "\xFE\xDC\xBA\x98\x76\x54\x32\x10"
               "\x01\x23\x45\x67\x89\xAB\xCD\xEF",
        .ptext = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
                 "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb"
                 "\xcc\xcc\xcc\xcc\xcc\xcc\xcc\xcc"
                 "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
                 "\xee\xee\xee\xee\xee\xee\xee\xee"
                 "\xff\xff\xff\xff\xff\xff\xff\xff"
                 "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
                 "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
        .ctext = "\x5d\xcc\xcd\x25\xb9\x5a\xb0\x74"
                 "\x17\xa0\x85\x12\xee\x16\x0e\x2f"
                 "\x8f\x66\x15\x21\xcb\xba\xb4\x4c"
                 "\xc8\x71\x38\x44\x5b\xc2\x9e\x5c"
                 "\x0a\xe0\x29\x72\x05\xd6\x27\x04"
                 "\x17\x3b\x21\x23\x9b\x88\x7f\x6c"
                 "\x8c\xb5\xb8\x00\x91\x7a\x24\x88"
                 "\x28\x4b\xde\x9e\x16\xea\x29\x06",
    },

    {
        /* From: draft-ribose-cfrg-SCE_SM4-02, paragraph 12.2.3 */
        .klen = 16,
        .len = 32,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = OFB,
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef"
               "\xfe\xdc\xba\x98\x76\x54\x32\x10",
        .iv = "\x01\x23\x45\x67\x89\xab\xcd\xef"
              "\xfe\xdc\xba\x98\x76\x54\x32\x10",
        .ptext = "\x01\x23\x45\x67\x89\xab\xcd\xef"
                 "\xfe\xdc\xba\x98\x76\x54\x32\x10"
                 "\x01\x23\x45\x67\x89\xab\xcd\xef"
                 "\xfe\xdc\xba\x98\x76\x54\x32\x10",
        .ctext = "\x69\x3d\x9a\x53\x5b\xad\x5b\xb1"
                 "\x78\x6f\x53\xd7\x25\x3a\x70\x56"
                 "\xf2\x07\x5d\x28\xb5\x23\x5f\x58"
                 "\xd5\x00\x27\xe4\x17\x7d\x2b\xce",
    },
    {
        /* From: draft-ribose-cfrg-sm4-09, appendix A.2.3, Example 1 */
        .klen = 16,
        .len = 32,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = OFB,
        .key = "\x01\x23\x45\x67\x89\xab\xcd\xef"
               "\xfe\xdc\xba\x98\x76\x54\x32\x10",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
        .ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
                 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
                 "\xee\xee\xee\xee\xff\xff\xff\xff"
                 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
        .ctext = "\xac\x32\x36\xcb\x86\x1d\xd3\x16"
                 "\xe6\x41\x3b\x4e\x3c\x75\x24\xb7"
                 "\x1d\x01\xac\xa2\x48\x7c\xa5\x82"
                 "\xcb\xf5\x46\x3e\x66\x98\x53\x9b",
    },
    {
        /* From: draft-ribose-cfrg-sm4-09, appendix A.2.3, Example 2 */
        .klen = 16,
        .len = 30,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = OFB,
        .key = "\xfe\xdc\xba\x98\x76\x54\x32\x10"
               "\x01\x23\x45\x67\x89\xab\xcd\xef",
        .iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
              "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
        .ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
                 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
                 "\xee\xee\xee\xee\xff\xff\xff\xff"
                 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
        .ctext = "\x5d\xcc\xcd\x25\xa8\x4b\xa1\x65"
                 "\x60\xd7\xf2\x65\x88\x70\x68\x49"
                 "\x33\xfa\x16\xbd\x5c\xd9\xc8\x56"
                 "\xca\xca\xa1\xe1\x01\x89\x7a\x97",
    },

    {
        .klen = 16,
        .len = 90,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = CTR,
        .key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
               "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
        .iv = "\x00\x00\x12\x34\x56\x78\x00\x00"
              "\x00\x00\xAB\xCD\x00\x00\x00\x02",
        .ptext = "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
                 "\xBB\xBB\xBB\xBB\xBB\xBB\xBB\xBB"
                 "\xCC\xCC\xCC\xCC\xCC\xCC\xCC\xCC"
                 "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
                 "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
                 "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"
                 "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
                 "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
                 "\xCC\xCC\xCC\xCC\xCC\xCC\xCC\xCC"
                 "\xBB\xBB\xBB\xBB\xBB\xBB\xBB\xBB"
                 "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
                 "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA",
        .ctext = "\x17\xf3\x99\xf0\x8c\x67\xd5\xee"
                 "\x19\xd0\xdc\x99\x69\xc4\xbb\x7d"
                 "\x5f\xd4\x6f\xd3\x75\x64\x89\x06"
                 "\x91\x57\xb2\x82\xbb\x20\x07\x35"
                 "\xd8\x27\x10\xca\x5c\x22\xf0\xcc"
                 "\xfa\x7c\xbf\x93\xd4\x96\xac\x15"
                 "\xa5\x68\x34\xcb\xcf\x98\xc3\x97"
                 "\xc3\x75\x3d\x51\xe6\x54\x4c\xfa"
                 "\xe2\x09\x61\x87\x79\x0d\x3c\xd2"
                 "\xa7\x51\x56\x3e\xe0\xc0\xea\x31"
                 "\xff\xab\xf9\xba\xc6\xe4\x90\xb6"
                 "\x5c\x66\x01\xee\x1c\x0f\x45\x5e",
    },
    { /* XTS-AES 1 */
        .klen = 32,
        .len = 32,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = XTS,
        .key    ="\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .iv     = "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ptext  ="\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ctext  ="\xd9\xb4\x21\xf7\x31\xc8\x94\xfd\xc3\x5b\x77\x29\x1f\xe4\xe3\xb0"
                        "\x2a\x1f\xb7\x66\x98\xd5\x9f\x0e\x51\x37\x6c\x4a\xda\x5b\xc7\x5d",
    },
    { /* XTS-AES 2 */
        .klen = 32,
        .len = 32,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = XTS,
        .key    ="\x11\x11\x11\x11\x11\x11\x11\x11"
                  "\x11\x11\x11\x11\x11\x11\x11\x11"
                  "\x22\x22\x22\x22\x22\x22\x22\x22"
                  "\x22\x22\x22\x22\x22\x22\x22\x22",
        .iv     = "\x33\x33\x33\x33\x33\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ptext  ="\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44",
        .ctext  = "\xa7\x4d\x72\x6c\x11\x19\x6a\x32\xbe\x04\xe0\x01\xff\x29\xd0\xc7"
                        "\x93\x2f\x9f\x3e\xc2\x9b\xfc\xb6\x4d\xd1\x7f\x63\xcb\xd3\xea\x31",
    },
    { /* XTS-AES 3 */
        .klen = 32,
        .len = 32,
        .ilen = 16,
        .alg = SCE_SM4,
        .mode = XTS,
        .key    ="\xff\xfe\xfd\xfc\xfb\xfa\xf9\xf8"
                  "\xf7\xf6\xf5\xf4\xf3\xf2\xf1\xf0"
                  "\x22\x22\x22\x22\x22\x22\x22\x22"
                  "\x22\x22\x22\x22\x22\x22\x22\x22",
        .iv     = "\x33\x33\x33\x33\x33\x00\x00\x00"
                  "\x00\x00\x00\x00\x00\x00\x00\x00",
        .ptext  ="\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44"
                  "\x44\x44\x44\x44\x44\x44\x44\x44",
        .ctext  =  "\x7f\x76\x08\x8e\xff\xad\xf7\x0c\x02\xea\x9f\x95\xda\x06\x28\xd3"
                                "\x51\xbf\xcb\x9e\xac\x05\x63\xbc\xf1\x7b\x71\x0d\xab\x0a\x98\x26",
    },
};

void sm4_test_one(char *alg_name, ske_testvec_t *test_vec)
{
    u8 *src, *dst, *key, *iv;

    src = kmalloc(1024 * 3, GFP_KERNEL);
    if (src == NULL) {
        pr_err("%s:kmalloc failed!\n", __func__);
        return;
    }

    memset(src, 0, 1024 * 3);
    dst = src + 1024;
    key = dst + 1024;
    iv = key + 512;

    // test enc
    memcpy(src, test_vec->ptext, test_vec->len);
    memcpy(key, test_vec->key, test_vec->klen);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }

    // test one segment
    skcipher_test(alg_name, 1, key, test_vec->klen, iv, src, test_vec->len,
                  dst);
    if (memcmp(dst, test_vec->ctext, test_vec->len)) {
        pr_err("encrypt %s err!\n", alg_name);
        goto out;
    }

    // test step
    if (0 != memcmp(alg_name, "xts(", 4)) {
    	if (test_vec->iv) {
    	    memcpy(iv, test_vec->iv, test_vec->ilen);
    	} else {
    	    iv = NULL;
    	}
    	memset(dst, 0, test_vec->klen);
    	skcipher_step_test(alg_name, 1, key, test_vec->klen, iv, src, test_vec->len,
    	                   dst);
    	if (memcmp(dst, test_vec->ctext, test_vec->len)) {
    	    pr_err("encrypt step %s err!\n", alg_name);
    	    goto out;
    	}
    }

    // test dec
    memcpy(src, test_vec->ctext, test_vec->len);
    memcpy(key, test_vec->key, test_vec->klen);
    if (test_vec->iv) {
        memcpy(iv, test_vec->iv, test_vec->ilen);
    } else {
        iv = NULL;
    }
    skcipher_test(alg_name, 0, key, test_vec->klen, iv, src, test_vec->len,
                  dst);
    if (memcmp(dst, test_vec->ptext, test_vec->len)) {
        pr_err("decrypt %s err!\n", alg_name);
        goto out;
    }

    // test dec step
    if (0 != memcmp(alg_name, "xts(", 4)) {
    	if (test_vec->iv) {
    	    memcpy(iv, test_vec->iv, test_vec->ilen);
    	} else {
    	    iv = NULL;
    	}
    	skcipher_step_test(alg_name, 0, key, test_vec->klen, iv, src, test_vec->len,
    	                   dst);
    	if (memcmp(dst, test_vec->ptext, test_vec->len)) {
    	    pr_err("decrypt %s err!\n", alg_name);
    	    goto out;
    	}
    }

    pr_info("Test %s klen %d successed!\n", alg_name, test_vec->klen);
out:
    kfree(src);
}

void sm4_test(char *mode, u32 klen, u32 slen, u32 elen)
{
    char alg_name[64] = {0};
    u32 alg_mode, i;
    u32 test = 0;
    ske_testvec_t *test_vec;

    sce_get_alg_name(alg_name, "sm4", mode);
    alg_mode = get_alg_mode(mode);
    if (alg_mode == 0xff) {
        pr_err("Invalid mode %s", mode);
        return;
    }

    pr_info("get mode %s, klen %d slen %d elen %d", mode, klen, slen, elen);

    // test specified vec
    for (i = 0; i < sizeof(sm4_tv_template) / sizeof(sm4_tv_template[0]); i++) {
        test_vec = &sm4_tv_template[i];
        if (test_vec->mode == alg_mode) {
            sm4_test_one(alg_name, test_vec);
            test = 1;
        }
    }

    if (0 == test) {
        pr_err("Missing test vector for alg %s klen %d", alg_name, klen);
    }
}
