/*
 * File Name:test.c - Phytium driver test
 *
 * Copyright (C) 2020 Phytium Technology Co.,Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */

#include <linux/module.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/mutex.h>
#include <linux/acpi.h>
#include <crypto/internal/hash.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/aead.h>
#include <crypto/internal/skcipher.h>
#include <crypto/internal/akcipher.h>
#include <crypto/internal/aead.h>
#include <linux/ioport.h>
#include <linux/io.h>

#include "sca/sca_test.h"
#include "hash/hash_test.h"
#include "common/common.h"
#include "trng/trng_test.h"
#include "sm2/sm2_test.h"

char *alg = NULL;
char *mode = NULL;

u32 klen = 16;
u32 slen = 1024;
u32 elen = 1024;
u32 help = 0;
u32 secs = 5;
u32 speed = 0;

module_param(alg, charp, 0);
module_param(mode, charp, 0);
module_param(secs, uint, 0);
module_param(klen, uint, 0);
module_param(slen, uint, 0);
module_param(elen, uint, 0);
module_param(help, uint, 0);
module_param(speed, uint, 0);

MODULE_PARM_DESC(
    alg, "Specify algorithm name to test, valid alg:aes, des, tdes, sm4,"
         "sha1, sha224, sha256, sha384, sm3, md5, trng");

MODULE_PARM_DESC(secs, "Specify duration to test speed, default 5 s");
MODULE_PARM_DESC(
    speed, "Enabled to test speed, default disabled, only valid for trng");

MODULE_PARM_DESC(mode, "Specify algorithm mode to test, valid mode:ecb, cbc, "
                       "ctr, ofb, xts, cbcmac, cmac, ccm, gcm (for cipher);"
                       "hash, hmac (for digest)");

MODULE_PARM_DESC(klen, "Key length, default 16");
MODULE_PARM_DESC(slen, "Start of range length, default 1024");
MODULE_PARM_DESC(elen, "End of range length, default 1024");

MODULE_PARM_DESC(help, "Print help messages");

void print_help(void)
{
    pr_info("------BAD PARAM!------\n");
    pr_info("alg:   aes, des, tdes, sm4, \n");
    pr_info("       sha1, sha224, sha256, sha384, sm3, md5, trng\n");
    pr_info("secs:  Specify duration to test speed, default 5 s\n");
    pr_info("speed: Enabled to test speed, default disabled, only valid for "
            "trng\n");
    pr_info("mode:  ecb, cbc, ctr, ofb (for "
            "cipher),rnp sm4 (for trng)\n");
    pr_info("       hash, hmac (for digest)\n");
    pr_info("klen:  key length, default 16\n");
    pr_info("slen:  start of range length, default 1024\n");
    pr_info("elen:  end of range length, default 1024\n");
    pr_info("help:  print this message, default 0\n");
    pr_info("------BAD PARAM!------\n");
}

static int sce_get_mode(u32 *alg_mode)
{
    static char *cipher_algs[] = {"aes", "des", "tdes", "sm4"};

    static char *hash_algs[] = {"sha1",   "sha224", "sha256", "sha384",
                                "sha512", "sm3",    "md5"};

    u32 i;

    for (i = 0; i < sizeof(cipher_algs) / sizeof(cipher_algs[0]); i++) {
        if (strlen(cipher_algs[i]) == strlen(alg) &&
            memcmp(cipher_algs[i], alg, strlen(alg)) == 0) {
            *alg_mode = TEST_OP_MODE_CIPHER; // cipher
/*
            if (0 == memcmp(mode, "cbcmac", strlen("cbcmac")) ||
                0 == memcmp(mode, "cmac", strlen("cmac"))) {
                *alg_mode = TEST_OP_MODE_HASH; // cbcmac / cmac
            }

            if (0 == memcmp(mode, "ccm", strlen("ccm")) ||
                0 == memcmp(mode, "gcm", strlen("gcm"))) {
                *alg_mode = TEST_OP_MODE_AEAD; // cbcmac / cmac
            }
*/
            return 0;
        }
    }

    for (i = 0; i < sizeof(hash_algs) / sizeof(hash_algs[0]); i++) {
        if (strlen(hash_algs[i]) == strlen(alg) &&
            memcmp(hash_algs[i], alg, strlen(alg)) == 0) {
            *alg_mode = TEST_OP_MODE_HASH; // cipher
            if (0 == memcmp(mode, "hmac", strlen("hmac"))) {
                *alg_mode = TEST_OP_MODE_HMAC;
            }
            return 0;
        }
    }

    return -1;
}

static int sce_test_trng(void)
{
    int trng_type = 0;
    if (mode == NULL || strlen(mode) != 3) {
	pr_err("unsupport trng mode!\n");
        return -1;
    }
    if (0 == memcmp(mode,"rnp",strlen("rnp"))) {
        trng_type = 0;
    } else if(0 == memcmp(mode,"sm4",strlen("sm4"))){
        trng_type = 1;
    } else {
        pr_err("unsupport trng mode!\n");
        return -1;
    }

    if (speed) {
        trng_speed(secs, trng_type);
    } else {
        trng_test(slen, elen, trng_type);
    }

    return 0;
}

static int sce_test_cipher(void)
{
    u32 op_mode;

    if (!mode) {
        mode = "cbc";
    }

    sce_get_mode(&op_mode);
    switch (op_mode) {
    case TEST_OP_MODE_CIPHER:
        cipher_test(alg, mode, klen, slen, elen);
        break;
/*
    case TEST_OP_MODE_AEAD:
	aead_check_test(alg, mode, klen, slen, elen);
	break;
*/
    case TEST_OP_MODE_HASH:
        hash_test(alg, mode, klen, slen, elen);
        break;

    case TEST_OP_MODE_HMAC:
        hmac_test(alg, mode, klen, slen, elen);
        break;

    default:
        return -1;
    }

    return 0;
}

static int sce_test_hash(void)
{
    u32 op_mode;

    if (!mode) {
        mode = "hash";
    }

    sce_get_mode(&op_mode);

    switch (op_mode) {
    case TEST_OP_MODE_HASH:
        hash_test(alg, mode, klen, slen, elen);
        break;

    case TEST_OP_MODE_HMAC:
        hmac_test(alg, mode, klen, slen, elen);
        break;

    default:
        return -1;
    }

    return 0;
}

static int sce_test_sm2(void)
{
    return sm2_test();
}
typedef int (*sce_test_alg_fn)(void);
typedef struct {
    char *alg;
    sce_test_alg_fn fn;
} sce_test_alg_map_t;

static sce_test_alg_map_t global_test_alg_maps[] = {
    {.alg = "aes", .fn = sce_test_cipher},
    {.alg = "des", .fn = sce_test_cipher},
    {.alg = "tdes", .fn = sce_test_cipher},
    {.alg = "sm4", .fn = sce_test_cipher},
    {.alg = "sha1", .fn = sce_test_hash},
    {.alg = "sha224", .fn = sce_test_hash},
    {.alg = "sha256", .fn = sce_test_hash},
    {.alg = "sha384", .fn = sce_test_hash},
    {.alg = "sha512", .fn = sce_test_hash},
    {.alg = "sm3", .fn = sce_test_hash},
    {.alg = "md5", .fn = sce_test_hash},
    {.alg = "sm2", .fn = sce_test_sm2},
    {.alg = "trng", .fn = sce_test_trng}};

static int sce_test_init(void)
{
    u32 i;

    if (alg == NULL) {
        print_help();
        return -1;
    }

    if (help) {
        print_help();
        return -1;
    }

    for (i = 0;
         i < sizeof(global_test_alg_maps) / sizeof(global_test_alg_maps[0]);
         i++) {
        if (strlen(alg) != strlen(global_test_alg_maps[i].alg)) {
            continue;
        }

        if (0 == memcmp(alg, global_test_alg_maps[i].alg, strlen(alg))) {
            if (global_test_alg_maps[i].fn()) { // invalid param
                print_help();
            }
            return -1;
        }
    }

    print_help();

    return -1;
}

static void sce_test_exit(void)
{
}

module_init(sce_test_init);
module_exit(sce_test_exit);
MODULE_DESCRIPTION("PHYTIUM SCE KERNEL CRYPTO TEST");
MODULE_LICENSE("GPL");
