/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __SM3_PHYTIUM_H__
#define __SM3_PHYTIUM_H__

/*SM3 digest and block length*/
#define SM3_DIGEST_WORD_LEN (8)
#define SM3_DIGEST_BYTE_LEN (SM3_DIGEST_WORD_LEN << 2)
#define SM3_BLOCK_WORD_LEN (16)
#define SM3_BLOCK_BYTE_LEN (SM3_BLOCK_WORD_LEN << 2)

#define HMAC_IPAD_VALUE 0x36
#define HMAC_OPAD_VALUE 0x5c

extern struct phytium_scto_dev scto;

/*some register offset*/
#define HASH_LAST_OFFSET (24)

/*SM3 return code*/
enum SM3_RET_CODE {
	SM3_SUCCESS = 0,
	SM3_BUFFER_NULL,
	SM3_INPUT_INVALID,
	SM3_LEN_OVERFLOW,
};

/*HASH register struct*/
typedef struct {
	u32 hash_ctrl;
	u32 hash_cfg;
	u32 hash_sr_1;
	u32 hash_sr_2;
	u32 rev_1[4];
	u32 hash_pcr_len[2];
	u32 rev_2[2];
	u32 hash_out[8];
	u32 rev_3[8];
	u32 hash_in[8];
	u32 rev_4[8];
	u32 hash_version;
	u32 rev_5[19];
	u32 hash_m_din[16];
} hash_reg_t;

struct sm3_context {
	u32 total[2]; /* number of bytes processed */
	u32 state[8]; /* intermediate digest state */
	u8 buffer[64] ____cacheline_aligned; /* data block being processed */
	u8 ipad[64];
	u8 opad[64];
};

struct sm3_tfm_context {
	u8 ipad[64];
	u8 opad[64];
	u32 mode; /* 0: hash , 1 : hmac */
	u32 *v_dma_buf;
	spinlock_t buf_lock;
	dma_addr_t dma_paddr;
	u8 dma_buf[PHYTIUM_SM3_DMA_BUF_SIZE] ____cacheline_aligned;
};

typedef struct {
	struct sm3_context sm3_ctx;
	struct sm3_tfm_context *tfm_ctx;
} phytium_sm3_context;

struct hmac_sm3_tfm_context {
	struct crypto_shash *hash;
};

/*API*/
int phytium_scto_register_sm3(void);

void phytium_scto_unregister_sm3(void);

#ifdef SCTO_ENABLED_SM2

void _software_sm3_init(struct sm3_context *ctx);

void _process_software_sm3(struct sm3_context *ctx, const u8 data[64]);

void _software_sm3_update(struct sm3_context *ctx, const u8 *input,
			  size_t ilen);

void _software_sm3_final(struct sm3_context *ctx, u8 output[32]);

#endif

#endif
