// SPDX-License-Identifier: GPL-2.0-or-later
#include <linux/delay.h>
#include <crypto/hash.h>
#include <linux/crypto.h>
#include <linux/delay.h>
#include <linux/kthread.h>
#include <crypto/internal/rng.h>
#include <linux/version.h>
#include "smx_common.h"
#include "phytium_scto.h"

#define TRNG_POOL_SIZE 32

static int trng_init(struct crypto_tfm *tfm)
{
	return 0;
}

static void trng_cleanup(struct crypto_tfm *tfm)
{
}

static int trng_reset(struct crypto_rng *tfm, const u8 *seed, unsigned int slen)
{
	return 0;
}

static u32 wait_trng_ready(void)
{
	u32 count = 0;

	//wait till done
	while (scto.trng_reg->fifo_sr & 0x1000000) {
		if (unlikely(count++ > 1024))
			schedule();

		if ((scto.trng_reg->sr & 1) || (count > 65536)) {
			scto.trng_reg->cr = 0x1E;
			scto.trng_reg->sr = 7; //clear flag
			scto.trng_reg->rtcr = 0;
			dsb(sy);
			scto.trng_reg->cr = 0x1F;
			dsb(sy);

			pr_warn("trng timeout!\n");
			break;
		}
	}

	// counts of random in FIFO
	return ((scto.trng_reg->fifo_sr & 0xFC0000) >> 16);
}

static int trng_read(char *a, size_t byteLen)
{
	u32 rng_data, rng_data_len;
	size_t count = 0;
	unsigned long flags;

	while (byteLen > 0) {
		spin_lock_irqsave(&scto.rng_lock, flags);
		rng_data_len = wait_trng_ready();
		while (rng_data_len) {
			rng_data = scto.trng_reg->dr;
			if (byteLen >= 4) {
				memcpy(a, &rng_data, 4);

				a += 4;
				byteLen -= 4;
				rng_data_len -= 4;
				count += 4;
			} else {
				memcpy(a, &rng_data, byteLen);
				count += byteLen;
				byteLen = 0;
				break;
			}
		}
		spin_unlock_irqrestore(&scto.rng_lock, flags);

		cond_resched();
	}

	return count;
}

static int trng_generate(struct crypto_rng *tfm, const u8 *src,
			 unsigned int slen, u8 *rdata, unsigned int dlen)
{
	int count;

	if (!rdata) {
		pr_err("Wrong rdata value.");
		return -1;
	}

	count = trng_read(rdata, dlen);

	return (count == dlen) ? 0 : -1;
}

static struct rng_alg scto_trng_alg = { .generate = trng_generate,
					.seed = trng_reset,
					.seedsize = 0,
					.base = {
						.cra_name = "trng",
						.cra_driver_name =
							"trng-phytium",
						.cra_priority = 300,
						.cra_ctxsize = 0,
						.cra_module = THIS_MODULE,
						.cra_init = trng_init,
						.cra_exit = trng_cleanup,
					} };

int scto_trng_alg_register(void)
{
	int ret = 0;

	ret = crypto_register_rng(&scto_trng_alg);
	if (ret)
		pr_err("%s:crypto_register_rng failed!ret:%d\n", __func__, ret);

	return ret;
}

void scto_trng_alg_unregister(void)
{
	crypto_unregister_rng(&scto_trng_alg);
}
