#define _GNU_SOURCE
#include <sched.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <signal.h>
#include <dsce/sce.h>
#include <dsce/sce_log.h>

#include "test.h"
#include "ske.h"

__attribute__((always_inline)) inline static void dmb(void)
{
	__asm__ volatile("dmb sy" ::: "memory");
}
gcm_testvec_t gcm_tv_template[] = {
	{
		/* From https://datatracker.ietf.org/doc/html/rfc8998#appendix-A.1 */
		.klen = 16,
		.plen = 0,
		.clen = 16,
		.alen = 0,
		.ilen = 12,
		.alg = SCE_SM4,
		.mode = GCM,
		.key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
		       "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.iv = "\x00\x00\x12\x34\x56\x78\x00\x00"
		      "\x00\x00\xAB\xCD",
		.ctext =
			"\x54\xf1\x57\xaf\x32\x74\x4b\xb8\x3b\xbe\x8a\xa6\xf1\x57\x8b\x71",
	},

	{
		/* From https://datatracker.ietf.org/doc/html/rfc8998#appendix-A.1 */
		.klen = 16,
		.plen = 64,
		.clen = 80,
		.alen = 20,
		.ilen = 12,
		.alg = SCE_SM4,
		.mode = GCM,
		.key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
		       "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.iv = "\x00\x00\x12\x34\x56\x78\x00\x00"
		      "\x00\x00\xAB\xCD",
		.ptext = "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
			 "\xBB\xBB\xBB\xBB\xBB\xBB\xBB\xBB"
			 "\xCC\xCC\xCC\xCC\xCC\xCC\xCC\xCC"
			 "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
			 "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
			 "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"
			 "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
			 "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA",
		.assoc = "\xFE\xED\xFA\xCE\xDE\xAD\xBE\xEF"
			 "\xFE\xED\xFA\xCE\xDE\xAD\xBE\xEF"
			 "\xAB\xAD\xDA\xD2",
		.ctext = "\x17\xF3\x99\xF0\x8C\x67\xD5\xEE"
			 "\x19\xD0\xDC\x99\x69\xC4\xBB\x7D"
			 "\x5F\xD4\x6F\xD3\x75\x64\x89\x06"
			 "\x91\x57\xB2\x82\xBB\x20\x07\x35"
			 "\xD8\x27\x10\xCA\x5C\x22\xF0\xCC"
			 "\xFA\x7C\xBF\x93\xD4\x96\xAC\x15"
			 "\xA5\x68\x34\xCB\xCF\x98\xC3\x97"
			 "\xB4\x02\x4A\x26\x91\x23\x3B\x8D"
			 "\x83\xDE\x35\x41\xE4\xC2\xB5\x81"
			 "\x77\xE0\x65\xA9\xBF\x7B\x62\xEC",
	},
	{
		.alg = SCE_AES_128,
		.mode = GCM,
		.ilen = 12,
		.key = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08",
		.klen = 16,
		.iv = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
		      "\xde\xca\xf8\x88",
		.ptext = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
			 "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
			 "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
			 "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
			 "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
			 "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
			 "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
			 "\xba\x63\x7b\x39\x1a\xaf\xd2\x55",
		.plen = 64,
		.ctext = "\x42\x83\x1e\xc2\x21\x77\x74\x24"
			 "\x4b\x72\x21\xb7\x84\xd0\xd4\x9c"
			 "\xe3\xaa\x21\x2f\x2c\x02\xa4\xe0"
			 "\x35\xc1\x7e\x23\x29\xac\xa1\x2e"
			 "\x21\xd5\x14\xb2\x54\x66\x93\x1c"
			 "\x7d\x8f\x6a\x5a\xac\x84\xaa\x05"
			 "\x1b\xa3\x0b\x39\x6a\x0a\xac\x97"
			 "\x3d\x58\xe0\x91\x47\x3f\x59\x85"
			 "\x4d\x5c\x2a\xf3\x27\xcd\x64\xa6"
			 "\x2c\xf3\x5a\xbd\x2b\xa6\xfa\xb4",
		.clen = 80,
	},
	{
		.alg = SCE_AES_128,
		.mode = GCM,
		.ilen = 12,
		.alen = 20,
		.klen = 16,
		.plen = 60,
		.clen = 76,
		.key = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08",
		.iv = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
		      "\xde\xca\xf8\x88",
		.ptext = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
			 "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
			 "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
			 "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
			 "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
			 "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
			 "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
			 "\xba\x63\x7b\x39",
		.assoc = "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
			 "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
			 "\xab\xad\xda\xd2",
		.ctext = "\x42\x83\x1e\xc2\x21\x77\x74\x24"
			 "\x4b\x72\x21\xb7\x84\xd0\xd4\x9c"
			 "\xe3\xaa\x21\x2f\x2c\x02\xa4\xe0"
			 "\x35\xc1\x7e\x23\x29\xac\xa1\x2e"
			 "\x21\xd5\x14\xb2\x54\x66\x93\x1c"
			 "\x7d\x8f\x6a\x5a\xac\x84\xaa\x05"
			 "\x1b\xa3\x0b\x39\x6a\x0a\xac\x97"
			 "\x3d\x58\xe0\x91"
			 "\x5b\xc9\x4f\xbc\x32\x21\xa5\xdb"
			 "\x94\xfa\xe9\x5a\xe7\x12\x1a\x47",
	},
	{
		.alg = SCE_AES_128,
		.mode = GCM,
		.ilen = 12,
		.key = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08",
		.klen = 16,
		.iv = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
		      "\xde\xca\xf8\x88",
		.ptext = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
			 "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
			 "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
			 "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
			 "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
			 "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
			 "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
			 "\xba\x63\x7b\x39",
		.plen = 60,
		.assoc = "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
			 "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
			 "\xab\xad\xda\xd2",
		.alen = 20,
		.ctext = "\x42\x83\x1e\xc2\x21\x77\x74\x24"
			 "\x4b\x72\x21\xb7\x84\xd0\xd4\x9c"
			 "\xe3\xaa\x21\x2f\x2c\x02\xa4\xe0"
			 "\x35\xc1\x7e\x23\x29\xac\xa1\x2e"
			 "\x21\xd5\x14\xb2\x54\x66\x93\x1c"
			 "\x7d\x8f\x6a\x5a\xac\x84\xaa\x05"
			 "\x1b\xa3\x0b\x39\x6a\x0a\xac\x97"
			 "\x3d\x58\xe0\x91"
			 "\x5b\xc9\x4f\xbc\x32\x21\xa5\xdb"
			 "\x94\xfa\xe9\x5a\xe7\x12\x1a\x47",
		.clen = 76,
	},
	{
		.alg = SCE_AES_192,
		.mode = GCM,
		.ilen = 12,
		.key = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08"
		       "\xfe\xff\xe9\x92\x86\x65\x73\x1c",
		.klen = 24,
		.iv = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
		      "\xde\xca\xf8\x88",
		.ptext = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
			 "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
			 "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
			 "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
			 "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
			 "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
			 "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
			 "\xba\x63\x7b\x39\x1a\xaf\xd2\x55",
		.plen = 64,
		.ctext = "\x39\x80\xca\x0b\x3c\x00\xe8\x41"
			 "\xeb\x06\xfa\xc4\x87\x2a\x27\x57"
			 "\x85\x9e\x1c\xea\xa6\xef\xd9\x84"
			 "\x62\x85\x93\xb4\x0c\xa1\xe1\x9c"
			 "\x7d\x77\x3d\x00\xc1\x44\xc5\x25"
			 "\xac\x61\x9d\x18\xc8\x4a\x3f\x47"
			 "\x18\xe2\x44\x8b\x2f\xe3\x24\xd9"
			 "\xcc\xda\x27\x10\xac\xad\xe2\x56"
			 "\x99\x24\xa7\xc8\x58\x73\x36\xbf"
			 "\xb1\x18\x02\x4d\xb8\x67\x4a\x14",
		.clen = 80,
	},
	{
		.alg = SCE_AES_256,
		.mode = GCM,
		.ilen = 12,
		.key = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08"
		       "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08",
		.klen = 32,
		.iv = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
		      "\xde\xca\xf8\x88",
		.ptext = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
			 "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
			 "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
			 "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
			 "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
			 "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
			 "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
			 "\xba\x63\x7b\x39\x1a\xaf\xd2\x55",
		.plen = 64,
		.ctext = "\x52\x2d\xc1\xf0\x99\x56\x7d\x07"
			 "\xf4\x7f\x37\xa3\x2a\x84\x42\x7d"
			 "\x64\x3a\x8c\xdc\xbf\xe5\xc0\xc9"
			 "\x75\x98\xa2\xbd\x25\x55\xd1\xaa"
			 "\x8c\xb0\x8e\x48\x59\x0d\xbb\x3d"
			 "\xa7\xb0\x8b\x10\x56\x82\x88\x38"
			 "\xc5\xf6\x1e\x63\x93\xba\x7a\x0a"
			 "\xbc\xc9\xf6\x62\x89\x80\x15\xad"
			 "\xb0\x94\xda\xc5\xd9\x34\x71\xbd"
			 "\xec\x1a\x50\x22\x70\xe3\xcc\x6c",
		.clen = 80,
	},
	{
		.alg = SCE_AES_256,
		.mode = GCM,
		.ilen = 12,
		.key = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08"
		       "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08",
		.klen = 32,
		.iv = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
		      "\xde\xca\xf8\x88",
		.ptext = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
			 "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
			 "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
			 "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
			 "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
			 "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
			 "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
			 "\xba\x63\x7b\x39",
		.plen = 60,
		.assoc = "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
			 "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
			 "\xab\xad\xda\xd2",
		.alen = 20,
		.ctext = "\x52\x2d\xc1\xf0\x99\x56\x7d\x07"
			 "\xf4\x7f\x37\xa3\x2a\x84\x42\x7d"
			 "\x64\x3a\x8c\xdc\xbf\xe5\xc0\xc9"
			 "\x75\x98\xa2\xbd\x25\x55\xd1\xaa"
			 "\x8c\xb0\x8e\x48\x59\x0d\xbb\x3d"
			 "\xa7\xb0\x8b\x10\x56\x82\x88\x38"
			 "\xc5\xf6\x1e\x63\x93\xba\x7a\x0a"
			 "\xbc\xc9\xf6\x62"
			 "\x76\xfc\x6e\xce\x0f\x4e\x17\x68"
			 "\xcd\xdf\x88\x53\xbb\x2d\x55\x1b",
		.clen = 76,
	},
	{
		.alg = SCE_AES_192,
		.mode = GCM,
		.ilen = 12,
		.key = "\xfe\xff\xe9\x92\x86\x65\x73\x1c"
		       "\x6d\x6a\x8f\x94\x67\x30\x83\x08"
		       "\xfe\xff\xe9\x92\x86\x65\x73\x1c",
		.klen = 24,
		.iv = "\xca\xfe\xba\xbe\xfa\xce\xdb\xad"
		      "\xde\xca\xf8\x88",
		.ptext = "\xd9\x31\x32\x25\xf8\x84\x06\xe5"
			 "\xa5\x59\x09\xc5\xaf\xf5\x26\x9a"
			 "\x86\xa7\xa9\x53\x15\x34\xf7\xda"
			 "\x2e\x4c\x30\x3d\x8a\x31\x8a\x72"
			 "\x1c\x3c\x0c\x95\x95\x68\x09\x53"
			 "\x2f\xcf\x0e\x24\x49\xa6\xb5\x25"
			 "\xb1\x6a\xed\xf5\xaa\x0d\xe6\x57"
			 "\xba\x63\x7b\x39",
		.plen = 60,
		.assoc = "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
			 "\xfe\xed\xfa\xce\xde\xad\xbe\xef"
			 "\xab\xad\xda\xd2",
		.alen = 20,
		.ctext = "\x39\x80\xca\x0b\x3c\x00\xe8\x41"
			 "\xeb\x06\xfa\xc4\x87\x2a\x27\x57"
			 "\x85\x9e\x1c\xea\xa6\xef\xd9\x84"
			 "\x62\x85\x93\xb4\x0c\xa1\xe1\x9c"
			 "\x7d\x77\x3d\x00\xc1\x44\xc5\x25"
			 "\xac\x61\x9d\x18\xc8\x4a\x3f\x47"
			 "\x18\xe2\x44\x8b\x2f\xe3\x24\xd9"
			 "\xcc\xda\x27\x10"
			 "\x25\x19\x49\x8e\x80\xf1\x47\x8f"
			 "\x37\xba\x55\xbd\x6d\x27\x61\x8c",
		.clen = 76,
	},
	{
		.alg = SCE_AES_256,
		.mode = GCM,
		.ilen = 12,
		.key = "\x62\x35\xf8\x95\xfc\xa5\xeb\xf6"
		       "\x0e\x92\x12\x04\xd3\xa1\x3f\x2e"
		       "\x8b\x32\xcf\xe7\x44\xed\x13\x59"
		       "\x04\x38\x77\xb0\xb9\xad\xb4\x38",
		.klen = 32,
		.iv = "\x00\xff\xff\xff\xff\x00\x00\xff"
		      "\xff\xff\x00\xff",
		.ptext = "\x42\xc1\xcc\x08\x48\x6f\x41\x3f"
			 "\x2f\x11\x66\x8b\x2a\x16\xf0\xe0"
			 "\x58\x83\xf0\xc3\x70\x14\xc0\x5b"
			 "\x3f\xec\x1d\x25\x3c\x51\xd2\x03"
			 "\xcf\x59\x74\x1f\xb2\x85\xb4\x07"
			 "\xc6\x6a\x63\x39\x8a\x5b\xde\xcb"
			 "\xaf\x08\x44\xbd\x6f\x91\x15\xe1"
			 "\xf5\x7a\x6e\x18\xbd\xdd\x61\x50"
			 "\x59\xa9\x97\xab\xbb\x0e\x74\x5c"
			 "\x00\xa4\x43\x54\x04\x54\x9b\x3b"
			 "\x77\xec\xfd\x5c\xa6\xe8\x7b\x08"
			 "\xae\xe6\x10\x3f\x32\x65\xd1\xfc"
			 "\xa4\x1d\x2c\x31\xfb\x33\x7a\xb3"
			 "\x35\x23\xf4\x20\x41\xd4\xad\x82"
			 "\x8b\xa4\xad\x96\x1c\x20\x53\xbe"
			 "\x0e\xa6\xf4\xdc\x78\x49\x3e\x72"
			 "\xb1\xa9\xb5\x83\xcb\x08\x54\xb7"
			 "\xad\x49\x3a\xae\x98\xce\xa6\x66"
			 "\x10\x30\x90\x8c\x55\x83\xd7\x7c"
			 "\x8b\xe6\x53\xde\xd2\x6e\x18\x21"
			 "\x01\x52\xd1\x9f\x9d\xbb\x9c\x73"
			 "\x57\xcc\x89\x09\x75\x9b\x78\x70"
			 "\xed\x26\x97\x4d\xb4\xe4\x0c\xa5"
			 "\xfa\x70\x04\x70\xc6\x96\x1c\x7d"
			 "\x54\x41\x77\xa8\xe3\xb0\x7e\x96"
			 "\x82\xd9\xec\xa2\x87\x68\x55\xf9"
			 "\x8f\x9e\x73\x43\x47\x6a\x08\x36"
			 "\x93\x67\xa8\x2d\xde\xac\x41\xa9"
			 "\x5c\x4d\x73\x97\x0f\x70\x68\xfa"
			 "\x56\x4d\x00\xc2\x3b\x1f\xc8\xb9"
			 "\x78\x1f\x51\x07\xe3\x9a\x13\x4e"
			 "\xed\x2b\x2e\xa3\xf7\x44\xb2\xe7"
			 "\xab\x19\x37\xd9\xba\x76\x5e\xd2"
			 "\xf2\x53\x15\x17\x4c\x6b\x16\x9f"
			 "\x02\x66\x49\xca\x7c\x91\x05\xf2"
			 "\x45\x36\x1e\xf5\x77\xad\x1f\x46"
			 "\xa8\x13\xfb\x63\xb6\x08\x99\x63"
			 "\x82\xa2\xed\xb3\xac\xdf\x43\x19"
			 "\x45\xea\x78\x73\xd9\xb7\x39\x11"
			 "\xa3\x13\x7c\xf8\x3f\xf7\xad\x81"
			 "\x48\x2f\xa9\x5c\x5f\xa0\xf0\x79"
			 "\xa4\x47\x7d\x80\x20\x26\xfd\x63"
			 "\x0a\xc7\x7e\x6d\x75\x47\xff\x76"
			 "\x66\x2e\x8a\x6c\x81\x35\xaf\x0b"
			 "\x2e\x6a\x49\x60\xc1\x10\xe1\xe1"
			 "\x54\x03\xa4\x09\x0c\x37\x7a\x15"
			 "\x23\x27\x5b\x8b\x4b\xa5\x64\x97"
			 "\xae\x4a\x50\x73\x1f\x66\x1c\x5c"
			 "\x03\x25\x3c\x8d\x48\x58\x71\x34"
			 "\x0e\xec\x4e\x55\x1a\x03\x6a\xe5"
			 "\xb6\x19\x2b\x84\x2a\x20\xd1\xea"
			 "\x80\x6f\x96\x0e\x05\x62\xc7\x78"
			 "\x87\x79\x60\x38\x46\xb4\x25\x57"
			 "\x6e\x16\x63\xf8\xad\x6e\xd7\x42"
			 "\x69\xe1\x88\xef\x6e\xd5\xb4\x9a"
			 "\x3c\x78\x6c\x3b\xe5\xa0\x1d\x22"
			 "\x86\x5c\x74\x3a\xeb\x24\x26\xc7"
			 "\x09\xfc\x91\x96\x47\x87\x4f\x1a"
			 "\xd6\x6b\x2c\x18\x47\xc0\xb8\x24"
			 "\xa8\x5a\x4a\x9e\xcb\x03\xe7\x2a"
			 "\x09\xe6\x4d\x9c\x6d\x86\x60\xf5"
			 "\x2f\x48\x69\x37\x9f\xf2\xd2\xcb"
			 "\x0e\x5a\xdd\x6e\x8a\xfb\x6a\xfe"
			 "\x0b\x63\xde\x87\x42\x79\x8a\x68"
			 "\x51\x28\x9b\x7a\xeb\xaf\xb8\x2f"
			 "\x9d\xd1\xc7\x45\x90\x08\xc9\x83"
			 "\xe9\x83\x84\xcb\x28\x69\x09\x69"
			 "\xce\x99\x46\x00\x54\xcb\xd8\x38"
			 "\xf9\x53\x4a\xbf\x31\xce\x57\x15"
			 "\x33\xfa\x96\x04\x33\x42\xe3\xc0"
			 "\xb7\x54\x4a\x65\x7a\x7c\x02\xe6"
			 "\x19\x95\xd0\x0e\x82\x07\x63\xf9"
			 "\xe1\x2b\x2a\xfc\x55\x92\x52\xc9"
			 "\xb5\x9f\x23\x28\x60\xe7\x20\x51"
			 "\x10\xd3\xed\x6d\x9b\xab\xb8\xe2"
			 "\x5d\x9a\x34\xb3\xbe\x9c\x64\xcb"
			 "\x78\xc6\x91\x22\x40\x91\x80\xbe"
			 "\xd7\x78\x5c\x0e\x0a\xdc\x08\xe9"
			 "\x67\x10\xa4\x83\x98\x79\x23\xe7"
			 "\x92\xda\xa9\x22\x16\xb1\xe7\x78"
			 "\xa3\x1c\x6c\x8f\x35\x7c\x4d\x37"
			 "\x2f\x6e\x0b\x50\x5c\x34\xb9\xf9"
			 "\xe6\x3d\x91\x0d\x32\x95\xaa\x3d"
			 "\x48\x11\x06\xbb\x2d\xf2\x63\x88"
			 "\x3f\x73\x09\xe2\x45\x56\x31\x51"
			 "\xfa\x5e\x4e\x62\xf7\x90\xf9\xa9"
			 "\x7d\x7b\x1b\xb1\xc8\x26\x6e\x66"
			 "\xf6\x90\x9a\x7f\xf2\x57\xcc\x23"
			 "\x59\xfa\xfa\xaa\x44\x04\x01\xa7"
			 "\xa4\x78\xdb\x74\x3d\x8b\xb5",
		.plen = 719,
		.ctext = "\x84\x0b\xdb\xd5\xb7\xa8\xfe\x20"
			 "\xbb\xb1\x12\x7f\x41\xea\xb3\xc0"
			 "\xa2\xb4\x37\x19\x11\x58\xb6\x0b"
			 "\x4c\x1d\x38\x05\x54\xd1\x16\x73"
			 "\x8e\x1c\x20\x90\xa2\x9a\xb7\x74"
			 "\x47\xe6\xd8\xfc\x18\x3a\xb4\xea"
			 "\xd5\x16\x5a\x2c\x53\x01\x46\xb3"
			 "\x18\x33\x74\x6c\x50\xf2\xe8\xc0"
			 "\x73\xda\x60\x22\xeb\xe3\xe5\x9b"
			 "\x20\x93\x6c\x4b\x37\x99\xb8\x23"
			 "\x3b\x4e\xac\xe8\x5b\xe8\x0f\xb7"
			 "\xc3\x8f\xfb\x4a\x37\xd9\x39\x95"
			 "\x34\xf1\xdb\x8f\x71\xd9\xc7\x0b"
			 "\x02\xf1\x63\xfc\x9b\xfc\xc5\xab"
			 "\xb9\x14\x13\x21\xdf\xce\xaa\x88"
			 "\x44\x30\x1e\xce\x26\x01\x92\xf8"
			 "\x9f\x00\x4b\x0c\x4b\xf7\x5f\xe0"
			 "\x89\xca\x94\x66\x11\x21\x97\xca"
			 "\x3e\x83\x74\x2d\xdb\x4d\x11\xeb"
			 "\x97\xc2\x14\xff\x9e\x1e\xa0\x6b"
			 "\x08\xb4\x31\x2b\x85\xc6\x85\x6c"
			 "\x90\xec\x39\xc0\xec\xb3\xb5\x4e"
			 "\xf3\x9c\xe7\x83\x3a\x77\x0a\xf4"
			 "\x56\xfe\xce\x18\x33\x6d\x0b\x2d"
			 "\x33\xda\xc8\x05\x5c\xb4\x09\x2a"
			 "\xde\x6b\x52\x98\x01\xef\x36\x3d"
			 "\xbd\xf9\x8f\xa8\x3e\xaa\xcd\xd1"
			 "\x01\x2d\x42\x49\xc3\xb6\x84\xbb"
			 "\x48\x96\xe0\x90\x93\x6c\x48\x64"
			 "\xd4\xfa\x7f\x93\x2c\xa6\x21\xc8"
			 "\x7a\x23\x7b\xaa\x20\x56\x12\xae"
			 "\x16\x9d\x94\x0f\x54\xa1\xec\xca"
			 "\x51\x4e\xf2\x39\xf4\xf8\x5f\x04"
			 "\x5a\x0d\xbf\xf5\x83\xa1\x15\xe1"
			 "\xf5\x3c\xd8\x62\xa3\xed\x47\x89"
			 "\x85\x4c\xe5\xdb\xac\x9e\x17\x1d"
			 "\x0c\x09\xe3\x3e\x39\x5b\x4d\x74"
			 "\x0e\xf5\x34\xee\x70\x11\x4c\xfd"
			 "\xdb\x34\xb1\xb5\x10\x3f\x73\xb7"
			 "\xf5\xfa\xed\xb0\x1f\xa5\xcd\x3c"
			 "\x8d\x35\x83\xd4\x11\x44\x6e\x6c"
			 "\x5b\xe0\x0e\x69\xa5\x39\xe5\xbb"
			 "\xa9\x57\x24\x37\xe6\x1f\xdd\xcf"
			 "\x16\x2a\x13\xf9\x6a\x2d\x90\xa0"
			 "\x03\x60\x7a\xed\x69\xd5\x00\x8b"
			 "\x7e\x4f\xcb\xb9\xfa\x91\xb9\x37"
			 "\xc1\x26\xce\x90\x97\x22\x64\x64"
			 "\xc1\x72\x43\x1b\xf6\xac\xc1\x54"
			 "\x8a\x10\x9c\xdd\x8d\xd5\x8e\xb2"
			 "\xe4\x85\xda\xe0\x20\x5f\xf4\xb4"
			 "\x15\xb5\xa0\x8d\x12\x74\x49\x23"
			 "\x3a\xdf\x4a\xd3\xf0\x3b\x89\xeb"
			 "\xf8\xcc\x62\x7b\xfb\x93\x07\x41"
			 "\x61\x26\x94\x58\x70\xa6\x3c\xe4"
			 "\xff\x58\xc4\x13\x3d\xcb\x36\x6b"
			 "\x32\xe5\xb2\x6d\x03\x74\x6f\x76"
			 "\x93\x77\xde\x48\xc4\xfa\x30\x4a"
			 "\xda\x49\x80\x77\x0f\x1c\xbe\x11"
			 "\xc8\x48\xb1\xe5\xbb\xf2\x8a\xe1"
			 "\x96\x2f\x9f\xd1\x8e\x8a\x5c\xe2"
			 "\xf7\xd7\xd8\x54\xf3\x3f\xc4\x91"
			 "\xb8\xfb\x86\xdc\x46\x24\x91\x60"
			 "\x6c\x2f\xc9\x41\x37\x51\x49\x54"
			 "\x09\x81\x21\xf3\x03\x9f\x2b\xe3"
			 "\x1f\x39\x63\xaf\xf4\xd7\x53\x60"
			 "\xa7\xc7\x54\xf9\xee\xb1\xb1\x7d"
			 "\x75\x54\x65\x93\xfe\xb1\x68\x6b"
			 "\x57\x02\xf9\xbb\x0e\xf9\xf8\xbf"
			 "\x01\x12\x27\xb4\xfe\xe4\x79\x7a"
			 "\x40\x5b\x51\x4b\xdf\x38\xec\xb1"
			 "\x6a\x56\xff\x35\x4d\x42\x33\xaa"
			 "\x6f\x1b\xe4\xdc\xe0\xdb\x85\x35"
			 "\x62\x10\xd4\xec\xeb\xc5\x7e\x45"
			 "\x1c\x6f\x17\xca\x3b\x8e\x2d\x66"
			 "\x4f\x4b\x36\x56\xcd\x1b\x59\xaa"
			 "\xd2\x9b\x17\xb9\x58\xdf\x7b\x64"
			 "\x8a\xff\x3b\x9c\xa6\xb5\x48\x9e"
			 "\xaa\xe2\x5d\x09\x71\x32\x5f\xb6"
			 "\x29\xbe\xe7\xc7\x52\x7e\x91\x82"
			 "\x6b\x6d\x33\xe1\x34\x06\x36\x21"
			 "\x5e\xbe\x1e\x2f\x3e\xc1\xfb\xea"
			 "\x49\x2c\xb5\xca\xf7\xb0\x37\xea"
			 "\x1f\xed\x10\x04\xd9\x48\x0d\x1a"
			 "\x1c\xfb\xe7\x84\x0e\x83\x53\x74"
			 "\xc7\x65\xe2\x5c\xe5\xba\x73\x4c"
			 "\x0e\xe1\xb5\x11\x45\x61\x43\x46"
			 "\xaa\x25\x8f\xbd\x85\x08\xfa\x4c"
			 "\x15\xc1\xc0\xd8\xf5\xdc\x16\xbb"
			 "\x7b\x1d\xe3\x87\x57\xa7\x2a\x1d"
			 "\x38\x58\x9e\x8a\x43\xdc\x57"
			 "\xd1\x81\x7d\x2b\xe9\xff\x99\x3a"
			 "\x4b\x24\x52\x58\x55\xe1\x49\x14",
		.clen = 735,
	},
};
ccm_testvec_t ccm_tv_template[] = {
	{
		.alg = SCE_AES_128,
		.mode = CCM,
		.ilen = 16,
		.key = "\xab\x2f\x8a\x74\xb7\x1c\xd2\xb1"
		       "\xff\x80\x2e\x48\x7d\x82\xf8\xb9",
		.klen = 16,
		.iv = "\x03\xaf\x94\x87\x78\x35\x82\x81"
		      "\x7f\x88\x94\x68\x00\x00\x00\x00",
		.alen = 0,
		.ptext = "\x00",
		.plen = 0,
		.ctext = "\x41\x3c\xb8\x87\x73\xcb\xf3\xf3",
		.clen = 8,
	},

	{
		.klen = 16,
		.plen = 23,
		.clen = 31,
		.alen = 8,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CCM,
		.key = "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
		       "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf",
		.iv = "\x01\x00\x00\x00\x03\x02\x01\x00"
		      "\xa0\xa1\xa2\xa3\xa4\xa5\x00\x00",
		.assoc = "\x00\x01\x02\x03\x04\x05\x06\x07",
		.ptext = "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
			 "\x10\x11\x12\x13\x14\x15\x16\x17"
			 "\x18\x19\x1a\x1b\x1c\x1d\x1e",
		.ctext =
			"\x7B\xFF\x4A\x15\xF5\x73\xCE\x82\x6E\xC2\x31\x1D\xE2\x53\x02\xAC"
			"\xA4\x48\xF9\xE4\xF5\x1F\x81"
			"\x70\x18\xBC\xB6\x84\x01\xB8\xAE",
	},
	{
		.alg = SCE_AES_128,
		.mode = CCM,
		.ilen = 16,
		.key = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
		       "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
		.klen = 16,
		.iv = "\x01\x00\x33\x56\x8e\xf7\xb2\x63"
		      "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
		.assoc = "\x63\x01\x8f\x76\xdc\x8a\x1b\xcb",
		.alen = 8,
		.ptext = "\x90\x20\xea\x6f\x91\xbd\xd8\x5a"
			 "\xfa\x00\x39\xba\x4b\xaf\xf9\xbf"
			 "\xb7\x9c\x70\x28\x94\x9c\xd0\xec",
		.plen = 24,
		.ctext = "\x4c\xcb\x1e\x7c\xa9\x81\xbe\xfa"
			 "\xa0\x72\x6c\x55\xd3\x78\x06\x12"
			 "\x98\xc8\x5c\x92\x81\x4a\xbc\x33"
			 "\xc5\x2e\xe8\x1d\x7d\x77\xc0\x8a",
		.clen = 32,
	},
	{
		.klen = 16,
		.plen = 19,
		.clen = 29,
		.alen = 12,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CCM,
		.key = "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
		       "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf",
		.iv = "\x01\x00\x00\x00\x0c\x0b\x0a\x09"
		      "\xa0\xa1\xa2\xa3\xa4\xa5\x00\x00",
		.assoc = "\x00\x01\x02\x03\x04\x05\x06\x07"
			 "\x08\x09\x0a\x0b",
		.ptext = "\x0c\x0d\x0e\x0f\x10\x11\x12\x13"
			 "\x14\x15\x16\x17\x18\x19\x1a\x1b"
			 "\x1c\x1d\x1e",
		.ctext =
			"\x82\x55\x65\x01\xE0\xC6\x00\xD7\x2B\xEE\x4A\x8A\x88\xD3\x41\x2B"
			"\xF8\x38\x94"
			"\x38\xE9\x9A\xF2\x70\x67\x1C\x3F\xEE\x8F",
	},
	{
		.klen = 16,
		.plen = 24,
		.clen = 32,
		.alen = 8,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CCM,
		.key = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
		       "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
		.iv = "\x01\x00\x33\x56\x8e\xf7\xb2\x63"
		      "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
		.assoc = "\x63\x01\x8f\x76\xdc\x8a\x1b\xcb",
		.ptext = "\x90\x20\xea\x6f\x91\xbd\xd8\x5a"
			 "\xfa\x00\x39\xba\x4b\xaf\xf9\xbf"
			 "\xb7\x9c\x70\x28\x94\x9c\xd0\xec",
		.ctext =
			"\xDA\x5D\x76\x41\x96\x1A\x57\x73\x3D\x11\xB0\x51\xBA\xE6\x5A\x45"
			"\x6B\xB0\x83\x03\x10\xCC\x98\x0C"
			"\x52\x88\xF5\xA6\x8F\x0E\x54\xF0",
	},
	{
		.klen = 16,
		.plen = 21,
		.clen = 29,
		.alen = 12,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CCM,
		.key = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
		       "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
		.iv = "\x01\x00\xd5\x60\x91\x2d\x3f\x70"
		      "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
		.assoc = "\xcd\x90\x44\xd2\xb7\x1f\xdb\x81"
			 "\x20\xea\x60\xc0",
		.ptext = "\x64\x35\xac\xba\xfb\x11\xa8\x2e"
			 "\x2f\x07\x1d\x7c\xa4\xa5\xeb\xd9"
			 "\x3a\x80\x3b\xa8\x7f",
		.ctext =
			"\x5E\xA8\x04\xBE\x9D\x53\x26\x3E\x27\x49\x8E\x31\xA0\x44\xD2\x0F"
			"\x3F\x79\x4E\x8E\xE6"
			"\x99\x53\x24\x30\x05\xBE\x5C\x64",
	},

	{
		.alg = SCE_AES_128,
		.mode = CCM,
		.ilen = 16,
		.key = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
		       "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
		.klen = 16,
		.iv = "\x01\x00\xd5\x60\x91\x2d\x3f\x70"
		      "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
		.assoc = "\xcd\x90\x44\xd2\xb7\x1f\xdb\x81"
			 "\x20\xea\x60\xc0",
		.alen = 12,
		.ptext = "\x64\x35\xac\xba\xfb\x11\xa8\x2e"
			 "\x2f\x07\x1d\x7c\xa4\xa5\xeb\xd9"
			 "\x3a\x80\x3b\xa8\x7f",
		.plen = 21,
		.ctext = "\x00\x97\x69\xec\xab\xdf\x48\x62"
			 "\x55\x94\xc5\x92\x51\xe6\x03\x57"
			 "\x22\x67\x5e\x04\xc8\x47\x09\x9e"
			 "\x5a\xe0\x70\x45\x51",
		.clen = 29,
	},
	{
		.alg = SCE_AES_128,
		.mode = CCM,
		.ilen = 16,
		.key = "\xd7\x82\x8d\x13\xb2\xb0\xbd\xc3"
		       "\x25\xa7\x62\x36\xdf\x93\xcc\x6b",
		.klen = 16,
		.iv = "\x01\x00\x42\xff\xf8\xf1\x95\x1c"
		      "\x3c\x96\x96\x76\x6c\xfa\x00\x00",
		.assoc = "\xd8\x5b\xc7\xe6\x9f\x94\x4f\xb8",
		.alen = 8,
		.ptext = "\x8a\x19\xb9\x50\xbc\xf7\x1a\x01"
			 "\x8e\x5e\x67\x01\xc9\x17\x87\x65"
			 "\x98\x09\xd6\x7d\xbe\xdd\x18",
		.plen = 23,
		.ctext = "\xbc\x21\x8d\xaa\x94\x74\x27\xb6"
			 "\xdb\x38\x6a\x99\xac\x1a\xef\x23"
			 "\xad\xe0\xb5\x29\x39\xcb\x6a\x63"
			 "\x7c\xf9\xbe\xc2\x40\x88\x97\xc6"
			 "\xba",
		.clen = 33,
	},
	{
		/* This is taken from FIPS CAVS. */
		.alg = SCE_AES_128,
		.mode = CCM,
		.ilen = 16,
		.key = "\x83\xac\x54\x66\xc2\xeb\xe5\x05"
		       "\x2e\x01\xd1\xfc\x5d\x82\x66\x2e",
		.klen = 16,
		.iv = "\x03\x96\xac\x59\x30\x07\xa1\xe2\xa2\xc7\x55\x24\0\0\0\0",
		.alen = 0,
		.ptext = "\x19\xc8\x81\xf6\xe9\x86\xff\x93"
			 "\x0b\x78\x67\xe5\xbb\xb7\xfc\x6e"
			 "\x83\x77\xb3\xa6\x0c\x8c\x9f\x9c"
			 "\x35\x2e\xad\xe0\x62\xf9\x91\xa1",
		.plen = 32,
		.ctext = "\xab\x6f\xe1\x69\x1d\x19\x99\xa8"
			 "\x92\xa0\xc4\x6f\x7e\xe2\x8b\xb1"
			 "\x70\xbb\x8c\xa6\x4c\x6e\x97\x8a"
			 "\x57\x2b\xbe\x5d\x98\xa6\xb1\x32"
			 "\xda\x24\xea\xd9\xa1\x39\x98\xfd"
			 "\xa4\xbe\xd9\xf2\x1a\x6d\x22\xa8",
		.clen = 48,
	},
	{
		.alg = SCE_AES_128,
		.mode = CCM,
		.ilen = 16,
		.key = "\x1e\x2c\x7e\x01\x41\x9a\xef\xc0"
		       "\x0d\x58\x96\x6e\x5c\xa2\x4b\xd3",
		.klen = 16,
		.iv = "\x03\x4f\xa3\x19\xd3\x01\x5a\xd8"
		      "\x30\x60\x15\x56\x00\x00\x00\x00",
		.assoc = "\xda\xe6\x28\x9c\x45\x2d\xfd\x63"
			 "\x5e\xda\x4c\xb6\xe6\xfc\xf9\xb7"
			 "\x0c\x56\xcb\xe4\xe0\x05\x7a\xe1"
			 "\x0a\x63\x09\x78\xbc\x2c\x55\xde",
		.alen = 32,
		.ptext = "\x87\xa3\x36\xfd\x96\xb3\x93\x78"
			 "\xa9\x28\x63\xba\x12\xa3\x14\x85"
			 "\x57\x1e\x06\xc9\x7b\x21\xef\x76"
			 "\x7f\x38\x7e\x8e\x29\xa4\x3e\x7e",
		.plen = 32,
		.ctext = "\x8a\x1e\x11\xf0\x02\x6b\xe2\x19"
			 "\xfc\x70\xc4\x6d\x8e\xb7\x99\xab"
			 "\xc5\x4b\xa2\xac\xd3\xf3\x48\xff"
			 "\x3b\xb5\xce\x53\xef\xde\xbb\x02"
			 "\xa9\x86\x15\x6c\x13\xfe\xda\x0a"
			 "\x22\xb8\x29\x3d\xd8\x39\x9a\x23",
		.clen = 48,
	},
	{
		.alg = SCE_AES_192,
		.mode = CCM,
		.ilen = 16,
		.key = "\x56\xdf\x5c\x8f\x26\x3f\x0e\x42"
		       "\xef\x7a\xd3\xce\xfc\x84\x60\x62"
		       "\xca\xb4\x40\xaf\x5f\xc9\xc9\x01",
		.klen = 24,
		.iv = "\x03\xd6\x3c\x8c\x86\x84\xb6\xcd"
		      "\xef\x09\x2e\x94\x00\x00\x00\x00",
		.assoc = "\x02\x65\x78\x3c\xe9\x21\x30\x91"
			 "\xb1\xb9\xda\x76\x9a\x78\x6d\x95"
			 "\xf2\x88\x32\xa3\xf2\x50\xcb\x4c"
			 "\xe3\x00\x73\x69\x84\x69\x87\x79",
		.alen = 32,
		.ptext = "\x9f\xd2\x02\x4b\x52\x49\x31\x3c"
			 "\x43\x69\x3a\x2d\x8e\x70\xad\x7e"
			 "\xe0\xe5\x46\x09\x80\x89\x13\xb2"
			 "\x8c\x8b\xd9\x3f\x86\xfb\xb5\x6b",
		.plen = 32,
		.ctext = "\x39\xdf\x7c\x3c\x5a\x29\xb9\x62"
			 "\x5d\x51\xc2\x16\xd8\xbd\x06\x9f"
			 "\x9b\x6a\x09\x70\xc1\x51\x83\xc2"
			 "\x66\x88\x1d\x4f\x9a\xda\xe0\x1e"
			 "\xc7\x79\x11\x58\xe5\x6b\x20\x40"
			 "\x7a\xea\x46\x42\x8b\xe4\x6f\xe1",
		.clen = 48,
	},
	{
		.alg = SCE_AES_256,
		.mode = CCM,
		.ilen = 16,
		.key = "\xe0\x8d\x99\x71\x60\xd7\x97\x1a"
		       "\xbd\x01\x99\xd5\x8a\xdf\x71\x3a"
		       "\xd3\xdf\x24\x4b\x5e\x3d\x4b\x4e"
		       "\x30\x7a\xb9\xd8\x53\x0a\x5e\x2b",
		.klen = 32,
		.iv = "\x03\x1e\x29\x91\xad\x8e\xc1\x53"
		      "\x0a\xcf\x2d\xbe\x00\x00\x00\x00",
		.assoc = "\x19\xb6\x1f\x57\xc4\xf3\xf0\x8b"
			 "\x78\x2b\x94\x02\x29\x0f\x42\x27"
			 "\x6b\x75\xcb\x98\x34\x08\x7e\x79"
			 "\xe4\x3e\x49\x0d\x84\x8b\x22\x87",
		.alen = 32,
		.ptext = "\xe1\xd9\xd8\x13\xeb\x3a\x75\x3f"
			 "\x9d\xbd\x5f\x66\xbe\xdc\xbb\x66"
			 "\xbf\x17\x99\x62\x4a\x39\x27\x1f"
			 "\x1d\xdc\x24\xae\x19\x2f\x98\x4c",
		.plen = 32,
		.ctext = "\x19\xb8\x61\x33\x45\x2b\x43\x96"
			 "\x6f\x51\xd0\x20\x30\x7d\x9b\xc6"
			 "\x26\x3d\xf8\xc9\x65\x16\xa8\x9f"
			 "\xf0\x62\x17\x34\xf2\x1e\x8d\x75"
			 "\x4e\x13\xcc\xc0\xc3\x2a\x54\x2d",
		.clen = 40,
	},
	{
		.alg = SCE_AES_256,
		.mode = CCM,
		.ilen = 16,
		.key = "\x7c\xc8\x18\x3b\x8d\x99\xe0\x7c"
		       "\x45\x41\xb8\xbd\x5c\xa7\xc2\x32"
		       "\x8a\xb8\x02\x59\xa4\xfe\xa9\x2c"
		       "\x09\x75\x9a\x9b\x3c\x9b\x27\x39",
		.klen = 32,
		.iv = "\x03\xf9\xd9\x4e\x63\xb5\x3d\x9d"
		      "\x43\xf6\x1e\x50\0\0\0\0",
		.assoc = "\x57\xf5\x6b\x8b\x57\x5c\x3d\x3b"
			 "\x13\x02\x01\x0c\x83\x4c\x96\x35"
			 "\x8e\xd6\x39\xcf\x7d\x14\x9b\x94"
			 "\xb0\x39\x36\xe6\x8f\x57\xe0\x13",
		.alen = 32,
		.ptext = "\x3b\x6c\x29\x36\xb6\xef\x07\xa6"
			 "\x83\x72\x07\x4f\xcf\xfa\x66\x89"
			 "\x5f\xca\xb1\xba\xd5\x8f\x2c\x27"
			 "\x30\xdb\x75\x09\x93\xd4\x65\xe4",
		.plen = 32,
		.ctext = "\xb0\x88\x5a\x33\xaa\xe5\xc7\x1d"
			 "\x85\x23\xc7\xc6\x2f\xf4\x1e\x3d"
			 "\xcc\x63\x44\x25\x07\x78\x4f\x9e"
			 "\x96\xb8\x88\xeb\xbc\x48\x1f\x06"
			 "\x39\xaf\x39\xac\xd8\x4a\x80\x39"
			 "\x7b\x72\x8a\xf7",
		.clen = 44,
	},
	{
		.alg = SCE_AES_256,
		.mode = CCM,
		.ilen = 16,
		.key = "\xab\xd0\xe9\x33\x07\x26\xe5\x83"
		       "\x8c\x76\x95\xd4\xb6\xdc\xf3\x46"
		       "\xf9\x8f\xad\xe3\x02\x13\x83\x77"
		       "\x3f\xb0\xf1\xa1\xa1\x22\x0f\x2b",
		.klen = 32,
		.iv = "\x03\x24\xa7\x8b\x07\xcb\xcc\x0e"
		      "\xe6\x33\xbf\xf5\x00\x00\x00\x00",
		.assoc = "\xd4\xdb\x30\x1d\x03\xfe\xfd\x5f"
			 "\x87\xd4\x8c\xb6\xb6\xf1\x7a\x5d"
			 "\xab\x90\x65\x8d\x8e\xca\x4d\x4f"
			 "\x16\x0c\x40\x90\x4b\xc7\x36\x73",
		.alen = 32,
		.ptext = "\xf5\xc6\x7d\x48\xc1\xb7\xe6\x92"
			 "\x97\x5a\xca\xc4\xa9\x6d\xf9\x3d"
			 "\x6c\xde\xbc\xf1\x90\xea\x6a\xb2"
			 "\x35\x86\x36\xaf\x5c\xfe\x4b\x3a",
		.plen = 32,
		.ctext = "\x83\x6f\x40\x87\x72\xcf\xc1\x13"
			 "\xef\xbb\x80\x21\x04\x6c\x58\x09"
			 "\x07\x1b\xfc\xdf\xc0\x3f\x5b\xc7"
			 "\xe0\x79\xa8\x6e\x71\x7c\x3f\xcf"
			 "\x5c\xda\xb2\x33\xe5\x13\xe2\x0d"
			 "\x74\xd1\xef\xb5\x0f\x3a\xb5\xf8",
		.clen = 48,
	},
	{
		.alg = SCE_AES_128,
		.mode = CCM,
		.ilen = 16,
		.key = "\x61\x0e\x8c\xae\xe3\x23\xb6\x38"
		       "\x76\x1c\xf6\x3a\x67\xa3\x9c\xd8",
		.klen = 16,
		.iv = "\x03\x05\xe0\xc9\x0f\xed\x34\xea"
		      "\x97\xd4\x3b\xdf\x00\x00\x00\x00",
		.assoc = "\x49\x5c\x50\x1f\x1d\x94\xcc\x81"
			 "\xba\xb7\xb6\x03\xaf\xa5\xc1\xa1"
			 "\xd8\x5c\x42\x68\xe0\x6c\xda\x89"
			 "\x05\xac\x56\xac\x1b\x2a\xd3\x86",
		.alen = 32,
		.ptext = "\x75\x05\xbe\xc2\xd9\x1e\xde\x60"
			 "\x47\x3d\x8c\x7d\xbd\xb5\xd9\xb7"
			 "\xf2\xae\x61\x05\x8f\x82\x24\x3f"
			 "\x9c\x67\x91\xe1\x38\x4f\xe4\x0c",
		.plen = 32,
		.ctext = "\x39\xbe\x7d\x15\x62\x77\xf3\x3c"
			 "\xad\x83\x52\x6d\x71\x03\x25\x1c"
			 "\xed\x81\x3a\x9a\x16\x7d\x19\x80"
			 "\x72\x04\x72\xd0\xf6\xff\x05\x0f"
			 "\xb7\x14\x30\x00\x32\x9e\xa0\xa6"
			 "\x9e\x5a\x18\xa1\xb8\xfe\xdb\xd3",
		.clen = 48,
	},
	{
		.alg = SCE_AES_192,
		.mode = CCM,
		.ilen = 16,
		.key = "\x58\x5d\xa0\x96\x65\x1a\x04\xd7"
		       "\x96\xe5\xc5\x68\xaa\x95\x35\xe0"
		       "\x29\xa0\xba\x9e\x48\x78\xd1\xba",
		.klen = 24,
		.iv = "\x03\xee\x49\x83\xe9\xa9\xff\xe9"
		      "\x57\xba\xfd\x9e\x00\x00\x00\x00",
		.assoc = "\x44\xa6\x2c\x05\xe9\xe1\x43\xb1"
			 "\x58\x7c\xf2\x5c\x6d\x39\x0a\x64"
			 "\xa4\xf0\x13\x05\xd1\x77\x99\x67"
			 "\x11\xc4\xc6\xdb\x00\x56\x36\x61",
		.alen = 32,
		.ptext = "\x85\x34\x66\x42\xc8\x92\x0f\x36"
			 "\x58\xe0\x6b\x91\x3c\x98\x5c\xbb"
			 "\x0a\x85\xcc\x02\xad\x7a\x96\xe9"
			 "\x65\x43\xa4\xc3\x0f\xdc\x55\x81",
		.plen = 32,
		.ctext = "\xfb\xe5\x5d\x34\xbe\xe5\xe8\xe7"
			 "\x5a\xef\x2f\xbf\x1f\x7f\xd4\xb2"
			 "\x66\xca\x61\x1e\x96\x7a\x61\xb3"
			 "\x1c\x16\x45\x52\xba\x04\x9c\x9f"
			 "\xb1\xd2\x40\xbc\x52\x7c\x6f\xb1",
		.clen = 40,
	},
	{
		.alg = SCE_AES_256,
		.mode = CCM,
		.ilen = 16,
		.key = "\x58\x5d\xa0\x96\x65\x1a\x04\xd7"
		       "\x96\xe5\xc5\x68\xaa\x95\x35\xe0"
		       "\x29\xa0\xba\x9e\x48\x78\xd1\xba"
		       "\x0d\x1a\x53\x3b\xb5\xe3\xf8\x8b",
		.klen = 32,
		.iv = "\x03\xcf\x76\x3f\xd9\x95\x75\x8f"
		      "\x44\x89\x40\x7b\x00\x00\x00\x00",
		.assoc = "\x8f\x86\x6c\x4d\x1d\xc5\x39\x88"
			 "\xc8\xf3\x5c\x52\x10\x63\x6f\x2b"
			 "\x8a\x2a\xc5\x6f\x30\x23\x58\x7b"
			 "\xfb\x36\x03\x11\xb4\xd9\xf2\xfe",
		.alen = 32,
		.ptext = "\xc2\x54\xc8\xde\x78\x87\x77\x40"
			 "\x49\x71\xe4\xb7\xe7\xcb\x76\x61"
			 "\x0a\x41\xb9\xe9\xc0\x76\x54\xab"
			 "\x04\x49\x3b\x19\x93\x57\x25\x5d",
		.plen = 32,
		.ctext = "\x48\x58\xd6\xf3\xad\x63\x58\xbf"
			 "\xae\xc7\x5e\xae\x83\x8f\x7b\xe4"
			 "\x78\x5c\x4c\x67\x71\x89\x94\xbf"
			 "\x47\xf1\x63\x7e\x1c\x59\xbd\xc5"
			 "\x7f\x44\x0a\x0c\x01\x18\x07\x92"
			 "\xe1\xd3\x51\xce\x32\x6d\x0c\x5b",
		.clen = 48,
	},
};

ske_testvec_t ske_mac_tv_template[] = {
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
		       "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
		       "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96",
		.ctext = "\xa5\x1c\x52\x77\x25\x63\x2c\xcf",
		.len = 8,
		.klen = 24,
		.taglen = 8,
		.alg = SCE_TDES,
		.mode = CBC_MAC,
	},
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
		       "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
		       "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57",
		.ctext = "\x7a\xe0\xe1\x7e\x61\x86\x99\x74",
		.len = 20,
		.klen = 24,
		.taglen = 8,
		.alg = SCE_TDES,
		.mode = CBC_MAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2\x49\x03\xDD\xC6\xB8\xCA\x55\x7A"
		       "\xd0\xe7\x4b\xfb\x5d\xe5\x0c\xe7",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
		.ctext = "\x6c\xe5\xbf\xa1\x49\x9e\xd4\x49",
		.len = 32,
		.klen = 24,
		.taglen = 8,
		.alg = SCE_TDES,
		.mode = CBC_MAC,
	},
#if 0
    {
        .key		= "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                    "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                    "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .ctext		= "\x00\x00\x00\x00\x00\x00\x00\x00",
        .len		= 0,
        .klen          = 24,
        .taglen          = 8,
        .alg            = SCE_TDES,
        .mode           = CBC_MAC,
    },
#endif
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
		       "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
		       "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51",
		.ctext = "\xdf\xbb\x75\xaf\xe7\x48\xba\xa0",
		.len = 32,
		.klen = 24,
		.taglen = 8,
		.alg = SCE_TDES,
		.mode = CBC_MAC,
	},
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96",
		.ctext = "\x4e\xd6\x35\x72\x6d\x82\x43\x06",
		.len = 8,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CBC_MAC,
	},
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57",
		.ctext = "\xb6\xc5\x18\x67\x5b\x83\x16\x16",
		.len = 20,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CBC_MAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
		.ctext = "\xfd\x64\x03\xf0\x51\x74\x81\xf7",
		.len = 32,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CBC_MAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
			"\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4"
			"\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b",
		.ctext = "\x0b\xcc\x79\xcf\x40\xd5\x85\x54",
		.len = 64,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CBC_MAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
			"\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4"
			"\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
			"\x59\xda\x19\x4f\xec\x07\x8e\xe2\xf0\x61\xf9\x27\x61\x6f\xf8\xdf"
			"\x62\x4d\xaf\x06\xfe\x41\xa6\xa6\xf9\xa2\x06\x40\xb3\x04\xbd\xe6"
			"\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e\xa6\x58\xdc\x91\x17\x58\x42\x95"
			"\xa3\x7c\x81\x78\xa6\x3d\x3f\x75\x74\x17\x1a\xd3\x6c\x2f\x48\x39",
		.ctext = "\x9b\xd2\xee\x0a\xa7\x51\xa7\xa4",
		.len = 128,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CBC_MAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
			"\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4"
			"\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
			"\x59\xda\x19\x4f\xec\x07\x8e\xe2\xf0\x61\xf9\x27\x61\x6f\xf8\xdf"
			"\x62\x4d\xaf\x06\xfe\x41\xa6\xa6\xf9\xa2\x06\x40\xb3\x04\xbd\xe6"
			"\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e\xa6\x58\xdc\x91\x17\x58\x42\x95"
			"\xa3\x7c\x81\x78\xa6\x3d\x3f\x75\x74\x17\x1a\xd3\x6c\x2f\x48\x39"
			"\x20\x20\xc1\x9a\x29\x84\x7d\x2d\x52\xa1\xf9\x5c\xf3\x4f\x91\xfc"
			"\x75\xcf\xd6\x2d\xe7\x9a\x59\x6e\x00\x0e\x8d\x22\x17\xbd\xa0\xdd"
			"\x79\x1f\x71\xe6\xcd\x2f\xb1\xb6\xbc\xc3\xdb\x02\x91\x41\x9b\x09"
			"\xa9\xd2\x7e\xbd\x2c\x18\xae\xc0\x93\x0c\x02\x9a\xdb\x4e\xaa\xeb"
			"\x84\x4b\x43\x5e\xf0\x98\xf2\x5f\x86\x70\x96\x90\x15\x30\xcf\x3a"
			"\xc9\x33\x21\xec\x59\x86\xfc\x65\x7d\xbe\xb9\xf8\x97\xf9\x30\xa9"
			"\x6d\xfc\x0c\x6e\x36\x67\xd5\xa6\x67\xd9\xbd\x9b\x34\x5d\xa7\xdd"
			"\xda\x46\x33\x25\x60\x4a\x18\xf1\x55\x07\xb2\xb7\x26\x7b\xa6\x1e",
		.ctext = "\x9c\xbe\x25\x7b\xa1\xc0\xb6\x41",
		.len = 256,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CBC_MAC,
	},
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
		       "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
		       "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96",
		.ctext = "\x8e\x8f\x29\x31\x36\x28\x37\x97",
		.len = 8,
		.klen = 24,
		.taglen = 8,
		.alg = SCE_TDES,
		.mode = CMAC,
	},
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
		       "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
		       "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57",
		.ctext = "\x74\x3d\xdb\xe0\xce\x2d\xc2\xed",
		.len = 20,
		.klen = 24,
		.taglen = 8,
		.alg = SCE_TDES,
		.mode = CMAC,
	},
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
		       "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
		       "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51",
		.ctext = "\x33\xe6\xb1\x09\x24\x00\xea\xe5",
		.len = 32,
		.klen = 24,
		.taglen = 8,
		.alg = SCE_TDES,
		.mode = CMAC,
	},
#if 0
    {
        .key		= "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
                    "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
                    "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
        .ctext		= "\xb7\xa6\x88\xe1\x22\xff\xaf\x95",
        .len		= 0,
        .klen          = 24,
        .taglen          = 8,
        .alg            = SCE_TDES,
        .mode           = CMAC,
    },
#endif
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62"
		       "\x0b\xc1\xbf\x19\xfb\xb6\xcd\x58"
		       "\xbc\x31\x3d\x4a\x37\x1c\xa8\xb5",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51",
		.ctext = "\x33\xe6\xb1\x09\x24\x00\xea\xe5",
		.len = 32,
		.klen = 24,
		.taglen = 8,
		.alg = SCE_TDES,
		.mode = CMAC,
	},
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96",
		.ctext = "\x20\x37\x34\xc0\x22\xb2\x26\xc8",
		.len = 8,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CMAC,
	},
	{
		.key = "\x8a\xa8\x3b\xf8\xcb\xda\x10\x62",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57",
		.ctext = "\xe3\xa8\xdd\x10\x1a\x7b\xcb\xb5",
		.len = 20,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CMAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b",
		.ctext = "\x1B\x57\xDD\xBD\xE6\xA4\x44\x70",
		.len = 32,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CMAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
			"\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4"
			"\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b",
		.ctext = "\x65\x20\xB3\xAE\xAB\x60\x7B\xEB",
		.len = 64,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CMAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
			"\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4"
			"\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
			"\x59\xda\x19\x4f\xec\x07\x8e\xe2\xf0\x61\xf9\x27\x61\x6f\xf8\xdf"
			"\x62\x4d\xaf\x06\xfe\x41\xa6\xa6\xf9\xa2\x06\x40\xb3\x04\xbd\xe6"
			"\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e\xa6\x58\xdc\x91\x17\x58\x42\x95"
			"\xa3\x7c\x81\x78\xa6\x3d\x3f\x75\x74\x17\x1a\xd3\x6c\x2f\x48\x39",
		.ctext = "\xB8\xEC\x22\x52\x22\x9A\xDA\x62",
		.len = 128,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CMAC,
	},
	{
		.key = "\xE4\x23\x33\x8A\x35\x64\x61\xE2",
		.ptext =
			"\xff\xca\xfb\xf1\x38\x20\x2f\x7b\x24\x98\x26\x7d\x1d\x9f\xb3\x93"
			"\xd9\xef\xbd\xad\x4e\x40\xbd\x60\xe9\x48\x59\x90\x67\xd7\x2b\x7b"
			"\x8a\xe0\x4d\xb0\x70\x38\xcc\x48\x61\x7d\xee\xd6\x35\x49\xae\xb4"
			"\xaf\x6b\xdd\xe6\x21\xc0\x60\xce\x0a\xf4\x1c\x2e\x1c\x8d\xe8\x7b"
			"\x59\xda\x19\x4f\xec\x07\x8e\xe2\xf0\x61\xf9\x27\x61\x6f\xf8\xdf"
			"\x62\x4d\xaf\x06\xfe\x41\xa6\xa6\xf9\xa2\x06\x40\xb3\x04\xbd\xe6"
			"\xc8\x17\xfb\x56\x6f\xa9\x3b\x8e\xa6\x58\xdc\x91\x17\x58\x42\x95"
			"\xa3\x7c\x81\x78\xa6\x3d\x3f\x75\x74\x17\x1a\xd3\x6c\x2f\x48\x39"
			"\x20\x20\xc1\x9a\x29\x84\x7d\x2d\x52\xa1\xf9\x5c\xf3\x4f\x91\xfc"
			"\x75\xcf\xd6\x2d\xe7\x9a\x59\x6e\x00\x0e\x8d\x22\x17\xbd\xa0\xdd"
			"\x79\x1f\x71\xe6\xcd\x2f\xb1\xb6\xbc\xc3\xdb\x02\x91\x41\x9b\x09"
			"\xa9\xd2\x7e\xbd\x2c\x18\xae\xc0\x93\x0c\x02\x9a\xdb\x4e\xaa\xeb"
			"\x84\x4b\x43\x5e\xf0\x98\xf2\x5f\x86\x70\x96\x90\x15\x30\xcf\x3a"
			"\xc9\x33\x21\xec\x59\x86\xfc\x65\x7d\xbe\xb9\xf8\x97\xf9\x30\xa9"
			"\x6d\xfc\x0c\x6e\x36\x67\xd5\xa6\x67\xd9\xbd\x9b\x34\x5d\xa7\xdd"
			"\xda\x46\x33\x25\x60\x4a\x18\xf1\x55\x07\xb2\xb7\x26\x7b\xa6\x1e",
		.ctext = "\xDE\xF1\x05\x9E\xBD\x6D\x7B\x37",
		.len = 256,
		.klen = 8,
		.taglen = 8,
		.alg = SCE_DES,
		.mode = CMAC,
	},
	{
		.ptext =
			"\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96\xe9\x3d\x7e\x11\x73\x93\x17\x2a",
		.ctext =
			"\x07\x0a\x16\xb4\x6b\x4d\x41\x44\xf7\x9b\xdd\x9d\xd0\x4a\x28\x7c",
		.key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6\xab\xf7\x15\x88\x09\xcf\x4f\x3c",

		.klen = 16,
		.taglen = 16,
		.len = 16,
		.alg = SCE_AES_128,
		.mode = CMAC,
	},
	{
		.klen = 16,
		.len = 16,
		.taglen = 16,
		.alg = SCE_SM4,
		.mode = CBC_MAC,
		.key = "\xff\xee\xdd\xcc\xbb\xaa\x99\x88"
		       "\x77\x66\x55\x44\x33\x22\x11\x00",
		.ptext = "\x01\x23\x45\x67\x89\xab\xcd\xef"
			 "\xfe\xdc\xba\x98\x76\x54\x32\x10",
		.ctext = "\x97\xb4\x75\x8f\x84\x92\x3d\x3f"
			 "\x86\x81\x0e\x0e\xea\x14\x6d\x73",
	},
	{
		.alg = SCE_AES_128,
		.mode = CBC_MAC,
		.taglen = 16,
		.key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
		       "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30",
		.ctext = "\x9d\x0d\xd0\x63\xfb\xcb\x24\x43"
			 "\xf8\xf2\x76\x03\xac\x39\xb0\x9d", //d
		.len = 33,
		.klen = 16,
	},
	{
		.klen = 24,
		.len = 33,
		.taglen = 16,
		.alg = SCE_AES_192,
		.mode = CBC_MAC,
		.key = "\xab\x23\x0a\x26\x8b\x6d\x28\xe5\xf1\xfb\x9e\x62\x4f\xed\xf6\xe0"
		       "\xf1\x7d\x92\x82\xc6\xde\x08\x9f",
		.ptext =
			"\x51\x74\x8f\x85\x0d\xd4\xca\x27\xce\x2c\x37\xec\xc7\x2b\xe5\x01"
			"\xfa\x8e\xbf\xb1\xaa\xa4\x74\xba\x08\x3a\xf2\x0a\x99\x55\xf7\x74"
			"\x10",
		.ctext =
			"\x5e\x20\x69\x1f\x94\xf3\x32\x77\x59\xdc\x6c\xa1\x48\x85\x67\xa1",
	},
	{
		.alg = SCE_AES_256,
		.mode = CBC_MAC,
		.taglen = 16,
		.key = "\x60\x3d\xeb\x10\x15\xca\x71\xbe"
		       "\x2b\x73\xae\xf0\x85\x7d\x77\x81"
		       "\x1f\x35\x2c\x07\x3b\x61\x08\xd7"
		       "\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
			 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
			 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
			 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10"
			 "\x1c",
		.ctext = "\x6a\x4e\xdb\x21\x47\x51\xdf\x4f"
			 "\xa8\x4d\x4c\x10\x3b\x72\x7d\xd6",
		.len = 65,
		.klen = 32,
	},
	//CMAC
	{
		.alg = SCE_AES_256,
		.mode = CMAC,
		.taglen = 16,
		.key = "\x60\x3d\xeb\x10\x15\xca\x71\xbe"
		       "\x2b\x73\xae\xf0\x85\x7d\x77\x81"
		       "\x1f\x35\x2c\x07\x3b\x61\x08\xd7"
		       "\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
			 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
			 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
			 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
		.ctext = "\xe1\x99\x21\x90\x54\x9f\x6e\xd5"
			 "\x69\x6a\x2c\x05\x6c\x31\x54\x10",
		.len = 64,
		.klen = 32,
	},
	{
		.klen = 24,
		.len = 63,
		.taglen = 16,
		.alg = SCE_AES_192,
		.mode = CMAC,
		.key = "\xf5\x4e\x17\x13\x3a\x5a\x88\xd7\xa7\x44\x9d\xf4\xbf\x89\x0c\x9d"
		       "\xa0\x60\x79\xc6\x5c\xac\x11\x4c",
		.ptext =
			"\xee\x0d\x63\x83\x58\x76\xb6\x0d\xf3\x8e\x6d\x07\x22\xc9\xd0\xf3"
			"\xd7\xff\xfe\xdb\xd6\xc9\x01\xd0\x37\x7a\xaa\x03\x58\xdb\x8e\x49"
			"\xaf\x38\x7c\xe5\xf3\x59\x41\x7f\x33\xdf\x50\xf7\x8d\x52\x88\xdd"
			"\xec\x40\x65\x8d\x83\xea\xa5\xdf\x94\x68\xbc\xd3\x55\xd8\xa6",
		.ctext =
			"\xfb\xd7\x63\x8e\x6d\x1e\x53\xfd\x86\x21\xb8\x48\xb1\xae\x1e\x09",
	},
	{
		.alg = SCE_AES_128,
		.mode = CMAC,
		.taglen = 16,
		.key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
		       "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a",
		.ctext = "\x07\x0a\x16\xb4\x6b\x4d\x41\x44"
			 "\xf7\x9b\xdd\x9d\xd0\x4a\x28\x7c",
		.len = 16,
		.klen = 16,
	},
	{
		.klen = 16,
		.len = 16,
		.taglen = 16,
		.alg = SCE_SM4,
		.mode = CMAC,
		.ilen = 16,
		.iv = "\x00\x00\x00\x00\x00\x00\x00\x00"
		      "\x00\x00\x00\x00\x00\x00\x00\x00",
		.key = "\xff\xee\xdd\xcc\xbb\xaa\x99\x88"
		       "\x77\x66\x55\x44\x33\x22\x11\x00",
		.ptext = "\x01\x23\x45\x67\x89\xab\xcd\xef"
			 "\xfe\xdc\xba\x98\x76\x54\x32\x10",
		.ctext = "\x00\xd4\x63\xb4\x9a\xf3\x52\xe2"
			 "\x74\xa9\x00\x55\x13\x54\x2a\xd1",
	},
};

ske_testvec_t ske_tv_template[] = {
#if 0
	//XTS

	{
		.klen = 64,
		.len = 512,
		.ilen = 16,
		.mode = XTS,
		.alg = SCE_AES_256,
		.key = "\x27\x18\x28\x18\x28\x45\x90\x45"
		       "\x23\x53\x60\x28\x74\x71\x35\x26"
		       "\x62\x49\x77\x57\x24\x70\x93\x69"
		       "\x99\x59\x57\x49\x66\x96\x76\x27"
		       "\x31\x41\x59\x26\x53\x58\x97\x93"
		       "\x23\x84\x62\x64\x33\x83\x27\x95"
		       "\x02\x88\x41\x97\x16\x93\x99\x37"
		       "\x51\x05\x82\x09\x74\x94\x45\x92",
		.iv = "\xff\x00\x00\x00\x00\x00\x00\x00"
		      "\x00\x00\x00\x00\x00\x00\x00\x00",
		.ptext = "\x00\x01\x02\x03\x04\x05\x06\x07"
			 "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
			 "\x10\x11\x12\x13\x14\x15\x16\x17"
			 "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f"
			 "\x20\x21\x22\x23\x24\x25\x26\x27"
			 "\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f"
			 "\x30\x31\x32\x33\x34\x35\x36\x37"
			 "\x38\x39\x3a\x3b\x3c\x3d\x3e\x3f"
			 "\x40\x41\x42\x43\x44\x45\x46\x47"
			 "\x48\x49\x4a\x4b\x4c\x4d\x4e\x4f"
			 "\x50\x51\x52\x53\x54\x55\x56\x57"
			 "\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f"
			 "\x60\x61\x62\x63\x64\x65\x66\x67"
			 "\x68\x69\x6a\x6b\x6c\x6d\x6e\x6f"
			 "\x70\x71\x72\x73\x74\x75\x76\x77"
			 "\x78\x79\x7a\x7b\x7c\x7d\x7e\x7f"
			 "\x80\x81\x82\x83\x84\x85\x86\x87"
			 "\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f"
			 "\x90\x91\x92\x93\x94\x95\x96\x97"
			 "\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f"
			 "\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7"
			 "\xa8\xa9\xaa\xab\xac\xad\xae\xaf"
			 "\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7"
			 "\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf"
			 "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
			 "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf"
			 "\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7"
			 "\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf"
			 "\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7"
			 "\xe8\xe9\xea\xeb\xec\xed\xee\xef"
			 "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
			 "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff"
			 "\x00\x01\x02\x03\x04\x05\x06\x07"
			 "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
			 "\x10\x11\x12\x13\x14\x15\x16\x17"
			 "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f"
			 "\x20\x21\x22\x23\x24\x25\x26\x27"
			 "\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f"
			 "\x30\x31\x32\x33\x34\x35\x36\x37"
			 "\x38\x39\x3a\x3b\x3c\x3d\x3e\x3f"
			 "\x40\x41\x42\x43\x44\x45\x46\x47"
			 "\x48\x49\x4a\x4b\x4c\x4d\x4e\x4f"
			 "\x50\x51\x52\x53\x54\x55\x56\x57"
			 "\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f"
			 "\x60\x61\x62\x63\x64\x65\x66\x67"
			 "\x68\x69\x6a\x6b\x6c\x6d\x6e\x6f"
			 "\x70\x71\x72\x73\x74\x75\x76\x77"
			 "\x78\x79\x7a\x7b\x7c\x7d\x7e\x7f"
			 "\x80\x81\x82\x83\x84\x85\x86\x87"
			 "\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f"
			 "\x90\x91\x92\x93\x94\x95\x96\x97"
			 "\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f"
			 "\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7"
			 "\xa8\xa9\xaa\xab\xac\xad\xae\xaf"
			 "\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7"
			 "\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf"
			 "\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7"
			 "\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf"
			 "\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7"
			 "\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf"
			 "\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7"
			 "\xe8\xe9\xea\xeb\xec\xed\xee\xef"
			 "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
			 "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff",
		.ctext = "\x1c\x3b\x3a\x10\x2f\x77\x03\x86"
			 "\xe4\x83\x6c\x99\xe3\x70\xcf\x9b"
			 "\xea\x00\x80\x3f\x5e\x48\x23\x57"
			 "\xa4\xae\x12\xd4\x14\xa3\xe6\x3b"
			 "\x5d\x31\xe2\x76\xf8\xfe\x4a\x8d"
			 "\x66\xb3\x17\xf9\xac\x68\x3f\x44"
			 "\x68\x0a\x86\xac\x35\xad\xfc\x33"
			 "\x45\xbe\xfe\xcb\x4b\xb1\x88\xfd"
			 "\x57\x76\x92\x6c\x49\xa3\x09\x5e"
			 "\xb1\x08\xfd\x10\x98\xba\xec\x70"
			 "\xaa\xa6\x69\x99\xa7\x2a\x82\xf2"
			 "\x7d\x84\x8b\x21\xd4\xa7\x41\xb0"
			 "\xc5\xcd\x4d\x5f\xff\x9d\xac\x89"
			 "\xae\xba\x12\x29\x61\xd0\x3a\x75"
			 "\x71\x23\xe9\x87\x0f\x8a\xcf\x10"
			 "\x00\x02\x08\x87\x89\x14\x29\xca"
			 "\x2a\x3e\x7a\x7d\x7d\xf7\xb1\x03"
			 "\x55\x16\x5c\x8b\x9a\x6d\x0a\x7d"
			 "\xe8\xb0\x62\xc4\x50\x0d\xc4\xcd"
			 "\x12\x0c\x0f\x74\x18\xda\xe3\xd0"
			 "\xb5\x78\x1c\x34\x80\x3f\xa7\x54"
			 "\x21\xc7\x90\xdf\xe1\xde\x18\x34"
			 "\xf2\x80\xd7\x66\x7b\x32\x7f\x6c"
			 "\x8c\xd7\x55\x7e\x12\xac\x3a\x0f"
			 "\x93\xec\x05\xc5\x2e\x04\x93\xef"
			 "\x31\xa1\x2d\x3d\x92\x60\xf7\x9a"
			 "\x28\x9d\x6a\x37\x9b\xc7\x0c\x50"
			 "\x84\x14\x73\xd1\xa8\xcc\x81\xec"
			 "\x58\x3e\x96\x45\xe0\x7b\x8d\x96"
			 "\x70\x65\x5b\xa5\xbb\xcf\xec\xc6"
			 "\xdc\x39\x66\x38\x0a\xd8\xfe\xcb"
			 "\x17\xb6\xba\x02\x46\x9a\x02\x0a"
			 "\x84\xe1\x8e\x8f\x84\x25\x20\x70"
			 "\xc1\x3e\x9f\x1f\x28\x9b\xe5\x4f"
			 "\xbc\x48\x14\x57\x77\x8f\x61\x60"
			 "\x15\xe1\x32\x7a\x02\xb1\x40\xf1"
			 "\x50\x5e\xb3\x09\x32\x6d\x68\x37"
			 "\x8f\x83\x74\x59\x5c\x84\x9d\x84"
			 "\xf4\xc3\x33\xec\x44\x23\x88\x51"
			 "\x43\xcb\x47\xbd\x71\xc5\xed\xae"
			 "\x9b\xe6\x9a\x2f\xfe\xce\xb1\xbe"
			 "\xc9\xde\x24\x4f\xbe\x15\x99\x2b"
			 "\x11\xb7\x7c\x04\x0f\x12\xbd\x8f"
			 "\x6a\x97\x5a\x44\xa0\xf9\x0c\x29"
			 "\xa9\xab\xc3\xd4\xd8\x93\x92\x72"
			 "\x84\xc5\x87\x54\xcc\xe2\x94\x52"
			 "\x9f\x86\x14\xdc\xd2\xab\xa9\x91"
			 "\x92\x5f\xed\xc4\xae\x74\xff\xac"
			 "\x6e\x33\x3b\x93\xeb\x4a\xff\x04"
			 "\x79\xda\x9a\x41\x0e\x44\x50\xe0"
			 "\xdd\x7a\xe4\xc6\xe2\x91\x09\x00"
			 "\x57\x5d\xa4\x01\xfc\x07\x05\x9f"
			 "\x64\x5e\x8b\x7e\x9b\xfd\xef\x33"
			 "\x94\x30\x54\xff\x84\x01\x14\x93"
			 "\xc2\x7b\x34\x29\xea\xed\xb4\xed"
			 "\x53\x76\x44\x1a\x77\xed\x43\x85"
			 "\x1a\xd7\x7f\x16\xf5\x41\xdf\xd2"
			 "\x69\xd5\x0d\x6a\x5f\x14\xfb\x0a"
			 "\xab\x1c\xbb\x4c\x15\x50\xbe\x97"
			 "\xf7\xab\x40\x66\x19\x3c\x4c\xaa"
			 "\x77\x3d\xad\x38\x01\x4b\xd2\x09"
			 "\x2f\xa7\x55\xc8\x24\xbb\x5e\x54"
			 "\xc4\xf3\x6f\xfd\xa9\xfc\xea\x70"
			 "\xb9\xc6\xe6\x93\xe1\x48\xc1\x51",
	},
	{
		/* XTS-SM4 1 */
		.klen = 32,
		.len = 32,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = XTS,
		.key = "\x00\x00\x00\x00\x00\x00\x00\x00"
		       "\x00\x00\x00\x00\x00\x00\x00\x00"
		       "\x00\x00\x00\x00\x00\x00\x00\x00"
		       "\x00\x00\x00\x00\x00\x00\x00\x00",
		.iv = "\x00\x00\x00\x00\x00\x00\x00\x00"
		      "\x00\x00\x00\x00\x00\x00\x00\x00",
		.ptext = "\x00\x00\x00\x00\x00\x00\x00\x00"
			 "\x00\x00\x00\x00\x00\x00\x00\x00"
			 "\x00\x00\x00\x00\x00\x00\x00\x00"
			 "\x00\x00\x00\x00\x00\x00\x00\x00",
		.ctext =
			"\xd9\xb4\x21\xf7\x31\xc8\x94\xfd\xc3\x5b\x77\x29\x1f\xe4\xe3\xb0"
			"\x2a\x1f\xb7\x66\x98\xd5\x9f\x0e\x51\x37\x6c\x4a\xda\x5b\xc7\x5d",
	},
	{
		/* XTS-AES 2 */
		.klen = 32,
		.len = 32,
		.ilen = 16,
		.alg = SCE_AES_128,
		.mode = XTS,
		.key = "\x11\x11\x11\x11\x11\x11\x11\x11"
		       "\x11\x11\x11\x11\x11\x11\x11\x11"
		       "\x22\x22\x22\x22\x22\x22\x22\x22"
		       "\x22\x22\x22\x22\x22\x22\x22\x22",
		.iv = "\x33\x33\x33\x33\x33\x00\x00\x00"
		      "\x00\x00\x00\x00\x00\x00\x00\x00",
		.ptext = "\x44\x44\x44\x44\x44\x44\x44\x44"
			 "\x44\x44\x44\x44\x44\x44\x44\x44"
			 "\x44\x44\x44\x44\x44\x44\x44\x44"
			 "\x44\x44\x44\x44\x44\x44\x44\x44",
		.ctext = "\xc4\x54\x18\x5e\x6a\x16\x93\x6e"
			 "\x39\x33\x40\x38\xac\xef\x83\x8b"
			 "\xfb\x18\x6f\xff\x74\x80\xad\xc4"
			 "\x28\x93\x82\xec\xd6\xd3\x94\xf0",
	},
	{
		/* XTS-AES 3 */
		.klen = 32,
		.len = 32,
		.ilen = 16,
		.alg = SCE_AES_128,
		.mode = XTS,
		.key = "\xff\xfe\xfd\xfc\xfb\xfa\xf9\xf8"
		       "\xf7\xf6\xf5\xf4\xf3\xf2\xf1\xf0"
		       "\x22\x22\x22\x22\x22\x22\x22\x22"
		       "\x22\x22\x22\x22\x22\x22\x22\x22",
		.iv = "\x33\x33\x33\x33\x33\x00\x00\x00"
		      "\x00\x00\x00\x00\x00\x00\x00\x00",
		.ptext = "\x44\x44\x44\x44\x44\x44\x44\x44"
			 "\x44\x44\x44\x44\x44\x44\x44\x44"
			 "\x44\x44\x44\x44\x44\x44\x44\x44"
			 "\x44\x44\x44\x44\x44\x44\x44\x44",
		.ctext = "\xaf\x85\x33\x6b\x59\x7a\xfc\x1a"
			 "\x90\x0b\x2e\xb2\x1e\xc9\x49\xd2"
			 "\x92\xdf\x4c\x04\x7e\x0b\x21\x53"
			 "\x21\x86\xa5\x97\x1a\x22\x7a\x89",
	},
#endif
	{
		/* GB/T 32907-2016 Example 1. */
		.klen = 16,
		.len = 16,
		.ilen = 0,
		.alg = SCE_SM4,
		.mode = ECB,
		.key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
		       "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.ptext = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
			 "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.ctext = "\x68\x1E\xDF\x34\xD2\x06\x96\x5E"
			 "\x86\xB3\xE9\x4F\x53\x6E\x42\x46",
	},
	{
		/* Last 10 iterations of GB/T 32907-2016 Example 2. */
		.klen = 16,
		.len = 160,
		.ilen = 0,
		.alg = SCE_SM4,
		.mode = ECB,
		.key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
		       "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.ptext = "\x99\x4a\xc3\xe7\xc3\x57\x89\x6a"
			 "\x81\xfc\xa8\xe\x38\x3e\xef\x80"
			 "\xb1\x98\xf2\xde\x3f\x4b\xae\xd1"
			 "\xf0\xf1\x30\x4c\x1\x27\x5a\x8f"
			 "\x45\xe1\x39\xb7\xae\xff\x1f\x27"
			 "\xad\x57\x15\xab\x31\x5d\xc\xef"
			 "\x8c\xc8\x80\xbd\x11\x98\xf3\x7b"
			 "\xa2\xdd\x14\x20\xf9\xe8\xbb\x82"
			 "\xf7\x32\xca\x4b\xa8\xf7\xb3\x4d"
			 "\x27\xd1\xcd\xe6\xb6\x65\x5a\x23"
			 "\xc2\xf3\x54\x84\x53\xe3\xb9\x20"
			 "\xa5\x37\x0\xbe\xe7\x7b\x48\xfb"
			 "\x21\x3d\x9e\x48\x1d\x9e\xf5\xbf"
			 "\x77\xd5\xb4\x4a\x53\x71\x94\x7a"
			 "\x88\xa6\x6e\x6\x93\xca\x43\xa5"
			 "\xc4\xf6\xcd\x53\x4b\x7b\x8e\xfe"
			 "\xb4\x28\x7c\x42\x29\x32\x5d\x88"
			 "\xed\xce\x0\x19\xe\x16\x2\x6e"
			 "\x87\xff\x2c\xac\xe8\xe7\xe9\xbf"
			 "\x31\x51\xec\x47\xc3\x51\x83\xc1",
		.ctext = "\xb1\x98\xf2\xde\x3f\x4b\xae\xd1"
			 "\xf0\xf1\x30\x4c\x1\x27\x5a\x8f"
			 "\x45\xe1\x39\xb7\xae\xff\x1f\x27"
			 "\xad\x57\x15\xab\x31\x5d\xc\xef"
			 "\x8c\xc8\x80\xbd\x11\x98\xf3\x7b"
			 "\xa2\xdd\x14\x20\xf9\xe8\xbb\x82"
			 "\xf7\x32\xca\x4b\xa8\xf7\xb3\x4d"
			 "\x27\xd1\xcd\xe6\xb6\x65\x5a\x23"
			 "\xc2\xf3\x54\x84\x53\xe3\xb9\x20"
			 "\xa5\x37\x0\xbe\xe7\x7b\x48\xfb"
			 "\x21\x3d\x9e\x48\x1d\x9e\xf5\xbf"
			 "\x77\xd5\xb4\x4a\x53\x71\x94\x7a"
			 "\x88\xa6\x6e\x6\x93\xca\x43\xa5"
			 "\xc4\xf6\xcd\x53\x4b\x7b\x8e\xfe"
			 "\xb4\x28\x7c\x42\x29\x32\x5d\x88"
			 "\xed\xce\x0\x19\xe\x16\x2\x6e"
			 "\x87\xff\x2c\xac\xe8\xe7\xe9\xbf"
			 "\x31\x51\xec\x47\xc3\x51\x83\xc1"
			 "\x59\x52\x98\xc7\xc6\xfd\x27\x1f"
			 "\x4\x2\xf8\x4\xc3\x3d\x3f\x66",
	},
	{
		/* A.2.1.1 SM4-ECB Example 1 */
		.klen = 16,
		.len = 32,
		.ilen = 0,
		.alg = SCE_SM4,
		.mode = ECB,
		.key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
		       "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
			 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
			 "\xee\xee\xee\xee\xff\xff\xff\xff"
			 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
		.ctext = "\x5e\xc8\x14\x3d\xe5\x09\xcf\xf7"
			 "\xb5\x17\x9f\x8f\x47\x4b\x86\x19"
			 "\x2f\x1d\x30\x5a\x7f\xb1\x7d\xf9"
			 "\x85\xf8\x1c\x84\x82\x19\x23\x04",
	},
	{
		/* A.2.1.2 SM4-ECB Example 2 */
		.klen = 16,
		.len = 32,
		.ilen = 0,
		.alg = SCE_SM4,
		.mode = ECB,
		.key = "\xFE\xDC\xBA\x98\x76\x54\x32\x10"
		       "\x01\x23\x45\x67\x89\xAB\xCD\xEF",
		.ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
			 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
			 "\xee\xee\xee\xee\xff\xff\xff\xff"
			 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
		.ctext = "\xC5\x87\x68\x97\xE4\xA5\x9B\xBB"
			 "\xA7\x2A\x10\xC8\x38\x72\x24\x5B"
			 "\x12\xDD\x90\xBC\x2D\x20\x06\x92"
			 "\xB5\x29\xA4\x15\x5A\xC9\xE6\x00",
	},

	{
		/* A.2.2.1 SM4-CBC Example 1 */
		.klen = 16,
		.len = 32,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CBC,
		.key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
		       "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
			 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
			 "\xee\xee\xee\xee\xff\xff\xff\xff"
			 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
		.ctext = "\x78\xEB\xB1\x1C\xC4\x0B\x0A\x48"
			 "\x31\x2A\xAE\xB2\x04\x02\x44\xCB"
			 "\x4C\xB7\x01\x69\x51\x90\x92\x26"
			 "\x97\x9B\x0D\x15\xDC\x6A\x8F\x6D",
	},
	{
		/* A.2.2.2 SM4-CBC Example 2 */
		.klen = 16,
		.len = 32,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CBC,
		.key = "\xFE\xDC\xBA\x98\x76\x54\x32\x10"
		       "\x01\x23\x45\x67\x89\xAB\xCD\xEF",
		.ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
			 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
			 "\xee\xee\xee\xee\xff\xff\xff\xff"
			 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
		.ctext = "\x0d\x3a\x6d\xdc\x2d\x21\xc6\x98"
			 "\x85\x72\x15\x58\x7b\x7b\xb5\x9a"
			 "\x91\xf2\xc1\x47\x91\x1a\x41\x44"
			 "\x66\x5e\x1f\xa1\xd4\x0b\xae\x38",
	},

	{
		/* A.2.5.1 SM4-CTR Example 1 */
		.klen = 16,
		.len = 64,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CTR,
		.key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
		       "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.ptext = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
			 "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb"
			 "\xcc\xcc\xcc\xcc\xcc\xcc\xcc\xcc"
			 "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			 "\xee\xee\xee\xee\xee\xee\xee\xee"
			 "\xff\xff\xff\xff\xff\xff\xff\xff"
			 "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
			 "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
		.ctext = "\xac\x32\x36\xcb\x97\x0c\xc2\x07"
			 "\x91\x36\x4c\x39\x5a\x13\x42\xd1"
			 "\xa3\xcb\xc1\x87\x8c\x6f\x30\xcd"
			 "\x07\x4c\xce\x38\x5c\xdd\x70\xc7"
			 "\xf2\x34\xbc\x0e\x24\xc1\x19\x80"
			 "\xfd\x12\x86\x31\x0c\xe3\x7b\x92"
			 "\x6e\x02\xfc\xd0\xfa\xa0\xba\xf3"
			 "\x8b\x29\x33\x85\x1d\x82\x45\x14",
	},
	{
		/* A.2.5.2 SM4-CTR Example 2 */
		.klen = 16,
		.len = 64,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CTR,
		.key = "\xFE\xDC\xBA\x98\x76\x54\x32\x10"
		       "\x01\x23\x45\x67\x89\xAB\xCD\xEF",
		.ptext = "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
			 "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb"
			 "\xcc\xcc\xcc\xcc\xcc\xcc\xcc\xcc"
			 "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd"
			 "\xee\xee\xee\xee\xee\xee\xee\xee"
			 "\xff\xff\xff\xff\xff\xff\xff\xff"
			 "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
			 "\xbb\xbb\xbb\xbb\xbb\xbb\xbb\xbb",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
		.ctext = "\x5d\xcc\xcd\x25\xb9\x5a\xb0\x74"
			 "\x17\xa0\x85\x12\xee\x16\x0e\x2f"
			 "\x8f\x66\x15\x21\xcb\xba\xb4\x4c"
			 "\xc8\x71\x38\x44\x5b\xc2\x9e\x5c"
			 "\x0a\xe0\x29\x72\x05\xd6\x27\x04"
			 "\x17\x3b\x21\x23\x9b\x88\x7f\x6c"
			 "\x8c\xb5\xb8\x00\x91\x7a\x24\x88"
			 "\x28\x4b\xde\x9e\x16\xea\x29\x06",
	},

	{
		/* From: draft-ribose-cfrg-SCE_SM4-02, paragraph 12.2.3 */
		.klen = 16,
		.len = 32,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = OFB,
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef"
		       "\xfe\xdc\xba\x98\x76\x54\x32\x10",
		.iv = "\x01\x23\x45\x67\x89\xab\xcd\xef"
		      "\xfe\xdc\xba\x98\x76\x54\x32\x10",
		.ptext = "\x01\x23\x45\x67\x89\xab\xcd\xef"
			 "\xfe\xdc\xba\x98\x76\x54\x32\x10"
			 "\x01\x23\x45\x67\x89\xab\xcd\xef"
			 "\xfe\xdc\xba\x98\x76\x54\x32\x10",
		.ctext = "\x69\x3d\x9a\x53\x5b\xad\x5b\xb1"
			 "\x78\x6f\x53\xd7\x25\x3a\x70\x56"
			 "\xf2\x07\x5d\x28\xb5\x23\x5f\x58"
			 "\xd5\x00\x27\xe4\x17\x7d\x2b\xce",
	},
	{
		/* From: draft-ribose-cfrg-sm4-09, appendix A.2.3, Example 1 */
		.klen = 16,
		.len = 32,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = OFB,
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef"
		       "\xfe\xdc\xba\x98\x76\x54\x32\x10",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
			 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
			 "\xee\xee\xee\xee\xff\xff\xff\xff"
			 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
		.ctext = "\xac\x32\x36\xcb\x86\x1d\xd3\x16"
			 "\xe6\x41\x3b\x4e\x3c\x75\x24\xb7"
			 "\x1d\x01\xac\xa2\x48\x7c\xa5\x82"
			 "\xcb\xf5\x46\x3e\x66\x98\x53\x9b",
	},
	{
		/* From: draft-ribose-cfrg-sm4-09, appendix A.2.3, Example 2 */
		.klen = 16,
		.len = 30,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = OFB,
		.key = "\xfe\xdc\xba\x98\x76\x54\x32\x10"
		       "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext = "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb"
			 "\xcc\xcc\xcc\xcc\xdd\xdd\xdd\xdd"
			 "\xee\xee\xee\xee\xff\xff\xff\xff"
			 "\xaa\xaa\xaa\xaa\xbb\xbb\xbb\xbb",
		.ctext = "\x5d\xcc\xcd\x25\xa8\x4b\xa1\x65"
			 "\x60\xd7\xf2\x65\x88\x70\x68\x49"
			 "\x33\xfa\x16\xbd\x5c\xd9\xc8\x56"
			 "\xca\xca\xa1\xe1\x01\x89\x7a\x97",
	},

	{
		.klen = 16,
		.len = 90,
		.ilen = 16,
		.alg = SCE_SM4,
		.mode = CTR,
		.key = "\x01\x23\x45\x67\x89\xAB\xCD\xEF"
		       "\xFE\xDC\xBA\x98\x76\x54\x32\x10",
		.iv = "\x00\x00\x12\x34\x56\x78\x00\x00"
		      "\x00\x00\xAB\xCD\x00\x00\x00\x02",
		.ptext = "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
			 "\xBB\xBB\xBB\xBB\xBB\xBB\xBB\xBB"
			 "\xCC\xCC\xCC\xCC\xCC\xCC\xCC\xCC"
			 "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
			 "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
			 "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"
			 "\xEE\xEE\xEE\xEE\xEE\xEE\xEE\xEE"
			 "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
			 "\xCC\xCC\xCC\xCC\xCC\xCC\xCC\xCC"
			 "\xBB\xBB\xBB\xBB\xBB\xBB\xBB\xBB"
			 "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
			 "\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA",
		.ctext = "\x17\xf3\x99\xf0\x8c\x67\xd5\xee"
			 "\x19\xd0\xdc\x99\x69\xc4\xbb\x7d"
			 "\x5f\xd4\x6f\xd3\x75\x64\x89\x06"
			 "\x91\x57\xb2\x82\xbb\x20\x07\x35"
			 "\xd8\x27\x10\xca\x5c\x22\xf0\xcc"
			 "\xfa\x7c\xbf\x93\xd4\x96\xac\x15"
			 "\xa5\x68\x34\xcb\xcf\x98\xc3\x97"
			 "\xc3\x75\x3d\x51\xe6\x54\x4c\xfa"
			 "\xe2\x09\x61\x87\x79\x0d\x3c\xd2"
			 "\xa7\x51\x56\x3e\xe0\xc0\xea\x31"
			 "\xff\xab\xf9\xba\xc6\xe4\x90\xb6"
			 "\x5c\x66\x01\xee\x1c\x0f\x45\x5e",
	},

	{
		/* From FIPS-197 */
		.klen = 16,
		.len = 16,
		.alg = SCE_AES_128,
		.mode = ECB,
		.key = "\x00\x01\x02\x03\x04\x05\x06\x07"
		       "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext = "\x00\x11\x22\x33\x44\x55\x66\x77"
			 "\x88\x99\xaa\xbb\xcc\xdd\xee\xff",
		.ctext = "\x69\xc4\xe0\xd8\x6a\x7b\x04\x30"
			 "\xd8\xcd\xb7\x80\x70\xb4\xc5\x5a",
	},
	{
		.klen = 24,
		.len = 16,
		.alg = SCE_AES_192,
		.mode = ECB,
		.key = "\x00\x01\x02\x03\x04\x05\x06\x07"
		       "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
		       "\x10\x11\x12\x13\x14\x15\x16\x17",
		.ptext = "\x00\x11\x22\x33\x44\x55\x66\x77"
			 "\x88\x99\xaa\xbb\xcc\xdd\xee\xff",
		.ctext = "\xdd\xa9\x7c\xa4\x86\x4c\xdf\xe0"
			 "\x6e\xaf\x70\xa0\xec\x0d\x71\x91",
	},
	{
		.klen = 32,
		.len = 16,
		.alg = SCE_AES_256,
		.mode = ECB,
		.key = "\x00\x01\x02\x03\x04\x05\x06\x07"
		       "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
		       "\x10\x11\x12\x13\x14\x15\x16\x17"
		       "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f",
		.ptext = "\x00\x11\x22\x33\x44\x55\x66\x77"
			 "\x88\x99\xaa\xbb\xcc\xdd\xee\xff",
		.ctext = "\x8e\xa2\xb7\xca\x51\x67\x45\xbf"
			 "\xea\xfc\x49\x90\x4b\x49\x60\x89",
	},
	{
		/* Generated with Crypto++ */
		.klen = 32,
		.len = 496,
		.alg = SCE_AES_256,
		.mode = ECB,
		.key = "\xA6\xC9\x83\xA6\xC9\xEC\x0F\x32"
		       "\x55\x0F\x32\x55\x78\x9B\xBE\x78"
		       "\x9B\xBE\xE1\x04\x27\xE1\x04\x27"
		       "\x4A\x6D\x90\x4A\x6D\x90\xB3\xD6",
		.ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
			 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
			 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
			 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
			 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
			 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
			 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
			 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
			 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
			 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
			 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
			 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
			 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
			 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
			 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
			 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
			 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
			 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
			 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
			 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
			 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
			 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
			 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
			 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
			 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
			 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
			 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
			 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
			 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
			 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
			 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB"
			 "\x54\xE0\x49\xB2\x1B\xA7\x10\x79"
			 "\x05\x6E\xD7\x40\xCC\x35\x9E\x07"
			 "\x93\xFC\x65\xF1\x5A\xC3\x2C\xB8"
			 "\x21\x8A\x16\x7F\xE8\x51\xDD\x46"
			 "\xAF\x18\xA4\x0D\x76\x02\x6B\xD4"
			 "\x3D\xC9\x32\x9B\x04\x90\xF9\x62"
			 "\xEE\x57\xC0\x29\xB5\x1E\x87\x13"
			 "\x7C\xE5\x4E\xDA\x43\xAC\x15\xA1"
			 "\x0A\x73\xFF\x68\xD1\x3A\xC6\x2F"
			 "\x98\x01\x8D\xF6\x5F\xEB\x54\xBD"
			 "\x26\xB2\x1B\x84\x10\x79\xE2\x4B"
			 "\xD7\x40\xA9\x12\x9E\x07\x70\xFC"
			 "\x65\xCE\x37\xC3\x2C\x95\x21\x8A"
			 "\xF3\x5C\xE8\x51\xBA\x23\xAF\x18"
			 "\x81\x0D\x76\xDF\x48\xD4\x3D\xA6"
			 "\x0F\x9B\x04\x6D\xF9\x62\xCB\x34"
			 "\xC0\x29\x92\x1E\x87\xF0\x59\xE5"
			 "\x4E\xB7\x20\xAC\x15\x7E\x0A\x73"
			 "\xDC\x45\xD1\x3A\xA3\x0C\x98\x01"
			 "\x6A\xF6\x5F\xC8\x31\xBD\x26\x8F"
			 "\x1B\x84\xED\x56\xE2\x4B\xB4\x1D"
			 "\xA9\x12\x7B\x07\x70\xD9\x42\xCE"
			 "\x37\xA0\x09\x95\xFE\x67\xF3\x5C"
			 "\xC5\x2E\xBA\x23\x8C\x18\x81\xEA"
			 "\x53\xDF\x48\xB1\x1A\xA6\x0F\x78"
			 "\x04\x6D\xD6\x3F\xCB\x34\x9D\x06"
			 "\x92\xFB\x64\xF0\x59\xC2\x2B\xB7"
			 "\x20\x89\x15\x7E\xE7\x50\xDC\x45"
			 "\xAE\x17\xA3\x0C\x75\x01\x6A\xD3"
			 "\x3C\xC8\x31\x9A\x03\x8F\xF8\x61"
			 "\xED\x56\xBF\x28\xB4\x1D\x86\x12",
		.ctext = "\x71\x73\xF7\xDB\x24\x93\x21\x6D"
			 "\x61\x1E\xBB\x63\x42\x79\xDB\x64"
			 "\x6F\x82\xC0\xCA\xA3\x9B\xFA\x0B"
			 "\xD9\x08\xC7\x4A\x90\xAE\x8F\x5F"
			 "\x5E\x06\xF0\x5F\x31\x51\x18\x37"
			 "\x45\xD7\xCA\x3A\xFD\x6C\x3F\xE1"
			 "\xDD\x8D\x22\x65\x2B\x00\x50\xCE"
			 "\xBA\x28\x67\xD7\xCE\x0E\x0D\xEA"
			 "\x78\x69\x7F\xAE\x8F\x8B\x69\x37"
			 "\x75\xE0\xDC\x96\xE0\xB7\xF4\x09"
			 "\xCB\x6D\xA2\xFB\xDA\xAF\x09\xF8"
			 "\x81\x82\x27\xFA\x45\x9C\x29\xA4"
			 "\x22\x8B\x78\x69\x5B\x46\xF9\x39"
			 "\x1B\xCC\xF9\x1D\x09\xEB\xBC\x5C"
			 "\x41\x72\x51\x97\x1D\x07\x49\xA0"
			 "\x1B\x8E\x65\x4B\xB2\x6A\x12\x03"
			 "\x6A\x60\x95\xAC\xBD\xAC\x1A\x64"
			 "\xDE\x5A\xA5\xF0\x83\x2F\xCB\xCA"
			 "\x22\x74\xA6\x6C\x9B\x73\xCE\x3F"
			 "\xE1\x8B\x22\x17\x59\x0C\x47\x89"
			 "\x33\xA1\xD6\x47\x03\x19\x4F\xA8"
			 "\x67\x69\xF0\x5B\xF0\x20\xAD\x06"
			 "\x27\x81\x92\xD8\xC5\xBA\x98\x12"
			 "\xBE\x24\xB5\x2F\x75\x02\xC2\xAD"
			 "\x12\x2F\x07\x32\xEE\x39\xAF\x64"
			 "\x05\x8F\xB3\xD4\xEB\x1B\x46\x6E"
			 "\xD9\x21\xF9\xC4\xB7\xC9\x45\x68"
			 "\xB4\xA1\x74\x9F\x82\x47\xEB\xCC"
			 "\xBD\x0A\x14\x95\x0F\x8B\xA8\x2F"
			 "\x4B\x1B\xA7\xBF\x82\xA6\x43\x0C"
			 "\xB9\x39\x4A\xA8\x10\x6F\x50\x7B"
			 "\x25\xFB\x26\x81\xE0\x2F\xF0\x96"
			 "\x8D\x8B\xAC\x92\x0F\xF6\xED\x64"
			 "\x63\x29\x4C\x8E\x18\x13\xC5\xBF"
			 "\xFC\xA0\xD9\xBF\x7C\x3A\x0E\x29"
			 "\x6F\xD1\x6C\x6F\xA5\xDA\xBF\xB1"
			 "\x30\xEA\x44\x2D\xC3\x8F\x16\xE1"
			 "\x66\xFA\xA3\x21\x3E\xFC\x13\xCA"
			 "\xF0\xF6\xF0\x59\xBD\x8F\x38\x50"
			 "\x31\xCB\x69\x3F\x96\x15\xD6\xF5"
			 "\xAE\xFF\xF6\xAA\x41\x85\x4C\x10"
			 "\x58\xE3\xF9\x44\xE6\x28\xDA\x9A"
			 "\xDC\x6A\x80\x34\x73\x97\x1B\xC5"
			 "\xCA\x26\x16\x77\x0E\x60\xAB\x89"
			 "\x0F\x04\x27\xBD\xCE\x3E\x71\xB4"
			 "\xA0\xD7\x22\x7E\xDB\xEB\x24\x70"
			 "\x42\x71\x51\x78\x70\xB3\xE0\x3D"
			 "\x84\x8E\x8D\x7B\xD0\x6D\xEA\x92"
			 "\x11\x08\x42\x4F\xE5\xAD\x26\x92"
			 "\xD2\x00\xAE\xA8\xE3\x4B\x37\x47"
			 "\x22\xC1\x95\xC1\x63\x7F\xCB\x03"
			 "\xF3\xE3\xD7\x9D\x60\xC7\xBC\xEA"
			 "\x35\xA2\xFD\x45\x52\x39\x13\x6F"
			 "\xC1\x53\xF3\x53\xDF\x33\x84\xD7"
			 "\xD2\xC8\x37\xB0\x75\xE3\x41\x46"
			 "\xB3\xC7\x83\x2E\x8A\xBB\xA4\xE5"
			 "\x7F\x3C\xFD\x8B\xEB\xEA\x63\xBD"
			 "\xB7\x46\xE7\xBF\x09\x9C\x0D\x0F"
			 "\x40\x86\x7F\x51\xE1\x11\x9C\xCB"
			 "\x88\xE6\x68\x47\xE3\x2B\xC5\xFF"
			 "\x09\x79\xA0\x43\x5C\x0D\x08\x58"
			 "\x17\xBB\xC0\x6B\x62\x3F\x56\xE9",
	},

	{
		/* From RFC 3602 */
		.klen = 16,
		.len = 16,
		.ilen = 16,
		.alg = SCE_AES_128,
		.mode = CBC,
		.key = "\x06\xa9\x21\x40\x36\xb8\xa1\x5b"
		       "\x51\x2e\x03\xd5\x34\x12\x00\x06",
		.iv = "\x3d\xaf\xba\x42\x9d\x9e\xb4\x30"
		      "\xb4\x22\xda\x80\x2c\x9f\xac\x41",
		.ptext = "Single block msg",
		.ctext = "\xe3\x53\x77\x9c\x10\x79\xae\xb8"
			 "\x27\x08\x94\x2d\xbe\x77\x18\x1a",
	},
	{
		.klen = 16,
		.len = 32,
		.ilen = 16,
		.alg = SCE_AES_128,
		.mode = CBC,
		.key = "\xc2\x86\x69\x6d\x88\x7c\x9a\xa0"
		       "\x61\x1b\xbb\x3e\x20\x25\xa4\x5a",
		.iv = "\x56\x2e\x17\x99\x6d\x09\x3d\x28"
		      "\xdd\xb3\xba\x69\x5a\x2e\x6f\x58",
		.ptext = "\x00\x01\x02\x03\x04\x05\x06\x07"
			 "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
			 "\x10\x11\x12\x13\x14\x15\x16\x17"
			 "\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f",
		.ctext = "\xd2\x96\xcd\x94\xc2\xcc\xcf\x8a"
			 "\x3a\x86\x30\x28\xb5\xe1\xdc\x0a"
			 "\x75\x86\x60\x2d\x25\x3c\xff\xf9"
			 "\x1b\x82\x66\xbe\xa6\xd6\x1a\xb1",
	},
	{
		/* From NIST SP800-38A */
		.klen = 24,
		.len = 64,
		.ilen = 16,
		.alg = SCE_AES_192,
		.mode = CBC,
		.key = "\x8e\x73\xb0\xf7\xda\x0e\x64\x52"
		       "\xc8\x10\xf3\x2b\x80\x90\x79\xe5"
		       "\x62\xf8\xea\xd2\x52\x2c\x6b\x7b",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
			 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
			 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
			 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
		.ctext = "\x4f\x02\x1d\xb2\x43\xbc\x63\x3d"
			 "\x71\x78\x18\x3a\x9f\xa0\x71\xe8"
			 "\xb4\xd9\xad\xa9\xad\x7d\xed\xf4"
			 "\xe5\xe7\x38\x76\x3f\x69\x14\x5a"
			 "\x57\x1b\x24\x20\x12\xfb\x7a\xe0"
			 "\x7f\xa9\xba\xac\x3d\xf1\x02\xe0"
			 "\x08\xb0\xe2\x79\x88\x59\x88\x81"
			 "\xd9\x20\xa9\xe6\x4f\x56\x15\xcd",
	},
	{
		.klen = 32,
		.len = 64,
		.ilen = 16,
		.alg = SCE_AES_256,
		.mode = CBC,
		.key = "\x60\x3d\xeb\x10\x15\xca\x71\xbe"
		       "\x2b\x73\xae\xf0\x85\x7d\x77\x81"
		       "\x1f\x35\x2c\x07\x3b\x61\x08\xd7"
		       "\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
			 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
			 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
			 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
		.ctext = "\xf5\x8c\x4c\x04\xd6\xe5\xf1\xba"
			 "\x77\x9e\xab\xfb\x5f\x7b\xfb\xd6"
			 "\x9c\xfc\x4e\x96\x7e\xdb\x80\x8d"
			 "\x67\x9f\x77\x7b\xc6\x70\x2c\x7d"
			 "\x39\xf2\x33\x69\xa9\xd9\xba\xcf"
			 "\xa5\x30\xe2\x63\x04\x23\x14\x61"
			 "\xb2\xeb\x05\xe2\xc3\x9b\xe9\xfc"
			 "\xda\x6c\x19\x07\x8c\x6a\x9d\x1b",
	},
	{
		/* Generated with Crypto++ */
		.klen = 32,
		.len = 496,
		.ilen = 16,
		.alg = SCE_AES_256,
		.mode = CBC,
		.key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55"
		       "\x0F\x32\x55\x78\x9B\xBE\x78\x9B"
		       "\xBE\xE1\x04\x27\xE1\x04\x27\x4A"
		       "\x6D\x90\x4A\x6D\x90\xB3\xD6\xF9",
		.iv = "\xE7\x82\x1D\xB8\x53\x11\xAC\x47"
		      "\xE2\x7D\x18\xD6\x71\x0C\xA7\x42",
		.ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
			 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
			 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
			 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
			 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
			 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
			 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
			 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
			 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
			 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
			 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
			 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
			 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
			 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
			 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
			 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
			 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
			 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
			 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
			 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
			 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
			 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
			 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
			 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
			 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
			 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
			 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
			 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
			 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
			 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
			 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB"
			 "\x54\xE0\x49\xB2\x1B\xA7\x10\x79"
			 "\x05\x6E\xD7\x40\xCC\x35\x9E\x07"
			 "\x93\xFC\x65\xF1\x5A\xC3\x2C\xB8"
			 "\x21\x8A\x16\x7F\xE8\x51\xDD\x46"
			 "\xAF\x18\xA4\x0D\x76\x02\x6B\xD4"
			 "\x3D\xC9\x32\x9B\x04\x90\xF9\x62"
			 "\xEE\x57\xC0\x29\xB5\x1E\x87\x13"
			 "\x7C\xE5\x4E\xDA\x43\xAC\x15\xA1"
			 "\x0A\x73\xFF\x68\xD1\x3A\xC6\x2F"
			 "\x98\x01\x8D\xF6\x5F\xEB\x54\xBD"
			 "\x26\xB2\x1B\x84\x10\x79\xE2\x4B"
			 "\xD7\x40\xA9\x12\x9E\x07\x70\xFC"
			 "\x65\xCE\x37\xC3\x2C\x95\x21\x8A"
			 "\xF3\x5C\xE8\x51\xBA\x23\xAF\x18"
			 "\x81\x0D\x76\xDF\x48\xD4\x3D\xA6"
			 "\x0F\x9B\x04\x6D\xF9\x62\xCB\x34"
			 "\xC0\x29\x92\x1E\x87\xF0\x59\xE5"
			 "\x4E\xB7\x20\xAC\x15\x7E\x0A\x73"
			 "\xDC\x45\xD1\x3A\xA3\x0C\x98\x01"
			 "\x6A\xF6\x5F\xC8\x31\xBD\x26\x8F"
			 "\x1B\x84\xED\x56\xE2\x4B\xB4\x1D"
			 "\xA9\x12\x7B\x07\x70\xD9\x42\xCE"
			 "\x37\xA0\x09\x95\xFE\x67\xF3\x5C"
			 "\xC5\x2E\xBA\x23\x8C\x18\x81\xEA"
			 "\x53\xDF\x48\xB1\x1A\xA6\x0F\x78"
			 "\x04\x6D\xD6\x3F\xCB\x34\x9D\x06"
			 "\x92\xFB\x64\xF0\x59\xC2\x2B\xB7"
			 "\x20\x89\x15\x7E\xE7\x50\xDC\x45"
			 "\xAE\x17\xA3\x0C\x75\x01\x6A\xD3"
			 "\x3C\xC8\x31\x9A\x03\x8F\xF8\x61"
			 "\xED\x56\xBF\x28\xB4\x1D\x86\x12",
		.ctext = "\xEA\x65\x8A\x19\xB0\x66\xC1\x3F"
			 "\xCE\xF1\x97\x75\xC1\xFD\xB5\xAF"
			 "\x52\x65\xF7\xFF\xBC\xD8\x2D\x9F"
			 "\x2F\xB9\x26\x9B\x6F\x10\xB7\xB8"
			 "\x26\xA1\x02\x46\xA2\xAD\xC6\xC0"
			 "\x11\x15\xFF\x6D\x1E\x82\x04\xA6"
			 "\xB1\x74\xD1\x08\x13\xFD\x90\x7C"
			 "\xF5\xED\xD3\xDB\x5A\x0A\x0C\x2F"
			 "\x0A\x70\xF1\x88\x07\xCF\x21\x26"
			 "\x40\x40\x8A\xF5\x53\xF7\x24\x4F"
			 "\x83\x38\x43\x5F\x08\x99\xEB\xE3"
			 "\xDC\x02\x64\x67\x50\x6E\x15\xC3"
			 "\x01\x1A\xA0\x81\x13\x65\xA6\x73"
			 "\x71\xA6\x3B\x91\x83\x77\xBE\xFA"
			 "\xDB\x71\x73\xA6\xC1\xAE\x43\xC3"
			 "\x36\xCE\xD6\xEB\xF9\x30\x1C\x4F"
			 "\x80\x38\x5E\x9C\x6E\xAB\x98\x2F"
			 "\x53\xAF\xCF\xC8\x9A\xB8\x86\x43"
			 "\x3E\x86\xE7\xA1\xF4\x2F\x30\x40"
			 "\x03\xA8\x6C\x50\x42\x9F\x77\x59"
			 "\x89\xA0\xC5\xEC\x9A\xB8\xDD\x99"
			 "\x16\x24\x02\x07\x48\xAE\xF2\x31"
			 "\x34\x0E\xC3\x85\xFE\x1C\x95\x99"
			 "\x87\x58\x98\x8B\xE7\xC6\xC5\x70"
			 "\x73\x81\x07\x7C\x56\x2F\xD8\x1B"
			 "\xB7\xB9\x2B\xAB\xE3\x01\x87\x0F"
			 "\xD8\xBB\xC0\x0D\xAC\x2C\x2F\x98"
			 "\x3C\x0B\xA2\x99\x4A\x8C\xF7\x04"
			 "\xE0\xE0\xCF\xD1\x81\x5B\xFE\xF5"
			 "\x24\x04\xFD\xB8\xDF\x13\xD8\xCD"
			 "\xF1\xE3\x3D\x98\x50\x02\x77\x9E"
			 "\xBC\x22\xAB\xFA\xC2\x43\x1F\x66"
			 "\x20\x02\x23\xDA\xDF\xA0\x89\xF6"
			 "\xD8\xF3\x45\x24\x53\x6F\x16\x77"
			 "\x02\x3E\x7B\x36\x5F\xA0\x3B\x78"
			 "\x63\xA2\xBD\xB5\xA4\xCA\x1E\xD3"
			 "\x57\xBC\x0B\x9F\x43\x51\x28\x4F"
			 "\x07\x50\x6C\x68\x12\x07\xCF\xFA"
			 "\x6B\x72\x0B\xEB\xF8\x88\x90\x2C"
			 "\x7E\xF5\x91\xD1\x03\xD8\xD5\xBD"
			 "\x22\x39\x7B\x16\x03\x01\x69\xAF"
			 "\x3D\x38\x66\x28\x0C\xBE\x5B\xC5"
			 "\x03\xB4\x2F\x51\x8A\x56\x17\x2B"
			 "\x88\x42\x6D\x40\x68\x8F\xD0\x11"
			 "\x19\xF9\x1F\x43\x79\x95\x31\xFA"
			 "\x28\x7A\x3D\xF7\x66\xEB\xEF\xAC"
			 "\x06\xB2\x01\xAD\xDB\x68\xDB\xEC"
			 "\x8D\x53\x6E\x72\x68\xA3\xC7\x63"
			 "\x43\x2B\x78\xE0\x04\x29\x8F\x72"
			 "\xB2\x2C\xE6\x84\x03\x30\x6D\xCD"
			 "\x26\x92\x37\xE1\x2F\xBB\x8B\x9D"
			 "\xE4\x4C\xF6\x93\xBC\xD9\xAD\x44"
			 "\x52\x65\xC7\xB0\x0E\x3F\x0E\x61"
			 "\x56\x5D\x1C\x6D\xA7\x05\x2E\xBC"
			 "\x58\x08\x15\xAB\x12\xAB\x17\x4A"
			 "\x5E\x1C\xF2\xCD\xB8\xA2\xAE\xFB"
			 "\x9B\x2E\x0E\x85\x34\x80\x0E\x3F"
			 "\x4C\xB8\xDB\xCE\x1C\x90\xA1\x61"
			 "\x6C\x69\x09\x35\x9E\xD4\xF4\xAD"
			 "\xBC\x06\x41\xE3\x01\xB4\x4E\x0A"
			 "\xE0\x1F\x91\xF8\x82\x96\x2D\x65"
			 "\xA3\xAA\x13\xCC\x50\xFF\x7B\x02",
	},

	{
		/* From NIST Special Publication 800-38A, Appendix F.5 */
		.klen = 16,
		.len = 64,
		.ilen = 16,
		.alg = SCE_AES_128,
		.mode = CTR,
		.key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
		       "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
		.iv = "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
		      "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
			 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
			 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
			 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
		.ctext = "\x87\x4d\x61\x91\xb6\x20\xe3\x26"
			 "\x1b\xef\x68\x64\x99\x0d\xb6\xce"
			 "\x98\x06\xf6\x6b\x79\x70\xfd\xff"
			 "\x86\x17\x18\x7b\xb9\xff\xfd\xff"
			 "\x5a\xe4\xdf\x3e\xdb\xd5\xd3\x5e"
			 "\x5b\x4f\x09\x02\x0d\xb0\x3e\xab"
			 "\x1e\x03\x1d\xda\x2f\xbe\x03\xd1"
			 "\x79\x21\x70\xa0\xf3\x00\x9c\xee",
	},
	{
		.klen = 24,
		.len = 64,
		.ilen = 16,
		.alg = SCE_AES_192,
		.mode = CTR,
		.key = "\x8e\x73\xb0\xf7\xda\x0e\x64\x52"
		       "\xc8\x10\xf3\x2b\x80\x90\x79\xe5"
		       "\x62\xf8\xea\xd2\x52\x2c\x6b\x7b",
		.iv = "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
		      "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
			 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
			 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
			 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
		.ctext = "\x1a\xbc\x93\x24\x17\x52\x1c\xa2"
			 "\x4f\x2b\x04\x59\xfe\x7e\x6e\x0b"
			 "\x09\x03\x39\xec\x0a\xa6\xfa\xef"
			 "\xd5\xcc\xc2\xc6\xf4\xce\x8e\x94"
			 "\x1e\x36\xb2\x6b\xd1\xeb\xc6\x70"
			 "\xd1\xbd\x1d\x66\x56\x20\xab\xf7"
			 "\x4f\x78\xa7\xf6\xd2\x98\x09\x58"
			 "\x5a\x97\xda\xec\x58\xc6\xb0\x50",
	},
	{
		.klen = 32,
		.len = 64,
		.ilen = 16,
		.alg = SCE_AES_256,
		.mode = CTR,
		.key = "\x60\x3d\xeb\x10\x15\xca\x71\xbe"
		       "\x2b\x73\xae\xf0\x85\x7d\x77\x81"
		       "\x1f\x35\x2c\x07\x3b\x61\x08\xd7"
		       "\x2d\x98\x10\xa3\x09\x14\xdf\xf4",
		.iv = "\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7"
		      "\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
			 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
			 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
			 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
		.ctext = "\x60\x1e\xc3\x13\x77\x57\x89\xa5"
			 "\xb7\xa7\xf5\x04\xbb\xf3\xd2\x28"
			 "\xf4\x43\xe3\xca\x4d\x62\xb5\x9a"
			 "\xca\x84\xe9\x90\xca\xca\xf5\xc5"
			 "\x2b\x09\x30\xda\xa2\x3d\xe9\x4c"
			 "\xe8\x70\x17\xba\x2d\x84\x98\x8d"
			 "\xdf\xc9\xc5\x8d\xb6\x7a\xad\xa6"
			 "\x13\xc2\xdd\x08\x45\x79\x41\xa6",
	},
	{
		/* Generated with Crypto++ */
		.klen = 32,
		.len = 496,
		.ilen = 16,
		.alg = SCE_AES_256,
		.mode = CTR,
		.key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55"
		       "\x0F\x32\x55\x78\x9B\xBE\x78\x9B"
		       "\xBE\xE1\x04\x27\xE1\x04\x27\x4A"
		       "\x6D\x90\x4A\x6D\x90\xB3\xD6\xF9",
		.iv = "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF"
		      "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFD",
		.ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
			 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
			 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
			 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
			 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
			 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
			 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
			 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
			 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
			 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
			 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
			 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
			 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
			 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
			 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
			 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
			 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
			 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
			 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
			 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
			 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
			 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
			 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
			 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
			 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
			 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
			 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
			 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
			 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
			 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
			 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB"
			 "\x54\xE0\x49\xB2\x1B\xA7\x10\x79"
			 "\x05\x6E\xD7\x40\xCC\x35\x9E\x07"
			 "\x93\xFC\x65\xF1\x5A\xC3\x2C\xB8"
			 "\x21\x8A\x16\x7F\xE8\x51\xDD\x46"
			 "\xAF\x18\xA4\x0D\x76\x02\x6B\xD4"
			 "\x3D\xC9\x32\x9B\x04\x90\xF9\x62"
			 "\xEE\x57\xC0\x29\xB5\x1E\x87\x13"
			 "\x7C\xE5\x4E\xDA\x43\xAC\x15\xA1"
			 "\x0A\x73\xFF\x68\xD1\x3A\xC6\x2F"
			 "\x98\x01\x8D\xF6\x5F\xEB\x54\xBD"
			 "\x26\xB2\x1B\x84\x10\x79\xE2\x4B"
			 "\xD7\x40\xA9\x12\x9E\x07\x70\xFC"
			 "\x65\xCE\x37\xC3\x2C\x95\x21\x8A"
			 "\xF3\x5C\xE8\x51\xBA\x23\xAF\x18"
			 "\x81\x0D\x76\xDF\x48\xD4\x3D\xA6"
			 "\x0F\x9B\x04\x6D\xF9\x62\xCB\x34"
			 "\xC0\x29\x92\x1E\x87\xF0\x59\xE5"
			 "\x4E\xB7\x20\xAC\x15\x7E\x0A\x73"
			 "\xDC\x45\xD1\x3A\xA3\x0C\x98\x01"
			 "\x6A\xF6\x5F\xC8\x31\xBD\x26\x8F"
			 "\x1B\x84\xED\x56\xE2\x4B\xB4\x1D"
			 "\xA9\x12\x7B\x07\x70\xD9\x42\xCE"
			 "\x37\xA0\x09\x95\xFE\x67\xF3\x5C"
			 "\xC5\x2E\xBA\x23\x8C\x18\x81\xEA"
			 "\x53\xDF\x48\xB1\x1A\xA6\x0F\x78"
			 "\x04\x6D\xD6\x3F\xCB\x34\x9D\x06"
			 "\x92\xFB\x64\xF0\x59\xC2\x2B\xB7"
			 "\x20\x89\x15\x7E\xE7\x50\xDC\x45"
			 "\xAE\x17\xA3\x0C\x75\x01\x6A\xD3"
			 "\x3C\xC8\x31\x9A\x03\x8F\xF8\x61"
			 "\xED\x56\xBF\x28\xB4\x1D\x86\x12",
		.ctext = "\x04\xF3\xD3\x88\x17\xEF\xDC\xEF"
			 "\x8B\x04\xF8\x3A\x66\x8D\x1A\x53"
			 "\x57\x1F\x4B\x23\xE4\xA0\xAF\xF9"
			 "\x69\x95\x35\x98\x8D\x4D\x8C\xC1"
			 "\xF0\xB2\x7F\x80\xBB\x54\x28\xA2"
			 "\x7A\x1B\x9F\x77\xEC\x0E\x6E\xDE"
			 "\xF0\xEC\xB8\xE4\x20\x62\xEE\xDB"
			 "\x5D\xF5\xDD\xE3\x54\xFC\xDD\xEB"
			 "\x6A\xEE\x65\xA1\x21\xD6\xD7\x81"
			 "\x47\x61\x12\x4D\xC2\x8C\xFA\x78"
			 "\x1F\x28\x02\x01\xC3\xFC\x1F\xEC"
			 "\x0F\x10\x4F\xB3\x12\x45\xC6\x3B"
			 "\x7E\x08\xF9\x5A\xD0\x5D\x73\x2D"
			 "\x58\xA4\xE5\xCB\x1C\xB4\xCE\x74"
			 "\x32\x41\x1F\x31\x9C\x08\xA2\x5D"
			 "\x67\xEB\x72\x1D\xF8\xE7\x70\x54"
			 "\x34\x4B\x31\x69\x84\x66\x96\x44"
			 "\x56\xCC\x1E\xD9\xE6\x13\x6A\xB9"
			 "\x2D\x0A\x05\x45\x2D\x90\xCC\xDF"
			 "\x16\x5C\x5F\x79\x34\x52\x54\xFE"
			 "\xFE\xCD\xAD\x04\x2E\xAD\x86\x06"
			 "\x1F\x37\xE8\x28\xBC\xD3\x8F\x5B"
			 "\x92\x66\x87\x3B\x8A\x0A\x1A\xCC"
			 "\x6E\xAB\x9F\x0B\xFA\x5C\xE6\xFD"
			 "\x3C\x98\x08\x12\xEC\xAA\x9E\x11"
			 "\xCA\xB2\x1F\xCE\x5E\x5B\xB2\x72"
			 "\x9C\xCC\x5D\xC5\xE0\x32\xC0\x56"
			 "\xD5\x45\x16\xD2\xAF\x13\x66\xF7"
			 "\x8C\x67\xAC\x79\xB2\xAF\x56\x27"
			 "\x3F\xCC\xFE\xCB\x1E\xC0\x75\xF1"
			 "\xA7\xC9\xC3\x1D\x8E\xDD\xF9\xD4"
			 "\x42\xC8\x21\x08\x16\xF7\x01\xD7"
			 "\xAC\x8E\x3F\x1D\x56\xC1\x06\xE4"
			 "\x9C\x62\xD6\xA5\x6A\x50\x44\xB3"
			 "\x35\x1C\x82\xB9\x10\xF9\x42\xA1"
			 "\xFC\x74\x9B\x44\x4F\x25\x02\xE3"
			 "\x08\xF5\xD4\x32\x39\x08\x11\xE8"
			 "\xD2\x6B\x50\x53\xD4\x08\xD1\x6B"
			 "\x3A\x4A\x68\x7B\x7C\xCD\x46\x5E"
			 "\x0D\x07\x19\xDB\x67\xD7\x98\x91"
			 "\xD7\x17\x10\x9B\x7B\x8A\x9B\x33"
			 "\xAE\xF3\x00\xA6\xD4\x15\xD9\xEA"
			 "\x85\x99\x22\xE8\x91\x38\x70\x83"
			 "\x93\x01\x24\x6C\xFA\x9A\xB9\x07"
			 "\xEA\x8D\x3B\xD9\x2A\x43\x59\x16"
			 "\x2F\x69\xEE\x84\x36\x44\x76\x98"
			 "\xF3\x04\x2A\x7C\x74\x3D\x29\x2B"
			 "\x0D\xAD\x8F\x44\x82\x9E\x57\x8D"
			 "\xAC\xED\x18\x1F\x50\xA4\xF5\x98"
			 "\x1F\xBD\x92\x91\x1B\x2D\xA6\xD6"
			 "\xD2\xE3\x02\xAA\x92\x3B\xC6\xB3"
			 "\x1B\x39\x72\xD5\x26\xCA\x04\xE0"
			 "\xFC\x58\x78\xBB\xB1\x3F\xA1\x9C"
			 "\x42\x24\x3E\x2E\x22\xBB\x4B\xBA"
			 "\xF4\x52\x0A\xE6\xAE\x47\xB4\x7D"
			 "\x1D\xA8\xBE\x81\x1A\x75\xDA\xAC"
			 "\xA6\x25\x1E\xEF\x3A\xC0\x6C\x63"
			 "\xEF\xDC\xC9\x79\x10\x26\xE8\x61"
			 "\x29\xFC\xA4\x05\xDF\x7D\x5C\x63"
			 "\x10\x09\x9B\x46\x9B\xF2\x2C\x2B"
			 "\xFA\x3A\x05\x4C\xFA\xD1\xFF\xFE"
			 "\xF1\x4C\xE5\xB2\x91\x64\x0C\x51",
	},
	{
		/* Generated with Crypto++ */
		.klen = 32,
		.len = 499,
		.ilen = 16,
		.alg = SCE_AES_256,
		.mode = CTR,
		.key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55"
		       "\x0F\x32\x55\x78\x9B\xBE\x78\x9B"
		       "\xBE\xE1\x04\x27\xE1\x04\x27\x4A"
		       "\x6D\x90\x4A\x6D\x90\xB3\xD6\xF9",
		.iv = "\xE7\x82\x1D\xB8\x53\x11\xAC\x47"
		      "\xE2\x7D\x18\xD6\x71\x0C\xA7\x42",
		.ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
			 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
			 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
			 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
			 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
			 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
			 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
			 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
			 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
			 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
			 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
			 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
			 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
			 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
			 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
			 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
			 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
			 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
			 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
			 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
			 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
			 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
			 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
			 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
			 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
			 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
			 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
			 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
			 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
			 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
			 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB"
			 "\x54\xE0\x49\xB2\x1B\xA7\x10\x79"
			 "\x05\x6E\xD7\x40\xCC\x35\x9E\x07"
			 "\x93\xFC\x65\xF1\x5A\xC3\x2C\xB8"
			 "\x21\x8A\x16\x7F\xE8\x51\xDD\x46"
			 "\xAF\x18\xA4\x0D\x76\x02\x6B\xD4"
			 "\x3D\xC9\x32\x9B\x04\x90\xF9\x62"
			 "\xEE\x57\xC0\x29\xB5\x1E\x87\x13"
			 "\x7C\xE5\x4E\xDA\x43\xAC\x15\xA1"
			 "\x0A\x73\xFF\x68\xD1\x3A\xC6\x2F"
			 "\x98\x01\x8D\xF6\x5F\xEB\x54\xBD"
			 "\x26\xB2\x1B\x84\x10\x79\xE2\x4B"
			 "\xD7\x40\xA9\x12\x9E\x07\x70\xFC"
			 "\x65\xCE\x37\xC3\x2C\x95\x21\x8A"
			 "\xF3\x5C\xE8\x51\xBA\x23\xAF\x18"
			 "\x81\x0D\x76\xDF\x48\xD4\x3D\xA6"
			 "\x0F\x9B\x04\x6D\xF9\x62\xCB\x34"
			 "\xC0\x29\x92\x1E\x87\xF0\x59\xE5"
			 "\x4E\xB7\x20\xAC\x15\x7E\x0A\x73"
			 "\xDC\x45\xD1\x3A\xA3\x0C\x98\x01"
			 "\x6A\xF6\x5F\xC8\x31\xBD\x26\x8F"
			 "\x1B\x84\xED\x56\xE2\x4B\xB4\x1D"
			 "\xA9\x12\x7B\x07\x70\xD9\x42\xCE"
			 "\x37\xA0\x09\x95\xFE\x67\xF3\x5C"
			 "\xC5\x2E\xBA\x23\x8C\x18\x81\xEA"
			 "\x53\xDF\x48\xB1\x1A\xA6\x0F\x78"
			 "\x04\x6D\xD6\x3F\xCB\x34\x9D\x06"
			 "\x92\xFB\x64\xF0\x59\xC2\x2B\xB7"
			 "\x20\x89\x15\x7E\xE7\x50\xDC\x45"
			 "\xAE\x17\xA3\x0C\x75\x01\x6A\xD3"
			 "\x3C\xC8\x31\x9A\x03\x8F\xF8\x61"
			 "\xED\x56\xBF\x28\xB4\x1D\x86\x12"
			 "\x7B\xE4\x4D",
		.ctext = "\xDA\x4E\x3F\xBC\xE8\xB6\x3A\xA2"
			 "\xD5\x4D\x84\x4A\xA9\x0C\xE1\xA5"
			 "\xB8\x73\xBC\xF9\xBB\x59\x2F\x44"
			 "\x8B\xAB\x82\x6C\xB4\x32\x9A\xDE"
			 "\x5A\x0B\xDB\x7A\x6B\xF2\x38\x9F"
			 "\x06\xF7\xF7\xFF\xFF\xC0\x8A\x2E"
			 "\x76\xEA\x06\x32\x23\xF3\x59\x2E"
			 "\x75\xDE\x71\x86\x3C\x98\x23\x44"
			 "\x5B\xF2\xFA\x6A\x00\xBB\xC1\xAD"
			 "\x58\xBD\x3E\x6F\x2E\xB4\x19\x04"
			 "\x70\x8B\x92\x55\x23\xE9\x6A\x3A"
			 "\x78\x7A\x1B\x10\x85\x52\x9C\x12"
			 "\xE4\x55\x81\x21\xCE\x53\xD0\x3B"
			 "\x63\x77\x2C\x74\xD1\xF5\x60\xF3"
			 "\xA1\xDE\x44\x3C\x8F\x4D\x2F\xDD"
			 "\x8A\xFE\x3C\x42\x8E\xD3\xF2\x8E"
			 "\xA8\x28\x69\x65\x31\xE1\x45\x83"
			 "\xE4\x49\xC4\x9C\xA7\x28\xAA\x21"
			 "\xCD\x5D\x0F\x15\xB7\x93\x07\x26"
			 "\xB0\x65\x6D\x91\x90\x23\x7A\xC6"
			 "\xDB\x68\xB0\xA1\x8E\xA4\x76\x4E"
			 "\xC6\x91\x83\x20\x92\x4D\x63\x7A"
			 "\x45\x18\x18\x74\x19\xAD\x71\x01"
			 "\x6B\x23\xAD\x9D\x4E\xE4\x6E\x46"
			 "\xC9\x73\x7A\xF9\x02\x95\xF4\x07"
			 "\x0E\x7A\xA6\xC5\xAE\xFA\x15\x2C"
			 "\x51\x71\xF1\xDC\x22\xB6\xAC\xD8"
			 "\x19\x24\x44\xBC\x0C\xFB\x3C\x2D"
			 "\xB1\x50\x47\x15\x0E\xDB\xB6\xD7"
			 "\xE8\x61\xE5\x95\x52\x1E\x3E\x49"
			 "\x70\xE9\x66\x04\x4C\xE1\xAF\xBD"
			 "\xDD\x15\x3B\x20\x59\x24\xFF\xB0"
			 "\x39\xAA\xE7\xBF\x23\xA3\x6E\xD5"
			 "\x15\xF0\x61\x4F\xAE\x89\x10\x58"
			 "\x5A\x33\x95\x52\x2A\xB5\x77\x9C"
			 "\xA5\x43\x80\x40\x27\x2D\xAE\xD9"
			 "\x3F\xE0\x80\x94\x78\x79\xCB\x7E"
			 "\xAD\x12\x44\x4C\xEC\x27\xB0\xEE"
			 "\x0B\x05\x2A\x82\x99\x58\xBB\x7A"
			 "\x8D\x6D\x9D\x8E\xE2\x8E\xE7\x93"
			 "\x2F\xB3\x09\x8D\x06\xD5\xEE\x70"
			 "\x16\xAE\x35\xC5\x52\x0F\x46\x1F"
			 "\x71\xF9\x5E\xF2\x67\xDC\x98\x2F"
			 "\xA3\x23\xAA\xD5\xD0\x49\xF4\xA6"
			 "\xF6\xB8\x32\xCD\xD6\x85\x73\x60"
			 "\x59\x20\xE7\x55\x0E\x91\xE2\x0C"
			 "\x3F\x1C\xEB\x3D\xDF\x52\x64\xF2"
			 "\x7D\x8B\x5D\x63\x16\xB9\xB2\x5D"
			 "\x5E\xAB\xB2\x97\xAB\x78\x44\xE7"
			 "\xC6\x72\x20\xC5\x90\x9B\xDC\x5D"
			 "\xB0\xEF\x44\xEF\x87\x31\x8D\xF4"
			 "\xFB\x81\x5D\xF7\x96\x96\xD4\x50"
			 "\x89\xA7\xF6\xB9\x67\x76\x40\x9E"
			 "\x9D\x40\xD5\x2C\x30\xB8\x01\x8F"
			 "\xE4\x7B\x71\x48\xA9\xA0\xA0\x1D"
			 "\x87\x52\xA4\x91\xA9\xD7\xA9\x51"
			 "\xD9\x59\xF7\xCC\x63\x22\xC1\x8D"
			 "\x84\x7B\xD8\x22\x32\x5C\x6F\x1D"
			 "\x6E\x9F\xFA\xDD\x49\x40\xDC\x37"
			 "\x14\x8C\xE1\x80\x1B\xDD\x36\x2A"
			 "\xD0\xE9\x54\x99\x5D\xBA\x3B\x11"
			 "\xD8\xFE\xC9\x5B\x5C\x25\xE5\x76"
			 "\xFB\xF2\x3F",
	},

	{
		/* From NIST Special Publication 800-38A, Appendix F.5 */
		.klen = 16,
		.len = 64,
		.ilen = 16,
		.alg = SCE_AES_128,
		.mode = OFB,
		.key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
		       "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07\x08"
		      "\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae\x2d\x8a\x57\x1e\x03\xac\x9c"
			 "\x9e\xb7\x6f\xac\x45\xaf\x8e\x51"
			 "\x30\xc8\x1c\x46\xa3\x5c\xe4\x11"
			 "\xe5\xfb\xc1\x19\x1a\x0a\x52\xef"
			 "\xf6\x9f\x24\x45\xdf\x4f\x9b\x17"
			 "\xad\x2b\x41\x7b\xe6\x6c\x37\x10",
		.ctext = "\x3b\x3f\xd9\x2e\xb7\x2d\xad\x20"
			 "\x33\x34\x49\xf8\xe8\x3c\xfb\x4a"
			 "\x77\x89\x50\x8d\x16\x91\x8f\x03\xf5"
			 "\x3c\x52\xda\xc5\x4e\xd8\x25"
			 "\x97\x40\x05\x1e\x9c\x5f\xec\xf6\x43"
			 "\x44\xf7\xa8\x22\x60\xed\xcc"
			 "\x30\x4c\x65\x28\xf6\x59\xc7\x78"
			 "\x66\xa5\x10\xd9\xc1\xd6\xae\x5e",
	},
	{
		/* > 16 bytes, not a multiple of 16 bytes */
		.klen = 16,
		.len = 17,
		.ilen = 16,
		.alg = SCE_AES_128,
		.mode = OFB,
		.key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
		       "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f\x96"
			 "\xe9\x3d\x7e\x11\x73\x93\x17\x2a"
			 "\xae",
		.ctext = "\x3b\x3f\xd9\x2e\xb7\x2d\xad\x20"
			 "\x33\x34\x49\xf8\xe8\x3c\xfb\x4a"
			 "\x77",
	},
	{
		/* < 16 bytes */
		.klen = 16,
		.len = 7,
		.ilen = 16,
		.alg = SCE_AES_128,
		.mode = OFB,
		.key = "\x2b\x7e\x15\x16\x28\xae\xd2\xa6"
		       "\xab\xf7\x15\x88\x09\xcf\x4f\x3c",
		.iv = "\x00\x01\x02\x03\x04\x05\x06\x07"
		      "\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f",
		.ptext = "\x6b\xc1\xbe\xe2\x2e\x40\x9f",
		.ctext = "\x3b\x3f\xd9\x2e\xb7\x2d\xad",
	},
	{
		.klen = 8,
		.len = 8,
		.ilen = 0,
		.alg = SCE_DES,
		.mode = ECB,
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7",
		.ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d",
	},
	{
		/* Same key, different plaintext block */
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.ptext = "\x22\x33\x44\x55\x66\x77\x88\x99",
		.ctext = "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b",
		.len = 8,
		.ilen = 0,
		.alg = SCE_DES,
		.mode = ECB,
	},
	{
		/* Sbox test from NBS */
		.key = "\x7c\xa1\x10\x45\x4a\x1a\x6e\x57",
		.klen = 8,
		.ptext = "\x01\xa1\xd6\xd0\x39\x77\x67\x42",
		.ctext = "\x69\x0f\x5b\x0d\x9a\x26\x93\x9b",
		.len = 8,
		.ilen = 0,
		.alg = SCE_DES,
		.mode = ECB,
	},
	{
		/* Three blocks */
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7"
			 "\x22\x33\x44\x55\x66\x77\x88\x99"
			 "\xca\xfe\xba\xbe\xfe\xed\xbe\xef",
		.ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d"
			 "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b"
			 "\xb4\x99\x26\xf7\x1f\xe1\xd4\x90",
		.len = 24,
		.ilen = 0,
		.alg = SCE_DES,
		.mode = ECB,
	},
	{
		/* Two blocks -- for testing encryption across pages */
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7"
			 "\x22\x33\x44\x55\x66\x77\x88\x99",
		.ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d"
			 "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b",
		.len = 16,
		.ilen = 0,
		.alg = SCE_DES,
		.mode = ECB,
	},
	{
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7"
			 "\xa3\x99\x7b\xca\xaf\x69\xa0\xf5",
		.ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d"
			 "\x69\x0f\x5b\x0d\x9a\x26\x93\x9b",
		.len = 16,
		.ilen = 0,
		.alg = SCE_DES,
		.mode = ECB,
	},
	{
		/* Four blocks -- for testing encryption with chunking */
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.ptext = "\x01\x23\x45\x67\x89\xab\xcd\xe7"
			 "\x22\x33\x44\x55\x66\x77\x88\x99"
			 "\xca\xfe\xba\xbe\xfe\xed\xbe\xef"
			 "\x22\x33\x44\x55\x66\x77\x88\x99",
		.ctext = "\xc9\x57\x44\x25\x6a\x5e\xd3\x1d"
			 "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b"
			 "\xb4\x99\x26\xf7\x1f\xe1\xd4\x90"
			 "\xf7\x9c\x89\x2a\x33\x8f\x4a\x8b",
		.len = 32,
		.ilen = 0,
		.alg = SCE_DES,
		.mode = ECB,
	},
	{
		/* Generated with Crypto++ */
		.ilen = 0,
		.alg = SCE_DES,
		.mode = ECB,
		.key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55",
		.klen = 8,
		.ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
			 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
			 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
			 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
			 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
			 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
			 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
			 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
			 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
			 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
			 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
			 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
			 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
			 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
			 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
			 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
			 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
			 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
			 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
			 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
			 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
			 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
			 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
			 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
			 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
			 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
			 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
			 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
			 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
			 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
			 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB",
		.ctext = "\x88\xCB\x1F\xAB\x2F\x2A\x49\x57"
			 "\x92\xB9\x77\xFF\x2F\x47\x58\xDD"
			 "\xD7\x8A\x91\x95\x26\x33\x78\xB2"
			 "\x33\xBA\xB2\x3E\x02\xF5\x1F\xEF"
			 "\x98\xC5\xA6\xD2\x7D\x79\xEC\xB3"
			 "\x45\xF3\x4C\x61\xAC\x6C\xC2\x55"
			 "\xE5\xD3\x06\x58\x8A\x42\x3E\xDD"
			 "\x3D\x20\x45\xE9\x6F\x0D\x25\xA8"
			 "\xA5\xC7\x69\xCE\xD5\x3B\x7B\xC9"
			 "\x9E\x65\xE7\xA3\xF2\xE4\x18\x94"
			 "\xD2\x81\xE9\x33\x2B\x2D\x49\xC4"
			 "\xFE\xDA\x7F\xE2\xF2\x8C\x9C\xDC"
			 "\x73\x58\x11\x1F\x81\xD7\x21\x1A"
			 "\x80\xD0\x0D\xE8\x45\xD6\xD8\xD5"
			 "\x2E\x51\x16\xCA\x09\x89\x54\x62"
			 "\xF7\x04\x3D\x75\xB9\xA3\x84\xF4"
			 "\x62\xF0\x02\x58\x83\xAF\x30\x87"
			 "\x85\x3F\x01\xCD\x8E\x58\x42\xC4"
			 "\x41\x73\xE0\x15\x0A\xE6\x2E\x80"
			 "\x94\xF8\x5B\x3A\x4E\xDF\x51\xB2"
			 "\x9D\xE4\xC4\x9D\xF7\x3F\xF8\x8E"
			 "\x37\x22\x4D\x00\x2A\xEF\xC1\x0F"
			 "\x14\xA0\x66\xAB\x79\x39\xD0\x8E"
			 "\xE9\x95\x61\x74\x12\xED\x07\xD7"
			 "\xDD\x95\xDC\x7B\x57\x25\x27\x9C"
			 "\x51\x96\x16\xF7\x94\x61\xB8\x87"
			 "\xF0\x21\x1B\x32\xFB\x07\x0F\x29"
			 "\x56\xBD\x9D\x22\xA2\x9F\xA2\xB9"
			 "\x46\x31\x4C\x5E\x2E\x95\x61\xEF"
			 "\xE1\x58\x39\x09\xB4\x8B\x40\xAC"
			 "\x5F\x62\xC7\x72\xD9\xFC\xCB\x9A",
		.len = 248,
	},
	{
		/* From OpenSSL */
		.ilen = 8,
		.alg = SCE_DES,
		.mode = CBC,
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.iv = "\xfe\xdc\xba\x98\x76\x54\x32\x10",
		.ptext = "\x37\x36\x35\x34\x33\x32\x31\x20"
			 "\x4e\x6f\x77\x20\x69\x73\x20\x74"
			 "\x68\x65\x20\x74\x69\x6d\x65\x20",
		.ctext = "\xcc\xd1\x73\xff\xab\x20\x39\xf4"
			 "\xac\xd8\xae\xfd\xdf\xd8\xa1\xeb"
			 "\x46\x8e\x91\x15\x78\x88\xba\x68",
		.len = 24,
	},
	{
		/* FIPS Pub 81 */
		.ilen = 8,
		.alg = SCE_DES,
		.mode = CBC,
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.iv = "\x12\x34\x56\x78\x90\xab\xcd\xef",
		.ptext = "\x4e\x6f\x77\x20\x69\x73\x20\x74",
		.ctext = "\xe5\xc7\xcd\xde\x87\x2b\xf2\x7c",
		.len = 8,
	},
	{
		.ilen = 8,
		.alg = SCE_DES,
		.mode = CBC,
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.iv = "\xe5\xc7\xcd\xde\x87\x2b\xf2\x7c",
		.ptext = "\x68\x65\x20\x74\x69\x6d\x65\x20",
		.ctext = "\x43\xe9\x34\x00\x8c\x38\x9c\x0f",
		.len = 8,
	},
	{
		.ilen = 8,
		.alg = SCE_DES,
		.mode = CBC,
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef",
		.klen = 8,
		.iv = "\x43\xe9\x34\x00\x8c\x38\x9c\x0f",
		.ptext = "\x66\x6f\x72\x20\x61\x6c\x6c\x20",
		.ctext = "\x68\x37\x88\x49\x9a\x7c\x05\xf6",
		.len = 8,
	},
	{
		/* Generated with Crypto++ */
		.ilen = 8,
		.alg = SCE_DES,
		.mode = CBC,
		.key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55",
		.klen = 8,
		.iv = "\xE7\x82\x1D\xB8\x53\x11\xAC\x47",
		.ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
			 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
			 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
			 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
			 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
			 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
			 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
			 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
			 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
			 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
			 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
			 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
			 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
			 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
			 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
			 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
			 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
			 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
			 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
			 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
			 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
			 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
			 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
			 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
			 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
			 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
			 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
			 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
			 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
			 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
			 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB",
		.ctext = "\x71\xCC\x56\x1C\x87\x2C\x43\x20"
			 "\x1C\x20\x13\x09\xF9\x2B\x40\x47"
			 "\x99\x10\xD1\x1B\x65\x33\x33\xBA"
			 "\x88\x0D\xA2\xD1\x86\xFF\x4D\xF4"
			 "\x5A\x0C\x12\x96\x32\x57\xAA\x26"
			 "\xA7\xF4\x32\x8D\xBC\x10\x31\x9E"
			 "\x81\x72\x74\xDE\x30\x19\x69\x49"
			 "\x54\x9C\xC3\xEB\x0B\x97\xDD\xD1"
			 "\xE8\x6D\x0D\x05\x83\xA5\x12\x08"
			 "\x47\xF8\x88\x03\x86\x51\x3C\xEF"
			 "\xE7\x11\x73\x4D\x44\x2B\xE2\x16"
			 "\xE8\xA5\x06\x50\x66\x70\x0E\x14"
			 "\xBA\x21\x3B\xD5\x23\x5B\xA7\x8F"
			 "\x56\xB6\xA7\x44\xDB\x86\xAB\x69"
			 "\x33\x3C\xBE\x64\xC4\x22\xD3\xFE"
			 "\x49\x90\x88\x6A\x09\x8F\x76\x59"
			 "\xCB\xB7\xA0\x2D\x79\x75\x92\x8A"
			 "\x82\x1D\xC2\xFE\x09\x1F\x78\x6B"
			 "\x2F\xD6\xA4\x87\x1E\xC4\x53\x63"
			 "\x80\x02\x61\x2F\xE3\x46\xB6\xB5"
			 "\xAA\x95\xF4\xEE\xA7\x64\x2B\x4F"
			 "\x20\xCF\xD2\x47\x4E\x39\x65\xB3"
			 "\x11\x87\xA2\x6C\x49\x7E\x36\xC7"
			 "\x62\x8B\x48\x0D\x6A\x64\x00\xBD"
			 "\x71\x91\x8C\xE9\x70\x19\x01\x4F"
			 "\x4E\x68\x23\xBA\xDA\x24\x2E\x45"
			 "\x02\x14\x33\x21\xAE\x58\x4B\xCF"
			 "\x3B\x4B\xE8\xF8\xF6\x4F\x34\x93"
			 "\xD7\x07\x8A\xD7\x18\x92\x36\x8C"
			 "\x82\xA9\xBD\x6A\x31\x91\x39\x11"
			 "\xC6\x4A\xF3\x55\xC7\x29\x2E\x63",
		.len = 248,
	},
	{
		/* Generated with Crypto++ */
		.ilen = 8,
		.alg = SCE_DES,
		.mode = CTR,
		.key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55",
		.klen = 8,
		.iv = "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFD",
		.ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
			 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
			 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
			 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
			 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
			 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
			 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
			 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
			 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
			 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
			 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
			 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
			 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
			 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
			 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
			 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
			 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
			 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
			 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
			 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
			 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
			 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
			 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
			 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
			 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
			 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
			 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
			 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
			 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
			 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
			 "\xC6\x2F\xBB\x24\x8D\x19\x82\xEB",
		.ctext = "\x2F\x96\x06\x0F\x50\xC9\x68\x03"
			 "\x0F\x31\xD4\x64\xA5\x29\x77\x35"
			 "\xBC\x7A\x9F\x19\xE7\x0D\x33\x3E"
			 "\x12\x0B\x8C\xAE\x48\xAE\xD9\x02"
			 "\x0A\xD4\xB0\xD6\x37\xB2\x65\x1C"
			 "\x4B\x65\xEB\x24\xB5\x8E\xAD\x47"
			 "\x0D\xDA\x79\x77\xA0\x29\xA0\x2B"
			 "\xC8\x0F\x85\xDC\x03\x13\xA9\x04"
			 "\x19\x40\xBE\xBE\x5C\x49\x4A\x69"
			 "\xED\xE8\xE1\x9E\x14\x43\x74\xDE"
			 "\xEC\x6E\x11\x3F\x36\xEF\x7B\xFB"
			 "\xBE\x4C\x91\x43\x22\x65\x72\x48"
			 "\xE2\x12\xED\x88\xAC\xA7\xC9\x91"
			 "\x14\xA2\x36\x1C\x29\xFF\xC8\x4F"
			 "\x72\x5C\x4B\xB0\x1E\x93\xC2\xFA"
			 "\x9D\x53\x86\xA0\xAE\xC6\xB7\x3C"
			 "\x59\x0C\xD0\x8F\xA6\xD8\xA4\x31"
			 "\xB7\x30\x1C\x21\x38\xFB\x68\x8C"
			 "\x2E\xF5\x6E\x73\xC3\x16\x5F\x12"
			 "\x0C\x33\xB9\x1E\x7B\x70\xDE\x86"
			 "\x32\xB3\xC1\x16\xAB\xD9\x49\x0B"
			 "\x96\x28\x72\x6B\xF3\x30\xA9\xEB"
			 "\x69\xE2\x1E\x58\x46\xA2\x8E\xC7"
			 "\xC0\xEF\x07\xB7\x77\x2C\x00\x05"
			 "\x46\xBD\xFE\x53\x81\x8B\xA4\x03"
			 "\x20\x0F\xDB\x78\x0B\x1F\x53\x04"
			 "\x4C\x60\x4C\xC3\x2A\x86\x86\x7E"
			 "\x13\xD2\x26\xED\x5D\x3E\x9C\xF2"
			 "\x5C\xC4\x15\xC9\x9A\x21\xC5\xCD"
			 "\x19\x7F\x99\x19\x53\xCE\x1D\x14"
			 "\x69\x74\xA1\x06\x46\x0F\x4E\x75",
		.len = 248,
	},
	{
		/* Generated with Crypto++ */
		.ilen = 8,
		.alg = SCE_DES,
		.mode = CTR,
		.key = "\xC9\x83\xA6\xC9\xEC\x0F\x32\x55",
		.klen = 8,
		.iv = "\xE7\x82\x1D\xB8\x53\x11\xAC\x47",
		.ptext = "\x50\xB9\x22\xAE\x17\x80\x0C\x75"
			 "\xDE\x47\xD3\x3C\xA5\x0E\x9A\x03"
			 "\x6C\xF8\x61\xCA\x33\xBF\x28\x91"
			 "\x1D\x86\xEF\x58\xE4\x4D\xB6\x1F"
			 "\xAB\x14\x7D\x09\x72\xDB\x44\xD0"
			 "\x39\xA2\x0B\x97\x00\x69\xF5\x5E"
			 "\xC7\x30\xBC\x25\x8E\x1A\x83\xEC"
			 "\x55\xE1\x4A\xB3\x1C\xA8\x11\x7A"
			 "\x06\x6F\xD8\x41\xCD\x36\x9F\x08"
			 "\x94\xFD\x66\xF2\x5B\xC4\x2D\xB9"
			 "\x22\x8B\x17\x80\xE9\x52\xDE\x47"
			 "\xB0\x19\xA5\x0E\x77\x03\x6C\xD5"
			 "\x3E\xCA\x33\x9C\x05\x91\xFA\x63"
			 "\xEF\x58\xC1\x2A\xB6\x1F\x88\x14"
			 "\x7D\xE6\x4F\xDB\x44\xAD\x16\xA2"
			 "\x0B\x74\x00\x69\xD2\x3B\xC7\x30"
			 "\x99\x02\x8E\xF7\x60\xEC\x55\xBE"
			 "\x27\xB3\x1C\x85\x11\x7A\xE3\x4C"
			 "\xD8\x41\xAA\x13\x9F\x08\x71\xFD"
			 "\x66\xCF\x38\xC4\x2D\x96\x22\x8B"
			 "\xF4\x5D\xE9\x52\xBB\x24\xB0\x19"
			 "\x82\x0E\x77\xE0\x49\xD5\x3E\xA7"
			 "\x10\x9C\x05\x6E\xFA\x63\xCC\x35"
			 "\xC1\x2A\x93\x1F\x88\xF1\x5A\xE6"
			 "\x4F\xB8\x21\xAD\x16\x7F\x0B\x74"
			 "\xDD\x46\xD2\x3B\xA4\x0D\x99\x02"
			 "\x6B\xF7\x60\xC9\x32\xBE\x27\x90"
			 "\x1C\x85\xEE\x57\xE3\x4C\xB5\x1E"
			 "\xAA\x13\x7C\x08\x71\xDA\x43\xCF"
			 "\x38\xA1\x0A\x96\xFF\x68\xF4\x5D"
			 "\xC6\x2F\xBB\x24\x8D\x19\x82",
		.ctext = "\x62\xE5\xF4\xDC\x99\xE7\x89\xE3"
			 "\xF4\x10\xCC\x21\x99\xEB\xDC\x15"
			 "\x19\x13\x93\x27\x9D\xB6\x6F\x45"
			 "\x17\x55\x61\x72\xC8\xD3\x7F\xA5"
			 "\x32\xD0\xD3\x02\x15\xA4\x05\x23"
			 "\x9C\x23\x61\x60\x77\x7B\x6C\x95"
			 "\x26\x49\x42\x2E\xF3\xC1\x8C\x6D"
			 "\xC8\x47\xD5\x94\xE7\x53\xC8\x23"
			 "\x1B\xA5\x0B\xCB\x12\xD3\x7A\x12"
			 "\xA4\x42\x15\x34\xF7\x5F\xDC\x58"
			 "\x5B\x58\x4C\xAD\xD1\x33\x8E\xE6"
			 "\xE5\xA0\xDA\x4D\x94\x3D\x63\xA8"
			 "\x02\x82\xBB\x16\xB8\xDC\xB5\x58"
			 "\xC3\x2D\x79\xE4\x25\x79\x43\xF9"
			 "\x6D\xD3\xCA\xC0\xE8\x12\xD4\x7E"
			 "\x04\x25\x79\xFD\x27\xFB\xC4\xEA"
			 "\x32\x94\x48\x92\xF3\x68\x1A\x7F"
			 "\x36\x33\x43\x79\xF7\xCA\xC2\x38"
			 "\xC0\x68\xD4\x53\xA9\xCC\x43\x0C"
			 "\x40\x57\x3E\xED\x00\x9F\x22\x6E"
			 "\x80\x99\x0B\xCC\x40\x63\x46\x8A"
			 "\xE8\xC4\x9B\x6D\x7A\x08\x6E\xA9"
			 "\x6F\x84\xBC\xB3\xF4\x95\x0B\x2D"
			 "\x6A\xBA\x37\x50\xC3\xCF\x9F\x7C"
			 "\x59\x5E\xDE\x0B\x30\xFA\x34\x8A"
			 "\xF8\xD1\xA2\xF8\x4E\xBD\x5D\x5E"
			 "\x7D\x71\x99\xE0\xF6\xE5\x7C\xE0"
			 "\x6D\xEE\x82\x89\x92\xD4\xF5\xD7"
			 "\xDF\x85\x2D\xE1\xB2\xD6\xAB\x94"
			 "\xA5\xA6\xE7\xB0\x51\x36\x52\x37"
			 "\x91\x45\x05\x3E\x58\xBF\x32",
		.len = 247,
	},

	{
		/* These are from openssl */
		.ilen = 0,
		.alg = SCE_TDES,
		.mode = ECB,
		.key = "\x01\x23\x45\x67\x89\xab\xcd\xef"
		       "\x55\x55\x55\x55\x55\x55\x55\x55"
		       "\xfe\xdc\xba\x98\x76\x54\x32\x10",
		.klen = 24,
		.ptext = "\x73\x6f\x6d\x65\x64\x61\x74\x61",
		.ctext = "\x18\xd7\x48\xe5\x63\x62\x05\x72",
		.len = 8,
	},
	{
		.ilen = 0,
		.alg = SCE_TDES,
		.mode = ECB,
		.key = "\x03\x52\x02\x07\x67\x20\x82\x17"
		       "\x86\x02\x87\x66\x59\x08\x21\x98"
		       "\x64\x05\x6a\xbd\xfe\xa9\x34\x57",
		.klen = 24,
		.ptext = "\x73\x71\x75\x69\x67\x67\x6c\x65",
		.ctext = "\xc0\x7d\x2a\x0f\xa5\x66\xfa\x30",
		.len = 8,
	},
	{
		.ilen = 0,
		.alg = SCE_TDES,
		.mode = ECB,
		.key = "\x10\x46\x10\x34\x89\x98\x80\x20"
		       "\x91\x07\xd0\x15\x89\x19\x01\x01"
		       "\x19\x07\x92\x10\x98\x1a\x01\x01",
		.klen = 24,
		.ptext = "\x00\x00\x00\x00\x00\x00\x00\x00",
		.ctext = "\xe1\xef\x62\xc3\x32\xfe\x82\x5b",
		.len = 8,
	},
	{
		/* Generated with Crypto++ */
		.ilen = 0,
		.alg = SCE_TDES,
		.mode = ECB,
		.key = "\xF3\x9C\xD6\xF3\x9C\xB9\x5A\x67"
		       "\x00\x5A\x67\x00\x2D\xCE\xEB\x2D"
		       "\xCE\xEB\xB4\x51\x72\xB4\x51\x72",
		.klen = 24,
		.ptext = "\x05\xEC\x77\xFB\x42\xD5\x59\x20"
			 "\x8B\x12\x86\x69\xF0\x5B\xCF\x56"
			 "\x39\xAD\x34\x9F\x66\xEA\x7D\xC4"
			 "\x48\xD3\xBA\x0D\xB1\x18\xE3\x4A"
			 "\xFE\x41\x28\x5C\x27\x8E\x11\x85"
			 "\x6C\xF7\x5E\xC2\x55\x3C\xA0\x0B"
			 "\x92\x65\xE9\x70\xDB\x4F\xD6\xB9"
			 "\x00\xB4\x1F\xE6\x49\xFD\x44\x2F"
			 "\x53\x3A\x8D\x14\x98\x63\xCA\x5D"
			 "\xC1\xA8\x33\xA7\x0E\x91\x78\xEC"
			 "\x77\xDE\x42\xD5\xBC\x07\x8B\x12"
			 "\xE5\x4C\xF0\x5B\x22\x56\x39\x80"
			 "\x6B\x9F\x66\xC9\x50\xC4\xAF\x36"
			 "\xBA\x0D\x94\x7F\xE3\x4A\xDD\x41"
			 "\x28\xB3\x1A\x8E\x11\xF8\x43\xF7"
			 "\x5E\x21\x55\x3C\x87\x6E\x92\x65"
			 "\xCC\x57\xDB\xA2\x35\xB9\x00\xEB"
			 "\x72\xE6\x49\xD0\x44\x2F\xB6\x19"
			 "\x8D\x14\xFF\x46\xCA\x5D\x24\xA8"
			 "\x33\x9A\x6D\x91\x78\xC3\x77\xDE"
			 "\xA1\x08\xBC\x07\xEE\x71\xE5\x4C"
			 "\xD7\x5B\x22\xB5\x1C\x80\x6B\xF2"
			 "\x45\xC9\x50\x3B\xAF\x36\x99\x60"
			 "\x94\x7F\xC6\x4A\xDD\xA4\x0F\xB3"
			 "\x1A\xED\x74\xF8\x43\x2A\x5E\x21"
			 "\x88\x13\x87\x6E\xF1\x58\xCC\x57"
			 "\x3E\xA2\x35\x9C\x67\xEB\x72\xC5"
			 "\x49\xD0\xBB\x02\xB6\x19\xE0\x4B"
			 "\xFF\x46\x29\x5D\x24\x8F\x16\x9A"
			 "\x6D\xF4\x5F\xC3\xAA\x3D\xA1\x08"
			 "\x93\x7A\xEE\x71\xD8\x4C\xD7\xBE"
			 "\x01\xB5\x1C\xE7\x4E\xF2\x45\x2C"
			 "\x50\x3B\x82\x15\x99\x60\xCB\x52"
			 "\xC6\xA9\x30\xA4\x0F\x96\x79\xED"
			 "\x74\xDF\x43\x2A\xBD\x04\x88\x13"
			 "\xFA\x4D\xF1\x58\x23\x57\x3E\x81"
			 "\x68\x9C\x67\xCE\x51\xC5\xAC\x37"
			 "\xBB\x02\x95\x7C\xE0\x4B\xD2\x46"
			 "\x29\xB0\x1B\x8F\x16\xF9\x40\xF4"
			 "\x5F\x26\xAA\x3D\x84\x6F\x93\x7A"
			 "\xCD\x54\xD8\xA3\x0A\xBE\x01\xE8"
			 "\x73\xE7\x4E\xD1\x45\x2C\xB7\x1E"
			 "\x82\x15\xFC\x47\xCB\x52\x25\xA9"
			 "\x30\x9B\x62\x96\x79\xC0\x74\xDF"
			 "\xA6\x09\xBD\x04\xEF\x76\xFA\x4D"
			 "\xD4\x58\x23\x8A\x1D\x81\x68\xF3"
			 "\x5A\xCE\x51\x38\xAC\x37\x9E\x61"
			 "\x95\x7C\xC7\x4B\xD2\xA5\x0C\xB0"
			 "\x1B\xE2\x75\xF9\x40\x2B\x5F\x26"
			 "\x89\x10\x84\x6F\xF6\x59\xCD\x54"
			 "\x3F\xA3\x0A\x9D\x64\xE8\x73\xDA"
			 "\x4E\xD1\xB8\x03\xB7\x1E\xE1\x48"
			 "\xFC\x47\x2E\x52\x25\x8C\x17\x9B"
			 "\x62\xF5\x5C\xC0\xAB\x32\xA6\x09"
			 "\x90\x7B\xEF\x76\xD9\x4D\xD4\xBF"
			 "\x06\x8A\x1D\xE4\x4F\xF3\x5A\x2D"
			 "\x51\x38\x83\x6A\x9E\x61\xC8\x53"
			 "\xC7\xAE\x31\xA5\x0C\x97\x7E\xE2"
			 "\x75\xDC\x40\x2B\xB2\x05\x89\x10"
			 "\xFB\x42\xF6\x59\x20\x54\x3F\x86"
			 "\x69\x9D\x64\xCF\x56\xDA\xAD\x34"
			 "\xB8\x03\xEA\x7D\xE1\x48\xD3\x47",
		.ctext = "\x4E\x9A\x40\x3D\x61\x7D\x17\xFA"
			 "\x16\x86\x88\x0B\xD8\xAE\xF8\xE4"
			 "\x81\x01\x04\x00\x76\xFA\xED\xD3"
			 "\x44\x7E\x21\x9D\xF0\xFB\x2B\x64"
			 "\xCA\x4E\x90\xE0\xC0\x63\x28\x92"
			 "\xF3\x1F\xA4\x53\x2C\x77\xCC\x77"
			 "\x69\x56\xD0\x19\xAD\x00\x2D\x97"
			 "\xBC\xDE\x49\x6A\x82\xBC\x16\xE2"
			 "\x2F\x3E\x72\xEE\xD1\xCE\xFC\x1B"
			 "\xEA\x32\x56\xE4\x0B\xAF\x27\x36"
			 "\xAF\x08\xB9\x61\xB7\x48\x23\x27"
			 "\xEE\x4D\xC8\x79\x56\x06\xEB\xC7"
			 "\x5B\xCA\x0A\xC6\x5E\x5C\xCB\xB6"
			 "\x9D\xDA\x04\x59\xE2\x09\x48\x7E"
			 "\x6B\x37\xC6\xFE\x92\xA9\x1E\x6E"
			 "\x0D\x19\xFA\x33\x0F\xEE\x36\x68"
			 "\x11\xBB\xF9\x5A\x73\xAB\x3A\xEA"
			 "\xAC\x28\xD8\xD5\x27\xE8\x6B\x16"
			 "\x45\x86\x50\x01\x70\x35\x99\x92"
			 "\xDF\x0C\x07\x88\x8B\x7F\x9E\x4B"
			 "\xD2\x04\x84\x90\xC4\x27\xDF\x0A"
			 "\x49\xA8\xA7\x1A\x6D\x78\x16\xCA"
			 "\xB3\x18\x5C\xC3\x93\x63\x5A\x68"
			 "\x77\x02\xBA\xED\x62\x71\xB1\xD9"
			 "\x5E\xE5\x6F\x1A\xCC\x1D\xBE\x2E"
			 "\x11\xF3\xA6\x97\xCA\x8E\xBF\xB4"
			 "\x56\xA1\x36\x6B\xB1\x0A\x3E\x70"
			 "\xEA\xD7\xCD\x72\x7B\x79\xC8\xAD"
			 "\x6B\xFE\xFB\xBA\x64\xAE\x19\xC1"
			 "\x82\xCF\x8A\xA1\x50\x17\x7F\xB2"
			 "\x6F\x7B\x0F\x52\xC5\x3E\x4A\x52"
			 "\x3F\xD9\x3F\x01\xA6\x41\x1A\xB3"
			 "\xB3\x7A\x0E\x8E\x75\xB2\xB1\x5F"
			 "\xDB\xEA\x84\x13\x26\x6C\x85\x4E"
			 "\xAE\x6B\xDC\xE7\xE7\xAD\xB0\x06"
			 "\x5C\xBA\x92\xD0\x30\xBB\x8D\xD2"
			 "\xAE\x4C\x70\x85\xA0\x07\xE3\x2C"
			 "\xD1\x27\x9C\xCF\xDB\x13\xB7\xE5"
			 "\xF9\x6A\x02\xD0\x39\x9D\xB6\xE7"
			 "\xD1\x17\x25\x08\xF9\xA9\xA6\x67"
			 "\x38\x80\xD1\x22\xAB\x1A\xD7\x26"
			 "\xAD\xCA\x19\x1B\xFA\x18\xA7\x57"
			 "\x31\xEC\xC9\xED\xDB\x79\xC0\x48"
			 "\xAC\x31\x9F\x03\x8B\x62\x5B\x7E"
			 "\x0E\xA6\xD0\x64\xEE\xEA\x00\xFC"
			 "\x58\xC8\xDE\x51\x4E\x17\x15\x11"
			 "\x66\x58\xB6\x90\xDC\xDF\xA1\x49"
			 "\xCA\x79\xE9\x31\x31\x42\xDC\x56"
			 "\x0B\xCD\xB6\x0D\xC7\x64\xF7\x19"
			 "\xD9\x42\x05\x7F\xBC\x2F\xFC\x90"
			 "\xAE\x29\x86\xAA\x43\x7A\x4F\x6B"
			 "\xCE\xEA\xBC\x31\x8D\x65\x9D\x46"
			 "\xEA\x77\xB4\xF9\x58\xEA\x5D\x84"
			 "\xE4\xDC\x14\xBB\xBD\x15\x0E\xDA"
			 "\xD8\xE4\xA4\x5D\x61\xF9\x58\x0F"
			 "\xE4\x82\x77\xCE\x87\xC0\x09\xF0"
			 "\xD6\x10\x9E\x34\xE1\x0C\x67\x55"
			 "\x7B\x6D\xD5\x51\x4B\x00\xEE\xBA"
			 "\xF2\x7B\xBE\x75\x07\x42\x9D\x99"
			 "\x12\xE1\x71\x4A\xF9\x2A\xF5\xF6"
			 "\x93\x03\xD7\x51\x09\xFA\xBE\x68"
			 "\xD8\x45\xFF\x33\xBA\xBB\x2B\x63",
		.len = 496,
	},

	{
		/* Generated from openssl */
		.ilen = 8,
		.alg = SCE_TDES,
		.mode = CBC,
		.key = "\xE9\xC0\xFF\x2E\x76\x0B\x64\x24"
		       "\x44\x4D\x99\x5A\x12\xD6\x40\xC0"
		       "\xEA\xC2\x84\xE8\x14\x95\xDB\xE8",
		.klen = 24,
		.iv = "\x7D\x33\x88\x93\x0F\x93\xB2\x42",
		.ptext = "\x6f\x54\x20\x6f\x61\x4d\x79\x6e"
			 "\x53\x20\x63\x65\x65\x72\x73\x74"
			 "\x54\x20\x6f\x6f\x4d\x20\x6e\x61"
			 "\x20\x79\x65\x53\x72\x63\x74\x65"
			 "\x20\x73\x6f\x54\x20\x6f\x61\x4d"
			 "\x79\x6e\x53\x20\x63\x65\x65\x72"
			 "\x73\x74\x54\x20\x6f\x6f\x4d\x20"
			 "\x6e\x61\x20\x79\x65\x53\x72\x63"
			 "\x74\x65\x20\x73\x6f\x54\x20\x6f"
			 "\x61\x4d\x79\x6e\x53\x20\x63\x65"
			 "\x65\x72\x73\x74\x54\x20\x6f\x6f"
			 "\x4d\x20\x6e\x61\x20\x79\x65\x53"
			 "\x72\x63\x74\x65\x20\x73\x6f\x54"
			 "\x20\x6f\x61\x4d\x79\x6e\x53\x20"
			 "\x63\x65\x65\x72\x73\x74\x54\x20"
			 "\x6f\x6f\x4d\x20\x6e\x61\x0a\x79",
		.ctext = "\x0e\x2d\xb6\x97\x3c\x56\x33\xf4"
			 "\x67\x17\x21\xc7\x6e\x8a\xd5\x49"
			 "\x74\xb3\x49\x05\xc5\x1c\xd0\xed"
			 "\x12\x56\x5c\x53\x96\xb6\x00\x7d"
			 "\x90\x48\xfc\xf5\x8d\x29\x39\xcc"
			 "\x8a\xd5\x35\x18\x36\x23\x4e\xd7"
			 "\x76\xd1\xda\x0c\x94\x67\xbb\x04"
			 "\x8b\xf2\x03\x6c\xa8\xcf\xb6\xea"
			 "\x22\x64\x47\xaa\x8f\x75\x13\xbf"
			 "\x9f\xc2\xc3\xf0\xc9\x56\xc5\x7a"
			 "\x71\x63\x2e\x89\x7b\x1e\x12\xca"
			 "\xe2\x5f\xaf\xd8\xa4\xf8\xc9\x7a"
			 "\xd6\xf9\x21\x31\x62\x44\x45\xa6"
			 "\xd6\xbc\x5a\xd3\x2d\x54\x43\xcc"
			 "\x9d\xde\xa5\x70\xe9\x42\x45\x8a"
			 "\x6b\xfa\xb1\x91\x13\xb0\xd9\x19",
		.len = 128,
	},
	{
		/* Generated with Crypto++ */
		.ilen = 16,
		.alg = SCE_TDES,
		.mode = CBC,
		.key = "\x9C\xD6\xF3\x9C\xB9\x5A\x67\x00"
		       "\x5A\x67\x00\x2D\xCE\xEB\x2D\xCE"
		       "\xEB\xB4\x51\x72\xB4\x51\x72\x1F",
		.klen = 24,
		.iv = "\xB2\xD7\x48\xED\x06\x44\xF9\x12"
		      "\xB7\x28\x4D\x83\x24\x59\xF2\x17",
		.ptext = "\x05\xEC\x77\xFB\x42\xD5\x59\x20"
			 "\x8B\x12\x86\x69\xF0\x5B\xCF\x56"
			 "\x39\xAD\x34\x9F\x66\xEA\x7D\xC4"
			 "\x48\xD3\xBA\x0D\xB1\x18\xE3\x4A"
			 "\xFE\x41\x28\x5C\x27\x8E\x11\x85"
			 "\x6C\xF7\x5E\xC2\x55\x3C\xA0\x0B"
			 "\x92\x65\xE9\x70\xDB\x4F\xD6\xB9"
			 "\x00\xB4\x1F\xE6\x49\xFD\x44\x2F"
			 "\x53\x3A\x8D\x14\x98\x63\xCA\x5D"
			 "\xC1\xA8\x33\xA7\x0E\x91\x78\xEC"
			 "\x77\xDE\x42\xD5\xBC\x07\x8B\x12"
			 "\xE5\x4C\xF0\x5B\x22\x56\x39\x80"
			 "\x6B\x9F\x66\xC9\x50\xC4\xAF\x36"
			 "\xBA\x0D\x94\x7F\xE3\x4A\xDD\x41"
			 "\x28\xB3\x1A\x8E\x11\xF8\x43\xF7"
			 "\x5E\x21\x55\x3C\x87\x6E\x92\x65"
			 "\xCC\x57\xDB\xA2\x35\xB9\x00\xEB"
			 "\x72\xE6\x49\xD0\x44\x2F\xB6\x19"
			 "\x8D\x14\xFF\x46\xCA\x5D\x24\xA8"
			 "\x33\x9A\x6D\x91\x78\xC3\x77\xDE"
			 "\xA1\x08\xBC\x07\xEE\x71\xE5\x4C"
			 "\xD7\x5B\x22\xB5\x1C\x80\x6B\xF2"
			 "\x45\xC9\x50\x3B\xAF\x36\x99\x60"
			 "\x94\x7F\xC6\x4A\xDD\xA4\x0F\xB3"
			 "\x1A\xED\x74\xF8\x43\x2A\x5E\x21"
			 "\x88\x13\x87\x6E\xF1\x58\xCC\x57"
			 "\x3E\xA2\x35\x9C\x67\xEB\x72\xC5"
			 "\x49\xD0\xBB\x02\xB6\x19\xE0\x4B"
			 "\xFF\x46\x29\x5D\x24\x8F\x16\x9A"
			 "\x6D\xF4\x5F\xC3\xAA\x3D\xA1\x08"
			 "\x93\x7A\xEE\x71\xD8\x4C\xD7\xBE"
			 "\x01\xB5\x1C\xE7\x4E\xF2\x45\x2C"
			 "\x50\x3B\x82\x15\x99\x60\xCB\x52"
			 "\xC6\xA9\x30\xA4\x0F\x96\x79\xED"
			 "\x74\xDF\x43\x2A\xBD\x04\x88\x13"
			 "\xFA\x4D\xF1\x58\x23\x57\x3E\x81"
			 "\x68\x9C\x67\xCE\x51\xC5\xAC\x37"
			 "\xBB\x02\x95\x7C\xE0\x4B\xD2\x46"
			 "\x29\xB0\x1B\x8F\x16\xF9\x40\xF4"
			 "\x5F\x26\xAA\x3D\x84\x6F\x93\x7A"
			 "\xCD\x54\xD8\xA3\x0A\xBE\x01\xE8"
			 "\x73\xE7\x4E\xD1\x45\x2C\xB7\x1E"
			 "\x82\x15\xFC\x47\xCB\x52\x25\xA9"
			 "\x30\x9B\x62\x96\x79\xC0\x74\xDF"
			 "\xA6\x09\xBD\x04\xEF\x76\xFA\x4D"
			 "\xD4\x58\x23\x8A\x1D\x81\x68\xF3"
			 "\x5A\xCE\x51\x38\xAC\x37\x9E\x61"
			 "\x95\x7C\xC7\x4B\xD2\xA5\x0C\xB0"
			 "\x1B\xE2\x75\xF9\x40\x2B\x5F\x26"
			 "\x89\x10\x84\x6F\xF6\x59\xCD\x54"
			 "\x3F\xA3\x0A\x9D\x64\xE8\x73\xDA"
			 "\x4E\xD1\xB8\x03\xB7\x1E\xE1\x48"
			 "\xFC\x47\x2E\x52\x25\x8C\x17\x9B"
			 "\x62\xF5\x5C\xC0\xAB\x32\xA6\x09"
			 "\x90\x7B\xEF\x76\xD9\x4D\xD4\xBF"
			 "\x06\x8A\x1D\xE4\x4F\xF3\x5A\x2D"
			 "\x51\x38\x83\x6A\x9E\x61\xC8\x53"
			 "\xC7\xAE\x31\xA5\x0C\x97\x7E\xE2"
			 "\x75\xDC\x40\x2B\xB2\x05\x89\x10"
			 "\xFB\x42\xF6\x59\x20\x54\x3F\x86"
			 "\x69\x9D\x64\xCF\x56\xDA\xAD\x34"
			 "\xB8\x03\xEA\x7D\xE1\x48\xD3\x47",
		.ctext = "\xF8\xF6\xB5\x60\x5C\x5A\x75\x84"
			 "\x87\x81\x53\xBA\xC9\x6F\xEC\xD5"
			 "\x1E\x68\x8E\x85\x12\x86\x1D\x38"
			 "\x1C\x91\x40\xCC\x69\x6A\xD5\x35"
			 "\x0D\x7C\xB5\x07\x7C\x7B\x2A\xAF"
			 "\x32\xBC\xA1\xB3\x84\x31\x1B\x3C"
			 "\x0A\x2B\xFA\xD3\x9F\xB0\x8C\x37"
			 "\x8F\x9D\xA7\x6D\x6C\xFA\xD7\x90"
			 "\xE3\x69\x54\xED\x3A\xC4\xF1\x6B"
			 "\xB1\xCC\xFB\x7D\xD8\x8E\x17\x0B"
			 "\x9C\xF6\x4C\xD6\xFF\x03\x4E\xD9"
			 "\xE6\xA5\xAD\x25\xE6\x17\x69\x63"
			 "\x11\x35\x61\x94\x88\x7B\x1C\x48"
			 "\xF1\x24\x20\x29\x6B\x93\x1A\x8E"
			 "\x43\x03\x89\xD8\xB1\xDA\x47\x7B"
			 "\x79\x3A\x83\x76\xDA\xAE\xC6\xBB"
			 "\x22\xF8\xE8\x3D\x9A\x65\x54\xD8"
			 "\x4C\xE9\xE7\xE4\x63\x2F\x5C\x73"
			 "\x5A\xC3\xAE\x46\xA8\xCD\x57\xE6"
			 "\x67\x88\xA5\x20\x6F\x5F\x97\xC7"
			 "\xCC\x15\xA2\x0A\x93\xEA\x33\xE7"
			 "\x03\x5F\xEC\x64\x30\x6F\xEE\xD7"
			 "\x7E\xDF\xD6\xE9\x6F\x3F\xD6\x1E"
			 "\xBE\x67\x6C\x5B\x97\xA0\x09\xE6"
			 "\xEE\xFE\x55\xA3\x29\x65\xE0\x12"
			 "\xA1\x6A\x8A\x6F\xF2\xE6\xF1\x96"
			 "\x87\xFB\x9C\x05\xDD\x80\xEC\xFF"
			 "\xC5\xED\x50\xFE\xFC\x91\xCD\xCE"
			 "\x25\x2C\x5F\xD9\xAD\x95\x7D\x99"
			 "\xF0\x05\xC4\x71\x46\x5F\xF9\x0D"
			 "\xD2\x63\xDF\x9B\x96\x2E\x2B\xA6"
			 "\x2B\x1C\xD5\xFB\x96\x24\x60\x60"
			 "\x54\x40\xB8\x62\xA4\xF8\x46\x95"
			 "\x73\x28\xA3\xA6\x16\x2B\x17\xE7"
			 "\x7A\xF8\x62\x54\x3B\x64\x69\xE1"
			 "\x71\x34\x29\x5B\x4E\x05\x9B\xFA"
			 "\x5E\xF1\x96\xB7\xCE\x16\x9B\x59"
			 "\xF1\x1A\x4C\x51\x26\xFD\x79\xE2"
			 "\x3B\x8E\x71\x69\x6A\x91\xB6\x65"
			 "\x32\x09\xB8\xE4\x09\x1F\xEA\x39"
			 "\xCE\x20\x65\x9F\xD6\xD1\xC7\xF0"
			 "\x73\x50\x08\x56\x20\x9B\x94\x23"
			 "\x14\x39\xB7\x2B\xB1\x2D\x6D\x6F"
			 "\x41\x5B\xCC\xE2\x18\xAE\x62\x89"
			 "\x78\x8E\x67\x23\xD0\xFB\x2B\xE5"
			 "\x25\xC9\x48\x97\xB5\xD3\x17\xD5"
			 "\x6A\x9F\xA7\x48\x0C\x2B\x73\x3B"
			 "\x57\x08\xAE\x91\xF2\xB7\x57\x89"
			 "\xF4\xD0\xB0\x07\xB0\x42\x6C\xAF"
			 "\x98\x1A\xE7\xD1\xAC\x1E\xB5\x02"
			 "\xD4\x56\x42\x79\x79\x7F\x2A\x77"
			 "\x25\xE9\x7D\xC1\x88\x19\x2B\x49"
			 "\x6F\x46\x59\xAB\x56\x1F\x61\xE0"
			 "\x0C\x24\x9C\xC9\x5B\x63\xA9\x12"
			 "\xCF\x88\x96\xB6\xA8\x24\xC6\xA8"
			 "\x21\x85\x1A\x62\x7E\x34\xBB\xEB"
			 "\xBD\x02\x2A\xC7\xD8\x89\x80\xC5"
			 "\xB1\xBB\x60\xA5\x22\xFC\x6F\x38"
			 "\x02\x80\xA3\x28\x22\x75\xE1\xE9"
			 "\x90\xE9\xFA\x4B\x00\x10\xAC\x58"
			 "\x83\x70\xFF\x86\xE6\xAA\x0F\x1F"
			 "\x95\x63\x73\xA2\x44\xAC\xF8\xA5",
		.len = 496,
	},
	{
		/* Generated with Crypto++ */
		.ilen = 8,
		.alg = SCE_TDES,
		.mode = CTR,
		.key = "\x9C\xD6\xF3\x9C\xB9\x5A\x67\x00"
		       "\x5A\x67\x00\x2D\xCE\xEB\x2D\xCE"
		       "\xEB\xB4\x51\x72\xB4\x51\x72\x1F",
		.klen = 24,
		.iv = "\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF",
		.ptext = "\x05\xEC\x77\xFB\x42\xD5\x59\x20"
			 "\x8B\x12\x86\x69\xF0\x5B\xCF\x56"
			 "\x39\xAD\x34\x9F\x66\xEA\x7D\xC4"
			 "\x48\xD3\xBA\x0D\xB1\x18\xE3\x4A"
			 "\xFE\x41\x28\x5C\x27\x8E\x11\x85"
			 "\x6C\xF7\x5E\xC2\x55\x3C\xA0\x0B"
			 "\x92\x65\xE9\x70\xDB\x4F\xD6\xB9"
			 "\x00\xB4\x1F\xE6\x49\xFD\x44\x2F"
			 "\x53\x3A\x8D\x14\x98\x63\xCA\x5D"
			 "\xC1\xA8\x33\xA7\x0E\x91\x78\xEC"
			 "\x77\xDE\x42\xD5\xBC\x07\x8B\x12"
			 "\xE5\x4C\xF0\x5B\x22\x56\x39\x80"
			 "\x6B\x9F\x66\xC9\x50\xC4\xAF\x36"
			 "\xBA\x0D\x94\x7F\xE3\x4A\xDD\x41"
			 "\x28\xB3\x1A\x8E\x11\xF8\x43\xF7"
			 "\x5E\x21\x55\x3C\x87\x6E\x92\x65"
			 "\xCC\x57\xDB\xA2\x35\xB9\x00\xEB"
			 "\x72\xE6\x49\xD0\x44\x2F\xB6\x19"
			 "\x8D\x14\xFF\x46\xCA\x5D\x24\xA8"
			 "\x33\x9A\x6D\x91\x78\xC3\x77\xDE"
			 "\xA1\x08\xBC\x07\xEE\x71\xE5\x4C"
			 "\xD7\x5B\x22\xB5\x1C\x80\x6B\xF2"
			 "\x45\xC9\x50\x3B\xAF\x36\x99\x60"
			 "\x94\x7F\xC6\x4A\xDD\xA4\x0F\xB3"
			 "\x1A\xED\x74\xF8\x43\x2A\x5E\x21"
			 "\x88\x13\x87\x6E\xF1\x58\xCC\x57"
			 "\x3E\xA2\x35\x9C\x67\xEB\x72\xC5"
			 "\x49\xD0\xBB\x02\xB6\x19\xE0\x4B"
			 "\xFF\x46\x29\x5D\x24\x8F\x16\x9A"
			 "\x6D\xF4\x5F\xC3\xAA\x3D\xA1\x08"
			 "\x93\x7A\xEE\x71\xD8\x4C\xD7\xBE"
			 "\x01\xB5\x1C\xE7\x4E\xF2\x45\x2C"
			 "\x50\x3B\x82\x15\x99\x60\xCB\x52"
			 "\xC6\xA9\x30\xA4\x0F\x96\x79\xED"
			 "\x74\xDF\x43\x2A\xBD\x04\x88\x13"
			 "\xFA\x4D\xF1\x58\x23\x57\x3E\x81"
			 "\x68\x9C\x67\xCE\x51\xC5\xAC\x37"
			 "\xBB\x02\x95\x7C\xE0\x4B\xD2\x46"
			 "\x29\xB0\x1B\x8F\x16\xF9\x40\xF4"
			 "\x5F\x26\xAA\x3D\x84\x6F\x93\x7A"
			 "\xCD\x54\xD8\xA3\x0A\xBE\x01\xE8"
			 "\x73\xE7\x4E\xD1\x45\x2C\xB7\x1E"
			 "\x82\x15\xFC\x47\xCB\x52\x25\xA9"
			 "\x30\x9B\x62\x96\x79\xC0\x74\xDF"
			 "\xA6\x09\xBD\x04\xEF\x76\xFA\x4D"
			 "\xD4\x58\x23\x8A\x1D\x81\x68\xF3"
			 "\x5A\xCE\x51\x38\xAC\x37\x9E\x61"
			 "\x95\x7C\xC7\x4B\xD2\xA5\x0C\xB0"
			 "\x1B\xE2\x75\xF9\x40\x2B\x5F\x26"
			 "\x89\x10\x84\x6F\xF6\x59\xCD\x54"
			 "\x3F\xA3\x0A\x9D\x64\xE8\x73\xDA"
			 "\x4E\xD1\xB8\x03\xB7\x1E\xE1\x48"
			 "\xFC\x47\x2E\x52\x25\x8C\x17\x9B"
			 "\x62\xF5\x5C\xC0\xAB\x32\xA6\x09"
			 "\x90\x7B\xEF\x76\xD9\x4D\xD4\xBF"
			 "\x06\x8A\x1D\xE4\x4F\xF3\x5A\x2D"
			 "\x51\x38\x83\x6A\x9E\x61\xC8\x53"
			 "\xC7\xAE\x31\xA5\x0C\x97\x7E\xE2"
			 "\x75\xDC\x40\x2B\xB2\x05\x89\x10"
			 "\xFB\x42\xF6\x59\x20\x54\x3F\x86"
			 "\x69\x9D\x64\xCF\x56\xDA\xAD\x34"
			 "\xB8\x03\xEA\x7D\xE1\x48\xD3\x47",
		.ctext = "\x07\xC2\x08\x20\x72\x1F\x49\xEF"
			 "\x19\xCD\x6F\x32\x53\x05\x22\x15"
			 "\xA2\x85\x2B\xDB\x85\xD2\xD8\xB9"
			 "\xDD\x0D\x1B\x45\xCB\x69\x11\xD4"
			 "\xEA\xBE\xB2\x45\x5D\x0C\xAE\xBE"
			 "\xA0\xC1\x27\xAC\x65\x9F\x53\x7E"
			 "\xAF\xC2\x1B\xB5\xB8\x6D\x36\x0C"
			 "\x25\xC0\xF8\x6D\x0B\x29\x01\xDA"
			 "\x13\x78\xDC\x89\x12\x12\x43\xFA"
			 "\xF6\x12\xEF\x8D\x87\x62\x78\x83"
			 "\xE2\xBE\x41\x20\x4C\x6D\x35\x1B"
			 "\xD1\x0C\x30\xCF\xE2\xDE\x2B\x03"
			 "\xBF\x45\x73\xD4\xE5\x59\x95\xD1"
			 "\xB3\x9B\x27\x62\x97\xBD\xDE\x7F"
			 "\xA4\xD2\x39\x80\xAA\x50\x23\xF0"
			 "\x74\x88\x3D\xA8\x6A\x18\x79\x3B"
			 "\xC4\x96\x6C\x8D\x22\x40\x92\x6E"
			 "\xD6\xAD\x2A\x1F\xDE\x63\xC0\xE7"
			 "\x07\xF7\x2D\xF7\xB5\xF3\xF0\xCC"
			 "\x01\x7C\x2A\x9B\xC2\x10\xCA\xAA"
			 "\xFD\x2B\x3F\xC5\xF3\xF6\xFC\x9B"
			 "\x45\xDB\x53\xE4\x5B\xF3\xC9\x7B"
			 "\x8E\x52\xFF\xC8\x02\xB8\xAC\x9D"
			 "\xA1\x00\x39\xDA\x3D\x2D\x0E\x01"
			 "\x09\x7D\x8D\x5E\xBE\x53\xB9\xB0"
			 "\x8E\xE7\xE2\x96\x6A\xB2\x78\xEA"
			 "\xDE\x23\x8B\xA5\xFA\x5C\xE3\xDA"
			 "\xBF\x8E\x31\x6A\x55\xD1\x6A\xB2"
			 "\xB5\x46\x6F\xA5\xF0\xEE\xBA\x1F"
			 "\x9F\x98\xB0\x66\x4F\xD0\x3F\xA9"
			 "\xDF\x5F\x58\xC4\xF4\xFF\x75\x5C"
			 "\x40\x3A\x09\x7E\x6E\x1C\x97\xD4"
			 "\xCC\xE7\xE7\x71\xCF\x0B\x15\x08"
			 "\x71\xFA\x07\x97\xCD\xE6\xCA\x1D"
			 "\x14\x28\x0C\xCF\x99\x13\x7A\xF1"
			 "\xEB\xFA\xFA\x92\x07\xDE\x1D\xA1"
			 "\xD3\x36\x69\xFE\x51\x4D\x9F\x2E"
			 "\x83\x37\x4F\x1F\x48\x30\xED\x04"
			 "\x4D\xA4\xEF\x3A\xCA\x76\xF4\x1C"
			 "\x41\x8F\x63\x37\x78\x2F\x86\xA6"
			 "\xEF\x41\x7E\xD2\xAF\x88\xAB\x67"
			 "\x52\x71\xC3\x8E\xF8\x26\x93\x72"
			 "\xAA\xD6\x0E\xE7\x0B\x46\xB1\x3A"
			 "\xB4\x08\xA9\xA8\xA0\xCF\x20\x0C"
			 "\x52\xBC\x8B\x05\x56\xB2\xBC\x31"
			 "\x9B\x74\xB9\x29\x29\x96\x9A\x50"
			 "\xDC\x45\xDC\x1A\xEB\x0C\x64\xD4"
			 "\xD3\x05\x7E\x59\x55\xC3\xF4\x90"
			 "\xC2\xAB\xF8\x9B\x8A\xDA\xCE\xA1"
			 "\xC3\xF4\xAD\x77\xDD\x44\xC8\xAC"
			 "\xA3\xF1\xC9\xD2\x19\x5C\xB0\xCA"
			 "\xA2\x34\xC1\xF7\x6C\xFD\xAC\x65"
			 "\x32\xDC\x48\xC4\xF2\x00\x6B\x77"
			 "\xF1\x7D\x76\xAC\xC0\x31\x63\x2A"
			 "\xA5\x3A\x62\xC8\x91\xB1\x03\x65"
			 "\xCB\x43\xD1\x06\xDF\xC3\x67\xBC"
			 "\xDC\xE0\xCD\x35\xCE\x49\x65\xA0"
			 "\x52\x7B\xA7\x0D\x07\xA9\x1B\xB0"
			 "\x40\x77\x72\xC2\xEA\x0E\x3A\x78"
			 "\x46\xB9\x91\xB6\xE7\x3D\x51\x42"
			 "\xFD\x51\xB0\xC6\x2C\x63\x13\x78"
			 "\x5C\xEE\xFC\xCF\xC4\x70\x00\x34",
		.len = 496,
	},
	{
		/* Generated with Crypto++ */
		.ilen = 8,
		.alg = SCE_TDES,
		.mode = CTR,
		.key = "\x9C\xD6\xF3\x9C\xB9\x5A\x67\x00"
		       "\x5A\x67\x00\x2D\xCE\xEB\x2D\xCE"
		       "\xEB\xB4\x51\x72\xB4\x51\x72\x1F",
		.klen = 24,
		.iv = "\xB2\xD7\x48\xED\x06\x44\xF9\x12",
		.ptext = "\x05\xEC\x77\xFB\x42\xD5\x59\x20"
			 "\x8B\x12\x86\x69\xF0\x5B\xCF\x56"
			 "\x39\xAD\x34\x9F\x66\xEA\x7D\xC4"
			 "\x48\xD3\xBA\x0D\xB1\x18\xE3\x4A"
			 "\xFE\x41\x28\x5C\x27\x8E\x11\x85"
			 "\x6C\xF7\x5E\xC2\x55\x3C\xA0\x0B"
			 "\x92\x65\xE9\x70\xDB\x4F\xD6\xB9"
			 "\x00\xB4\x1F\xE6\x49\xFD\x44\x2F"
			 "\x53\x3A\x8D\x14\x98\x63\xCA\x5D"
			 "\xC1\xA8\x33\xA7\x0E\x91\x78\xEC"
			 "\x77\xDE\x42\xD5\xBC\x07\x8B\x12"
			 "\xE5\x4C\xF0\x5B\x22\x56\x39\x80"
			 "\x6B\x9F\x66\xC9\x50\xC4\xAF\x36"
			 "\xBA\x0D\x94\x7F\xE3\x4A\xDD\x41"
			 "\x28\xB3\x1A\x8E\x11\xF8\x43\xF7"
			 "\x5E\x21\x55\x3C\x87\x6E\x92\x65"
			 "\xCC\x57\xDB\xA2\x35\xB9\x00\xEB"
			 "\x72\xE6\x49\xD0\x44\x2F\xB6\x19"
			 "\x8D\x14\xFF\x46\xCA\x5D\x24\xA8"
			 "\x33\x9A\x6D\x91\x78\xC3\x77\xDE"
			 "\xA1\x08\xBC\x07\xEE\x71\xE5\x4C"
			 "\xD7\x5B\x22\xB5\x1C\x80\x6B\xF2"
			 "\x45\xC9\x50\x3B\xAF\x36\x99\x60"
			 "\x94\x7F\xC6\x4A\xDD\xA4\x0F\xB3"
			 "\x1A\xED\x74\xF8\x43\x2A\x5E\x21"
			 "\x88\x13\x87\x6E\xF1\x58\xCC\x57"
			 "\x3E\xA2\x35\x9C\x67\xEB\x72\xC5"
			 "\x49\xD0\xBB\x02\xB6\x19\xE0\x4B"
			 "\xFF\x46\x29\x5D\x24\x8F\x16\x9A"
			 "\x6D\xF4\x5F\xC3\xAA\x3D\xA1\x08"
			 "\x93\x7A\xEE\x71\xD8\x4C\xD7\xBE"
			 "\x01\xB5\x1C\xE7\x4E\xF2\x45\x2C"
			 "\x50\x3B\x82\x15\x99\x60\xCB\x52"
			 "\xC6\xA9\x30\xA4\x0F\x96\x79\xED"
			 "\x74\xDF\x43\x2A\xBD\x04\x88\x13"
			 "\xFA\x4D\xF1\x58\x23\x57\x3E\x81"
			 "\x68\x9C\x67\xCE\x51\xC5\xAC\x37"
			 "\xBB\x02\x95\x7C\xE0\x4B\xD2\x46"
			 "\x29\xB0\x1B\x8F\x16\xF9\x40\xF4"
			 "\x5F\x26\xAA\x3D\x84\x6F\x93\x7A"
			 "\xCD\x54\xD8\xA3\x0A\xBE\x01\xE8"
			 "\x73\xE7\x4E\xD1\x45\x2C\xB7\x1E"
			 "\x82\x15\xFC\x47\xCB\x52\x25\xA9"
			 "\x30\x9B\x62\x96\x79\xC0\x74\xDF"
			 "\xA6\x09\xBD\x04\xEF\x76\xFA\x4D"
			 "\xD4\x58\x23\x8A\x1D\x81\x68\xF3"
			 "\x5A\xCE\x51\x38\xAC\x37\x9E\x61"
			 "\x95\x7C\xC7\x4B\xD2\xA5\x0C\xB0"
			 "\x1B\xE2\x75\xF9\x40\x2B\x5F\x26"
			 "\x89\x10\x84\x6F\xF6\x59\xCD\x54"
			 "\x3F\xA3\x0A\x9D\x64\xE8\x73\xDA"
			 "\x4E\xD1\xB8\x03\xB7\x1E\xE1\x48"
			 "\xFC\x47\x2E\x52\x25\x8C\x17\x9B"
			 "\x62\xF5\x5C\xC0\xAB\x32\xA6\x09"
			 "\x90\x7B\xEF\x76\xD9\x4D\xD4\xBF"
			 "\x06\x8A\x1D\xE4\x4F\xF3\x5A\x2D"
			 "\x51\x38\x83\x6A\x9E\x61\xC8\x53"
			 "\xC7\xAE\x31\xA5\x0C\x97\x7E\xE2"
			 "\x75\xDC\x40\x2B\xB2\x05\x89\x10"
			 "\xFB\x42\xF6\x59\x20\x54\x3F\x86"
			 "\x69\x9D\x64\xCF\x56\xDA\xAD\x34"
			 "\xB8\x03\xEA\x7D\xE1\x48\xD3\x47"
			 "\x2E\xB1\x18",
		.ctext = "\x23\xFF\x5C\x99\x75\xBB\x1F\xD4"
			 "\xBC\x27\x9D\x36\x60\xA9\xC9\xF7"
			 "\x94\x9D\x1B\xFF\x8E\x95\x57\x89"
			 "\x8C\x2E\x33\x70\x43\x61\xE6\xD2"
			 "\x82\x33\x63\xB6\xC4\x34\x5E\xF8"
			 "\x96\x07\xA7\xD2\x3B\x8E\xC9\xAA"
			 "\x7C\xA0\x55\x89\x2E\xE1\x85\x25"
			 "\x14\x04\xDA\x6B\xE0\xEE\x56\xCF"
			 "\x08\x2E\x69\xD4\x54\xDE\x22\x84"
			 "\x69\xA6\xA7\xD3\x3A\x9A\xE8\x05"
			 "\x63\xDB\xBF\x46\x3A\x26\x2E\x0F"
			 "\x58\x5C\x46\xEA\x07\x40\xDA\xE1"
			 "\x14\x1D\xCD\x4F\x06\xC0\xCA\x54"
			 "\x1E\xC9\x45\x85\x67\x7C\xC2\xB5"
			 "\x97\x5D\x61\x78\x2E\x46\xEC\x6A"
			 "\x53\xF4\xD0\xAE\xFA\xB4\x86\x29"
			 "\x9F\x17\x33\x24\xD8\xB9\xB2\x05"
			 "\x93\x88\xEA\xF7\xA0\x70\x69\x49"
			 "\x88\x6B\x73\x40\x41\x8D\xD9\xD9"
			 "\x7E\x78\xE9\xBE\x6C\x14\x22\x7A"
			 "\x66\xE1\xDA\xED\x10\xFF\x69\x1D"
			 "\xB9\xAA\xF2\x56\x72\x1B\x23\xE2"
			 "\x45\x54\x8B\xA3\x70\x23\xB4\x5E"
			 "\x8E\x96\xC9\x05\x00\xB3\xB6\xC2"
			 "\x2A\x02\x43\x7A\x62\xD5\xC8\xD2"
			 "\xC2\xD0\xE4\x78\xA1\x7B\x3E\xE8"
			 "\x9F\x7F\x7D\x40\x54\x30\x3B\xC0"
			 "\xA5\x54\xFD\xCA\x25\xEC\x44\x3E"
			 "\x1A\x54\x7F\x88\xD0\xE1\xFE\x71"
			 "\xCE\x05\x49\x89\xBA\xD6\x72\xE7"
			 "\xD6\x5D\x3F\xA2\xD9\xAB\xC5\x02"
			 "\xD6\x43\x22\xAF\xA2\xE4\x80\x85"
			 "\xD7\x87\xB9\xEA\x43\xDB\xC8\xEF"
			 "\x5C\x82\x2E\x98\x0D\x30\x41\x6B"
			 "\x08\x48\x8D\xF0\xF8\x60\xD7\x9D"
			 "\xE9\xDE\x40\xAD\x0D\xAD\x0D\x58"
			 "\x2A\x98\x35\xFE\xF7\xDD\x4B\x40"
			 "\xDE\xB0\x05\xD9\x7B\x09\x4D\xBC"
			 "\x42\xC0\xF1\x15\x0B\xFA\x26\x6B"
			 "\xC6\x12\x13\x4F\xCB\x35\xBA\x35"
			 "\xDD\x7A\x36\x9C\x12\x57\x55\x83"
			 "\x78\x58\x09\xD0\xB0\xCF\x7C\x5C"
			 "\x38\xCF\xBD\x79\x5B\x13\x4D\x97"
			 "\xC1\x85\x6F\x97\xC9\xE8\xC2\xA4"
			 "\x98\xE2\xBD\x77\x6B\x53\x39\x1A"
			 "\x28\x10\xE7\xE0\xE7\xDE\x9D\x69"
			 "\x78\x6F\x8E\xD2\xD9\x5D\xD2\x15"
			 "\x9E\xB5\x4D\x8C\xC0\x78\x22\x2F"
			 "\x17\x11\x2E\x99\xD7\xE3\xA4\x4F"
			 "\x65\xA5\x6B\x03\x2C\x35\x6F\xDA"
			 "\x8A\x19\x08\xE1\x08\x48\x59\x51"
			 "\x53\x4B\xD1\xDF\xDA\x14\x50\x5F"
			 "\xDF\xB5\x8C\xDF\xC6\xFD\x85\xFA"
			 "\xD4\xF9\x64\x45\x65\x0D\x7D\xF4"
			 "\xC8\xCD\x3F\x32\xAF\xDD\x30\xED"
			 "\x7B\xAA\xAC\xF0\xDA\x7F\xDF\x75"
			 "\x1C\xA4\xF1\xCB\x5E\x4F\x0B\xB4"
			 "\x97\x73\x28\xDE\xCF\xAF\x82\xBD"
			 "\xC4\xBA\xB4\x9C\x0D\x16\x77\x42"
			 "\x42\x39\x7C\x53\xA4\xD4\xDD\x40"
			 "\x5C\x60\x1F\x6E\xA7\xE2\xDC\xE7"
			 "\x32\x0F\x05\x2F\xF2\x4C\x95\x3B"
			 "\xF2\x79\xD9",
		.len = 499,
	},
};

int sca_tv_template_count(void)
{
	return ARRAY_SIZE(ske_tv_template);
}
int gcm_tv_template_count(void)
{
	return ARRAY_SIZE(gcm_tv_template);
}

int ccm_tv_template_count(void)
{
	return ARRAY_SIZE(ccm_tv_template);
}

static void *ske_io_map(void *usr, void *va)
{
	(void)usr;
	return (void *)sce_virt_to_io(va);
}
int gcm_test_one_vector(int vec_index, bool is_enc, int dma_mode,
			int src_link_num, int dst_link_num)
{
	const gcm_testvec_t *testvec;
	int i, offset, ret = 0, temp_src_link_num = 0, temp_dst_link_num = 0;
	sce_rsp_t *out_rsp = NULL;
	uint8_t *result = NULL, *src = NULL, *key = NULL, *iv = NULL;
	sce_queue_handle sce_queue_desc = NULL;
	void **op_datas = NULL;
	sce_op_data_t *op_buf = NULL;
	void *origal_buf = NULL;
	int count = 1;
	int tmp_src_len = 0, tmp_dst_len = 0;
	int enqueued_count;
	int dequeued_count;
	uint8_t *src_link_items = NULL, *dst_link_items = NULL;
	sce_link_list_item_t *temp_items = NULL;
	uint64_t tmp = 0;
	int k = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = ske_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	if (sce_request_queue(SCE_SCA, &sce_queue_desc)) {
		return -1;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &result);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &key);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &iv);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src_link_items);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &dst_link_items);
	if (ret)
		goto out;
	op_datas = (void **)malloc(sizeof(void *) * count);
	if (!op_datas) {
		SCE_LOG_ERR("alloc op_datas error");
		goto out;
	}

	origal_buf = (sce_op_data_t *)malloc(sizeof(sce_op_data_t) * count);
	if (!origal_buf) {
		SCE_LOG_ERR("alloc op_buf error");
		goto out;
	}
	op_buf = origal_buf;

	// dma_mode data
	dst_link_num = dst_link_num <= 32 ? dst_link_num : 32;
	src_link_num = src_link_num <= 32 ? src_link_num : 32;

	offset = 0;
	for (i = 0; i < 1; i++) {
		op_datas[i] = op_buf;
		op_buf->op_type = SCE_SCA;
		testvec = &gcm_tv_template[vec_index];
		memset(result + offset, 0xa5, testvec->clen + 64);
		op_buf->sca.out_length = testvec->clen;

		memcpy(src + offset, testvec->assoc, testvec->alen);
		if (is_enc) {
			memcpy(src + offset + testvec->alen, testvec->ptext,
			       testvec->plen);
			tmp_src_len = testvec->plen + testvec->alen;
			tmp_dst_len = testvec->clen;
		} else {
			memcpy(src + offset + testvec->alen, testvec->ctext,
			       testvec->clen);
			tmp_src_len = testvec->clen + testvec->alen;
			tmp_dst_len = testvec->plen;
		}
		if (dma_mode) {
			op_buf->sca.dma_mode = 0;
			if (dma_mode & 0x1) { // dst
				temp_items =
					(sce_link_list_item_t
						 *)(dst_link_items +
						    i * (dst_link_num + 3) *
							    sizeof(sce_link_list_item_t));
				if (((uint64_t)temp_items) & 0x1f) {
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items +
							    0x1f);
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items &
							    (~0x1f));
				}

				temp_dst_link_num = dst_link_num < tmp_dst_len ?
							    dst_link_num :
							    tmp_dst_len;
				tmp = sce_virt_to_io(result + offset);
				for (k = 0; k < temp_dst_link_num - 1; k++) {
					temp_items[k].address = tmp + k;
					temp_items[k].size = 0;
					temp_items[k].reserved = 0;
				}
				temp_items[k].address = tmp + k;
				temp_items[k].size = tmp_dst_len - k - 1;
				temp_items[k].reserved = 0;
				k++;

				temp_items[k].address = SCE_LINK_END_ADDRESS;
				temp_items[k].size = SCE_LINK_END_SIZE;
				temp_items[k].reserved = SCE_LINK_END_SIZE;

				op_buf->sca.out = (uint64_t)temp_items;
				op_buf->sca.dma_mode |= 0x1;
			} else {
				op_buf->sca.out = (uint64_t)result + offset;
			}

			if (dma_mode & 0x2) { // src
				temp_items =
					(sce_link_list_item_t
						 *)(src_link_items +
						    i * (src_link_num + 3) *
							    sizeof(sce_link_list_item_t));
				if (((uint64_t)temp_items) & 0x1f) {
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items +
							    0x1f);
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items &
							    (~0x1f));
				}

				temp_src_link_num = src_link_num < tmp_src_len ?
							    src_link_num :
							    tmp_src_len;
				tmp = sce_virt_to_io(src + offset);
				for (k = 0; k < temp_src_link_num - 1; k++) {
					temp_items[k].address = tmp + k;
					temp_items[k].size = 0;
					temp_items[k].reserved = 0;
				}

				temp_items[k].address = tmp + k;
				temp_items[k].size = tmp_src_len - k - 1;
				temp_items[k].reserved = 0;
				k++;

				temp_items[k].address = SCE_LINK_END_ADDRESS;
				temp_items[k].size = SCE_LINK_END_SIZE;
				temp_items[k].reserved = SCE_LINK_END_SIZE;
				k++;

				op_buf->sca.in = (uint64_t)temp_items;
				op_buf->sca.dma_mode |= 0x2;
			} else {
				op_buf->sca.in = (uint64_t)src + offset;
			}
		} else {
			op_buf->sca.out = (uint64_t)result + offset;
			op_buf->sca.in = (uint64_t)src + offset;
			op_buf->sca.dma_mode = 0;
		}

		op_buf->sca.in_length = testvec->plen;

		memcpy(key + offset, testvec->key, testvec->klen);
		op_buf->sca.key = (uint64_t)key + offset;

		if (testvec->ilen) {
			memcpy(iv + offset, testvec->iv, testvec->ilen);
			op_buf->sca.iv = (uint64_t)iv + offset;
		} else {
			op_buf->sca.iv = 0;
		}
		op_buf->sca.alg = testvec->alg;
		op_buf->sca.mode = testvec->mode;
		op_buf->sca.op_mode = is_enc ? SCE_ENC : SCE_DEC; //SCE_ENC;
		op_buf->sca.tag = i;
		op_buf->sca.aad_size = testvec->alen;
		op_buf->sca.tag_size = testvec->clen - testvec->plen;

		op_buf++;

		offset += testvec->clen + 80;
		offset += (64 - (offset & 0x3F)) & 0x3F;
	}
	enqueued_count = sce_enqueue(sce_queue_desc, op_datas, count);
	if (!enqueued_count)
		goto out;

	count = enqueued_count;
	out_rsp = malloc(sizeof(sce_rsp_t) * count);
	if (NULL == out_rsp) {
		goto out;
	}

	while (enqueued_count && running) {
		dequeued_count =
			sce_dequeue(sce_queue_desc, out_rsp, enqueued_count);
		if (0 == dequeued_count) {
			usleep(1000);
			continue;
		}
		enqueued_count -= dequeued_count;
		for (i = 0; i < dequeued_count; i++) {
			sce_rsp_t *rsp = &out_rsp[i];
			if (rsp->state != CMD_SUCCESS) {
				SCE_LOG_ERR(
					"gcm[%d] enc[%d] fail!alg[0x%x], len[%d], res[0x%x]\n",
					vec_index, is_enc, testvec->alg,
					testvec->plen, rsp->state);
				goto out;
			}

			testvec = &gcm_tv_template[vec_index];
			if (is_enc) {
				if (memcmp(result, testvec->ctext,
					   testvec->clen)) {
					SCE_LOG_ERR(
						"gcm[%d] enc error!alg[0x%x], len[%d]\n",
						vec_index, testvec->alg,
						testvec->plen);
					print_buf("ctext", result,
						  testvec->clen);
					print_buf("testvec ctext",
						  (uint8_t *)testvec->ctext,
						  testvec->clen);
					ret = -1;

					goto out;
				}
			} else {
				if (memcmp(result, testvec->ptext,
					   testvec->plen)) {
					SCE_LOG_ERR(
						"gcm[%d] dec error!alg[0x%x], len[%d]\n",
						vec_index, testvec->alg,
						testvec->plen);
					print_buf("ptext", result,
						  testvec->plen);
					print_buf("testvec ptext",
						  (uint8_t *)testvec->ptext,
						  testvec->plen);
					ret = -1;
					goto out;
				}
			}
		}
	}
	ret = 0;
out:
	if (op_datas)
		free(op_datas);

	if (origal_buf)
		free(origal_buf);

	if (out_rsp)
		free(out_rsp);

	sce_stop_queue(sce_queue_desc);
	if (sce_queue_desc)
		sce_release_queue(sce_queue_desc);

	sce_release_dmabuf(result);

	sce_release_dmabuf(src);

	sce_release_dmabuf(key);

	sce_release_dmabuf(iv);

	sce_release_dmabuf(src_link_items);

	sce_release_dmabuf(dst_link_items);

	sce_exit_lib();

	return ret;
}
int gcm_test(int dma_mode, int src_link_num, int dst_link_num)
{
	int i;
	int err_num = 0;
	for (i = 0; i < ARRAY_SIZE(gcm_tv_template); i++) {
		if (gcm_test_one_vector(i, true, dma_mode, src_link_num,
					dst_link_num)) {
			SCE_LOG_ERR("GCM[%d] enc test failed! ! !\n", i);
			err_num++;
		}
		if (gcm_test_one_vector(i, false, dma_mode, src_link_num,
					dst_link_num)) {
			SCE_LOG_ERR("GCM[%d] dec test failed! ! !\n", i);
			err_num++;
		}
	}
	if (!err_num)
		SCE_LOG_ALWAYS("test gcm success!\n");
	return 0;
}

int ccm_test_one_vector(int vec_index, bool is_enc, int dma_mode,
			int src_link_num, int dst_link_num)
{
	const ccm_testvec_t *testvec;
	ccm_flag_t *flag;
	int i, aad_head_len = 0, offset, ret = 0, temp_src_link_num = 0,
	       temp_dst_link_num = 0;
	sce_rsp_t *out_rsp = NULL;
	uint8_t *result = NULL, *src = NULL, *ccm_mac_src = NULL, *key = NULL,
		*iv = NULL;
	sce_queue_handle sce_queue_desc = NULL;
	void **op_datas = NULL;
	sce_op_data_t *op_buf = NULL;
	void *origal_buf = NULL;
	int count = 1;
	int enqueued_count;
	int dequeued_count;
	int tmp_src_len = 0, tmp_dst_len = 0;
	uint8_t *src_link_items = NULL, *dst_link_items = NULL;
	sce_link_list_item_t *temp_items = NULL;
	uint64_t tmp = 0;
	int k = 0;
	//    int link_items_num = 4;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = ske_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	if (sce_request_queue(SCE_SCA, &sce_queue_desc)) {
		return -1;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &result);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &ccm_mac_src);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &key);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &iv);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src_link_items);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &dst_link_items);
	if (ret)
		goto out;

	op_datas = (void **)malloc(sizeof(void *) * count);
	if (!op_datas) {
		SCE_LOG_ERR("alloc op_datas error");
		goto out;
	}

	origal_buf = (sce_op_data_t *)malloc(sizeof(sce_op_data_t) * count);
	if (!origal_buf) {
		SCE_LOG_ERR("alloc op_buf error");
		goto out;
	}
	op_buf = origal_buf;
	dst_link_num = dst_link_num <= 32 ? dst_link_num : 32;
	src_link_num = src_link_num <= 32 ? src_link_num : 32;

	offset = 0;
	for (i = 0; i < 1; i++) {
		op_datas[i] = op_buf;
		op_buf->op_type = SCE_SCA;
		testvec = &ccm_tv_template[vec_index];

		op_buf->sca.alg = testvec->alg;
		op_buf->sca.mode = testvec->mode;
		op_buf->sca.op_mode = is_enc ? SCE_ENC : SCE_DEC;
		op_buf->sca.tag = i;
		op_buf->sca.in_length = testvec->plen;
		op_buf->sca.out_length = testvec->clen;
		op_buf->sca.tag_size = testvec->clen - testvec->plen;

		memset(result, 0xa5, testvec->clen + 64);
		memcpy(src, testvec->iv, testvec->ilen);
		flag = (ccm_flag_t *)(src);
		flag->tlen = (op_buf->sca.tag_size - 2) / 2;
		flag->aad = !!testvec->alen;
		flag->reserved = 0;
		for (k = 0; k <= flag->qlen; k++) {
			src[offset + 15 - k] =
				(op_buf->sca.in_length >> (8 * k)) & 0xFF;
		}
		/* B1-Bu */
		if (testvec->alen) {
			if (testvec->alen < (0x10000 - 0x100)) {
				for (k = 0; k < 2; k++) {
					src[17 - k] =
						(testvec->alen >> (8 * k)) &
						0xFF;
				}
				aad_head_len = 2;
			} else if (testvec->alen <= 0xFFFFFFFF) {
				src[17] = 0xFF;
				src[18] = 0xFE;
				for (k = 0; k < 4; k++) {
					src[21 - k] =
						(testvec->alen >> (8 * k)) &
						0xFF;
				}
				aad_head_len = 6;
			} else {
				src[17] = 0xFF;
				src[18] = 0xFF;
				for (k = 0; k < 8; k++) {
					src[25 - k] =
						(testvec->alen >> (8 * k)) &
						0xFF;
				}
				aad_head_len = 10;
			}
			memcpy(src + 16 + aad_head_len, testvec->assoc,
			       testvec->alen);
		} else {
			aad_head_len = 0;
		}
		op_buf->sca.aad_size = 16 + aad_head_len + testvec->alen;

		/* Bu1-Bm*/
		if (is_enc) {
			if (testvec->plen) {
				memcpy(src + op_buf->sca.aad_size,
				       testvec->ptext, testvec->plen);
			}
			tmp_src_len = testvec->plen + op_buf->sca.aad_size;
			tmp_dst_len = testvec->clen;
		} else {
			if (testvec->clen)
				memcpy(src + offset + op_buf->sca.aad_size,
				       testvec->ctext, testvec->clen);
			tmp_src_len = testvec->clen + op_buf->sca.aad_size;
			tmp_dst_len = testvec->plen;
		}
		if (dma_mode) {
			op_buf->sca.dma_mode = 0;
			if (dma_mode & 0x1) { // dst
				temp_items =
					(sce_link_list_item_t
						 *)(dst_link_items +
						    i * (dst_link_num + 3) *
							    sizeof(sce_link_list_item_t));
				if (((uint64_t)temp_items) & 0x1f) {
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items +
							    0x1f);
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items &
							    (~0x1f));
				}

				temp_dst_link_num = dst_link_num < tmp_dst_len ?
							    dst_link_num :
							    tmp_dst_len;
				tmp = sce_virt_to_io(result + offset);
				for (k = 0; k < temp_dst_link_num - 1; k++) {
					temp_items[k].address = tmp + k;
					temp_items[k].size = 0;
					temp_items[k].reserved = 0;
				}
				temp_items[k].address = tmp + k;
				temp_items[k].size = tmp_dst_len - k - 1;
				temp_items[k].reserved = 0;
				k++;

				temp_items[k].address = SCE_LINK_END_ADDRESS;
				temp_items[k].size = SCE_LINK_END_SIZE;
				temp_items[k].reserved = SCE_LINK_END_SIZE;

				op_buf->sca.out = (uint64_t)temp_items;
				op_buf->sca.dma_mode |= 0x1;
			} else {
				op_buf->sca.out = (uint64_t)result + offset;
			}
			if (dma_mode & 0x2) { // src
				temp_items =
					(sce_link_list_item_t
						 *)(src_link_items +
						    i * (src_link_num + 3) *
							    sizeof(sce_link_list_item_t));
				if (((uint64_t)temp_items) & 0x1f) {
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items +
							    0x1f);
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items &
							    (~0x1f));
				}

				temp_src_link_num = src_link_num < tmp_src_len ?
							    src_link_num :
							    tmp_src_len;
				tmp = sce_virt_to_io(src + offset);
				for (k = 0; k < temp_src_link_num - 1; k++) {
					temp_items[k].address = tmp + k;
					temp_items[k].size = 0;
					temp_items[k].reserved = 0;
				}

				temp_items[k].address = tmp + k;
				temp_items[k].size = tmp_src_len - k - 1;
				temp_items[k].reserved = 0;
				k++;

				temp_items[k].address = SCE_LINK_END_ADDRESS;
				temp_items[k].size = SCE_LINK_END_SIZE;
				temp_items[k].reserved = SCE_LINK_END_SIZE;
				k++;

				op_buf->sca.in = (uint64_t)temp_items;
				op_buf->sca.dma_mode |= 0x2;
			} else {
				op_buf->sca.in = (uint64_t)src + offset;
			}
		} else {
			op_buf->sca.out = (uint64_t)result + offset;
			op_buf->sca.in = (uint64_t)src + offset;
			op_buf->sca.dma_mode = 0;
		}

		memcpy(key, testvec->key, testvec->klen);
		op_buf->sca.key = (uint64_t)key; // + offset;
		memcpy(iv, testvec->iv, testvec->ilen);
		op_buf->sca.iv = (uint64_t)iv; // + offset;

		op_buf->sca.tag = vec_index;

		op_buf++;
	}
	enqueued_count = sce_enqueue(sce_queue_desc, op_datas, count);
	if (!enqueued_count) {
		SCE_LOG_ERR("enqueue error\n");
		goto out;
	}
	count = enqueued_count;
	out_rsp = malloc(sizeof(sce_rsp_t) * count);
	if (NULL == out_rsp) {
		SCE_LOG_ERR("malloc out_rsp error\n");
		ret = -1;
		goto out;
	}

	while (enqueued_count && running) {
		dequeued_count =
			sce_dequeue(sce_queue_desc, out_rsp, enqueued_count);
		if (0 == dequeued_count) {
			usleep(1000);
			continue;
		}

		enqueued_count -= dequeued_count;
		for (i = 0; i < dequeued_count; i++) {
			sce_rsp_t *rsp = &out_rsp[i];
			if (rsp->state != CMD_SUCCESS) {
				SCE_LOG_ERR(
					"ccm[%d] enc[%d] fail!alg[0x%x], len[%d], res[0x%x]\n",
					vec_index, is_enc, testvec->alg,
					testvec->plen, rsp->state);
				ret = -1;
				goto out;
			}
			testvec = &ccm_tv_template[vec_index];
			if (is_enc) {
				if (memcmp(result, testvec->ctext,
					   testvec->clen)) {
					SCE_LOG_ERR(
						"ccm[%d] enc error!alg[0x%x], len[%d]\n",
						vec_index, testvec->alg,
						testvec->plen);
					print_buf("ctext", result,
						  testvec->clen);
					print_buf("testvec ctext",
						  (uint8_t *)testvec->ctext,
						  testvec->clen);
					ret = -1;
					goto out;
				}
			} else {
				if (memcmp(result, testvec->ptext,
					   testvec->plen)) {
					SCE_LOG_ERR(
						"ccm[%d] dec error!alg[0x%x], len[%d]\n",
						vec_index, testvec->alg,
						testvec->plen);
					print_buf("ptext", result,
						  testvec->plen);
					print_buf("testvec ptext",
						  (uint8_t *)testvec->ptext,
						  testvec->plen);
					ret = -1;
					goto out;
				}
			}
		}
	}
	ret = 0;

out:
	if (op_datas)
		free(op_datas);

	if (origal_buf)
		free(origal_buf);

	if (out_rsp)
		free(out_rsp);

	if (sce_queue_desc) {
		sce_stop_queue(sce_queue_desc);
		sce_release_queue(sce_queue_desc);
	}
	// release dma buf
	sce_release_dmabuf(result);

	sce_release_dmabuf(ccm_mac_src);

	sce_release_dmabuf(src);

	sce_release_dmabuf(key);

	sce_release_dmabuf(iv);

	sce_release_dmabuf(src_link_items);

	sce_release_dmabuf(dst_link_items);

	sce_exit_lib();

	return ret;
}

int ccm_test(int dma_mode, int src_link_num, int dst_link_num)
{
	int i, err_num = 0;
	for (i = 0; i < ARRAY_SIZE(ccm_tv_template); i++) {
		if (ccm_test_one_vector(i, true, dma_mode, src_link_num,
					dst_link_num)) {
			SCE_LOG_ERR("CCM[%d] test failed! ! !\n", i);
			err_num++;
			return -1;
		}

		if (ccm_test_one_vector(i, false, dma_mode, src_link_num,
					dst_link_num)) {
			SCE_LOG_ERR("CCM[%d] test failed! ! !\n", i);
			err_num++;
			return -1;
		}
	}
	if (!err_num)
		SCE_LOG_ALWAYS("test ccm success!\n");
	return 0;
}
int sca_mac_test(int dma_mode, int src_link_num, int dst_link_num)
{
	const ske_testvec_t *testvec;
	int i, k, offset, index, enc, ret = 0, temp_src_link_num = 0;
	sce_rsp_t *out_rsp = NULL;
	uint8_t *result = NULL, *src = NULL, *key = NULL, *iv = NULL,
		*tag = NULL;
	sce_queue_handle sce_queue_desc = NULL;
	void **op_datas = NULL;
	sce_op_data_t *op_buf = NULL;
	void *origal_buf = NULL;
	int count = ARRAY_SIZE(ske_mac_tv_template);
	int enqueued_count;
	int dequeued_count;
	uint8_t *src_link_items = NULL, *dst_link_items = NULL;
	sce_link_list_item_t *temp_items = NULL;
	uint64_t tmp = 0;
	int err_num = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = ske_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	if (sce_request_queue(SCE_SCA, &sce_queue_desc)) {
		return -1;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &result);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &key);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &iv);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src_link_items);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &dst_link_items);
	if (ret)
		goto out;

	op_datas = (void **)malloc(sizeof(void *) * count);
	if (!op_datas) {
		SCE_LOG_ERR("alloc op_datas error");
		goto out;
	}

	origal_buf = (sce_op_data_t *)malloc(sizeof(sce_op_data_t) * count);
	if (!origal_buf) {
		SCE_LOG_ERR("alloc op_buf error");
		goto out;
	}

	op_buf = origal_buf;

	// dma_mode data
	//dst_link_num = dst_link_num <= 32 ? dst_link_num : 32;
	src_link_num = src_link_num <= 32 ? src_link_num : 32;

	if (dma_mode & 0x1) {
		SCE_LOG_ERR("mac test unsupport dst dma mode!\n");
		return -1;
	}

	offset = 0;
	for (i = 0; i < ARRAY_SIZE(ske_mac_tv_template); i++) {
		op_datas[i] = op_buf;
		op_buf->op_type = SCE_SCA;
		testvec = &ske_mac_tv_template[i];
		memset(result + offset, 0xa5, testvec->len + 64);
		op_buf->sca.out_length = testvec->len;

		memcpy(src + offset, testvec->ptext, testvec->len);
		op_buf->sca.in_length = testvec->len;

		memcpy(key + offset, testvec->key, testvec->klen);
		op_buf->sca.key = (uint64_t)key + offset;

		if (testvec->ilen) {
			memcpy(iv + offset, testvec->iv, testvec->ilen);
			op_buf->sca.iv = (uint64_t)iv + offset;
		} else {
			op_buf->sca.iv = 0;
		}
		op_buf->sca.alg = testvec->alg;
		op_buf->sca.mode = testvec->mode;
		op_buf->sca.op_mode = SCE_ENC;
		op_buf->sca.tag = i;

		if (dma_mode) {
			op_buf->sca.dma_mode = 0;

			op_buf->sca.out = (uint64_t)result + offset;

			if (dma_mode & 0x2 && testvec->len > 1) { // src
				temp_items =
					(sce_link_list_item_t
						 *)(src_link_items +
						    i * (src_link_num + 3) *
							    sizeof(sce_link_list_item_t));
				if (((uint64_t)temp_items) & 0x1f) {
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items +
							    0x1f);
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items &
							    (~0x1f));
				}

				temp_src_link_num =
					src_link_num < testvec->len ?
						src_link_num :
						testvec->len;
				tmp = sce_virt_to_io(src + offset);
				for (k = 0; k < temp_src_link_num - 1; k++) {
					temp_items[k].address = tmp + k;
					temp_items[k].address |= 0x300000000000;
					temp_items[k].size = 0;
					temp_items[k].reserved = 0;
				}
				// end of link_list
				temp_items[k].address = tmp + k;
				temp_items[k].address |= 0x300000000000;
				temp_items[k].size = testvec->len - k - 1;
				temp_items[k].reserved = 0;
				k++;

				//cbcmac len add 0
				if (testvec->len & 0xf &&
				    testvec->mode == CBC_MAC &&
				    testvec->alg != SCE_DES &&
				    testvec->alg != SCE_TDES) {
					temp_items[k].address =
						tmp + testvec->len;
					temp_items[k].address |= 0x300000000000;
					temp_items[k].size =
						16 - (testvec->len & 0xf) - 1;
					memset(src + offset + testvec->len, 0,
					       16 - (testvec->len & 0xf));
					temp_items[k].reserved = 0;
					op_buf->sca.in_length +=
						16 - (testvec->len & 0xf);
					k++;
				}

				if (testvec->len & 0x7 &&
				    testvec->mode == CBC_MAC &&
				    (testvec->alg == SCE_DES ||
				     testvec->alg == SCE_TDES)) {
					temp_items[k].address =
						tmp + testvec->len;
					temp_items[k].address |= 0x300000000000;
					temp_items[k].size =
						8 - (testvec->len & 0x7) - 1;
					memset(src + offset + testvec->len, 0,
					       8 - (testvec->len & 0x7));
					temp_items[k].reserved = 0;
					op_buf->sca.in_length +=
						8 - (testvec->len & 0x7);
					k++;
				}

				temp_items[k].address = SCE_LINK_END_ADDRESS;
				temp_items[k].size = SCE_LINK_END_SIZE;
				temp_items[k].reserved = SCE_LINK_END_SIZE;
				k++;

				op_buf->sca.in = (uint64_t)temp_items;
				op_buf->sca.dma_mode |= 0x2;
			} else {
				op_buf->sca.in = (uint64_t)src + offset;
			}
		} else {
			op_buf->sca.out = (uint64_t)result + offset;
			op_buf->sca.in = (uint64_t)src + offset;
			op_buf->sca.dma_mode = 0;
		}

		op_buf->sca.tag <<= 48;
		op_buf->sca.tag |= (uint64_t)result + offset;
		op_buf->sca.tag |= 1ul << 63;
		op_buf++;

		offset += testvec->len + 80;
		offset += (64 - (offset & 0x3F)) & 0x3F;
	}

	enqueued_count = sce_enqueue(sce_queue_desc, op_datas, count);
	if (!enqueued_count)
		goto out;

	count = enqueued_count;
	out_rsp = malloc(sizeof(sce_rsp_t) * count);
	if (NULL == out_rsp) {
		goto out;
	}

	while (enqueued_count && running) {
		dequeued_count =
			sce_dequeue(sce_queue_desc, out_rsp, enqueued_count);
		if (0 == dequeued_count) {
			usleep(1000);
			continue;
		}

		enqueued_count -= dequeued_count;
		for (i = 0; i < dequeued_count; i++) {
			sce_rsp_t *rsp = &out_rsp[i];
			index = (rsp->tag >> 48) & 0x7FFF;
			if (rsp->state != CMD_SUCCESS) {
				SCE_LOG_ERR(
					"ske_mac[%d] enc[%d] fail!alg[0x%x], len[%d], res[0x%x]\n",
					index, enc, testvec->alg, testvec->len,
					rsp->state);
				continue;
			}

			enc = rsp->tag >> 63;
			tag = (uint8_t *)(rsp->tag & 0xFFFFFFFFFFFF);
			testvec = &ske_mac_tv_template[index];

			if (memcmp(tag, testvec->ctext, testvec->taglen)) {
				SCE_LOG_ERR(
					"ske_mac[%d] enc error!alg[0x%x], len[%d]\n",
					index, testvec->alg, testvec->len);
				print_buf("ctext", tag, testvec->len);
				print_buf("testvec ctext",
					  (uint8_t *)testvec->ctext,
					  testvec->len);
				err_num++;
				continue;
			}
#if 1
			if (memcmp(tag + testvec->len,
				   "\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5"
				   "\xa5\xa5\xa5",
				   16)) {
				SCE_LOG_ERR(
					"ske_mac[%d] enc[%d] error!alg[0x%x], len[%d]\n",
					index, enc, testvec->alg,
					testvec->taglen);
				print_buf("text add", tag + testvec->taglen,
					  64);
				err_num++;
				continue;
			}
#endif
		}
	}

	if (!err_num)
		SCE_LOG_ALWAYS("test cmac cbcmac success!\n");

out:
	if (op_datas)
		free(op_datas);

	if (origal_buf)
		free(origal_buf);

	if (out_rsp)
		free(out_rsp);

	sce_stop_queue(sce_queue_desc);
	if (sce_queue_desc)
		sce_release_queue(sce_queue_desc);

	sce_release_dmabuf(result);

	sce_release_dmabuf(src);

	sce_release_dmabuf(key);

	sce_release_dmabuf(iv);

	sce_release_dmabuf(src_link_items);

	sce_release_dmabuf(dst_link_items);

	sce_exit_lib();

	return ret;
}

int sca_test(int dma_mode, int src_link_num, int dst_link_num)
{
	const ske_testvec_t *testvec;
	int i, j, k, offset, index, enc, ret = 0, temp_src_link_num = 0,
					 temp_dst_link_num = 0;
	sce_rsp_t *out_rsp = NULL;
	uint8_t *result = NULL, *src = NULL, *key = NULL, *iv = NULL,
		*tag = NULL;
	sce_queue_handle sce_queue_desc = NULL;
	void **op_datas = NULL;
	sce_op_data_t *op_buf = NULL;
	void *origal_buf = NULL;
	int count = ARRAY_SIZE(ske_tv_template) * 2;
	int enqueued_count;
	int dequeued_count;
	int err_num;
	uint8_t *src_link_items = NULL, *dst_link_items = NULL;
	sce_link_list_item_t *temp_items = NULL;
	uint64_t tmp = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = ske_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	if (sce_request_queue(SCE_SCA, &sce_queue_desc)) {
		return -1;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &result);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &key);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &iv);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src_link_items);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(0x100000, &dst_link_items);
	if (ret)
		goto out;

	op_datas = (void **)malloc(sizeof(void *) * count);
	if (!op_datas) {
		SCE_LOG_ERR("alloc op_datas error");
		goto out;
	}

	origal_buf = (sce_op_data_t *)malloc(sizeof(sce_op_data_t) * count);
	if (!origal_buf) {
		SCE_LOG_ERR("alloc op_buf error");
		goto out;
	}
	op_buf = origal_buf;

	// dma_mode data
	dst_link_num = dst_link_num <= 32 ? dst_link_num : 32;
	src_link_num = src_link_num <= 32 ? src_link_num : 32;

	offset = 0;
	for (i = 0; i < ARRAY_SIZE(ske_tv_template); i++) {
		op_datas[i] = op_buf;
		op_buf->op_type = SCE_SCA;
		testvec = &ske_tv_template[i];
		memset(result + offset, 0xa5, testvec->len + 64);
		op_buf->sca.out_length = testvec->len;

		memcpy(src + offset, testvec->ptext, testvec->len);
		op_buf->sca.in_length = testvec->len;

		memcpy(key + offset, testvec->key, testvec->klen);
		op_buf->sca.key = (uint64_t)key + offset;

		if (testvec->ilen) {
			memcpy(iv + offset, testvec->iv, testvec->ilen);
			op_buf->sca.iv = (uint64_t)iv + offset;
		} else {
			op_buf->sca.iv = 0;
		}
		op_buf->sca.alg = testvec->alg;
		if (testvec->mode == XTS || testvec->mode == CBC_MAC ||testvec->mode == CMAC
			|| testvec->mode == GCM || testvec->mode ==CCM)
		{
			SCE_LOG_ERR("unsupport mode!\n");
			return -1;
		}
		op_buf->sca.mode = testvec->mode;
		op_buf->sca.op_mode = SCE_ENC;
		op_buf->sca.tag = i;

		if (dma_mode) {
			op_buf->sca.dma_mode = 0;
			if (dma_mode & 0x1 && testvec->len > 16) { // dst
				temp_items =
					(sce_link_list_item_t
						 *)(dst_link_items +
						    i * (dst_link_num + 2) *
							    sizeof(sce_link_list_item_t));
				if (((uint64_t)temp_items) & 0x1f) {
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items +
							    0x1f);
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items &
							    (~0x1f));
				}

				temp_dst_link_num =
					dst_link_num < testvec->len ?
						dst_link_num :
						testvec->len;
				tmp = sce_virt_to_io(result + offset);
				for (k = 0; k < temp_dst_link_num - 1; k++) {
					temp_items[k].address = tmp + k;
					temp_items[k].address |= 0x300000000000;
					temp_items[k].size = 0;
					temp_items[k].reserved = 0;
				}
				temp_items[k].address = tmp + k;
				temp_items[k].address |= 0x300000000000;
				temp_items[k].size = testvec->len - k - 1;
				temp_items[k].reserved = 0;
				k++;

				temp_items[k].address = SCE_LINK_END_ADDRESS;
				temp_items[k].size = SCE_LINK_END_SIZE;
				temp_items[k].reserved = SCE_LINK_END_SIZE;

				op_buf->sca.out = (uint64_t)temp_items;
				op_buf->sca.dma_mode |= 0x1;
			} else {
				op_buf->sca.out = (uint64_t)result + offset;
			}

			if (dma_mode & 0x2 && testvec->len > 1) { // src
				temp_items =
					(sce_link_list_item_t
						 *)(src_link_items +
						    i * (src_link_num + 2) *
							    sizeof(sce_link_list_item_t));
				if (((uint64_t)temp_items) & 0x1f) {
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items +
							    0x1f);
					temp_items =
						(sce_link_list_item_t
							 *)((uint64_t)
								    temp_items &
							    (~0x1f));
				}

				temp_src_link_num =
					src_link_num < testvec->len ?
						src_link_num :
						testvec->len;
				tmp = sce_virt_to_io(src + offset);
				for (k = 0; k < temp_src_link_num - 1; k++) {
					temp_items[k].address = tmp + k;
					temp_items[k].address |= 0x300000000000;
					temp_items[k].size = 0;
					temp_items[k].reserved = 0;
				}
				// end of link_list
				temp_items[k].address = tmp + k;
				temp_items[k].address |= 0x300000000000;
				temp_items[k].size = testvec->len - k - 1;
				temp_items[k].reserved = 0;
				k++;

				temp_items[k].address = SCE_LINK_END_ADDRESS;
				temp_items[k].size = SCE_LINK_END_SIZE;
				temp_items[k].reserved = SCE_LINK_END_SIZE;

				op_buf->sca.in = (uint64_t)temp_items;
				op_buf->sca.dma_mode |= 2;
			} else {
				op_buf->sca.in = (uint64_t)src + offset;
			}
		} else {
			op_buf->sca.out = (uint64_t)result + offset;
			op_buf->sca.in = (uint64_t)src + offset;
			op_buf->sca.dma_mode = 0;
		}

		op_buf->sca.tag <<= 48;
		op_buf->sca.tag |= (uint64_t)result + offset;
		op_buf->sca.tag |= 1ul << 63;
		op_buf++;

		offset += testvec->len + 80;
		offset += (64 - (offset & 0x3F)) & 0x3F;
	}

	j = i;
	for (i = 0; i < ARRAY_SIZE(ske_tv_template); i++) {
		op_datas[j + i] = op_buf;
		op_buf->op_type = SCE_SCA;
		testvec = &ske_tv_template[i];
		memset(result + offset, 0xa5, testvec->len + 64);
		op_buf->sca.out = (uint64_t)result + offset;
		op_buf->sca.out_length = testvec->len;

		memcpy(src + offset, testvec->ctext, testvec->len);
		op_buf->sca.in = (uint64_t)src + offset;
		op_buf->sca.in_length = testvec->len;

		memcpy(key + offset, testvec->key, testvec->klen);
		op_buf->sca.key = (uint64_t)key + offset;

		if (testvec->ilen) {
			memcpy(iv + offset, testvec->iv, testvec->ilen);
			op_buf->sca.iv = (uint64_t)iv + offset;
		} else {
			op_buf->sca.iv = 0;
		}

		op_buf->sca.alg = testvec->alg;
		op_buf->sca.mode = testvec->mode;
		op_buf->sca.op_mode = SCE_DEC;

		op_buf->sca.tag = i;

		op_buf->sca.tag <<= 48;
		op_buf->sca.tag |= (uint64_t)result + offset;
		op_buf++;

		offset += testvec->len + 80;
		offset += (64 - (offset & 0x3F)) & 0x3F;
	}

	enqueued_count = sce_enqueue(sce_queue_desc, op_datas, count);
	if (!enqueued_count)
		goto out;

	count = enqueued_count;
	out_rsp = malloc(sizeof(sce_rsp_t) * count);
	if (NULL == out_rsp) {
		goto out;
	}

	while (enqueued_count && running) {
		dequeued_count =
			sce_dequeue(sce_queue_desc, out_rsp, enqueued_count);
		if (0 == dequeued_count) {
			usleep(1000);
			continue;
		}

		enqueued_count -= dequeued_count;
		for (i = 0; i < dequeued_count; i++) {
			sce_rsp_t *rsp = &out_rsp[i];
			index = (rsp->tag >> 48) & 0x7FFF;
			if (rsp->state != CMD_SUCCESS) {
				SCE_LOG_ERR(
					"ske[%d] enc[%d] fail!alg[0x%x], len[%d], res[0x%x]\n",
					index, enc, testvec->alg, testvec->len,
					rsp->state);
				continue;
			}

			enc = rsp->tag >> 63;
			tag = (uint8_t *)(rsp->tag & 0xFFFFFFFFFFFF);
			testvec = &ske_tv_template[index];

			if (enc) {
				if (memcmp(tag, testvec->ctext, testvec->len)) {
					SCE_LOG_ERR(
						"ske[%d] enc error!alg[0x%x], len[%d]\n",
						index, testvec->alg,
						testvec->len);
					print_buf("ctext", tag, testvec->len);
					print_buf("testvec ctext",
						  (uint8_t *)testvec->ctext,
						  testvec->len);
					err_num++;
					continue;
				}
			} else {
				if (memcmp(tag, testvec->ptext, testvec->len)) {
					SCE_LOG_ERR(
						"ske[%d] dec error!alg[0x%x], len[%d]\n",
						index, testvec->alg,
						testvec->len);
					err_num++;
					print_buf("ptext", tag, testvec->len);
					print_buf("testvec ptext",
						  (uint8_t *)testvec->ptext,
						  testvec->len);
					continue;
				}
			}

			if (memcmp(tag + testvec->len,
				   "\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5"
				   "\xa5\xa5\xa5",
				   16)) {
				SCE_LOG_ERR(
					"ske[%d] enc[%d] error!alg[0x%x], len[%d]\n",
					index, enc, testvec->alg, testvec->len);
				print_buf("text add", tag + testvec->len, 64);
				err_num++;
				continue;
			}
		}
	}
	if (!err_num)
		SCE_LOG_ALWAYS("test ske success!\n");

out:
	if (op_datas)
		free(op_datas);

	if (origal_buf)
		free(origal_buf);

	if (out_rsp)
		free(out_rsp);

	sce_stop_queue(sce_queue_desc);
	if (sce_queue_desc)
		sce_release_queue(sce_queue_desc);

	// release dma buf
	sce_release_dmabuf(result);

	sce_release_dmabuf(src);

	sce_release_dmabuf(key);

	sce_release_dmabuf(iv);

	sce_release_dmabuf(src_link_items);

	sce_release_dmabuf(dst_link_items);

	sce_exit_lib();

	return ret;
}
int gcm_speed_test(int queue_num, long num, int alg, int mode, int op_mode,
		   int size, int dma_mode, int src_link_num, int dst_link_num)
{
	struct timeval start, end;
	long cmd_total, rsp_total = 0, i, ret = -1;
	sce_rsp_t rsp[8];
	sce_rsp_t *out_rsp = NULL;
	unsigned long time;
	sce_queue_handle sce_queue_desc[4] = { NULL };
	uint8_t *buf = NULL;
	uint64_t tmp = 0, in = 0, result = 0, key = 0, iv = 0, ult_in = 0,
		 ult_result = 0, src_link_items = 0, dst_link_items = 0;
	sce_op_data_t op;
	sce_link_list_item_t *temp_items = NULL;
	void *op_datas[1] = { &op };
	int enqueued_count = 0, dequeued_count = 0, src_len = 0, dst_len = 0,
	    sca_dma_mode = 0;

	sce_lib_cfg_t cfg = {
		.op_addr_mode =
			SCE_MEM_MODE_IOVA, // no need to translate every time.
		.iomap = { .usr = NULL, .iova_map = NULL }
	};

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	size = (size + 0xF) & (~0xF);
	for (i = 0; i < queue_num; i++) {
		if (sce_request_queue(SCE_SCA, &sce_queue_desc[i])) {
			goto out;
		}

		ret = sce_init_queue(sce_queue_desc[i]);
		if (ret)
			goto out;
	}

	ret = sce_requset_dmabuf(size + 16, &buf);
	if (ret)
		goto out;
	in = (uint64_t)buf;
	ret = sce_requset_dmabuf(size + 16, &buf);
	if (ret)
		goto out;
	result = (uint64_t)buf;
	ret = sce_requset_dmabuf(32, &buf);
	if (ret)
		goto out;
	key = (uint64_t)buf;
	ret = sce_requset_dmabuf(12, &buf);
	if (ret)
		goto out;
	iv = (uint64_t)buf;

	ret = sce_requset_dmabuf(0x100000, &buf);
	if (ret)
		goto out;
	src_link_items = (uint64_t)buf;

	ret = sce_requset_dmabuf(0x100000, &buf);
	if (ret)
		goto out;
	dst_link_items = (uint64_t)buf;

	dst_link_num = dst_link_num <= 32 ? dst_link_num : 32;
	src_link_num = src_link_num <= 32 ? src_link_num : 32;

	cmd_total = num;
	rsp_total = 0;

	//ult_in = sce_virt_to_io((void *)in);
	//ult_result = sce_virt_to_io((void *)result);
	if (op_mode == SCE_DEC) {
		src_len = size + 16;
		dst_len = size;
		op.op_type = SCE_SCA;
		op.sca.alg = alg_val[alg];
		op.sca.mode = mode_val[mode];
		op.sca.op_mode = SCE_ENC;
		op.sca.dma_mode = 0;
		op.sca.in_length = size;
		op.sca.out_length = size + 16;
		op.sca.in = sce_virt_to_io((void *)result);
		op.sca.out = sce_virt_to_io((void *)in);
		op.sca.key = sce_virt_to_io((void *)key);
		op.sca.iv = sce_virt_to_io((void *)iv);
		op.sca.aad_size = 0;
		op.sca.tag_size = 16;
		enqueued_count = sce_enqueue(sce_queue_desc[0], op_datas, 1);
		if (!enqueued_count)
			goto out;
		out_rsp = malloc(sizeof(sce_rsp_t) * 1);
		if (NULL == out_rsp) {
			goto out;
		}

		while (enqueued_count && running) {
			dequeued_count =
				sce_dequeue(sce_queue_desc[0], out_rsp, 1);
			if (0 == dequeued_count) {
				usleep(1000);
				continue;
			}
			enqueued_count -= dequeued_count;
			for (i = 0; i < dequeued_count; i++) {
				sce_rsp_t *rsp = &out_rsp[0];
				if (rsp->state != CMD_SUCCESS) {
					SCE_LOG_ERR("gcm res[0x%x]\n",
						    rsp->state);
					goto out;
				}
			}
		}
		if (out_rsp)
			free(out_rsp);
	} else {
		src_len = size;
		dst_len = size + 16;
	}
	enqueued_count = 0;

	gettimeofday(&start, NULL);

	if (dma_mode & 0x2 && src_link_num > 1) {
		src_link_num = src_link_num < size ? src_link_num : size;
		temp_items = (sce_link_list_item_t *)src_link_items;
		tmp = sce_virt_to_io((void *)in);
		for (i = 0; i < src_link_num - 1; i++) {
			temp_items[i].address = tmp + i;
			temp_items[i].size = 0;
			temp_items[i].reserved = 0;
		}
		temp_items[i].address = tmp + i;
		temp_items[i].size = src_len - i - 1;
		temp_items[i].reserved = 0;
		i++;

		temp_items[i].address = SCE_LINK_END_ADDRESS;
		temp_items[i].size = SCE_LINK_END_SIZE;
		temp_items[i].reserved = SCE_LINK_END_SIZE;
		ult_in = sce_virt_to_io((void *)src_link_items);
		sca_dma_mode |= 0x2;
	} else {
		ult_in = sce_virt_to_io((void *)in);
	}

	if (dma_mode & 0x1 && dst_link_num > 1) {
		dst_link_num = dst_link_num < size ? dst_link_num : size;
		temp_items = (sce_link_list_item_t *)dst_link_items;
		tmp = sce_virt_to_io((void *)result);
		for (i = 0; i < dst_link_num - 1; i++) {
			temp_items[i].address = tmp + i;
			temp_items[i].size = 0;
			temp_items[i].reserved = 0;
		}
		temp_items[i].address = tmp + i;
		temp_items[i].size = dst_len - i - 1;
		temp_items[i].reserved = 0;
		i++;

		temp_items[i].address = SCE_LINK_END_ADDRESS;
		temp_items[i].size = SCE_LINK_END_SIZE;
		temp_items[i].reserved = SCE_LINK_END_SIZE;
		ult_result = sce_virt_to_io((void *)dst_link_items);
		sca_dma_mode |= 0x1;
	} else {
		ult_result = sce_virt_to_io((void *)result);
	}

	do {
		for (i = 0; i < queue_num; i++) {
			if (cmd_total) {
				op.op_type = SCE_SCA;
				op.sca.alg = alg_val[alg];
				op.sca.mode = mode_val[mode];
				op.sca.op_mode = op_mode;
				op.sca.dma_mode = sca_dma_mode;
				op.sca.in_length = size;
				op.sca.out_length =
					dst_len; //加密：size+16,解密：size
				op.sca.in = ult_in;
				op.sca.out = ult_result;
				op.sca.key = sce_virt_to_io((void *)key);
				op.sca.iv = sce_virt_to_io((void *)iv);
				op.sca.aad_size = 0;
				op.sca.tag_size = 16;

				enqueued_count = sce_enqueue(sce_queue_desc[i],
							     op_datas, 1);
				cmd_total -= enqueued_count;
			}
			rsp_total += sce_dequeue(sce_queue_desc[i], rsp, 1);
		}
		if (rsp_total >= num)
			break;
	} while (running);
	gettimeofday(&end, NULL);

	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS("alg[%s] mode[%s] qnum[0x%x] testnum:%ld, testsize:%d, "
		       "complete:%ld,  speed:%ld Mbps-%ld pps\n",
		       alg_name[alg], mode_name[mode], queue_num, num, size,
		       rsp_total, (long)rsp_total * size * 8 / time,
		       (long)rsp_total * 1000000ul / time);

out:

	for (i = 0; i < queue_num; i++) {
		sce_stop_queue(sce_queue_desc[i]);
	}

	for (i = 0; i < queue_num; i++) {
		if (sce_queue_desc[i])
			sce_release_queue(sce_queue_desc[i]);
	}
	sce_release_dmabuf((void *)src_link_items);

	sce_release_dmabuf((void *)dst_link_items);

	sce_release_dmabuf((void *)in);

	sce_release_dmabuf((void *)result);

	sce_release_dmabuf((void *)key);

	sce_release_dmabuf((void *)iv);

	sce_exit_lib();

	return ret;
}

int ccm_speed_test(int queue_num, long num, int alg, int mode, int op_mode,
		   int size, int dma_mode, int src_link_num, int dst_link_num)
{
	struct timeval start, end;
	long cmd_total, rsp_total = 0, i, ret = -1;
	sce_rsp_t rsp[8];
	unsigned long time;
	sce_queue_handle sce_queue_desc[4] = { NULL };
	uint8_t *src = NULL, *result = NULL, *iv = NULL, *key = NULL,
		*src_link_items = NULL, *dst_link_items = NULL, *ult_src = NULL,
		*ult_dst = NULL;
	uint64_t tmp = 0, ult_in = 0, ult_result = 0;
	sce_op_data_t op;
	void *op_datas[1] = { &op };
	int enqueued_count = 0, dequeued_count = 0;
	ccm_flag_t *flag;
	int k, aad_head_len, src_len = 0, dst_len = 0;
	char *pre_iv =
		"\x01\x00\x00\x00\x03\x02\x01\x00\xa0\xa1\xa2\xa3\xa4\xa5\x00\x00";
	char *assoc = "\x63\x01\x8f\x76\xdc\x8a\x1b\xcb";
	int alen = 0, sca_dma_mode = 0;
	sce_rsp_t *out_rsp = NULL;
	sce_link_list_item_t *temp_items = NULL;

	sce_lib_cfg_t cfg = {
		.op_addr_mode =
			SCE_MEM_MODE_IOVA, // no need to translate every time.
		.iomap = { .usr = NULL, .iova_map = NULL }
	};

	if (sce_init_lib(&cfg)) {
		return -1;
	}
	size = (size + 0xF) & (~0xF);
	for (i = 0; i < queue_num; i++) {
		if (sce_request_queue(SCE_SCA, &sce_queue_desc[i])) {
			goto out;
		}

		ret = sce_init_queue(sce_queue_desc[i]);
		if (ret)
			goto out;
	}

	ret = sce_requset_dmabuf(0x100000, &src);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &result);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(16, &iv);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(32, &key);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &src_link_items);
	if (ret)
		goto out;
	ret = sce_requset_dmabuf(0x100000, &dst_link_items);
	if (ret)
		goto out;

	cmd_total = num;
	rsp_total = 0;

	dst_link_num = dst_link_num <= 32 ? dst_link_num : 32;
	src_link_num = src_link_num <= 32 ? src_link_num : 32;

	memcpy(iv, pre_iv, 16);
	memcpy(src, pre_iv, 16);
	flag = (ccm_flag_t *)(src);
	flag->tlen = (op.sca.tag_size - 2) / 2;
	flag->aad = !!alen;
	flag->reserved = 0;

	/* B1-Bu */
	if (alen) {
		if (alen < (0x10000 - 0x100)) {
			for (k = 0; k < 2; k++) {
				src[17 - k] = (alen >> (8 * k)) & 0xFF;
			}
			aad_head_len = 2;
		} else if (alen <= 0xFFFFFFFF) {
			src[17] = 0xFF;
			src[18] = 0xFE;
			for (k = 0; k < 4; k++) {
				src[21 - k] = (alen >> (8 * k)) & 0xFF;
			}
			aad_head_len = 6;
		} else {
			src[17] = 0xFF;
			src[18] = 0xFF;
			for (k = 0; k < 8; k++) {
				src[25 - k] = (alen >> (8 * k)) & 0xFF;
			}
			aad_head_len = 10;
		}
		memcpy(src + 16 + aad_head_len, assoc, alen);
	} else {
		aad_head_len = 0;
	}

	memcpy(result, src, 16 + aad_head_len + alen);

	if (op_mode == SCE_ENC) {
		src_len = size + 16 + aad_head_len + alen;
		dst_len = size + 16;
		ult_src = src;
		ult_dst = result + 16 + aad_head_len + alen;
	} else {
		src_len = size + 16 + 16 + aad_head_len + alen;
		dst_len = size;
		ult_src = result;
		ult_dst = src + 16 + aad_head_len + alen;
	}
	op.op_type = SCE_SCA;
	op.sca.alg = alg_val[alg];
	op.sca.mode = mode_val[mode];
	op.sca.op_mode = SCE_ENC;
	op.sca.tag = 1;
	op.sca.dma_mode = 0;
	op.sca.in_length = size;
	op.sca.out_length = size + 16;
	op.sca.tag_size = 16;
	op.sca.in = sce_virt_to_io((void *)(uint64_t)src);
	;
	op.sca.out = sce_virt_to_io((void *)(uint64_t)result + 16 +
				    aad_head_len + alen);
	op.sca.key = sce_virt_to_io((void *)(uint64_t)key);
	op.sca.iv = sce_virt_to_io((void *)(uint64_t)iv);
	op.sca.aad_size = 16 + aad_head_len + alen;

	enqueued_count = sce_enqueue(sce_queue_desc[0], op_datas, 1);
	if (!enqueued_count) {
		SCE_LOG_ERR("get testvec enqueue fail!\n");
		goto out;
	}
	out_rsp = malloc(sizeof(sce_rsp_t));
	if (NULL == out_rsp) {
		SCE_LOG_ERR("malloc out_rsp error\n");
		ret = -1;
		goto out;
	}

	while (enqueued_count && running) {
		dequeued_count = sce_dequeue(sce_queue_desc[0], out_rsp, 1);
		if (0 == dequeued_count) {
			usleep(1000);
			continue;
		}
		enqueued_count -= dequeued_count;
		sce_rsp_t *rsp = &out_rsp[0];
		if (rsp->state != CMD_SUCCESS) {
			SCE_LOG_ERR("get testvec dequeue fail!\n");
			goto out;
		}
	}
	enqueued_count = 0;
	gettimeofday(&start, NULL);
	if (dma_mode & 0x2 && src_link_num > 1) {
		src_link_num = src_link_num < size ? src_link_num : size;
		temp_items = (sce_link_list_item_t *)src_link_items;
		tmp = sce_virt_to_io((void *)(uint64_t)ult_src);
		for (i = 0; i < src_link_num - 1; i++) {
			temp_items[i].address = tmp + i;
			temp_items[i].size = 0;
			temp_items[i].reserved = 0;
		}
		temp_items[i].address = tmp + i;
		temp_items[i].size = src_len - i - 1;
		temp_items[i].reserved = 0;
		i++;

		temp_items[i].address = SCE_LINK_END_ADDRESS;
		temp_items[i].size = SCE_LINK_END_SIZE;
		temp_items[i].reserved = SCE_LINK_END_SIZE;
		ult_in = sce_virt_to_io((void *)(uint64_t)src_link_items);
		sca_dma_mode |= 0x2;
	} else {
		ult_in = sce_virt_to_io((void *)(uint64_t)ult_src);
	}

	if (dma_mode & 0x1 && dst_link_num > 1) {
		dst_link_num = dst_link_num < size ? dst_link_num : size;
		temp_items = (sce_link_list_item_t *)dst_link_items;
		tmp = sce_virt_to_io((void *)(uint64_t)ult_dst);
		for (i = 0; i < dst_link_num - 1; i++) {
			temp_items[i].address = tmp + i;
			temp_items[i].size = 0;
			temp_items[i].reserved = 0;
		}
		temp_items[i].address = tmp + i;
		temp_items[i].size = dst_len - i - 1;
		temp_items[i].reserved = 0;
		i++;

		temp_items[i].address = SCE_LINK_END_ADDRESS;
		temp_items[i].size = SCE_LINK_END_SIZE;
		temp_items[i].reserved = SCE_LINK_END_SIZE;
		ult_result = sce_virt_to_io((void *)(uint64_t)dst_link_items);
		sca_dma_mode |= 0x1;
	} else {
		ult_result = sce_virt_to_io((void *)(uint64_t)ult_dst);
	}
	do {
		for (i = 0; i < queue_num; i++) {
			if (cmd_total) {
				op.op_type = SCE_SCA;
				op.sca.alg = alg_val[alg];
				op.sca.mode = mode_val[mode];
				op.sca.op_mode = op_mode;
				op.sca.tag = i;
				op.sca.dma_mode = sca_dma_mode;
				op.sca.in_length = size;
				op.sca.out_length = size;
				op.sca.tag_size = 16;
				op.sca.in = ult_in;
				op.sca.out = ult_result;
				op.sca.key =
					sce_virt_to_io((void *)(uint64_t)key);
				op.sca.iv =
					sce_virt_to_io((void *)(uint64_t)iv);
				memcpy(src, pre_iv, 16);
				op.sca.aad_size = 16 + aad_head_len + alen;

				enqueued_count = sce_enqueue(sce_queue_desc[i],
							     op_datas, 1);
				cmd_total -= enqueued_count;
			}
			rsp_total += sce_dequeue(sce_queue_desc[i], rsp, 1);
		}
		if (rsp_total >= num)
			break;
	} while (running);
	gettimeofday(&end, NULL);

	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS("alg[%s] mode[%s] qnum[0x%x] testnum:%ld, testsize:%d, "
		       "complete:%ld,  speed:%ld Mbps-%ld pps\n",
		       alg_name[alg], mode_name[mode], queue_num, num, size,
		       rsp_total, (long)rsp_total * size * 8 / time,
		       (long)rsp_total * 1000000ul / time);

out:
	for (i = 0; i < queue_num; i++) {
		sce_stop_queue(sce_queue_desc[i]);
	}
	for (i = 0; i < queue_num; i++) {
		if (sce_queue_desc[i])
			sce_release_queue(sce_queue_desc[i]);
	}
	if (out_rsp)
		free(out_rsp);

	sce_release_dmabuf(src);
	sce_release_dmabuf(result);
	sce_release_dmabuf(key);
	sce_release_dmabuf(iv);
	sce_release_dmabuf(src_link_items);

	sce_release_dmabuf(dst_link_items);
	sce_exit_lib();

	return ret;
}

int sca_speed_test(int queue_num, long num, int alg, int mode, int op_mode,
		   int size, int dma_mode, int src_link_num, int dst_link_num)
{
	struct timeval start, end;
	long cmd_total, rsp_total = 0, i, ret = -1;
	sce_rsp_t rsp[8];
	unsigned long time;
	sce_queue_handle sce_queue_desc[4] = { NULL };
	uint8_t *buf;
	uint64_t in = 0, result = 0, key = 0, iv = 0, src_link_items = 0,
		 dst_link_items = 0, tmp = 0, ult_in = 0, ult_result = 0;
	sce_op_data_t op;
	void *op_datas[1] = { &op };
	int enqueued_count = 0;
	sce_link_list_item_t *temp_items = NULL;
	int sca_dma_mode = 0;

	if (mode_val[mode] == GCM) {
		return gcm_speed_test(queue_num, num, alg, mode, op_mode, size,
				      dma_mode, src_link_num, dst_link_num);
	}
	if (mode_val[mode] == CCM) {
		return ccm_speed_test(queue_num, num, alg, mode, op_mode, size,
				      dma_mode, src_link_num, dst_link_num);
	}

	sce_lib_cfg_t cfg = {
		.op_addr_mode =
			SCE_MEM_MODE_IOVA, // no need to translate every time.
		.iomap = { .usr = NULL, .iova_map = NULL }
	};

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	size = (size + 0xF) & (~0xF);
	for (i = 0; i < queue_num; i++) {
		if (sce_request_queue(SCE_SCA, &sce_queue_desc[i])) {
			goto out;
		}

		ret = sce_init_queue(sce_queue_desc[i]);
		if (ret)
			goto out;
	}

	ret = sce_requset_dmabuf(size, &buf);
	if (ret)
		goto out;
	in = (uint64_t)buf;
	ret = sce_requset_dmabuf(size, &buf);
	if (ret)
		goto out;
	result = (uint64_t)buf;
	ret = sce_requset_dmabuf(256, &buf);
	if (ret)
		goto out;
	key = (uint64_t)buf;
	ret = sce_requset_dmabuf(256, &buf);
	if (ret)
		goto out;
	iv = (uint64_t)buf;
	ret = sce_requset_dmabuf(0x100000, &buf);
	if (ret)
		goto out;
	src_link_items = (uint64_t)buf;

	ret = sce_requset_dmabuf(0x100000, &buf);
	if (ret)
		goto out;
	dst_link_items = (uint64_t)buf;

	cmd_total = num;
	rsp_total = 0;
	gettimeofday(&start, NULL);
	dst_link_num = dst_link_num <= 32 ? dst_link_num : 32;
	src_link_num = src_link_num <= 32 ? src_link_num : 32;

	// fill src_link_items dst_link_items
	if (dma_mode & 0x2 && src_link_num > 1) {
		src_link_num = src_link_num < size ? src_link_num : size;
		temp_items = (sce_link_list_item_t *)src_link_items;
		tmp = sce_virt_to_io((void *)in);
		for (i = 0; i < src_link_num - 1; i++) {
			temp_items[i].address = tmp + i;
			temp_items[i].address |= 0x300000000000;
			temp_items[i].size = 0;
			temp_items[i].reserved = 0;
		}
		// end of link_list
		temp_items[i].address = tmp + i;
		temp_items[i].address |= 0x300000000000;
		temp_items[i].size = size - i - 1;
		temp_items[i].reserved = 0;
		i++;

		temp_items[i].address = SCE_LINK_END_ADDRESS;
		temp_items[i].size = SCE_LINK_END_SIZE;
		temp_items[i].reserved = SCE_LINK_END_SIZE;
		ult_in = sce_virt_to_io((void *)src_link_items);
		sca_dma_mode |= 0x2;
	} else {
		ult_in = sce_virt_to_io((void *)in);
	}

	if (dma_mode & 0x1 && dst_link_num > 1) {
		dst_link_num = dst_link_num < size ? dst_link_num : size;
		temp_items = (sce_link_list_item_t *)dst_link_items;
		tmp = sce_virt_to_io((void *)result);
		for (i = 0; i < dst_link_num - 1; i++) {
			temp_items[i].address = tmp + i;
			temp_items[i].address |= 0x300000000000;
			temp_items[i].size = 0;
			temp_items[i].reserved = 0;
		}
		// end of link_list
		temp_items[i].address = tmp + i;
		temp_items[i].address |= 0x300000000000;
		temp_items[i].size = size - i - 1;
		temp_items[i].reserved = 0;
		i++;

		temp_items[i].address = SCE_LINK_END_ADDRESS;
		temp_items[i].size = SCE_LINK_END_SIZE;
		temp_items[i].reserved = SCE_LINK_END_SIZE;
		ult_result = sce_virt_to_io((void *)dst_link_items);
		sca_dma_mode |= 0x1;
	} else {
		ult_result = sce_virt_to_io((void *)result);
	}

	do {
		for (i = 0; i < queue_num; i++) {
			if (cmd_total) {
				op.op_type = SCE_SCA;
				op.sca.alg = alg_val[alg];
				op.sca.mode = mode_val[mode];
				op.sca.op_mode = op_mode;
				op.sca.dma_mode = sca_dma_mode;
				op.sca.in_length = size;
				op.sca.out_length = size;
				op.sca.in = ult_in;
				op.sca.out = ult_result;
				op.sca.key = sce_virt_to_io((void *)key);
				op.sca.iv = sce_virt_to_io((void *)iv);

				enqueued_count = sce_enqueue(sce_queue_desc[i],
							     op_datas, 1);
				cmd_total -= enqueued_count;
			}
			rsp_total += sce_dequeue(sce_queue_desc[i], rsp, 8);
		}
		if (rsp_total >= num)
			break;
	} while (running);
	gettimeofday(&end, NULL);

	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS("alg[%s] mode[%s] qnum[0x%x] testnum:%ld, testsize:%d, "
		       "complete:%ld,  speed:%ld Mbps-%ld pps\n",
		       alg_name[alg], mode_name[mode], queue_num, num, size,
		       rsp_total, (long)rsp_total * size * 8 / time,
		       (long)rsp_total * 1000000ul / time);

out:

	for (i = 0; i < queue_num; i++) {
		sce_stop_queue(sce_queue_desc[i]);
	}

	for (i = 0; i < queue_num; i++) {
		if (sce_queue_desc[i])
			sce_release_queue(sce_queue_desc[i]);
	}

	sce_release_dmabuf((void *)in);

	sce_release_dmabuf((void *)result);

	sce_release_dmabuf((void *)key);

	sce_release_dmabuf((void *)iv);

	sce_release_dmabuf((void *)src_link_items);

	sce_release_dmabuf((void *)dst_link_items);

	sce_exit_lib();

	return ret;
}

int sca_delay_test(int alg, int mode, int op_mode, int size, int dma_mode,
		   int src_link_num, int dst_link_num)
{
	struct timeval start, end;
	long rsp_total = 0, ret = -1;
	sce_rsp_t rsp[8];
	uint8_t *buf;
	unsigned long time;
	sce_queue_handle sce_queue_desc = NULL;
	uint64_t in = 0, result = 0, key = 0, iv = 0, i = 0, src_link_items = 0,
		 dst_link_items = 0, tmp = 0;
	sce_op_data_t op;
	void *op_datas[1] = { &op };
	sce_link_list_item_t *temp_items = NULL;
	int sca_dma_mode = 0;

	sce_lib_cfg_t cfg = { .op_addr_mode = SCE_MEM_MODE_VA,
			      .iomap = { .usr = NULL,
					 .iova_map = ske_io_map } };

	if (sce_init_lib(&cfg)) {
		return -1;
	}

	ret = sce_requset_dmabuf(0x100000, &buf);

	if (ret)
		goto out;
	src_link_items = (uint64_t)buf;

	ret = sce_requset_dmabuf(0x100000, &buf);
	if (ret)
		goto out;
	dst_link_items = (uint64_t)buf;

	if (sce_request_queue(SCE_SCA, &sce_queue_desc)) {
		goto out;
	}

	ret = sce_init_queue(sce_queue_desc);
	if (ret)
		goto out;

	ret = sce_requset_dmabuf(size, &buf);
	if (ret)
		goto out;
	in = (uint64_t)buf;

	ret = sce_requset_dmabuf(size, &buf);
	if (ret)
		goto out;
	result = (uint64_t)buf;

	ret = sce_requset_dmabuf(256, &buf);
	if (ret)
		goto out;
	key = (uint64_t)buf;

	ret = sce_requset_dmabuf(256, &buf);
	if (ret)
		goto out;
	iv = (uint64_t)buf;
	rsp_total = 0;
	gettimeofday(&start, NULL);

	// fill src_link_items dst_link_items
	if (dma_mode & 0x2 && src_link_num > 1) {
		src_link_num = src_link_num < size ? src_link_num : size;
		temp_items = (sce_link_list_item_t *)src_link_items;
		tmp = sce_virt_to_io((void *)in);
		for (i = 0; i < src_link_num - 1; i++) {
			temp_items[i].address = tmp + i;
			temp_items[i].address |= 0x300000000000;
			temp_items[i].size = 0;
			temp_items[i].reserved = 0;
		}
		// end of link_list
		temp_items[i].address = tmp + i;
		temp_items[i].address |= 0x300000000000;
		temp_items[i].size = size - i - 1;
		temp_items[i].reserved = 0;
		i++;

		temp_items[i].address = SCE_LINK_END_ADDRESS;
		temp_items[i].size = SCE_LINK_END_SIZE;
		temp_items[i].reserved = SCE_LINK_END_SIZE;
		op.sca.in = src_link_items;
		op.sca.in_length = size;
		sca_dma_mode |= 0x2;
	} else {
		op.sca.in = in;
		op.sca.in_length = size;
	}

	if (dma_mode & 0x1 && dst_link_num > 1) {
		dst_link_num = dst_link_num < size ? dst_link_num : size;
		temp_items = (sce_link_list_item_t *)dst_link_items;
		tmp = sce_virt_to_io((void *)result);
		for (i = 0; i < dst_link_num - 1; i++) {
			temp_items[i].address = tmp + i;
			temp_items[i].address |= 0x300000000000;
			temp_items[i].size = 0;
			temp_items[i].reserved = 0;
		}
		temp_items[i].address = tmp + i;
		temp_items[i].address |= 0x300000000000;
		temp_items[i].size = size - i - 1;
		temp_items[i].reserved = 0;
		i++;

		temp_items[i].address = SCE_LINK_END_ADDRESS;
		temp_items[i].size = SCE_LINK_END_SIZE;
		temp_items[i].reserved = SCE_LINK_END_SIZE;
		op.sca.out = dst_link_items;
		op.sca.out_length = size;
		sca_dma_mode |= 0x1;
	} else {
		op.sca.out = result;
		op.sca.out_length = size;
	}

	op.op_type = SCE_SCA;
	op.sca.alg = alg_val[alg];
	op.sca.mode = mode_val[mode];
	op.sca.op_mode = op_mode;
	op.sca.dma_mode = sca_dma_mode; // dma_mode
	op.sca.key = key;
	op.sca.iv = iv;

	ret = sce_enqueue(sce_queue_desc, op_datas, 1);
	if (!ret)
		goto out;
	do {
		rsp_total += sce_dequeue(sce_queue_desc, rsp, 1);
		if (rsp_total >= 1)
			break;
	} while (running);
	gettimeofday(&end, NULL);

	time = (end.tv_sec - start.tv_sec) * 1000000ul +
	       (end.tv_usec - start.tv_usec);
	SCE_LOG_ALWAYS("alg[%s] mode[%s] testsize:%d  time:%ld us\n",
		       alg_name[alg], mode_name[mode], size, time);

out:
	sce_stop_queue(sce_queue_desc);

	sce_release_dmabuf((void *)src_link_items);

	sce_release_dmabuf((void *)dst_link_items);

	sce_release_queue(sce_queue_desc);

	sce_release_dmabuf((void *)in);

	sce_release_dmabuf((void *)result);

	sce_release_dmabuf((void *)key);

	sce_release_dmabuf((void *)iv);

	sce_exit_lib();

	return ret;
}
